c
c
c******************************************************************************
c     MODULE:INPUT
c******************************************************************************
c
      module input
       integer, parameter :: maxvalen = 6
       integer, parameter :: maxatm = 31900
       integer, parameter :: outfile = 6, groupfile = 5
c     
       type mmvalen 
        integer             natoms    
        integer             lprnatmprm
        double precision    atmcharge(maxatm)
        integer             atmid(maxatm) 
        character*4         atmtypname(maxatm)
        integer             atmtyp(maxatm)    
        integer             atmvalen(maxatm)  
        integer             atmvalenlist(maxatm,maxvalen)  
       end type mmvalen                                                  
      end module input

c
c******************************************************************************
c  MAIN
c******************************************************************************
c
c    Created:  Jul. 26, 2010 by Soroosh Pezeshki
c    Purpose:  Creats a list of group for the QM/MM partitioning
c    Usage createGroups crd_file grouping_method
c
c
c

      program main

      use input
      implicit none
       type (mmvalen)  :: mmgeom



      integer       narg, method, icint
      character*80  fname, gminput

c
c Input can be set by arguments. 
c If not enough arguments are given the progam ask for more input 
c
      narg=command_argument_count()
      if (narg.eq.0) then
          write(*,*) 'Enter crd file name'
          read(*,*) fname
          goto 100
      elseif (narg.eq.1) then
         call getarg(1, fname)
100       write(*,*) 'Grouping method:'
          write(*,*) '1: Each molecule (valence bond) a seperate group'
          write(*,*) '2: Each residue a seperate group'
          write(*,*) 
     *      '3: seperate backbone and side chain in seperate groups'
          read(*,*)  gminput
          goto 200
      else
         call getarg(1, fname)
         call getarg(2, gminput)
200      method=icint(gminput)
         if ((method.lt.1).or.(method.gt.3)) goto 6004
      endif

c
c Outputfiles are opened
c
      open (unit=groupfile, file='groups.log', status='unknown',
     *      form='formatted', err=6001)
      open (unit=outfile, file='groups.error', status='unknown',
     *      form='formatted', err=6003)
c
c the atoms are read in
c
      call rmmgeom(mmgeom, fname)

c
c the grouping method is caled
c
      call findresidues(mmgeom, method)

      close (1)
      close (outfile)
      return

6001  write (*,'(/a)') 'ERROR: Cannot open the groups.log.'
      stop 'main 1'
6003  write (*,'(/a), (a)') 'ERROR: Cannot open ', fname
      stop 'main 1'
6004  write (*,'(/a), (a)') 'ERROR: Grouping method must be 1, 2, or 3'
      stop 'main 1'

      end


c
c
c*****************************************************************************
c  FINDRESIDUES
c*****************************************************************************
c
      subroutine findresidues(mmgeom, methode)
c
c     Subroutine findresidues
c     
c     Created:  Jul. 26, 2010 by Soroosh Pezeshki
c
c     Purpose:  finds all residues and set the flags in molecule
c                                                                       
c 
c     Called by:  initmd
c     Calls:  calccom
c
c     Variables defined here:
c
      use input
      implicit none
       type (mmvalen)  :: mmgeom

c
c  local variables
c
      integer i,j,k,m, natoms, molid, valence, segcomp
      integer groupsize(maxatm), resid(maxatm), residue
      integer molecule(maxatm), maxmolecule, methode

c set the molid of atom 1 and all direct bondatoms to 1
c
      molid=2
      molecule(1)=1  
      
      natoms=mmgeom%natoms
      do i=1, mmgeom%atmvalen(1)
        valence=mmgeom%atmvalenlist(1,i)
        molecule(valence)=1
      enddo
c
c loop over all atoms beside atom 1
c
      do i =2, natoms
c
c if no molid is set set it to molid and than incr. molid
c
        if (molecule(i).eq.0) then
            molecule(i)=molid
            molid=molid+1
        endif
c
c set all direct bonds with the same molid if the molid is bigger or 0
c else set the molid of atom i
c loop over all direct bonds
c
        do j=1, mmgeom%atmvalen(i)
            valence=mmgeom%atmvalenlist(i,j)
            segcomp=molecule(valence)
c
c if the molid of a direct bond is less than the molid of atom i
c set all atom with molid to segcomp
c
            if ((segcomp.ne.0).and.(segcomp.lt.molid)) then
                do k=2, natoms
                    if (molecule(k).eq.molid) then
                        molecule(k)=segcomp
                    endif
                enddo
c
c else if the molid of the direct bonded atom is greater set to molid
c
            elseif (segcomp.gt.molid) then
                do k=2, natoms
                    if (molecule(k).eq.segcomp) then
                        molecule(k)=molid
                    endif
                enddo
c
c if the direct bonded atom has the molid 0 set it to the same as of atom i
c
            elseif (segcomp.eq.0) then
                molecule(valence)=molid
            endif
        enddo
      enddo
      maxmolecule=molid
      residue = 0
c
c loop over all atoms, determine group size
c
      if(methode.gt.1) then 
        groupsize(:)=0
        resid(:)=0
        do i =1, natoms
            groupsize(molecule(i))=groupsize(molecule(i))+1
        enddo

c
c loop over atoms, call 
c
        do i =1, natoms
            if (resid(i).eq.0) then
                call chain(i, mmgeom, resid, 0, 0, residue, methode)
            endif
        if (resid(i).ge.1) molecule(i)=resid(i)+maxmolecule
        enddo
      endif

c
c write the molid of each atom to groupfile
c
      write(groupfile,*)  maxmolecule-1+residue,  natoms
      write(groupfile,2002)  '#Group', 'index'
      j=0
      do i=1, natoms
        if (i .ne. j+1) cycle
        do j=i+1, natoms
            if (molecule(j) .ne. molecule(i)) exit
        enddo
        j=j-1
        write(groupfile,2001) molecule(i),  mmgeom%atmid(i), 
     *     mmgeom%atmid(j)
      enddo
      write(groupfile,2002)  'END'

2001  format(I5,t7,I6,t15,I6)
2002  format(a6,t9,a5)

      end

c
c
c*****************************************************************************
c  CHAIN
c*****************************************************************************
c
      recursive subroutine chain(atom, mmgeom, resid, flag,
     *                    before, residue, methode)
c
c     Subroutine chain
c     
c     Created:  Jul. 26, 2010 by Soroosh Pezeshki
c
c     Purpose:  mark the protien backbone
c     First the NCC chain is determind 
c     After that the side atoms get the same resid as the NCC
c     Backbone and the side chain can be split up (method 3) 
c
c     Variables defined here:
c
      use input
      implicit none
       type (mmvalen)  :: mmgeom

      integer flag, resid(maxatm), atom, methode, doside
      integer i,j, before, residue, resnow, flagbefore

c
c store the flag
c
      flagbefore=flag
c
c Check if atom is in the NCC sequence or any other atom
c N has mod(flag,3)=0, Cs have mod(flag,3)=1 and 2
c
      if (mod(flag,3).eq.0) then
        if (mmgeom%atmtypname(atom)(1:1).eq."N") then
c
c N is the begining of a amino acid; incr. residue
c
            residue=residue+1
            resnow=residue
            flag=flag+1
c
c Needed for Proline
c
            if (flag.gt.3) resid(atom)=resnow
c
c Look for all valence bonds
c
            do i=1,mmgeom%atmvalen(atom)
                j=mmgeom%atmvalenlist(atom,i)
c
c Don't go backwards
c
                if (j.eq.before) goto 100
c
c Continue with the next bonded atom
c
                call chain(j, mmgeom, resid, flag, atom, 
     *                 residue, methode)
100         enddo
        endif
      elseif (mod(flag,3).gt.0) then
        if (mmgeom%atmtypname(atom)(1:1).eq."C") then
            flag=flag+1
            resnow=residue
c
c Look for all valence bonds
c
            do i=1,mmgeom%atmvalen(atom)
                j=mmgeom%atmvalenlist(atom,i)
c
c Don't go backwards
c
                if (j.eq.before) goto 200
c
c Continue with the next bonded atom
c
                call chain(j, mmgeom, resid, flag, atom, 
     *                  residue, methode)
200         enddo
        endif
      endif

c
c For the case that the side chain sshould have a different group 
c than the backbone. Side chain is bonded to C alpha.
c
      if ((methode.eq.3).and.(mod(flagbefore,3).eq.1)) then
        residue=residue+1
        doside=1
      else 
        doside=0
      endif

c
c Change the resid and the resid of all bonded atoms in the same residue
c
      if (flagbefore+1.lt.flag) then
        if (flag.gt.3) then
c
c Change the resid
c
            resid(atom)=resnow
c
c Look for all valence bonds
c
            do i=1,mmgeom%atmvalen(atom)
                j=mmgeom%atmvalenlist(atom,i)
c
c Don't go backwards
c
                if(j.eq.before) goto 300
c
c For the case that the side chain sshould have a different group 
c than the backbone. Side chain is bonded to C alpha and is not H.
c
                if(doside.eq.1) then
                    if (mmgeom%atmtypname(j)(1:1).ne."H") then
                        call side1(j, resid, residue, mmgeom)
                    endif
                endif
c
c Change the resid of all bonded atoms in the same residue
c
                call side1(j, resid, resnow, mmgeom)
300         enddo
        endif
      endif

      return
      end

c
c
c*****************************************************************************
c  side1
c*****************************************************************************
c
      recursive subroutine side1(atom, resid, resnow, mmgeom)
c
c     Subroutine chain
c     
c     Created:  Jul. 26, 2010 by Soroosh Pezeshki
c
c     Purpose:  mark the side atoms of the protein backbone
c                                                                       
c 
c
c     Variables defined here:
c
      use input
      implicit none
       type (mmvalen)  :: mmgeom

      integer backbone, resid(maxatm), atom
      integer i,j, resnow

c
c only change if a resid is not assiegned to atom
c
      if (resid(atom).le.1) then
        resid(atom)=resnow
c
c continue with covalently bonded atoms without resid
c
        do i=1,mmgeom%atmvalen(atom)
            j=mmgeom%atmvalenlist(atom,i)
            if(resid(j).gt.1) goto 100
            call side1(j,resid, resnow, mmgeom)
100     enddo
      endif

      return
      end


c
c
c*******************************************************************************
c  RMMGEOM
c*******************************************************************************
c
      subroutine rmmgeom(mmgeom, fname)
c
c     Created:  Jul. 26, 2010 by Soroosh Pezeshki
c     Based on emmgeom
c     Purpose:  Read the MM atomic information from the .crd file.
c
c
c
c
      use input
      implicit none

c
c  i/o variables
c
      integer istrt, iend
      integer numatom, geomunit
      type (mmvalen) :: mmgeom
      character*80  string, fname

c
c  local variables
c                 
      integer i, isect, iatom, ierr, iscend
      integer j, iiatom, ifields, imaxfields, itemp(maxvalen)
      integer ifin, ibeg, ifind, iasym, chindex
      integer nwords, length, icint
      integer ieof, isec, t41crdf

      
      double precision cfloat
      character*80  words(40)

c
c mm information required at least 7 fields
c    the maximum valency is 6
c      RMMGEOM
      ifields = 7
      imaxfields = 13
      t41crdf= 4
c      
c open the t41.crd file
c
      open (unit=t41crdf, file=fname, status='old',
     *      form='formatted', err=5000)
c      
c read t41.crd for the number of atoms
c
      call readln(t41crdf,words,nwords,isec,ieof,iscend) 
c      
      if (iscend .eq. 1 .or. ieof .eq. 1) goto 4000
      iiatom = icint(words(1))
c      
c read each line in t41.crd:
c  index, atom symbol, atom type name, x, y, z, atom type, connectivities 
c
      iatom = 0
      do while (iscend .ne. 1 .and. ieof .ne. 1)  
         call readln(t41crdf,words,nwords,isec,ieof,iscend)
         if (iscend .ne. 1) then 
        if(nwords .lt. ifields) then
           write(outfile,1100) 
1100           format(/3x,'Error: insufficient data in .crd file')
           goto 6000           
        endif   
            if(nwords .gt. imaxfields) then
               write(outfile,1200)
1200           format(/3x,'Error: the allowed maximum valency is 6')
               goto 6000
            endif
c
c read mm atom id
c       
            iatom = iatom + 1 
            mmgeom%atmid(iatom) = icint(words(1))
            mmgeom%atmtyp(iatom) = icint(words(7))
            mmgeom%atmtypname(iatom) = words(3)(1:4)
        if(iatom .ne. mmgeom%atmid(iatom)) then
           write(outfile,1210) iatom
           goto 6000
        endif
1210  format(/2x,'Error: Atom ID',i4,/2x,
     *   'Atom ID is not given correctly.',/2x,
     *   'It should start from 1 and increases sequentially.')
c
c read the valency and connectivities
c
            j = nwords - ifields
            mmgeom%atmvalen(iatom) = j
            do i = 1, j
               mmgeom%atmvalenlist(iatom,i) = icint(words(ifields+i))
            enddo
c       
         endif
      enddo
c
c  return the final iatom value.
c
      if (iiatom .ne. iatom) then
         write(*,1300)
1300     format(/3x,'Error: pleass check atom number in .crd file')
         goto 6000
      endif
      numatom = iatom
      mmgeom%natoms = numatom
      close (t41crdf)
      return                 
c
4000  write (*,'(/a)') 'ERROR: The .crd file is empty.'
      goto 6000
c
5000  write (*,'(/a)') 'ERROR: Cannot open the .crd file.'
6000  close (t41crdf)
      close (outfile)
      stop 'rmmgeom 1'
c              
      end


c**********************************************************************
c readln
c**********************************************************************
c
      subroutine readln(iunit,word,nword,isec,ieof,iscend)
c
c     Created:  Feb 22. 2004 by Hai Lin
c
c     This subroutine finds the first non-comment and non-blank line
c     from the input file (iunit), 
c     where each line has at most 80 characters,
c     and chop the line into words according to the space locations.
c
c     It returns the number of the words in (nword), and the individual 
c     word in array (word) after changing to upper case.
c
c     The flags are also set 
c        if it is beginning of a section (isec), 
c     or if it is end of the file (ieof)
c     or if it is end of the section (iscend).
c
      implicit none
c
c  i/o variables
c
      integer  iunit, nword
      integer  ieof, isec, iscend
      character*80  word(40)
c            
c  local variables
c
      integer  ip, icom, ibeg      
      character*81  line, sename, string, upcse
c
      isec = 0
      ieof = 0
      iscend = 0
      nword = 0
c
10    read(iunit,'(a80)',end=500) string
c      
      ip = 0
c
c     find the first nonblank character in a line
c
20    ip = ip + 1
      if (string(ip:ip) .eq. ' ' .and. ip .le. 80) goto 20           
c
c     if it all blank or it is a comment line then read the next line
c
      if (ip .gt. 80 .or. string(ip:ip) .eq. '#')  goto 10

      line = upcse(string)
C
C     Find the position of a comment character, if any,
C     in a nonblank noncomment line
C
      icom = ip 
50    icom = icom + 1
      if (line(icom:icom) .ne. '#' .and. icom .le. 80)  goto 50         
C
55    if (ip .lt. icom) then
         ibeg = ip
60       ip = ip + 1
         if (line(ip:ip) .ne. ' ' .and. ip .lt. icom) goto 60 
         nword = nword + 1
         word(nword) = line(ibeg:ip-1)
70       ip = ip + 1
         if (line(ip:ip) .eq. ' ' .and. ip .lt. icom) goto 70
         goto 55
      endif
c
c     find the section name
c
      if (word(1)(1:1) .eq. '*') then
         sename = word(1)(2:)
         word(1) = sename
         isec = 1
      endif
c
c
c     find if section ends
c
      if (word(1)(1:3) .eq. 'END') then
         iscend = 1
      endif
c
      return
c
500   ieof = 1
c
      return
c
      end

c
c
c*******************************************************************************
c  ICINT
c*******************************************************************************
c
      function icint(svalue)
c
c     Created:  April 20, 1999 by P. L. Fast
c     Revised:
c     Purpose:  Convert a string into an integer
c
c     Called by:  rmlgen,rmlopt,
c     Calls:  fchar, fspace
c
      use input, only : outfile
      implicit none
c
      integer icint, ibeg, ifin, iblnk, ierr, length, isign
      integer istart, i, n
      character*(*) svalue
      character ch
      length = len(svalue)
      if (length .eq. 0) then
         icint = 0
         return
      endif
c
c     find the first nonblank character
c
      ibeg = 1
      call fchar(svalue, ibeg, iblnk)
c
c     if it is a blank string set function to zero
c    
      if (iblnk .eq. 1) then
         icint = 0
         return
      endif
c
c     find the first blank character after the number
c
      ifin = ibeg+1
      call fspace(svalue, ifin, ierr)
      if (ierr .eq. 1) then
         ifin = length
      else
         ifin = ifin - 1
      endif
c
c     strip the blanks before and after the number
c
      svalue = svalue(ibeg:ifin)
      length = ifin - ibeg + 1
c
c
c     check for negative or positive sign (- or +)
c
      icint = 0
      istart = 2
      isign = +1
      if (svalue(1:1).eq.'+') then
         isign = +1
         ibeg = ibeg + 1
      elseif (svalue(1:1).eq.'-') then
         isign = -1
         ibeg = ibeg + 1
      else
         istart = 1
      endif
c
c     now convert each character to an integer - and store in icint
c
      do 70 i = ibeg,length
         ch = svalue(i:i)
         if (ch .ge. '0' .and. ch .le. '9') then
            n = ichar(ch) - ichar('0')
            icint = icint * 10 + n
         else
            write(outfile,1000) svalue
            stop 'icint 1'
         endif
70    continue
      icint = icint * isign
c
      return
1000  format(/3x,'ERROR: illegal digit in an integer: ',A80)
      end

c  
c*******************************************************************************
c  FCHAR
c*******************************************************************************
c  
      subroutine fchar(string, istrt, ierr)
c               
c     Created:  June 18, 1999 by J. M. Rodgers
c     Revised:
c     Purpose:  Find the next character on the line
c
c     Description:  Scrolls through the positions in string starting
c         at position istrt.  Sets istrt to position of first character.
c         If there is no character left on the line, sets ierr to 1.
c
c     Called by:  rline, rlist, rword, cfloat, icint, rcoef
c     Calls:
c
      implicit none
c
      integer istrt, ierr, char, length
      character*(*) string
      ierr = 0
      char = 0
      length = len(string)
      do while (istrt .le. length .and. char .eq. 0)
         if (string(istrt:istrt) .ne. ' ') then
            char = 1
         elseif (istrt .eq. length) then
            ierr = 1
         endif
         istrt = istrt + 1
      enddo
c
c     Sets istrt back 1 in order to counteract increment at end
c       of do-while loop.
c
      istrt = istrt -1
      return
      end

c
c
c*******************************************************************************
c  FSPACE
c*******************************************************************************
c
      subroutine fspace(string, istrt, ierr)
c
c     Created:  June 18, 1999 by J. M. Rodgers
c     Revised:
c     Purpose:  Find the next space on the line
c
c     Description:  Scrolls through the positions in string starting
c        at position istrt.  Sets istrt to the position of the first
c        space.  Or if there is no spacebar left on the line, sets 
c        ierr to 1.
c
c     Called by:  rword, cfloat, icint, rvar, rcoef
c     Calls:  
c
      implicit none
c
      integer istrt, ierr, blnk, length
      character*(*) string
      ierr=0
      blnk = 0
      length = len(string)
      do while (istrt .le. length .and. blnk .eq. 0)
         if (string(istrt:istrt) .eq. ' ') then
            blnk = 1
         elseif (istrt .eq. length) then
            ierr = 1
         endif
         istrt = istrt + 1
      enddo
c
c     Sets istrt back 1 in order to counter act increment at end
c       of do while loop.
c
      istrt = istrt - 1
      return
      end
c
c
c***************************************************************************
c upcse 
c***************************************************************************
c
      function upcse(string)
c      
c     Purpose:  consistent case
c     
c     Created:  Feb. 22, 2004 by Hai Lin
c
c     Revised:
c
c     Description:  this function takes a string of 80 characters and
c                   converts all the lower case letters to upper case.
c
c     Called by: 
c     Calls:
C   Function which takes a string of 80 characters and converts the 
C   lower case letters in the string to upper case letters
C   This functions is a modified version of CASE which was written
C   by Rozeanne Steckler
C
      implicit none 
c
c  i/o variables
c
      character*80 string, upcse
c      
c  local variables
c
      integer  itry, i
      character*80 line
      character * 1 xlett
c
      line = string
      do 10 i = 1, 80
         xlett = line(i:i)
         itry = ichar (xlett)
         if (xlett .ge. 'a' .and. xlett .le. 'z') then 
            itry = itry - 32
            line(i:i) = char (itry)
         endif
10    continue
c
      upcse  = line 
c
      return
c
      end
