#!/usr/bin/perl
#
#  Created by  Hai Lin,  Nov. 11 2004

# This is to read in the energy, gradient, and Hessians from the summary
#  file in qmmm Hessian job, and to converte these data in a Gaussian03
#  format so that can be used for a ts optimization.
#
# The motivation of doing so is the observation that Gaussian03 optimization 
#  with the external option will always calculation Hessian numerically!
#
# We want to feed the program with the analytically calculated Hessian!
#
##########################################################################

#                    ##################
#                    #                #  
###################### Read Me First! ####################################
#                    #                #
#                    ##################
#
# To use, follow the steps:

#  1. copy the file <testxxxx.sum> to a new subdir, where xxxx is the index
#  2. in the file <testxxxx.sum>, replace "D+" by "E+", "D-" by "E-"
#  3. run the script by typing "qmmmhess2g03.pl xxxx"
#  4. check the outputfile <testxxxx.hesstog03>, delete unwanted chacters.
#  5. now you can copy and paste 
#
###########################################################################


$infilename=$ARGV[0];
#$outfilename=$ARGV[1];

open(FILE4, "<test$infilename.sum");
open(FILE5, ">test$infilename.hesstog03");
@lines10=(<FILE4>);
close FILE4;

$leng  = 0;
$lgrad = 0;
$lhess = 0;

for $i (1 .. @lines10){
  $lines10[$i] =~tr/a-z/A-Z/;
  @info10 = ($lines10[$i] =~ /\S+/g);
  if ($info10[1] eq "ENERGY")  {
     $leng  = 1;
     $engstart = $i;
     $eng   = $info10[2];
  }
  if ($info10[1] eq "GRADIENTS")  {
     $lgrad  = 1;
     $gradstart = $i+1;
  }
  if ($info10[1] eq "FORCE")     { 
     $lhess = 1; 
     $hessstart = $i+1;
  }
}

$j = 0;
for $i ($gradstart .. $hessstart - 3){
  $lines10[$i] =~tr/a-z/A-Z/;
  @info10 = ($lines10[$i] =~ /\S+/g);
  $grad[$j+0] = $info10[0];
  $grad[$j+1] = $info10[1];  
  $grad[$j+2] = $info10[2];  
  $j++; $j++; $j++;
}

printf (FILE5 "! %20.14e\n",$eng);
for $jj (0 .. $j/6){
  printf (FILE5 "!");
  for $kk (0 .. 5)  {
    printf (FILE5 "%12.8f",$grad[$jj*6+$kk]);
  }
  printf (FILE5 "\n");
}
printf (FILE5 "Hess\n");
  
$j = 0;
for $i ($hessstart .. @lines10 -1){
  $lines10[$i] =~tr/a-z/A-Z/;
  @info10 = ($lines10[$i] =~ /\S+/g);
  $hess[$j+0] = $info10[0];
  $hess[$j+1] = $info10[1];  
  $hess[$j+2] = $info10[2];  
  $j++; $j++; $j++;
}
for $jj (0 .. $j/6){
  printf (FILE5 "!");
  for $kk (0 .. 5)  {
    printf (FILE5 "%12.8f",$hess[$jj*6+$kk]);
  }
  printf (FILE5 "\n");
} 

# 
# End of script
#
##########################################################################
