#!/bin/csh
#
# Usage run.ml 
#
# This can be used as a template for writing scripts for QMMM runs
#
# Revised May 7, 2009 by Hai Lin
# 
#                        ##################
#                        #                #  
##########################  Set the Name  ################################
#                        #                #
#                        ##################
#
# Note! You need to change the name here, e.g., test4002
#
   set name = test4002
#
# You also need to change the name later 
#
##########################################################################
#
#                    #########################
#                    #                       #  
###################### Set Working Directory #############################
#                    #                       #
#                    #########################

if ($#argv != 0) then
   set actdir = `pwd`
   set wrkdir = $argv[1]
else
   set actdir = `pwd`
   set wrkdir = $actdir/$name
check:
   if (-e $wrkdir) then
      set tmp = $wrkdir/$name
      set wrkdir = $tmp
      goto check
   endif
   mkdir $wrkdir
endif

#                #########################################
#                #                                       #  
################## Copy Input Files to Working Directory #################
#                #                                       #
#                #########################################

   cp $name.dat $wrkdir

   /bin/cp  $name.prm $wrkdir/t41.prm
   /bin/cp  $name.crd $wrkdir/t41.crd
   
##########################################################################

#                ########################################
#                #                                      #  
################## Copy QMMM Files to Working Directory ##################
#                #                                      #
#                ########################################
   cd $wrkdir
   mkdir par

   set multidir = `cat ~/.qmmm_path2018`
   set scriptdir = $multidir/script
   set basisdir = $multidir/basis
   set exedir = $multidir/exe
   set exe = $exedir/qmmm.exe
#
#---1. Select QM Packages
#  
#  a) Copy the g03 shuttles if g03 is desired
#
   cp $scriptdir/g03shuttle $wrkdir/g03shuttle

#
#  b) Copy the orca shuttles if orca is desired
#
   cp $scriptdir/orcashuttle $wrkdir/orcashuttle

#
#  c) Copy the gamess shuttles if gamess is desired
#
   cp $scriptdir/gmsshuttle $wrkdir/gmsshuttle

#
#---2. Select MM Packages
#  
#  Copy the tinker shuttles 
#
   cp $scriptdir/t41shuttle $wrkdir/t41shuttle

#
#---3. Select Multi-level Optimization 
#  
# a) If the internal multi-level optimizer is desired, skip this step
#
# b) If the g03 optimzer is desired (through the external option)
#   
    cp $scriptdir/ex_shuttle $wrkdir/ex_shuttle         
    cp $scriptdir/Gau_External $wrkdir/Gau_External
#
#    If g03 is the c01 version, you need one more (Gau_External2) script
#   
    cp $scriptdir/Gau_External2 $wrkdir/Gau_External2
#
#---4. Copy the basis sets
#
   cp $basisdir/*.gbs $wrkdir
   cp $basisdir/*.gbs $wrkdir/par

##########################################################################

#                #########################################
#                #                                       #  
##################      Now the QM/MM Calculation        #################
#                #                                       #
#                #########################################
#
# Note! You need to change the name here again, e.g., test4002
#
   foreach i (test4002)
      cp ${i}.dat ml.inp
      (time $exe) >& ${i}.jobs
      echo " " >> ${i}.jobs
      
      mv ml.out ${i}.out  >>& /dev/null
      mv ml.sum ${i}.sum  >>& /dev/null
      rm extopt.inp >>& /dev/null
      mv extopt.out ${i}.extopt >>& /dev/null
      mv g03.out ${i}.g03out >>& /dev/null
      mv g03.inp.1 ${i}.g03inp >>& /dev/null
      mv Test.FChk ${i}.fchk >>& /dev/null
      rm g03.inp.* g03.out.* Test.FChk.* >>& /dev/null
      rm ml.inp >>& /dev/null
   end

#
# Clean Up
#
   rm $wrkdir/g03shuttle  >>& /dev/null
   rm $wrkdir/*.gbs  >>& /dev/null
   rm $wrkdir/*.g03inp *.g03out *.jobs *.fchk  >>& /dev/null
   rm -Rf par
   mv $wrkdir/*.* $actdir
   cd $actdir
   rm -rf $wrkdir
#
#  End of script
#
##########################################################################
