c
c
c*****************************************************************************
c  PARTITION
c*****************************************************************************
c
      subroutine partition(progrms, geom, mcoptl,
     *   ltest, ltestmm, mmgeom, lqmmm, qmmminp,
     *   tmeth, tbas, toptl, ierr, dyninp, stepnow)
c
c     Subroutine partition
c
c     Created:  Jul. 19, 2010 by Soroosh Pezeshki
c
c     Purpose:  Set the groups and calls the ap subprograms
c
c     Description:  
c
c
c     Called by:  runmd
c
c     Calls:  
c
c     Variables defined here:
c
      use input
      use files,    only : outfile
      use caltyp,   only : tdebug
      use gradient, only : qmmmgrad, totalgrad
      use energy,   only : qmmmener
      use optimize, only : ener

      implicit none
c      
c  i/o variables
c also add mmgeom definition below                                      
c
      type (proglist) :: progrms
      integer            ierr, limohc, ltest, ltestmm, lqmmm, stepnow
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      character*20    :: tmeth, tbas
c
      character*80    :: mcoptl(maxoptl), toptl(maxoptl)
      type (qmmminf)  :: qmmminp                                        
      type (dynamicsinf) :: dyninp
      double precision ri(maxatm), riv(3*maxatm)

c
c calculate the distance of the groups to the active center
c and set the buffer and active groups
c
      ri(:) = 0.d0
      call calcgroupcom(geom%natoms, dyninp, geom, ri, riv, qmmminp)
      call setgroupaffiliation(geom, qmmminp, ri, dyninp, mmgeom)

c
c Sorted AP
c
      if (dyninp%apmethode .eq. 1) then
        call sortedap(progrms, mmgeom, geom, dyninp, qmmminp,
     *                   ri, riv, totalgrad, ener)
c
c Permuted AP
c
      elseif (dyninp%apmethode .eq. 2) then
        call permutedap(progrms, mmgeom, geom, dyninp, qmmminp,
     *                   ri, riv, totalgrad, ener)
c
c  Hot spot or no buffer  
c
      elseif (dyninp%apmethode .eq. 3) then
        call hotspot(progrms, geom, mcoptl,
     *   ltest, ltestmm, mmgeom, lqmmm, qmmminp,
     *   tmeth, tbas, toptl, ierr, dyninp, ri, totalgrad, ener)
c
c  ONIOM-XS                                                                 
c
      elseif (dyninp%apmethode .eq. 4) then
        call oniomxs(progrms, geom, mcoptl,
     *   ltest, ltestmm, mmgeom, lqmmm, qmmminp, tmeth, tbas, 
     *   toptl, ierr, dyninp, ri, riv, totalgrad, ener)

! c
! c  Differential-Based Adaptive Solvation                                    
! c
!       elseif (dyninp%apmethode .eq. 5) then
!         call das(progrms, mmgeom, geom, dyninp, qmmminp, 
!      *                      ri, totalgrad, ener, stepnow)
! c
! c Sorted AP with book keeping
! c
!       elseif (dyninp%apmethode .eq. 7) then
!         call sortedcorr(progrms, mmgeom, geom, dyninp, qmmminp,
!      *                   ri, riv, totalgrad, ener, stepnow)
      endif

c
c AP test output
c
      if(dyninp%LiTest.gt.0) then
        call litest(geom, ener)
      endif


100   return
      end

c
c
c*****************************************************************************
c  LITEST
c*****************************************************************************
c
      subroutine litest(geom, ener)
c
c     Subroutine litest
c
c     Created:  Feb. 25, 2011 by Soroosh Pezeshki
c
c     Purpose:  Output force and energy in the Li+ test
c
c     Description:  
c
c
c     Called by:  partition
c
c     Calls:  
c
c     Variables defined here:
c
      use input,    only : energyconvert, lengthconvert, cartgeom
      use caltyp,   only : tdebug
      use gradient, only : totalgrad

      implicit none
      type (cartgeom) :: geom
c
c lokale variables
c
      double precision force, distance, d3(3), force1, ener
      integer a, i, a3
c
c The steered O2
c
        a = 3 * 23
c
c Distance and vector O2 and Li+
c
        d3(1) = geom%angcoord(a - 2) - geom%angcoord(1)
        d3(2) = geom%angcoord(a - 1) - geom%angcoord(2)
        d3(3) = geom%angcoord(a) - geom%angcoord(3)
        distance = d3(1) * d3(1) + d3(2) * d3(2) + d3(3) * d3(3)
        distance = dsqrt(distance)
        d3(1:3) = d3(1:3) / distance
        distance = distance / lengthconvert
c
c output energy
c
        write(20,*) distance, ener * energyconvert
c
c Force on Li+
c
        force = totalgrad(1) * totalgrad(1) + totalgrad(2)
     *          * totalgrad(2) + totalgrad(3) * totalgrad(3)
        force = dsqrt(force) * energyconvert
        force1 = (d3(1) * totalgrad(1) + d3(2) * totalgrad(2)
     *              + d3(3) * totalgrad(3)) * energyconvert
        write(21,*) distance, -force1, force
c
c Force on O2
c
        force = totalgrad(a - 2) * totalgrad(a - 2) + totalgrad(a-1)
     *          * totalgrad(a - 1) + totalgrad(a) * totalgrad(a)
        force = dsqrt(force) * energyconvert
        force1 = (d3(1) * totalgrad(a - 2) + d3(2) * totalgrad(a - 1)
     *              +d3(3) * totalgrad(a)) * energyconvert
        write(22,*) distance, -force1, force
c
c Force on H1
c
        a = 3 * 12
        force = totalgrad(a - 2) * totalgrad(a - 2) + totalgrad(a - 1)
     *         * totalgrad(a - 1) + totalgrad(a) * totalgrad(a)
        force = dsqrt(force) * energyconvert
        force1 = (d3(1) * totalgrad(a - 2) + d3(2) * totalgrad(a - 1)
     *              + d3(3) * totalgrad(a)) * energyconvert
        write(23,*) distance,-force1, force

      end

c
c
c*****************************************************************************
c  SETGROUPAFFILIATION
c*****************************************************************************
c
      subroutine setgroupaffiliation (geom, qmmminp, ri, dyninp,
     *                           mmgeom)
c
c     Subroutine setgroupaffiliation
c
c     Created:  Jul. 19, 2010 by Soroosh Pezeshki
c
c     Purpose:  Set the QM atoms in the active and in the buffer zone
c
c     Description:  
c
c
c     Called by:  partitioning
c
c     Calls:  
c
c     Variables defined here:
c
      use input
      use files,    only : outfile
      use caltyp,   only : tdebug

      implicit none
c      
c  i/o variables
c also add mmgeom definition below                                      
c
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom      
      type (qmmminf)  :: qmmminp                                        
      type (dynamicsinf) :: dyninp
      double precision ri(maxatm)
c
c  local variables
c      
      integer a, ag, b, i, j, m, m3, ngrups, n3tm
      double precision rmax, activer, delta

c
c set the inital values
c
      ngrups = dyninp%maxgroups
!       delta = dyninp%partbuffer / 1.d3
      delta = 0.d0
      rmax = dyninp%activeradius + dyninp%partbuffer - delta
      activer = dyninp%activeradius + delta
      a  = 0
      ag = 0
      b  = 0
      n3tm = 3 * geom%natoms
      dyninp%ap_atom_zone(:) = 0
      qmmminp%qmatm(:) = 0
      qmmminp%qmcharge = 0
      dyninp%buffergroupps(:) = 0
c
c Loop over all groups
c
       do m = 1, dyninp%maxgroups
c
c if group is not allowed to be active or buffer cycle
c
        if (dyninp%ap_group_zone(m) .eq. -2) cycle
        dyninp%ap_group_zone(m) = 0
c
c set the QM atoms in the active zone
c
        if(ri(m) .lt. activer) then
            ag = ag + 1
            qmmminp%qmcharge = qmmminp%qmcharge+ dyninp%groupcharge(m)
            dyninp%ap_group_zone(m) = 1
            do i = 1, geom%natoms
                if (dyninp%group(i) .eq. m) then
                    a = a + 1
                    qmmminp%qmatm(a) = i
                    dyninp%ap_atom_zone(i) = 1
                endif
            enddo
            if (tdebug .gt. 0) then
                write(outfile,*) "Active zone group: ", m, ri(m)
            endif
c
c set the QMMM groups in the buffer zone
c
        elseif(ri(m) .lt. rmax) then
            b = b + 1
            dyninp%ap_group_zone(m) = 2
            if (b .gt. maxbuffergroupps) then
                write(outfile,*) 'Error: To many groups in buffer zone'
                stop 'partion 1'
            endif
            do i = 1, geom%natoms
                if (dyninp%group(i) .eq. m) then
                    dyninp%ap_atom_zone(i) = 2
                endif
            enddo
            dyninp%buffergroupps(b) = m
            if (tdebug .gt. 0) then
                write(*,*) "Buffer zone group: ", m, ri(m)
            endif
        endif
      enddo
c
c Set the number of QM atoms and buffer groups
c
      qmmminp%qmnatm = a
      dyninp%numberbuffergroupps = b
      dyninp%nagroups = ag
      if (dyninp%argontest.eq.0)
     *   call resetqmmm(qmmminp, geom, mmgeom, dyninp)
      if (tdebug .gt. 0) then
          write(outfile,*) ag, " active groups and ", 
     *              b, " buffer groups"
      endif
c
c Some statistics
c
      dyninp%averageactive = dyninp%averageactive + ag
      dyninp%averagebuffer = dyninp%averagebuffer + b
      if (dyninp%minbuffer .gt. b) dyninp%minbuffer = b
      if (dyninp%maxbuffer .lt. b) dyninp%maxbuffer = b
      if (dyninp%minactive .gt. ag) dyninp%minactive = ag
      if (dyninp%maxactive .lt. ag) dyninp%maxactive = ag


100   return
      end

c
c
c*****************************************************************************
c  CALCGROUPCOM
c*****************************************************************************
c
      subroutine calcgroupcom (natoms, dyninp, geom, ri,riv, qmmminp)
c
c     Subroutine calcgroupcom
c     
c     Created:  Jul. 23, 2010 by Soroosh Pezeshki
c
c     Purpose:  calculates the center of mass of each groups
c     Description: ri: distance center active zone - center of group
c                  riv: vector center active zone - center of group
c 
c     Called by:  partition
c     Calls:
c
c     Variables defined here:
c
      use input, only: cartgeom, dynamicsinf, maxatm, qmmminf
      implicit none
      type (dynamicsinf) :: dyninp
      type (cartgeom) :: geom
      type (qmmminf)  :: qmmminp
      integer natoms, active
      double precision riv(3*maxatm), com3(3*maxatm), ri(maxatm)
      double precision center(3)

c
c  local variables
c
      integer i, i3, j, m,m3
      double precision comsrm(3*maxatm)
c
c reset the variable and set variables
c
      com3(:)   = 0.d0
      comsrm(:) = 0.d0
      riv(:)     = 0.d0
      active = 3 * dyninp%activecenter
      center(1:3) = geom%angcoord(active - 2 : active)


c
c if center of mass is center of group
c
        if (dyninp%groupcom .eq. 1) then
c
c sum up the com for each coordinate
c
            do i = 1, natoms
                i3 = i * 3
                m = 3 * dyninp%group(i)
                comsrm(m - 2 : m) = comsrm(m - 2 : m) +
     *                    geom%atmmss(i) * geom%angcoord(i3 - 2 : i3)
            enddo
c
c divide the sum through the mass of the groups
c
            do m = 1, dyninp%maxgroups
                m3 = m * 3
                if (dyninp%groupmass(m) .ne. 0.d0) then
                    com3(m3-2:m3)=comsrm(m3-2:m3)/dyninp%groupmass(m)
                endif
                riv(m3 - 2 : m3) = com3(m3 - 2 : m3) - center(1 : 3)
                ri(m) = dsqrt(riv(m3-2)**2 +riv(m3-1)**2 +riv(m3)**2)
            enddo
c
c if atom is center of group
c
        else
            do m = 1, dyninp%maxgroups
                m3 = m * 3
                i = dyninp%groupatom(m)
                i3 = i * 3
                riv(m3-2:m3) = geom%angcoord(i3-2:i3) - center(1:3)
                ri(m) = dsqrt(riv(m3-2)**2 +riv(m3-1)**2 +riv(m3)**2)
            enddo
        endif

      end

c
c
c*****************************************************************************
c  QUICKSORT
c*****************************************************************************
c
      recursive subroutine quicksort(length, array1,array2,recurscall)
c
c     Subroutine quicksort
c     
c     Created:  Jul. 23, 2010 by Soroosh Pezeshki
c
c     Purpose:  sort the array1 and array2. 
c               The entrise are sorted from the smallest array1 to the highest.
c               array2 is sorted equal to array1. 
c               array1 is double precision. (contains the distances)
c               array2 is integer           (contains the group index)
c 
c     Called by:  partition
c     Calls:
c
c     Variables defined here:
c
      use input, only: maxbuffergroupps, maxrecursive
      implicit none

      integer length, array2(maxbuffergroupps)
      double  precision array1(maxbuffergroupps)
      integer i, l, g, temp2, recurscall
      integer less2(maxbuffergroupps), greater2(maxbuffergroupps)
      double  precision less1(maxbuffergroupps), temp1
      double  precision greater1(maxbuffergroupps)

c
c set inital values
c
      recurscall = recurscall + 1
      if (recurscall .ge. maxrecursive) then
            write(*,*) "ERROR: quicksort failed"
            STOP "Quicksort"
      endif
      l = 0
      g = 0
      less1(:) = 0.d0
      less2(:) = 0 
      greater1(:) = 0.d0
      greater2(:) = 0
      temp1 = array1(length)
      temp2 = array2(length)
c
c stop if the list contains only 1 element
c
        if (length .le. 1) return
        do i = 1, length - 1
c
c compare all values with the last one. Put the smaller values in less
c the bigger in greater 
c
            if (array1(i) .le. temp1) then
                l = l + 1
                less1(l) = array1(i)
                less2(l) = array2(i)
            else
                g = g + 1
                greater1(g) = array1(i)
                greater2(g) = array2(i)
            endif
        enddo
c
c Sort the sublist less and greater
c
        call quicksort(l, less1, less2, recurscall)
        call quicksort(g, greater1, greater2, recurscall)
c
c combine the sublists
c
        do i = 1, l
            array1(i) = less1(i)
            array2(i) = less2(i)
        enddo
        l = l + 1
        array1(l) = temp1
        array2(l) = temp2
        do i = 1, g
            array1(i + l) = greater1(i)
            array2(i + l) = greater2(i)
        enddo

      return
      end

c
c*****************************************************************************
c  ADDCAPATOMS
c*****************************************************************************
c
      subroutine addcapatoms(qmmminp, mmgeom, dyninp)
c
c     Subroutine addcapatoms
c     
c     Created:  March. 24, 2011 by Soroosh Pezeshki
c
c     Purpose:  searchs capatoms
c     Called by:  resetqmmm
c     Calls:
c
c     Variables defined here:
c
      use input, only : dynamicsinf, qmmminf, maxcapatm, mmvalen
      use files, only : outfile
      use caltyp, only : tdebug
      implicit none
       type (qmmminf) :: qmmminp
       type (mmvalen) :: mmgeom
       type (dynamicsinf) :: dyninp
c
c  local variables
c                 
      integer i, j, k, l, m, n, o

      m = 0

      do l = 1, qmmminp%qmnatm
        i = qmmminp%qmatm(l)
        do j = 1, mmgeom%atmvalen(i)
            k = mmgeom%atmvalenlist(i, j)
            do n = 1, qmmminp%qmnatm
                if (qmmminp%qmatm(n) .eq. k) goto 300
            enddo
            m = m + 1
            if (m .gt. maxcapatm) then
                write(outfile,*) 'ERROR: Number of',
     *              ' capatoms is too high. ',
     *              'Please, reduce the number of groups.'
                stop 'possiblecapatoms 1'
            else
                 write(outfile,*) '     adding cap atom', k
            endif
            qmmminp%capatm%capatm(m) = k
            do o = 1, dyninp%capatm%nlist
                if (k .eq. dyninp%capatm%list(o)) then
                    qmmminp%capatm%capatmtypname(m) =
     *                  dyninp%capatm%typname(o)
                    qmmminp%capatm%capatmtyp(m) =
     *                  dyninp%capatm%atmtyp(o)
                    qmmminp%capatm%capbondscale(m) =
     *                  dyninp%capatm%bondscale(o)
                    goto 100
                endif
            enddo
            qmmminp%capatm%capatmtypname(m) =
     *              dyninp%capatm%defaulttypname
            qmmminp%capatm%capatmtyp(m) = dyninp%capatm%defaulttyp
            qmmminp%capatm%capbondscale(m) =
     *              dyninp%capatm%defaultbondscale
100         continue
            if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
                write(*,200) '     Cap atom: ',
     *          qmmminp%capatm%capatm(m), 
     *          qmmminp%capatm%capatmtypname(m),
     *          qmmminp%capatm%capatmtyp(m),
     *          qmmminp%capatm%capbondscale(m)
200             format(a, i5, ' ', a4, i4, f8.5)
            endif
300         continue
        enddo
      enddo
      qmmminp%capatm%capnatm = m

      end

c
c
c*****************************************************************************
c  READGROUPS
c*****************************************************************************
c
      subroutine readgroups(dyninp, geom, mmgeom)
c
c     Subroutine readgroups
c     
c     Created:  Jul. 27, 2010 by Soroosh Pezeshki
c
c     Purpose:  reads the group for partitioing from file 
c                    and calculates group mass 
c     Called by:  initmd
c     Calls:      readzeroe
c
c     Variables defined here:
c
      use input, only : dynamicsinf, mmvalen, cartgeom
      use files, only : groupfile, groupfilename, outfile
      implicit none
       type (dynamicsinf) :: dyninp
       type (cartgeom) :: geom
       type (mmvalen)  :: mmgeom
c
c  local variables
c                 
      integer i, isec, iscend,ieof, icint, j, k, l, m
      integer iiatom, nwords, molid, natoms
      integer ifin, ibeg, ifind, iasym, chindex
c
      character*80  words(40), string
      double precision   cfloat, sumcharge, tmp, tmp2

      natoms = geom%natoms

c      
c open the group file
c
      open (unit=groupfile, file=groupfilename, status='old',
     *      form='formatted', err=5000)

      call readln(groupfile, words, nwords, isec, ieof, iscend) 
c      
      if (iscend .eq. 1 .or. ieof .eq. 1) then
        write (*,'(/a)') 'ERROR: The group file is empty.'
        stop 'readgroup 1'
      endif
      iiatom = icint(words(2))
      if (iiatom.ne.natoms) then
         write(*,1300)
1300     format(/3x,'Error: please, check atom number in group file')
         stop 'readgroup 2'
      endif

      dyninp%maxgroups = icint(words(1))
c      call readln(groupfile, words, nwords, isec, ieof, iscend)
      call readln(groupfile, words, nwords, isec, ieof, iscend)

c      
c read group of ech atom:
c  group index, atom index, everything else is ignored 
c
      j = 1
      do while  ((ieof.eq.0) .and. (iscend.eq.0))
         if (ieof .eq. 1)  then
            write(*,1400)
1400    format(/3x,'Error: please check group file.',
     *              ' All atoms must be affiliated to a group.')
            stop 'readgroup 3'
         endif
         i = icint(words(2))
         k = icint(words(1))
         if (k .gt. dyninp%maxgroups) then
            write(*,*) 'ERROR: Group index can not be greather than',
     *          dyninp%maxgroups
            stop 'readgroup 4'
         elseif (k .lt. 1) then
            write(*,*) 'ERROR: Group index can not be less than 1'
            stop 'readgroup 5'
         endif
         if (i .gt. natoms) then
            write(*,*) 'ERROR: Atom index can not be greather than',
     *          natoms
            stop 'readgroup 8'
         elseif (i .lt. 1) then
            write(*,*) 'ERROR: Atom index can not be less than 1'
            stop 'readgroup 9'
         endif
         dyninp%group(i) = k
         j = j + 1

c
c set all atoms between the two numbers i and l as group k
c
         if (nwords .gt. 2) then
            l = icint(words(3))
            if (l .gt. natoms) then
                write(*,*) 'ERROR: 2nd atom index can not be',
     *               'greather than', natoms
                stop 'readgroup 10'
            elseif (l .lt. i) then
                write(*,*) 'ERROR: Atom index can not be less than',i
                stop 'readgroup 11'
            endif
            do m = i + 1, l
                dyninp%group(m) = k
            enddo
            j = j + l - i
         endif

c set the first atom in group k as the center of the group
         if (dyninp%groupatom(k) .eq. 0) dyninp%groupatom(k) = i

c read next line
         call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c check if all atoms have a group
c
      do j = 1, natoms 
         if (dyninp%group(j) .eq. 0)  then
            write(*,1500) j
1500    format(/3x,'ERROR: atom',i6,
     *              ' has no group number.')
            stop 'readgroup 6'
         endif
      enddo

c
c Read zero energies
c
      if (ieof.eq.0) 
     *    call readzeroe(dyninp)

c
c close group file
c
      close (groupfile)

c
c sum up the mass and the size of the groups
c
c        dyninp%groupsize(:)=0
        dyninp%groupmass(:) = 0.d0
        do i = 1, natoms
          molid = dyninp%group(i)
          dyninp%groupmass(molid) = dyninp%groupmass(molid)
     *              + geom%atmmss(i)
c          dyninp%groupsize(molid)=dyninp%groupsize(molid)+1
        enddo

c
c sum up the charge of the groups
c

        do k = 1, dyninp%maxgroups
            sumcharge = 0.d0
            do i = 1, natoms
                if (dyninp%group(i) .eq. k) then
                    sumcharge = sumcharge + mmgeom%atmcharge(i)
c       write(*,*) "atom charge", i, mmgeom%atmcharge(i)
                endif
            enddo
            dyninp%groupcharge(k) = Int(sumcharge)
c       write(*,*) "charge", k, dyninp%groupcharge(k), sumcharge
c
c if group has no mass delete zero energy and reset
c
            if (dyninp%groupmass(k) .le. 0.d0) then
                dyninp%ap_group_zone(k) = -2
                dyninp%ape0_1_qm(k) = 0.d0
                dyninp%ape0_1_mm(k) = 0.d0
            endif
        enddo

      return

5000  write (*,'(/a)') 'ERROR: Cannot open the group file.'
6000  stop 'readgroup 7'


      end

c
c
c*****************************************************************************
c  READZEROE
c*****************************************************************************
c
      subroutine readzeroe(dyninp)
c
c     Subroutine readzeroe
c     
c     Created:  Jul. 27, 2011 by Soroosh Pezeshki
c
c     Purpose:  reads the zero energies (infinite seperate energy)
c               of the group for partitioing from file 
c     Called by:  readgroups
c     Calls:
c
c     Variables defined here:
c
c
c are groups allowed to be active and buffer
c dyninp%ap_group_zone(i) = -2 : Group is always MM
c dyninp%ap_group_zone(i) = 0  : Group is at this timestep MM
c dyninp%ap_group_zone(i) = 1  : Group is at this timestep QM
c dyninp%ap_group_zone(i) = 2  : Group is at this timestep buffer
c dyninp%ap_group_zone(i) = 3  : buffer group is treated in this permutation as QM
c

      use input, only : dynamicsinf, maxbuffergroupps
      use files, only : groupfile, outfile
      implicit none
       type (dynamicsinf) :: dyninp

c
c  local variables
c                 
      integer i, isec, iscend,ieof, icint, j, k, l, m
      integer iiatom, nwords, molid
      integer ifin, ibeg, ifind, iasym, chindex
c
      character*80  words(40), string
      double precision   cfloat, tmp, tmp2


      dyninp%ap_group_zone(:) = -2
      dyninp%ape0_1_qm(:) = 0.d0
      dyninp%ape0_1_mm(:) = 0.d0
      dyninp%ape0_2_qm(:) = 0.d0
      dyninp%ape0_2_mm(:) = 0.d0
      dyninp%ape0_3_qm(:) = 0.d0
      dyninp%ape0_3_mm(:) = 0.d0
      dyninp%ape0_4_qm(:) = 0.d0
      dyninp%ape0_4_mm(:) = 0.d0
      dyninp%ape0_5_qm(:) = 0.d0
      dyninp%ape0_5_mm(:) = 0.d0
      dyninp%iape0_2(:,1) = 0
      dyninp%iape0_2(:,2) = 0
      dyninp%iape0_3(:,1) = 0
      dyninp%iape0_3(:,2) = 0
      dyninp%iape0_3(:,3) = 0
      dyninp%iape0_4(:,1) = 0
      dyninp%iape0_4(:,2) = 0
      dyninp%iape0_4(:,3) = 0
      dyninp%iape0_4(:,4) = 0
      dyninp%iape0_5(:,1) = 0
      dyninp%iape0_5(:,2) = 0
      dyninp%iape0_5(:,3) = 0
      dyninp%iape0_5(:,4) = 0
      dyninp%iape0_5(:,5) = 0


      call readln(groupfile, words, nwords, isec, ieof, iscend)

c      
c solo groups
c zero energy high level, zero energy low level, 
c
      do while ((ieof.eq.0) .and. (iscend.eq.0))
c      write(*,*) words(1), words(2)
c
c equal groups can be definesd in one line
c
        if (words(1) .eq. "E") then
            i = icint(words(2))
            l = icint(words(3))
            string = words(4)
            tmp2 = cfloat(words(5))
            dyninp%ape0_1_mm(i) = tmp2
            if (string(1:2) .eq. "NO") then
                dyninp%ap_group_zone(i) = -2
                tmp = 0.d0
            else 
                dyninp%ap_group_zone(i) = 0
                tmp = cfloat(words(4))
                dyninp%ape0_1_qm(i) = tmp
!                 if (nwords .gt. 5) then
!                     dyninp%iape0_2_mm(i) = icint(words(6))
!                 endif
            endif
            do m = i + 1, l
                dyninp%ape0_1_qm(m) = tmp
                dyninp%ape0_1_mm(m) = tmp2
                dyninp%ap_group_zone(m) = dyninp%ap_group_zone(i)
            enddo
            goto 2000
        endif

        i = icint(words(1))
        string = words(2)
        dyninp%ape0_1_mm(i) = cfloat(words(3))
        if (string(1:2) .eq. "NO") then
            dyninp%ap_group_zone(i) = -2
        else
            dyninp%ap_group_zone(i) = 0
            dyninp%ape0_1_qm(i) = cfloat(words(2))
        endif
2000    call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo


c
c zero energy of duo super groups
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         i = icint(words(1))
         l = icint(words(2))
         dyninp%iape0_2(j,1) = i
         dyninp%iape0_2(j,2) = l
         dyninp%ape0_2_qm(j) = cfloat(words(3))
         dyninp%ape0_2_mm(j) = cfloat(words(4))
3000     call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c zero energy of trio super groups
c (Groups 1-2-3 are QM or MM)
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         dyninp%iape0_3(j,1) = icint(words(1))
         dyninp%iape0_3(j,2) = icint(words(2))
         dyninp%iape0_3(j,3) = icint(words(3))
         dyninp%ape0_3_qm(j) = cfloat(words(4))
         dyninp%ape0_3_mm(j) = cfloat(words(5))
4000     call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c zero energy of quartet super groups
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         dyninp%iape0_4(j,1) = icint(words(1))
         dyninp%iape0_4(j,2) = icint(words(2))
         dyninp%iape0_4(j,3) = icint(words(3))
         dyninp%iape0_4(j,4) = icint(words(4))
         dyninp%ape0_4_qm(j) = cfloat(words(5))
         dyninp%ape0_4_mm(j) = cfloat(words(6))
         call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c zero energy of quintet super groups
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         dyninp%iape0_5(j,1) = icint(words(1))
         dyninp%iape0_5(j,2) = icint(words(2))
         dyninp%iape0_5(j,3) = icint(words(3))
         dyninp%iape0_5(j,4) = icint(words(4))
         dyninp%iape0_5(j,5) = icint(words(5))
         dyninp%ape0_5_qm(j) = cfloat(words(6))
         dyninp%ape0_5_mm(j) = cfloat(words(7))
         call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c zero energy of sextet super groups
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         dyninp%iape0_6(j,1) = icint(words(1))
         dyninp%iape0_6(j,2) = icint(words(2))
         dyninp%iape0_6(j,3) = icint(words(3))
         dyninp%iape0_6(j,4) = icint(words(4))
         dyninp%iape0_6(j,5) = icint(words(5))
         dyninp%iape0_6(j,6) = icint(words(6))
         dyninp%ape0_6_qm(j) = cfloat(words(7))
         dyninp%ape0_6_mm(j) = cfloat(words(8))
         call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c zero energy of septet super groups
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         dyninp%iape0_7(j,1) = icint(words(1))
         dyninp%iape0_7(j,2) = icint(words(2))
         dyninp%iape0_7(j,3) = icint(words(3))
         dyninp%iape0_7(j,4) = icint(words(4))
         dyninp%iape0_7(j,5) = icint(words(5))
         dyninp%iape0_7(j,6) = icint(words(6))
         dyninp%iape0_7(j,7) = icint(words(7))
         dyninp%ape0_7_qm(j) = cfloat(words(8))
         dyninp%ape0_7_mm(j) = cfloat(words(9))
         call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c zero energy of octet super groups
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         dyninp%iape0_8(j,1) = icint(words(1))
         dyninp%iape0_8(j,2) = icint(words(2))
         dyninp%iape0_8(j,3) = icint(words(3))
         dyninp%iape0_8(j,4) = icint(words(4))
         dyninp%iape0_8(j,5) = icint(words(5))
         dyninp%iape0_8(j,6) = icint(words(6))
         dyninp%iape0_8(j,7) = icint(words(7))
         dyninp%iape0_8(j,8) = icint(words(8))
         dyninp%ape0_8_qm(j) = cfloat(words(9))
         dyninp%ape0_8_mm(j) = cfloat(words(10))
         call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c zero energy of nonet super groups
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         dyninp%iape0_9(j,1) = icint(words(1))
         dyninp%iape0_9(j,2) = icint(words(2))
         dyninp%iape0_9(j,3) = icint(words(3))
         dyninp%iape0_9(j,4) = icint(words(4))
         dyninp%iape0_9(j,5) = icint(words(5))
         dyninp%iape0_9(j,6) = icint(words(6))
         dyninp%iape0_9(j,7) = icint(words(7))
         dyninp%iape0_9(j,8) = icint(words(8))
         dyninp%iape0_9(j,9) = icint(words(9))
         dyninp%ape0_9_qm(j) = cfloat(words(10))
         dyninp%ape0_9_mm(j) = cfloat(words(11))
         call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c zero energy of decet super groups
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         dyninp%iape0_10(j,1) = icint(words(1))
         dyninp%iape0_10(j,2) = icint(words(2))
         dyninp%iape0_10(j,3) = icint(words(3))
         dyninp%iape0_10(j,4) = icint(words(4))
         dyninp%iape0_10(j,5) = icint(words(5))
         dyninp%iape0_10(j,6) = icint(words(6))
         dyninp%iape0_10(j,7) = icint(words(7))
         dyninp%iape0_10(j,8) = icint(words(8))
         dyninp%iape0_10(j,9) = icint(words(9))
         dyninp%iape0_10(j,10) = icint(words(10))
         dyninp%ape0_10_qm(j) = cfloat(words(11))
         dyninp%ape0_10_mm(j) = cfloat(words(12))
         call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c zero energy of 11 super groups
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         dyninp%iape0_11(j,1) = icint(words(1))
         dyninp%iape0_11(j,2) = icint(words(2))
         dyninp%iape0_11(j,3) = icint(words(3))
         dyninp%iape0_11(j,4) = icint(words(4))
         dyninp%iape0_11(j,5) = icint(words(5))
         dyninp%iape0_11(j,6) = icint(words(6))
         dyninp%iape0_11(j,7) = icint(words(7))
         dyninp%iape0_11(j,8) = icint(words(8))
         dyninp%iape0_11(j,9) = icint(words(9))
         dyninp%iape0_11(j,10) = icint(words(10))
         dyninp%iape0_11(j,11) = icint(words(11))
         dyninp%ape0_11_qm(j) = cfloat(words(12))
         dyninp%ape0_11_mm(j) = cfloat(words(13))
         call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c zero energy of 12 super groups
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         dyninp%iape0_12(j,1) = icint(words(1))
         dyninp%iape0_12(j,2) = icint(words(2))
         dyninp%iape0_12(j,3) = icint(words(3))
         dyninp%iape0_12(j,4) = icint(words(4))
         dyninp%iape0_12(j,5) = icint(words(5))
         dyninp%iape0_12(j,6) = icint(words(6))
         dyninp%iape0_12(j,7) = icint(words(7))
         dyninp%iape0_12(j,8) = icint(words(8))
         dyninp%iape0_12(j,9) = icint(words(9))
         dyninp%iape0_12(j,10) = icint(words(10))
         dyninp%iape0_12(j,11) = icint(words(11))
         dyninp%iape0_12(j,12) = icint(words(12))
         dyninp%ape0_12_qm(j) = cfloat(words(13))
         dyninp%ape0_12_mm(j) = cfloat(words(14))
         call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c zero energy of 13 super groups
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         dyninp%iape0_13(j,1) = icint(words(1))
         dyninp%iape0_13(j,2) = icint(words(2))
         dyninp%iape0_13(j,3) = icint(words(3))
         dyninp%iape0_13(j,4) = icint(words(4))
         dyninp%iape0_13(j,5) = icint(words(5))
         dyninp%iape0_13(j,6) = icint(words(6))
         dyninp%iape0_13(j,7) = icint(words(7))
         dyninp%iape0_13(j,8) = icint(words(8))
         dyninp%iape0_13(j,9) = icint(words(9))
         dyninp%iape0_13(j,10) = icint(words(10))
         dyninp%iape0_13(j,11) = icint(words(11))
         dyninp%iape0_13(j,12) = icint(words(12))
         dyninp%iape0_13(j,13) = icint(words(13))
         dyninp%ape0_13_qm(j) = cfloat(words(14))
         dyninp%ape0_13_mm(j) = cfloat(words(15))
         call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c zero energy of 14 super groups
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         dyninp%iape0_14(j,1) = icint(words(1))
         dyninp%iape0_14(j,2) = icint(words(2))
         dyninp%iape0_14(j,3) = icint(words(3))
         dyninp%iape0_14(j,4) = icint(words(4))
         dyninp%iape0_14(j,5) = icint(words(5))
         dyninp%iape0_14(j,6) = icint(words(6))
         dyninp%iape0_14(j,7) = icint(words(7))
         dyninp%iape0_14(j,8) = icint(words(8))
         dyninp%iape0_14(j,9) = icint(words(9))
         dyninp%iape0_14(j,10) = icint(words(10))
         dyninp%iape0_14(j,11) = icint(words(11))
         dyninp%iape0_14(j,12) = icint(words(12))
         dyninp%iape0_14(j,13) = icint(words(13))
         dyninp%iape0_14(j,14) = icint(words(14))
         dyninp%ape0_14_qm(j) = cfloat(words(15))
         dyninp%ape0_14_mm(j) = cfloat(words(16))
         call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

c
c zero energy of 15 super groups
c
      if (ieof.eq.0) 
     *    call readln(groupfile, words, nwords, isec, ieof, iscend)
      j = 0
      do while ((ieof.eq.0) .and. (iscend.eq.0))
         j = j + 1
         dyninp%iape0_15(j,1) = icint(words(1))
         dyninp%iape0_15(j,2) = icint(words(2))
         dyninp%iape0_15(j,3) = icint(words(3))
         dyninp%iape0_15(j,4) = icint(words(4))
         dyninp%iape0_15(j,5) = icint(words(5))
         dyninp%iape0_15(j,6) = icint(words(6))
         dyninp%iape0_15(j,7) = icint(words(7))
         dyninp%iape0_15(j,8) = icint(words(8))
         dyninp%iape0_15(j,9) = icint(words(9))
         dyninp%iape0_15(j,10) = icint(words(10))
         dyninp%iape0_15(j,11) = icint(words(11))
         dyninp%iape0_15(j,12) = icint(words(12))
         dyninp%iape0_15(j,13) = icint(words(13))
         dyninp%iape0_15(j,14) = icint(words(14))
         dyninp%iape0_15(j,15) = icint(words(15))
         dyninp%ape0_15_qm(j) = cfloat(words(16))
         dyninp%ape0_15_mm(j) = cfloat(words(17))
         call readln(groupfile, words, nwords, isec, ieof, iscend)
      enddo

      end


c
c******************************************************************************
c  HOTSPOT
c******************************************************************************
c
      subroutine hotspot(progrms, geom, mcoptl,
     *   ltest, ltestmm, mmgeom, lqmmm, qmmminp,
     *   tmeth, tbas, toptl, ierr, dyninp, ri, grad, ener)
c  
c     Subroutine hotspot
c
c     Created:  Jul. 28, 2010 Soroosh Pezeshki
c
c     Purpose:  To calculate the QM/MM gradient with the hot spot method 
c     Description: Rode's "hot spot" methode; V=V_MM_ALL+V_QM_A+B-V_MM_A+B
c                           F_i=F_MM_ALL_i-S(r_i)*(F_QM_A+B_i-F_MM_A+B_i)
c
c     Called by:  partitioning
c     Calls:  imohcghk, mg3ghk, 
c             mqcghk, mcomp2ghk, mccoghk, progghk, progmmghk   
c          
c
      use input,    only : cartgeom, proglist, mmvalen,        
     *                     qmmminf, maxoptl, dynamicsinf, maxatm
      use files,    only : outfile
      use gradient, only : qmmmgrad   
      use energy,   only : qmmmener

      implicit none
c      
c  i/o variables
c also add mmgeom definition below                                      
c
      type (proglist) :: progrms
      integer            ierr, limohc, ltest, ltestmm, lqmmm            
      type (cartgeom) :: geom, geom1
      type (mmvalen)  :: mmgeom, mmgeom1
      character*20    :: tmeth, tbas
c
      character*80    :: mcoptl(maxoptl), toptl(maxoptl)
      type (qmmminf)  :: qmmminp                                        
      type (dynamicsinf) :: dyninp
      double precision ri(maxatm)
      double precision ener, grad(3*maxatm)
c
c  local variables
c
      integer n3tm, i, m,  a, j
      double precision gradpart(3*maxatm), gradmm(3*maxatm)
      double precision s(maxatm), rmin, rmax
      double precision enerpart, enermm, enerqm, zeroenergy
c      double precision hlzeroenergy, llzeroenergy
      
      rmin = dyninp%activeradius
      rmax = dyninp%activeradius + dyninp%partbuffer
      n3tm = 3 * geom%natoms
      s(:)        = 1.d0
      grad(:)     = 0.d0
      qmmmgrad(:) = 0.d0
      gradpart(:) = 0.d0
      gradmm(:)   = 0.d0
      zeroenergy  = 0.d0

c
c add the buffer atoms to the QM atoms
c
        a = qmmminp%qmnatm
        do j = 1, geom%natoms
            if (dyninp%ap_atom_zone(j) .eq. 2) then
                a = a + 1
                qmmminp%qmatm(a) = j
            endif
        enddo
        qmmminp%qmnatm = a
        if (dyninp%argontest .eq. 0) 
     *      call resetqmmm(qmmminp, geom, mmgeom, dyninp)

        do i = 1, a
            j = qmmminp%qmatm(i)
            geom1%atm(i) = geom%atm(j)
            geom1%atmmss(i) = geom%atmmss(j)
            geom1%angcoord(3*i-2 : 3*i) = geom%angcoord(3*j-2 : 3*j)
            mmgeom1%atmcharge(i) = mmgeom%atmcharge(j)
            mmgeom1%atmid(i) = mmgeom%atmid(j)
            mmgeom1%atmtypname(i) = mmgeom%atmtypname(j)
            mmgeom1%atmtyp(i) = mmgeom%atmtyp(j)
            mmgeom1%atmvalen(i) = mmgeom%atmvalen(j)
            mmgeom1%atmvalenlist(i,:) = mmgeom%atmvalenlist(j,:)
        enddo
        geom1%charge = geom%charge
        geom1%mult = geom%mult
        geom1%natoms = a
        mmgeom1%natoms = a
        mmgeom1%lprnatmprm = mmgeom%lprnatmprm

c
c Calculate the scaling parameter S(r) for the buffer group
c and calculate the qmcharge
c
        do j = 1, dyninp%numberbuffergroupps
            m = dyninp%buffergroupps(j)
            dyninp%ap_group_zone(m) = 3
            s(m) = (rmax**2 + 2.d0 * ri(m)**2 - 3.d0 * rmin**2)*
     *            (rmax**2 - ri(m)**2)**2 / (rmax**2 - rmin**2)**3
            qmmminp%qmcharge = qmmminp%qmcharge+dyninp%groupcharge(m)
        enddo
c
c TEST with argon atoms
c
        if (dyninp%argontest .gt. 0) then
            call mmargontest(qmmminp,2,geom1,dyninp,gradpart,enerpart)
            call mmargontest(qmmminp, 2,geom,dyninp,gradmm,enermm)
            call mmargontest(qmmminp, 1,geom,dyninp,qmmmgrad,enerqm)
            ener = enermm + enerqm - enerpart
c
c Calculate the gradient
c
            grad(1:n3tm) = gradmm(1:n3tm)
            do j = 1, qmmminp%qmnatm
                i = qmmminp%qmatm(j)
                m = dyninp%group(i)
                i = 3 * i
                grad(i-2 : i) = gradmm(i-2 : i) + s(m) * 
     *              (qmmmgrad(i-2 : i) - gradpart(j*3-2 : j*3))
            enddo

c
c QM/MM
c
        else
            call calculatezeroenergy (dyninp, zeroenergy)
c
c MM gradient all
c
            call progmmghkmd(1,progrms%testmmprog, tmeth, tbas, geom,
     *          mmgeom, toptl, enermm, gradmm(:n3tm), ierr, 'null')
! c
! c MM gradient active + buffer
! c
!             call progmmghkmd(1,progrms%testmmprog, tmeth, tbas, geom1,
!      *          mmgeom1, toptl, enerpart, gradpart, ierr, 'null')
c
c QM/MM gradient of active+buffer / all
c

            call qmmmghkmd(progrms%qmprog, progrms%mmprog, geom,
     *             mmgeom, qmmminp, ierr, qmmmgrad, qmmmener)
            ener = qmmmener - zeroenergy
            dyninp%zeroenergy = zeroenergy
c
c Calculate the gradient
c
            grad(1:n3tm) = gradmm(1:n3tm)
            do j=1, qmmminp%qmnatm
                i = qmmminp%qmatm(j)
                m = dyninp%group(i)
                i = 3 * i 
                grad(i-2:i) = (1.d0 - s(m)) * gradmm(i-2:i) +
     *              s(m) * (qmmmgrad(i-2:i))
            enddo

        endif


100   return
      end

c
c
c*****************************************************************************
c  SMOOTHINGFUNCTION
c*****************************************************************************
c
      subroutine smoothingfunction (dyninp, ri, pi, dpi)
c
c     Subroutine smoothingfunction
c
c     Created:  Aug. 24, 2010 by Soroosh Pezeshki
c
c     Purpose:  Calculates the fifth order spline 
c               and the gradient
c
c     Called by:  sortedAP, oniomxs, permutedAP
c
c     Calls:  fifthorderspline
c
c     Variables defined here:
c
      use input, only : dynamicsinf, maxatm, maxbuffergroupps
      use caltyp, only : tdebug
      implicit none
c
c  i/o variables
c
      type (dynamicsinf)  :: dyninp
      double precision ri(maxatm), fos, dfos
      double precision pi(maxbuffergroupps)
      double precision dpi(maxbuffergroupps)
c
c  local variables
c      
      integer i, j, numbufgr
      double precision rmin, alpha

      rmin = dyninp%activeradius
      pi(:)  = 0.d0
      dpi(:) = 0.d0
      numbufgr = dyninp%numberbuffergroupps
c
c calculate the smoothing function and the derivative of the smoothing function
c
        do i = 1, numbufgr
            j = dyninp%buffergroupps(i)
            alpha = (ri(j) - rmin) / dyninp%partbuffer
            call fifthorderspline(alpha, fos, dfos)
            pi(i) = fos
            dpi(i) = dfos / (ri(j) * dyninp%partbuffer)
        enddo
c
c output for debugging
c
        if (((tdebug.eq.5).or.(tdebug.eq.1)).and.(numbufgr.gt.0)) then
            write(*,110) 'Buffer g','group', 'r', 'alpha', 'Pi', 'dPi'
            do i = 1, numbufgr
                j = dyninp%buffergroupps(i)
                alpha = (ri(j) - rmin) / dyninp%partbuffer
                write(*,111)  i, j, ri(j), alpha, pi(i), dpi(i)
            enddo
110     format(t5,a8,t13,a5,t25,a1, t30, a5, t43, a2, t52, a3) 
111     format(5x,I3,t10,I5,t20,f8.5,t30,f8.5,t40,f8.5,t50,d20.9)
        endif

      end

c
c******************************************************************************
c FIFTHORDERSPLINE
c******************************************************************************
c
      subroutine fifthorderspline(alpha, fos, dfos)
c  
c     subroutine fifthorderspline
c
c     Created:  Jan. 31, 2011 Soroosh Pezeshki
c
c     Purpose:  calculates the fifth order spline and the derivative
c
c     Called by:  SMOOTHINGFUNCTION, MMARGONTEST
c     Calls:    
c     
c
        double precision fos, dfos, alpha, tmp1, tmp2, tmp3
c
c tmp1= alpha^2, tmp3 = -30*alpha^2
c
        tmp1 = alpha * alpha
        tmp3 = -30.d0 * tmp1
c
c tmp1= alpha^3, tmp2 = -10*alpha^3, tmp3 = 60*alpha^3-30*alpha^2
c
        tmp1 = alpha * tmp1
        tmp2 = -10.d0 * tmp1
        tmp3 = tmp3 + 60.d0 * tmp1
c
c tmp1= alpha^4, tmp2=15*alpha^4-10*alpha^3,
c dfos = -30*alpha^4+60*alpha^3-30*alpha^2
c
        tmp1 = tmp1 * alpha
        tmp2 = tmp2 + 15.d0 * tmp1
        dfos = tmp3 - 30.d0 * tmp1
c
c fos=-6*alpha^5+15*alpha^4-10*alpha^3+1
c
        fos = tmp2 - 6.d0 * tmp1 * alpha + 1.d0
      end


c
c******************************************************************************
c  ONIOMXS
c******************************************************************************
c
      subroutine oniomxs(progrms, geom, mcoptl,
     *   ltest, ltestmm, mmgeom, lqmmm, qmmminp, tmeth, tbas, 
     *   toptl, ierr, dyninp, ri, riv, grad, ener)
c  
c     Subroutine oniomxs
c
c     Created:  Jul. 29, 2010 Soroosh Pezeshki
c
c     Purpose:  To calculate the QM/MM gradient with the ONIOM-XS method 
c     Description: Morokuma's "ONIOM-XS" methode
c
c     Called by:  partitioning
c     Calls:  imohcghk, mg3ghk, 
c             mqcghk, mcomp2ghk, mccoghk, progghk, progmmghk   
c     
c
      use input
      use caltyp,   only : tdebug
      use files,    only : outfile
      use gradient, only : qmmmgrad   
      use energy,   only : qmmmener

      implicit none
c      
c  i/o variables
c also add mmgeom definition below                                      
c
      type (proglist) :: progrms
      integer            ierr, limohc, ltest, ltestmm, lqmmm
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      character*20    :: tmeth, tbas
c
      character*80    :: mcoptl(maxoptl), toptl(maxoptl)
      type (qmmminf)  :: qmmminp, qmmminp1
      type (dynamicsinf) :: dyninp
      double precision ri(maxatm), riv(3*maxatm)
      double precision dpi(maxbuffergroupps), pi(maxbuffergroupps)
      double precision ener, grad(3*maxatm) !, qmmmgrad(3*maxatm)

c
c  local variables
c
      integer i, m, a, j, n, b, i3, n3
      integer n3tm, numbufgr
      double precision gradmm(3*maxatm), gradpart(3*maxatm)
      double precision grada(3*maxatm), gradab(3*maxatm)
      double precision enermm, enera, enerpart, enerqm, enerab
      double precision alpha, p, rmin, rmax, temp, temp1
c      double precision hlzeroenergy, llzeroenergy, 
      double precision zeroenergy

c
c inital values
c
      rmin = dyninp%activeradius
      rmax = dyninp%activeradius + dyninp%partbuffer
      n3tm = 3 * geom%natoms
      numbufgr = dyninp%numberbuffergroupps
      enermm      = 0.d0
      p           = 0.d0
      gradmm(:)   = 0.d0
      grad(:)     = 0.d0
      qmmmgrad(:) = 0.d0
      gradpart(:) = 0.d0
      grada(:)    = 0.d0
      gradab(:)   = 0.d0

c
c calculate the fifth order spline and the gradient
c
        call smoothingfunction(dyninp, ri, pi, dpi)

c
c Create qminpab with active + buffer
c
        if (numbufgr .gt. 0) then
            do j = 1, numbufgr
                p = p + pi(j)
            enddo
            p = p / (1.d0 * numbufgr)
            b = 0
            qmmminp1%qmatm(:) = 0
            do j = 1, geom%natoms
                if (dyninp%ap_atom_zone(j) .gt. 0) then
                    b = b + 1
                    qmmminp1%qmatm(b) = j
                endif
            enddo
            qmmminp1%qmnatm=  b
        endif
c
c TEST: ARGON atoms with own MM
c
        if (dyninp%argontest .gt. 0) then
            call mmargontest(qmmminp, 2, geom, dyninp, gradmm, enermm)

c
c TEST ARGON atoms: QM active zone, MM rest
c
            call mmargontest(qmmminp,3,geom,dyninp,gradpart,enerpart)
            call mmargontest(qmmminp,1,geom,dyninp,qmmmgrad,enerqm)
            enera = enermm + enerqm - enerpart
            grada(:) = gradmm(:)
            do a = 1, qmmminp%qmnatm
                i = 3 * qmmminp%qmatm(a)
                grada(i-2:i) = gradmm(j-2:j) +
     *              (qmmmgrad(i-2:i) - gradpart(i-2:i))
            enddo

c
c TEST ARGON atoms: QM gradient active+buffer zone, enviroment MM
c
            if (numbufgr .gt. 0) then
                qmmmgrad(:) = 0.d0
                gradpart(:) = 0.d0
                call mmargontest(qmmminp1, 3, geom, dyninp, gradpart,
     *                      enerpart)
                call mmargontest(qmmminp1, 1, geom, dyninp, qmmmgrad,
     *                      enerqm)
                enerab = enermm + enerqm - enerpart
                gradab(:) = gradmm(:)
                do b = 1, qmmminp1%qmnatm
                    j = 3 * qmmminp1%qmatm(b)
                    gradab(j-2:j) = gradmm(j-2:j) +
     *                  (qmmmgrad(j-2:j) - gradpart(j-2:j))
                enddo
            endif

        else
c
c QM calculation active zone, MM rest
c
            rmax = dyninp%activeradius
            call calculatezeroenergy (dyninp, zeroenergy)
            call qmmmghkmd(progrms%qmprog, progrms%mmprog, geom,
     *             mmgeom, qmmminp, ierr, qmmmgrad, qmmmener)
            enera = qmmmener - zeroenergy
            grada(1:n3tm) = qmmmgrad(1:n3tm)
            dyninp%zeroenergy = zeroenergy * (1.d0 - p)

c
c QM calculation active+buffer zone
c
            if (numbufgr .gt. 0) then
                qmmminp1%qmcharge = qmmminp%qmcharge
                do j = 1, numbufgr
                    m = dyninp%buffergroupps(j)
                    qmmminp1%qmcharge = qmmminp1%qmcharge + 
     *                  dyninp%groupcharge(m)
                enddo
                call copyqminp(qmmminp, qmmminp1)
                call resetqmmm(qmmminp1, geom, mmgeom, dyninp)
                rmax = dyninp%activeradius + dyninp%partbuffer
                do m = 1, numbufgr
                    i = dyninp%buffergroupps(m)
                    dyninp%ap_group_zone(i) = 3
                enddo
                call calculatezeroenergy (dyninp, zeroenergy)
                call qmmmghkmd(progrms%qmprog, progrms%mmprog,
     *             geom, mmgeom, qmmminp1, ierr, qmmmgrad, qmmmener)
                enerab = qmmmener - zeroenergy
                dyninp%zeroenergy = dyninp%zeroenergy +
     *              zeroenergy * p
                gradab(1:n3tm) = qmmmgrad(1:n3tm)
            endif
        endif


c
c calculate the potential and the gradient
c
        a = 3 * dyninp%activecenter
        grad(1:n3tm) = p * gradab(1:n3tm) + (1.d0-p) * grada(1:n3tm)
        ener = p * enerab + (1.d0 - p) * enera
c
c Debug output
c
        if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
            write(*,*) 'p: ', p
            write(*,*) 'Energy: ', ener
            write(*,*) 'Enerab: ', enerab
            write(*,*) 'Enera: ', enera
c            write(*,*) llzeroenergy, hlzeroenergy, zeroenergy
            if (dyninp%argontest .gt. 0) then
                write(*,*) 'enerqm: ', enerqm
                write(*,*) 'enermm: ', enermm
                write(*,*) 'enerpart: ', enerpart
            endif
        endif
c
c Gradient due to gradient of smoothing function
c
        if (numbufgr .gt. 0) then
            temp1 = (enerab-enera) * lengthconvert / (1.d0*numbufgr)
            do i = 1, geom%natoms
                if (dyninp%ap_atom_zone(i) .eq. 2) then
                    i3 = i * 3
                    j = dyninp%group(i)
                    do m = 1, numbufgr
                        n = dyninp%buffergroupps(m)
                        if (n .eq. j) then
                            n3 = n * 3
                            temp = temp1 * dpi(m) *
     *                          geom%atmmss(i) / dyninp%groupmass(n)
                        grad(i3-2:i3) =grad(i3-2:i3)+riv(n3-2:n3)*temp
                        grad(a-2:a) = grad(a-2:a) - riv(n3-2:n3)* temp
                        endif
                    enddo
                endif
            enddo
        endif

      return
      end


c
c
c*****************************************************************************
c  SORTEDAP
c*****************************************************************************
c
      subroutine sortedap (progrms, mmgeom, geom, dyninp, qmmminp,
     *                      ri, riv, grad, ener)
c
c     Subroutine sortedap
c
c     Created:  Jul. 20, 2010 by Soroosh Pezeshki
c
c     Purpose:  perform the sorted AP
c
c     Description:  Heyden et al. "Sorted AP" methode
c
c
c     Called by:  partition
c
c     Calls:  sortedsmoothing, quicksort
c
c     Variables defined here:
c
      use caltyp, only : tdebug
      use input
      use gradient, only : qmmmgrad   
      use energy,   only : qmmmener
      use files,    only : outfile
      implicit none
c
c  i/o variables
c

      type (proglist) :: progrms
      type (mmvalen)  :: mmgeom
      type (cartgeom) :: geom
      type (qmmminf)  :: qmmminp, qmmminp1
      type (dynamicsinf)  :: dyninp
      double precision ri(maxatm), riv(3*maxatm)
      double precision phi(maxbuffergroupps)
      double precision PIphi(maxbuffergroupps)
      double precision dphi(maxbuffergroupps, maxbuffergroupps)
      double precision dPIphi, PIdP, PdPI, phijPIj
      double precision dsumSmooth, sumSmooth, abssdS
      double precision distancegroupps(maxbuffergroupps)
      double precision ener, grad(3*maxatm)

      integer ierr, recurscall
c
c  local variables
c      
      integer a, a3, i, i3, j, b, m, mj, n3tm, nbg
      double precision enerpart, enerqm, enermm, gradsmooth, gradsmooth1
      double precision gradmm(3*maxatm), gradpart(3*maxatm)
      double precision gradqm(3*maxatm), phij, PIj, rmax, zeroenergy
!       double precision hlzeroenergy, llzeroenergy, zeroenergy2

      character*3 tt
c
c inital values
c
      n3tm = 3*geom%natoms
      nbg = dyninp%numberbuffergroupps
      sumSmooth  = -1.d0
      dsumSmooth = 0.d0
      abssdS     = 0.d0
c
c sort the groups from the closest to the active zone to the most far
c
c   recurscall...counter for recrusive repeats
      recurscall = 0
      do j = 1, dyninp%numberbuffergroupps
        m = dyninp%buffergroupps(j)
        distancegroupps(j) = ri(m)
      enddo
      call quicksort (dyninp%numberbuffergroupps,
     *           distancegroupps, dyninp%buffergroupps, recurscall)
! c
! c Calculate the zero energy
! c
!       rmax = dyninp%activeradius
!       hlzeroenergy = 0.d0
!       llzeroenergy = 0.d0
!       do m = 1, dyninp%maxgroups
!           if(ri(m) .lt. rmax) then
!               hlzeroenergy = hlzeroenergy + dyninp%ape0_1_qm(m)
!           else
!               llzeroenergy = llzeroenergy + dyninp%ape0_1_mm(m)
!           endif
!       enddo
!       zeroenergy2 = llzeroenergy + hlzeroenergy
c
c calculate the smoothing function
c
      call sortedsmoothing (dyninp, ri, phi, PIphi, dphi)
c
c Debug output
c
      if (((tdebug.eq.1) .or. (tdebug.eq.5)) .and. (nbg.gt.0)) then
        write(299,*) "Buffer groups sorted:"
        write(299,*) "#   Group   Distance      Phi       PiPhi"
c     *                   ,"     dPhi(z)    dPiPhi(z)"
        do m = 1, dyninp%numberbuffergroupps
            write(299,200) m, dyninp%buffergroupps(m),
     *                  distancegroupps(m), phi(m), PIphi(m)
        enddo
200   format(I3, t5, I5, t12, f10.5, t25, f8.5, t35, f8.5)
      endif

c
c TEST: ARGON atoms with own MM
c
      grad(:)   = 0.d0
      gradmm(:) = 0.d0
      ener = 0.d0
      dyninp%zeroenergy = 0.d0
      if (dyninp%argontest .gt. 0) then
          call mmargontest(qmmminp, 2, geom, dyninp, gradmm, enermm)
      endif

c
c loop over all groups in the buffer, from most close to active zone to most far
c j=0: no buffer group
c
        do j = 0, nbg

c
c add the buffer atoms to the QM atoms and adjust isolated energy
c
            a = qmmminp%qmnatm
            qmmminp1%qmatm(:) = qmmminp%qmatm(:)
            qmmminp1%qmcharge = qmmminp%qmcharge
!             hlzeroenergy = 0.d0
!             llzeroenergy = 0.d0
            do mj = 1, j 
                m = dyninp%buffergroupps(mj)
                dyninp%ap_group_zone(m) = 3
                if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
                    write(outfile,*) 'Adding group', m, mj 
                endif
                do i = 1, geom%natoms
                    if (dyninp%group(i) .eq. m) then
                        a = a + 1
                        qmmminp1%qmatm(a) = i
                        if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
                            write(outfile,*) 'Adding atom', i 
                        endif
                    endif
                enddo
                qmmminp1%qmcharge = qmmminp1%qmcharge + 
     *                  dyninp%groupcharge(m)
!                 hlzeroenergy = hlzeroenergy + dyninp%ape0_1_qm(m)
!                 llzeroenergy = llzeroenergy - dyninp%ape0_1_mm(m)
            enddo
            qmmminp1%qmnatm = a
            call calculatezeroenergy (dyninp, zeroenergy)
!             zeroenergy = zeroenergy2 + llzeroenergy + hlzeroenergy

c
c TEST ARGON atoms
c
            if (dyninp%argontest .gt. 0) then
                gradqm(:)   = 0.d0
                gradpart(:) = 0.d0
                call mmargontest (qmmminp1, 3, geom, dyninp,
     *               gradpart, enerpart)
                call mmargontest(qmmminp1,1,geom,dyninp,gradqm,enerqm)
                qmmmener = enermm + enerqm - enerpart
                qmmmgrad(1:n3tm) = gradmm(1:n3tm)
                do a = 1, qmmminp1%qmnatm
                    i3 = 3 * qmmminp1%qmatm(a)
                    qmmmgrad(i3-2:i3) = gradmm(i3-2:i3) + 
     *                  gradqm(i3-2:i3) - gradpart(i3-2:i3)
                enddo
            else

c
c QM calculation
c
                call copyqminp (qmmminp, qmmminp1)
                call resetqmmm (qmmminp1, geom, mmgeom, dyninp)
                call qmmmghkmd (progrms%qmprog, progrms%mmprog, geom,
     *               mmgeom, qmmminp1, ierr, qmmmgrad, qmmmener)
            endif
c
c Sum up the potential
c
            a3 = 3 * dyninp%activecenter
c
c special case j=0
c
            if (j .eq. 0) then
                phij = 1.d0
                PIj = PIphi(1) * (1.d0 - phi(1))
c
c all other j's
c
            else
                phij = phi(j)
                PIj = PIphi(j)
            endif
            if (phij .gt. 1.d0) phij = 1.d0
            if (PIj .gt. 1.d0) PIj = 1.d0
            phijPIj = phij * PIj
            sumSmooth = sumSmooth + phijPIj
c
c energy and gradient (due to gradient energy)
c
            qmmmener = qmmmener - zeroenergy
            ener = ener + qmmmener * phijPIj
            dyninp%zeroenergy = dyninp%zeroenergy +
     *              zeroenergy * phijPIj
            grad(1:n3tm) = grad(1:n3tm) + qmmmgrad(1:n3tm) * phijPIj
c
c Debug output
c
            if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
                write(*,111) ' Phi','PI(1-Phi)','V_I', 'Sum V_1...i',
     *              'Phi*PI'
                write(*,112) phij, PIj, qmmmener, ener, phijPIj
111     format(a,t11,a,t21,a,t31,a,t41,a,t55,a)
112     format(f9.5,t10,f9.5,t20,f9.5,t30,f9.5,t40,f9.5,t54,f9.5)
!                 write(*,*) 'Gradient'
!                 write(*,2055) qmmmgrad(:n3tm)
                write(*,211) 'bfg','dPhi*V*Pi',
     *                          'Phi*V*dPI','dPhi','dPI'

2055  format(t20, f15.8, t40, f15.8, t60, f15.8)
            endif
c
c Gradient due to gradient of smoothing function
c
            do b = 1, nbg
                m = dyninp%buffergroupps(b)
                mj = 3 * m
c
c the gradient of phij*qmmmener*PIj in respect of PIj and phij
c (scalar part, rij is multiplied later)
c
                PdPI = phij * dpiphi(j, b, dphi, nbg, phi)
                if (j .ne. 0) then 
                    PIdP = dphi (j, b) * PIj
                else
                    PIdP = 0.d0
                endif
                gradsmooth1 = (PIdP + PdPI) * lengthconvert * qmmmener
                if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
                    write(*,212) b, PIdP, PdPI, dphi(j, b),
     *                          dpiphi (j, b, dphi, nbg, phi)
211     format(a3, t5, a, t20, a, t35, a, t50, a)
212     format(i3, t5, f10.6, t20, f10.6, t35, f10.6, t50, f10.6)
                    dsumSmooth = dsumSmooth + PIdP + PdPI
                    abssdS = abssdS + dabs(gradsmooth1 / qmmmener)
                endif
                do i = 1, geom%natoms
                    if (m .eq. dyninp%group(i)) then
                        i3 = i * 3
                        gradsmooth = gradsmooth1 *
     *                      (geom%atmmss(i) / dyninp%groupmass(m))
                        grad(i3-2:i3) = grad(i3-2:i3)
     *                      + gradsmooth * riv(mj-2:mj)
                        grad(a3-2:a3) = grad(a3-2:a3)
     *                      - gradsmooth * riv(mj-2:mj)
                        if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
                            write(*,*) 'gradient smoothing function',i
                            write(*,2055) gradsmooth * riv(mj-2:mj)
                        endif
                    endif
                enddo
            enddo
c
c Sum up the gradient for checking
c
            do i = 1, 3 * geom%natoms
                dyninp%sum1 = dyninp%sum1 + phij * qmmmgrad(i) * PIj
            enddo
        enddo
c
c Warn if sum is higher than a threshold
c
        if (dabs(dyninp%sum1) .gt. dyninp%eps) write(*,*) 
     *              '  Sum gradient V: ', dyninp%sum1
        if (dabs(sumSmooth) .gt. dyninp%eps) write(*,*) 
     *      "  Warrning: Sum of smoothing is not 1: ", sumSmooth
c
c Output for debug
c
        if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
            write(*,*) 'Sum gradient V: ', dyninp%sum1
            write(*,*) 'Sum gradient Smooth: ', dsumSmooth
            write(*,*) 'Abs(Sum d Smooth): ', abssdS
        endif

      return
      end

c
c
c*****************************************************************************
c  DPIPHI
c*****************************************************************************
c
      function dpiphi (l, b, dphi, N, phi)
c
c     function dpiphi
c
c     Created:  Nov. 18, 2010 by Soroosh Pezeshki
c
c     Purpose:  Calculates the gradient of the product from l to N 
c               of d(1-Phi_i)/dP_b for sorted AP 
c
c     Called by:  sortedAP
c
c     Calls:  
c
c     Variables defined here:
c
      use input, only : maxbuffergroupps
      implicit none
      integer l, b, N, i, j
      double precision dphi(maxbuffergroupps, maxbuffergroupps)
      double precision phi(maxbuffergroupps)
      double precision dPIphi, temp

      dPIphi = 0.d0
      do j = l + 1, N
        temp = dphi (j, b)
        do i = l + 1, N
            if (i .ne. j) then
                temp = temp * (1.d0 - phi(i))
            endif
        enddo
        dPIphi = dPIphi - temp
      enddo
      return
      end

c
c
c*****************************************************************************
c  SORTEDSMOOTHING
c*****************************************************************************
c
      subroutine sortedsmoothing (dyninp, ri, phi, PIphi, dphi)
c
c     Subroutine sortedsmoothing
c
c     Created:  Aug. 24, 2010 by Soroosh Pezeshki
c
c     Purpose:  Calculates the fifth order spline 
c               and the product of (1-Phi)
c               and the gradient
c
c     Called by:  sortedAP
c
c     Calls:  
c
c     Variables defined here:
c
      use input, only : dynamicsinf, maxatm, maxbuffergroupps
      use caltyp, only : tdebug
      implicit none
c
c  i/o variables
c
      type (dynamicsinf)  :: dyninp       
      double precision ri(maxatm)
      double precision phi(maxbuffergroupps)
      double precision piphi(maxbuffergroupps)
      double precision dphi(maxbuffergroupps, maxbuffergroupps)
      double precision dp(maxbuffergroupps), p(maxbuffergroupps)
c
c  local variables
c      
      integer i, j, numbufgr, nan 
      double precision chi(maxbuffergroupps), epsilon2
      double precision dtempj, dtempi, diff2, epsilon1, temp1

      numbufgr = dyninp%numberbuffergroupps
      phi(:) = 0.d0
      chi(:) = 0.d0
      piphi(:) = 1.d0
      epsilon1 = 1.d-9
      epsilon2 = 1.d-4
      do i = 1, maxbuffergroupps
        dphi(:,i) = 0.d0
      enddo
c
c calculate the smoothing function
c
        call smoothingfunction (dyninp, ri, p, dp)

        do i = 1, numbufgr
            nan = 0
            if(dabs(p(i)) .lt. epsilon1) then
                nan = 1
            else
                chi(i) = (1.d0 - p(i)) / p(i)
            endif
            do j = 1, i - 1
                if(dabs(p(j) - p(i)) .lt. epsilon1) then
                    nan = 1
                else
                    chi(i) = chi(i) + (1.d0 - p(j)) / (p(j) - p(i))
                endif
            enddo
            do j = 1 + i, numbufgr
                if(dabs(p(j) - p(i)) .lt. epsilon1) then
                    nan = 1
                else
                    chi(i) = chi(i) + (1.d0-p(i)) / (p(i)-p(j)) * p(j)
                endif
            enddo
            if (dabs(1.d0 + chi(i)) .lt. epsilon2) nan = 1
            if (nan .eq. 1) then
                phi(i) = 0.d0
                chi(i) = -1.d0
!                 if (tdebug.gt.0) then
!                     write(*,22) i
! 22      format(5x,'Warning: chi(',i5,')=+-infinity')
!                 endif
            else
c                phi(i) = 1.d0 / ((1.d0 + chi(i))**3)
                phi(i) = 1.d0 + chi(i)
                phi(i) = phi(i) * phi(i) * phi(i)
                phi(i) = 1.d0 / phi(i)
            endif
        enddo

        do i = 1, numbufgr
            do j = i + 1, numbufgr
                piphi(i) = piphi(i) * (1.d0 - phi(j))
            enddo
        enddo

c
c calculate  the derivative of the smoothing function
c
c The gradient of phi_i: dphi(i,j)=dphi_i/dj
c
        do i = 1, numbufgr
            if ((chi(i)) .lt. epsilon2) cycle
            temp1 = (-3.d0 / (1.d0 + chi(i))**4)
            dtempi = 0.d0
            do j = 1, i - 1
                diff2 = (p(j) - p(i))**2
                if (diff2 .gt. epsilon1) then 
                    dtempj = -(1.d0 - p(i)) / diff2
                    dtempi = dtempi + (1.d0 - p(j)) /diff2
                    dphi(i,j) = dtempj * dp(j) * temp1
                endif
            enddo
            do j = i + 1, numbufgr
                diff2 = (p(j) - p(i))**2
                if (diff2 .gt. epsilon1) then
                    dtempj = p(i) * (1.d0 - p(i)) / diff2
                    dtempi = dtempi - p(j) * (1.d0 - p(j)) / diff2
                    dphi(i,j) = dtempj * dp(j) * temp1
                endif
            enddo
            if(dabs(p(i)) .gt. epsilon2) then
                dtempi = dtempi - 1.d0 / (p(i)**2)
                dphi(i,i) = dtempi * dp(i) * temp1
            endif
!             if (tdebug.eq.2) then
!                 write(*,*) 'i,    chi_i,   dtempi,  temp1'
!                 write(*,111) i, chi(i), dtempi, temp1
! 111     format(i3,t10,e13.5,t30,e13.5,t50,f8.5)
!             endif

        enddo

      end

c
c
c*****************************************************************************
c  PERMUTEDAP
c*****************************************************************************
c
      subroutine permutedap(progrms, mmgeom, geom, dyninp, qmmminp,
     *                           ri, riv, grad, ener)
c
c     Subroutine permutedap
c
c     Created:  Sep. 01, 2010 by Soroosh Pezeshki
c
c     Purpose:  perform the permuted AP
c
c     Description:  Order higher than  dyninp%userporder are truncated
c
c
c     Called by:  partition
c
c     Calls:  
c
c     Variables defined here:
c
      use caltyp, only : tdebug
      use input
      use gradient, only : qmmmgrad
      use energy,   only : qmmmener
      use files, only : outfile
      implicit none
c
c  i/o variables
c

      type (proglist) :: progrms
      type (mmvalen)  :: mmgeom
      type (cartgeom) :: geom
      type (qmmminf)  :: qmmminp
      type (dynamicsinf)  :: dyninp       
      double precision ener, grad(3*maxatm)
      double precision ri(maxatm), riv(3*maxatm), enersum
      double precision dp(maxbuffergroupps), pi(maxbuffergroupps)
      double precision gradsum(3*maxatm), sumSmooth
      integer ierr
      integer*8 buffer(maxpcalc,2), binomial, nper
      integer ba, bs, bi, order
c
c  local variables
c      
      integer a, b, d, i, i3, j, j3, k, m, n3tm, ngr
      character*3 tt


      n3tm = 3 * geom%natoms
      ngr = dyninp%numberbuffergroupps
      gradsum(:) = 0.d0
      enersum    = 0.d0
      sumSmooth  = -1.d0
      dyninp%zeroenergy = 0.d0
      dyninp%gradforce(1:3) = 0.d0
      dyninp%smoothforce(1:3) = 0.d0

c
c calculate the smoothing function
c
      call smoothingfunction (dyninp, ri, pi, dp)

c
c calculate the permutation
c if more buffer groups than order
c #permutation = 1 (0 order) + ngr (1st order) + binomial(ngr, higher order)
c else #permutation=2**ngr
c
      order = dyninp%userporder
123   if (ngr .gt. order) then
        nper = 1 + ngr
        do i = 2, order
            nper = nper + binomial (ngr, i)
        enddo
        if (nper .lt. 0) goto 124
      else
        nper = 2**ngr
      endif

c
c check if the number of needed calculation is higher than the limit.
c if the number is higher and user allow changes, 
c change the order and recalculate the number of calculations
c stop if user don't allow changes
c
      if (nper .gt. dyninp%userpcalc) then
124     if (dyninp%reduceorder .eq. 1) then
            write(*,*) "Warning: Number of buffer groups is too ",
     *              "high"
            write(*,*) "     Number of permutations is higher",
     *              " than set in PCALCULATION"
            write(*,*) "     Reducing the permutation order for",
     *              " truncation to", order - 1
            order = order - 1
            goto 123
        else
            write(*,*) "Error: Number of buffer groups and order",
     *              " is too high"
            write(*,*) "     Number of permutations is higher",
     *              " than set in PCALCULATION"
            write(*,*) "     Please reduce PERMUTEDORDER or/and ",
     *                  "increase PCALCULATION"
            STOP "PERMUTEDAP"
        endif
      endif

c
c debug output
c
      if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
           write(outfile,*) 
     *      "DebugDebugDebugDebugDebugDebugDebugDebugDebugDebug"
            write(outfile,*) '# of permutations: ', nper
            write(outfile,*) 'Order of PAP: ', order
      endif

c
c store all possible permutations in buffer
c
      buffer(:,1) = 0
      buffer(:,2) = 0
      bi = 1
      bs = 1
      do ba = 1, ngr
        bi = bi + 1
        call permutedcombination(ba, bi, bs, ngr, order, 
     *                 buffer)
      enddo
c
c loop over all groups in the buffer
c
        do bi = 1, nper
            call permutedgradient (bi, qmmminp, dyninp, geom,
     *                  enersum, pi, dp, progrms, mmgeom, buffer, riv,
     *                  gradsum, nper, sumSmooth)
        enddo
      dyninp%avghl = dyninp%avghl + nper

      grad(:) = gradsum(:)
      ener = enersum
c
c check the sum of the gradient
c
      if (dyninp%sum1 .gt. dyninp%eps) write(*,*) 
     *              '  Sum gradient V: ', dyninp%sum1
      if (dabs(sumSmooth) .gt. dyninp%eps) write(*,*) 
     *      "  Warrning: Sum smoothing is not 1: ", sumSmooth
c
c debug output
c
      if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
            write(outfile,*) 'Sum gradient V: ', dyninp%sum1
            write(outfile,*) 'Sum Smoothing Function: ', sumSmooth
           write(outfile,*) 
     *      "DebugDebugDebugDebugDebugDebugDebugDebugDebugDebug"
      endif
      if (tdebug.eq.6) then
        write(51,*) riv(4),  grad(1), grad(4)
        write(52,*) riv(5),  grad(2), grad(5)
        write(53,*) riv(6),  grad(3), grad(6)
        write(55,*) ri(2), ener 
      endif
      if (tdebug.eq.5) then
cccccccccc
!         write(*,*) "i, F (kcal\(mol Ang))" 
!         do i =553, geom%natoms
!         write(*,202) i, grad(i*3-2:i*3) * forceconvert3
!         enddo
! 202   format(i4,t5,3e15.5)
cccccccccc
      endif

      end

c
c
c*****************************************************************************
c  PERMUTEDCOMBINATION
c*****************************************************************************
c
      recursive subroutine permutedcombination (ba, bi, bs, ngr, 
     *                                  order, buffer)
c
c     Subroutine permutedcombination
c
c     Created:  Oct. 08, 2010 by Soroosh Pezeshki
c
c     Purpose:  store all permutaion in the array buffer for permuted AP
c
c     Description:  ba...buffer group to add
c                   buffer... list with the permutations and the number 
c                           of high level groups in the permuation
c                   bi... index of the matrix buffer
c                   bs...index of the calling permutation 
c                   ngr... number of buffer groups
c                   order   order after which the calculation should be truncated
c                             
c
c     Called by:  permutedAP
c
c     Calls:  permutedcombination (recursive)
c
c     Variables defined here:
c
      use input, only : maxpcalc 
      use caltyp, only : tdebug
      use files, only : outfile
      implicit none
c
c  variables
c
      integer ban, ba, bi, bs,bsn,bsnc, ngr, order, i
      integer*8 k, buffer(maxpcalc,2)
      character permutation(64)

      bsn = bi
c
c  Copy the permutation from the calling permutationn and add the group ba
c  as a bit in the position ba-1
c and store it as a single integer in buffer(bi,1)
c
        k = IBSET(buffer(bs,1), ba-1)
        buffer(bi,1) = k
c
c calculate the number of high level groups in the buffer 
c    (=previous permutaion+1)
c and store it in buffer(bi,2)
c
        k = 1 + buffer(bs,2)
        buffer(bi,2) = k

c
c debug output
c
      if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
        write(outfile,*) 'Permutation: ', bi, " size: ", k 
        do i = 0, ngr - 1
            if (btest (buffer(bi,1), i)) then
                permutation(i+1) = '1'
            else
                permutation(i+1) = "0"
            endif
        enddo
        write(outfile,*) permutation(:ngr)
      endif
c
c if the numbe of high level groups is equal to order, stop adding more groups
c
        if (k .eq. order) return
c
c calculate the next permutation
c
        do ban = ba + 1, ngr
            bsnc = bsn
            bi = bi + 1
            call permutedcombination(ban, bi, bsnc,ngr,order,buffer)
        enddo

      end

c
c
c*****************************************************************************
c  BINOMIAL
c*****************************************************************************
c
      function binomial (n, k)
c
c    function binominal
c
c     Created:  Oct. 10, 2010 by Soroosh Pezeshki
c
c     Purpose:  Calculates the binomial coefficient of the two arguments
c
c     Called by:  permutedap, permutedgradient
c
c     Calls:  
c
c     Variables defined here:
c
      implicit none
      integer i, n, k
      integer*8 p, binomial

        p = n
        do i = 2, k
            p = p * (n + 1 - i) / i
        enddo
        binomial = p
        return
      end

c
c
c*****************************************************************************
c  PERMUTEDGRADIENT
c*****************************************************************************
c
      subroutine permutedgradient (bi, qmmminp, dyninp, geom,
     *                 enersum, pi, dp, progrms, mmgeom, buffer, riv,
     *                 gradsum, npermuations, sumSmooth)
c
c     Subroutine permutedgradient
c
c     Created:  Sep. 03, 2010 by Soroosh Pezeshki
c
c     Purpose:  calculate the energy and the gradient for permuted AP
c
c     Description: Heyden et al. 'permuted AP' method.
c                  to reduce the memory usage, the original equation 
c                  V=V^A+Sum_i P_i(V_i-V^A)+ 
c                       Sum_i Sum_j  P_iP_j(V_i,j-[V^A+ Sum_r (V_r-V^A)+...
c                  have been reordered to:
c                  V=V^A(1 - Sum_i P_i + Sum_i Sum_j P_iP_j - ...)
c                    + Sum_i V_i(P_i - Sum_j P_iP_j+...)
c                    + Sum_i Sum_j V_i,j (P_iP_j - ...) + ...
c
c     Parameter     bi...index of the array buffer
c                   enersum... the sum of energy
c                   pi... the smoothing function P(i)
c                   buffer..list with 1 for groups to considered and 0 else
c
c     Called by:  permutedAP
c
c     Calls:  permutedsmoothing, permutedsmoothing2
c
c     Variables defined here:
c
      use input
      use caltyp, only : ttiming, tdebug
      use files, only : outfile
      use energy,   only : qmmmener
      use gradient, only : qmmmgrad   
      implicit none
c
c  i/o variables
c

      type (proglist) :: progrms
      type (mmvalen)  :: mmgeom
      type (cartgeom) :: geom
      type (qmmminf)  :: qmmminp, qmmminp1
      type (dynamicsinf)  :: dyninp
      double precision dp(maxbuffergroupps), pi(maxbuffergroupps)
      double precision riv(3*maxatm), sumSmooth
      integer ierr, bi, npermuations
      integer*8 buffer(maxpcalc, 2)
c
c  local variables
c
      integer a, a3, i, i3, l, m, l3, m3, n3tm, ngr
      double precision enerpart, enerqm, enermm, enersum
      double precision gradmm(3*maxatm), gradpart(3*maxatm), temp(3)
      double precision gradsum(3*maxatm), temp1
      double precision gradqm(3*maxatm), sumpi, signes, sumpidpi
      double precision zeroe

c
c  timing variables
c
      real timing1, timing2, deltatiming
      call CPU_TIME(timing2)


      n3tm = 3 * geom%natoms
      ngr = dyninp%numberbuffergroupps
      a3 = 3 * dyninp%activecenter

c
c add the buffer atoms to the QM atoms
c
        qmmminp1%qmatm(:) = qmmminp%qmatm(:)
        qmmminp1%qmcharge = qmmminp%qmcharge
        a = qmmminp%qmnatm
        do l = 1, ngr
            if (btest(buffer(bi,1), l-1)) then
                m = dyninp%buffergroupps(l)
                dyninp%ap_group_zone(m) = 3
                if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
                    write(outfile,*) 'Adding group', m, l 
                endif
                do i = 1, geom%natoms
                    if (dyninp%group(i) .eq. m) then
                        a = a + 1
                        qmmminp1%qmatm(a) = i
                        if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
                            write(outfile,*) 'Adding atom', i 
                        endif
                    endif
                enddo
c adjust the  qm charge
                qmmminp1%qmcharge = qmmminp1%qmcharge +
     *              dyninp%groupcharge(m)
            endif
        enddo
c      write(*,*) "charge", qmmminp%qmcharge
        qmmminp1%qmnatm = a
        call calculatezeroenergy (dyninp, zeroe)
c        call calculatezeroenergy1(dyninp,zeroe,qmmminp%capatm%capnatm)

c
c calculate the smooting factor
c
        call permutedsmoothing(ngr, buffer, sumpi, pi,bi,npermuations)
c
c TEST: ARGON atoms with own MM
c
        if (dyninp%argontest .gt. 0) then
            call mmargontest(qmmminp1,3,geom,dyninp,gradpart,enerpart)
            call mmargontest(qmmminp1,2,geom,dyninp, gradmm,enermm)
            call mmargontest(qmmminp1,1,geom,dyninp,gradqm,enerqm)
            qmmmener = enermm + enerqm - enerpart
            qmmmgrad(1:n3tm) = gradmm(1:n3tm)
            do a = 1, qmmminp1%qmnatm
                i3 = 3 * qmmminp1%qmatm(a)
                qmmmgrad(i3-2:i3) = gradmm(i3-2:i3) +
     *            gradqm(i3-2:i3) - gradpart(i3-2:i3)
            enddo
        else
c
c QM gradient of before + next group in buffer
c
            call copyqminp(qmmminp, qmmminp1)
            call resetqmmm(qmmminp1, geom, mmgeom, dyninp)
            call qmmmghkmd(progrms%qmprog, progrms%mmprog, geom,
     *             mmgeom, qmmminp1, ierr, qmmmgrad, qmmmener)
        endif
c
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming = timing2 - timing1
            timing1 = timing2
            write (outfile,*) 'Permuted AP V_', bi, ':'
     &      ,' time=', deltatiming
        endif

        qmmmener = qmmmener - zeroe
        dyninp%zeroenergy = dyninp%zeroenergy + zeroe * sumpi

c
c Sum up the potential
c
         enersum = enersum + qmmmener * sumpi
         sumSmooth = sumSmooth + sumpi
c
c the gradient of the potential
c
         gradsum(1:n3tm) = gradsum(1:n3tm) + qmmmgrad(1:n3tm) * sumpi

c
c Output for debugging
c
        if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
            write (outfile,*) 'Permutation, groups in permutation,',
     *         ' smoothing' 
            write (outfile,101) bi, buffer(bi,2), sumpi
            write (outfile,*) 'interaction Potential ',
     *           " (hartree, kcal/mol), scaled interaction Potential",
     *           " (hartree, kcal/mol)"
            write (outfile,103) qmmmener, qmmmener * energyconvert,
     *          qmmmener * sumpi, qmmmener * energyconvert * sumpi
cccccccccc
!         write(*,*) "i, grad V (kcal\mol\ Ang), Pi * grad V, Zone" 
!         do i = 1, geom%natoms
!         write(*,201) i, qmmmgrad(3*i) * forceconvert3, 
!      *      qmmmgrad(3*i) * sumpi * forceconvert3, 
!      *      dyninp%ap_atom_zone(i)
!         enddo
!         write(*,*) qmmmgrad(a3-2) * forceconvert3, 
!      *      qmmmgrad(a3-2) * sumpi * forceconvert3
!         write(*,*) qmmmgrad(a3-1) * forceconvert3, 
!      *      qmmmgrad(a3-1) * sumpi * forceconvert3
!         write(*,*) qmmmgrad(a3) * forceconvert3, 
!      *      qmmmgrad(a3) * sumpi * forceconvert3
! 
! 201   format(i2,t5,e15.5,t20,e15.5,t40,i2)
cccccccccc
101   format(i5,t15,i5,t32,f10.5)
103   format(f10.5,f10.5,f10.5,f10.5)
        endif

c
c restraint debug
c
         if(dyninp%forceOutAtom.gt.0) then
            dyninp%gradforce(1:3) = dyninp%gradforce(1:3) +  sumpi *
     *          qmmmgrad(dyninp%forceOutAtom-2:dyninp%forceOutAtom)
         endif
c
c the gradient of the smoothing function
c
         do i = 1, ngr
            m = dyninp%buffergroupps(i)
            call permutedsmoothing2(ngr, buffer, sumpidpi, pi, bi, i,
     *                              npermuations)
            temp1 = qmmmener * lengthconvert * sumpidpi * dp(i)
c
c Output for debugging
c
            if ((tdebug.eq.1) .or. (tdebug.eq.5)) then
                write (outfile,*) '     #, d_i,',
     *                      ' smoothing, gradient smoothing,',
     *                      ' Force due to gradient smoothing'
                write (outfile,102) bi,i, sumpidpi, dp(i), temp1
102   format(4x, i2,t9, i2, t13, f10.5,t25, f10.5,t45, d15.5)
            endif
            do l = 1, geom%natoms
                if (m .eq. dyninp%group(l)) then
                    l3 = l * 3
                    m3 = m * 3
                    temp(1:3) = temp1 * riv(m3-2:m3) *
     *                  (geom%atmmss(l) / dyninp%groupmass(m))
                    gradsum(l3-2:l3) = gradsum(l3-2:l3) + temp(1:3)
                    gradsum(a3-2:a3) = gradsum(a3-2:a3) - temp(1:3)
cccccccccc
!         write(*,*) "i, V * dPi (kcal\mol \Ang)" 
! c        write(*,202) dyninp%activecenter, -temp(1:3) * forceconvert3
!         write(*,202) l, temp(1:3) * forceconvert3
! 202   format(i4,t5,3e15.5)
cccccccccc
c
c the gradient of the smoothing function
c
                    if (dyninp%forceOutAtom.gt.0) then
                        dyninp%smoothforce(1:3) = 
     *                      dyninp%smoothforce(1:3) - temp(1:3)
                    endif
                endif
            enddo
         enddo
         do l = 1, 3 * geom%natoms
            dyninp%sum1 = dyninp%sum1 + qmmmgrad(l) * sumpi
         enddo


c
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming = timing2 - timing1
            timing1 = timing2
            write (outfile,*) 'Permuted AP F_', bi, ':'
     &      ,' time=', deltatiming
        endif
        
c
c reset buffer groups to 2
c
        do l = 1, ngr
            if (btest(buffer(bi,1), l-1)) then
                m = dyninp%buffergroupps(l)
                dyninp%ap_group_zone(m) = 2
            endif
        enddo
        
      end


c
c
c*****************************************************************************
c  PERMUTEDSMOOTHING
c*****************************************************************************
c   
      subroutine permutedsmoothing(ngr, buffer, sumpi,  
     *                      pi, bi, npermuations)
c
c     Subroutine permutedsmoothing
c
c     Created:  Sep. 07, 2010 by Soroosh Pezeshki
c
c     Purpose:  the sum of the Pi factors for permuted AP
c
c     Description:  ngr.... number of buffer groups
c                   buffer..array with 1 for groups to considered and 0 else
c                   sumpi.... sum of the Pi factors
c                   pi.... list with all Pi
c                   bi...index of the matrix buffer
c
c
c     Called by:  permutedgradient
c
c     Calls:  
c
c     Variables defined here:
c
      use input, only : maxpcalc, maxbuffergroupps
      implicit none

      double precision sumpi, pi(maxbuffergroupps)
      double precision PI1, PI2
      double precision prod, signe
      integer i, size1, size2, diffsize
      integer bi, k, ngr, npermuations
      integer*8 buffer(maxpcalc,2)

      size1 = buffer(bi, 2)
c
c if j=1 (only active zone) all combination have to be considered
c
      if (bi .eq. 1) then
        sumpi = 1.d0
        do i = 2, npermuations
            prod = 1.d0
            do k = 1, ngr
                if (btest(buffer(i,1),k-1)) then
                    prod = prod * pi(k)
                endif
            enddo
            size2 = buffer(i,2)
            diffsize = size2 - size1
            signe = 1.d0 - 2.d0 * mod(diffsize,2)
            sumpi = sumpi + signe * prod
        enddo
      else

c
c find all other combiniations
c
      sumpi = 0.d0
      do i = 2, npermuations
        size2 = buffer(i,2)
        if (size1 .gt. size2) cycle
        do k = 0 , ngr - 1
            if (btest(buffer(bi,1),k)) then
                if (btest(buffer(i,1),k) .eqv. .false.) goto 100
            endif
        enddo
        prod = 1.d0
        do k = 1, ngr
            if (btest(buffer(i,1),k-1)) then
                prod = prod * pi(k)
            endif
        enddo
        diffsize = size2 - size1
        signe = 1.d0 - 2.d0 * mod(diffsize,2)
        sumpi = sumpi + signe * prod
100   enddo
      
      endif
c
c reorderd equation (as it is only valid without truncation)
c
!         PI1 = 1.d0
!         PI2 = 1.d0
!         do k = 1, ngr 
!             if (btest(buffer(bi,1), k-1)) then
!                 PI1 = PI1 * pi(k)
!             else
!                 PI2 = PI2 * (1.d0 - pi(k))
!             endif 
!         enddo
!         sumpi = PI1 * PI2

      end

c
c
c*****************************************************************************
c  PERMUTEDSMOOTHING2
c*****************************************************************************
c
      subroutine permutedsmoothing2(ngr, buffer, sumdpi,
     *                  pi, bi, bd, npermuations)
c
c     Subroutine permutedsmoothing2
c
c     Created:  Sep. 07, 2010 by Soroosh Pezeshki
c
c     Purpose:  the sum of the Pi factors without P(j) for permuted AP
c               for the gradient calcualation
c
c     Description:  ngr.... number of buffer groups
c                   buffer..matrix with 1 for groups to considered and 0 else
c                   sumdpi.... sum of the Pi factors
c                   pi.... list with all Pi
c                   bi.... index of the matrix buffer 
c                   bd.... the deriative of the buffer with this index is 
c                           calculated
c
c     Called by:  permutedgradient
c
c     Calls:  
c
c     Variables defined here:
c
      use input, only : maxpcalc, maxbuffergroupps
      implicit none

      double precision sumdpi, pi(maxbuffergroupps)
      double precision PI1, PI2
      double precision prod, signe, eps
      integer i, size1, size2, diffsize
      integer bi, bd, ngr, k, npermuations
      integer*8 buffer(maxpcalc,2)

      size1 = buffer(bi,2)
      sumdpi = 0.d0
      eps = 1.d-9
c
c if j=1 (only active zone) all combination have to be considered
c
      if (bi.eq.1) then
        do i = 2, npermuations
            if (btest(buffer(i,1),bd-1)) then
                prod = 1.d0
                do k = 1, ngr 
                    if (btest(buffer(i,1),k-1) .and. (k.ne.bd)) then
                        prod = prod * pi(k)
                    endif
                enddo
                size2 = buffer(i,2)
                diffsize = size2 - size1
                signe = 1.d0 - 2.d0 * mod(diffsize,2)
                sumdpi = sumdpi + signe * prod
            endif
         enddo
c
c find all other combiniations
c
      else
        do i = 2, npermuations
            if (btest(buffer(i,1),bd-1).eqv..false.) cycle
            size2 = buffer(i,2)
            if(size1.gt.size2) cycle
            do k = 0, ngr - 1
                if (btest(buffer(bi,1),k)) then
                    if (btest(buffer(i,1),k).eqv..false.) goto 100
                endif 
            enddo
            prod = 1.d0
            do k = 1, ngr 
                if (btest(buffer(i,1),k-1).and.(k.ne.bd)) then
                    prod = prod * pi(k)
                endif
            enddo
            diffsize = size2 - size1
            signe = 1.d0 - 2.d0 * mod(diffsize,2)
            sumdpi = sumdpi + signe * prod
100     enddo
      endif

c
c reorderd equation (as it is now, only valid without truncation)
c
!         PI1 = 1.d0
!         PI2 = 1.d0
!         do k = 1, ngr 
!             if (btest(buffer(bi,1), k-1)) then
!                 if (k .eq. bd) cycle
!                 PI1 = PI1 * pi(k)
!             else
!                 if (k .eq. bd) then
!                     PI2 = -PI2
!                 else
!                     PI2 = PI2 * (1.d0 - pi(k))
!                 endif
!             endif 
!         enddo
!         sumdpi = PI1 * PI2

      end

c
c
c*****************************************************************************
c  CALCULATEARGONSHIFT
c*****************************************************************************
c
      subroutine calculateArgonShift (dyninp)
c
c     Subroutine calculateArgonShift
c
c     Created:  Jan. 28, 2011 by Soroosh Pezeshki
c
c     Purpose:  Calculates the force shift parameter for the MM/MM argon test
c
c
c     Called by:  initmd
c
c     Calls:  mmargontest
c
c     Variables defined here:
c
      use input, only : cartgeom, dynamicsinf, qmmminf, energyconvert,
     *                   maxatm, lengthconvert
      implicit none

      type (cartgeom) :: g
      type (dynamicsinf)  :: dyninp       
      type (qmmminf)  :: q
      integer method
      double precision grad(3*maxatm), ener

        q%qmnatm = 2
        q%qmatm(1) = 1
        q%qmatm(2) = 2
        g%angcoord(1:5) = 0.d0
        g%angcoord(6) = dsqrt(dyninp%argoncutoff)
        g%angcoord(6) = g%angcoord(6) - g%angcoord(6) * 0.1d-5


        method = 1
        call mmargontest(q, method, g, dyninp, grad, ener)
        dyninp%shiftFM = grad(6) / lengthconvert
        call mmargontest(q, method, g, dyninp, grad, ener)
        dyninp%shiftPM = -ener
        write(*,*) '     Force shift Morse: ', dyninp%shiftFM
        write(*,*) '     Potential shift Morse: ', dyninp%shiftPM

        method = 3
        call mmargontest(q, method, g, dyninp, grad, ener)
        dyninp%shiftFLJ = grad(6) / lengthconvert
        call mmargontest(q, method, g, dyninp, grad, ener)
        dyninp%shiftPLJ = -ener
        write(*,*) '     Force shift LJ: ', dyninp%shiftFLJ
        write(*,*) '     Potential shift LJ: ', dyninp%shiftPLJ

      end


c
c
c*****************************************************************************
c  MMARGONTEST
c*****************************************************************************
c
      subroutine mmargontest (qmmminp, method, geom, dyninp,
     *                                           grad, ener)
c
c     Subroutine mmargontest
c
c     Created:  Aug. 03, 2010 by Soroosh Pezeshki
c
c     Purpose:  Calculates the MM potential for Argon to compare with 
c     Heyden et.al 2007
c
c     Description:  method 1: morse potential  for qmmminp atoms
c                   method 2: 6-12-V-d-W for all atoms
c                   method 3: 6-12-V-d-W for qmmminp atoms
c
c     Called by:  hotspot, oniomxs, sortedap, permutedgradient
c
c     Calls:  fifthorderspline
c
c     Variables defined here:
c
      use input, only : cartgeom, dynamicsinf, qmmminf, energyconvert,
     *                   maxatm, lengthconvert
      implicit none
c
c  i/o variables
c
      type (cartgeom) :: geom
      type (dynamicsinf)  :: dyninp       
      type (qmmminf)  :: qmmminp
      integer method
      double precision grad(3*maxatm), ener, fos, dfos
c
c  local variables
c      
      double precision sigma6, alpha, d, re, epsilonx24, epsilonx4
      integer i, j, qm, ai, aj, l
      double precision sigr6, sigr12, box(3), mbox(3), rij(3)
      double precision expr,fact, extexpr, rij2, rijsqrt
      double precision dswitch, switch, rswitch, enertmp

      grad(:) = 0.d0
      ener = 0.d0

c
c Periodic box
c
      box(:) = dyninp%boxsize(:) * 0.5d0
      mbox(:) = -1.d0 * box(:)
c
c set the v-d-w parameters; sigma=3.405d0, epsilon=0.23787d0
c sigma6=sigma**6, espsilon24=24*epsilon, espsilon4=4*epsilon
c
      epsilonx24 = 5.70888d0 / energyconvert
      epsilonx4 = 0.95148d0 / energyconvert
      sigma6 = 1558.48525417d0
c
c Morse only for selected atoms
c
      if (method.eq.1) then
c
c set the morse potential parameters
c
        d = 0.23787d0 / energyconvert
        re = 3.822d0
        alpha = 1.570d0
        fact = 2.d0 * alpha * d
c
c loop over all atoms in the qm zone
c
        do ai = 1, qmmminp%qmnatm - 1
            i = 3 * qmmminp%qmatm(ai)
            do aj = ai + 1, qmmminp%qmnatm
                switch = 1.d0
                dswitch = 0.d0
                j = 3 * qmmminp%qmatm(aj)
                rij(1:3) = geom%angcoord(i-2:i) - geom%angcoord(j-2:j)
                if(dyninp%periodic.eq.1) then
                    do l = 1, 3
                        if (rij(l).gt.box(l)) then
                            rij(l) = rij(l) - dyninp%boxsize(l)
                        elseif (rij(l).lt.mbox(l)) then
                            rij(l) = rij(l) + dyninp%boxsize(l)
                        endif
                    enddo
                endif
                rij2 = rij(1)*rij(1) + rij(2)*rij(2) + rij(3)*rij(3)
                if (rij2.lt.dyninp%argoncutoff) then
                    rijsqrt = dsqrt(rij2)
                    if(rijsqrt.lt.0.1d0) rijsqrt = 0.1d0
                    if((dyninp%argontest.eq.2) .and.
     *                  (rijsqrt.gt.dyninp%argonswitch)) then
                        rswitch = (rijsqrt-dyninp%argonswitch)
     *                          / dyninp%switchdist
                        call fifthorderspline(rswitch, fos, dfos)
                        switch = fos
                        dswitch = dfos / (rijsqrt * dyninp%switchdist)
                    endif
                    expr = dexp(alpha * (re - rijsqrt))
                    enertmp = (d * (1.d0 - expr)**2 - d -
     *                     dyninp%shiftFM * rijsqrt) + dyninp%shiftPM
                    ener = ener + switch * enertmp
                    extexpr = (fact*(1.d0-expr)*expr-dyninp%shiftFM)
     *                      / rijsqrt *switch + dswitch * enertmp
                    grad(i-2:i) = grad(i-2:i) + rij(:) * extexpr
                    grad(j-2:j) = grad(j-2:j) - rij(:) * extexpr
                endif
            enddo
        enddo

c
c LJ for all atoms
c
      elseif (method.eq.2) then
        do ai = 1, geom%natoms - 1
            i = ai * 3
            do aj = ai + 1, geom%natoms
                switch = 1.d0
                dswitch = 0.d0
                j = aj * 3
c
c Calculate the distance; consider periodic box
c
                rij(1:3) = geom%angcoord(i-2:i) - geom%angcoord(j-2:j)
                if(dyninp%periodic.eq.1) then
                    do l = 1, 3
                        if (rij(l).gt.box(l)) then
                            rij(l) = rij(l) - dyninp%boxsize(l)
                        elseif (rij(l).lt.mbox(l)) then
                            rij(l) = rij(l) + dyninp%boxsize(l)
                        endif
                    enddo
                endif
                rij2 = rij(1)*rij(1) + rij(2)*rij(2) + rij(3)*rij(3)
c
c skip if distance is more than cut off
c
                if (rij2.lt.dyninp%argoncutoff) then
                    if (rij2.lt.0.01d0) rij2 = 0.01d0
                    rijsqrt = dsqrt(rij2)
                    if((dyninp%argontest.eq.2).and.
     *                  (rijsqrt.gt.dyninp%argonswitch)) then
                        rswitch = (rijsqrt-dyninp%argonswitch)
     *                          / dyninp%switchdist
                        call fifthorderspline(rswitch, fos, dfos)
                        switch = fos
                        dswitch = dfos / (rijsqrt * dyninp%switchdist)
                    endif

c
c calculate the gradient
c
                    sigr6 = sigma6 / (rij2 * rij2 * rij2)
                    sigr12 = sigr6 * sigr6
                    enertmp = epsilonx4 * (sigr12 - sigr6)
     *                  + dyninp%shiftFLJ * rijsqrt + dyninp%shiftPLJ
                    ener = ener + enertmp * switch
                    fact = epsilonx24 * (2.d0 * sigr12 - sigr6)
     *                  / rij2* switch + dyninp%shiftFLJ / rijsqrt
     *                  - dswitch * enertmp
                    grad(i-2:i) = grad(i-2:i) - rij(:) * fact
                    grad(j-2:j) = grad(j-2:j) + rij(:) * fact
                endif
            enddo
        enddo
c
c LJ only for the selected atoms
c
      else
        do ai = 1, qmmminp%qmnatm - 1
            i = 3 * qmmminp%qmatm(ai)
            do aj = ai + 1, qmmminp%qmnatm
                switch = 1.d0
                dswitch = 0.d0
                j = 3 * qmmminp%qmatm(aj)
c
c Calculate the distance; consider periodic box
c
                rij(1:3) = geom%angcoord(i-2:i) - geom%angcoord(j-2:j)
                if(dyninp%periodic.eq.1) then
                    do l = 1, 3
                        if (rij(l) .gt. box(l)) then
                            rij(l) = rij(l) - dyninp%boxsize(l)
                        elseif (rij(l) .lt. mbox(l)) then
                            rij(l) = rij(l) + dyninp%boxsize(l)
                        endif
                    enddo
                endif
                rij2 = rij(1)*rij(1) + rij(2)*rij(2) + rij(3)*rij(3)
c
c avoid division with 0 and very high forces through small distances
c
                if (rij2.lt.dyninp%argoncutoff) then
                    if (rij2.lt.0.01d0) rij2 = 0.01d0
                    rijsqrt = dsqrt(rij2)
                    if((dyninp%argontest.eq.2).and.
     *                  (rijsqrt.gt.dyninp%argonswitch)) then
                        rswitch = (rijsqrt - dyninp%argonswitch)
     *                          / dyninp%switchdist
                        call fifthorderspline(rswitch, fos, dfos)
                        switch = fos
                        dswitch = dfos / (rijsqrt * dyninp%switchdist)
                    endif
c
c calculate the gradient
c
                    sigr6 = sigma6 / (rij2 * rij2 * rij2)
                    sigr12 = sigr6 * sigr6
                    enertmp = epsilonx4 * (sigr12 - sigr6)
     *                  + dyninp%shiftFLJ * rijsqrt + dyninp%shiftPLJ
                    ener = ener + enertmp * switch
                    fact = epsilonx24 * (2.d0 * sigr12 - sigr6) / rij2
     *                  * switch + dyninp%shiftFLJ / rijsqrt
     *                  - dswitch * enertmp
                    grad(i-2:i) = grad(i-2:i) - rij(:) * fact
                    grad(j-2:j) = grad(j-2:j) + rij(:) * fact
                endif
            enddo
        enddo
      endif

c
c Convert the gradient from E_h/Ang to E_h/a_0
c
      grad(:) = grad(:) * lengthconvert

      return
      end



c
c
c*****************************************************************************
c  RESETQMMM
c*****************************************************************************
c
      subroutine resetqmmm(qmmminp, geom, mmgeom, dyninp)
c
c     Subroutine resetqmmm
c
c     Created:  Feb. 14, 2011 by Soroosh Pezeshki
c     Based on: RQMMM
c
c     Purpose:  reset the cap atom, ... after changing the QM atoms
c     Called by:  sortedap, hotspot, oniomxs, permutedgradient, setgroupaffiliation
c     Calls: system, addcapatoms
c
c     Variables defined here:
c
      use input, only : qmmminf, cartgeom, mmvalen, maxganum, 
     *              dynamicsinf
      use files, only : outfile
      implicit none
      type (qmmminf) :: qmmminp
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (dynamicsinf)  :: dyninp
c
c  local variables
c
      integer jj, ii, j, i, kk, k, temp, t1, t2, t3, mm, m,m2, m3, fbg
      double precision mindist, dist

      fbg = 0
c
c 0) find capatoms
c
      call addcapatoms(qmmminp, mmgeom, dyninp)

c
c  1) Set the M1 atom (cap atom host) id, Cap-M1 Link
c
      jj = qmmminp%capatm%capnatm
      qmmminp%mm1stlay%natm1stlay     = jj
      qmmminp%mm1stlay%atm1stlay(:jj) = qmmminp%capatm%capatm(:jj)
      qmmminp%capatm%capatmm1(:jj)    = qmmminp%capatm%capatm(:jj)
c
c  2) Set the M1-QM and M1-M2 links
c
      m2 = 0 
      ii = qmmminp%qmnatm + qmmminp%capatm%capnatm
      do j = 1, jj
         i = qmmminp%mm1stlay%atm1stlay(j)
         kk = mmgeom%atmvalen(i)
         temp = 0

         do k = 1, kk
            t2 = mmgeom%atmvalenlist(i,k)
c
c  QM link
c
            mm = qmmminp%qmnatm
            do m = 1, mm
               t3 = qmmminp%qmatm(m)
               if (t2 .eq. t3) then
                  qmmminp%mm1stlay%atm1stlayqm(j) = t3
                  qmmminp%capatm%capatmqm(j) = t3
                  goto 2000
               endif
            enddo
c
c  M2 link
c
           temp = temp + 1
           m2 = m2 + 1
           qmmminp%mm1stlay%atm1stlaym2(j,temp) = t2
           qmmminp%mm2ndlay%atm2ndlay(m2) = t2
           qmmminp%mm2ndlay%atm2ndlaym1(m2) = i
           qmmminp%chredist%m1(m2) = i
           qmmminp%chredist%m2(m2) = t2
2000     continue
         enddo

         if(temp .ne. (kk-1)) then
            write(outfile,*)
     *        'Error: Only one QM atom should connect to M1 atom:', i
         endif
         qmmminp%mm1stlay%atm1stlaym2num(j) = kk - 1

      continue
      enddo
      qmmminp%mm2ndlay%natm2ndlay = m2
c
c  3) check if capatm too close to M1 atom when bordercharge = scale
c
      if (qmmminp%embed .ne. 'mechanical')  then
         if (qmmminp%bordercharge .eq. 'scale') then
            mindist = 1.0d-3
            ii = qmmminp%capatm%capnatm
            do i = 1, ii
                dist = 1.0d0 - qmmminp%capatm%capbondscale(i)
                if((dist .le. mindist) .or. (-dist .ge. -mindist)) then
                    write(outfile,3000) qmmminp%capatm%capatm(i)
                    stop 'rqmmm 9'
                endif
            enddo
         endif
      endif
3000  format(/2x,'Error: Cap atom ',i5,' too close to MM host.')
c
c 5) Set the M2-M3 links
c
      temp = 0
      ii = geom%natoms
      jj = qmmminp%qmnatm
      kk = qmmminp%capatm%capnatm
      mm = qmmminp%mm2ndlay%natm2ndlay
      do m = 1, mm
         m2 = qmmminp%mm2ndlay%atm2ndlay(m)
         t1 = mmgeom%atmvalen(m2)
         do t2 = 1, t1
            m3 = mmgeom%atmvalenlist(m2,t2)
            do j = 1, jj
               k = qmmminp%qmatm(j)
               if (m3 .eq. k) goto 3500
            enddo
            do j = 1, kk
               k = qmmminp%capatm%capatm(j)
               if  (m3 .eq. k) goto 3500
            enddo
            do j = 1, mm
               k = qmmminp%mm2ndlay%atm2ndlay(j)
               if  (m3 .eq. k) goto 3500
            enddo
            temp = temp + 1
            qmmminp%mm3rdlay%atm3rdlay(temp) = m3
            qmmminp%mm3rdlay%atm3rdlaym2(temp) = m2
3500     continue
         enddo
      enddo
      qmmminp%mm3rdlay%natm3rdlay = temp
c
c 4) Set the MM atoms excluding the M1 (cap host) atoms
c
      qmmminp%mmnatm = geom%natoms - qmmminp%qmnatm
     *                 - qmmminp%capatm%capnatm
      if (qmmminp%mmnatm .lt. 0) then
         write(outfile,4000)
4000  format(/3x,'ERROR: Too many QM and Cap atoms!')
         stop 'rqmmm 6'
      endif
      temp = 0
      ii = geom%natoms
      jj = qmmminp%qmnatm
      kk = qmmminp%capatm%capnatm
      mm = qmmminp%mm2ndlay%natm2ndlay
      m3 = qmmminp%mm3rdlay%natm3rdlay
      do i = 1, ii
         k = mmgeom%atmid(i)
         do j = 1, jj
            if (k .eq. qmmminp%qmatm(j)) goto 4500
         enddo
         do j = 1, kk
            if (k .eq. qmmminp%capatm%capatm(j)) goto 4500
         enddo
         temp = temp + 1
         qmmminp%mmatm(temp) = k
         do j = 1, mm
            if (k .eq. qmmminp%mm2ndlay%atm2ndlay(j)) then
               qmmminp%mlist(temp) = j
               goto 4500
            endif   
         enddo
         do j = 1, m3
            if (k .eq. qmmminp%mm3rdlay%atm3rdlay(j)) then
               qmmminp%m3list(temp) = j
               goto 4500
            endif
         enddo
4500     continue
      enddo

      return
      end

c
c
c*****************************************************************************
c  COPYQMINP
c*****************************************************************************
c
      subroutine copyqminp(qmmminp, qmmminp1)
c
c     Subroutine copyqminp
c     
c     Created:  Feb. 15, 2011 by Soroosh Pezeshki
c
c     Purpose:  copy the setings from one type into the other
c     Called by:  sortedap, oniomxs, permutedgradient
c     Calls: 
c
c     Variables defined here:
c
      use input, only : qmmminf, maxpolgidnum, maxganum
      implicit none
      type (qmmminf) :: qmmminp, qmmminp1
c
c  local variables
c                 
         integer i

         qmmminp1%qmmeth   =    qmmminp%qmmeth
         qmmminp1%qmbas    =    qmmminp%qmbas
         qmmminp1%mmmeth   =    qmmminp%mmmeth
         qmmminp1%mmbas    =    qmmminp%mmbas

         qmmminp1%lcutoff      = qmmminp%lcutoff
         qmmminp1%ccutoffid    = qmmminp%ccutoffid
         qmmminp1%selcent      = qmmminp%selcent
         qmmminp1%cutoffatm(:) = qmmminp%cutoffatm(:)
         qmmminp1%rcutoff      = qmmminp%rcutoff
         qmmminp1%cutoffcoor(:)= qmmminp%cutoffcoor(:)

         qmmminp1%polmeth     = qmmminp%polmeth
         qmmminp1%polmpot     = qmmminp%polmpot
         qmmminp1%polparal(:) = qmmminp%polparal(:)
         qmmminp1%polgnum     = qmmminp%polgnum 
         qmmminp1%polgidnum(:)= qmmminp%polgidnum(:)
         qmmminp1%polcycl     = qmmminp%polcycl
         qmmminp1%polmaxdq    = qmmminp%polmaxdq
         qmmminp1%polrmsdq    = qmmminp%polrmsdq
         do i=1,maxpolgidnum
            qmmminp1%polgid(:,i) = qmmminp%polgid(:,i)
         enddo
         
c         qmmminp1%qmcharge = qmmminp%qmcharge
         qmmminp1%qmmult   = qmmminp%qmmult

         qmmminp1%qmoptl(:) = qmmminp%qmoptl(:)
         qmmminp1%mmoptl(:) = qmmminp%mmoptl(:)
         qmmminp1%border    = qmmminp%border
         qmmminp1%embed     = qmmminp%embed
         qmmminp1%bordercharge = qmmminp%bordercharge

         qmmminp1%capatm%capnatm=qmmminp%capatm%capnatm
         qmmminp1%capatm%capatm(:)=qmmminp%capatm%capatm(:)
         qmmminp1%capatm%capconstrain=qmmminp%capatm%capconstrain
         qmmminp1%capatm%capatmtyp(:)=qmmminp%capatm%capatmtyp(:)
        qmmminp1%capatm%capbondscale(:)=qmmminp%capatm%capbondscale(:)
      qmmminp1%capatm%capatmtypname(:)=qmmminp%capatm%capatmtypname(:)
!          qmmminp1%capatm%nlist=qmmminp%capatm%nlist
!          qmmminp1%capatm%list(:)=qmmminp%capatm%list(:)
!          qmmminp1%capatm%atmtyp(:)=qmmminp%capatm%atmtyp(:)
!          qmmminp1%capatm%defaulttyp=qmmminp%capatm%defaulttyp
!          qmmminp1%capatm%typname(:)=qmmminp%capatm%typname(:)
!          qmmminp1%capatm%defaulttypname=qmmminp%capatm%defaulttypname
!          qmmminp1%capatm%bondscale(:)=qmmminp%capatm%bondscale(:)
!       qmmminp1%capatm%defaultbondscale=qmmminp%capatm%defaultbondscale

        qmmminp1%chscale%chargelay = qmmminp%chscale%chargelay
        qmmminp1%chscale%chargescale(:)=qmmminp%chscale%chargescale(:)

         qmmminp1%iflexbound = qmmminp%iflexbound
         qmmminp1%asqmcharge = qmmminp%asqmcharge
         qmmminp1%asqmmult   = qmmminp%asqmmult
         qmmminp1%maxfbcycl  = qmmminp%maxfbcycl 
         qmmminp1%fbgid      = qmmminp%fbgid
         qmmminp1%fbgan(:)   = qmmminp%fbgan(:)
         qmmminp1%fbgnum     = qmmminp%fbgnum
         qmmminp1%sscp(:)    = qmmminp%sscp(:)
         qmmminp1%fbccmeth   = qmmminp%fbccmeth
         qmmminp1%maxfbdq    = qmmminp%maxfbdq
         qmmminp1%rmsfbdq    = qmmminp%rmsfbdq
         qmmminp1%maxfbdqtot = qmmminp%maxfbdqtot
         qmmminp1%temprt     = qmmminp%temprt
         do i = 1, maxganum
            qmmminp1%fbgaid(:,i) = qmmminp%fbgaid(:,i)
         enddo

         qmmminp1%modmmcharg = qmmminp%modmmcharg

         qmmminp1%mm1stlay%atm1stlaycharge(:) = 
     *          qmmminp%mm1stlay%atm1stlaycharge(:)
         qmmminp1%mm2ndlay%atm2ndlaycharge(:) = 
     *          qmmminp%mm2ndlay%atm2ndlaycharge(:)
         qmmminp1%mm3rdlay%atm3rdlaycharge(:) = 
     *          qmmminp%mm3rdlay%atm3rdlaycharge(:)

        qmmminp1%chredist%chposit = qmmminp%chredist%chposit
        qmmminp1%chredist%dipposit = qmmminp%chredist%dipposit
        qmmminp1%chredist%dipdist = qmmminp%chredist%dipdist
        qmmminp1%chredist%m1(:) = qmmminp%chredist%m1(:)
        qmmminp1%chredist%m2(:) = qmmminp%chredist%m2(:)

      return
      end


c
c
c*****************************************************************************
c  CALCULATEZEROENERGY
c*****************************************************************************
c
      subroutine calculatezeroenergy (dyninp, zeroenergy)
c
c     Subroutine calculatezeroenergy
c
c     Created:  July 27, 2011 by Soroosh Pezeshki
c
c     Purpose:  Calculates the zero of energy, check for super groups
c
c
c     Called by:  hotspot, oniomxs, sortedap, permutedgradient
c
c     Calls:  
c
c     Variables defined here:
c
      use input, only : dynamicsinf, maxatm
      use caltyp, only : tdebug
      implicit none

      type (dynamicsinf)  :: dyninp       
      double precision zeroenergy

c
c local variables
c
      integer k1, k2, k3, k4, k5, k6, k7, k8, k9, k10, k11, k12, k13
      integer i, j, l, m, already(maxatm), k14, k15
      double precision llzeroenergy, hlzeroenergy


      hlzeroenergy = 0.d0
      llzeroenergy = 0.d0
      already(:) = 0

c
c 15 super groups
c
      do j = 1, maxatm
        if (dyninp%iape0_15(j,1) .eq. 0) exit
        k1 = dyninp%iape0_15(j,1)
        k2 = dyninp%iape0_15(j,2)
        k3 = dyninp%iape0_15(j,3)
        k4 = dyninp%iape0_15(j,4)
        k5 = dyninp%iape0_15(j,5)
        k6 = dyninp%iape0_15(j,6)
        k7 = dyninp%iape0_15(j,7)
        k8 = dyninp%iape0_15(j,8)
        k9 = dyninp%iape0_15(j,9)
        k10 = dyninp%iape0_15(j,10)
        k11 = dyninp%iape0_15(j,11)
        k12 = dyninp%iape0_15(j,12)
        k13 = dyninp%iape0_15(j,13)
        k14 = dyninp%iape0_15(j,14)
        k15 = dyninp%iape0_15(j,15)
        i= already(k1) + already(k2) + already(k3) + already(k4) +
     *               already(k5) + already(k6) + already(k7) + 
     *               already(k8) + already(k9) + already(k10) +
     *               already(k11) + already(k12) + already(k13) +
     *               already(k14) + already(k15)
        if (i .gt. 0) cycle
        m = mod(dyninp%ap_group_zone(k1), 2) + 
     *          mod(dyninp%ap_group_zone(k2), 2) +
     *          mod(dyninp%ap_group_zone(k3), 2) +
     *          mod(dyninp%ap_group_zone(k4), 2) +
     *          mod(dyninp%ap_group_zone(k5), 2) +
     *          mod(dyninp%ap_group_zone(k6), 2) +
     *          mod(dyninp%ap_group_zone(k7), 2) +
     *          mod(dyninp%ap_group_zone(k8), 2) +
     *          mod(dyninp%ap_group_zone(k9), 2) +
     *          mod(dyninp%ap_group_zone(k10), 2) +
     *          mod(dyninp%ap_group_zone(k11), 2) +
     *          mod(dyninp%ap_group_zone(k12), 2) +
     *          mod(dyninp%ap_group_zone(k13), 2) +
     *          mod(dyninp%ap_group_zone(k14), 2) +
     *          mod(dyninp%ap_group_zone(k15), 2) 
c
c 15 in QM super group
c
        if (m .eq. 15) then 
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            already(k10) = 1
            already(k11) = 1
            already(k12) = 1
            already(k13) = 1
            already(k14) = 1
            already(k15) = 1
            hlzeroenergy = hlzeroenergy + dyninp%ape0_15_qm(j)
c
c 15 in MM super group
c
        elseif (m .eq. 0) then
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            already(k10) = 1
            already(k11) = 1
            already(k12) = 1
            already(k13) = 1
            already(k14) = 1
            already(k15) = 1
            llzeroenergy = llzeroenergy + dyninp%ape0_15_mm(j)
        endif
      enddo


c
c 14 super groups
c
      do j = 1, maxatm
        if (dyninp%iape0_14(j,1) .eq. 0) exit
        k1 = dyninp%iape0_14(j,1)
        k2 = dyninp%iape0_14(j,2)
        k3 = dyninp%iape0_14(j,3)
        k4 = dyninp%iape0_14(j,4)
        k5 = dyninp%iape0_14(j,5)
        k6 = dyninp%iape0_14(j,6)
        k7 = dyninp%iape0_14(j,7)
        k8 = dyninp%iape0_14(j,8)
        k9 = dyninp%iape0_14(j,9)
        k10 = dyninp%iape0_14(j,10)
        k11 = dyninp%iape0_14(j,11)
        k12 = dyninp%iape0_14(j,12)
        k13 = dyninp%iape0_14(j,13)
        k14 = dyninp%iape0_14(j,14)
        i= already(k1) + already(k2) + already(k3) + already(k4) +
     *               already(k5) + already(k6) + already(k7) + 
     *               already(k8) + already(k9) + already(k10) +
     *               already(k11) + already(k12) + already(k13) +
     *               already(k14)
        if (i .gt. 0) cycle
        m = mod(dyninp%ap_group_zone(k1), 2) + 
     *          mod(dyninp%ap_group_zone(k2), 2) +
     *          mod(dyninp%ap_group_zone(k3), 2) +
     *          mod(dyninp%ap_group_zone(k4), 2) +
     *          mod(dyninp%ap_group_zone(k5), 2) +
     *          mod(dyninp%ap_group_zone(k6), 2) +
     *          mod(dyninp%ap_group_zone(k7), 2) +
     *          mod(dyninp%ap_group_zone(k8), 2) +
     *          mod(dyninp%ap_group_zone(k9), 2) +
     *          mod(dyninp%ap_group_zone(k10), 2) +
     *          mod(dyninp%ap_group_zone(k11), 2) +
     *          mod(dyninp%ap_group_zone(k12), 2) +
     *          mod(dyninp%ap_group_zone(k13), 2) +
     *          mod(dyninp%ap_group_zone(k14), 2) 
c
c 14 in QM super group
c
        if (m .eq. 14) then 
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            already(k10) = 1
            already(k11) = 1
            already(k12) = 1
            already(k13) = 1
            already(k14) = 1
            hlzeroenergy = hlzeroenergy + dyninp%ape0_14_qm(j)
c
c 14 in MM super group
c
        elseif (m .eq. 0) then
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            already(k10) = 1
            already(k11) = 1
            already(k12) = 1
            already(k13) = 1
            already(k14) = 1
            llzeroenergy = llzeroenergy + dyninp%ape0_14_mm(j)
        endif
      enddo


c
c 13 super groups
c
      do j = 1, maxatm
        if (dyninp%iape0_13(j,1) .eq. 0) exit
        k1 = dyninp%iape0_13(j,1)
        k2 = dyninp%iape0_13(j,2)
        k3 = dyninp%iape0_13(j,3)
        k4 = dyninp%iape0_13(j,4)
        k5 = dyninp%iape0_13(j,5)
        k6 = dyninp%iape0_13(j,6)
        k7 = dyninp%iape0_13(j,7)
        k8 = dyninp%iape0_13(j,8)
        k9 = dyninp%iape0_13(j,9)
        k10 = dyninp%iape0_13(j,10)
        k11 = dyninp%iape0_13(j,11)
        k12 = dyninp%iape0_13(j,12)
        k13 = dyninp%iape0_13(j,13)
        i= already(k1) + already(k2) + already(k3) + already(k4) +
     *               already(k5) + already(k6) + already(k7) + 
     *               already(k8) + already(k9) + already(k10) +
     *               already(k11) + already(k12) + already(k13)
        if (i .gt. 0) cycle
        m = mod(dyninp%ap_group_zone(k1), 2) + 
     *          mod(dyninp%ap_group_zone(k2), 2) +
     *          mod(dyninp%ap_group_zone(k3), 2) +
     *          mod(dyninp%ap_group_zone(k4), 2) +
     *          mod(dyninp%ap_group_zone(k5), 2) +
     *          mod(dyninp%ap_group_zone(k6), 2) +
     *          mod(dyninp%ap_group_zone(k7), 2) +
     *          mod(dyninp%ap_group_zone(k8), 2) +
     *          mod(dyninp%ap_group_zone(k9), 2) +
     *          mod(dyninp%ap_group_zone(k10), 2) +
     *          mod(dyninp%ap_group_zone(k11), 2) +
     *          mod(dyninp%ap_group_zone(k12), 2) +
     *          mod(dyninp%ap_group_zone(k13), 2) 
c
c 13 in QM super group
c
        if (m .eq. 13) then 
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            already(k10) = 1
            already(k11) = 1
            already(k12) = 1
            already(k13) = 1
            hlzeroenergy = hlzeroenergy + dyninp%ape0_13_qm(j)
c
c 13 in MM super group
c
        elseif (m .eq. 0) then
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            already(k10) = 1
            already(k11) = 1
            already(k12) = 1
            already(k13) = 1
            llzeroenergy = llzeroenergy + dyninp%ape0_13_mm(j)
        endif
      enddo


c
c 12 super groups
c
      do j = 1, maxatm
        if (dyninp%iape0_12(j,1) .eq. 0) exit
        k1 = dyninp%iape0_12(j,1)
        k2 = dyninp%iape0_12(j,2)
        k3 = dyninp%iape0_12(j,3)
        k4 = dyninp%iape0_12(j,4)
        k5 = dyninp%iape0_12(j,5)
        k6 = dyninp%iape0_12(j,6)
        k7 = dyninp%iape0_12(j,7)
        k8 = dyninp%iape0_12(j,8)
        k9 = dyninp%iape0_12(j,9)
        k10 = dyninp%iape0_12(j,10)
        k11 = dyninp%iape0_12(j,11)
        k12 = dyninp%iape0_12(j,12)
        i= already(k1) + already(k2) + already(k3) + already(k4) +
     *               already(k5) + already(k6) + already(k7) + 
     *               already(k8) + already(k9) + already(k10) +
     *               already(k11) + already(k12)
        if (i .gt. 0) cycle
        m = mod(dyninp%ap_group_zone(k1), 2) + 
     *          mod(dyninp%ap_group_zone(k2), 2) +
     *          mod(dyninp%ap_group_zone(k3), 2) +
     *          mod(dyninp%ap_group_zone(k4), 2) +
     *          mod(dyninp%ap_group_zone(k5), 2) +
     *          mod(dyninp%ap_group_zone(k6), 2) +
     *          mod(dyninp%ap_group_zone(k7), 2) +
     *          mod(dyninp%ap_group_zone(k8), 2) +
     *          mod(dyninp%ap_group_zone(k9), 2) +
     *          mod(dyninp%ap_group_zone(k10), 2) +
     *          mod(dyninp%ap_group_zone(k11), 2) +
     *          mod(dyninp%ap_group_zone(k12), 2) 
c
c 12 in QM super group
c
        if (m .eq. 12) then 
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            already(k10) = 1
            already(k11) = 1
            already(k12) = 1
            hlzeroenergy = hlzeroenergy + dyninp%ape0_12_qm(j)
c
c 12 in MM super group
c
        elseif (m .eq. 0) then
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            already(k10) = 1
            already(k11) = 1
            already(k12) = 1
            llzeroenergy = llzeroenergy + dyninp%ape0_12_mm(j)
        endif
      enddo


c
c 11 super groups
c
      do j = 1, maxatm
        if (dyninp%iape0_11(j,1) .eq. 0) exit
        k1 = dyninp%iape0_11(j,1)
        k2 = dyninp%iape0_11(j,2)
        k3 = dyninp%iape0_11(j,3)
        k4 = dyninp%iape0_11(j,4)
        k5 = dyninp%iape0_11(j,5)
        k6 = dyninp%iape0_11(j,6)
        k7 = dyninp%iape0_11(j,7)
        k8 = dyninp%iape0_11(j,8)
        k9 = dyninp%iape0_11(j,9)
        k10 = dyninp%iape0_11(j,10)
        k11 = dyninp%iape0_11(j,11)
        i= already(k1) + already(k2) + already(k3) + already(k4) +
     *               already(k5) + already(k6) + already(k7) + 
     *               already(k8) + already(k9) + already(k10) +
     *               already(k11)
        if (i .gt. 0) cycle
        m = mod(dyninp%ap_group_zone(k1), 2) + 
     *          mod(dyninp%ap_group_zone(k2), 2) +
     *          mod(dyninp%ap_group_zone(k3), 2) +
     *          mod(dyninp%ap_group_zone(k4), 2) +
     *          mod(dyninp%ap_group_zone(k5), 2) +
     *          mod(dyninp%ap_group_zone(k6), 2) +
     *          mod(dyninp%ap_group_zone(k7), 2) +
     *          mod(dyninp%ap_group_zone(k8), 2) +
     *          mod(dyninp%ap_group_zone(k9), 2) +
     *          mod(dyninp%ap_group_zone(k10), 2)+
     *          mod(dyninp%ap_group_zone(k11), 2) 
c
c 11 in QM super group
c
        if (m .eq. 11) then 
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            already(k10) = 1
            already(k11) = 1
            hlzeroenergy = hlzeroenergy + dyninp%ape0_11_qm(j)
c
c 11 in MM super group
c
        elseif (m .eq. 0) then
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            already(k10) = 1
            already(k11) = 1
            llzeroenergy = llzeroenergy + dyninp%ape0_11_mm(j)
        endif
      enddo


c
c 10 super groups
c
      do j = 1, maxatm
        if (dyninp%iape0_10(j,1) .eq. 0) exit
        k1 = dyninp%iape0_10(j,1)
        k2 = dyninp%iape0_10(j,2)
        k3 = dyninp%iape0_10(j,3)
        k4 = dyninp%iape0_10(j,4)
        k5 = dyninp%iape0_10(j,5)
        k6 = dyninp%iape0_10(j,6)
        k7 = dyninp%iape0_10(j,7)
        k8 = dyninp%iape0_10(j,8)
        k9 = dyninp%iape0_10(j,9)
        k10 = dyninp%iape0_10(j,10)
        i= already(k1) + already(k2) + already(k3) + already(k4) +
     *               already(k5) + already(k6) + already(k7) + 
     *               already(k8) + already(k9) + already(k10)
        if (i .gt. 0) cycle
        m = mod(dyninp%ap_group_zone(k1), 2) + 
     *          mod(dyninp%ap_group_zone(k2), 2) +
     *          mod(dyninp%ap_group_zone(k3), 2) +
     *          mod(dyninp%ap_group_zone(k4), 2) +
     *          mod(dyninp%ap_group_zone(k5), 2) +
     *          mod(dyninp%ap_group_zone(k6), 2) +
     *          mod(dyninp%ap_group_zone(k7), 2) +
     *          mod(dyninp%ap_group_zone(k8), 2) +
     *          mod(dyninp%ap_group_zone(k9), 2) +
     *          mod(dyninp%ap_group_zone(k10), 2) 
c
c 10 in QM super group
c
        if (m .eq. 10) then 
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            already(k10) = 1
            hlzeroenergy = hlzeroenergy + dyninp%ape0_10_qm(j)
c
c 10 in MM super group
c
        elseif (m .eq. 0) then
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            already(k10) = 1
            llzeroenergy = llzeroenergy + dyninp%ape0_10_mm(j)
        endif
      enddo


c
c Nonuplet super groups
c
      do j = 1, maxatm
        if (dyninp%iape0_9(j,1) .eq. 0) exit
        k1 = dyninp%iape0_9(j,1)
        k2 = dyninp%iape0_9(j,2)
        k3 = dyninp%iape0_9(j,3)
        k4 = dyninp%iape0_9(j,4)
        k5 = dyninp%iape0_9(j,5)
        k6 = dyninp%iape0_9(j,6)
        k7 = dyninp%iape0_9(j,7)
        k8 = dyninp%iape0_9(j,8)
        k9 = dyninp%iape0_9(j,9)
        i= already(k1) + already(k2) + already(k3) + already(k4) +
     *               already(k5) + already(k6) + already(k7) + 
     *               already(k8) + already(k9)
        if (i .gt. 0) cycle
        m = mod(dyninp%ap_group_zone(k1), 2) + 
     *          mod(dyninp%ap_group_zone(k2), 2) +
     *          mod(dyninp%ap_group_zone(k3), 2) +
     *          mod(dyninp%ap_group_zone(k4), 2) +
     *          mod(dyninp%ap_group_zone(k5), 2) +
     *          mod(dyninp%ap_group_zone(k6), 2) +
     *          mod(dyninp%ap_group_zone(k7), 2) +
     *          mod(dyninp%ap_group_zone(k8), 2) +
     *          mod(dyninp%ap_group_zone(k9), 2) 
c
c 9 in QM super group
c
        if (m .eq. 9) then 
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            hlzeroenergy = hlzeroenergy + dyninp%ape0_9_qm(j)
c
c 9 in MM super group
c
        elseif (m .eq. 0) then
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            already(k9) = 1
            llzeroenergy = llzeroenergy + dyninp%ape0_9_mm(j)
        endif
      enddo


c
c octruplet super groups
c
      do j = 1, maxatm
        if (dyninp%iape0_8(j,1) .eq. 0) exit
        k1 = dyninp%iape0_8(j,1)
        k2 = dyninp%iape0_8(j,2)
        k3 = dyninp%iape0_8(j,3)
        k4 = dyninp%iape0_8(j,4)
        k5 = dyninp%iape0_8(j,5)
        k6 = dyninp%iape0_8(j,6)
        k7 = dyninp%iape0_8(j,7)
        k8 = dyninp%iape0_8(j,8)
        i= already(k1) + already(k2) + already(k3) + already(k4) +
     *               already(k5) + already(k6) + already(k7) + 
     *               already(k8)
        if (i .gt. 0) cycle
        m = mod(dyninp%ap_group_zone(k1), 2) + 
     *          mod(dyninp%ap_group_zone(k2), 2) +
     *          mod(dyninp%ap_group_zone(k3), 2) +
     *          mod(dyninp%ap_group_zone(k4), 2) +
     *          mod(dyninp%ap_group_zone(k5), 2) +
     *          mod(dyninp%ap_group_zone(k6), 2) +
     *          mod(dyninp%ap_group_zone(k7), 2) +
     *          mod(dyninp%ap_group_zone(k8), 2) 
c
c 8 in QM super group
c
        if (m .eq. 8) then 
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            hlzeroenergy = hlzeroenergy + dyninp%ape0_8_qm(j)
c
c 8 in MM super group
c
        elseif (m .eq. 0) then
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            already(k8) = 1
            llzeroenergy = llzeroenergy + dyninp%ape0_8_mm(j)
        endif
      enddo


c
c septruplet super groups
c
      do j = 1, maxatm
        if (dyninp%iape0_7(j,1) .eq. 0) exit
        k1 = dyninp%iape0_7(j,1)
        k2 = dyninp%iape0_7(j,2)
        k3 = dyninp%iape0_7(j,3)
        k4 = dyninp%iape0_7(j,4)
        k5 = dyninp%iape0_7(j,5)
        k6 = dyninp%iape0_7(j,6)
        k7 = dyninp%iape0_7(j,7)
        i= already(k1) + already(k2) + already(k3) + already(k4) +
     *               already(k5) + already(k6) + already(k7)
        if (i .gt. 0) cycle
        m = mod(dyninp%ap_group_zone(k1), 2) + 
     *          mod(dyninp%ap_group_zone(k2), 2) +
     *          mod(dyninp%ap_group_zone(k3), 2) +
     *          mod(dyninp%ap_group_zone(k4), 2) +
     *          mod(dyninp%ap_group_zone(k5), 2) +
     *          mod(dyninp%ap_group_zone(k6), 2) +
     *          mod(dyninp%ap_group_zone(k7), 2) 
c
c 7 in QM super group
c
        if (m .eq. 7) then 
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            hlzeroenergy = hlzeroenergy + dyninp%ape0_7_qm(j)
c
c 7 in MM super group
c
        elseif (m .eq. 0) then
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            already(k7) = 1
            llzeroenergy = llzeroenergy + dyninp%ape0_7_mm(j)
        endif
      enddo

c
c sextruplet super groups
c
      do j = 1, maxatm
        if (dyninp%iape0_6(j,1) .eq. 0) exit
        k1 = dyninp%iape0_6(j,1)
        k2 = dyninp%iape0_6(j,2)
        k3 = dyninp%iape0_6(j,3)
        k4 = dyninp%iape0_6(j,4)
        k5 = dyninp%iape0_6(j,5)
        k6 = dyninp%iape0_6(j,6)
        i= already(k1) + already(k2) + already(k3) + already(k4) +
     *               already(k5) + already(k6)
        if (i .gt. 0) cycle
        m = mod(dyninp%ap_group_zone(k1), 2) + 
     *          mod(dyninp%ap_group_zone(k2), 2) +
     *          mod(dyninp%ap_group_zone(k3), 2) +
     *          mod(dyninp%ap_group_zone(k4), 2) +
     *          mod(dyninp%ap_group_zone(k5), 2) +
     *          mod(dyninp%ap_group_zone(k6), 2) 
c
c 6 in QM super group
c
        if (m .eq. 6) then 
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            hlzeroenergy = hlzeroenergy + dyninp%ape0_6_qm(j)
c
c 6 in MM super group
c
        elseif (m .eq. 0) then
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            already(k6) = 1
            llzeroenergy = llzeroenergy + dyninp%ape0_6_mm(j)
        endif
      enddo

c
c quintuplet super groups
c
      do j = 1, maxatm
        if (dyninp%iape0_5(j,1) .eq. 0) exit
        k1 = dyninp%iape0_5(j,1)
        k2 = dyninp%iape0_5(j,2)
        k3 = dyninp%iape0_5(j,3)
        k4 = dyninp%iape0_5(j,4)
        k5 = dyninp%iape0_5(j,5)
        i= already(k1) + already(k2) + already(k3) + already(k4) +
     *               already(k5)
        if (i .gt. 0) cycle
        m = mod(dyninp%ap_group_zone(k1), 2) + 
     *          mod(dyninp%ap_group_zone(k2), 2) +
     *          mod(dyninp%ap_group_zone(k3), 2) +
     *          mod(dyninp%ap_group_zone(k4), 2) +
     *          mod(dyninp%ap_group_zone(k5), 2)
c
c 5 in QM super group
c
        if (m .eq. 5) then 
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            hlzeroenergy = hlzeroenergy + dyninp%ape0_5_qm(j)
c
c 5 in MM super group
c
        elseif (m .eq. 0) then
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            already(k5) = 1
            llzeroenergy = llzeroenergy + dyninp%ape0_5_mm(j)
        endif
      enddo

c
c quadruplet super groups
c
      do j = 1, maxatm
        if (dyninp%iape0_4(j,1) .eq. 0) exit
        k1 = dyninp%iape0_4(j,1)
        k2 = dyninp%iape0_4(j,2)
        k3 = dyninp%iape0_4(j,3)
        k4 = dyninp%iape0_4(j,4)
        i= already(k1) + already(k2) + already(k3) + already(k4)
        if (i .gt. 0) cycle
        m = mod(dyninp%ap_group_zone(k1), 2) + 
     *          mod(dyninp%ap_group_zone(k2), 2) +
     *          mod(dyninp%ap_group_zone(k3), 2) +
     *          mod(dyninp%ap_group_zone(k4), 2)
c
c 4 in QM super group
c
        if (m .eq. 4) then 
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            hlzeroenergy = hlzeroenergy + dyninp%ape0_4_qm(j)
c
c 4 in MM super group
c
        elseif (m .eq. 0) then
            already(k1) = 1
            already(k2) = 1
            already(k3) = 1
            already(k4) = 1
            llzeroenergy = llzeroenergy + dyninp%ape0_4_mm(j)
        endif
      enddo

c
c triple super groups
c
      do j = 1, maxatm
        if (dyninp%iape0_3(j,1) .eq. 0) exit
        k1 = dyninp%iape0_3(j,1)
        k2 = dyninp%iape0_3(j,2)
        k3 = dyninp%iape0_3(j,3)
        i= already(k1) + already(k2) + already(k3)
        if (i .gt. 0) cycle
        m = mod(dyninp%ap_group_zone(k1), 2) + 
     *          mod(dyninp%ap_group_zone(k2), 2) +
     *          mod(dyninp%ap_group_zone(k3), 2)
c
c 3 in QM super group
c
        if (m .eq. 3) then 
            already(k2) = 1
            already(k1) = 1
            already(k3) = 1
            hlzeroenergy = hlzeroenergy + dyninp%ape0_3_qm(j)
            cycle
c
c 3 in MM super group
c
        elseif (m .eq. 0) then
            already(k2) = 1
            already(k1) = 1
            already(k3) = 1
            llzeroenergy = llzeroenergy + dyninp%ape0_3_mm(j)
        endif
      enddo

c
c twin super group
c
      do j = 1, maxatm
        if (dyninp%iape0_2(j,1) .eq. 0) exit
        i = dyninp%iape0_2(j,1)
        m = dyninp%iape0_2(j,2)
        l = already(i) + already(m)
        k1 = mod(dyninp%ap_group_zone(i), 2) + 
     *       mod(dyninp%ap_group_zone(m), 2)
        if (l .eq. 0) then
c
c 2 in QM super group
c
            if (k1 .eq. 2) then
                already(i) = 1
                already(m) = 1
                hlzeroenergy = hlzeroenergy + dyninp%ape0_2_qm(j)
!         write(*,2) "Add QM", i,m, dyninp%ape0_2_qm(j)
            endif
c
c 2 in MM super group
c
            if (k1 .eq. 0) then
                already(i) = 1
                already(m) = 1
                llzeroenergy = llzeroenergy + dyninp%ape0_2_mm(j)
!         write(*,2) "Add MM", i,m, dyninp%ape0_2_mm(j)
            endif
!         elseif (l .eq. 1) then
! c
! c end of multi QM super group
! c
!             if (k1 .eq. 2) then
!                 if (already(i) .eq. 1) then
!                     already(m) = 1
!                     hlzeroenergy = hlzeroenergy
!      *                  + dyninp%ape0_2_qm(j) 
!      *                  - dyninp%ape0_1_qm(i)
! !         write(*,1) "Add QM end", m, dyninp%ape0_2_qm(j) 
! !      *                  - dyninp%ape0_1_qm(i)
!                 else
!                     already(i) = 1
!                     hlzeroenergy = hlzeroenergy
!      *                  + dyninp%ape0_2_qm(j) 
!      *                  - dyninp%ape0_1_qm(m)
! !         write(*,1) "Add QM end", i, dyninp%ape0_2_qm(j) 
! !      *                  - dyninp%ape0_1_qm(m)
!                 endif
!             endif
! c
! c end of multi MM super group
! c
!             if (k1 .eq. 0) then
!                 if (already(i) .eq. 1) then
!                     already(m) = 1
!                     llzeroenergy = llzeroenergy 
!      *                  + dyninp%ape0_2_mm(j)
!      *                  - dyninp%ape0_1_mm(i)
! !         write(*,1) "Add MM end", m, dyninp%ape0_2_mm(j) 
! !      *                  - dyninp%ape0_1_mm(i)
!                 else
!                     already(i) = 1
!                     llzeroenergy = llzeroenergy 
!      *                  + dyninp%ape0_2_mm(j)
!      *                  - dyninp%ape0_1_mm(m)
! !         write(*,1) "Add MM end", i, dyninp%ape0_2_mm(j) 
! !      *                  - dyninp%ape0_1_mm(m)
!                 endif
!             endif
        endif

300   enddo


1       format(a,i4,f15.10)
2       format(a,i4,i4,f15.10)
3       format(a,i4,i4,i4,f15.10)

c
c single QM and MM groups
c
      do m = 1, dyninp%maxgroups
          if (already(m) .eq. 1) cycle
          already(m) = 1
          if (mod(dyninp%ap_group_zone(m), 2) .eq. 1) then
              hlzeroenergy = hlzeroenergy + dyninp%ape0_1_qm(m)
!         write(*,1) "Add QM", m,dyninp%ape0_1_qm(m)
          else
              llzeroenergy = llzeroenergy + dyninp%ape0_1_mm(m)
!         write(*,1) "Add MM", m,dyninp%ape0_1_mm(m)
          endif
      enddo


      zeroenergy = llzeroenergy + hlzeroenergy
!       if (tdebug .gt. 0) then
!         write(*,4) "Infinite isolated energy ", zeroenergy
!         write(*,4) "Infinite isolated MM energy ", llzeroenergy
!         write(*,4) "Infinite isolated QM energy ", hlzeroenergy
!       endif
4       format(5x,a,f15.10)

      end


c
c
c*****************************************************************************
c  CALCULATEZEROENERGY1
c*****************************************************************************
c
      subroutine calculatezeroenergy1 (dyninp, zeroenergy, qa)
c
c     Subroutine calculatezeroenergy1
c
c     Created:  July 27, 2011 by Soroosh Pezeshki
c
c     Purpose:  Calculates the zero of energy, check for super groups
c
c
c     Called by:  hotspot, oniomxs, sortedap, permutedgradient
c
c     Calls:  
c
c     Variables defined here:
c
      use input, only : dynamicsinf, maxatm
      use caltyp, only : tdebug
      implicit none

      type (dynamicsinf)  :: dyninp       
      double precision zeroenergy
      integer qa

c
c local variables
c
      integer i, j, k1, k2, k3, l, m, sqa
      double precision llzeroenergy, hlzeroenergy


      hlzeroenergy = 0.d0
      llzeroenergy = 0.d0
      sqa = 0
      dyninp%hzeroenergy = -0.512d0
 
c
c simple QM and MM groups
c
      do m = 1, dyninp%maxgroups
          if (mod(dyninp%ap_group_zone(m), 2) .eq. 1) then
              hlzeroenergy = hlzeroenergy + dyninp%ape0_1_qm(m)
              sqa = sqa + dyninp%iape0_2(m,1)
          else
              llzeroenergy = llzeroenergy + dyninp%ape0_1_mm(m)
          endif
      enddo

      hlzeroenergy = hlzeroenergy - (sqa - qa) * dyninp%hzeroenergy
      zeroenergy = llzeroenergy + hlzeroenergy

      end
