c
c*******************************************************************************
c  QMMMCUTOFF
c*******************************************************************************
c
      subroutine qmmmcutoff(mlgeninp,geom,mmgeom,qmmminp)
c
c     Subroutine qmmmcutoff
c
c     Created:  Aug. 28, 2006 by Yan Zhang
c     Revised:
c     Purpose:  determine the cut-ff MM atoms in the embedded CPS calculation.  
c               In the cutoff calculations, the QM calculation for the embedded
c               CPS includes the CPS and uncut-off background charges.  
c
c
c     Called by: read5 
c     Calls:  
c
      use input
      use files
      implicit none
c
c  i/o variables
c
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (qmmminf)	:: qmmminp
      type (general)	:: mlgeninp
c
c  local variables
c			      
      integer i, iatm,icutoff,natm
      integer idcent,iselcent,iunit,lhess
      double precision r0, rx 
      double precision x0, y0, z0, x, y, z
      double precision angtoau
c
c  parameter
c
      angtoau = 0.529177249d0
c
c  prepare variables
c
      icutoff = qmmminp%lcutoff
      natm    = geom%natoms
      idcent  = qmmminp%ccutoffid
      r0      = qmmminp%rcutoff
      iselcent= qmmminp%selcent
      iunit   = mlgeninp%geomunit
      lhess   = mlgeninp%lhess
      
      if(icutoff.eq.0) return

      if(lhess.eq.1) then
         write(outfile,*) 'Can not calculate Hessian with QMMMCUTOFF ',
     &         'keyword in the QM/MM section.'
         stop 'in qmmmcutoff'
      end if
c
c   Determind the center for the QMMMCUTOFF calculation.
c

      if(iselcent.eq.1) then 
         x0=geom%angcoord(3*idcent-2)
         y0=geom%angcoord(3*idcent-1)
         z0=geom%angcoord(3*idcent)
      elseif(iselcent.eq.2) then
c         if(iunit.eq.1) then
           x0=qmmminp%cutoffcoor(1)
           y0=qmmminp%cutoffcoor(2)
           z0=qmmminp%cutoffcoor(3)
c         else
c           x0=qmmminp%cutoffcoor(1)*angtoau
c           y0=qmmminp%cutoffcoor(2)*angtoau
c           z0=qmmminp%cutoffcoor(3)*angtoau
c         endif
      end if
c
c   Determind the cutoff MM atoms for the QMMMCUTOFF calculation.
c   qmmminp%cutoffatm(iatm)=1 for the cutoff atoms
c   qmmminp%cutoffatm(iatm)=0 for the non-cutoff atoms
c
      do i=1,natm
         iatm=mmgeom%atmid(i)
         x=geom%angcoord(3*iatm-2)
         y=geom%angcoord(3*iatm-1)
         z=geom%angcoord(3*iatm)
         rx=(x-x0)**2+(y-y0)**2+(z-z0)**2
         rx=dsqrt(rx)
         if(rx.gt.r0) qmmminp%cutoffatm(iatm)=1
      end do

      return
      end
c 
c******************************************************************************
c  QMMMCOGEOM
c******************************************************************************
c
      subroutine qmmmcogeom(ssgeom,ssmmgeom,cogeom,commgeom,
     *            qmmminp, mmnatm,mmcutoff)

c
c     Subroutine qmmmcogeom
c
c     Created:  Aug. 28, 2006 by Yan Zhang
c     Revised:  
c     Purpose:  To assign appropriate cutoff geometries to cogeom and
c               commgeom. The QM calculation for the embedded CPS is performed
c               at the reassign geometry.
c
c     Called by:  qmmmehk, qmmmghk, qmmmhhk
c     Calls:
c
      use input
      use files, only : outfile
      implicit none
c
c i/o variables
c
      integer            mmcutoff(maxatm)
      type (cartgeom) :: ssgeom, cogeom
      type (mmvalen)  :: ssmmgeom, commgeom
      type (qmmminf)  :: qmmminp
c
c local variables
c      
      integer  capnatm, qmnatm, mmnatm, bgchnatm
      integer  ssnatm, cssnatm, lsnatm, conatm      
      integer  i,j,k, ii,jj,kk
      integer  bgconatm
      double precision angtoau
c
c
      angtoau = 0.529177249d0
c
c  prepare atom numbers
c  
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      ssnatm   = ssmmgeom%natoms
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + mmnatm
      bgchnatm = ssnatm - lsnatm
c
c  Fill with charge and multiplicity
c      
      cogeom%charge = ssgeom%charge
      cogeom%mult   = ssgeom%mult
c
c  Fill QM and Cap Atom
c                   
      do i=1, qmnatm
         cogeom%atm(i)          = ssgeom%atm(i)
         cogeom%atmmss(i)       = ssgeom%atmmss(i)
         cogeom%angcoord(3*i-2) = ssgeom%angcoord(3*i-2)
         cogeom%angcoord(3*i-1) = ssgeom%angcoord(3*i-1)
         cogeom%angcoord(3*i)   = ssgeom%angcoord(3*i)

         commgeom%atmid(i)      = ssmmgeom%atmid(i)
         commgeom%atmtyp(i)     = ssmmgeom%atmtyp(i)
         commgeom%atmtypname(i) = ssmmgeom%atmtypname(i)
         commgeom%atmcharge(i)  = ssmmgeom%atmcharge(i)
         commgeom%atmvalen(i)   = ssmmgeom%atmvalen(i)

         jj = commgeom%atmvalen(i)
         do ii=1, jj
            commgeom%atmvalenlist(i,ii) = ssmmgeom%atmvalenlist(i,ii)
         enddo
      enddo
c
c  Add the Cap atoms after QM atoms
c  
		  
      do i=qmnatm+1, cssnatm
         cogeom%atm(i)          = ssgeom%atm(i)
         cogeom%atmmss(i)       = ssgeom%atmmss(i)
         cogeom%angcoord(3*i-2) = ssgeom%angcoord(3*i-2)
         cogeom%angcoord(3*i-1) = ssgeom%angcoord(3*i-1)
         cogeom%angcoord(3*i)   = ssgeom%angcoord(3*i)

         commgeom%atmid(i)      = ssmmgeom%atmid(i)
         commgeom%atmtyp(i)     = ssmmgeom%atmtyp(i)
         commgeom%atmtypname(i) = ssmmgeom%atmtypname(i)
         commgeom%atmcharge(i)  = ssmmgeom%atmcharge(i)
         commgeom%atmvalen(i)   = ssmmgeom%atmvalen(i)	 	     
         commgeom%atmvalenlist(i,1)= ssmmgeom%atmvalenlist(i,1)
      
      enddo
c
c  Add the MM atoms after Cap and QM atoms
c  
      conatm= 0
      kk    = 0

      do i=cssnatm+1,lsnatm
         ii    = ssmmgeom%atmid(i)
         jj    = qmmminp%cutoffatm(ii)

         if(jj.eq.0) then
            kk=kk+1
            j =cssnatm+kk
            mmcutoff(kk)           = i-cssnatm
            cogeom%atm(j)          = ssgeom%atm(i)
            cogeom%atmmss(j)       = ssgeom%atmmss(i)
            cogeom%angcoord(3*j-2) = ssgeom%angcoord(3*i-2)
            cogeom%angcoord(3*j-1) = ssgeom%angcoord(3*i-1)
            cogeom%angcoord(3*j)   = ssgeom%angcoord(3*i)

            commgeom%atmid(j)      = ssmmgeom%atmid(j)
            commgeom%atmtyp(j)     = ssmmgeom%atmtyp(i)
            commgeom%atmtypname(j) = ssmmgeom%atmtypname(i)
            commgeom%atmcharge(j)  = ssmmgeom%atmcharge(i)
            commgeom%atmvalen(j)   = ssmmgeom%atmvalen(i)	 
         end if
      enddo
      conatm=kk

c
c  Add the redistribut atom
c
      do i=lsnatm+1,ssnatm
         j    = cssnatm + conatm + i - lsnatm

         cogeom%atm(j)          = ssgeom%atm(i)
         cogeom%atmmss(j)       = ssgeom%atmmss(i)
         cogeom%angcoord(3*j-2) = ssgeom%angcoord(3*i-2)
         cogeom%angcoord(3*j-1) = ssgeom%angcoord(3*i-1)
         cogeom%angcoord(3*j)   = ssgeom%angcoord(3*i)

         commgeom%atmid(j)      = ssmmgeom%atmid(i)
         commgeom%atmtyp(j)     = ssmmgeom%atmtyp(i)
         commgeom%atmtypname(j) = ssmmgeom%atmtypname(i)
         commgeom%atmcharge(j)  = ssmmgeom%atmcharge(i)
         commgeom%atmvalen(j)   = ssmmgeom%atmvalen(i)	 
      enddo
c
c  Calculate the coordinates in AU
c  
      bgconatm=cssnatm+conatm+bgchnatm
      cogeom%aucoord(:bgconatm*3)=cogeom%angcoord(:bgconatm*3)/angtoau
c
c  
      commgeom%natoms = bgconatm
      cogeom%natoms   = bgconatm
c
c
      qmmminp%mmnatm  = conatm
      return
      end

c*****************************************************************************
c  PROGMMSTEHK
c*****************************************************************************
c
      subroutine progmmstehk(prognme, esmeth, esbas, 
     * geom, mmgeom, optl, ener, ierr, qmmminp)
c
c     Subroutine progmmstehk
c
c     Created:  Aug. 28, 2006 by Yan Zhang
c               based on progmmstehk1.
c
c     Purpose:  Get a single point interaction energy witin
c               the background point charge. 
c
c     Called by:  qmmmehk
c     Calls:   progmmstehk1, qmmmcogeom
c
      use caltyp
      use files
      use input 
      implicit none
c
c  i/o variables
c
      character*10       prognme
      character*20       esmeth                      
      character*20       esbas                                                   
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
c
      character*80       optl(maxoptl)
      double precision   ener
      integer            ierr
      type (qmmminf)  :: qmmminp 
c
c  local variables
c
      type (cartgeom) ::  cogeom
      type (mmvalen)	::  commgeom

      integer   mmcutoff(maxatm)
      integer   i, j, k, ii, jj, kk
      integer   icutoff, conatm, mmnatm
c
c
      icutoff = qmmminp%lcutoff
      if(icutoff.eq.0) then
           call progmmstehk1(prognme, esmeth, esbas, 
     * geom, mmgeom, optl, ener, ierr, qmmminp)
      elseif(icutoff.eq.1) then
           call qmmmcogeom(geom, mmgeom, cogeom, commgeom,
     *            qmmminp, mmnatm,mmcutoff)
           call progmmstehk1(prognme, esmeth, esbas, 
     * cogeom, commgeom, optl, ener, ierr, qmmminp)
      qmmminp%mmnatm=mmnatm
      end if

      return
      end


c*****************************************************************************
c  PROGESTHK
c*****************************************************************************
c
      subroutine progesthk(prognme, esmeth, esbas, qmcharge,qmmult,
     *            ssgeom, optl, ener, ierr, qmmminp, ssmmgeom)
c
c     Subroutine progesthk
c
c     Created:  Aug. 28, 2006 by Yan Zhang
c               bases on progesthk1
c
c     Purpose:  Get a single point energy for the small system in the 
c        presence of background charges.
c
c
c     Called by:  optehk, qmmmehk
c     Calls:  progesthk1, qmmmcogeom
c
c     Variables defined here:
c
      use caltyp
      use files
      use input
      implicit none
c
c i/o variables
c
      character*10       prognme
      character*20       esmeth                      
      character*20       esbas                                                   
c
      character*80       optl(maxoptl)
      double precision   ener
      integer            ierr
      integer      qmcharge, qmmult
      type (cartgeom) :: ssgeom
      type (mmvalen)  :: ssmmgeom   
      type (qmmminf)  :: qmmminp
c
c local variables
c
      type (cartgeom) ::  cogeom
      type (mmvalen)	::  commgeom

      integer   mmcutoff(maxatm)
      integer   i, j, k, ii, jj, kk
      integer   icutoff, conatm, mmnatm
c
c
      icutoff = qmmminp%lcutoff
      if(icutoff.eq.0) then
          call progesthk1(prognme, esmeth, esbas, qmcharge,qmmult,
     *            ssgeom, optl, ener, ierr, qmmminp, ssmmgeom)

      elseif(icutoff.eq.1) then
           call qmmmcogeom(ssgeom, ssmmgeom, cogeom, commgeom,
     *            qmmminp, mmnatm,mmcutoff)

          call progesthk1(prognme, esmeth, esbas, qmcharge,qmmult,
     *            cogeom, optl, ener, ierr, qmmminp, commgeom)

      qmmminp%mmnatm=mmnatm
      end if

      return
      end

c
c*****************************************************************************
c  PROGMMSTGHK
c*****************************************************************************
c
      subroutine progmmstghk(ppnum, prognme, esmeth, esbas, geom, 
     *                   mmgeom, optl, ener, grad, ierr, ver_lab,
     *                   qmmminp)
c
c     Subroutine progmmstghk
c
c     Created:  Aug. 29, 2006 by Yan Zhang 
c
c     Purpose:  Get a single point interaction gradient between the given 
c               geom and the background point charge. 
c              
c
c     Called by:  optghk, qmmmghk
c     Calls:  t41inp, t41outg, system
c
      use caltyp
      use files
      use input
      implicit none
c
c  i/o variables
c
      character*10       prognme
      character*20       esmeth, esbas, ver_lab                                                
c
      character*80       optl(maxoptl)
      integer            ierr, ppnum

      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom	          
      type (qmmminf)  :: qmmminp
      double precision   ener, grad(geom%natoms*3)             
c
c  local variables
c      
      type (cartgeom) ::  cogeom
      type (mmvalen)	::  commgeom

      integer   mmcutoff(maxatm)
      integer   i, j, k, ii, jj, kk
      integer   icutoff, conatm, mmnatm
      double precision   grad1(geom%natoms*3)             

c
c
      icutoff = qmmminp%lcutoff

      if(icutoff.eq.0) then
           call progmmstghk1(ppnum, prognme, esmeth, esbas, geom, 
     *                   mmgeom, optl, ener, grad, ierr, ver_lab,
     *                   qmmminp)

      elseif(icutoff.eq.1) then
           call qmmmcogeom(geom, mmgeom, cogeom, commgeom,
     *            qmmminp, mmnatm,mmcutoff)
           call progmmstghk1(ppnum, prognme, esmeth, esbas, cogeom, 
     *                   commgeom, optl, ener, grad1, ierr, ver_lab,
     *                   qmmminp)
           call cutoffgrad(mmgeom, commgeom, grad, grad1, 
     &                   qmmminp, mmnatm,mmcutoff)

      end if

      return

      end
c

c*****************************************************************************
c  PROGGSTHK
c*****************************************************************************
c
      subroutine proggsthk(ppnum, prognme, esmeth, esbas, 
     *   qmcharge, qmmult, ssgeom, 
     *   optl, ener, grad, ierr, ver_lab, qmmminp,ssmmgeom)
c
c     Subroutine proggsthk
c
c     Created:  Aug. 28, 2006 by Yan Zhang
c
c
c     Purpose:  Get a single point energy and gradient for the small 
c        system in the presence of background charges.
c
c
c     Called by:  qmmmghk
c
c     Calls: proggsthk,  
c
      use caltyp
      use files
      use input
      implicit none
c
c  i/o variables
c
      character*10 prognme
      character*20 esmeth, esbas, ver_lab                                                
      
      type (cartgeom) :: ssgeom
      type (mmvalen)  :: ssmmgeom   
      type (qmmminf)  :: qmmminp
      
c
      character*80 optl(maxoptl)
      double precision ener, grad(3*ssgeom%natoms)
      integer ierr, ppnum
      integer qmcharge, qmmult
c
c  local variables
c
      type (cartgeom) ::  cogeom
      type (mmvalen)	::  commgeom

      integer   mmcutoff(maxatm)
      integer   i, j, k, ii, jj, kk
      integer   icutoff, conatm, mmnatm
      double precision   grad1(ssgeom%natoms*3)             
c
c
      icutoff = qmmminp%lcutoff

      if(icutoff.eq.0) then
           call proggsthk1(ppnum, prognme, esmeth, esbas, 
     *   qmcharge, qmmult, ssgeom, 
     *   optl, ener, grad, ierr, ver_lab, qmmminp,ssmmgeom)

      elseif(icutoff.eq.1) then
           call qmmmcogeom(ssgeom, ssmmgeom, cogeom, commgeom,
     *            qmmminp, mmnatm,mmcutoff)
           call proggsthk1(ppnum, prognme, esmeth, esbas, 
     *   qmcharge, qmmult, cogeom, 
     *   optl, ener, grad1, ierr, ver_lab, qmmminp,commgeom)

           call cutoffgrad(ssmmgeom, commgeom, grad, grad1, 
     &                   qmmminp, mmnatm,mmcutoff)

      end if

      return
      end
c 
c******************************************************************************
c  CUTOFFGRAD
c******************************************************************************
c
      subroutine cutoffgrad(mmgeom, commgeom, grad, grad1, 
     &                   qmmminp, mmnatm,mmcutoff)

c
c     Subroutine cutoffgrad
c
c     Created:  Aug. 29, 2006 by Yan Zhang
c     Revised:  
c
c     Purpose:  To assign cutoff gradient to the entire system gradient.
c
c
c     Called by: progmmstghk,	proggsthk
c     Calls:
c
      use input
      use files, only : outfile
      implicit none
c
c i/o variables
c
      type (qmmminf)  :: qmmminp

      type (mmvalen)  :: mmgeom,commgeom

      integer   mmcutoff(maxatm)     
      double precision grad(mmgeom%natoms*3), grad1(mmgeom%natoms*3)
c
c local variables
c      
      integer  capnatm, qmnatm, mmnatm, bgchnatm
      integer  ssnatm, cssnatm, lsnatm, conatm, lsmmnatm      
      integer  i,j,k, ii,jj,kk
      integer  bgconatm
      double precision angtoau
c
c
      angtoau = 0.529177249d0
c
c  prepare atom numbers
c  
      qmnatm   = qmmminp%qmnatm
      conatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      ssnatm   = commgeom%natoms
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + conatm
      bgchnatm = ssnatm - lsnatm
      lsmmnatm = cssnatm + mmnatm
c
c  Fill QM and Cap Atom
c                   

      do i=1, cssnatm*3
         grad(i)=grad1(i)
      enddo
c
c  Add the MM atoms after Cap and QM atoms
c  

      do i=cssnatm+1,lsnatm
         ii = mmcutoff(i-cssnatm)
         j  = cssnatm + ii
         grad(j*3-2) = grad1(i*3-2)
         grad(j*3-1) = grad1(i*3-1)
         grad(j*3  ) = grad1(i*3)

      enddo
c
c  Add the redistribut atom
c
      do i=lsnatm+1,ssnatm
         j = lsmmnatm + i - lsnatm 
         grad(j*3-2) = grad1(i*3-2)
         grad(j*3-1) = grad1(i*3-1)
         grad(j*3  ) = grad1(i*3)

      enddo
c
c
      qmmminp%mmnatm  = mmnatm
      return
      end
c
c******************************************************************
c******************************************************************
c


