c
c*******************************************************************************
c  MLHEDR
c*******************************************************************************
c
      subroutine mlhedr
c
c     Subroutine mlhedr
c
c     Created:  Feb. 21, 2005 by Hai Lin
c     Revised:  May  8 , 2006   by Yan Zhang
c     Revised:  June 28, 2010  by Soroosh Pezeshki
c     Revised:  Jun. 16, 2014  by Bo Wang
c     Revised:  Sept. 21, 2017  by Xin-Ping Wu
c     Revised:  Sept. 19, 2018  by Xin-Ping Wu
c     Purpose:  write QMMM program header to outfile
c
c     Description:  this subroutine writes the general program information
c                   to the output file.
c
c     Called by:  main
c     Calls:
c
c     Variables defined here:
c
c
      use files, only : outfile
      implicit none
c
      write (outfile,1000)
      write (outfile,1050)
      write (outfile,1100)
      write (outfile,1110)
      write (outfile,1120)
      write (outfile,1200)
      write (outfile,1300)
      write (outfile,1000)


c
 1000 format (/,80(1h*))
c 1050 format (/3x,'QMMM 1.3.7')      
c 1050 format (/3x,'QMMM 1.4.0')                                         0628SP10
c
c 1050  format (/3x,'QMMM 2017')                                         0921XW17
 1050  format (/3x,'QMMM 2018')                                         0919XW18
c 1050  format (/3x,'QMMM 2015')                                         0330BW15
 1100 format (/3X,'Hai Lin, Yan Zhang, and Soroosh Pezeshki')           0628SP10
c 1100 format (/3X,'Hai Lin and Yan Zhang')

c
c modified by Yan Zhang, June 17, 2009
c
c 1110  format (/3x,'University of Colorado at Denver and ',
c     *     'Health Sciences Center') 
1110  format (/3x,'University of Colorado Denver')
c
c end June 17, 2009
c
c 1120 format (/3x, 'Donald G. Truhlar')
c
 1120 format (/3x, 'Bo Wang, Xin-Ping Wu, ',                            0921XW17
     *        'Laura Gagliardi, and Donald G. Truhlar')                 0921XW17
c 1120 format (/3x, 'Bo Wang and Donald G. Truhlar')                     0621BW14

 1200 format (/3x,'Department of Chemistry and Supercomputer Institute',
     *        /3x,'University of Minnesota, Minneapolis, Minnesota',/)  
c 1300 format (3x,'Copyright 2004-2009',  
c 1300 format (3x,'Copyright 2004-2010',                                 0628SP10
c     *        /3x,'Donald G. Truhlar and Regents of the University ',
c     *            'of Minnesota')
c 1300 format (3x,'Copyright 2004-2015',                                 0621BW14
c 1300 format (3x,'Copyright 2004-2017',                                 0921XW17
 1300 format (3x,'Copyright 2004-2018',                                 0919XW18
     *        /3x,'Donald G. Truhlar and Regents of the University ',
     *            'of Minnesota')

c
      end

c
c************************************************************************
c     INSUMRY
c************************************************************************
c
      subroutine insumry( mlgeninp, progrms, mloptinp, 
     *      extoptinp, mcoptl, lqmmm, qmmminp, ltest, ltestmm, 
     *      ldynamic, dyninp,                                           0524SP10
     *      tmeth, tbas, toptl)                                         
c
c     Created:  Feb. 21, 2005 by Hai Lin
c     Revised:  May  23, 2006 by Yan Zhang
c               Aug.  8, 2006 by Yan Zhang
c               Dec. 13, 2006 by Yan Zhang
c               Sep. 04, 2007 by Yan Zhang
c               Jun. 8, 2010 by Soroosh Pezeshki
c
c     Purpose:  Write out a summary of all input parameters
c
c
      use input, only : general, mlopt, extopt, proglist, qmmminf,
     *     maxoptl, dynamicsinf                                            
      use files, only : outfile
      use param, only : linear, nonlin, lints, nonlints, uniths, 
     *      lowqmhs, qmmmhs,mmhs
      use caltyp                                                        

      implicit none
c
c  local variables
c
      character*20  ver_lab
      character*1   cswit(0:2)
      character*3   swit(0:1)
      character*3   yswit(0:1)
c
c   modified 2/10/2007 by Christoph Riplinger
c
c      character*6   aswit(0:5)
      character*7   aswit(0:6)
c
c   end 2/10/2007 by Christoph 
c
c
c   add 24/05/2010 by Soroosh Pezeshki
c
      integer :: ldynamic
      type (dynamicsinf) :: dyninp
      double precision dtemp

c
c  end add 5/24/2010 Soroosh Pezeshki
c
      integer       i,j,k,jj
c
c  i/o variables
c
      type (proglist) :: progrms
      type (general)  :: mlgeninp
      integer            lqmmm, ltest, ltestmm              
c
      character*80    :: mcoptl(maxoptl), toptl(maxoptl)
      character*20    :: tmeth, tbas
      type (qmmminf) :: qmmminp                                         
          
      type (mlopt)   :: mloptinp
      type (extopt)  :: extoptinp
c          
      data swit /'off','on'/
      data yswit /'no','yes'/
c
c   add 2/10/2007 by Christoph Riplinger
c
c      data aswit /'NR','BFGS','DFP','EF','GAUEXT','LBFGS'/
      data aswit /'NR','BFGS','DFP','EF','GAUEXT','LBFGS','ORCAEXT'/
c
c   end 2/10/2007 by Christoph 
c
      data cswit /'Z','X','Y'/
c            
                   
c*********************************************************************
c           write out current date and time and then the title
c
c*********************************************************************
c
      write(outfile,1000)
      call dattim(outfile)
      do 1 i=1,5
          if (mlgeninp%mtitle(i).ne.' ')
     *           write(outfile,1001) mlgeninp%mtitle(i)
1     continue
c
1000  format(//,80('-'),/80('-')/)
c
1001  format(a80)
c
c
c*********************************************************************
c           write out data read in *MULTIGEN section
c
c*********************************************************************
c
      write(outfile,1100)
c
1100  format(//,80('-'),/80('-'),//2x,' Multigen Section Parameters: ',
     *   //80('-')/)
c
c CALMODEL                                                              
c    QM, MM, or QM/MM
c
      write(outfile,1130) mlgeninp%calmodel 
      if(mlgeninp%calmodel .eq. 'qm/mm') then
         write(outfile,1140) 
      elseif(mlgeninp%calmodel .eq. 'mm') then
         write(outfile,1150) 
      else
         write(outfile,1160) 
      endif       
1130  format (/2x,'CALMODEL: ',t50,a20) 
1140  format (/20x,'Combined QM/MM model is selected.')
1150  format (/20x,'Molecular Mechanics model is selected.')
1160  format (/20x,'Quantum Mechanics model is selected.')
c
c MMVALVEN
c   Geometry and connectivity
c
      if(mlgeninp%lmmvalen .eq. 1) then
        write(outfile,1170)
      endif 
1170  format (/2x,'MMVALEN: Geometry and connectivies will be ',
     *    'read from .crd file. ')
c
c PRNATMPRM
c   Print MM parameters
c 
      if(mlgeninp%mmgeom%lprnatmprm .eq. 1) then
        write(outfile,1180)
      endif 
1180  format (/2x,'PRNATMPRM: MM parameters will be printed out')
c               

c
c  NATOMS
c    Number of atoms in total
c
      write(outfile,1200) mlgeninp%geom%natoms
c1200  format (/2x,'NATOMS:',t50,i6)
1200  format (/2x,'NATOMS:',t46,i6)                                     0825SP11
c
c  Charge and Multiplicity
c    Note: For QM/MM model, the entire system and the capped 
c          primary system must have the same charge and multiplicity
c
      write(outfile,1300) mlgeninp%geom%charge
      write(outfile,1350) mlgeninp%geom%mult
1300  format (/2x,'System CHARGE:',t50,i2)
1350  format (2x,'System MULTIPLICITY:',t50,i2)
c
c  Geometry unit and type
c
      if (mlgeninp%geomunit.eq.1) then
         write (outfile,1400) 'angstroms'
      else
         write (outfile,1400) 'a.u.'
      endif
      if (mlgeninp%geomtype.eq.0) then
         write (outfile,1450) 'cartesian'
      endif
1400  format (/2x,'GEOMUNIT: Units for input geometries:',t50,a10)
1450  format (2x,'GEOMTYPE: Form of input geometries:',t50,a10)
c
c  Input geometry
c
      if(mlgeninp%mmgeom%lprnatmprm .eq. 1) then                        0516SP11
      write(outfile,1455)
1455  format(/2x,'** GEOMETRY:'/)
      write(outfile,1460)
1460  format(/5x,'Atomic information:',/12x,'Atom ',4x,
     *  'Atomic Symbol',5x,'Mass (amu)',/)
      do 2 i=1,mlgeninp%geom%natoms
         write(outfile,1465) i, mlgeninp%geom%atm(i),
     *          mlgeninp%geom%atmmss(i)
   2  continue
1465  format(13x,i6,15x,a2,10x,f10.6)
      write(outfile,1470)
1470  format(//5x,'Atomic Coordinates (in a.u.):',//8x,'Atom ',t28,
     *  'X', t48, 'Y', t68, 'Z'/)
      do 5 i=1, mlgeninp%geom%natoms
         write(outfile,1473) i, mlgeninp%geom%aucoord(3*i-2),
     *        mlgeninp%geom%aucoord(3*i-1), mlgeninp%geom%aucoord(3*i)
   5  continue
      write(outfile,1471)
1471  format(//5x,'Atomic Coordinates (in ang.):',//8x,'Atom ',t28,
     *  'X', t48, 'Y', t68, 'Z'/)
      do 7 i=1, mlgeninp%geom%natoms
         write(outfile,1473) i, mlgeninp%geom%angcoord(3*i-2),
     *      mlgeninp%geom%angcoord(3*i-1), mlgeninp%geom%angcoord(3*i)
   7  continue
1473  format(9x,i6,t20,d15.8,t40,d15.8,t60,d15.8)
      endif                                                             0516SP11
c
c  Switches for calculations
c
      write(outfile,1500)
1500  format(/2x,'** Calculations:'/)
c
      write(outfile,1535) yswit(mlgeninp%lextopt)
      write(outfile,1540) yswit(mlgeninp%lopt)
      write(outfile,1510) yswit(mlgeninp%lener)
      write(outfile,1520) yswit(mlgeninp%lgrad)
      write(outfile,1530) yswit(mlgeninp%lhess)
c
1510  format(4x,'ENERGY:  the energy is calculated:',t60,a3)
1520  format(4x,'GRADIENT:  the gradient is calculated:',t60,a3)
1530  format(4x,'HESSIAN:  the hessian is calculated:',t60,a3)
1535  format(4x,'EXTOPT:  optimization with an external program:',
     *           t60, a3)
1540  format(4x,'MULTIOPT:  optimization with Multilevel:',t60,a3)

      if (mlgeninp%lextopt .eq. 1) then
         write(outfile,1600)
c
1600  format(//,80('-'),/80('-'),//2x,' EXTOPT Section Parameters: ',
     *   //80('-')/)
         write(outfile,1610) progrms%extoptprog
c 
         if( progrms%extoptprog .eq. 'tinker') then                   
            write(outfile,1620) extoptinp%meth
        write(outfile,1622) extoptinp%bas
      else
            write(outfile,1620) extoptinp%meth
            write(outfile,1630) extoptinp%bas
      endif 
c    

         write(outfile,1640)
         do 37 i=1,maxoptl   
            if (extoptinp%optl(i).ne.'                              ')
     *            write (outfile,1650) extoptinp%optl(i)
  37     continue
      endif
1610  format(5x,'PROGRAM:  external optimization program:',t60,a)
1620  format(5x,'METHOD:  electronic structure method:',t60,a)
1630  format(5x,'BASIS:  basis set:',t60,a)
1622  format(5x,'VERSION:  current version of tinker:',t60,a)
1632  format(5x,'FORCEFIELD:  force field parameters:',t60,a)           
1640  format(/5x,'OPTIONS for PROGRAM:')
1650  format(5x,a75)

      if (mlgeninp%lopt .eq. 1) then
         write(outfile,1660)        
c
1660  format(//,80('-'),/80('-'),//2x,' MULTIOPT Section Parameters: ',
     *   //80('-')/)
         write(outfile,1670) aswit(mloptinp%algrthm)
         write(outfile,1680) mloptinp%mlmeth
c
c  display the parameter for the different algorithm
c      revised by Yan Zhang 12/13/2006
c  BFGS, NR, EF, DFP
c
        if (mloptinp%algrthm.le.3) then

           if (mloptinp%iop .eq. linear) then
              write(outfile,1690) 'Linear Minimum'
           elseif (mloptinp%iop .eq. nonlin) then
              write(outfile,1690) 'Non-linear Minimum'
           elseif (mloptinp%iop .eq. lints) then
              write(outfile,1690) 'Linear Transition State'
           elseif (mloptinp%iop .eq. nonlints) then
              write(outfile,1690) 'Non-linear Transition State'
           endif
           if (mloptinp%htyp .eq. uniths) then
              write(outfile,1691) 'Scaled Unit Matrix'
           elseif (mloptinp%htyp .eq. lowqmhs) then
              write(outfile,1691) 'Low Level QM Hessian'
              write(outfile,1693) mloptinp%hmeth
              write(outfile,1694) mloptinp%hbas
           elseif (mloptinp%htyp .eq. qmmmhs) then
              write(outfile,1691) 'High Level QM/MM Hessian'
           endif
           write(outfile,1695) yswit(mloptinp%ihunit)
           if (mloptinp%ihunit.eq.0 .or.
     *           mloptinp%htyp .eq. uniths) then
              write(outfile,1696) mloptinp%hscale
           endif
           write(outfile,1710) mloptinp%gcomp
           write(outfile,1720) mloptinp%scale
           write(outfile,1730) mloptinp%stptol
           write(outfile,1740) mloptinp%hrec
           write(outfile,1750) mloptinp%niter
c                                                                      
           if (mloptinp%llinmn .eq. 1) then
              write(outfile,1755) 'on'
           else
              write(outfile,1755) 'off'
           endif                
c                                                                      
           write(outfile,1760) swit(mloptinp%retry)
           write(outfile,1765) 
c           write(outfile,1767)
           do 39 i=1, 3*mlgeninp%geom%natoms
              if (mloptinp%constant(i) .eq. 1) then
                 write(outfile,1766) (i+2)/3, cswit(mod(i,3))
              endif
  39       continue
c
c GAUEXT
c
c
c GAUEXT
c
c
c   modified 2/10/2007 by Christoph Riplinger
c
c        elseif (mloptinp%algrthm.eq.4) then
c           if(mloptinp%ipartcal.eq.1) then
c              write(outfile,2100)
c              write(outfile,2101)mloptinp%pcharge
c              write(outfile,2102)mloptinp%pmult
c              write(outfile,2103)
c              write(outfile,2104) (mloptinp%partatmid(i),i=1,
c     $                              mloptinp%partatmnum)
c           endif
c           write(outfile,1767)
c           do i=1,maxoptl
c              if (mloptinp%optl(i).ne.'                           ')
c     *           write (outfile,8600) mloptinp%optl(i)
c           end do
        elseif ((mloptinp%algrthm.eq.4).or.(mloptinp%algrthm.eq.6)) then
           if(mloptinp%part%ipartcal.eq.1) then
              write(outfile,2100)
              write(outfile,2101)mloptinp%part%pcharge
              write(outfile,2102)mloptinp%part%pmult
           endif
           if (mloptinp%algrthm.eq.4) then
             write(outfile,1767)
           else if (mloptinp%algrthm.eq.6) then
             write(outfile,1768)
           endif
           do i=1,maxoptl
              if (mloptinp%optl(i).ne.'                           ')
     *           write (outfile,8600) mloptinp%optl(i)
           end do           
c
c   end 2/10/2007 by Christoph 
c
c LBFGS
c
        elseif (mloptinp%algrthm.eq.5) then

           if (mloptinp%iop .eq. linear) then
              write(outfile,1690) 'Linear Minimum'
           elseif (mloptinp%iop .eq. nonlin) then
              write(outfile,1690) 'Non-linear Minimum'
           endif
           write(outfile,1765) 
           do i=1, 3*mlgeninp%geom%natoms
              if (mloptinp%constant(i) .eq. 1) then
                 write(outfile,1766) (i+2)/3, cswit(mod(i,3))
              endif
           end do

           write(outfile,2201) mloptinp%gcomp
           write(outfile,2202) mloptinp%rmsgrad
           write(outfile,2203) mloptinp%maxdx
           write(outfile,2204) mloptinp%rmsdx
           write(outfile,2205) mloptinp%niter
           write(outfile,2206) mloptinp%maxstep
           write(outfile,2207) mloptinp%minstep
           write(outfile,2208) mloptinp%cappa
           write(outfile,2209) mloptinp%maxslope
           write(outfile,2210) mloptinp%maxang
           write(outfile,2211) mloptinp%minenr
           write(outfile,2212) mloptinp%intpolat

        endif

      endif

1670  format(5x,'ALGORITHM:  optimization process:',t60,a)
1680  format(5x,'METHOD:  Multilevel method used:',t60,a)
1690  format(5x,'MOLECULE TYPE:', t60, a)
1691  format(5x,'HESSIAN:  type used to calculate step size:',t60,a)
1693  format(5x,'HMETH:  low level hessian method:',t60,a)
1694  format(5x,'HBAS:  low level hessian basis set:',t60,a)
1695  format(5x,'HESSINIT:  hessian calculation at first step:',t60,a)
1696  format(5x,'HSCALE:  scaling of the unit matrix:',t60,d9.2)
1710  format(5x,'GCOMP:  convergence threshold for gradient:',
     *        t60, d10.3)
1720  format(5x,'SCALE:  max rms step size:', t60, d10.3)
1730  format(5x,'STPTOL:  optimization failure condition:',t60, d10.3)
1740  format(5x,'HREC:  number of steps between hessian ',
     *        'calculations:',t60, i3)
1750  format(5x,'NITER:  number of optimization iterations:',t60,i3)
1755  format(5x,'LINMN:  use Brent line minimization:',t60,a)
1760  format(5x,'RETRY:  switch to full Newton if NR/BFGS/DFP fail:',
     *        t60, a)
1765  format(5x,'CONSTANT:  Coordinates to be held constant:')
1766  format(20x,i3,1x,a1)
1767  format(5x,'OPTIONS specified for Gau_External calculations:')     0222HL04

2100  format(5x,'PARTIAL:  partial optimization')
2101  format(5x,'PARTCHARGE:  optimized atom charge:',t60,i2)
2102  format(5x,'PARTMULT:  optimized atom multiplicity:',t60,i2)
c
c   add 2/10/2007 by Christoph Riplinger
c
1768  format(5x,'OPTIONS specified for ORCA_External calculations:')    
c
c   end 2/10/2007 by Christoph 
c

2201  format (5x,'GCOMP:',t60,d10.3)
2202  format (5x,'RMSGRAD:',t60,d10.3)
2203  format (5x,'MAXDX:',t60,d10.3)
2204  format (5x,'RMSDX:',t60,d10.3)
2205  format (5x,'NITER:',t60,i10)
2206  format (5x,'MAXSTEP:',t60,d10.3)
2207  format (5x,'MINSTEP:',t60,d10.3)
2208  format (5x,'CAPPA:',t60,d10.3)
2209  format (5x,'MAXSLOPE:',t60,d10.3)
2210  format (5x,'MAXANG:',t60,d10.3)
2211  format (5x,'MINENR:',t60,d10.3)
2212  format (5x,'INTPOLAT:',t60,i9)
c
c  end by Yan Zhang 12/13/2006
c
c
c*********************************************************************
c           write out data read in *TEST section
c
c*********************************************************************
c
      if (ltest.eq.1)  then
         write(outfile,1775)
c
1775  format(//,80('-'),/80('-'),//2x,' TEST Section Parameters: ',
     *   //80('-')/)
         write (outfile,1783) progrms%testprog
c
c     add 5/23/2006 by Yan Zhang
c
         if(progrms%testprog.eq.'gamess') tbas='readoptions'
c
c   end 5/23/2006 by Yan Zhang
c
         write (outfile,1780) tmeth
         write (outfile,1782) tbas
         do 1787 i=1,maxoptl
            if (toptl(i).ne.'                           ')
     *           write (outfile,8600) toptl(i)
1787     continue
      endif                                                             
1780  format (/10x,'METHOD:',t50,a20)
1782  format (10x,'BASIS:',t50,a20)
1783  format (/2x,'PROGRAM: Program called for the TEST cal.:',t60,a)
1785  format (/10x,'OPTIONS specified for TEST calculations:')
c
c
c*********************************************************************
c           write out data read in *TESTMM section
c*********************************************************************
c
      if (ltestmm.eq.1)  then                                           
         write(outfile,1790)                                            
c
1790  format(//,80('-'),/80('-'),//2x,' TESTMM Section Parameters: ',   
     *   //80('-')/)                                                    
         write (outfile,1796) progrms%testmmprog
         write (outfile,1794) tmeth
         write (outfile,1795) tbas
         write (outfile,1797)
         do 1792 i=1,maxoptl
            if (toptl(i).ne.'                           ')
     *           write (outfile,8600) toptl(i)
1792     continue
         write (outfile,*)
      endif

1794  format (/10x,'VERSION:',t50,a20)
1795  format (10x,'FORCE FIELD:',t50,a20)
1796  format (/2x,'PROGRAM: Program called for the TESTMM cal.:',t60,a)
1797  format (/10x,'OPTIONS specified for TESTMM calculations:')
8600  format (5x,a75)
c
c*********************************************************************
c           write out data read in *QMMM section
c
c*********************************************************************
c
      if (lqmmm.eq.1)  then
         write(outfile,9800)
c
9800  format(//,80('-'),/80('-'),//2x,' QMMM Section Parameters: ',
     *   //80('-')/)
         write (outfile,9900) swit(lqmmm)
9900  format (/5x,'QMMM:',t50,a3)
c
c  Border type
c    Currently, only cap atom treatment is implemented
c
         write (outfile,9902) qmmminp%border     
9902  format (/7x,'BORDERTYPE: Treament for border:',t50,a20)
c
c  QM options
c     
         write (outfile,9911) progrms%qmprog
         write (outfile,9905) qmmminp%qmmeth
         write (outfile,9910) qmmminp%qmbas
c
c  CPS charge and multiplicity added by Yan Zhang 12/13/2006
c
         write (outfile,9913) qmmminp%qmcharge
         write (outfile,9914) qmmminp%qmmult
c
c end by Yan Zhang 12/13/2006
c
         write (outfile,9912) 
         do 9805 i=1,maxoptl
            if (qmmminp%qmoptl(i).ne.'                           ')
     *           write (outfile,8600) qmmminp%qmoptl(i)
9805     continue

9905  format (/7x,'QMKEY:METHOD:',t50,a20)
9910  format (7x,'QMKEY:BASIS:',t50,a20)
9911  format (7x,'QMKEY:PROGRAM:  QM Package called:',t50,a10)
9912  format (7x,'QMKEY:OPTIONS:')
9913  format (7x,'QMKEY:CHARGE:',t50,i2)
9914  format (7x,'QMKEY:MULTIPLICITY:',t50,i2)
c
c  QMMMCUTOFF options 12/13/2006 by Yan Zhang
c     
         if(qmmminp%lcutoff.eq.1) then
            if(qmmminp%selcent.eq.1) then
               write (outfile,9201) qmmminp%ccutoffid
               write (outfile,9202) qmmminp%rcutoff
            elseif (qmmminp%selcent.eq.2) then
               write (outfile,9203) qmmminp%cutoffcoor(1), 
     $             qmmminp%cutoffcoor(2),qmmminp%cutoffcoor(3)
               write (outfile,9202) qmmminp%rcutoff
            endif
          endif
9201   format (/7x,'QMMMCUTOFF:CUTOFFCENTID:',t50,i6)
9202   format (7x,'QMMMCUTOFF:CUTOFFRAD:',t50,f10.6)
9203   format (/7x,'QMMCUTOFF:CUTOFFCENTXYZ:',t50,3(f12.6,2x))
c
c  end by Yan Zhang 12/13/2006
c
c  POLAR options 8/8/2006 by Yan Zhang
c     
         if(qmmminp%polmeth(:5).ne.'none') then
            write (outfile,901) qmmminp%polmeth
            write (outfile,902) qmmminp%polmaxdq
            write (outfile,903) qmmminp%polrmsdq
            write (outfile,904) qmmminp%polcycl
            write (outfile,905) qmmminp%polgnum
            write (outfile,906) 
            do i=1,103
            if (qmmminp%polparal(i).ne.'                           ')
     *           write (outfile,8600) qmmminp%polparal(i)
            end do
            write (outfile,907)
            do i=1,qmmminp%polgnum
                jj=qmmminp%polgidnum(i)
        write(outfile,*)'      group:',(qmmminp%polgid(i,j),j=1,jj)
          end do
             write(outfile,*)
         end if

901   format (/7x,'POLAR:METHOD:',t50,a20)
902   format (7x,'POLAR:MAXDQ:',t50,f10.6)
903   format (7x,'POLAR:RMSDQ:',t50,f10.6)
904   format (7x,'POLAR:CYCLE:',t50,i6)
905   format (7x,'POLAR:GROUPNUM:',t50,i6)
906   format (7x,'POLAR:PARAMETER:')
907   format (7x,'POLAR:GROUP:')
c
c  end 8/8/2006 by Yan Zhang
c
c  Add by Yan Zhang 9/04/2007
c  Flexible-boundary treatment
c     
         if(qmmminp%iflexbound.eq.1) then
            write (outfile,910) qmmminp%asqmcharge
            write (outfile,911) qmmminp%asqmmult
            write (outfile,912) qmmminp%maxfbcycl
            write (outfile,913) qmmminp%maxfbdq
            write (outfile,914) qmmminp%rmsfbdq
            write (outfile,915) qmmminp%maxfbdqtot
            write (outfile,916) qmmminp%temprt
            write (outfile,917) qmmminp%fbccmeth
            write (outfile,918) qmmminp%fbgid

            write(outfile,921)
            do i=1,103
            if (qmmminp%sscp(i).ne.'  ')
     *           write (outfile,8600) qmmminp%sscp(i)
            end do

            do i=1,qmmminp%fbgnum
              jj=qmmminp%fbgan(i)
              write(outfile,922) i
              write(outfile,923)qmmminp%fbgaid(i,1:jj)
              write(outfile,924) 
            end do

         end if

910   format (/,7x,'FLEXBOUND:CHARGE:',t50,i4)
911   format (7x,'FLEXBOUND:MULT:',t50,i4)
912   format (7x,'FLEXBOUND:MAXCYCLE:',t50,I5)
913   format (7x,'FLEXBOUND:MAXDQ:',t50,f10.6)
914   format (7x,'FLEXBOUND:RMSDQ:',t50,f10.6)
915   format (7x,'FLEXBOUND:MAXCHARGTRANS:',t50,f10.6)
916   format (7x,'FLEXBOUND:TEMPERATURE:',t50,f15.6)
917   format (7x,'FLEXBOUND:CALCHARGMETH:',t50,a9)
918   format (7x,'FLEXBOUND:FBGROUPID:',t50,i4)
921   format (7x,'FLEXBOUND:PARAMETER:')
922   format (7x,'FLEXBOUND:GROUP',2x,i4)
923   format (17x,10(i6,1x))
924   format (17x,'END')
c
c  END by Yan Zhang 9/04/2007
c
c  MM options
c
         write (outfile,9921) progrms%mmprog
         write (outfile,9915) qmmminp%mmmeth
         write (outfile,9920) qmmminp%mmbas
         write (outfile,9925)
         do 9810 i=1,maxoptl
            if (qmmminp%mmoptl(i).ne.'                           ')
     *           write (outfile,8600) qmmminp%mmoptl(i)
9810     continue
9915  format (/7x,'MMKEY:VERSION:',t50,a20)
9920  format (7x,'MMKEY:FORCEFIELD:',t50,a20)
9921  format (7x,'MMKEY:PROGRAM:  MM Package called:',t50,a10)
9925  format (7x,'MMKEY:OPTIONS:')
c
c  Cap atom
c
         write (outfile,9930) qmmminp%capatm%capnatm
         write (outfile,9931) 
         do i=1,qmmminp%capatm%capnatm
            write (outfile,9932) qmmminp%capatm%capatm(i),
     *        qmmminp%capatm%capatmtyp(i),
     *        qmmminp%capatm%capatmtypname(i),
     *        qmmminp%capatm%capbondscale(i),
     *        qmmminp%capatm%capatmqm(i),
     *        qmmminp%capatm%capatmm1(i)
         enddo
9930  format (/7x,'CAPATOM: # of capping atoms:',t50,i4)
9931  format (7x,'CAPATOM: AtomID  Atomtype(name)  Bondscale',  
     *        2x,'QM-link  M1-link')
9932  format (7x,10x,i4,2x,i5,'(',a5,')',2x,f12.8,2x,i4,4x,i4)

c
c  Cap+QM
c         
         write (outfile,9935) qmmminp%qmnatm + qmmminp%capatm%capnatm
         write (outfile,9939)
         write (outfile,9940) qmmminp%qmatm(:qmmminp%qmnatm)
         write (outfile,9940) 
     *   qmmminp%capatm%capatm(:qmmminp%capatm%capnatm)
9935  format (/7x,'CPS:  # of atoms in primary system:',t50,i3)
9939  format (7x,'CPS:  Atoms in primary system:')
9940  format (15x,i3)
c
c  M1 layer
c
         write (outfile,9942) qmmminp%mm1stlay%natm1stlay
      do i=1, qmmminp%mm1stlay%natm1stlay
        j = qmmminp%mm1stlay%atm1stlaym2num(i)
        write (outfile,9943)
     *         qmmminp%mm1stlay%atm1stlay(i),
     *         qmmminp%mm1stlay%atm1stlayqm(i),
     *         j, qmmminp%mm1stlay%atm1stlaym2(i,:j)
      enddo
9942  format (/2x,'Number of M1 layer (cap atom hosts) atoms:',
     *  t50,i4,/2x,'AtomID QM-link  (# of M2-links)  M2-links')
9943  format (2x,i5,3x,i5,10x,'(',i2,')',4x,5(i4,2x))     
c
c  M2 layer
c        
      i = qmmminp%mm2ndlay%natm2ndlay
      write (outfile,9944) i
      do j=1, i
        write(outfile, 9945) qmmminp%mm2ndlay%atm2ndlay(j),
     *         qmmminp%mm2ndlay%atm2ndlaym1(j)      
      enddo
9944  format (/2x,'Number of M2 layer (bonded to M1) atoms:',
     *   t50,i4,/2x,'AtomID (M1-link)')
9945  format (2x,i5,3x,'(',i5,')')     
c
c  M3 layer
c
         i = qmmminp%mm3rdlay%natm3rdlay
         write (outfile,9950) i
         do j=1, i
            write(outfile, 9945) qmmminp%mm3rdlay%atm3rdlay(j),
     *         qmmminp%mm3rdlay%atm3rdlaym2(j)
         enddo
9950  format (/2x,'Number of M3 layer (bonded to M2) atoms:',
     *   t50,i4,/2x,'AtomID (M2-link)')                              
c
c  MM atoms (excluding the M1 atoms)
c
      if (dyninp%apmethode.lt.1) then                                   0516SP11
      i = qmmminp%mmnatm
      write (outfile,9946) i
      j = 1
      do while (j .le. i)
        k = min(j + 4, i)
        write (outfile,9948) qmmminp%mmatm(j:k)
        j = j + 5
      enddo
9946  format (/2x,'Number of MM (excluding cap atom hosts) atoms:'
     *        ,t50,i4,/2x,'The ID of these atoms are: (5 per line)')
9948  format (2x,5(i5,1x))   
      endif                                                             0516SP11
c
c  M2 in MM (debug only)
c
      if (tdebug .eq. 1) then 
         i = qmmminp%mmnatm
      write(outfile, 9980)
         write(outfile, 9982) 
         do j=1, i
            write(outfile,*) j, qmmminp%mmatm(j), qmmminp%mlist(j)
         enddo
         write(outfile, 9981)
      endif  
9980  format (/,10('Debug')) 
9981  format (10('Debug'),/)
9982  format ('M2-in-MM list (0 = non-M2):',/,'MM-ix  MM-id  M2-ix')     
c
c  Border charge
c
         write (outfile,9951) qmmminp%embed
         
         if(qmmminp%embed .eq. 'electric') then
            write (outfile,9952) qmmminp%bordercharge
         
            if(qmmminp%bordercharge .eq. 'scale') then
               write (outfile,9953) qmmminp%chscale%chargelay
               write (outfile,9954)        
           do i=1, qmmminp%chscale%chargelay
                  write (outfile,9955) i,
     *              qmmminp%chscale%chargescale(i)
               enddo
            elseif(qmmminp%bordercharge .eq. 'shift') then
               write (outfile,9960)
               write (outfile,9961) qmmminp%chredist%dipdist
           
            elseif(qmmminp%bordercharge .eq. 'redist1') then
               write (outfile,9962)
           write (outfile,9963) qmmminp%chredist%chposit

            elseif(qmmminp%bordercharge .eq. 'redist2') then
               write (outfile,9964)
           write (outfile,9963) qmmminp%chredist%chposit
           write (outfile,9965) qmmminp%chredist%dipposit
           write (outfile,9961) qmmminp%chredist%dipdist

            elseif(qmmminp%bordercharge .eq. 'redist3') then
               write (outfile,9967)
               write (outfile,9963) qmmminp%chredist%chposit
                        
            endif   
         endif                   
      write (outfile,9966)
      
9951  format (/7x,'EMBED:',t50,a20)
9952  format (7x,'BORDERCHARGE:',t50,a20)

9953  format (/7x,'CHARGELAYER:',t50,i4,/9x,
     *        '(# of layers where charges scaled,',/9x,
     *        ' the 1st layer is the cap atom host)')
9954  format (/7x,'CHARGESCALE:  MM layer       Scale factor')     
9955  format (22x,i4,10x,f12.8)

9960  format (/9x,'Charge at MM 1st layer shifted to ',
     *        /9x,'MM 2nd layer with dipole compensation',
     *        /9x,'by a pair of additional charges.',
     *        /9x,'(the 1st layer is the cap atom hosts)')
9961  format (/7X,'DIPSIST:     ratio of r/R:',t50,f12.8,/9x,
     *        'r is distance between two point charges',/9x,
     *        'to mimic the M1-M2 dipole, R is M1-M2 bondlength')       
9962  format (/9x,'Charge at MM 1st layer redistributed on',
     *        /9x,'M1-M2 bond without dipole compensation.',     
     *        /9x,'(the 1st layer is the cap atom hosts)')
9963  format (/7X,'CHARGEPOSIT: ratio of rc/R:',t50,f12.8,/9x,
     *        'rc is distance between M1 and the',/9x, 
     *        'redistributed point charge, R is M1-M2 bondlength')     
9964  format (/9x,'Charge at MM 1st layer redistributed on',
     *        /9x,'M1-M2 bond with dipole compensation',
     *        /9x,'by a pair of additional charges.',     
     *        /9x,'(the 1st layer is the cap atom hosts)')     
9965  format (/7X,'DIPPOSIT:    ratio of rd/R:',t50,f12.8,/9x,
     *        'rd is distance between M1 and the center ',
     *        /9x,'of compensation dipole, R is M1-M2 bondlength')

9966  format (/,80('-'))     
9967  format (/9x,'Charge at MM 1st layer redistributed on',
     *        /9x,'M1-M2 bond with dipole compensation.',
     *        /9x,'(the 1st layer is the cap atom hosts)')
          
      endif

c
c  Add Jun. 8, 2010 by Soroosh Pezeshki
c
c*********************************************************************
c           write out data read in *DYNAMICS section
c*********************************************************************
c
      if (ldynamic.eq.1)  then                                           
         write(outfile,3000)                                            
3000  format(//,80('-'),/80('-'),//2x,' DYNAMICS Section Parameters: ',   
     *   //80('-')/)                                                    
         write (outfile,3001)
3001  format (/10x,'OPTIONS specified for DYNAMICS calculations:',t70)
         write (outfile,*)
c
c General settings
c
         write (outfile,3100) dyninp%steps
3100  format (5x,'Number of dynamics steps:', t50,I8)
         write (outfile,3101) dyninp%firsttimestep
3101  format (5x,'First time step:', t50,I8)
         write (outfile,3102) dyninp%timestepl
3102  format (5x,'Time step length [fs]:', t51,f10.2)
         write (outfile,3103) dyninp%trjdumps
3103  format (5x,'Trajectory output frequency [steps]:', t50,I8)
         write (outfile,3104) dyninp%outputfreq
3104  format (5x,'Energy output frequency [steps]:', t50,I8)
         write (outfile,3105) dyninp%seed
3105  format (5x,'Random seed:', t44,I14)
         if (dyninp%restart.eq.1)  write (outfile,3106)
3106  format (5x,'Restarting simulation with dynamics.restart')
         if (dyninp%fixcom.eq.1)  write (outfile,3107)
3107  format (5x,'Center of mass is fixed')

c
c Ensemble, tenperature and annealing
c
         if(dyninp%lanneal.eq.0) then
            if (dyninp%ensemble.eq.1) then
                write (outfile,3200)
                if (dyninp%restart.eq.0) 
     *               write (outfile,3211) dyninp%temperature
3200  format (5x,'NVE Ensemble')
            elseif (dyninp%ensemble.eq.2) then
                write (outfile,3201)
                write (outfile,3205) dyninp%temperature
3201  format (5x,'NVT Ensemble')
3205  format (5x,'Temperature [Kelvin]:', t53,f10.4)
            endif
c            if (dyninp%ensemble.eq.3) write (outfile,3202) 
c3202  format (5x,'NpE Ensemble')
c            if (dyninp%ensemble.eq.4) write (outfile,3203)
c3203  format (5x,'NpT Ensemble')
c            write (outfile,3204) dyninp%pressure
c3204  format (5x,'Pressure [atm]:', t53,f10.4)
         else 
            if (dyninp%lanneal.eq.1) write (outfile,3210)
3210  format (5x,'Linear temperature annealing')
            write (outfile,3211) dyninp%temperature
3211  format (5x,'Initial temperature [Kelvin]:', t53, f10.4)
            write (outfile,3212) dyninp%finaltemp
3212  format (5x,'Final Temperature [Kelvin]:', t53, f10.4)
            write (outfile,3213) dyninp%equistep
3213  format (5x,'Number of equilibration steps:', t50,I8)
            write (outfile,3214) dyninp%coolstep
3214  format (5x,'Number of cooling protocol steps:', t50,I8)
         endif
         if (dyninp%ensemble.eq.2) then
            write(outfile,3206) dyninp%tempcoupling
3206  format (5x,'Temperature bath coupling [ps]:',t53,f10.4)
3207  format (5x,'Thermostat:',t53,a)
            if (dyninp%thermostat .eq. 1) then
                write(outfile,3207) "Berendsen"
            elseif (dyninp%thermostat .eq. 2) then
                write(outfile,3207) "Nose-Hoover"
            endif

         endif

c
c Periodic box and box size
c
         if (dyninp%periodic.eq.1) then
           write (outfile,3300)
3300  format (5x,'Periodic boundary condition:', t55,'Yes')
           dtemp=dyninp%boxsize(1)*dyninp%boxsize(2)*dyninp%boxsize(3)
           if (dtemp.eq.0.d0) write (outfile,3301) dyninp%buffersize
3301  format (5x,'Buffer size: ', t53, f10.4)
           if (dyninp%boxsize(1).ne.0.d0) then
                write (outfile,3302) dyninp%boxsize(1)
3302  format (5x,'x box side length [Angstrom]: ', t53, f10.4)
           endif
           if (dyninp%boxsize(2).ne.0.d0) then
                write (outfile,3303) dyninp%boxsize(2)
3303  format (5x,'y box side length [Angstrom]: ', t53, f10.4)
           endif
           if (dyninp%boxsize(3).ne.0.d0) then
                write (outfile,3304) dyninp%boxsize(3)
3304  format (5x,'z box side length [Angstrom]: ', t53, f10.4)
           endif
         else
            write (outfile,3305)
3305  format (5x,'Periodic boundary condition:', t56,'No')
         endif

c
c Rattle algorithm
c
         if (dyninp%rattle.eq.1) write (outfile,3400)
3400  format (5x,'Rattle constraints on water molecules',
     *             ' using TIP parameters')
         if (dyninp%rattle.eq.2) write (outfile,3401)
3401  format (5x,'Rattle constraints on water molecules ',
     *             'using SPC parameters')
         if (dyninp%rattleh.eq.1) write (outfile,3402)
3402  format (5x,'Rattle constraints on non water hydrogen bonds')
         if (dyninp%rattleh.eq.2) write (outfile,3403)
3403  format (5x,'Rattle constraints on non water hydrogen bonds in'
     *               ,' MM zone')
         if (dyninp%rattle+dyninp%rattleh.gt.0) then
            write (outfile,3404) dyninp%rattleeps
            write (outfile,3405) dyninp%rattlemaxiteriation
3404  format (5x,'Rattle Epsilon', t50, d13.6)
3405  format (5x,'Max. number of rattle iterations:', t50,I8)
         endif

c
c Adaptive Partitioning
c
         if (dyninp%apmethode.eq.1) write (outfile,3500)
3500  format (5x,'Partitioning: Sorted AP algorithm')
         if (dyninp%apmethode.eq.2) write (outfile,3501)
3501  format (5x,'Partitioning: Permuted AP algorithm')
         if (dyninp%apmethode.eq.3) write (outfile,3502)
3502  format (5x,'Partitioning:  Hotspot algorithm')
         if (dyninp%apmethode.eq.4) write (outfile,3503)
3503  format (5x,'Partitioning:  ONIOM-XS algorithm')
         if (dyninp%apmethode.eq.5) write (outfile,3504)
3504  format (5x,'Partitioning:  Difference-Based Adaptive Solvation')
         if (dyninp%apmethode.eq.6) write (outfile,3508)
3508  format (5x,'Exchange of QM and MM groups ',
     *              'without smoothing/ buffer')
         if (dyninp%apmethode.gt.0) then
            if (dyninp%activecenter.gt.0) then 
                write (outfile,3505) dyninp%activecenter
            endif
            write (outfile,3506) dyninp%activeradius
            write (outfile,3507) dyninp%partbuffer
3505  format (5x,'Active center (atom index):', t50,I8)
3506  format (5x,'Radius active center [Angstrom]:', t53, f10.4)
3507  format (5x,'Radius buffer zone [Angstrom]:', t55, f10.6)
            if (dyninp%groupcom.eq.0) then
                write (outfile,3509) "Designated atom"
            elseif (dyninp%groupcom.eq.1) then
                write (outfile,3509) "Center of mass"
            endif
3509  format (5x,'Distance calculation based on:', t45, a)
          endif

         write (outfile,3011)
3011  format (/,80('-'),//)     
      endif


3010  format (5x,a45,a20)

c
c End add Jun. 8, 2010 by Soroosh Pezeshki
c


      return
      end

c
c
c***********************************************************************
c  DATTIM
c***********************************************************************
c
      subroutine dattim(io)
c
c     version for Silicon Graphics computers
c     write date and time to device io
c     
c     Called by: insumry
c     Calls:     date
c
      implicit none
      integer io
      character*9 today

c      call date(today)
C      write(io,1100) today

      return

1100  format(1x,a9)

      end
c
c*****************************************************************************
c  LOWDISP
c*****************************************************************************
c
      subroutine lowdisp ( natoms )
c
c     Subroutine lowdisp
c
c     Created:  August 4, 1999 by J. M. Rodgers
c     Revised:
c     Purpose:  Display low level hessian collected for ML optimization
c
c     Called by:   opthhk
c     Calls:  mtoltm
c
c     Variables defined here:
c
      use caltyp
      use param 
      use files, only : outfile
      use optimize, only : hess
      implicit none
      
      double precision, dimension(:), allocatable :: hdisp
      integer n3tm, natoms
      
      n3tm = 3*natoms
      
      allocate( hdisp(n3tm*(n3tm+1)/2) )
      
      write(outfile,1030)
1030  format(//,80('-'),/80('-')//3x,
     *   'Low Level Hessian Requested by User:',//80('-')/)
     
1055  format(t20,d15.8,t40,d15.8,t60,d15.8)

      write(outfile, 1065)
      call mtoltm(hess(:n3tm,:n3tm),hdisp,n3tm)
      write(outfile, 1055) hdisp

1065  format(/5x,'Cartesian Force Constants (lower triang.):')
       
      return
      end
c
c*****************************************************************************
c  TESTDISP
c*****************************************************************************
c
      subroutine testdisp ( natoms, type )
c
c     Subroutine testdisp
c
c     Created:  Feb. 21, 2005 by Hai Lin
c     Revised:
c     Purpose:  Display all energies, gradients, and hessians collected
c               for test in the ML optimization routine.
c
c     Called by:   optehk, optghk, opthhk
c     Calls:  mtoltm

      use caltyp
      use param
      use files,    only : outfile
      use optimize, only : ener, grad, hess
      implicit none
c
c i/o variables
c
      integer  natoms, type
c
c     local variables
c
      integer n3tm 
      double precision, dimension(:), allocatable :: hdisp 
c

      n3tm = 3*natoms

      allocate( hdisp(n3tm*(n3tm+1)/2) )

      write(outfile,1030)
1030  format(//,80('-'),/80('-')//3x,
     *   'Test QM or MM Calculation Requested by User:',//80('-')/)
      
1055  format(t20,d15.8,t40,d15.8,t60,d15.8)
      
      write(outfile,1060) ener
1060  format(/5x,'Energy =',t30,f19.12)

c      if (type.eq.tgrad .or. type.eq.thess) then
      if (type.eq.tgrad) then
         write(outfile,1062)
         write(outfile,1055) grad(:n3tm)
      endif
1062  format(/5x,'Cartesian Gradient:')
      
      if (type.eq.thess) then
         write(outfile, 1065)
         call mtoltm(hess(:n3tm,:n3tm),hdisp,n3tm)
         write(outfile, 1055) hdisp
      endif
1065  format(/5x,'Cartesian Force Constants (lower triang.):')

      return
      end
c
c*****************************************************************************
c  SUMDISP
c*****************************************************************************
c
c   modified 2/10/2007 by Christoph Riplinger
c   for the partial hessian we need an extra variable: hn3tm
c
c      subroutine sumdisp(geom, n3tm, type, ltest, ltestmm, 
c     * lqmmm, qmmminp)
      subroutine sumdisp(geom, n3tm, hn3tm, type, ltest, ltestmm, 
     * lqmmm, qmmminp)
c
c   end 2/10/2007 by Christoph 
c
c     Created by : Feb. 21, 2005 by Hai Lin
c     Revised by : Aug. 23, 2006 by Yan Zhang
c
c     Purpose    : generate a summary output file providing information to
c                  interface program QMMMRATE
c     called by  :
c                  main
c

c
      use param
      use caltyp
      use input,    only : cartgeom, qmmminf
      use files,    only : sumfile
      use optimize, only : ener, grad, hess
      use energy
      use gradient
      use hessian

      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom
      type (qmmminf)  :: qmmminp
c
      integer ltest, ltestmm, n3tm, type, lqmmm
c
c   add 2/10/2007 by Christoph Riplinger
c
      integer hn3tm
c
c   end 2/10/2007 by Christoph 
c
c local variables
c
      integer           i, id
c      double precision  hdisp(n3tm*(n3tm+1)/2)
      double precision, dimension(:), allocatable :: hdisp 

c            
c --- NATOMS
      write(sumfile,1210) geom%natoms
1210  format ('Number of Atoms', I3)


c --- CHARGE and MULTIPLICITY
      write(sumfile,1310) geom%charge
      write(sumfile,1360) geom%mult
1310  format('Charge',I3)
1360  format('Multiplicity',I3)



c --- GEOMETRY
      write(sumfile, 1410)
      write(sumfile, 1450)

      do i = 1, geom%natoms
          write(sumfile,1500)  geom%atm(i), geom%aucoord(3*i-2),
     *                         geom%aucoord(3*i-1), geom%aucoord(3*i)

      end do

      write(sumfile, 1550)
      do i = 1, geom%natoms
          write(sumfile,1500)  geom%atm(i), geom%angcoord(3*i-2),
     *                         geom%angcoord(3*i-1), geom%angcoord(3*i)

      end do


1410  format(/, 'Geometry')
1450  format(/2x,'Atomic Coordinates (in a.u.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)
1500  format(9x,a2,t20,e15.8,t40,e15.8,t60,e15.8)
1550  format(/2x,'Atomic Coordinates (in ang.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)



c --- ENERGY, GRADIENT, HESSIAN



c
c TEST / TESTMM
c
      if ((ltest .eq. 1) .or. (ltestmm .eq. 1)) then

          write(sumfile, 1060) ener
          if (type .eq. tgrad .or. type .eq. thess) then
              write(sumfile, 1062)
              write(sumfile, 1055) grad(:n3tm)
          end if
          if (type .eq. thess) then
c
c    used the allocate defination to avoid the variable too large
c         add 8/23 by Yan Zhang
c
c
c   modified 2/10/2007 by Christoph Riplinger
c
c              allocate(hdisp(n3tm*(n3tm+1)/2))
c              write(sumfile, 1065)
c              call mtoltm(hess(:n3tm, :n3tm), hdisp, n3tm)
              allocate(hdisp(hn3tm*(hn3tm+1)/2))
              write(sumfile, 1065)
              call mtoltm(hess(:hn3tm, :hn3tm), hdisp, hn3tm)
c
c   end 2/10/2007 by Christoph Riplinger
c
              write(sumfile, 1055) hdisp
          end if
c
c        end 8/23 by Yan Zhang
c
c QMMM                                                                  
c
      else if (lqmmm .eq. 1) then
          write(sumfile, 1060) qmmmener
          if (type .eq. tgrad .or. type .eq. thess) then
              write(sumfile, 1062)
              write(sumfile, 1055) qmmmgrad(:n3tm)
          end if
          if (type .eq. thess) then
c
c   modified 2/10/2007 by Christoph Riplinger
c
c              allocate(hdisp(n3tm*(n3tm+1)/2))
c              write(sumfile, 1065)
c              call mtoltm(qmmmhess(:n3tm, :n3tm), hdisp, n3tm)
              allocate(hdisp(hn3tm*(hn3tm+1)/2))
              write(sumfile, 1065)
              call mtoltm(qmmmhess(:hn3tm, :hn3tm), hdisp, hn3tm)
c
c   end 2/10/2007 by Christoph Riplinger
c
              write(sumfile, 1055) hdisp
          end if
	  	      	      	      	  	  	      	      	  	        
c
c Nothing to do
c 
      else
         write(sumfile, 1070)
      end if

      write(sumfile, 1070)

1055  format(t20,e15.8,t40,e15.8,t60,e15.8)
1060  format(/,'Total energy',f19.12)
1062  format(/,'Cartesian Gradients')
1065  format(/, 'Cartesian Force Constants')
1070  format(//)


      return
      end

