c
c******************************************************************************
c  POLAREECH
c******************************************************************************
c
      subroutine polareech(qmprognme, mmprognme, geom, mmgeom, 
     &                     ssgeom, ssmmgeom, qmmminp)
c
c     Subroutine polareech
c
c     Created:  Aug. 7, 2006 by Yan Zhang
c     Revised: 
C               
c     Purpose:  Iteratively to calculate the polarized charges with the QEQ-RG,
c               EQE-bt or EEM method in the polarized calculation. The M1 charge 
c               is as external potential in two manner.
c               um1---M1 as the external potential.
c               uq0---the redistributed charges q0 as the external potential.
c
c
c     Called by: qmmmehk, qmmmghk, qmmmhhk 
c     Calls:   eecharge, qmmmssgeom, bgchscale, bgchshift, qmmmbgchgeom
c  
      use caltyp
      use input,  only : cartgeom, mmvalen, qmmminf, maxpolcycl
      use files
      use energy, only : qmmmener, qmes, mmes
      implicit none
c
c i/o variables
c
      character*10       qmprognme, mmprognme
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (qmmminf)  :: qmmminp
c
c local variables
c      
      type (cartgeom) :: ssgeom, bgchgeom
      type (mmvalen)  :: ssmmgeom
      integer            lsnatm, ssnatm, bgchnum
      integer            ncycl, npcycl, ngroup, nchnum, i,ierr 
      double precision   rmsq, maxq, pmaxq, prmsq
      double precision   mmesb, ener

      character*20       extmpot
c
c    none return
c
      if(qmmminp%polmeth.eq.'none') return
c
c   Only g03 was used in the polarized calculations.
c
      if((qmprognme.ne.'g03').and.(qmprognme.ne.'orca')) then
         write(outfile,40)qmprognme
         stop 'Error: QM method in the calculations.'
      end if
40    format(/,'Only the program g03 is permited ',
     $  'in the polarized calculations.',2x,
     $ 'Error QM program: ', a7)
c
c    prepare values
c
      pmaxq  = qmmminp%polmaxdq
      prmsq  = qmmminp%polrmsdq
      npcycl = qmmminp%polcycl
      ngroup = qmmminp%polgnum
      extmpot= qmmminp%polmpot
      if((extmpot.ne.'um1').and.(extmpot.ne.'uq0'))
     &  stop 'Error keyword mpot'
      if(npcycl.gt.maxpolcycl) goto 6000
      nchnum=0
      do i=1,ngroup
         nchnum=nchnum+qmmminp%polgidnum(i)
      end do
      rmsq = 1.d0
      maxq = 1.d0
      ncycl= 0

c
c    Charge calculation cycle
c
      write(outfile,90)
      write(outfile,80)

80    format(//3x,'Charge polarized calculation.',//)
90    format(80('-'))

      open(unit=chargfile,file=chargoutf,status='unknown',err=5000)

100   format(80('='))
110   format(/2x,'Polarized charge calculation cycle: '
     *       ,2x,i5,2x/)
120   format(/2x,'Tolerance: maxdq= ',f12.9,2x,'rmsdq= ',f12.9,/)

      do while(((rmsq.gt.prmsq).or.(maxq.gt.pmaxq)).and.
     &         (ncycl.le.npcycl))
     
         ncycl=ncycl+1
         write(chargfile,100) 
         write(chargfile,110)	ncycl
         write(chargfile,120)pmaxq,prmsq

         call eecharge(qmprognme,mmprognme, geom,mmgeom,ssgeom,ssmmgeom,
     *            qmmminp, maxq, rmsq, ener)

         rmsq=dsqrt(rmsq/nchnum)
         write(chargfile,130)ncycl,maxq,rmsq

130   format(/2x,'charge calculation cycle: ',i4,2x,'maxdq=',f12.9,
     &        2x,'rmsdq=',f12.9,/)
c                     
c  Assign the appropriate geometries to ssgeom
c                     
         if(extmpot.eq.'um1') then
            call qmmmssgeom(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)           
c
c  Work on background charges
c
            call bgchscale(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
      
            if ((qmmminp%bordercharge .eq. 'scale')) goto 3000
      
c
c  Shift/redistribute M1 charges
c     
            call bgchshift(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
c
c  Embed the primary system with backgound charges
c
3000        call qmmmbgchgeom(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
         end if

      end do

      if((rmsq.lt.prmsq).and.(maxq.lt.pmaxq)) then
         write(outfile,140)
         write(chargfile,140)
140   format(//3x,'Charge polarized EE calculation succeed.',//)
      else
         write(outfile,150)
         write(outfile,100)
          stop
150   format(/3x,'Charge is not convergence. ')
      end if

      close(chargfile)
      return
5000  write(outfile,*)'Error: can not open the ml.charg.'
      stop 'polareech 1'
6000  write(outfile,*)'Error: cycle exceeds the maximum allowed value.'
      stop 'polareech 2'

      end

c
c******************************************************************************
c  EECHARGE
c******************************************************************************
c
      subroutine eecharge(qmprog,mmprognme, geom,mmgeom,ssgeom,ssmmgeom,
     *            qmmminp, maxq, rmsq, ener)
c
c     Subroutine eechcharge
c
c     Created:  July 19, 2006 by Yan Zhang
c     Revised:  Mar  29, 2007 by Yan Zhang
c
c     Purpose:  To calculate the atomic charge in a molecule with
c               EEM and QEQ method.
c
c     Called by: polareech 
c     Calls:	chargcoor, eeapc, chreset, progespp
c
      use input
      use atminfo
      use files 
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, mmgeom1, ssmmgeom
      type (qmmminf)  :: qmmminp
      integer ngroup
      double precision maxq, rmsq
c
c local variables
c      
      integer  ngidn,gidn,gidn1,gnum
      integer  ind,tdebug,ierr 
      integer  ig,i,k,j,ii,jj,kk      
      integer  ssnatm, natmg

      double precision    qtot, angtoau,potg(maxpolgidnum)
      double precision    qn(maxatm), x(maxatm), y(maxatm), z(maxatm)
      double precision    pot(maxatm), ener,mmesb
      character*2  eeatm(maxatm)
      character*10 qmprog, mmprognme
c
c
      angtoau = 0.529177249d0
      tdebug=0
c
c  prepare group numbers
c  
      ngroup = qmmminp%polgnum
c
c   added by Yan Zhang 3/29/2007
c

      ssnatm  = mmgeom%natoms

c
c   end by Yan zhang 3/29/2007
c

      do i=1,maxatm
         pot(i)=0.d0
      end do

      rmsq = 0.d0
      maxq = 0.d0
c
c     none return 
c
      if(qmmminp%polmeth.eq.'none') return
      write(chargfile,200) qmmminp%polmeth,qmmminp%polmpot
c
c  check atom in the group
c
      gnum=ngroup
      do i=1,gnum
        gidn = qmmminp%polgidnum(i)
        do j=1,gidn
          k = qmmminp%polgid(i,j)

          do ii=1, qmmminp%qmnatm
            jj = qmmminp%qmatm(ii)
             if(k.eq.jj) goto 9100
          end do
            
          do ii=1, qmmminp%mm1stlay%natm1stlay
            jj = qmmminp%mm1stlay%atm1stlay(ii)
            if(k.eq.jj) goto 9100
          end do

          ind=0
          do ii=1,gnum
            gidn1 = qmmminp%polgidnum(ii)
            do jj=1,gidn1
              kk = qmmminp%polgid(ii,jj)
              if(k.eq.kk) ind=ind+1
            end do
          enddo
          if(ind.gt.1) goto 9100
        end do
      enddo		 
c
c   Calculate the potential of the polarized chaege with the G03 program
c
      call progespp(qmprog, geom, mmgeom,
     *            ssgeom, ssmmgeom,qmmminp, pot, ener)
c
c  Calculate the electrosatic energy within the background charge
c
      if(tdebug.eq.1)	then
      
         write(chargfile,425)
425   format(/2x,'Electricstatic interactions within ',
     *  'the background charges:',/)
     
         call progmmstehk(mmprognme, qmmminp%mmmeth, qmmminp%mmbas, 
     *  ssgeom, ssmmgeom, qmmminp%mmoptl, mmesb, ierr, qmmminp)

         write(chargfile,435) mmesb
435   format(/2x,'Additional MM energy (a.u.):',t50,f20.10,/2x,
     *  '(due to electricstatic interactions within',/2x,
     *  ' background point charges)')
         write(chargfile,445) ener-mmesb

445   format(/2x,'The sum of QM CPS energy and electricstatic',
     *  /4x,' interactions between CPS and background point ',
     *    /4x,'charges (a.u.):',2x,f20.10,/2x,
     *  '(excluded electricstatic interactions within',/2x,
     *  ' background point charges)')
      endif
c
c   Calculate the charge with EQE or EEM
c
c
c   added by Yan Zhang 3/29/2007
c

      mmgeom1%atmcharge(:ssnatm) = mmgeom%atmcharge(:ssnatm) 

c
c   end by Yan zhang 3/29/2007
c
      do ig=1,ngroup
         ngidn=qmmminp%polgidnum(ig)
c
c      Assign the charge geometries  of the EEM and QEQ method
c      

         call chargcoor(geom, mmgeom, ssgeom, ssmmgeom, qmmminp,
     *            ngroup, eeatm, x, y, z, qtot,ig)
c
c     Calculate the atomic partial charge in molecular with the QEQ
c        and EEM method.
c
         write(chargfile,600)ig
         do i=1,ngidn
            k=qmmminp%polgid(ig,i)
            potg(i)=pot(k)
            write(chargfile,500)k,x(i)*angtoau,
     &        y(i)*angtoau,z(i)*angtoau,potg(i)
         end do
         write(chargfile,700)

         call eeapc(geom, mmgeom, ssgeom, ssmmgeom,qmmminp, 
     &             ngidn,eeatm,x,y,z,qtot,
     *             qmmminp%polmeth,qmmminp%polparal,ig,potg,qn)
c
c     Reset the M2, M3 and redistributed charge after
c     EEM and QEQ calculation.
c
c
c    revised by Yan Zhang 3/29/2007
c
c        call chreset(geom, mmgeom, ssgeom, ssmmgeom, qmmminp, ngroup,
c    *            qn, x, y, z,eeatm,ig, maxq, rmsq)
         call chreset(geom, mmgeom, mmgeom1, ssgeom, ssmmgeom, 
     &            qmmminp, ngroup,
     *            qn, x, y, z,eeatm,ig, maxq, rmsq)
c
c  end by Yan Zhang 3/29/2007
c
      end do
c
c   added by Yan Zhang 3/29/2007
c
      mmgeom%atmcharge(:ssnatm) = mmgeom1%atmcharge(:ssnatm) 
c
c   end by Yan zhang 3/29/2007
c

100   format(///2x,'Now calculate the M1, M2 and M3 charges',
     *   'with the EE method. ',/)
200   format(/2x,'The EE keyword: ',2x,a20,2x,a20,/)
300   format(/2x,70('*'),2x) 
400   format(2x,70('*'),2x,/)
500   format(2x,'Atomic id: ',i5,3x,'XYZ: ',3(f15.9,2x),2x,
     $       'potential: ',f12.9,2x) 
600   format(/,' Group ',i3 )
700   format(//)
      return     
9100  write(outfile,*)'Error: unresonable ID in the group of polar'
      stop

      end	
c
c******************************************************************************
c  CHARGCOOR
c******************************************************************************
c
      subroutine chargcoor(geom, mmgeom, ssgeom, ssmmgeom, qmmminp,
     *            ngroup, eeatm, x, y, z, qtot,idg)
c
c     Subroutine chargcoor
c
c     Created:  July 18, 2006 by Yan Zhang
c     Revised:  
c     Purpose:  To assign appropriate geometries to calculate the charge with
c               EEM and QEQ method.
c
c     Called by: eecharge  
c     Calls:
c
      use input
      use atminfo
      use files, only : outfile, chargfile
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, ssmmgeom
      type (qmmminf)  :: qmmminp

      integer :: ngroup, idg
      double precision :: x(maxatm), y(maxatm), z(maxatm)
      double precision    qtot
      character*2 eeatm(maxatm)
c
c local variables
c      
      integer  capnatm, qmnatm, mmnatm
      integer  ssnatm, cssnatm, lsnatm      
      integer  m1st, m2nd, m3rd 
      integer  i, j, k, ii,jj,kk,ngidn

      double precision   angtoau, tqm1, tq0

      character*20 extmpot
c     
c
      angtoau = 0.529177249d0
c
c  prepare atom numbers
c  
      ngidn    = qmmminp%polgidnum(idg)
      extmpot  = qmmminp%polmpot
      capnatm  = qmmminp%capatm%capnatm
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      cssnatm  = capnatm + qmnatm 
      ssnatm   = ssmmgeom%natoms
      lsnatm   = cssnatm + mmnatm

      m1st = qmmminp%mm1stlay%natm1stlay
      m2nd = qmmminp%mm2ndlay%natm2ndlay     
      m3rd = qmmminp%mm3rdlay%natm3rdlay

c
c   Add the group atoms 
c
      do i=1,ngidn
         k = qmmminp%polgid(idg,i)
         eeatm(i)= geom%atm(k)
         x(i) = geom%angcoord(3*k-2)
         y(i) = geom%angcoord(3*k-1)
         z(i) = geom%angcoord(3*k)
      enddo    
c
c    unit converse
c
      do i=1,ngidn
        x(i)=x(i)/angtoau
        y(i)=y(i)/angtoau
        z(i)=z(i)/angtoau
      end do
c     
c  To calculate the total charge of the group.
c
      qtot=0.d0
      tqm1=0.d0
      tq0 =0.d0
      do i=1,ngidn
         k = qmmminp%polgid(idg,i)
         tqm1 = tqm1 + mmgeom%atmcharge(k)

         do ii=1,mmnatm
            jj = ii + cssnatm
            kk = qmmminp%mmatm(ii)
            if(k.eq.kk) tq0= tq0 + ssmmgeom%atmcharge(jj)
         end do

      enddo
      if(extmpot.eq.'um1') then
         qtot=tqm1
      elseif(extmpot.eq.'uq0') then
         qtot=tq0
      endif
      write(chargfile,*)'EE QTOT = ',qtot
c
      return     
      end

c
c******************************************************************************
c  CHRESET
c******************************************************************************
c
      subroutine chreset(geom,mmgeom, mmgeom1, ssgeom, ssmmgeom, 
     &            qmmminp, ngroup,
     *            qn, x, y, z, eeatm,idg, maxq, rmsq)

c
c     Subroutine chreset
c
c     Created:  July 19, 2006 by Yan Zhang
c     Revised:  Mar  29, 2007 by Yan Zhang
c
c
c     Purpose:  Reset the M2, M3 and redistributed charge after
c               EEM and QEQ calculation.
c
c     Called by:  eecharge
c     Calls:
c
      use input
      use atminfo
      use files, only : outfile, chargfile
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, mmgeom1, ssmmgeom
      type (qmmminf)  :: qmmminp

      integer ngroup, idg
      double precision    maxq,rmsq
      double precision    x(maxatm), y(maxatm), z(maxatm)
      double precision    qn(maxatm)
      character*2  eeatm(maxatm)
c
c local variables
c      
      integer  capnatm, qmnatm, mmnatm
      integer  ssnatm, cssnatm, lsnatm      
      integer  m1st, m2nd, m3rd 
      integer  i, j, k, ii,jj,kk,ngidn
      
      double precision    angtoau,dq 

      character*20 extmpot
c
c
      angtoau = 0.529177249d0
c
c  prepare atom numbers
c  
      ngidn    = qmmminp%polgidnum(idg)
      capnatm  = qmmminp%capatm%capnatm
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      extmpot  = qmmminp%polmpot
      cssnatm  = capnatm + qmnatm 
      ssnatm   = ssmmgeom%natoms
      lsnatm   = cssnatm + mmnatm

      m1st = qmmminp%mm1stlay%natm1stlay
      m2nd = qmmminp%mm2ndlay%natm2ndlay     
      m3rd = qmmminp%mm3rdlay%natm3rdlay

      do i=1,ngidn
      x(i)=x(i)*angtoau
      y(i)=y(i)*angtoau
      z(i)=z(i)*angtoau
      end do
c
c    charge
c
      
      do i=1, ngidn
        k = qmmminp%polgid(idg,i)
c
c  return charge value
c
        if(extmpot.eq.'um1') then
c
c  maxdq and rmsdq
c
            dq=dabs(mmgeom%atmcharge(k)-qn(i))
            if(dq.gt.maxq) maxq=dq
            rmsq=dq*dq+rmsq

c            mmgeom%atmcharge(k) = qn(i)
c    modify by Yan Zhang 03/29/2007
c
            mmgeom1%atmcharge(k) = qn(i)
c
c     end by Yan Zhang 03/29/2007		  	       	       
		  	       	       
         elseif(extmpot.eq.'uq0') then
           do ii=1,mmnatm
            jj = ii + cssnatm
            kk = qmmminp%mmatm(ii)
            if(k.eq.kk) then
              dq=dabs(ssmmgeom%atmcharge(jj)-qn(i))
              if(dq.gt.maxq) maxq=dq
              rmsq=dq*dq+rmsq
              ssmmgeom%atmcharge(jj)= qn(i)
c
c    added Yan Zhang 03/29/2007
c
              mmgeom1%atmcharge(k) = qn(i)
c
c     end by Yan Zhang 03/29/2007		  	       	       
c
             end if
           end do
        end if

         write(chargfile,201)idg,k,eeatm(i),qn(i),x(i),y(i),z(i) 
      end do
201   format(2x, 'group: ',i3,2x,'Atom:',1x, i6,2x,a2, 2x,
     &       'charge: ',1x,f8.4,2x,'XYZ: ',1x,3(f12.8,2x))
      return     
      end

c
c*****************************************************************************
c  EEAPC
c*****************************************************************************
c
      subroutine eeapc(geom, mmgeom, ssgeom, ssmmgeom, qmmminp, 
     &             napc,name,x,y,z,qtot,
     *             methd,paral,idg,pot,qn)
c
c     Subroutine eeapc
c     
c     Created:  July 10, 2006 by Yan Zhang
c
c     Purpose:  Calculate the atomic partial point charge in molecular with the QEQ
c        and EEM method.
c 
c     Called by: eecharge 
c     Calls:	  wordstr, scase, qeqchk, qeqchg, eemch
c
      use input
      use files
      use caltyp                                                        
      use atminfo,  only : asymb
      use input
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, ssmmgeom
      type (qmmminf)  :: qmmminp

      integer 		   napc, idg
      character*2        name(napc)
c
      character*80       paral(103)
      character*20       methd
      double precision    qtot, pot(napc)
      double precision   x(napc), y(napc), z(napc), qn(napc)

c
c local variables
c      
      integer  capnatm, qmnatm, mmnatm, bgchnatm
      integer  ssnatm, cssnatm, lsnatm      
      integer  i,j,k, ii,jj,kk
      integer  m1st, m2nd, m3rd 

      integer 		   ierr
      integer 		   iparal, nparalw, npara,ifindatm
      integer            lparalw(40)

      double precision   cqtot, cfloat
      double precision   eex0(103), eeh0(103)
	      
      character*2        asymp(103)
      character*20       extmpot
c
      character*80       string
c
      character*80       paralw(40)
c
c
      integer            gnum, gidn
      integer            ind, gidn1

c
c
      integer            im2, im3, nuchatm
      double precision   xu(maxatm), yu(maxatm), zu(maxatm)
      double precision   uchq(maxatm)
      character*2        uchatm(maxatm)
c
      double precision   angtoau
c
c
      angtoau = 0.529177249d0
c
c
      capnatm  = qmmminp%capatm%capnatm
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      cssnatm  = capnatm + qmnatm 
      ssnatm   = ssmmgeom%natoms
      lsnatm   = cssnatm + mmnatm

      m1st = qmmminp%mm1stlay%natm1stlay
      m2nd = qmmminp%mm2ndlay%natm2ndlay     
      m3rd = qmmminp%mm3rdlay%natm3rdlay
      gnum =  qmmminp%polgnum
      extmpot = qmmminp%polmpot
c
c   Assign the charge unchange MM atom 
c
      nuchatm=0
      do i=1,mmnatm
         k=qmmminp%mmatm(i)

         ind=0
         gidn=qmmminp%polgidnum(idg)
         do j=1,gidn
            kk = qmmminp%polgid(idg,j)
            if(k.eq.kk) ind=1
         end do

         if(ind.eq.0) then
            nuchatm=nuchatm+1
            uchatm(nuchatm) = geom%atm(k)
            xu(nuchatm)     = geom%angcoord(3*k-2)/angtoau
            yu(nuchatm)     = geom%angcoord(3*k-1)/angtoau
            zu(nuchatm)     = geom%angcoord(3*k)/angtoau
            uchq(nuchatm)   = mmgeom%atmcharge(k)
            write(chargfile,*)'unchange charge MM: ',nuchatm,' ID:  ',k, 
     &   uchq(nuchatm)
         end if
      end do

      if(extmpot.eq.'uq0') then
         do i=lsnatm+1,ssnatm
            k = ssmmgeom%atmid(i)
            ii = qmmminp%chredist%m1(i-lsnatm)
            nuchatm=nuchatm+1
            uchatm(nuchatm) = geom%atm(ii)
            xu(nuchatm)     = ssgeom%angcoord(3*k-2)/angtoau
            yu(nuchatm)     = ssgeom%angcoord(3*k-1)/angtoau
            zu(nuchatm)     = ssgeom%angcoord(3*k)/angtoau
            uchq(nuchatm)   = ssmmgeom%atmcharge(k)
            write(chargfile,*)'unchange charge Q0: ',nuchatm, 
     &             uchq(nuchatm)
         end do
      elseif(extmpot.eq.'um1') then
         do i=1,m1st
            k = qmmminp%mm1stlay%atm1stlay(i)
            nuchatm=nuchatm+1
            uchatm(nuchatm) = geom%atm(k)
            xu(nuchatm)     = geom%angcoord(3*k-2)/angtoau
            yu(nuchatm)     = geom%angcoord(3*k-1)/angtoau
            zu(nuchatm)     = geom%angcoord(3*k)/angtoau
            uchq(nuchatm)   = mmgeom%atmcharge(k)
            write(chargfile,*)'unchange charge M1: ',nuchatm, 
     &                     uchq(nuchatm)
         end do
      else
         write(outfile,*) 'Error keyword MPOT'
         stop
      end if
2000  format(/2x,'Error:QM atom id in the group. ',i5,2x,i5)
c
c     parameters input in EEAPC section 
c
      iparal=0
      i=1
      do while (paral(i).ne.'   ' .and. i.le.103)

        string=paral(i)
c
        call wordstr(string,80,paralw,nparalw,lparalw,ierr)
        if(nparalw.ne.3)  goto 9000

        asymp(i)=paralw(1)(1:2)          
        call scase(2,asymp(i),asymp(i))
        ifindatm=0
        do j=1,103
           if(asymp(i).eq.asymb(j)) ifindatm=1
        end do
        if(ifindatm.eq.0) goto 9000

        eex0(i)=cfloat(paralw(2))
        eeh0(i)=cfloat(paralw(3))

        i = i + 1
      enddo

      if(i.gt.1) iparal=1 
      if(iparal.eq.1) npara=i-1
c
c   calcalate the atomic charge
c
      if(methd.eq.'qeqbt') then
        call qeqchk(napc,qmmminp, name,x,y,z,qtot,pot,
     *                  nuchatm, uchatm, xu, yu, zu, uchq,
     &                  iparal,npara,asymp,eex0,eeh0,qn)
      else if(methd.eq.'qeqrg') then
        call qeqchg(napc,qmmminp,name,x,y,z,qtot,pot,
     *                  nuchatm, uchatm, xu, yu, zu, uchq,
     &                  iparal,npara,asymp,eex0,eeh0,qn)
      else if(methd.eq.'eem') then
        call eemch(napc,qmmminp,name,x,y,z,qtot,pot,
     *                  nuchatm, uchatm, xu, yu, zu, uchq,
     &                  iparal,npara,asymp,eex0,eeh0,qn)
      else
        write(outfile,*)'Error the method for the calculation'
      stop
      end if


      cqtot=0.d0
      do i=1,napc
        cqtot=cqtot+qn(i)
      end do
      if((cqtot-qtot).gt.1.d-9) goto 7000 

c      write(outfile,1000)
c      write(outfile,1001)
c      write(outfile,1002)
c      do i=1,napc
c        write(outfile,100)name(i),qn(i),pot(i)
c      end do

1000  format(/2x,70('*'))
1001  format(2x,'The calculations of the atomic charges in the',
     &           ' molecular succeed')
1002  format(2x,70('*'),/)

100   format('Atom',3x,a2,4x,f15.9,3x,f15.9)
      return

7000  write(6,*)'Error calculating the atomic charge.'
      stop
9000  write(outfile,*)'Error reading the parameters.'
      stop

9100  write(outfile,*)'Error: unresonable ID in the group of polar'
      stop
      end

c
c***********************************************************************
c     QEQCHK
c***********************************************************************
C
      subroutine qeqchk(n, qmmminp, name, x, y, z, qtot,pot, 
     *                  nuchatm, uchatm, xu, yu, zu, uchq,
     &                  ipa,npara,asymp,eex0,eeh0,qn)

c     subroutine qeqchk
c
c     Created:  Jun 19, 2006 by Yan Zhang
c          
c     Revised:
c
c     This subroutine structures the QEQ matrix and solve the atomic charge 
C     with the equation CQ=D. 
c
c     Called by: eeapc
c     Calls:	scase, dgesv
c
      use files
      use input
      use ehard, only: qeqat, qeqet, qeqht
      implicit none
c
      type (qmmminf)  :: qmmminp
c
      integer :: n, i, j, ipa, npara
      integer :: ifind, info
      integer :: ipiv(n+1)

      character*2 name(n),lett, asymp(103)

      double precision :: eex0(103), eeh0(103)
      double precision :: x(n), y(n), z(n)
      double precision :: qn(n), j12(n,n),xn0(n)
      double precision :: c(n+1,n+1),d(n+1),pot(n)
      double precision :: qtot, angtoau, ev
      double precision :: r2,coul
c
      integer    nuchatm, nrelay
      character*2 uchatm(nuchatm)
      double precision :: xu(nuchatm), yu(nuchatm), zu(nuchatm)
      double precision :: juch(nuchatm), exte(n), uchq(nuchatm)

c
c transformation between angstroms and bohr (au)
c
c      angtoau = 0.52918d0
      angtoau = 0.529177249d0
      ev=27.21066      
c
c     Read x0(i) and J0(i,i) 
c
c   parameters in J. comput. chem. 1995 17 87-108 
c
      if(ipa.eq.0) then

      do i=1,n
        call scase(2,name(i),lett)
        ifind=0
        do j=1,4
          if(lett.eq.qeqat(j)) ifind=j
        end do
        if(ifind.eq.0) goto 9000
        xn0(i) = qeqet(ifind)/ev
        j12(i,i) = qeqht(ifind)/ev
      end do

c
c    The charge unchange MM atom
c
      do i=1,nuchatm
        call scase(2,uchatm(i),lett)
        ifind=0
        do j=1,4
          if(lett.eq.qeqat(j)) ifind=j
        end do
        if(ifind.eq.0) goto 9000
        juch(i) = qeqht(ifind)/ev
      end do
c
c   parameters read from input file 
c
      else if(ipa.eq.1) then

      do i=1,n
        call scase(2,name(i),lett)
        ifind=0
        do j=1,npara
          if(lett.eq.asymp(j)) ifind=j
        end do
        if(ifind.eq.0) goto 9000
        xn0(i) = eex0(ifind)/ev
        j12(i,i) = eeh0(ifind)/ev
      end do
c
c    The charge unchange MM atom
c
      do i=1,nuchatm
        call scase(2,uchatm(i),lett)
        ifind=0
        do j=1,npara
          if(lett.eq.asymp(j)) ifind=j
        end do
        if(ifind.eq.0) goto 9000
        juch(i) = eeh0(ifind)/ev
      end do

      else
        goto 9000
      end if


      do i=1,n
        d(i)=-xn0(i)-pot(i)
      end do

c
c     calculate the Jij term with the Ohno-Klopman empirical equations
c	J(i,j)=1/sqrt(R(i,j)**2+(1/(2*J(i,i))+1/(2*J(j,j)))**2)
c     Journal of Molecular Structure 634 ()2003 159-170
c     structure the C matrix
c
      do i=1,n

        do j=i+1,n
          r2=(x(i)-x(j))**2+(y(i)-y(j))**2+(z(i)-z(j))**2
          coul= (1.d0/(2*j12(i,i))+1.d0/(2*j12(j,j)))**2
          j12(i,j)=1.d0/dsqrt(r2+coul)
          j12(j,i)=j12(i,j)
        end do
c
c    The charge unchange MM atom
c
        exte(i)=0.d0
        do j=1,nuchatm
          r2=(x(i)-xu(j))**2+(y(i)-yu(j))**2+(z(i)-zu(j))**2
          coul= (1.d0/(2*j12(i,i))+1.d0/(2*juch(j)))**2
          exte(i)=exte(i)+uchq(j)/dsqrt(r2+coul)
        end do

      end do

c
      do i=1,n
        d(i)=d(i)-exte(i)
c        write(*,20)i,xn0(i),pot(i),d(i),exte(i)
      end do
20    format(i3,2x,4(f12.6,2x))
      d(n+1)=qtot
c
      do i=1,n
        do j=1,n
          c(i,j)=j12(i,j)
c          write(outfile,*)i,j,c(i,j)
        end do
      end do

      do i=1,n
        c(i,n+1)=-1.d0

        c(n+1,i)=1.d0
      end do

      c(n+1,n+1)=0.d0
c
c     solve the atomic charges in the molecular with the iterative method
c
      
      call dgesv( n+1, 1, c, n+1, ipiv, d, n+1, info )

      
      if(info.ne.0) stop 'Error: can not solve the linear equation'
      do i=1,n
        qn(i)=d(i)
        write(chargfile,100)name(i),exte(i),qn(i)
      end do


100   format(2x,'Atom: ',a2,2x,'External EE: ',f12.6,2x,'EE charge: ',
     &       f12.6)

      return
9000  write(outfile,*)'Error: can not find the QEQ parameters'
      stop 
      end
c
c***********************************************************************
c     QEQCHG
c***********************************************************************
C
      subroutine qeqchg(n, qmmminp, name, x, y, z, qtot,pot,
     *                  nuchatm, uchatm, xu, yu, zu, uchq,
     &                  ipa,npara,asymp,eex0,eeh0,qn)

c     subroutine qeqchg
c
c     Created:  Jun 19, 2006 by Yan Zhang
c          
c     Revised:
c
c     This subroutine structures the QEQ matrix and solve the atomic charge 
C     with the equation CQ=D. 
c
c     Called by: eeapc
c     Calls:	scase, dgesv
c
      use input
      use files
      use ehard, only:qeqag, qeqeg,qeqhg
      implicit none
c
c
      type (qmmminf)  :: qmmminp
c
      integer :: n, i, j, ipa, npara
      integer :: ifind, info
      integer :: ipiv(n+1)

      character*2 name(n),lett, asymp(103)

      double precision :: eex0(103), eeh0(103)
      double precision :: x(n), y(n), z(n)
      double precision :: qn(n), j12(n,n),xn0(n)
      double precision :: c(n+1,n+1),d(n+1),pot(n)
      double precision :: qtot, angtoau,ev
      double precision :: r2
c
      integer    nuchatm, nrelay
      character*2 uchatm(nuchatm)
      double precision :: xu(nuchatm), yu(nuchatm), zu(nuchatm)
      double precision :: juch(nuchatm), exte(n), uchq(nuchatm)
      double precision :: coul
c
c transformation between angstroms and bohr (au)
c
c      angtoau = 0.52918d0
      angtoau = 0.529177249d0
      ev=27.21066
c
c     structure the D matrix
c     Read x0(i) and J0(i,i) 
c
      if(ipa.eq.0) then
c
c   parameters in J.Phys.Chem.1991,95,3358-3363
c
      do i=1,n
        call scase(2,name(i),lett)
        ifind=0
        do j=1,16
          if(lett.eq.qeqag(j)) ifind=j
        end do
        if(ifind.eq.0) goto 9000
        xn0(i) = qeqeg(ifind)/ev
        j12(i,i) = qeqhg(ifind)/ev
      end do

c
c   parameters read from input file 
c
      else if(ipa.eq.1) then

      do i=1,n
        call scase(2,name(i),lett)
        ifind=0
        do j=1,npara
          if(lett.eq.asymp(j)) ifind=j
        end do
        if(ifind.eq.0) goto 9000
        xn0(i) = eex0(ifind)/ev
        j12(i,i) = eeh0(ifind)/ev
      end do

      else
        goto 9000
      end if


      do i=1,n
        d(i)=-xn0(i)-pot(i)
      end do

c
c     calculate the Jij term with the equations J(i,j)=14.4/(e*R)
c     (J.Phys.Chem.1991,95,3358-3363)
c     structure the C matrix
c
      do i=1,n
        do j=i+1,n
          r2=(x(i)-x(j))**2+(y(i)-y(j))**2+(z(i)-z(j))**2
          j12(i,j)=14.4d0/(2.d0*dsqrt(r2)*angtoau*ev)
          j12(j,i)=j12(i,j)
        end do

c
c    The charge unchange MM atom
c
        exte(i)=0.d0
        do j=1,nuchatm
          r2=(x(i)-xu(j))**2+(y(i)-yu(j))**2+(z(i)-zu(j))**2
          coul= 14.4d0/(2.d0*dsqrt(r2)*angtoau*ev)
          exte(i)=exte(i)+uchq(j)*coul
        end do

      end do
c
      do i=1,n
        d(i)=d(i)-exte(i)
c        write(*,20)i,xn0(i),pot(i),d(i),exte(i)
      end do
      d(n+1)=qtot
20    format(i3,2x,4(f12.6,2x))

c
      do i=1,n
        do j=1,n
          c(i,j)=j12(i,j)
        end do
      end do

      do i=1,n
        c(i,n+1)=-1.d0
        c(n+1,i)=1.d0
      end do

      c(n+1,n+1)=0.d0

c     solve the atomic charges in the molecular with the iterative method
c
      
      call dgesv( n+1, 1, c, n+1, ipiv, d, n+1, info )

      
      if(info.ne.0) stop 'Error: can not solve the linear equation'
      do i=1,n
        qn(i)=d(i)
        write(chargfile,100)name(i),exte(i),qn(i)
      end do


100   format(2x,'Atom: ',a2,2x,'External EE: ',f12.6,2x,'EE charge: ',
     &      f12.6)
      return
9000  write(outfile,*)'Error: can not find the QEQ parameters'
      stop 
      end

c
c***********************************************************************
c     EEMCH
c***********************************************************************
C
      subroutine eemch(n,qmmminp, name,x,y,z,qtot,pot,
     *                  nuchatm, uchatm, xu, yu, zu, uchq,
     &                  ipa,npara,asymp,eex0,eeh0,qn)

c     subroutine eemch
c
c     Created:  Jun 20, 2006 by Yan Zhang
c          
c     Revised:
c
c     This subroutine structures the EEM matrix and solve the atomic charge. 
C     with the equation CQ=D. 
c
c     Called by: eeapc
c     Calls:	  scase, dgesv
c
      use files
      use input
      use ehard, only:eemam,eemem, eemhm
      implicit none
c
c
c
      type (qmmminf)  :: qmmminp
c
      integer :: n, i, j, ipa, npara
      integer :: ifind, info
      integer :: ipiv(n+1)

      character*2 name(n),lett, asymp(103)

      double precision :: eex0(103), eeh0(103)
      double precision :: x(n), y(n), z(n),pot(n)
      double precision :: qn(n), j12(n,n),xn0(n+1),c(n+1,n+1),d(n+1)
      double precision :: qtot, angtoau,ev
      double precision :: r2
c
      integer    nuchatm, nrelay
      character*2 uchatm(nuchatm)
      double precision :: xu(nuchatm), yu(nuchatm), zu(nuchatm)
      double precision :: juch(nuchatm), exte(n), uchq(nuchatm)

c
c transformation between angstroms and bohr (au)
c
c      angtoau = 0.52918d0
      angtoau = 0.529177249d0
      ev=27.21066d0
c
c     structure the D matrix
c     Read x0(i) and J0(i,i) 
c
c   parameters in J. Phys. Chem. A 108 2004 10359-10366
c
      if(ipa.eq.0) then

      do i=1,n
        call scase(2,name(i),lett)
        ifind=0
        do j=1,4
          if(lett.eq.eemam(j)) ifind=j
        end do
        if(ifind.eq.0) goto 9000
        xn0(i) = eemem(ifind)/ev
        j12(i,i) = 2.d0*eemhm(ifind)/ev
      end do
c
c   parameters read from input file 
c
      else if(ipa.eq.1) then

      do i=1,n
        call scase(2,name(i),lett)
        ifind=0
        do j=1,npara
          if(lett.eq.asymp(j)) ifind=j
        end do
        if(ifind.eq.0) goto 9000
        xn0(i) = eex0(ifind)/ev
        j12(i,i) = 2.d0*eeh0(ifind)/ev
      end do

      else
        goto 9000
      end if

      do i=1,n
        d(i)=-xn0(i)-pot(i)
      end do
c
c     calculate the Jij term. J(i,j)=1/r(i,j)
c     J. Phys. Chem. A 106 2002 7887-7894
c     structure the C matrix
c
      do i=1,n
        do j=i+1,n
          r2=(x(i)-x(j))**2+(y(i)-y(j))**2+(z(i)-z(j))**2        
          j12(i,j)=1.d0/dsqrt(r2)
          j12(j,i)=j12(i,j)
        end do
c
c    The charge unchange MM atom
c
        exte(i)=0.d0
        do j=1,nuchatm
          r2=(x(i)-xu(j))**2+(y(i)-yu(j))**2+(z(i)-zu(j))**2
          exte(i)=exte(i)+uchq(j)/dsqrt(r2)
        end do

      end do
c
      do i=1,n
        d(i)=d(i)-exte(i)
c        write(*,*)i,d(i),exte(i),nuchatm
      end do
      d(n+1)=qtot
c
      do i=1,n
        do j=1,n
          c(i,j)=j12(i,j)
        end do
      end do

      do i=1,n
        c(i,n+1)=-1.d0
        c(n+1,i)=1.d0
      end do

      c(n+1,n+1)=0.d0
c
c     solve the atomic charges in the molecular with the iterative method
c
      call dgesv( n+1, 1, c, n+1, ipiv, d, n+1, info )

      if(info.ne.0) stop 'Error: can not solve the linear equation'
      do i=1,n
        qn(i)=d(i)
        write(chargfile,100)name(i),exte(i),qn(i)
      end do

100   format(2x,'Atom: ',a2,2x,'External EE: ',f12.6,2x,'EE charge: ',
     &      f12.6)

      return
9000  write(6,*)'Error: can not find the EEM parameters'
      stop 
      end
c
c*****************************************************************************
c  PROGESPP
c*****************************************************************************
c
      subroutine progespp(prognme,  geom, mmgeom,
     *            ssgeom, ssmmgeom,qmmminp, pot,ener)
c
c     Subroutine progespp
c
c     Created:  Aug. 3, 2006 by Yan Zhang
c     Revised:  
c
c     Purpose:  Get ESP charges and M1, M2 and M3 potential in the 
c               presence of background charges.
c
c
c     Called by: eecharge
c     Calls:  g03chinp, g03oute, g03outp, g03outesp
c
c     Variables defined here:
c
      use caltyp
      use files
      use input
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, ssmmgeom   
      type (qmmminf)  :: qmmminp

      double precision pot(1)
      character*10  prognme
c
c local variables
c
      
      integer qmnatm, capnatm, cssnatm
      integer      qmcharge, qmmult
      double precision ener
      character*20 ver_lab                                                 
      character*20 esmeth
c
c
c
      qmnatm   = qmmminp%qmnatm
      capnatm  = qmmminp%capatm%capnatm
      cssnatm  = qmnatm + capnatm
      qmcharge = qmmminp%qmcharge
      qmmult   = qmmminp%qmmult
c
c  GAUSSIAN03
c
      if (prognme(:3).eq.'g03') then
         open (unit=g03inf, status='unknown', file=g03in, err=9993)
         call g03chinp(g03inf, qmmminp%qmmeth, qmmminp%qmbas, 
     &         qmcharge, qmmult, ssgeom,  
     &         qmmminp%qmoptl, 1, ver_lab, qmmminp, ssmmgeom)
         close (g03inf, err=9993)
         call system('g03shuttle g03.inp g03.out')
         call g03oute(1,esmeth, ener)                                      
         call g03outop(geom, mmgeom, ssgeom,ssmmgeom, qmmminp, pot)

         write(chargfile,100) ener
         call g03outesp(cssnatm)
c
c  ORCA
c
      else if(prognme(:4).eq.'orca') then
         open (unit=orcainf, status='unknown', file=orcain, err=9994)
         call orcastpotinp(orcainf, qmmminp%qmmeth, qmmminp%qmbas, 
     *      qmcharge, qmmult, ssgeom, 
     *      qmmminp%qmoptl, 1, ver_lab, qmmminp, ssmmgeom)
         close (orcainf, err=9994)
         call system('orcapotshuttle orca.inp orca.out')
         call orcaoute(1,esmeth, ener)                                      
         call orcaoutpot(ssmmgeom, qmmminp, pot)	 
         write(chargfile,100) ener
c
c
      else
         write(outfile,*)' Only g03 and ORCA are available in progespp',
     $                    ' for the polarized calculation'
         stop 'in progespp'

      endif

100   format(6x,'Energy = ',f16.10/)
      return
9993  write (outfile,*) 'Error creating Gaussian03 SPE',
     *                 ' input file of the polarized calculation'
	stop 'progespp 1'

9994  write (outfile,*) 'Error creating Orca SPE', 
     *                  ' input file of the polarized calculation'
      stop 'progespp 2'
c
c    end 3/28/2006 by Yan Zhang
c

      end
c
c
c*****************************************************************************
c  G03CHINP
c*****************************************************************************
c
      subroutine g03chinp(iunit,esmeth,esbas,qmcharge, qmmult,
     *   ssgeom,optl,type,ver_lab, qmmminp, ssmmgeom)
c
c     Subroutine g03chinp
c     
c     Created:  Aug 2, 2006 by Yan Zhang
c     Revised:  
c
c     Purpose:  Write a Gaussian03 input file for the ESP charge 
C        and MM layer optential calculation of 
c        the capped small system in presence of background point charges.
c                                                                       
c 
c     Called by: progespp 
c     Calls:
c
      use files
      use caltyp                                                        
      use input
      implicit none
c
c i/o variables
c 
      character*20 ver_lab
      character*20 esmeth
      character*20 esbas
c
      character*80 optl(maxoptl)   
      integer      iunit, type
      type (cartgeom) :: ssgeom   
      type (mmvalen)  :: ssmmgeom   
      type (qmmminf)  :: qmmminp
c
c local variables
c
      logical      linkzero
      integer      mfin, bfin, mend, bend, i, j, k
      
      integer      ii, jj, kk
      integer      lsymbol, ltypname
      integer      qmnatm,capnatm,cssnatm,lsnatm
      integer      ssnatm,bgchnatm,mmnatm
      integer      m1st,m2nd,m3rd
      integer      ntotch, ngroup, ngidn
      
      character*20 oldbas	    
      character*12 tswit(4)

c
c Revised by Hai Lin, 5/3/2009
c   to add cartesian keyword for oniom gradient calculations
c
c      character*25 typekey1, typekey2, typekey3, typekey4
      character*40 typekey1, typekey2, typekey3, typekey4
c
c End by Hai Lin, 5/3/2009
c
      character*40 mmkey, connkey
      character*16 chkkey                                               1020XW17
      character*12 oniomkey, mmprmkey 
      character*4  keykey
      character*2  atmsymbol
      character*4  typname
      
      double precision  charge

      integer qmcharge, qmmult

      data tswit /'Energy', 'Gradient', 'Hessian', 'Optimization'/
      
      typekey1 = 'sp '
c
c Revised by Hai Lin, 5/3/2009
c   to add cartesian keyword for oniom gradient calculations
c
c      typekey2 = 'opt=(maxcyc=1,nomicro) '
      typekey2 = 'opt=(maxcyc=1,nomicro,cartesian) '
c
c End by Hai Lin, 5/3/2009
c
      typekey3 = 'freq '
      typekey4 = 'opt=(nomicro) '                 
      
      keykey   = '#p '
      oniomkey = 'oniom( '
      mmkey    = ':amber=softfirst)=(scalecharge=555) '
c      connkey  = 'fchk geom=connectivity nosym '                       1020XW17
      connkey  = 'geom=connectivity nosym '                             1020XW17
c
c added by Xin-Ping Wu, 10/20/2017
c
      chkkey   = '%chk=Test.chk'
c
c end by Xin-Ping Wu, 10/20/2017
c
c     
c Revised by Hai Lin, 5/11/2009
c        
c      mmprmkey = 'NonBon  0 0 '
      mmprmkey = 'NonBon  0 1 '
c
c End by Hai Lin, 5/11/2009
c

      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      ssnatm   = ssmmgeom%natoms
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + mmnatm
      bgchnatm = ssnatm - lsnatm

      m1st = qmmminp%mm1stlay%natm1stlay
      m2nd = qmmminp%mm2ndlay%natm2ndlay     
      m3rd = qmmminp%mm3rdlay%natm3rdlay

      ngroup=qmmminp%polgnum

      mfin = 1
      bfin = 1
      mend = 0
      bend = 0	 

      write(outfile,999) 'Layer Potential', esmeth, esbas
 999  format('Running Gaussian Job:',3x,a15,3x,a10,3x,a10)

c
c  If the basis is pDZ+,g3large, mg3, or mg3s change the basis
c    specification to Gen so that the non-standard basis will be read in
c    from the foot of the input file. 
c
      if (esbas(:8).eq.'pdz+' .or. esbas(:3).eq.'mg3'
     *     .or. esbas(:7).eq.'g3large') then
         oldbas = esbas
         esbas = 'gen     '
      endif
c
c  Find ends of method and basis set specification
c
      call fspace(esmeth, mfin, mend)
      if (mend.eq.1) then
         mfin = len(esmeth)
      else
         mfin = mfin-1
      endif

      call fspace(esbas, bfin, bend)
      if (bend.eq.1) then
         bfin = len(esbas)
      else
         bfin = bfin-1
      endif
c
c     Memory and processors (specified in the user inputed options)      
c     linkzero is a logical (boolean) variable to determine if the current
c     line being read is a link0 command or a keyword option
c
      linkzero = .false.
      i = 1                                                             
      do 10 j=1,10                                                      
      if (optl(i)(j:j) .eq. '%' ) then                                  
      linkzero = .true.                                                 
      endif                                                             
10    CONTINUE                                                          

      do while (linkzero .and. i.le.maxoptl)             
         write(iunit,*) optl(i)                                         
         i = i+1                                                        
         linkzero = .false.                                             
          do 12 j=1,10                                                  
            if (optl(i)(j:j) .eq. '%' ) then                            
               linkzero = .true.                                        
            endif                                                       
12    CONTINUE                                                          
      end do                                                            
c
c added by Xin-Ping Wu, 10/20/2017
c
      write(iunit,*) chkkey
c
c end by Xin-Ping Wu, 10/20/2017
c
c
c  Oniom keywords
c
      write(iunit,2000) keykey, typekey1(:3), oniomkey(:7), 
     *      esmeth(:mfin), '/', esbas(:bfin), mmkey(:36), connkey(:29)
2000  format(a4,a,a,a,a1,a,a,/,'# ',a)
c
c  User inputed keywords
c
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
c
         do j=1, 79
            if (optl(i)(j:j) .eq. '!') then
               k = 1
            endif
         enddo
         if (k .eq. 0) then
            write(iunit,3000) '#', optl(i)
         endif
         i = i+1
      end do
c
3000  format (a1,a79)      
     
c
c   ESP charge and potential
c
      write(iunit,3001) '#','pop=mk'
c      write(iunit,3002) '#','prop=(potential, grid)'
      write(iunit,3002) '#','prop=(potential)'

3001  format (a1,5x,a6)      
3002  format (a1,5x,a25)      

c
c  Title
c
      write(iunit,3021) 
3021  format (/'ESP Charge and External Potential Calculation'/)

c
c  Charge and multiplicity
c
      write(iunit,3050) qmcharge, qmmult
3050  format (1x,i3,1x,i3)      
c
c  QM: atomic symbols and coordinates
c      atom type = 'CT' defined parameter

      do i=1, qmnatm
         lsymbol = 0
         atmsymbol = ssgeom%atm(i)	    	 
         do j=1, 2
            if (atmsymbol(j:j) .ne. ' ') then 
               lsymbol = lsymbol + 1
            endif 
         enddo

         ltypname = 2
         typname  = qmtypname        
         
         write(iunit,4000)
     *      atmsymbol(:lsymbol), typname(:ltypname), 
     *      ssmmgeom%atmcharge(i),  ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)

      enddo     
4000  format (a,'-',a,'-',d10.4,t20,'0 ',3(d16.9,3x),'H')
c
c  Cap: atomic symbols and coordinates
c       take H as symbol and HC as atomtype, as defined parameters
c
      do i = qmnatm + 1, cssnatm
        atmsymbol = capname
        lsymbol   = 1
        typname   = captypname
        ltypname  = 2
        write(iunit,4000)
     *      atmsymbol(:lsymbol), typname(:ltypname),
     *      ssmmgeom%atmcharge(i),  ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)				 
      enddo
c
c  MM and M1/auxiliary charges: atomic symbols and coordinates
c     take He as symbol and HC as atomtype, as defined parameters     
c
      do i=cssnatm+1,ssnatm
         atmsymbol = chargename
         lsymbol   = 2
         typname   = chargetypname
         ltypname  = 2
         write(iunit,4100)
     *      atmsymbol(:lsymbol), typname(:ltypname),
     *      ssmmgeom%atmcharge(i),  ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)           	 	 
      enddo
      
      write(iunit,*)
4100  format (a,'-',a,'-',d10.4,t20,'0 ',3(d16.9,3x),'L')
c
c  Connectivity: only those for cap atoms are provided
c
c Revised by Hai Lin, 5/11/2009
C No longer need a bond between 1st and 2nd atoms, since NoBond is set to 0 1
c
c  revised by Hai Lin, 5/3/2009
c  add a bond between the 1st and 2nd QM atoms
c
c      do i=1, qmnatm
c         write (iunit,4500) i
c      enddo	 

c      write (iunit,4500) 1, 2, 1.0
c      do i=2, qmnatm
      do i=1, qmnatm
         write (iunit,4500) i
      enddo
c
c  end by Hai Lin, 5/3/2009
c End by Hai Lin, 5/11/2009
c

      do i=qmnatm+1, cssnatm
         jj = ssmmgeom%atmvalenlist(i,1)
        do j=1, qmnatm
           if (jj .eq. qmmminp%qmatm(j)) then
               write (iunit,4500) i, j, 1.0 
            endif
        enddo
      enddo	     	        
      
      do i=cssnatm+1, ssnatm
         write (iunit,4500) i
      enddo	 
      
      write(iunit,*)          
4500  format (2x,i4,2x,i4,2x,f4.2)
c
c write additional options for molecule specification                   
c
      i = 1
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
         jj = 1
c
         do j=1, 79
            if (optl(i)(j:j) .eq. '!') then
               k = 1
               jj = j + 1
            endif
         enddo
         if (k .eq. 1) then
c
            write(iunit,5000) optl(i)(jj:80)
         endif
         i = i + 1
      enddo
c      write(iunit,*)
c
5000  format (a79)      
c
c  Include basis set file at end of the input file if needed
c
      if (esbas(:3).eq.'gen') then
         if (oldbas(:8).eq.'pdz+') then
            write (iunit, 1051) '@', pDZsloc
         elseif (oldbas(:4).eq.'mg3s') then
            write (iunit, 1051) '@', mg3sloc
         elseif (oldbas(:3).eq.'mg3') then
            write (iunit, 1051) '@', mg3loc
         elseif (oldbas(:7).eq.'g3large') then
            write (iunit, 1051) '@', g3Lloc
         endif
      endif
1051  format (2a,/)
c
c  Change the basis set name back if they've been changed
c
      if (esbas(:3).eq.'gen') then
         esbas = oldbas
      endif
c
c  NonBon key
c
      write(iunit,*) mmprmkey

c
c  	prop=(potential,grid) 
c     N,NEFG,LTape,KTape
c
      write(iunit,*)
c      write(iunit,*)m1st,', ','3, ','50, ','51'
c
c  Close the file
c
      write(iunit,*)
      write(iunit,*)      
      close (iunit, err=9999)
      
      return

9999  write (outfile,*) 'Error creating Gaussian input file'
      stop 'g03inp'

1001  format (/)
1010  format (2a)
1011  format (4a,3(3x,a))
1012  format (4a,2(3x,a))

      end
c
c
c*****************************************************************************
c  G03OUTOP
c*****************************************************************************
c
      subroutine g03outop(geom, mmgeom, ssgeom,ssmmgeom, qmmminp, pot)
c
c     Subroutine g03outop
c     
c     Created:  Aug 8, 2006 by Yan Zhang
c     Revised:  
c
c     Purpose:  Read the background charge potential in the g03.out file.
c                                                                       
c 
c     Called by: progespp 
c     Calls:	case
c
      use files
      use caltyp                                                        
      use input
      implicit none
c
c i/o variables
c 
      type (cartgeom) :: geom, ssgeom   
      type (mmvalen)  :: mmgeom,ssmmgeom   
      type (qmmminf)  :: qmmminp
      
      double precision   pot(1)
c
c local variables
c
      integer      i, j, k, ii, jj, kk
      integer      qmnatm,capnatm,cssnatm,lsnatm
      integer      ssnatm,bgchnatm,mmnatm
      integer      m1st, m2nd, m3rd

      double precision ptmp
 
      character*4  ctmp
c
      character*80 string1, string, case

c
c
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      ssnatm   = ssmmgeom%natoms
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + mmnatm
      bgchnatm = ssnatm - lsnatm

      m1st = qmmminp%mm1stlay%natm1stlay
      m2nd = qmmminp%mm2ndlay%natm2ndlay     
      m3rd = qmmminp%mm3rdlay%natm3rdlay

      string1 = '   '
c
c    Open the g03.out
c
      open(unit=100,status='unknown', file='g03.out', err=9900)            
c
c    Read QM and cap atom
c

c
      read( 100, '(a80)', end=9910 ) string                               
      string = case(string)

      do while( (string1(16:23).ne.'electric').or.
     &             (string(16:24).ne.'potential'))
         string1=string
c
         read( 100, '(a80)', end=9910 ) string                            
         string = case(string)
      end do

c
      read( 100, '(a80)', end=9910 ) string

      do i=1,cssnatm
         read(100, *, end=9910) j, ctmp, ptmp
      end do
c
c    Read background charge electric potential
c
      do i=cssnatm+1,lsnatm
         k=ssmmgeom%atmid(i)
         read(100,*,end=9910)ii,ctmp,pot(k)
      enddo
      close(100)
      return
9900  write( outfile,*) 'Error opening Gaussian output file'
      stop 'g03outop 1'

9910  write( outfile,*) 'Error reading Gaussian output file'
      stop 'g03outop 2'
      end
c
c
c*****************************************************************************
c  g03OUTESP
c*****************************************************************************
c
      subroutine g03outesp(natm)
c
c     Subroutine g03outesp
c     
c     Created:  Aug. 3, 2006 by Yan Lin
c     Revised:
c     Purpose:  Get ESP charge out of the Gaussian03 output.
c
c
c     Called by:	progespp
c     Calls:
c
c     Variables defined here:
c
      use files
      use numfmt
      use input
      implicit none

      integer i, j, k, natm

      double precision charge

      character*2  atm
c
      character*80 string, case

c
c  Open the g03 output file.
c

      open(unit=100,status='unknown', file='g03.out', err=9900)            

c
      read( 100, '(a80)', end=9910 ) string                               
      string = case(string)

      do while( string(2:21).ne.'charges from esp fit' )
c
         read( 100, '(a80)', end=9920 ) string                            
         string = case(string)
      end do

c
      read( 100, '(a80)', end=9910 ) string
c
      read( 100, '(a80)', end=9910 ) string

      write(chargfile,190)
      do i=1,natm
      read(100, *, end=9910)k,atm, charge
      write(chargfile,200)atm,charge
      end do

190   format(6x,'QM atomic ESP charge:')
200   format(6x,'QM Atom:',2x,a2,3x,'charge:',3x,f15.9)

9920  continue
      close(100)                                                   

      return
9900  write( outfile,*) 'Error opening Gaussian output file'
      stop 'g03outesp 1'
9910  write( outfile,*) 'Error reading Gaussian output file'
      stop 'g03outesp 2'

      end
c
c
c*******************************************************************************
c  RIDLIST
c*******************************************************************************
c
      subroutine ridlist(idlist, idnum, iend) 
c 
c     Created:  Aug 7, 2006 by Yan Zhang 
c     Revised:  
c               
c
c     Purpose:  read a atmoic id list after a keyword.
c 
c     Description:  Using the index i, loop through each line after the
c     input file line w/ section name.  Reads in a line and finds the
c     number of the words in this line, then converts the words into integer.
c
c     Called by:  rmlgen, rextopt 
c     Calls:  fchar, chkln
c

      use files, only : outfile, infile
      use input, only : maxoptl, maxidnum
      implicit none
c
c i/o variables
c
      integer       idlist(maxidnum)
      integer       idnum

      integer       iend
c
c local variables
c            
      integer        nword,icint
      integer        j, idw, isect, iblnk, iscend, icommt,ieof
c
      character*80   word(40)
c
      idw=0
      isect = 0
      iend = 0
      iscend = 0
      idnum = 0
c
c     Reads and checks the next non-blank, non-comment line.
c
  20  continue
      call readln(infile,word,nword,isect,ieof,iscend)

c
c     Exits if a new section is reached.  Ends if an 'END' is found.
c     Exits if the idnum is larger than maxidnum and no 'END' is found.
c
      if (isect .eq. 1) then
         write (outfile, 1000)
1000     format(/3x, 'ERROR:  No termination of reading id')
         stop 'ridlist 1'
      elseif (iscend .eq. 1) then
         goto 40
      elseif (idnum .gt. maxidnum) then
         write (outfile, 1100) maxidnum
1100     format (/3x, 'ERROR: the number of id is no larger than ',
     *      i5)
         stop 'ridlist 2'
      else
         if(nword.gt.5) goto 9999

         idnum=idnum+nword
         do j=1,nword
         idw=idw+1
         idlist(idw)=icint(word(j))
         end do

         goto 20
      endif

   40 return

9999  write (outfile, 1200)
1200  format(/3x, 'ERROR: the id number in a line no larger than 5')
      stop 'ridlist 3'
      end

c
c*****************************************************************************
c  OrcaSTPOTINP
c*****************************************************************************
c
      subroutine orcastpotinp(iunit,esmeth,esbas,
     *   qmcharge, qmmult, ssgeom,optl,type,ver_lab,
     *   qmmminp, ssmmgeom)
c
c     Subroutine orcastpotinp
c     
c     Created:  Nov. 28, 2006 by Yan Zhang
c     Revised:  
c
c     Purpose:  Write a ORCA input file for the potential calculation of 
c        the capped small system in presence of background point charges
c        Type indicates whether it is an energy, gradient, or
c        hessian calculation. 
c                                                                       
c 
c     Called by:  progehk, mprogehk, progghk, proghhk, progohk
c     Calls:
c
      use files
      use caltyp                                                        
      use input
      implicit none
c
c i/o variables
c 
      character*20 ver_lab
      character*20 esmeth
      character*20 esbas
c
      character*80 optl(maxoptl)   
      integer      iunit, type
      type (cartgeom) :: ssgeom   
      type (mmvalen)  :: ssmmgeom   
      type (qmmminf)  :: qmmminp
c
c local variables
c
      logical      linkzero
      integer      mfin, bfin, mend, bend, i, j, k
      
      integer      ii, jj, kk
      integer      lsymbol, ltypname
      integer      qmnatm,capnatm,cssnatm,lsnatm
      integer      ssnatm,bgchnatm,mmnatm
      
      character*20 oldbas	    
      character*12 tswit(4)
      
      character*40 mmkey, connkey
      character*12 oniomkey, mmprmkey 
      character*4  keykey
      character*2  atmsymbol
      character*4  typname
      
      double precision  charge

c
      character*80   wmeth(40),wbas(40),blockopt
      integer  nwmeth,nwbas,ierr
      integer  lwmeth(40),lwbas(40)

      integer  qmcharge, qmmult

      data tswit /'Energy', 'Gradient', 'Hessian', 'Optimization'/
      

      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      ssnatm   = ssmmgeom%natoms
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + mmnatm
      bgchnatm = ssnatm - lsnatm
      
      mfin = 1
      bfin = 1
      mend = 0
      bend = 0	 

      write(outfile,998)
      write(outfile,999) tswit(type), esmeth, esbas
 999  format('Running Orca Job:',3x,a,3x,a10,3x,a10)
 998  format(/,'ORCA potential calculation for the small system',
     $       ' in the presence of the backgroud charges.',/)

c
c  Find ends of method and basis set specification
c
      call wordstr(esmeth,20,wmeth,nwmeth,lwmeth,ierr)

      call wordstr(esbas,20,wbas,nwbas,lwbas,ierr)

c
c  Title
c
      if (type .eq. tener) then
         write(iunit,3021) 
      elseif (type .eq. tgrad) then
         write(iunit,3022)
      elseif (type .eq. thess) then
         write(iunit,3023)
      elseif (type .eq. topt) then
         write(iunit,3024)
      endif
c
c  
c  print out method
c
      write(iunit,1013)'%method  method',wmeth(1)(:lwmeth(1))
      write(iunit,'(10x,a6)')'end'
c
c  print out basis
c
      write(iunit,1013)'%basis  basis',wbas(1)(:lwbas(1))
      write(iunit,'(10x,a6)')'end'

c
c print out calculation type
c
      if (type .eq. tener) then
         write(iunit,'(a8)')'! energy' 
      elseif (type .eq. tgrad) then
         write(iunit,'(a8)')'! engrad'
      elseif (type .eq. thess) then
         write(iunit,'(a9)')  '! NumFreq'
      elseif (type .eq. topt) then
         write(iunit,'(a5)') '! opt'
      endif
c
c  print out input block
c    ORCA use END as keyword in several inputs.
c    In order to distinguish the END keywords (i.e., QRCA END keyword and 
c     the QMMM END keyword), we require that the ORCA option keywords 
c      start with an indicator >.  
c
      i = 1
      do while (i.le.maxoptl)

        if(optl(i).ne.'   ') then
            blockopt=optl(i)
            j=1
c
            do while(blockopt(j:j).eq.' '.and.j.le.80)
                j=j+1
            enddo
            if(blockopt(j:j).eq.'>')    
c
     $	        write(iunit,'(a79)') blockopt(j+1:80)
        endif
        i = i + 1
      enddo
c
c  print out background point charge file 
c    The background charges are listed in a separated file 
c    called orca.pc.
c
      write(iunit,'(a24)') '%pointcharges "orca.pc" ' 
      write(iunit,'(a40)') '%scf  keepdensity true end'

3021  format ('#Single Point Energy Calculation'/)
3022  format ('#Gradient Calculation'/)
3023  format ('#Hessian Calculations'/)
3024  format ('#External Optimization'/)
c
c  Charge and multiplicity
c
      write(iunit,3050) '* xyz',qmcharge, qmmult
3050  format (a5,1x,i3,1x,i3)      
c
c  QM: atomic symbols and coordinates
c      

      do i=1, qmnatm
         lsymbol = 0
         atmsymbol = ssgeom%atm(i)	    	 
         do j=1, 2
            if (atmsymbol(j:j) .ne. ' ') then 
               lsymbol = lsymbol + 1
            endif 
         enddo

         write(iunit,4000)
     *      atmsymbol(:lsymbol), 
     *      ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)

      enddo     
4000  format (2x,a3,2x,3(f16.9,3x))

c
c  Cap: atomic symbols and coordinates
c    
c
      do i = qmnatm + 1, cssnatm
         atmsymbol = capname
         lsymbol   = 1
         typname   = captypname
         ltypname  = 2
         write(iunit,4000)
     *      atmsymbol(:lsymbol),
     *      ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)				 
      enddo
      write(iunit,*)'*'

c
c  MM and M1/auxiliary charges: atomic symbols and coordinates
c          
c
      open(unit=200,file='orca.pc',err=9999)
      write(200,'(i5)')ssnatm-cssnatm

      do i=cssnatm+1,ssnatm
         atmsymbol = chargename
         lsymbol   = 2
         typname   = chargetypname
         ltypname  = 2
         write(200,4100)
     *      ssmmgeom%atmcharge(i),  ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)           	 	 
      enddo
      
      write(200,*)
4100  format (f7.4,2x,3(f16.9,3x))

      close(200,err=9999)
c
c  Write the  files of the grid coordinates for the potential calculations. 
c
      open(unit=300,file='orca.pot.xyz',err=9999)
      write(300,'(i5)')lsnatm-cssnatm

      do i=cssnatm+1,lsnatm
         write(300,4200)
     *       ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)           	 	 
      enddo
      
c
      write(300,'(a80)')
4200  format (3(f16.9,3x))

      close(300,err=9999)
c
c
      
      return

9999  write (outfile,*) 'Error creating Orca input file'
      stop 'orcastinp'

1001  format (/)
1010  format (2a)
1011  format (4a,3(3x,a))
1012  format (4a,2(3x,a))
1013	format (a,2x,a)
      end
c
c
c*****************************************************************************
c  ORCAOUTPOT
c*****************************************************************************
c
      subroutine orcaoutpot(ssmmgeom, qmmminp, pot)
c
c     Subroutine orcaoutpot
c     
c     Created:  Nov. 28, 2006 by Yan Zhang
c     Revised:  
c
c     Purpose:  Read the background charge potential in the orca.pot file.
c                                                                       
c 
c     Called by: progespp 
c     Calls:	case
c
      use files
      use caltyp                                                        
      use input
      implicit none
c
c i/o variables
c 
      type (mmvalen)  :: ssmmgeom   
      type (qmmminf)  :: qmmminp
      
      double precision   pot(1)
c
c local variables
c
      integer      i, j, k
      integer      qmnatm,capnatm,cssnatm,lsnatm
      integer      ssnatm,bgchnatm,mmnatm
      integer      m1st, m2nd, m3rd

      double precision x1,x2,x3,ptmp, angtoau

      character*4  ctmp
c
      character*80 string1, string, case
c
      angtoau = 0.529177249d0
c
c
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      ssnatm   = ssmmgeom%natoms
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + mmnatm
      bgchnatm = ssnatm - lsnatm

      m1st = qmmminp%mm1stlay%natm1stlay
      m2nd = qmmminp%mm2ndlay%natm2ndlay     
      m3rd = qmmminp%mm3rdlay%natm3rdlay

      string1 = '   '
c
c    Open the g03.out
c
      open(unit=100,status='unknown', file='orca.pot', err=9910)            
c
c    Read the grid potential
c

      read( 100, '(i6)', end=9910 )i
      if(i.ne.(lsnatm-cssnatm)) then
         write(outfile,'(a60)')
     $ 'Error: the number of the grids for the potential calculations.'                               
      stop 'orcaoutpot 1'
      end if
      
      do i=cssnatm+1,lsnatm
         k=ssmmgeom%atmid(i)
         read(100,*,end=9910)x1,x2,x3,ptmp      
         pot(k)=ptmp*angtoau
      end do
      close(100)      

      return

9910  write(outfile,*) 'Error reading the orca.pot file.'
      stop 'orcaoutpot 2'

      end
c
c******************************************************************************
c******************************************************************************
