c					  
c******************************************************************************
c  MLEHOOK
c******************************************************************************
c
      subroutine mlehook(progrms, geom, mcoptl,
     *   ltest, ltestmm, mmgeom,                      
     *   lqmmm, qmmminp,
     *   tmeth, tbas, toptl, ierr)                                      
c  
c     Subroutine mlehook
c
c     Created:  Feb. 22, 2005 by Hai Lin
c
c     Purpose:  To call the appropriate energy subhook
c
c     Called by:  main
c     Calls:   
c             progehk, progmmehk
c
c
      use input,    only : cartgeom, proglist, mmvalen,        
     *                     qmmminf, maxoptl                   
      use files,    only : outfile
      use optimize, only : ener
      use caltyp,   only : tener, tprm
      use energy,   only : qmmmener

      implicit none
c
c  i/o variables
c also add mmgeom definition below                                      
c
      integer            ierr, limohc, ltest, ltestmm, lqmmm            
      type (proglist) :: progrms
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom	    
c
      character*80    :: mcoptl(maxoptl), toptl(maxoptl)
      character*20    :: tmeth, tbas
      type (qmmminf)  :: qmmminp                                        
c
c  local variables
c

c
c  Make calls to subhooks depending on activated switches
c
c  TEST
c
      if (ltest .eq. 1)  then
         call progehk(progrms%testprog,tmeth,tbas,
     &  geom%charge, geom%mult, geom,toptl,ener,ierr)
         call testdisp( geom%natoms, tener )
      endif
c      
c  TESTMM                                                               
c
      if (ltestmm .eq. 1)  then                                         
         call progmmehk(progrms%testmmprog,                             
     * tmeth,tbas,geom,mmgeom,toptl,ener,ierr)      
         call testdisp( geom%natoms, tener )                            
      endif	 		  	                                
c
c  QMMM                                                                 
c
      if (lqmmm .eq. 1)  then
         tprm = 1
         call qmmmehk(progrms%qmprog, progrms%mmprog, geom, mmgeom,
     *            qmmminp, ierr)
        ener = qmmmener
      endif	                                                        
      
      return
      end
c
c     
c*****************************************************************************
c  PROGEHK
c*****************************************************************************
c
      subroutine progehk(prognme, esmeth, esbas, qmcharge, qmmult,
     *           geom, optl, ener, ierr)
c
c     Subroutine progehk
c
c     Created:  Feb. 22, 2005 by Hai Lin
c     Revised:  Mar. 04, 2006 by Yan Zhang
c               May. 22, 2006 by Yan Zhang
c
c     Purpose:  Get a single point energy for the given geom with the
c        specified electronic structure package.
c
c     Description:  Calls a function to write a G03 SPE input file,
c         runs this calculation on the system, and then calls a fxn that
c         reads the energy from the formatted checkpoint file created by
c         G03.
c
c     Note:  Currently only G03 is enabled but it would be easily
c        expanded to other electronic structure programs.  new _inpe and
c        _oute subroutines would have to be written for each different
c        program though.
c
c     Called by:  optehk, 
c     Calls:  g03inp, g03oute, system, orcainp, orcaoute
c
c     Variables defined here:
c
      use caltyp
      use files
      use input, only : cartgeom, maxoptl
      implicit none

      character*10 prognme
      character*20 ver_lab                                                 
      character*20 esmeth                      
      character*20 esbas                                                   
      type (cartgeom) :: geom
c
      character*80 optl(maxoptl)
      double precision ener
      integer ierr
c
      integer  qmcharge, qmmult

c
c  GAUSSIAN03
c
c      if (prognme(:3).eq.'g03') then
c      if ((prognme(:3).eq.'g03') .or. (prognme(:3).eq.'g09')) then      0621BW14 add g09
      if ((prognme(:3).eq.'g03') .or. (prognme(:3).eq.'g09')            1020XW17
     * .or. (prognme(:3).eq.'g16')) then                                1020XW17

         open (unit=g03inf, status='unknown', file=g03in, err=9993)
         call g03inp(g03inf, esmeth, esbas, qmcharge, qmmult,
     *        geom, optl,tener,ver_lab)
         close (g03inf, err=9993)
         call system('g03shuttle g03.inp g03.out')
         call g03oute(1,esmeth, ener)                                      
c
c  ORCA added on 3/24/2006 by Yan Zhang
c
      else if (prognme(:4).eq.'orca') then
         open (unit=orcainf, status='unknown', file=orcain, err=9994)
         call orcainp(orcainf, esmeth, esbas, qmcharge, qmmult,
     *        geom, optl,tener,ver_lab)
         close (orcainf, err=9994)
         call system('orcashuttle orca.inp orca.out')
         call orcaoute(1,esmeth, ener)                                      
c
c   end 3/24/2006  by Yan Zhang
C
c   Gamess   5/22/2006 by Yan Zhang
c
      else if (prognme(:6).eq.'gamess') then
         open (unit=gmsinf, status='unknown', file=gmsin, err=9995)
         call gmsinp(gmsinf,esmeth,esbas,qmcharge, qmmult,
     *       geom,optl,tener,ver_lab)
         close(gmsinf,err=9995)
         call system('gmsshuttle gms.inp gms.out')
         call gmsoute (1,esmeth, ener)
c
c   end 5/22/2006  by Yan Zhang
C
      endif

      return

c9993  write (outfile,*) 'Error creating Gaussian03 SPE input file'
9993  write (outfile,*) 'Error creating Gaussian SPE input file'        1020XW17
        stop 'progehk 3'

9994  write (outfile,*) 'Error creating Orca SPE input file'
      stop 'progehk 4'      

9995  write (outfile,*) 'Error creating Gamess SPE input file'
      stop 'progehk 5'      


      end
c
c
c*****************************************************************************
c  PROGMMEHK
c*****************************************************************************
c
      subroutine progmmehk(prognme, esmeth, esbas, 
     * geom, mmgeom, optl, ener, ierr)
c
c     Subroutine progmmehk
c
c     Created:  Feb. 22, 2004 by Hai Lin
c               based on progehk.
c
c     Purpose:  Get a single point energy for the given geom with the
c        specified force field package.
c
c     Description:  Calls a subroutine to write a input file,
c         runs this calculation on the system, and then calls a subroutine 
c         that reads the energy from the output file.
c
c     Note:  Currently only TINKER is enabled but it would be easily
c        expanded to other force field programs.  new _inpe and
c        _oute subroutines would have to be written for each different
c        program though.
c
c     Called by:  mlehook, optehk, 
c     Calls:  t41inp, t41oute, system
c
      use caltyp
      use files
      use input 
      implicit none
c
c  i/o variables
c
      character*10       prognme
      character*20       esmeth                      
      character*20       esbas                                                   
      type (cartgeom) :: geom
      type (mmvalen) ::  mmgeom
c
      character*80       optl(maxoptl)
      double precision   ener
      integer            ierr
c
c  local variables
c
      character*20 ver_lab      
c
c  TINKER                                                            
c
      if (prognme(:6).eq.'tinker') then
c
c write .xyz and .key files    
c      
         open (unit=t41inf, status='unknown', file=t41in, err=9994)
         open (unit=t41keyf, status='unknown', file=t41key, err=9995)
         call t41inp(t41inf, t41keyf,
     *    esmeth, esbas, geom, mmgeom,optl,tener,ver_lab)
         close (t41inf, err=9994)
         close (t41keyf, err=9995)        	    
c
c read the atomic MM charge 
c  also print MM parameters if required
c 	 
         if((tprm .eq. 1) .or. (mmgeom%lprnatmprm .eq. 1)) then
            call system('t41shuttle t41 p')
            call t41outp(mmgeom)
         endif    
c
c compute the energy
c	 	 
         call system('t41shuttle t41 e')                                
         call t41oute(1,esmeth,ener)	 
      endif
      return
c
9994  write (outfile,*) 'Error creating TINKER .xyz file'
      stop 'progmmehk 4'
9995  write (outfile,*) 'Error creating TINKER .key file'
      stop 'progmmehk 5'
      end
c
c
c*****************************************************************************
c  PROGMM0EHK
c*****************************************************************************
c
      subroutine progmm0ehk(prognme, esmeth, esbas, 
     * geom, mmgeom, optl, ener, ierr, qmmminp)
c
c     Subroutine progmm0ehk
c
c     Created:  Feb. 22, 2004 by Hai Lin
c               based on progmmehk.
c
c     Purpose:  Get a single point energy for the given geom with the
c        specified force field package, with MM charges on QM atoms
c        set to zero.
c
c     Description:  Calls a subroutine to write a input file,
c         runs this calculation on the system, and then calls a subroutine 
c         that reads the energy from the output file.
c
c     Note:  Currently only TINKER is enabled but it would be easily
c        expanded to other force field programs.  new _inpe and
c        _oute subroutines would have to be written for each different
c        program though.
c
c     Called by:  mlehook, optehk, 
c     Calls:  t41inp, t41oute, system
c
      use caltyp
      use files
      use input 
      implicit none
c
c  i/o variables
c
      character*10       prognme
      character*20       esmeth                      
      character*20       esbas                                                   
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
c
      character*80       optl(maxoptl)
      double precision   ener
      integer            ierr
      type (qmmminf)  :: qmmminp 
c
c  local variables
c
      character*20 ver_lab   
      integer      i, ii   
      integer      j, mmatm, mmnum                                      0621BW14 variables to count numbers of MM atoms
      double precision mmcharge,mm1charge                               0621BW14

c
c  TINKER                                                            
c
      if (prognme(:6) .eq. 'tinker') then
c
c write .xyz and .key files    
c      
         open (unit=t41inf, status='unknown', file=t41in, err=9994)
         open (unit=t41keyf, status='unknown', file=t41key, err=9995)
         call t41inp(t41inf, t41keyf,
     *    esmeth, esbas, geom, mmgeom,optl,tener,ver_lab)
         close (t41inf, err=9994)
c
c set MM charges on QM atoms to zero
c  for the entire system, the index read in for the QM atoms will be used
c  for the capped system, the index will be from 1 to N, where N is the 
c      summed number for the QM and HL atoms.
c  the tcssmmkey in the common block caltyp specify the system, i.e.
c      whether the entire or the capped system
         
        if (tcssmmkey .eq. 1) then
           ii = qmmminp%qmnatm
           do i=1, ii
              write (t41keyf, *) 'charge ', -qmmminp%qmatm(i), ' 0.0'
           enddo		  
        elseif (tcssmmkey .eq. 2) then
           ii = qmmminp%qmnatm + qmmminp%capatm%capnatm 
           do i=1, ii
              write (t41keyf, *) 'charge ', -i, ' 0.0'
           enddo
        else
           write (outfile,*) 'Error! Please specify CPS or ES.'
           stop 'progmm0ehk 6'
        endif
           close (t41keyf, err=9995)        	    
c
c read the atomic MM charge 
c  also print MM parameters if required
c 	 
         if((tprm .eq. 1) .or. (mmgeom%lprnatmprm .eq. 1)) then
            call system('t41shuttle t41 p')

            call t41outp(mmgeom)
         endif    

c   Add by Bo Wang 0621BW14 
c   If charge correction is used,
c   use balanced M1 charge for MM part calculation in QM/MM energy
         if (tcssmmkey .eq. 1 .and. qmmminp%lchgcorr .ne. 0) then                   
             open (unit=t41keyf, status='old',position='append',
     &                     file=t41key, err=9995)
             do i=1, qmmminp%mm1stlay%natm1stlay
               ii = qmmminp%mm1stlay%atm1stlay(i)
               mmcharge=0
               mmnum = qmmminp%balgroupn(i)
               do j=2,mmnum
                 mmatm=qmmminp%balgroup(i,j)
                 mmcharge = mmcharge + mmgeom%atmcharge(mmatm)
               enddo
               mm1charge = qmmminp%groupchg(i) - mmcharge
               write (t41keyf, 100) -ii,mm1charge
100   format (1x, 'charge',3x,i5,3x,d14.6)
             enddo
             close (t41keyf, err=9995)
         endif
c   end by Bo Wang  0621BW14 

c
c compute the energy
c	 	 
         call system('t41shuttle t41 e')                                
         call t41oute(1,esmeth,ener)	 
      endif
      return
c
9994  write (outfile,*) 'Error creating TINKER .xyz file'
      stop 'progmm0ehk 4'
9995  write (outfile,*) 'Error creating TINKER .key file'
      stop 'progmm0ehk 5'
      end
c
c*****************************************************************************
c  g03INP
c*****************************************************************************
c
      subroutine g03inp(iunit,esmeth,esbas,qmcharge, qmmult,
     *         geom,optl,type,ver_lab)
c
c     Subroutine g03inp
c     
c     Created:  Feb 22, 2004 by Hai Lin
c     Revised:  Apr 12, 2011 by Soroosh Pezeshki
c
c     Purpose:  Write a Gaussian03 input file for the calculation
c        specified.  Type indicates whether it is an energy, gradient, or
c        hessian calculation. 
c
c        If the options line begins with '!' then to be written after
c        the molecule specification.
c 
c     Called by:  progehk, mprogehk, progghk, proghhk, progohk
c     Calls:
c
      use files
      use caltyp                                                        
      use input, only : cartgeom, maxoptl
      implicit none
c
c i/o variables
c 
      character*20 ver_lab
      character*20 esmeth
      character*20 esbas
      
      type (cartgeom) :: geom
c
      character*80       optl(maxoptl)
      integer            iunit
c                        
c local variables             
c
      character*20   oldbas                                        
      integer        mfin, bfin, mend, bend, i, j, type
      character*12   tswit(4)
      logical        linkzero
           
      integer        k, jj    
      integer        qmcharge, qmmult

      data tswit /'Energy','Gradient','Hessian','Optimization'/
c
      mfin = 1
      bfin = 1
      mend = 0
      bend = 0

      if (tdynamics.ne.1) then                                          0412SP11
      write(*,999) tswit(type), esmeth, esbas
 999  format('Running Gaussian Job:',3x,a,3x,a10,3x,a10)
      endif                                                             0412SP11
c
c  If the basis is pDZ+,g3large, mg3, or mg3s change the basis
c    specification to Gen so that the non-standard basis will be read in
c    from the foot of the input file. 
c
      if (esbas(:8).eq.'pdz+' .or. esbas(:3).eq.'mg3'
     *     .or. esbas(:7).eq.'g3large') then
         oldbas = esbas
         esbas = 'gen     '
      endif
c
c  Find ends of method and basis set specification
c
      call fspace(esmeth, mfin, mend)
      if (mend.eq.1) then
         mfin = len(esmeth)
      else
         mfin = mfin-1
      endif

      call fspace(esbas, bfin, bend)
      if (bend.eq.1) then
         bfin = len(esbas)
      else
         bfin = bfin-1
      endif

c
c  Print out Route specifications - different for each possible type
c    specified.
c

c     memory and processors are now specified in the options      
c     linkzero is a logical (boolean) variable to determine if the current
c     line being read is a link0 command or a keyword option

      linkzero = .false.
      i = 1                                                             
      do 10 j=1,10                                                      
      if (optl(i)(j:j) .eq. '%' ) then                                  
      linkzero = .true.                                                 
      endif                                                             
10    CONTINUE                                                          
      do while (linkzero .and. i.le.maxoptl)                            
         write(iunit,*) optl(i)                                         
         i = i+1                                                        
         linkzero = .false.                                             
          do 12 j=1,10                                                  
            if (optl(i)(j:j) .eq. '%' ) then                            
               linkzero = .true.                                        
            endif                                                       
12    CONTINUE                                                          
      end do                                                            
c
c added by Xin-Ping Wu, 10/20/2017
c
      write(iunit,*) '%chk=Test.chk'
c
c end by Xin-Ping Wu, 10/20/2017
c

      if( type .eq. tener ) then

      if(esmeth(:12) .eq. 'qcisd(t)=e4t') then                          
               write(iunit,1011) '#','qcisd(t,e4t,maxcycle=100)',       
c     *               '/',esbas(:bfin),'  ','FChk=All','NoSymm'          
     *               '/',esbas(:bfin),'  ','NoSymm'                     1020XW17
      else                                                              
         write(iunit,1011) '#',esmeth(:mfin),'/',esbas(:bfin),
c     *           '  ','FChk=All','NoSymm'
     *           '  ','NoSymm'                                          1020XW17
      endif                                                             


      elseif( type .eq. tgrad ) then

         if( esmeth(:4).eq.'mp4 ' .or.
     *        esmeth(:4).eq.'ccsd' .or. esmeth(:8).eq.'qcisd(t)' ) then
            if(esmeth(:12) .eq. 'qcisd(t)=e4t') then                    
               write(iunit,1011) '#','qcisd(t,e4t,maxcycle=100)',       
c     *         '/',esbas(:bfin),'Force=EnOnly','FChk=All','NoSymm'      
     *         '/',esbas(:bfin),'Force=EnOnly','NoSymm'                 1020XW17
            else                                                        
            write(iunit,1011) '#',esmeth(:mfin),'/',esbas(:bfin),
c     *              'Force=EnOnly','FChk=All','NoSymm'
     *              'Force=EnOnly','NoSymm'                             1020XW17
            endif                                                       
         else
            write(iunit,1011) '#',esmeth(:mfin),'/',esbas(:bfin),
c     *              'Force','FChk=All','NoSymm'
     *              'Force','NoSymm'                                    1020XW17
         endif

      elseif( type .eq. thess ) then
         write(iunit,1011) '#',esmeth(:mfin),'/',esbas(:bfin),
c     *           'Freq','FChk=All','NoSymm'
     *           'Freq','NoSymm'                                        1020XW17
      elseif( type .eq. topt ) then
         write(iunit,1012) '#',esmeth(:mfin),'/',esbas(:bfin),
c     *           'FChk=All','NoSymm'
     *           'NoSymm'                                               1020XW17
      endif

c
c skip the line if begin with '!'                                       
c
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
c
         do j=1, 79
            if (optl(i)(j:j) .eq. '!') then 
               k = 1
            endif
         enddo 
         if (k .eq. 0) then    
            write(iunit,1016) '#', optl(i)
         endif    
         i = i+1
      end do
c
c  Print out title
c
      if (type .eq. tener) then
         write(iunit,1021) 
      elseif (type .eq. tgrad) then
         write(iunit,1022)
      elseif (type .eq. thess) then
         write(iunit,1023)
      elseif (type .eq. topt) then
         write(iunit,1024)
      endif
c
c  Print out charge, multiplicity, and atomic symbols and coordinates
c
      write(iunit,1031) qmcharge, qmmult
      do 20 i=1, geom%natoms
         write(iunit,1041) geom%atm(i), geom%angcoord(3*i-2),
     *      geom%angcoord(3*i-1), geom%angcoord(3*i)
  20  continue

      write(iunit,*)
c
c write additional options for molecule specification                   
c
      i = 1
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
         jj = 1
c
         do j=1, 79
            if (optl(i)(j:j) .eq. '!') then
               k = 1
               jj = j + 1
            endif   
         enddo    
         if (k .eq. 1) then
c
            write(iunit,1060) optl(i)(jj:80)
         endif
         i = i + 1
      enddo
c
c  Include basis set file at end of the input file if needed
c
      if (esbas(:3).eq.'gen') then
         if (oldbas(:8).eq.'pdz+') then
            write (iunit, 1051) '@', pDZsloc
         elseif (oldbas(:4).eq.'mg3s') then
            write (iunit, 1051) '@', mg3sloc
         elseif (oldbas(:3).eq.'mg3') then
            write (iunit, 1051) '@', mg3loc
         elseif (oldbas(:7).eq.'g3large') then
            write (iunit, 1051) '@', g3Lloc
         endif
      endif
c
c  Change the method and basis set name back if they've been changed
c
      if (esmeth(:11).eq.'ccsd(t)=e4t') then
         esmeth = 'ccsd(t)    '
      elseif (esmeth(:12).eq.'qcisd(t)=e4t') then
         esmeth = 'qcisd(t)    '
      endif
      if (esbas(:3).eq.'gen') then
         esbas = oldbas
      endif
c
c  Close the file
c
      close (iunit, err=9999)
      
      return

9999  write (outfile,*) 'Error creating Gaussian input file'
      stop 'g03inp'

1001  format (/)
1010  format (2a)
c1011  format (4a,3(3x,a))
1011  format (4a,2(3x,a))                                               1020XW17
1012  format (4a,2(3x,a))
1016  format (a1,a75)
1021  format (/'Single Point Energy Calculation'/)
1022  format (/'Gradient Calculation'/)
1023  format (/'Hessian Calculations'/)
1024  format (/'External Optimization'/)
1031  format (1x,i3,1x,i3)
1041  format (a2,5x,3(d16.9,3x))
1051  format (2a,/)
c
1060  format (a79)

      end

c
c
c*****************************************************************************
c  g03OUTE
c*****************************************************************************
c
      subroutine g03oute (ppnum,esmeth, ener)
c
c     Subroutine g03oute
c     
c     Created:  Feb. 21, 2005 by Hai Lin
c     Revised:
c     Purpose:  Get a single energy out of the Gaussian03 formatted
c        checkpoint file. 
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c
c     Called by:  progehk, progghk, proghhk
c     Calls:
c
c     Variables defined here:
c
      use files
      use numfmt
      implicit none

      character*20  esmeth                                                    
      double precision  ener, cfltfmt
      integer enerfnd,ppnum,fchknum                                           

c
      character*80 string, case
      character*30 label
      character*22 etxt
      character*14 fchkname                                                   

      enerfnd = 0
c
c  Open the formatted checkpoint file.
c

      fchknum = fchk                                                          
      fchkname = testfchk                                                     

      open(unit=fchknum,status='unknown', file=fchkname, err=9900)            

c
      read( fchknum, '(a80)', end=9910 ) string                               
      string = case(string)
      label = string(:llen(efmt))
      etxt = string(nstrt(efmt):nstrt(efmt)+nlen(efmt)-1)

      do while( label(:12).ne.'total energy' )
c
         read( fchknum, '(a80)', end=9910 ) string                            
         string = case(string)
         label = string(:llen(efmt))
         etxt = string(nstrt(efmt):nstrt(efmt)+nlen(efmt)-1)
      end do

      ener = cfltfmt(etxt,efmt)

      close( unit=fchknum )                                                   

      return

9900  write( outfile,*) 'Error opening Gaussian checkpoint file'
      stop 'g03oute 1'
9910  write( outfile,*) 'Error reading Gaussian checkpoint file'
      stop 'g03oute 2'

      end
c
c
c*****************************************************************************
c  T41INP
c*****************************************************************************
c
      subroutine t41inp(ixyz,ikey,esmeth,esbas,geom,mmgeom,
     * optl,type,ver_lab)
c
c     Subroutine t41inp
c
c     Created:  Feb. 22, 2004 by Hai Lin
c     Revised:  Apr. 12, 2011 by Soroosh Pezeshki
c
c     Purpose:  Write a TINKER input file for the calculation
c        specified.  Type indicates whether it is an energy, gradient, or
c        hessian calculation. 
c
c 
c     Called by:  progmmehk, (mprogehk, progghk, proghhk, progohk)
c     Calls:
c
c     Variables defined here:
c
      use files
      use caltyp
      use input
      implicit none
c
c i/o variables 
c
      character*20 ver_lab
      character*20 esmeth
      character*20 esbas

      integer ixyz, ikey

      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom

c
      character*80  optl(maxoptl)
c
c local variables
c
      character*20 oldbas

      integer          mfin, bfin, mend, bend, i, j, k, jj, type
      integer          iiatom, iivalen, iid, iityp
      character*12     tswit(4)
      double precision mmx, mmy, mmz


      if (tdynamics.ne.1) then                                          0412SP11
      data tswit /'Energy','Gradient','Hessian','Optimization'/

      write(*,999) tswit(type), esmeth, esbas
 999  format('Running Tinker Job:',3x,a,3x,a10,3x,a10)
      endif                                                             0412SP11

c
c  Write the .xyz file
c
      iiatom = mmgeom%natoms
      write(ixyz,*) iiatom
c
      do i = 1, iiatom
         iid = mmgeom%atmid(i)
         iityp = mmgeom%atmtyp(i)
         iivalen = mmgeom%atmvalen(i)
         mmx = geom%angcoord(3*i-2)
         mmy = geom%angcoord(3*i-1)
         mmz = geom%angcoord(3*i)
c
c modiefied by Xin-Ping Wu, Sept. 21, 2017
c for mofs including transition metals
c we want to read up to eight valencies
c
         if(iivalen .le. 8) then
c
c   modified 2/10/2007 by Christoph Riplinger
c   we want to read up to six valencies
c
c         if(iivalen .le. 4) then
c         if(iivalen .le. 6) then
            write(ixyz,2000) iid,mmgeom%atmtypname(i), mmx,mmy,mmz,
     *        iityp, (mmgeom%atmvalenlist(i,j),j=1,iivalen)     
         endif
c
c   end 2/10/2007 by Christoph 
c
c end by Xin-Ping Wu, Sept. 21, 2017
c
      enddo            

c
c Write the .key file 
c   Write the parameter filename t41.prm
c
c     4) Tinker will always renumber atoms in xyz file first,
c         and everything afterwords including interpretation of the keywords
c         in the key file are using the new (renumbered) atom id.
c
      write(ikey,3000)
      write(ikey,2010) 
c
c   Write the content given as the OPTIONS section in .dat file
c     those begin with '!1' are for whole system only, while 
c     those begin with '!2' are for capped small system only,
c     this feature is very useful for charge modification.
c
      i = 1
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k  = 0
         jj = 1
c
         do j = 1, 78
            if (optl(i)(j:j+1) .eq. '!1') then
               k = 1
               jj = j + 2
            elseif (optl(i)(j:j+1) .eq. '!2') then
               k = 2
               jj = j + 2
            endif
         enddo
         if (k .eq. 0) then
            write(ikey,2020) optl(i)
         elseif (((k .eq. 1) .and. (tcssmmkey .eq. 1)) .or.
     *           ((k .eq. 2) .and. (tcssmmkey .eq. 2))) then
c
            write(ikey,2030) optl(i)(jj:80)
         endif
         i = i+1
      end do

      return

c9999  write (outfile,*) 'Error creating Gaussian input file'
c      stop 'g03inp'
9999  write (outfile,*) 'Error creating Tinker input file'              0510SP11
      stop 't41inp'                                                     0510SP11

c
c modified by Xin-Ping Wu, Sept. 21, 2017 
c   we want to read up to eight valencies                               
c
2000  format(i4,1x,a2,1x,3(f10.5,1x),i4,1x,8(i4,1x))                    
c
c   modified 2/10/2007 by Christoph Riplinger
c   we want to read up to six valencies
c
c2000  format(i6,2x,a4,3(f11.6,1x),i5,1x,4(i5,1x))
c2000  format(i6,2x,a4,3(f11.6,1x),i5,1x,6(i5,1x))
c
c   end 2/10/2007 by Christoph 
c
c end by Xin-Ping Wu, Sept. 21, 2017
c
2010  format('parameters t41.prm')
c
2020  format(a80)
c
2030  format(a78)
3000  format('# Tinker will always renumber sequentially atom ID', 
     *   /,'# (instead of re-order atoms according to their ID)',
     *   /,'# in the xyz file firstly,',
     *   /,'# and everything afterwords including interpretation of',
     *   /,'# the keywords in this key file will follow the new'
     *   /,'# (sequentially renumbered starting from 1) atom ID.',/)

      end

c
c
c*****************************************************************************
c  T41OUTE
c*****************************************************************************
c
      subroutine t41oute (ppnum,esmeth, ener)
c
c     Subroutine t41oute
c     
c     Created:  Feb. 22, 2004 by Hai Lin
c     Revised:
c     Purpose:  Get a single energy out of the TINKER output file
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c
c     Called by:  progmmehk, progmmghk, progmmhhk
c     Calls:
c
      use files
      use numfmt
      implicit none
c
c  i/o variables
c 
      character*20      esmeth                                                    
      double precision  ener
      integer           ppnum
c      
c  local variables
c
      integer          i, nwords, isec, ieof, iscend
      integer          iefind
      double precision kcal2au, au2kcal, ekcal, eau, eread
      double precision cfloat
c
      character*80     string
c
      character*80     words(40)      
c
c  Define energy converting factor 
c
      au2kcal = 627.5095d0
      kcal2au = 1.0d0 / au2kcal
c      
      open(unit=t41outf,status='unknown', file=t41out, err=9900)         
      ieof = 0
      nwords = 6
      iefind = 0
c
c Search for 'Total Potential Energy :'
c      read the energy and convert it into au.
c      
      do while (ieof .ne. 1)
         call readln(t41outf,words,nwords,isec,ieof,iscend)
            
         if(words(1) .eq. 'TOTAL'     .and.
     *      words(2) .eq. 'POTENTIAL' .and.
     *      words(3) .eq. 'ENERGY'    .and. 
     *      words(4) .eq. ':')        then
	       
           eread = cfloat(words(5))
           iefind = 1
           if(words(6) .eq. 'KCAL/MOLE') then
                ekcal = eread
                eau = ekcal * kcal2au
                goto 100
            elseif (words(6) .eq. 'AU') then
                eau = eread
                ekcal = eau *au2kcal
                goto 100
            else
                write(outfile,*) 'Error: No energy unit given'
                stop 't41oute 3'
            endif
         endif
      enddo

100   if(iefind .eq. 0) then 
         write(outfile, *) 'Error: No energy given in TINKER output.'
         stop 't41oute 4'
      endif
          
c
c  Read and write the energy components
c     
c
110   read(t41outf, '(a80)', end=9910 ) string                               
c
      write(outfile, '(a80)') string	    
      goto 110

9900  write( outfile,*) 'Error opening TINKER output file'
      stop 't41oute 1'
      
9910  write( outfile,1000) ekcal
1000  format(/1x,'Total energy',10x,f30.8)
      
      ener = eau
      close( unit=t41outf )
            
      return

      end
c
c
c*****************************************************************************
c  T41OUTP
c*****************************************************************************
c
      subroutine t41outp (mmgeom)
c
c     Subroutine t41outp
c     
c     Created:  Feb. 22, 2004 by Hai Lin
c     Revised:
c     Purpose:  Read the MM point charges 
c               and print the MM parameters if needed.
c
c     Called by:  progmmehk, progmmghk, progmmhhk
c     Calls:
c
      use caltyp
      use files
      use input, only: mmvalen
      implicit none
c
c  i/o variables
c
      type (mmvalen) :: mmgeom
c
c  local variables
c
      integer  i, j, nwords, isec, ieof, iscend
      integer  ichargefind, numatm
      integer  icint
      
      double precision  chargeread
      double precision  cfloat

c
      character*80  string
c
      character*80  words(40)
c      
      open(unit=t41outf,status='unknown', file=t41out, err=9900)         
      ieof   = 0
      nwords = 6
      ichargefind = 0
      numatm = mmgeom%natoms      
c
c Check if point charges need import (usually needed only one time)
c
      if (tprm .eq. 0) goto 1000
c
c Search for 'Atomic Partial Charge Parameters :'
c      read the energy and convert it into au.
c          :
      do while (ieof .ne. 1)
         call readln(t41outf,words,nwords,isec,ieof,iscend)

         if(words(1) .eq. 'ATOMIC'      .and.
     *      words(2) .eq. 'PARTIAL'     .and.
     *      words(3) .eq. 'CHARGE'      .and. 
     *      words(4) .eq. 'PARAMETERS')  then
	    
            ichargefind = 1
            goto 100
         endif
      enddo
         
100   if(ichargefind .eq. 0) then 
         write(outfile, *) 
     *    'Warning: No point charge given in TINKER output.'
c hailin20050117
c         stop 't41outp 2'
          goto 990
c hailinend	  
      endif

c
c skip the title line and read the point charges
c
c  1) Tinker will not show atoms with charge = 0.0      
c  2) Tinker will always renumber atoms in xyz file
c        and everything including keywords in key file are using
c        the new (renumbered) atom id.
c
      call readln(t41outf,words,nwords,isec,ieof,iscend)
      call readln(t41outf,words,nwords,isec,ieof,iscend)

      do while (ieof .ne. 1)
         call readln(t41outf,words,nwords,isec,ieof,iscend)
         j = icint(words(2))
         if (j .gt. 0) then
            chargeread = cfloat(words(3))
            mmgeom%atmcharge(j) = chargeread
         endif
      enddo
            
990   close( unit=t41outf )
1000  continue

c
c  Read and write the MM parameters if required
c     
      if(mmgeom%lprnatmprm .eq. 1) then
         mmgeom%lprnatmprm = 0
         open(unit=t41outf,status='unknown', 
     *        file=t41out, err=9900)

         write(outfile,2000) 
2000     format(/2x,'List the force field parameters:',/)
c
c to skip the comment lines in TINKER output file
c
         call readln(t41outf,words,nwords,isec,ieof,iscend)
	 	    
         if(ieof.eq.1) goto 9920 

c
110      read(t41outf, '(a80)', end=9910 ) string                               
c
         write(outfile, '(a80)') string	    
         goto 110
      endif
c
c  Debug info
c            
      if (tdebug .eq. 1) then 
         write (outfile, 3000)
         write (outfile, 3010)
         write (outfile, 3020) mmgeom%atmcharge(:numatm)
         write (outfile, 3005)
      endif      
3000  format (/,10('Debug'))
3005  format (10('Debug'),/)
3010  format (2x,'The atom point charges are: ')
3020  format (10(f8.4))      

      return
      
9900  write( outfile,*) 'Error opening TINKER output file'
      stop 't41outp 1'
9920  write( outfile,*) 'Error reading TINKER output file'
      stop 't41outp 2'

9910  close( unit=t41outf )
      write( outfile,2200) 
2200  format(/2x,'End of force field parameter list.',/)

      return
      
      end
c
c
c*****************************************************************************
c  CFLTFMT
c*****************************************************************************
c
      function cfltfmt (number, fmt)
c
c     Function cfltfmt
c
c     Created:  June 28, 1999 by J. M. Rodgers
c               Partially based on Polyrate cfloat function
c     Revised:
c     Purpose:  Convert a string in the specified FChk format (fmt) into a
c        double precision number.
c
c     Called by:  g03oute, g03outg, g03outh, g03outo
c     Calls:
c
c     Variables defined here:
c
c
c  Figure out the type of floating point gathered from the formatted
c    checkpoint file and use the appropriate module.
c
      use numfmt
      use files, only : outfile
      implicit none

      double precision cfltfmt
      double precision sign, esign, f1, f2, f3
      integer i, j, n
      character*22  number
      character  ch
      integer fmt
c
c  Error checking - if decimal place and exponent place are incorrect exit
c
      if (number(inum(fmt)+1:inum(fmt)+1) .ne. '.') then
         write (outfile,1001) number
      endif
      if (number(decfin(fmt)+1:decfin(fmt)+1) .ne. 'e'
     *      .and.  number(decfin(fmt)+1:decfin(fmt)+1) .ne. 'd' ) then
         write (outfile,1001) number
      endif

1001  format('ERROR: This number has the wrong format.',5x,a)

c
c  Get sign of number
c
      if (number(isign(fmt):isign(fmt)).eq.'-') then
         sign = -1.0000d0
      else
         sign = +1.0000d0
      endif
c
c  Get sign of exponent
c
      if (number(iesign(fmt):iesign(fmt)).eq.'-') then
         esign = -1.0000d0
      else
         esign = +1.0000d0
      endif
c
c  Get number before decimal
c
      ch = number(inum(fmt):inum(fmt))
      n  = ichar(ch) - ichar('0')
      f1 = dble(n)
c
c  Get number after decimal
c
      f2 = 0.0000d0
      j = 0
      do 60 i=decbeg(fmt), decfin(fmt)
         ch = number(i:i)
         n  = ichar(ch) - ichar('0')         
         f2 = f2*10.0000d0 + dble(n)
         j  = j+1
  60  continue
      f2 = f2 / 10.0000d0 ** dble(j)
c
c  Get the exponent
c
      f3 = 0.0000d0
      do 80 i=expbeg(fmt), expfin(fmt)
         ch = number (i:i)
         n = ichar(ch) - ichar('0')
         f3 = f3*10.0000d0 + dble(n)
  80  continue

      cfltfmt = (sign*(f1+f2)) * 10.0000d0 ** (esign*f3)

      return

      end
c
c
c******************************************************************************
c  QMMMEHK
c******************************************************************************
c
      subroutine qmmmehk(qmprognme, mmprognme, geom, 
     * mmgeom, qmmminp, ierr)
c
c     Subroutine qmmmehk
c
c     Created:  Feb. 22, 2004 by Hai Lin
c     Revised:  Dec. 03, 2004 by Hai Lin
c                 Set the MM charges for QM atoms to zero.
c               May  22, 2006 by Yan Zhang
c               May  25, 2006 by Yan Zhang
c               Aug.  8, 2006 by Yan Zhang
C               
c     Purpose:  To determine the QMMM energy
c
c     Note:     Due to historic reasons, the (capped) QM system was called 
c               the (capped) small system, in line with the IMOHC convention.
c 
c               Now we decide that the (capped) QM system should 
c               be called the (capped) primary system, and the MM part 
c               should be called secondary system.    
c
c               The large system is the (primary + background-charge) system.
c               The entire system is the real system.
c
c               We however decided not to change the name of the variables,
c               in order to avoid extensive change and bugs!
c
c     Called by:  mlehook, optehk
c     Calls:  progmmehk, progqmehk, qmmmdisp, progmm0ehk
c  
      use caltyp
      use input,  only : cartgeom, mmvalen, qmmminf
      use files,  only : outfile
      use energy, only : qmmmener, qmes, mmes
      implicit none
c
c i/o variables
c
      character*10       qmprognme, mmprognme
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (qmmminf)  :: qmmminp
      integer            ierr
c
c local variables
c      
      type (cartgeom) :: ssgeom, bgchgeom
      type (mmvalen)  :: ssmmgeom
      integer            lsnatm, ssnatm, bgchnum,i
      double precision   px, qmesis, qmesas
c      double precision   mmst
c      double precision  r, theta, phi, dumg(12), dumh(12,12)

c
c  Save the prnprm switch value
c
      ssmmgeom%lprnatmprm = mmgeom%lprnatmprm
c     
c     The program can not calculate the QM/MM single-point energy
C     using Gamess with electric embedding
c	revised 5/22/2006 by Yan Zhang
c
      if((qmprognme(:6).eq.'gamess').
     $	and.(qmmminp%embed .ne. 'mechanical')) then
	  write(outfile,*) 'The program can not calculate',
     $  ' the QM/MM single-point energy using Gamess with electric ',
     $			'embedding.'
         stop 'qmmmehk 1'
      end if
c
c    end 5/22/2006 by Yan Zhang
C
c
c  MM calculation on the entrie system
c      
      write(outfile,100)
      write(outfile,110)
      tcssmmkey = 1
      if (qmmminp%embed .eq. 'mechanical') then
         call progmmehk(mmprognme, qmmminp%mmmeth, qmmminp%mmbas, 
     *      geom, mmgeom, qmmminp%mmoptl, mmes(2), ierr)
      else
         call progmm0ehk(mmprognme, qmmminp%mmmeth, qmmminp%mmbas,
     *      geom, mmgeom, qmmminp%mmoptl, mmes(2), ierr, qmmminp)
      endif	
      write(outfile,115) mmes(2)

100   format(80('-'))
110   format(/2x,'MM calculation on the entire system:',/)
115   format('MM energy (a.u.) for the entire system:',f20.10)      

c                     
c  Assign the appropriate geometries to ssgeom
c                     
      write(outfile,100)
      write(outfile,120)
      call qmmmssgeom(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)           
     
120   format(' Construct the capped primary system:')

c
c  MM calculation on the primary system, no point charges import
c
      write(outfile,100)
      write(outfile,130)
      tprm = 0
      tcssmmkey = 2
      if (qmmminp%embed .eq. 'mechanical') then      
         call progmmehk(mmprognme, qmmminp%mmmeth, qmmminp%mmbas, 
     *      ssgeom, ssmmgeom, qmmminp%mmoptl, mmes(1), ierr)
      else
         call progmm0ehk(mmprognme, qmmminp%mmmeth, qmmminp%mmbas, 
     *      ssgeom, ssmmgeom, qmmminp%mmoptl, mmes(1), ierr, qmmminp)	       
      endif
      write(outfile,135) mmes(1)
      
130   format(//2x,'MM calculation on the capped primary system',
     *   ' without background charges:',/)
135   format(/2x,'MM energy (a.u.) for capped primary system:',f20.10)

c
c  Work on background charges
c
      if (qmmminp%embed .eq. 'mechanical') goto 2000
      
      write(outfile,100)            
      write(outfile,1000)
1000  format(//2x,'Treatment for the background point charges:')
      
c
c  Scale M1, M2 charges
c      
      call bgchscale(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
      
      if ((qmmminp%bordercharge .eq. 'scale')) goto 3000
      
c
c  Shift/redistribute M1 charges
c     
      call bgchshift(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
c
c  Embed the primary system with backgound charges
c
3000  call qmmmbgchgeom(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
c
c  EE polarized charge calculation with the QEQ or EEM method
c  8/8/2006 by Yan Zhang                                                           
c
      call polareech(qmprognme, mmprognme, geom, mmgeom, 
     &                     ssgeom, ssmmgeom, qmmminp)

c
c  end 8/8/2006 by Yan Zhang
c
c  Add by Yan Zhang 8/31/2007
c  Flexible-boundary charge calculation
c
      if(qmmminp%iflexbound .eq. 1) then
         call fbbpc(geom, mmgeom, ssgeom, ssmmgeom, qmmminp, 
     *    qmprognme, px) 
      endif
c
c  End by Yan Zhang 8/31/2007
c
c  Electricstatic interactions within background charges
c
c    add 5/25/2006 by Yan Zhang
c  The single-point energy with background point charge calculated by ORCA
C  does not include the electricstatic interactions between the background 
C  point charge. It is not to subtract the electricstatic interactions from
C  the total energy.
c
      if(qmprognme.eq.'orca') goto 2000
c
c     end 5/25/2006 by Yan Zhang
c
      write(outfile,100)
      write(outfile,400)
400   format(/2x,'Electricstatic interactions within ',
     *  'the background charges:',/)
     
      tcssmmkey = 2 
      call progmmstehk(mmprognme, qmmminp%mmmeth, qmmminp%mmbas, 
     *  ssgeom, ssmmgeom, qmmminp%mmoptl, mmes(3), ierr, qmmminp)

      write(outfile,435) mmes(3)
      mmes(1) = mmes(1) + mmes(3)
435   format(/2x,'Additional MM energy (a.u.):',t50,f20.10,/2x,
     *  '(due to electricstatic interactions within',/2x,
     *  ' background point charges)')

c
2000  continue      
c
c  QM or QM* calculation on the primary system
c
      write(outfile,100)
      write(outfile,140)

c
c for flexible boundary treatment added by Yan Zhang, Apr. 15, 2009
c
      if(qmmminp%iflexbound .eq. 1) goto 5000
c
c end Apr. 15, 2009
c

      if (qmmminp%embed .eq. 'mechanical') then      
         call progehk(qmprognme, qmmminp%qmmeth, qmmminp%qmbas,
     *           qmmminp%qmcharge, qmmminp%qmmult,	    
     *           ssgeom, qmmminp%qmoptl, qmes(1), ierr)
      else
         call progesthk(qmprognme, qmmminp%qmmeth, qmmminp%qmbas,
     *     qmmminp%qmcharge, qmmminp%qmmult,
     *     ssgeom, qmmminp%qmoptl, qmes(1), ierr, qmmminp, ssmmgeom)	 
      endif
      
      write(outfile,145) qmes(1)
140   format(/2x,'QM calculation on the capped primary system:',/)
145   format('QM energy (a.u.) for capped primary system:',f20.10)      

c
5000  continue

c
c for flexible boundary treatment added by Yan Zhang, Apr. 15, 2009
c
      if(qmmminp%iflexbound .eq. 1) then
         call progesthk(qmprognme, qmmminp%qmmeth, qmmminp%qmbas,
     *     qmmminp%qmcharge, qmmminp%qmmult,
     *     ssgeom, qmmminp%qmoptl, qmesis, ierr, qmmminp, ssmmgeom)
         write(outfile,9001) qmesis
         call progesthk(qmprognme, qmmminp%qmmeth, qmmminp%qmbas,
     *     qmmminp%asqmcharge, qmmminp%asqmmult,
     *     ssgeom, qmmminp%qmoptl, qmesas, ierr, qmmminp, ssmmgeom)
         write(outfile,9002) qmesas
         qmes(1)=qmesis*(1.d0-px)+qmesas*px
         write(outfile,9003) qmes(1)
      endif
9001  format('CPS* energy of initial state:',f16.10)
9002  format('CPS* energy of added state:',f16.10)
9003  format('final CPS* energy:',f16.10)
c end Apr. 15, 2009
c
c  Calculate the QMMM energy
c
      write(outfile,100)
      write(outfile,9000)            
      qmmmener = qmes(1) - mmes(1) + mmes(2)
      write(outfile,9100) qmmmener, qmes(1), mmes(1), mmes(2)
      
9000  format(/,'Display of QMMM energy (a.u.):',/2x,'QMMM      ',
     *   10x,'QM;CPS**',10x,'MM;CPS*',10x,'MM;ES') 
9100  format(4(f16.10,2x))      

      return
      end
c 
c******************************************************************************
c  QMMMSSGEOM
c******************************************************************************
c
      subroutine qmmmssgeom(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *            qmmminp)

c
c     Subroutine qmmmssgeom
c
c     Created:  Feb 22, 2004 by Hai Lin
c     Revised:  Jun. 28, 2010 by Soroosh Pezeshki
c     Purpose:  To assign appropriate geometries to lsgeom and ssgeom
c
c     Called by:  qmmmehk, qmmmghk, qmmmhhk
c     Calls:
c
      use input
      use atminfo
      use caltyp, only : tdynamics, tdebug                              0628SP10
      use files, only : outfile
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom, bgchgeom
      type (mmvalen)  :: mmgeom, ssmmgeom
      type (qmmminf)  :: qmmminp
c
c local variables
c      
      integer  capnatm, qmnatm, mmnatm, bgchnatm
      integer  ssnatm, cssnatm      
      integer  i,j,k, ii,jj,kk
      integer  nqmcon, allcon, qmcon
     
      double precision    bs1, bs2, xq,yq,zq, xm,ym,zm, xc,yc,zc
      double precision    dr                                            0621BW14      
c      double precision, parameter :: angtoau = 0.52918d0
      double precision, parameter :: angtoau = 0.529177249d0
            
c
c  prepare atom numbers
c  
      capnatm = qmmminp%capatm%capnatm
      qmnatm  = qmmminp%qmnatm
      mmnatm  = qmmminp%mmnatm
      cssnatm = qmnatm + capnatm
c
c  Fill with charge and multiplicity
c      
      ssgeom%charge = geom%charge
      ssgeom%mult   = geom%mult
c
c  Fill with the QM atoms
c     
      ssnatm  = qmnatm
      ssgeom%natoms = ssnatm
      ssmmgeom%natoms = ssnatm
                   
      do i=1, qmnatm
         k = qmmminp%qmatm(i)

         ssgeom%atm(i)          = geom%atm(k)
         ssgeom%atmmss(i)       = geom%atmmss(k)
         ssgeom%angcoord(3*i-2) = geom%angcoord(3*k-2)
         ssgeom%angcoord(3*i-1) = geom%angcoord(3*k-1)
         ssgeom%angcoord(3*i)   = geom%angcoord(3*k)

         ssmmgeom%atmid(i)      = mmgeom%atmid(k)
         ssmmgeom%atmtyp(i)     = mmgeom%atmtyp(k)
         ssmmgeom%atmtypname(i) = mmgeom%atmtypname(k)
         ssmmgeom%atmcharge(i)  = mmgeom%atmcharge(k)
         ssmmgeom%atmvalen(i)   = mmgeom%atmvalen(k)

c         write(*,*) i,k, ssmmgeom%atmcharge(i), mmgeom%atmcharge(k)
c         write(*,*) i,k, ssmmgeom%atmtypname(i), mmgeom%atmtypname(k)
     
         jj = ssmmgeom%atmvalen(i)
         do ii=1, jj
            ssmmgeom%atmvalenlist(i,ii) = mmgeom%atmvalenlist(k,ii)

c            write(*,*) i,k, ssmmgeom%atmvalenlist(i,ii), 
c     *                       mmgeom%atmvalenlist(k,ii)
         enddo
         
      enddo
c
c  Add the Cap atoms after QM atoms
c  
      ssnatm  = qmnatm + capnatm
      ssgeom%natoms = ssnatm
      ssmmgeom%natoms = ssnatm
          
      do i=1, capnatm
         j = i + qmnatm
         k = qmmminp%capatm%capatm(i)

c  Add by Bo Wang 0621BW14 
c  set the link atom  the same as the first letter
c   or the first two letters in link atom type
         if (qmmminp%capatm%capatmtypname(i)(1:1) .eq. 'h' .or.
     *      qmmminp%capatm%capatmtypname(i)(1:1) .eq. 'H') then
            ssgeom%atm(j)    = 'H'
            ssgeom%atmmss(j) = ams(1)
         elseif (qmmminp%capatm%capatmtypname(i)(1:1) .eq. 'f' .or.
     *      qmmminp%capatm%capatmtypname(i)(1:1) .eq. 'F') then
            ssgeom%atm(j)    = 'F'
            ssgeom%atmmss(j) = ams(9)
         elseif (qmmminp%capatm%capatmtypname(i)(1:2) .eq. 'CL' .or.
     *      qmmminp%capatm%capatmtypname(i)(1:2) .eq. 'Cl') then
            ssgeom%atm(j)    = 'Cl'
            ssgeom%atmmss(j) = ams(17)
         elseif (qmmminp%capatm%capatmtypname(i)(1:1) .eq. 'c' .or.
     *      qmmminp%capatm%capatmtypname(i)(1:1) .eq. 'C') then
            ssgeom%atm(j)    = 'C'
            ssgeom%atmmss(j) = ams(6)
         endif
c         ssgeom%atm(j)    = capname
c         ssgeom%atmmss(j) = ams(capams)
c  End by Bo Wang 0621BW14 

         ssmmgeom%atmid(j)      = k
         ssmmgeom%atmtyp(j)     = qmmminp%capatm%capatmtyp(i)
         ssmmgeom%atmtypname(j) = qmmminp%capatm%capatmtypname(i)
         ssmmgeom%atmvalen(j)   = 1
         qmcon                  = qmmminp%capatm%capatmqm(i)
         ssmmgeom%atmvalenlist(j,1) = qmcon
     
cx         bs1 = qmmminp%capatm%capbondscale(i)                         0621BW14 revise to also implement fix bond method
cx         bs2 = 1.0d0 - bs1                                            0621BW14
         
      if ((tdebug.eq.1).or.(tdynamics.eq.0)) then                       0628SP10
        write (outfile,200) k, ssmmgeom%atmtypname(j),
     *      k, mmgeom%atmtypname(k),
     *      qmcon, mmgeom%atmtypname(qmcon)
200      format(/1x,'Cap atom ',i4,' (',a4,') replaces MM atom ',
     *      i4,'(',a4,') bonded to QM atom ',i4,'(',a4,').')
      endif                                                             0628SP10
c
c  Compute the cap atom coordinates
c    
         xm = geom%angcoord(3*k-2)
         ym = geom%angcoord(3*k-1)
         zm = geom%angcoord(3*k)
               
         xq = geom%angcoord(3*qmcon-2)
         yq = geom%angcoord(3*qmcon-1)
         zq = geom%angcoord(3*qmcon)

c Add by Bo Wang 0621BW14
c prepare the scale factor used to determine the link atom coord
         if (qmmminp%capatm%capconstrain .eq. 'scale') then
           bs1 = qmmminp%capatm%capbondscale(i)
         elseif (qmmminp%capatm%capconstrain .eq. 'fix') then
           dr= sqrt ((xm-xq)**2+(ym-yq)**2+(zm-zq)**2)
           bs1 = qmmminp%capatm%capbondfix(i)/dr
         endif
         bs2 = 1.0d0 - bs1
c End by Bo Wang 0621BW14
               
         xc = bs2 * xq + bs1 * xm
         yc = bs2 * yq + bs1 * ym
         zc = bs2 * zq + bs1 * zm
                 
         ssgeom%angcoord(3*j-2) = xc
         ssgeom%angcoord(3*j-1) = yc
         ssgeom%angcoord(3*j)   = zc

      if ((tdebug.eq.1).or.(tdynamics.eq.0)) then                       0628SP10
         write (outfile,300) bs1, xm, ym, zm, xq, yq, zq, xc, yc, zc
300      format(/3x,'Scale bondlength by ',f12.8,
     *     /3x,'MM  coordinates (ang):',3(f12.8),
     *     /3x,'QM  coordinates (ang):',3(f12.8), 
     *     /3x,'Cap coordinates (ang):',3(f12.8)) 
      endif                                                             0628SP10
                                                                 
      enddo
c
c  Calculate the coordinates in AU
c  
      ssgeom%aucoord(:ssnatm*3) =  ssgeom%angcoord(:ssnatm*3)/angtoau
c
      return
      end
c
c******************************************************************************

c******************************************************************************
c  BGCHSCALE
c******************************************************************************
c
      subroutine bgchscale(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *            qmmminp)

c
c     Subroutine bgchscale
c
c     Created:  Feb 22, 2004 by Hai Lin
c     Revised:  Jun. 28, 2010 by Soroosh Pezeshki
c     Purpose:  To scale the MM charge near the QM/MM border
c               and assign them the cap atom type
c
c     Called by:  qmmmehk, qmmmghk, qmmmhhk
c     Calls:
c
      use input
      use atminfo
      use caltyp, only : tdynamics, tdebug                              0628SP10
      use files, only : outfile
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom, bgchgeom
      type (mmvalen)  :: mmgeom, ssmmgeom
      type (qmmminf)  :: qmmminp
c
c local variables
c      
      integer  capnatm, qmnatm, mmnatm, bgchnatm
      integer  ssnatm      
      integer  i,j,k, ii,jj,kk
      integer  nqmcon, allcon, qmcon
      integer  cssnatm, m1st, m2nd, lsnatm
      integer  m3rd
      
      integer             chscale
      double precision    chscale1, chscale2, chscale3
      double precision    charge
      double precision    bs1, bs2, xq,yq,zq, xm,ym,zm, xc,yc,zc
      
c      double precision, parameter :: angtoau = 0.52918d0
      double precision, parameter :: angtoau = 0.529177249d0
c
c  prepare the charge scale factor
c
      if (qmmminp%bordercharge .eq. 'scale') then
         chscale  = 1    
         chscale1 = qmmminp%chscale%chargescale(1)
         chscale2 = qmmminp%chscale%chargescale(2)    
         chscale3 = qmmminp%chscale%chargescale(3)  
      else
         chscale  = 0
         chscale1 = 0.0d0
         chscale2 = 1.0d0
         chscale3 = 1.0d0            
      endif  
c
c  prepare atom numbers
c  
      capnatm = qmmminp%capatm%capnatm
      qmnatm  = qmmminp%qmnatm
      mmnatm  = qmmminp%mmnatm
      ssnatm  = ssgeom%natoms 
      cssnatm = capnatm + qmnatm 
      lsnatm   = capnatm + qmnatm + mmnatm
      m1st = qmmminp%mm1stlay%natm1stlay
      m2nd = qmmminp%mm2ndlay%natm2ndlay                  
      m3rd = qmmminp%mm3rdlay%natm3rdlay
      
c
c  Adding MM (excluding M1) atoms after QM and Cap atoms
c
      if (tdynamics.ne.1) then                                          0628SP10
        write(outfile,50)
50      format(///2x,'Now scale the M1 and M2 charges:',//2x,
     *   'For bordercharge = scale, scale factors can be modified',
     *   ' by user;',/2x, 
     *   'For other options, scale factors are fixed to 0.0 ',
     *   'for M1 and 1.0 for M2;',/2x,
     *   'In particular, M2 charge may be further modified',
     *   ' if bordercharge = shift.',/)
      endif                                                             0628SP10
      
      ssnatm = cssnatm + mmnatm
      ssgeom%natoms = ssnatm
      ssmmgeom%natoms = ssnatm

      do i=1, mmnatm
         j = i + cssnatm
         k = qmmminp%mmatm(i)

         ssgeom%atm(j)          = geom%atm(k)
         ssgeom%atmmss(j)       = geom%atmmss(k)
         ssgeom%angcoord(3*j-2) = geom%angcoord(3*k-2)
         ssgeom%angcoord(3*j-1) = geom%angcoord(3*k-1)
         ssgeom%angcoord(3*j)   = geom%angcoord(3*k)

         ssmmgeom%atmid(j)      = mmgeom%atmid(k)
         ssmmgeom%atmtyp(j)     = mmgeom%atmtyp(k)
         ssmmgeom%atmtypname(j) = mmgeom%atmtypname(k)
         ssmmgeom%atmcharge(j)  = mmgeom%atmcharge(k)
         ssmmgeom%atmvalen(j)   = 0  
c
c  Scale charge if M2
c    
      do ii=1,m2nd
        jj = qmmminp%mm2ndlay%atm2ndlay(ii)
        if (k .eq. jj) then
            charge = ssmmgeom%atmcharge(j) * chscale2
            ssmmgeom%atmcharge(j) = charge                  
            qmmminp%mm2ndlay%atm2ndlaycharge(ii) = charge
            if   ((tdebug.eq.1).or.(tdynamics.eq.0)) then               0628SP10
                write(outfile,100) jj, mmgeom%atmcharge(k), charge
100             format('M2 atom ',i4,': old charge ',f16.8,
     *           '  scaled charge ',f16.8)         
            endif                                                       0628SP10
        endif
      enddo    
c
c  Scale charge if M3
c    
      do ii=1,m3rd
        jj = qmmminp%mm3rdlay%atm3rdlay(ii)
        if (k .eq. jj) then
           charge = ssmmgeom%atmcharge(j) * chscale3
           ssmmgeom%atmcharge(j) = charge
           qmmminp%mm3rdlay%atm3rdlaycharge(ii) = charge
           if   ((tdebug.eq.1).or.(tdynamics.eq.0)) then                0628SP10
               write(outfile,110) jj, mmgeom%atmcharge(k), charge
110            format('M3 atom ',i4,': old charge ',f16.8,
     *           '  scaled charge ',f16.8)
           endif                                                        0628SP10
        endif
      enddo         
      enddo      
c              
c  Fill the coordinates in AU
c
      ssgeom%aucoord(cssnatm*3+1:ssnatm*3) =
     *   ssgeom%angcoord(cssnatm*3+1:ssnatm*3)/angtoau                 
c
c  Scale M1 charges (but not yet add to small system)
c      
c       write(outfile,*)
      if ((tdebug.eq.1).or.(tdynamics.eq.0)) write(outfile,*)           0628SP10
      do i=1, m1st
         j = qmmminp%mm1stlay%atm1stlay(i)
         charge = mmgeom%atmcharge(j) * chscale1     
         qmmminp%mm1stlay%atm1stlaycharge(i) = charge
         if     ((tdebug.eq.1).or.(tdynamics.eq.0)) then                0628SP10
            write(outfile,200) j,mmgeom%atmcharge(j),charge
200         format('M1 atom ',i4,': old charge ',f16.8,
     *           '  scaled charge ',f16.8)   
         endif                                                          0628SP10
      enddo
c            
      return     
      end

c******************************************************************************
c  BGCHSHIFT
c******************************************************************************
c
      subroutine bgchshift(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *            qmmminp)

c
c     Subroutine bgchshift
c
c     Created:  Feb 22, 2004 by Hai Lin
c     Revised:  Jun. 28, 2010 by Soroosh Pezeshki
c     Purpose:  To shift the M1 charge to/toward M2 atoms
c               Add dipole as required in bordercharge = shift and 
c               redist2.
c
c     Called by:  qmmmehk, qmmmghk, qmmmhhk
c     Calls:
c
      use input
      use atminfo
      use caltyp, only : tdynamics, tdebug                              0628SP10
      use files, only : outfile
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom, bgchgeom
      type (mmvalen)  :: mmgeom, ssmmgeom
      type (qmmminf)  :: qmmminp
c
c local variables
c      
      integer  capnatm, qmnatm, mmnatm, bgchnatm
      integer  ssnatm      
      integer  i,j,k, ii,jj,kk
      integer  l,j_min                                                  0621BW14 to determine the nearest M2 atom
      integer  nqmcon, allcon, qmcon
      integer  cssnatm, m1st, m2nd, lsnatm
      
      integer             chargemode
      integer             id1, id2, m2, m2num
      integer             m2numx, m2numx1st                             0808XW18
      integer             id0                                           0621BW14
      integer             m3num, mmnum, mmatm,m3                        0621BW14
      integer             presigmmnum, sigmmnum                         0921XW17
      integer             sw                                            0808XW18

      double precision    dipposit, dipdist, chposit
      double precision    charge1, charge2, charge0
      double precision    x1, y1, z1, x2, y2, z2
      double precision    bs1, bs2, bs1m, bs2m, bs1p, bs2p
      double precision    x, y, z, q, qd, qr3
      double precision    charge3, charge4, mmcharge, mm1charge         0621BW14
      double precision    qd2                                           0621BW14
      double precision    dist, dist_min                                0621BW14

      
c      double precision, parameter :: angtoau = 0.52918d0
      double precision, parameter :: angtoau = 0.529177249d0
c
c  prepare atom numbers
c  
      capnatm = qmmminp%capatm%capnatm
      qmnatm  = qmmminp%qmnatm
      mmnatm  = qmmminp%mmnatm
      ssnatm  = ssgeom%natoms 
      cssnatm = capnatm + qmnatm 
      lsnatm  = capnatm + qmnatm + mmnatm
      m1st = qmmminp%mm1stlay%natm1stlay
      m2nd = qmmminp%mm2ndlay%natm2ndlay                  

c
c  prepare the distance scale factor
c
      chposit  = qmmminp%chredist%chposit
      dipposit = qmmminp%chredist%dipposit
      dipdist  = qmmminp%chredist%dipdist

      if ((qmmminp%bordercharge .eq. 'shift' )       .or.
     *     (qmmminp%bordercharge .eq. 'shiftbal' )   .or.               0621BW14
     *     (qmmminp%bordercharge .eq. 'shiftbal2' ))     then           0621BW14
         chargemode  = 2     
         dipposit    = 1.0d0
         chposit     = 1.0d0
         qmmminp%chredist%nredist = m2nd * 2
      elseif ((qmmminp%bordercharge .eq. 'redist1') .or.
     *   (qmmminp%bordercharge .eq. 'rc')           .or.
     *   (qmmminp%bordercharge .eq. 'rcd')          .or.
     *   (qmmminp%bordercharge .eq. 'redist3')      .or.
     *   (qmmminp%bordercharge .eq. 'rcbal')        .or.                0621BW14
     *   (qmmminp%bordercharge .eq. 'rcdbal') ) then                    0621BW14
         chargemode  = 3
         qmmminp%chredist%nredist = m2nd 
c add by Bo Wang 0621BW14
      elseif ((qmmminp%bordercharge .eq. 'rcbal2')  .or. 
     *   (qmmminp%bordercharge .eq. 'rcbal3')   .or.
     *   (qmmminp%bordercharge .eq. 'amber2')   .or.
     *   (qmmminp%bordercharge .eq. 'amber1') ) then
         chargemode  = 4
         chposit     = 1.0d0
         qmmminp%chredist%nredist = 0
      elseif (qmmminp%bordercharge .eq. 'seebal') then
         chargemode  = 5
         qmmminp%chredist%nredist = 1
c end by Bo Wang 0621BW14
      else
         chargemode  = 7                                                0621BW14
         qmmminp%chredist%nredist = m2nd * 3     
      endif  
      if        ((tdebug.eq.1).or.(tdynamics.eq.0)) then                0628SP10
         write(outfile,100)
100      format(///2x,'Generate auxiliary charges:'/)
      endif                                                             0628SP10
c
c  Generate auxiliary charges
c
      bs1  = chposit 
      bs2  = 1.0d0    - bs1
      bs1m = dipposit - dipdist * 0.5d0
      bs2m = 1.0d0    - bs1m
      bs1p = dipposit + dipdist * 0.5d0
      bs2p = 1.0d0    - bs1p
         
      m2 = 0      
      m3 = 0                                                            0621BW14 number of M3 atoms
c
      presigmmnum = 0                                                   0921XW17
      sigmmnum = 0                                                      0921XW17
      sw = 0                                                            0808XW18
c
c modified by Xin-Ping Wu, August 8, 2018
c for amber-2 with one charge balancing group and multiple M1 atoms
      if ((qmmminp%nbalgroup .eq. 1) 
     *     .and. (m1st .gt. 1)
     *     .and. (qmmminp%bordercharge .eq. 'amber2')) then
         sw = qmmminp%nbalgroup
      else
         sw = m1st
      endif

      do i=1, sw
c      do i=1, m1st
c end by Xin-Ping Wu, August 8, 2018
c
         id1     = qmmminp%mm1stlay%atm1stlay(i)
         x1      = geom%angcoord(3*id1-2)
         y1      = geom%angcoord(3*id1-1)
         z1      = geom%angcoord(3*id1)
         charge1 = mmgeom%atmcharge(id1)
c
c modified by Xin-Ping Wu, August 8, 2018
c for rcbal,rcdbal,rcbal2, and rcbal3 with one charge balancing group and multiple M1 atoms
c         m2num   = qmmminp%mm1stlay%atm1stlaym2num(i)
         if ((qmmminp%nbalgroup .eq. 1)
     *        .and. (m1st .gt. 1))     then
             if ((qmmminp%bordercharge .eq. 'rcbal')
     *        .or. (qmmminp%bordercharge .eq. 'rcdbal')
     *        .or. (qmmminp%bordercharge .eq. 'rcbal2')
     *        .or. (qmmminp%bordercharge .eq. 'rcbal3')) then
                m2num   = qmmminp%mm2ndlay%natm2ndlay
             endif
         else 
             m2num   = qmmminp%mm1stlay%atm1stlaym2num(i)
         endif
c end by Xin-Ping Wu, August 8, 2018
c
         m3num   = qmmminp%mm3rdlay%natm3rdlay                          0621BW14
         charge0 = charge1 / m2num                               

         qd      = charge0 * (chposit / dipdist)
         qr3     = charge0 * chposit / (1.0d0 - chposit )
     
c add by Bo Wang 0621BW14
c   Calculate the balancing charges (rcbal,rcdbal,rcbal2 rcbal3, amber2
c         amber1,shiftbal2) can be applied for multiple M1 atoms,but
c       need the input from "BALGROUP"
         if (qmmminp%lchgbal .eq. 1) then
            mmcharge=0
c     group i, total groups should be the same as number of M1 atoms
            mmnum = qmmminp%balgroupn(i)
            do j=2,mmnum
             mmatm=qmmminp%balgroup(i,j)
             mmcharge=mmcharge+mmgeom%atmcharge(mmatm)
            enddo
            mm1charge=qmmminp%groupchg(i) - mmcharge
c  calculate the redistributed charges for different balanced charge schemes
            charge2 =  mm1charge / m2num           !for rcbal and rcbal2
            charge3 =  mm1charge / (m2num + m3num) !for rcbal3
c
c modified by Xin-Ping Wu, August 8, 2018
c for amber-2 with one charge balancing group and multiple M1 atoms
c
cc bug fixed by Xin-Ping Wu, Sept. 21, 2017
cc                charge4 =  mm1charge / (mmnum - 1)     !for amber2
cc            charge4 =  mm1charge / mmnum           !for amber2
cc end by Xin-Ping Wu, Sept. 21, 2017
c
            if ((qmmminp%nbalgroup .eq. 1) .and. (m1st .ge. 1)) then
                charge4 =  mm1charge / (mmnum - m1st)     !for amber2
            elseif ((qmmminp%nbalgroup .gt. 1)
     *              .and. (m1st .eq. qmmminp%nbalgroup)) then
                charge4 =  mm1charge / (mmnum - 1)     !for amber2
            elseif ((qmmminp%nbalgroup .gt. 1)
     *              .and. (m1st .ne. qmmminp%nbalgroup)) then
                write(outfile,*)
     *     'ERROR: Only one M1 atom is allowed to appear in each group.'
                stop ' in subroutine bgchshift'
            endif
c
c end by Xin-Ping Wu, August 8, 2018
c
            qd2     = charge2 * (chposit / dipdist)!dipole for balance_shift (Sherwood)
         endif
c    end by Bo Wang 0126BW13


         if      ((tdebug.eq.1).or.(tdynamics.eq.0)) then               0628SP10
            write(outfile,1000) id1,charge1,x1,y1,z1
1000        format (//,'M1:   ',i5,2x,'charge: ',f8.4,2x,
     *     'XYZ:  ',3(f12.8,2x))
        endif                                                           0628SP10

c
c modified by Xin-Ping Wu, August 8, 2018         
c         do j=1, m2num
         if ((qmmminp%bordercharge .eq. 'rcbal')
     *       .or. (qmmminp%bordercharge .eq. 'rc')
     *       .or. (qmmminp%bordercharge .eq. 'redist1')
     *       .or. (qmmminp%bordercharge .eq. 'rcd')
     *       .or. (qmmminp%bordercharge .eq. 'redist3')
     *       .or. (qmmminp%bordercharge .eq. 'rcdbal')) then 
             m2numx = qmmminp%mm1stlay%atm1stlaym2num(i)  
               if (i .eq. 1) then
                  m2numx1st = m2numx
               endif
         elseif ((qmmminp%bordercharge .eq. 'rcbal2')
     *       .or. (qmmminp%bordercharge .eq. 'rcbal3')
     *       .or. (qmmminp%bordercharge .eq. 'amber2')) then
               if ((qmmminp%nbalgroup .eq. 1)
     *           .and. (m1st .gt. 1))     then
                   m2numx = qmmminp%mm2ndlay%natm2ndlay 
               else
                   m2numx = qmmminp%mm1stlay%atm1stlaym2num(i)
               endif
         endif
         do j=1, m2numx
c end by Xin-Ping Wu, August 8, 2018
c
            m2 = m2 + 1
            qmmminp%chredist%q(m2)  = charge0
            qmmminp%chredist%qd(m2) = qd
c add by Bo Wang 0621BW14  ? need revision for more link atoms
            if (qmmminp%bordercharge .eq. 'rcbal3') then    
              qmmminp%chredist%qb(m2)  = charge3
            else
              qmmminp%chredist%qb(m2)  = charge2
            endif
            qmmminp%chredist%qd2(m2) = qd2
c end by Bo Wang 0621BW14

            id2     = qmmminp%mm1stlay%atm1stlaym2(i,j)
            x2      = geom%angcoord(3*id2-2)
            y2      = geom%angcoord(3*id2-1)
            z2      = geom%angcoord(3*id2)

            if      ((tdebug.eq.1).or.(tdynamics.eq.0)) then            0628SP10
                write(outfile,2000) id2,x2,y2,z2
2000            format ('M2:   ',i5,2x,'charge: ',8x,2x,
     *          'XYZ:  ',3(f12.8,2x))
            endif                                                       0628SP10
     
            x   = bs2 * x1 + bs1 * x2
            y   = bs2 * y1 + bs1 * y2
            z   = bs2 * z1 + bs1 * z2
            qmmminp%chredist%x(m2) = x
            qmmminp%chredist%y(m2) = y
           qmmminp%chredist%z(m2) = z
            if      ((tdebug.eq.1).or.(tdynamics.eq.0)) then            0628SP10
                write(outfile,3000) m2,charge0,x,y,z
3000            format ('Q:    ',i5,2x,'charge: ',f8.4,2x,
     *          'XYZ:  ',3(f12.8,2x))
            endif                                                       0628SP10

            x   = bs2m * x1 + bs1m * x2
            y   = bs2m * y1 + bs1m * y2
            z   = bs2m * z1 + bs1m * z2
            qmmminp%chredist%mx(m2) = x
            qmmminp%chredist%my(m2) = y
            qmmminp%chredist%mz(m2) = z
            if      ((tdebug.eq.1).or.(tdynamics.eq.0)) then            0628SP10
                write(outfile,4000) m2,qd,x,y,z
4000            format ('Dip-: ',i5,2x,'charge: ',f8.4,2x,
     *          'XYZ:  ',3(f12.8,2x))
            endif                                                       0628SP10
     
            x   = bs2p * x1 + bs1p * x2
            y   = bs2p * y1 + bs1p * y2
            z   = bs2p * z1 + bs1p * z2
            qmmminp%chredist%px(m2) = x
            qmmminp%chredist%py(m2) = y
            qmmminp%chredist%pz(m2) = z
            if     ((tdebug.eq.1).or.(tdynamics.eq.0)) then             0628SP10
                write(outfile,5000) m2,-qd,x,y,z
5000            format ('Dip+: ',i5,2x,'charge: ',f8.4,2x,
     *          'XYZ:  ',3(f12.8,2x))
            endif                                                       0628SP10
         enddo
c
c add by Xin-Ping Wu, August 8, 2018 
       if ((qmmminp%nbalgroup .eq. 1)
     *     .and. (m1st .gt. 1))  then
         if (m2 .eq. m2num) then
           if ((qmmminp%bordercharge .eq. 'rcbal')
c     *       .or. (qmmminp%bordercharge .eq. 'rc')
c     *       .or. (qmmminp%bordercharge .eq. 'rcd')
     *       .or. (qmmminp%bordercharge .eq. 'rcdbal')) then
              if (m2numx .lt. m2num) then
                 m2 = m2numx1st    
                 do j=1, (m2num-m2numx1st)
                  m2 = m2 + 1
c                  qmmminp%chredist%q(m2)  = charge0
c                  qmmminp%chredist%qd(m2) = qd
                  qmmminp%chredist%qb(m2)  =
     *                  qmmminp%chredist%qb(m2-1) 
                  qmmminp%chredist%qd2(m2) = qd2
                 enddo
              endif
           endif
         endif
       endif
c end by Xin-Ping Wu, August 8, 2018
c
c add by Bo Wang 0621BW14
c  Redistribute the charges to MM atoms for Amber 1, 
c  balanced RC3 ,Amber 2, and Balanced SEE 
c  only work for 1 link atom.
         if (qmmminp%bordercharge .eq. 'amber1') then   
            j_min=1
            do j=1, m2num
               id2     = qmmminp%mm1stlay%atm1stlaym2(i,j)
                x2      = geom%angcoord(3*id2-2)
                y2      = geom%angcoord(3*id2-1)
                z2      = geom%angcoord(3*id2)
                dist    = sqrt ((x1-x2)**2+(y1-y2)**2+(z1-z2)**2)
                if (j==1) then
                    j_min=j
                    dist_min=dist
                elseif (dist .lt. dist_min) then
                    j_min=j
                    dist_min=dist
                endif
             enddo
             qmmminp%chredist%qc(j_min) =  mm1charge
c Balance_RC3, redistribute the charges to M2, M3 atoms
         elseif (qmmminp%bordercharge .eq. 'rcbal3') then
            do j=1,m3num
              m3 = m3 + 1
              qmmminp%chredist%qc(m3)  = charge3
            enddo
c
c Amber-2, redistribute the charges to all atoms
c         elseif (qmmminp%bordercharge .eq. 'amber2') then
c            do j=1,mmnum
c              qmmminp%chredist%qc(j)  = charge4
c            enddo
c
c modified by Xin-Ping Wu, August 8, 2018
c for amber-2 with one charge balancing group and multiple M1 atoms
c
cc added by Xin-Ping Wu, Sept. 21, 2017
cc Amber-2, redistribute the charges to all atoms,
cc except the M1 atoms
cc works for multiple balgroups
cc
cc         elseif (qmmminp%bordercharge .eq. 'amber2') then
cc                presigmmnum = sigmmnum
cc                sigmmnum = sigmmnum + (mmnum - 1)
cc            do j=presigmmnum+1,sigmmnum
cc              qmmminp%chredist%qc(j)  = charge4
cc            enddo
cc end by Xin-Ping Wu, Sept. 21, 2017
c
         elseif (qmmminp%bordercharge .eq. 'amber2') then
            presigmmnum = sigmmnum
            if ((qmmminp%nbalgroup .eq. 1) .and. (m1st .ge. 1)) then
                sigmmnum = sigmmnum + (mmnum - 1)
            elseif ((qmmminp%nbalgroup .gt. 1)
     *              .and. (m1st .eq. qmmminp%nbalgroup)) then
                sigmmnum = sigmmnum + (mmnum - 1)
            elseif ((qmmminp%nbalgroup .gt. 1)
     *              .and. (m1st .ne. qmmminp%nbalgroup)) then
                write(outfile,*) 
     *     'ERROR: Only one M1 atom is allowed to appear in each group.'
                stop ' in subroutine bgchshift'
            endif
            do j=presigmmnum+1,sigmmnum
              qmmminp%chredist%qc(j)  = charge4
            enddo
c
c end by Xin-Ping Wu, August 8, 2018
c
c Balance_SEE, redistribute the charge on M1
         elseif (qmmminp%bordercharge .eq. 'seebal') then
            qmmminp%chredist%qc(i) =  mm1charge
         endif
c end by Bo Wang 0621BW14
     
      enddo
               
      return     
      end

c******************************************************************************
c  QMMMBGCHGEOM
c******************************************************************************
c
      subroutine qmmmbgchgeom(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *   qmmminp)
c
c     Subroutine qmmmbgchgeom
c
c     Created:  Feb 22, 2004 by Hai Lin
c     Revised:  Jun. 28, 2010 by Soroosh Pezeshki
c     Purpose:  To add the M1 and auxiliary 
c               (shifted/redistributed/dipole-mimic)
c               charge near the QM/MM border
c               and assign them the cap atom type
c
c     Called by:  qmmmehk, qmmmghk, qmmmhhk
c     Calls:
c

      use input
      use atminfo
      use caltyp, only : tdynamics, tdebug                              0628SP10
      use files, only : outfile
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom, bgchgeom
      type (mmvalen)  :: mmgeom, ssmmgeom
      type (qmmminf)  :: qmmminp
c
c local variables
c      
      integer  capnatm, qmnatm, mmnatm, bgchnatm
      integer  ssnatm, cssnatm, lsnatm      
      integer  i,j,k, ii,jj,kk
      integer  nqmcon, allcon, qmcon
      integer  m1st, m2nd 
      
      integer             chargemode      
      double precision    charge, charge1, charge2
      double precision    bs1, bs2, xq,yq,zq, xm,ym,zm, xc,yc,zc
      double precision    dpscale1, dpscale2
      
c      double precision, parameter :: angtoau = 0.52918d0
      double precision, parameter :: angtoau = 0.529177249d0
c
c  prepare atom numbers
c  
      capnatm = qmmminp%capatm%capnatm
      qmnatm  = qmmminp%qmnatm
      mmnatm  = qmmminp%mmnatm
      ssnatm  = ssgeom%natoms 
      cssnatm = capnatm + qmnatm 
      lsnatm  = capnatm + qmnatm + mmnatm
      m1st = qmmminp%mm1stlay%natm1stlay
      m2nd = qmmminp%mm2ndlay%natm2ndlay     
c
c  scale factor for redistributed charge to preserve bond dipole
c     used for redist3 
c      
      bs1  = qmmminp%chredist%chposit
      dpscale1 = bs1 / (1.0d0 - bs1)
      dpscale2 = 1.0d0 + dpscale1
c
c  Adding M1 atoms and auxiliary charges after QM, Cap, and MM atoms
c
      if (tdynamics.ne.1) then                                          0628SP10
        write(outfile,50)
50      format(///2x,'Now add the M1 and auxiliary charges ',
     *   'according to bordercharge keyword: ',//2x,
     *   'If scale, only M1 charges will be added;',/2x,
     *   'If shift, dipole-mimic charges will be added,',
     *     ' and M2 charges will be adjusted;',/2x, 
     *   'If redist1/rc, redistributed ',
     *     'charges will be added;',/2x, 
     *   'If redist2/rcd2, redistributed and dipole-mimic ',
     *     'charges will be added;',/2x,
     *   'If redist3/rcd, redistributed charges will be scaled ',
     *     'and added, ',
c  add by Bo Wang 0621BW14
     *     'and M2 charges will be modified;',/2x,
     *   'If seebal, redistributed ',
     *     'charges will be added on M1;',/2x,
     *   'If rcbal, redistributed ',
     *     'charges will be added;',/2x,
     *   'If rcbal2, redistributed',
     *     ' charges will be added on M2 atoms;',/2x,
     *   'If amber1, redistributed',
     *     ' charges will be added on the nearest M2 atom;',/2x,
     *   'If rcbal3, redistributed',
     *     ' charges will be added on M2 and M3 atoms;',/2x,
     *   'If amber2, redistributed',
c modified by Xin-Ping Wu, Sept. 21, 2017
     *     ' charges will be added on all MM atoms, except',
     *     ' the M1 atoms;',/2x,
c     *     'charges will be added on all MM atoms.',/2x,
c end by Xin-Ping Wu, Sept. 21, 2017
     *   'If shiftbal, redistributed charges will be scaled ',
     *     'and added, ',
     *     'and M2 charges will be modified;',/2x,
     *   'If shiftbal2, redistributed charges will be scaled ',
     *     'and added, ',
     *     'and M2 charges will be modified.')
c  end by Bo Wang 0621BW14
      endif                                                             0628SP10
          
      if     (qmmminp%bordercharge .eq. 'scale')   then
         chargemode = 1
      elseif (qmmminp%bordercharge .eq. 'shift')   then   
         chargemode = 2
      elseif (qmmminp%bordercharge .eq. 'redist1') then   
         chargemode = 3
      elseif (qmmminp%bordercharge .eq. 'rc')      then   
         chargemode = 3
      elseif (qmmminp%bordercharge .eq. 'redist2') then   
         chargemode = 4
      elseif (qmmminp%bordercharge .eq. 'rcd2')    then   
         chargemode = 4
      elseif (qmmminp%bordercharge .eq. 'redist3') then
         chargemode = 5            
      elseif (qmmminp%bordercharge .eq. 'rcd')     then
         chargemode = 5            
c add by Bo Wang 0621BW14
      elseif (qmmminp%bordercharge .eq. 'rcbal')   then
         chargemode = 6
      elseif (qmmminp%bordercharge .eq. 'rcbal2')  then
         chargemode = 7
      elseif (qmmminp%bordercharge .eq. 'rcbal3')  then
         chargemode = 8
      elseif (qmmminp%bordercharge .eq. 'rcdbal')  then
         chargemode = 9
      elseif (qmmminp%bordercharge .eq. 'amber2') then
         chargemode = 10
      elseif (qmmminp%bordercharge .eq. 'seebal')  then
         chargemode = 11
      elseif (qmmminp%bordercharge .eq. 'amber1') then
         chargemode = 12
      elseif (qmmminp%bordercharge .eq. 'shiftbal') then
         chargemode = 13
      elseif (qmmminp%bordercharge .eq. 'shiftbal2') then
         chargemode = 14
c end by Bo Wang 0621BW14

      else
         chargemode = 1      
      endif   
c
c  bordercharge = scale
c
      if (chargemode .eq. 1) then
         ssnatm = lsnatm + m1st
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm
               
         do i=1, m1st
            j = i + lsnatm
            k = qmmminp%mm1stlay%atm1stlay(i)

            ssgeom%atm(j)          = geom%atm(k)
            ssgeom%atmmss(j)       = geom%atmmss(k)
            ssgeom%angcoord(3*j-2) = geom%angcoord(3*k-2)
            ssgeom%angcoord(3*j-1) = geom%angcoord(3*k-1)
            ssgeom%angcoord(3*j)   = geom%angcoord(3*k)

            ssmmgeom%atmid(j)      = j
            ssmmgeom%atmtyp(j)     = mmgeom%atmtyp(k)
            ssmmgeom%atmtypname(j) = mmgeom%atmtypname(k)
            ssmmgeom%atmcharge(j)  = 
     *         qmmminp%mm1stlay%atm1stlaycharge(i)
            ssmmgeom%atmvalen(j)   = 0
         enddo
c
c  bordercharge = shift
c    
      elseif (chargemode .eq. 2) then       
         ssnatm = lsnatm + qmmminp%chredist%nredist
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm
         if   ((tdebug.eq.1).or.(tdynamics.eq.0)) then                  0628SP10
            write (outfile,1000)
1000    format(/2x,'Modify M2 atom charges:',/,'AtomID',2x,'M2-index',
     *      2x,'old charge',2x,'add charge',2x,'new charge',/)
         endif                                                          0628SP10
c
c  shift: modify M2 charges
c
      do i=cssnatm+1, lsnatm  
        j = qmmminp%mlist(i - cssnatm)
        if (j .gt. 0) then
           charge1 = qmmminp%chredist%q(j)
           charge2 = ssmmgeom%atmcharge(i)
           charge  = charge1 + charge2
           ssmmgeom%atmcharge(i) = charge
           qmmminp%mm2ndlay%atm2ndlaycharge(j) = charge
           if       ((tdebug.eq.1).or.(tdynamics.eq.0)) then            0628SP10
                write (outfile,1100) i,j, charge2, charge1, 
     *            ssmmgeom%atmcharge(i)
           endif                                                        0628SP10
        endif
      enddo
 1100  format (i5,2x,i5,3x,(3f8.4,5x))      
c
c  shift: add auxiliary charges, atom typ set to the same as M1 (or M2) 
c
        do i=1, m2nd
c      ii = qmmminp%mm2ndlay%atm2ndlay(i)
       ii = qmmminp%chredist%m1(i)
       do k=-1,0
          j = i*2 + k + lsnatm 

          if (qmmminp%ldampchg .ne. 0) then                             !1122BW14 when smeared charge schemes are used
            ssgeom%atm(j)    = dummyname                                !1122BW14 one needs to use He to provides a site for
            ssgeom%atmmss(j) = ams(dummyams)                            !1122BW14 pseudpotetentials in embedded QM calculations
          else
            ssgeom%atm(j)    = chargename
            ssgeom%atmmss(j) = ams(chargeams)
          endif
          if (k .eq. -1) then
             ssgeom%angcoord(3*j-2) =  qmmminp%chredist%mx(i)
             ssgeom%angcoord(3*j-1) =  qmmminp%chredist%my(i)
             ssgeom%angcoord(3*j)   =  qmmminp%chredist%mz(i)
             ssmmgeom%atmcharge(j)  =  qmmminp%chredist%qd(i)
          else
             ssgeom%angcoord(3*j-2) =  qmmminp%chredist%px(i)
             ssgeom%angcoord(3*j-1) =  qmmminp%chredist%py(i)
             ssgeom%angcoord(3*j)   =  qmmminp%chredist%pz(i)
             ssmmgeom%atmcharge(j)  = -qmmminp%chredist%qd(i)
          endif  
             ssmmgeom%atmid(j)      = j
             if (qmmminp%ldampchg .ne. 0) then                          !1122BW14
               ssmmgeom%atmtyp(j)     = dummytyp                        !1122BW14
               ssmmgeom%atmtypname(j) = dummytypname                    !1122BW14
             else
               ssmmgeom%atmtyp(j)     = mmgeom%atmtyp(ii)
               ssmmgeom%atmtypname(j) = mmgeom%atmtypname(ii)
             endif
             ssmmgeom%atmvalen(j)   = 0
           enddo
         enddo
c
c  bordercharge = redist1 or rc
c     add auxiliary charges, atom typ set to the same as M1 (or M2)
c
      elseif (chargemode .eq. 3) then    
         ssnatm = lsnatm + qmmminp%chredist%nredist
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm

         do i=1, m2nd
            ii = qmmminp%chredist%m1(i)
            j  = i + lsnatm 
            if (qmmminp%ldampchg .ne. 0) then                           !1122BW14
              ssgeom%atm(j)    = dummyname                              !1122BW14
              ssgeom%atmmss(j) = ams(dummyams)                          !1122BW14
            else
              ssgeom%atm(j)    = chargename
              ssgeom%atmmss(j) = ams(chargeams)
            endif
            ssgeom%angcoord(3*j-2) =  qmmminp%chredist%x(i)
            ssgeom%angcoord(3*j-1) =  qmmminp%chredist%y(i)
            ssgeom%angcoord(3*j)   =  qmmminp%chredist%z(i)
            ssmmgeom%atmcharge(j)  =  qmmminp%chredist%q(i)
            ssmmgeom%atmid(j)      = j
            if (qmmminp%ldampchg .ne. 0) then                           !1122BW14
              ssmmgeom%atmtyp(j)     = dummytyp                         !1122BW14
              ssmmgeom%atmtypname(j) = dummytypname                     !1122BW14
            else
              ssmmgeom%atmtyp(j)     = mmgeom%atmtyp(ii)
              ssmmgeom%atmtypname(j) = mmgeom%atmtypname(ii)
            endif
            ssmmgeom%atmvalen(j)   = 0
          enddo

c
c  bordercharge = redist2 or rcd2 (dipole preserved)
c     add auxiliary charges, atom typ set to the same as M1 (or M2)
c      
      elseif (chargemode .eq. 4) then
         ssnatm = lsnatm + qmmminp%chredist%nredist
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm

         do i=1, m2nd
            ii = qmmminp%chredist%m1(i)
            do k=-2,0
                j = i*3 + k + lsnatm 
          
                if (qmmminp%ldampchg .ne. 0) then                       !1122BW14
                  ssgeom%atm(j)    = dummyname                          !1122BW14
                  ssgeom%atmmss(j) = ams(dummyams)                      !1122BW14
                else
                  ssgeom%atm(j)    = chargename
                 ssgeom%atmmss(j) = ams(chargeams)
                endif
                if (k .eq. -2) then
                ssgeom%angcoord(3*j-2) =  qmmminp%chredist%x(i)
                ssgeom%angcoord(3*j-1) =  qmmminp%chredist%y(i)
                ssgeom%angcoord(3*j)   =  qmmminp%chredist%z(i)
                ssmmgeom%atmcharge(j)  =  qmmminp%chredist%q(i)
                elseif (k .eq. -1) then
                ssgeom%angcoord(3*j-2) =  qmmminp%chredist%mx(i)
                ssgeom%angcoord(3*j-1) =  qmmminp%chredist%my(i)
                ssgeom%angcoord(3*j)   =  qmmminp%chredist%mz(i)
                ssmmgeom%atmcharge(j)  =  qmmminp%chredist%qd(i)
                else
                ssgeom%angcoord(3*j-2) =  qmmminp%chredist%px(i)
                ssgeom%angcoord(3*j-1) =  qmmminp%chredist%py(i)
                ssgeom%angcoord(3*j)   =  qmmminp%chredist%pz(i)
                ssmmgeom%atmcharge(j)  = -qmmminp%chredist%qd(i)
                endif     
                ssmmgeom%atmid(j)      = j
                if (qmmminp%ldampchg .ne. 0) then                       !1122BW14
                  ssmmgeom%atmtyp(j)     = dummytyp                     !1122BW14
                  ssmmgeom%atmtypname(j) = dummytypname                 !1122BW14
                else
                  ssmmgeom%atmtyp(j)     = mmgeom%atmtyp(ii)
                  ssmmgeom%atmtypname(j) = mmgeom%atmtypname(ii)
                endif
                ssmmgeom%atmvalen(j)   = 0
            enddo
         enddo
c
c  bordercharge = redist3 or rcd (dipole preserved)
c
        elseif (chargemode .eq. 5) then    
         ssnatm = lsnatm + qmmminp%chredist%nredist
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm
c!          write (outfile,1000)
         if      ((tdebug.eq.1).or.(tdynamics.eq.0)) then               0628SP10
            write (outfile,1000)
         endif                                                          0628SP10
c
c  redist3/rcd: modify M2 charges
c
      do i=cssnatm+1, lsnatm  
        j = qmmminp%mlist(i - cssnatm)
        if (j .gt. 0) then
           charge1 = - qmmminp%chredist%q(j) * dpscale1
           charge2 = ssmmgeom%atmcharge(i)
           charge  = charge1 + charge2
           ssmmgeom%atmcharge(i) = charge
           qmmminp%mm2ndlay%atm2ndlaycharge(j) = charge
           if        ((tdebug.eq.1).or.(tdynamics.eq.0)) then           0628SP10
                write (outfile,1100) i,j, charge2, charge1, 
     *            ssmmgeom%atmcharge(i)
           endif                                                        0628SP10
        endif
      enddo
c
c  redist3/rcd: add auxiliary charges, atom typ set to the same as M1 (or M2)
c
c!          write (outfile, 1200) dpscale2
       if           ((tdebug.eq.1).or.(tdynamics.eq.0)) then            0628SP10
            write (outfile, 1200) dpscale2               
       endif                                                            0628SP10
       do i=1, m2nd
        ii = qmmminp%chredist%m1(i)
            j  = i + lsnatm 
        if (qmmminp%ldampchg .ne. 0) then        !1122BW14
          ssgeom%atm(j)    = dummyname           !1122BW14
          ssgeom%atmmss(j) = ams(dummyams)       !1122BW14
        else
          ssgeom%atm(j)    = chargename
          ssgeom%atmmss(j) = ams(chargeams)
        endif
        ssgeom%angcoord(3*j-2) =  qmmminp%chredist%x(i)
        ssgeom%angcoord(3*j-1) =  qmmminp%chredist%y(i)
        ssgeom%angcoord(3*j)   =  qmmminp%chredist%z(i)
        ssmmgeom%atmcharge(j)  =  qmmminp%chredist%q(i) * dpscale2
        ssmmgeom%atmid(j)      = j
        if (qmmminp%ldampchg .ne. 0) then       !1122BW14
          ssmmgeom%atmtyp(j)     = dummytyp     !1122BW14
          ssmmgeom%atmtypname(j) = dummytypname !1122BW14
        else
          ssmmgeom%atmtyp(j)     = mmgeom%atmtyp(ii)
          ssmmgeom%atmtypname(j) = mmgeom%atmtypname(ii)
        endif
        ssmmgeom%atmvalen(j)   = 0
        if            ((tdebug.eq.1).or.(tdynamics.eq.0)) then          0628SP10
            write (outfile, 1300) ssmmgeom%atmcharge(j),
     *         ssgeom%angcoord(3*j-2),
     *         ssgeom%angcoord(3*j-1), 
     *         ssgeom%angcoord(3*j)
        endif                                                           0628SP10
      enddo

c Start by Bo Wang 1122BW14
c bordercharge = rcbal (redistribute the charge
c  to midpoint of M1-M2 and keep the MM region neutral)

      elseif (chargemode .eq. 6) then
         ssnatm = lsnatm + qmmminp%chredist%nredist
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm
c Add the redistributed charges
         do i=1, m2nd
            ii = qmmminp%chredist%m1(i)
            j  = i + lsnatm
            if (qmmminp%ldampchg .eq. 0) then
              ssgeom%atm(j)    = chargename
              ssgeom%atmmss(j) = ams(chargeams)
            else
              ssgeom%atm(j)    = dummyname
              ssgeom%atmmss(j) = ams(dummyams)
            endif
            ssgeom%angcoord(3*j-2) =  qmmminp%chredist%x(i)
            ssgeom%angcoord(3*j-1) =  qmmminp%chredist%y(i)
            ssgeom%angcoord(3*j)   =  qmmminp%chredist%z(i)
            ssmmgeom%atmcharge(j)  =  qmmminp%chredist%qb(i)  !use balanced charge
            ssmmgeom%atmid(j)      = j
            if (qmmminp%ldampchg .eq. 0) then
              ssmmgeom%atmtyp(j)     = mmgeom%atmtyp(ii)
              ssmmgeom%atmtypname(j) = mmgeom%atmtypname(ii)
            else
              ssmmgeom%atmtyp(j)     = dummytyp
              ssmmgeom%atmtypname(j) = dummytypname
            endif
            ssmmgeom%atmvalen(j)   = 0
         enddo

c bordercharge = rcbal2 (redistribute the charge
c          to M2 atoms and keep the MM region neutral)

      elseif (chargemode .eq. 7) then
         ssnatm = lsnatm + qmmminp%chredist%nredist
c          ssnatm = lsnatm + qmmminp%chredist%nredist + m1st
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm
         write (outfile,1000)
c
c Modify the M2 charges by adding the redistributed charges
c
         do i=cssnatm+1, lsnatm
            j = qmmminp%mlist(i - cssnatm)
            if (j .gt. 0) then
               charge1 = qmmminp%chredist%qb(j)
               charge2 = ssmmgeom%atmcharge(i)
               charge  = charge1 + charge2
               ssmmgeom%atmcharge(i) = charge

               if (qmmminp%ldampchg .eq. 1
     &              .or. qmmminp%ldampchg .eq. 3) then    !smear the redistributed charge part on M2 atom
                 qmmminp%dampnatm = qmmminp%dampnatm +1
                 qmmminp%dampatm(qmmminp%dampnatm) = ssmmgeom%atmid(i) !the id of the M2 atom, assume that the M2 atom is screened.
                 qmmminp%dampzeta(qmmminp%dampnatm)
     &                                  = 0.5291772/qmmminp%lambda !change the lambda value to zeta value
                 qmmminp%dampval(qmmminp%dampnatm) = 0.0
                 qmmminp%dampchg(qmmminp%dampnatm) = charge1 !0.0-charge1 is the number of electrons to be screened
                 ssmmgeom%atmtyp(i)     = htyp               !set the atom type as HC
                 ssmmgeom%atmtypname(i) = htypname           !set the atom type name as HC
                 ssgeom%atm(i) = hname                       !set atom as H, used in CPS** calculations
               endif

               qmmminp%mm2ndlay%atm2ndlaycharge(j) = charge
               qmmminp%mm2ndlay%atm2ndlaycharge(j) = charge
               write (outfile,1100) i,j, charge2, charge1,
     *            ssmmgeom%atmcharge(i)
            endif
         enddo

c bordercharge = rcbal3 (redistribute the charge
c                              to M2,M3 and keep the MM region neutral)

      elseif (chargemode .eq. 8) then
         ssnatm = lsnatm + qmmminp%chredist%nredist
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm
         write (outfile,1000)
c
c Modify the M2, M3 charges by adding the redistributed charges
c
         do i=cssnatm+1, lsnatm
            j = qmmminp%mlist(i - cssnatm)
            k=  qmmminp%m3list(i - cssnatm)      !list of M3 atoms
            if (j .gt. 0) then
               charge1 = qmmminp%chredist%qb(j)  
               charge2 = ssmmgeom%atmcharge(i)
               charge  = charge1 + charge2
                                                                                                                      
               ssmmgeom%atmcharge(i) = charge
               qmmminp%mm2ndlay%atm2ndlaycharge(j) = charge

               if (qmmminp%ldampchg .eq. 1
     &              .or. qmmminp%ldampchg .eq. 3) then  !smear the redistributed charge part on M2 atom
                 qmmminp%dampnatm = qmmminp%dampnatm +1
                 qmmminp%dampatm(qmmminp%dampnatm) = ssmmgeom%atmid(i)  !the id of the M2 atom, assume that the M2 atom is screened.
                 qmmminp%dampzeta(qmmminp%dampnatm)
     &                                  = 0.5291772/qmmminp%lambda   !change the lambda value to zeta value
                 qmmminp%dampval(qmmminp%dampnatm) = 0.0
                 qmmminp%dampchg(qmmminp%dampnatm) = charge1    !0.0-charge1 is the number of electrons to be screened
                 ssmmgeom%atmtyp(i)     = htyp                  !set the atom type as HC
                 ssmmgeom%atmtypname(i) = htypname              !set the atom type name as HC
                 ssgeom%atm(i) = hname                          !set atom as H, used in CPS** calculations
               endif

               write (outfile,1100) i,j, charge2, charge1,
     *            ssmmgeom%atmcharge(i)
            elseif (k .gt. 0) then
               charge1 = qmmminp%chredist%qc(k)    
               charge2 = ssmmgeom%atmcharge(i)
               charge  = charge1 + charge2
               ssmmgeom%atmcharge(i) = charge

               if (qmmminp%ldampchg .eq. 1
     &              .or. qmmminp%ldampchg .eq. 3) then
                 qmmminp%dampnatm = qmmminp%dampnatm +1
                 qmmminp%dampatm(qmmminp%dampnatm) = ssmmgeom%atmid(i)  !id of the M3 atom
                 qmmminp%dampzeta(qmmminp%dampnatm)
     &                                  = 0.5291772/qmmminp%lambda
                 qmmminp%dampval(qmmminp%dampnatm) = 0.0
                 qmmminp%dampchg(qmmminp%dampnatm) = charge1
                 ssmmgeom%atmtyp(i)     = htyp
                 ssmmgeom%atmtypname(i) = htypname
                 ssgeom%atm(i) = hname
               endif

               write (outfile,1100) i,k, charge2, charge1,
     *            ssmmgeom%atmcharge(i)
            endif
         enddo

c
c bordercharge = rcdbal (redistribute the charge to midpoint of M1-M2
c          change M2 to keep the dipole, and keep the MM region neutral)

      elseif (chargemode .eq. 9) then
         ssnatm = lsnatm + qmmminp%chredist%nredist
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm
         write (outfile,1000)
c
c Modify the M2 charges
c
         do i=cssnatm+1, lsnatm
            j = qmmminp%mlist(i - cssnatm)
            if (j .gt. 0) then
               charge1 = - qmmminp%chredist%qb(j) * dpscale1
               charge2 = ssmmgeom%atmcharge(i)
               charge  = charge1 + charge2
               ssmmgeom%atmcharge(i) = charge
               qmmminp%mm2ndlay%atm2ndlaycharge(j) = charge

               if (qmmminp%ldampchg .eq. 1
     &              .or. qmmminp%ldampchg .eq. 3) then
                 qmmminp%dampnatm = qmmminp%dampnatm +1
                 qmmminp%dampatm(qmmminp%dampnatm) = ssmmgeom%atmid(i)
                 qmmminp%dampzeta(qmmminp%dampnatm)
     &                                  = 0.5291772/qmmminp%lambda
                  qmmminp%dampval(qmmminp%dampnatm) = 0.0
                 qmmminp%dampchg(qmmminp%dampnatm) = charge1
                 ssmmgeom%atmtyp(i)     = htyp
                 ssmmgeom%atmtypname(i) = htypname
                 ssgeom%atm(i) = hname
               endif
               write (outfile,1100) i,j, charge2, charge1,
     *            ssmmgeom%atmcharge(i)
            endif
         enddo
c
c Add auxiliary charges, atom typ set to the same as M1 (or M2) or dummy
c
         write (outfile, 1200) dpscale2
         do i=1, m2nd
            ii = qmmminp%chredist%m1(i)
            j  = i + lsnatm
            if (qmmminp%ldampchg .eq. 0) then
              ssgeom%atm(j)    = chargename
              ssgeom%atmmss(j) = ams(chargeams)
            else
              ssgeom%atm(j)    = dummyname
              ssgeom%atmmss(j) = ams(dummyams)
            endif
            ssgeom%angcoord(3*j-2) =  qmmminp%chredist%x(i)
            ssgeom%angcoord(3*j-1) =  qmmminp%chredist%y(i)
            ssgeom%angcoord(3*j)   =  qmmminp%chredist%z(i)
            ssmmgeom%atmcharge(j)  =  qmmminp%chredist%qb(i) * dpscale2
            ssmmgeom%atmid(j)      = j

            if (qmmminp%ldampchg .eq. 0) then
              ssmmgeom%atmtyp(j)     = mmgeom%atmtyp(ii)
              ssmmgeom%atmtypname(j) = mmgeom%atmtypname(ii)
            else
              ssmmgeom%atmtyp(j)     = dummytyp
              ssmmgeom%atmtypname(j) = dummytypname
            endif
            ssmmgeom%atmvalen(j)   = 0
            write (outfile, 1300) ssmmgeom%atmcharge(j),
     *         ssgeom%angcoord(3*j-2),
     *         ssgeom%angcoord(3*j-1),
     *         ssgeom%angcoord(3*j)
         enddo

c
c  bourdercharge=Amber2 (redistribute the charge to all the MM atoms and
c                          keep the MM region neutral

      elseif (chargemode .eq. 10) then
         ssnatm = lsnatm + qmmminp%chredist%nredist
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm
         write (outfile,1000)
c
c Modify all MM charges by adding the redistributed charges
c
         do i=cssnatm+1, lsnatm
            charge1 = qmmminp%chredist%qc(i-cssnatm)
            charge2 = ssmmgeom%atmcharge(i)
            charge  = charge1 + charge2
            ssmmgeom%atmcharge(i) = charge
            j = qmmminp%mlist(i - cssnatm)  !mlist provides M2 atoms
               if (j .gt. 0) then           !Also change the M2 layer atom charges
                  qmmminp%mm2ndlay%atm2ndlaycharge(j) = charge
               endif
           write (outfile,1100) i,j, charge2, charge1,
     *          ssmmgeom%atmcharge(i)
         enddo
c
c bordercharge= seebal (the Q1 charge keep the total charge neutral)
c
      elseif (chargemode .eq. 11) then
         ssnatm = lsnatm + m1st
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm

         do i=1, m1st
            j = i + lsnatm
            k = qmmminp%mm1stlay%atm1stlay(i)

            if (qmmminp%ldampchg .eq. 0) then
              ssgeom%atm(j)    = chargename
              ssgeom%atmmss(j) = ams(chargeams)
            else
              ssgeom%atm(j)    = dummyname
              ssgeom%atmmss(j) = ams(dummyams)
            endif
            ssgeom%angcoord(3*j-2) = geom%angcoord(3*k-2)
            ssgeom%angcoord(3*j-1) = geom%angcoord(3*k-1)
            ssgeom%angcoord(3*j)   = geom%angcoord(3*k)

            ssmmgeom%atmid(j)      = j
            if (qmmminp%ldampchg .eq. 0) then
              ssmmgeom%atmtyp(j)     = mmgeom%atmtyp(ii)
              ssmmgeom%atmtypname(j) = mmgeom%atmtypname(ii)
            else
              ssmmgeom%atmtyp(j)     = dummytyp
              ssmmgeom%atmtypname(j) = dummytypname
            endif
            ssmmgeom%atmcharge(j)  = qmmminp%chredist%qc(i)
            ssmmgeom%atmvalen(j)   = 0
         enddo

c
c bordercharge = Amber1 (redistribute the charge to the nearest M2
c                      and keep MM region neutral)

      elseif (chargemode .eq. 12) then
         ssnatm = lsnatm + qmmminp%chredist%nredist
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm
         write (outfile,1000)

c Modify the M2 charges by adding the redistributed charges
c
         do i=cssnatm+1, lsnatm
            j = qmmminp%mlist(i - cssnatm)
            if (j .gt. 0) then
               charge1 = qmmminp%chredist%qc(j)
               charge2 = ssmmgeom%atmcharge(i)
               charge  = charge1 + charge2
               ssmmgeom%atmcharge(i) = charge
               qmmminp%mm2ndlay%atm2ndlaycharge(j) = charge
               write (outfile,1100) i,j, charge2, charge1,
     *            ssmmgeom%atmcharge(i)
            endif
         enddo
c
c  shift_balance, add the dipole and also preserve the total charge
c
      elseif (chargemode .eq. 13) then
         ssnatm = lsnatm + qmmminp%chredist%nredist
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm
         write (outfile,1000)
c
c  add charges
c
         do i=cssnatm+1, lsnatm
            j = qmmminp%mlist(i - cssnatm)
            if (j .gt. 0) then
               charge1 = qmmminp%chredist%qb(j)
               charge2 = ssmmgeom%atmcharge(i)
               charge  = charge1 + charge2
               ssmmgeom%atmcharge(i) = charge
               qmmminp%mm2ndlay%atm2ndlaycharge(j) = charge
               write (outfile,1100) i,j, charge2, charge1,
     *            ssmmgeom%atmcharge(i)
            endif
         enddo
c
c  shift: add auxiliary charges, atom typ set to the same as M1 (or M2) or dummy
c
        do i=1, m2nd
c          ii = qmmminp%mm2ndlay%atm2ndlay(i)
           ii = qmmminp%chredist%m1(i)
           do k=-1,0
              j = i*2 + k + lsnatm
              if (qmmminp%ldampchg .eq. 0) then
                ssgeom%atm(j)    = chargename
                ssgeom%atmmss(j) = ams(chargeams)
              else
                ssgeom%atm(j)    = dummyname
                ssgeom%atmmss(j) = ams(dummyams)
              endif
              if (k .eq. -1) then
                 ssgeom%angcoord(3*j-2) =  qmmminp%chredist%mx(i)
                 ssgeom%angcoord(3*j-1) =  qmmminp%chredist%my(i)
                 ssgeom%angcoord(3*j)   =  qmmminp%chredist%mz(i)
                 ssmmgeom%atmcharge(j)  =  qmmminp%chredist%qd(i)
              else
                 ssgeom%angcoord(3*j-2) =  qmmminp%chredist%px(i)
                 ssgeom%angcoord(3*j-1) =  qmmminp%chredist%py(i)
                 ssgeom%angcoord(3*j)   =  qmmminp%chredist%pz(i)
                 ssmmgeom%atmcharge(j)  = -qmmminp%chredist%qd(i)
              endif
              ssmmgeom%atmid(j)      = j
              if (qmmminp%ldampchg .eq. 0) then
                ssmmgeom%atmtyp(j)     = mmgeom%atmtyp(ii)
                ssmmgeom%atmtypname(j) = mmgeom%atmtypname(ii)
              else
                ssmmgeom%atmtyp(j)     = dummytyp
                ssmmgeom%atmtypname(j) = dummytypname
              endif
              ssmmgeom%atmvalen(j)   = 0
           enddo
        enddo
c
c shift_balance2, add the dipole and also preserve the total charge.
c               the dipole compensates the charge moves
      elseif (chargemode .eq. 14) then
         ssnatm = lsnatm + qmmminp%chredist%nredist
         ssgeom%natoms = ssnatm
         ssmmgeom%natoms = ssnatm
         write (outfile,1000)
c
c  add charges
c
         do i=cssnatm+1, lsnatm
            j = qmmminp%mlist(i - cssnatm)
            if (j .gt. 0) then
               charge1 = qmmminp%chredist%qb(j)
               charge2 = ssmmgeom%atmcharge(i)
               charge  = charge1 + charge2
               ssmmgeom%atmcharge(i) = charge
               qmmminp%mm2ndlay%atm2ndlaycharge(j) = charge
               write (outfile,1100) i,j, charge2, charge1,
     *            ssmmgeom%atmcharge(i)
            endif
         enddo
c  shift: add auxiliary charges, atom typ set to the same as M1 (or M2) or dummy
c
        do i=1, m2nd
c          ii = qmmminp%mm2ndlay%atm2ndlay(i)
           ii = qmmminp%chredist%m1(i)
           do k=-1,0
              j = i*2 + k + lsnatm
              if (qmmminp%ldampchg .eq. 0) then
                ssgeom%atm(j)    = chargename
                ssgeom%atmmss(j) = ams(chargeams)
              else
                ssgeom%atm(j)    = dummyname
                ssgeom%atmmss(j) = ams(dummyams)
              endif

              if (k .eq. -1) then
                 ssgeom%angcoord(3*j-2) =  qmmminp%chredist%mx(i)
                 ssgeom%angcoord(3*j-1) =  qmmminp%chredist%my(i)
                 ssgeom%angcoord(3*j)   =  qmmminp%chredist%mz(i)
                 ssmmgeom%atmcharge(j)  =  qmmminp%chredist%qd2(i)
              else
                 ssgeom%angcoord(3*j-2) =  qmmminp%chredist%px(i)
                 ssgeom%angcoord(3*j-1) =  qmmminp%chredist%py(i)
                 ssgeom%angcoord(3*j)   =  qmmminp%chredist%pz(i)
                 ssmmgeom%atmcharge(j)  = -qmmminp%chredist%qd2(i)
              endif
              ssmmgeom%atmid(j)      = j
              if (qmmminp%ldampchg .eq. 0) then
                ssmmgeom%atmtyp(j)     = mmgeom%atmtyp(ii)
                ssmmgeom%atmtypname(j) = mmgeom%atmtypname(ii)
              else
                ssmmgeom%atmtyp(j)     = dummytyp
                ssmmgeom%atmtypname(j) = dummytypname
              endif
              ssmmgeom%atmvalen(j)   = 0
           enddo
        enddo

cc End by Bo Wang 1122BW14

1200  format(/2x,'Redistributed charges scaled by a factor of ',
     *    2x,f10.5,/)
 1300  format('Q: Charge: ',f8.4,2x,'XYZ: ',3(f12.8,2x))

      endif    

c
c  Fill the coordinates in AU
c      
       ssgeom%aucoord(lsnatm*3+1:ssnatm*3) =  
     *   ssgeom%angcoord(lsnatm*3+1:ssnatm*3)/angtoau
c            
      return     
      end

c******************************************************************

c*****************************************************************************
c  PROGMMSTEH
c*****************************************************************************
c
      subroutine progmmstehk1(prognme, esmeth, esbas, 
     * geom, mmgeom, optl, ener, ierr, qmmminp)
c
c     Subroutine progmmstehk
c
c     Created:  Feb. 22, 2004 by Hai Lin
c               based on progmmehk.
c
c     Purpose:  Get a single point interaction energy witin
c               the background point charge with the 
c               specified force field package.
c
c     Description:  Calls a subroutine to write a input file,
c         runs this calculation on the system, and then calls a subroutine 
c         that reads the energy from the output file.
c
c     Note:  Currently only TINKER is enabled but it would be easily
c        expanded to other force field programs.  new _inpe and
c        _oute subroutines would have to be written for each different
c        program though.
c
c     Called by:  qmmmehk
c     Calls:  t41inp, t41oute, system
c
      use caltyp
      use files
      use input 
      implicit none
c
c  i/o variables
c
      character*10       prognme
      character*20       esmeth                      
      character*20       esbas                                                   
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
c
      character*80       optl(maxoptl)
      double precision   ener
      integer            ierr,i
      type (qmmminf)  :: qmmminp 
c
c  local variables
c
      character*20       ver_lab      
      double precision   v11, v12, v22
c
c  TINKER 
c
      if (prognme(:6).eq.'tinker')  then
c
c write .xyz and .key files    
c      

         open (unit=t41inf, status='unknown', file=t41in, err=9994)
         open (unit=t41keyf, status='unknown', file=t41key, err=9995)
         
         call t41inp(t41inf, t41keyf,
     *      esmeth, esbas, geom, mmgeom,optl,tener,ver_lab)


c
c  additional key words
c
         v11 = 0.0d0
         v12 = 0.0d0
         v22 = 1.0d0
         if (qmmminp%ldampchg .eq. 0) then                      !1222BW14
         call t41keyadd(t41keyf, geom, mmgeom, qmmminp,
     *      v11, v12, v22, ierr)
         else                                                   !1122BW14 damping (screened) scheme
           v11 = 0.0d0                                          !1122BW14
           v12 = 1.0d0                                          !1122BW14
           v22 = 0.0d0                                          !1122BW14
           call t41keyadd1(t41keyf, geom, mmgeom, qmmminp,      !1122BW14
     *     v11, v12, v22, ierr)                                 !1122BW14
         endif

         close (t41inf, err=9994)
         close (t41keyf, err=9995)        	    

c
c read the atomic MM charge 
c  also print MM parameters if required
c 
         if((tprm .eq. 1) .or. (mmgeom%lprnatmprm .eq. 1)) then
            call system('t41shuttle t41 p')	    
            call t41outp(mmgeom)
         endif    

c
c compute the energy
c	 	 
         call system('t41shuttle t41 e')                                	 
         call t41oute(1,esmeth,ener)	 
      endif

      return
c
9994  write (outfile,*) 'Error creating TINKER .xyz file'
      stop 'progehk 4'
9995  write (outfile,*) 'Error creating TINKER .key file'
      stop 'progehk 5'
      end
c
c
c*****************************************************************************
c  T41KEYADD
c*****************************************************************************
c
      subroutine t41keyadd(ikey, geom, mmgeom, qmmminp, 
     *   v11, v12, v22, ierr)
c
c     Subroutine t41keyadd
c
c     Created:  Feb. 22, 2004 by Hai Lin
c     Revised:  Aug. 31, 2007 by Yan Zhang
c
c     Purpose:  Add additional keywords to t41.key to compute 
c               electrostatic interaction between CPS and/or  
c               the background point charges, as controlled by the 
c               input selection v11, v12, and v22 
c
c     Description: If scale, one has only M1 charges;
c                  If shift, one has M1 and dipole-mimic charges;
c                  If redist1/rc, one has M1 and redistributed charges;
c                  If redist2/rcd2, one has M1, redistributed, 
c                     and dipole-mimic charges
c                  If redist3/rcd, one has M1 and redistributed charges;
c
c     Called by:  progmmstehk
c
      use caltyp
      use files
      use input 
      implicit none
c
c  i/o variables
c
      type (cartgeom) :: geom   
      type (mmvalen)  :: mmgeom   
      type (qmmminf)  :: qmmminp
      
      integer            ierr, ikey
      double precision   v11, v12, v22
c
c  local variables
c
      integer  capnatm, qmnatm, mmnatm, m1chnatm
      integer  ssnatm, cssnatm, lsnatm
      integer  i,j,k, ii,jj,kk,ig,igid
      integer  m1st, m2nd, m3rd
      integer  alist(geom%natoms)

      integer             chargemode
c
c  prepare atom numbers
c
      capnatm = qmmminp%capatm%capnatm
      qmnatm  = qmmminp%qmnatm
      mmnatm  = qmmminp%mmnatm
      ssnatm  = mmgeom%natoms
      cssnatm = capnatm + qmnatm
      lsnatm  = capnatm + qmnatm + mmnatm
      m1chnatm = ssnatm - lsnatm
      m1st = qmmminp%mm1stlay%natm1stlay
      m2nd = qmmminp%mm2ndlay%natm2ndlay
      m3rd = qmmminp%mm3rdlay%natm3rdlay

      do i=1, ssnatm
         alist(i) = i
      enddo
c
c  write the keyword
c
      write (ikey,1000)
c
c  Special Note:
c
c     4) Tinker will always renumber atoms in xyz file first,
c         and everything afterwords including interpretation of the keywords 
c         in the key file are using the new (renumbered) atom id.
		     
c
c  modifiy M2 and M3 point charges
c            
cx      do i = 1, m2nd
cx         write (ikey,1400) - qmmminp%mm2ndlay%atm2ndlay(i), 
cx     *     qmmminp%mm2ndlay%atm2ndlaycharge(i)
cx      enddo	 
c
c        do i = cssnatm + 1, lsnatm
c           j = qmmminp%mlist(i-cssnatm)
c           if (j .gt. 0) then
c                write (ikey,1400) -i, 
c     *        qmmminp%mm2ndlay%atm2ndlaycharge(j)         
c           endif
c            j = qmmminp%m3list(i-cssnatm)
c           if (j .gt. 0) then
c                write (ikey,1400) -i,
c     *        qmmminp%mm3rdlay%atm3rdlaycharge(j)               
c           endif
c         enddo
c  Revised 8/31/2007 by Yan Zhang
c  Rewrite this section to modify MM point charges. Add a new variable 
c  modmmcharg for the different manner of the boundary treatment. 
c  modmmcharg:  1  non-polarization and non-flexible-boundary
c               2  polarization. The background point charges are calculated
c                  by the QEQ-RG, QEQ-BT and EEM method in the poarized 
c                  calculations. The polarized charges are reset in the MM 
c                  calculations.
c               3  flexible-boundary 
c               4  MMFF94 FF, using bond charge increment   !1014BW14
c                  need to specify charge in the key file   !1014BW14
c
c
      if(qmmminp%modmmcharg.eq.1) then
        do i = 1, mmnatm
          kk = cssnatm + i
          k  = mmgeom%atmid(kk) 
          do ii=1,m2nd
            jj = qmmminp%mm2ndlay%atm2ndlay(ii)
            if (k .eq. jj) then
              write (ikey,1400) -kk,mmgeom%atmcharge(kk) 
            endif
          enddo    
          do ii=1,m3rd
            jj = qmmminp%mm3rdlay%atm3rdlay(ii)
            if (k .eq. jj) then
              write (ikey,1400) -kk,mmgeom%atmcharge(kk) 
            endif
          enddo               
        enddo            
       
      else if(qmmminp%modmmcharg.eq.2) then
        do i = 1, mmnatm
          jj= cssnatm + i
          k = mmgeom%atmid(jj) 
          do ig=1,qmmminp%polgnum
            igid= qmmminp%polgidnum(ig)
            do j=1,igid
              kk = qmmminp%polgid(ig,j)
              if(k.eq.kk)
     *          write (ikey,1400) -jj, mmgeom%atmcharge(jj)
            end do
          end do               
        end do

      else if(qmmminp%modmmcharg.eq.3) then
        do i = 1,mmnatm
          jj = cssnatm + i
          k  = mmgeom%atmid(jj)
          do ig = 1, qmmminp%fbgnum
            do j = 1, qmmminp%fbgan(ig)
              kk = qmmminp%fbgaid(ig,j)
              if(k.eq.kk)
     *          write (ikey,1400) -jj, mmgeom%atmcharge(jj)
            end do
          end do
        end do
c
c Add by Bo Wang 1122BW14
c MMFF94 uses bond increment charges to assign charges
c because no bond is assigned to MM in the current calculation,
c tinker can't assign MM charges automatically
c one need to print out the MM charges in the t41.key file.
      else if(qmmminp%modmmcharg.eq.4) then
        do i = 1, mmnatm
          kk = cssnatm + i
          k  = mmgeom%atmid(kk)
              write (ikey,1400) -kk,mmgeom%atmcharge(kk)
        enddo
c end by Bo Wang 1122BW14

      else
        write(outfile,*) 'Error boundary of treatment.'
        stop ' in subroutine t41keyadd'
      end if

      write(ikey,*) "CHG-BUFFER 0.00"                                   !1122BW14  the default is 0.05 in mmff

c
c   End by Yan Zhang 8/31/2007
c     
      write (ikey,*)
c
c  modified M1 and M1-associated charges, which are stored 
c     after QM, Cap, MM (excluding M1) atoms
c
      do i = lsnatm + 1, ssnatm
c         write (ikey,1400) - mmgeom%atmid(i), mmgeom%atmcharge(i)
         write (ikey,1400) -i, mmgeom%atmcharge(i)
      enddo
      write (ikey,*)	       
c
c  ii is the number of atom ID per line
c
      ii = 2	 	             
c
c  group CSS
c      
      jj = 1      
      do while(jj .le. cssnatm)
         kk = min(jj + ii - 1, cssnatm)
cx         write (ikey,1100) mmgeom%atmid(jj:kk)
         write (ikey,1100) alist(jj:kk)
         jj = jj + ii
      enddo	 
      write (ikey,*)            
c
c  group background charges
c      	 
      jj = cssnatm + 1
      do while(jj .le. ssnatm)
         kk = min(jj + ii - 1, ssnatm)	       
cx         write (ikey,1200) mmgeom%atmid(jj:kk)
         write (ikey,1200) alist(jj:kk)
         jj = jj + ii
      enddo
      write (ikey,*)
c
c  select groups
c            	 
      write (ikey,1300) 1, 1, v11
      write (ikey,1300) 1, 2, v12
      write (ikey,1300) 2, 2, v22
                    
1000  format(/2x,'chargeterm only ',
     *       /2x,'chg-12-scale 1.0 ',
     *       /2x,'chg-13-scale 1.0 ',
     *       /2x,'chg-14-scale 1.0 ',
     *       /2x,'chg-15-scale 1.0 ',/)
1100  format('group 1 ',2(i4,2x))
1200  format('group 2 ',2(i4,2x))
1300  format('group-select ',i2,2x,i2,2x,f16.8)
1400  format('charge  ',i4,2x,f16.8)
      return
      end
c**************************************************************      

c*****************************************************************************
c  T41KEYADD1
c*****************************************************************************
c
      subroutine t41keyadd1(ikey, geom, mmgeom, qmmminp,           !0130BW10 new subroutine
     *   v11, v12, v22, ierr)
c
c     Subroutine t41keyadd1
c
c     Created:  Jan. 30,2010 by Bo Wang
c               From t41keyadd1
c     Purpose:  Add additional keywords to t41.key to correct the energy
c               This subroutine is used for screened charge and smeared charge
c               schemes. Since the pseudopotentials are used to represent these
c               charges, an MM correction is needed to calculate the
c               interactions between the pseudopotentials and nuclei
c
c     Description: If scale, one has only M1 charges;
c                  If shift, one has M1 and dipole-mimic charges;
c                  If redist1/rc, one has M1 and redistributed charges;
c                  If redist2/rcd2, one has M1, redistributed,
c                     and dipole-mimic charges
c                  If redist3/rcd, one has M1 and redistributed charges;
c
c     Called by:  progmmstehk
c
      use caltyp
      use files
      use input
      use atminfo
      implicit none
c
c  i/o variables
c
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (qmmminf)  :: qmmminp

      integer            ierr, ikey
      double precision   v11, v12, v22
c
c  local variables
c
      integer  capnatm, qmnatm, mmnatm, m1chnatm
      integer  ssnatm, cssnatm, lsnatm
      integer  i,j,k, ii,jj,kk,ig,igid
      integer  m1st, m2nd, m3rd
      integer  alist(geom%natoms)

      integer             chargemode
      character*2  atmsymbol
      double precision zeta
      double precision val,chg

c
c  prepare atom numbers
c
      capnatm = qmmminp%capatm%capnatm
      qmnatm  = qmmminp%qmnatm
      mmnatm  = qmmminp%mmnatm
      ssnatm  = mmgeom%natoms
      cssnatm = capnatm + qmnatm
      lsnatm  = capnatm + qmnatm + mmnatm
      m1chnatm = ssnatm - lsnatm
      m1st = qmmminp%mm1stlay%natm1stlay
      m2nd = qmmminp%mm2ndlay%natm2ndlay
      m3rd = qmmminp%mm3rdlay%natm3rdlay

      do i=1, ssnatm
         alist(i) = i
      enddo
c
c  write the keyword
c
      write (ikey,1000)
c
      if (qmmminp%ldampchg .eq. 1 .or. qmmminp%ldampchg .eq. 2) then
        write (ikey,100)                                              !use ODS
100     format(/2x,'ODS ')
      endif
      write (ikey, 200) qmmminp%ngto1, qmmminp%ngto2
200   format(/2x, 'NGTO1  ', I5 / 1x,'NGTO2  ', I5)
      if (qmmminp%ldampchg .eq. 1 .or. qmmminp%ldampchg .eq. 3) then  !screening and smearing
        write (ikey,300)
300   format(/2x,'DAMPRC 1 ')
      elseif (qmmminp%ldampchg .eq. 2) then                           !screening but no smearing
        write (ikey,400)
400   format(/2x,'DAMPRC 0 ')
      endif
      write (ikey,*)

c  specify which atom charges should be screened
      do i = 1, mmnatm                                  ! find out the atoms need screening
        kk = cssnatm + i
        k  = mmgeom%atmid(kk)
        do ii=1,qmmminp%dampnatm
          jj = qmmminp%dampatm(ii)
          if (k .eq. jj) then
            zeta=qmmminp%dampzeta(ii)
            val=qmmminp%dampval(ii)
            chg=qmmminp%dampchg(ii)
            if (zeta .eq. -1.0) then
                write (ikey,500) kk
            elseif (chg .eq. 999.0 ) then
                write (ikey,600) kk,zeta,val
            else
              write (ikey,700) kk,zeta,val,chg
            endif
          endif
        enddo
      enddo
500   format('damping ', i4)
600   format('damping3 ', i4, 3x, f10.4,3x,f7.4)
700   format('damping4 ', i4, 3x, f10.4,3x,f7.4,3x,f7.4)
      write (ikey,*)

c
c specify the rcwidth used for the smeared redistributed charges
c
      write (ikey,800) qmmminp%lambda
800   format('lambda',2x, f10.5)

c
c  Special Note:
c
c     4) Tinker will always renumber atoms in xyz file first,
c         and everything afterwords including interpretation of the keywords
c         in the key file are using the new (renumbered) atom id.

c
c  modifiy M2 and M3 point charges
c
cx      do i = 1, m2nd
cx         write (ikey,1400) - qmmminp%mm2ndlay%atm2ndlay(i),
cx     *     qmmminp%mm2ndlay%atm2ndlaycharge(i)
cx      enddo
c
c        do i = cssnatm + 1, lsnatm
c           j = qmmminp%mlist(i-cssnatm)
c           if (j .gt. 0) then
c                write (ikey,1400) -i,
c     *        qmmminp%mm2ndlay%atm2ndlaycharge(j)
c           endif
c            j = qmmminp%m3list(i-cssnatm)
c           if (j .gt. 0) then
c                write (ikey,1400) -i,
c     *        qmmminp%mm3rdlay%atm3rdlaycharge(j)
c           endif
c         enddo
c  Revised 8/31/2007 by Yan Zhang
c  Rewrite this section to modify MM point charges. Add a new variable
c  modmmcharg for the different manner of the boundary treatment.
c  modmmcharg:  1  non-polarization and non-flexible-boundary
c               2  polarization. The background point charges are calculated
c                  by the QEQ-RG, QEQ-BT and EEM method in the poarized
c                  calculations. The polarized charges are reset in the MM
c                  calculations.
c               3  flexible-boundary
c
c               4  MMFF94 FF, using bond charge increment
c                  need to specify charge in the key file
c
c
c
      if(qmmminp%modmmcharg.eq.1) then
        do i = 1, mmnatm
          kk = cssnatm + i
          k  = mmgeom%atmid(kk)
          do ii=1,m2nd
            jj = qmmminp%mm2ndlay%atm2ndlay(ii)
            if (k .eq. jj) then
              write (ikey,1400) -kk,mmgeom%atmcharge(kk)
            endif
          enddo
          do ii=1,m3rd
            jj = qmmminp%mm3rdlay%atm3rdlay(ii)
            if (k .eq. jj) then
              write (ikey,1400) -kk,mmgeom%atmcharge(kk)
            endif
          enddo
        enddo

      else if(qmmminp%modmmcharg.eq.2) then
        do i = 1, mmnatm
          jj= cssnatm + i
          k = mmgeom%atmid(jj)
          do ig=1,qmmminp%polgnum
            igid= qmmminp%polgidnum(ig)
            do j=1,igid
              kk = qmmminp%polgid(ig,j)
              if(k.eq.kk)
     *          write (ikey,1400) -jj, mmgeom%atmcharge(jj)
            end do
          end do
        end do

      else if(qmmminp%modmmcharg.eq.3) then
        do i = 1,mmnatm
          jj = cssnatm + i
          k  = mmgeom%atmid(jj)
          do ig = 1, qmmminp%fbgnum
            do j = 1, qmmminp%fbgan(ig)
              kk = qmmminp%fbgaid(ig,j)
              if(k.eq.kk)
     *          write (ikey,1400) -jj, mmgeom%atmcharge(jj)
            end do
          end do
        end do

      else if(qmmminp%modmmcharg.eq.4) then  !MMFF94 use bond increment charges
        do i = 1, mmnatm                     !because no bond is assigned to MM atoms here, the tinker can not
          kk = cssnatm + i                   !figure out the charges for each MM atom.
          k  = mmgeom%atmid(kk)
              write (ikey,1400) -kk,mmgeom%atmcharge(kk)
        enddo

      else
        write(outfile,*) 'Error boundary of treatment.'
        stop ' in subroutine t41keyadd'
      end if
c
c   End by Yan Zhang 8/31/2007
c
      write(ikey,*) "CHG-BUFFER 0.00"
      write (ikey,*)
c
c  modified M1 and M1-associated charges, which are stored
c     after QM, Cap, MM (excluding M1) atoms
c
      do i = lsnatm + 1, ssnatm
cx         write (ikey,1400) - mmgeom%atmid(i), mmgeom%atmcharge(i)
         write (ikey,1400) -i, mmgeom%atmcharge(i)
      enddo
      write (ikey,*)
c  write the nucleus charges of QM atoms
      do i = 1, qmnatm
        atmsymbol = geom%atm(i)
        do j=1,36
          if (asymbu(j) .eq. atmsymbol) then
            write (ikey,1500) -i, -j           !use -j to correct the nucleus charge
1500  format('charge  ',i4,2x,i5)
           exit
          endif
        enddo
      enddo

c write the nucleus charges of capped atoms
      do i = qmnatm+1, cssnatm
        atmsymbol = geom%atm(i)
        do j=1,36
          if (asymbu(j) .eq. atmsymbol) then
            if (qmmminp%lppf .eq. 1 .and. j .eq. 9)  then  !tuned F link atom, nucleas charge is only 7
              write (ikey, 1500) -i,-(j-2)
            else
              write (ikey,1500) -i, -j                      !use -j to correct the nucleus charge
            endif
            exit
          endif
        enddo
      enddo
      write (ikey,*)

c
c  ii is the number of atom ID per line
c
      ii = 2
c
c  group CSS
c
      jj = 1
      do while(jj .le. cssnatm)
         kk= min(jj + ii - 1, cssnatm)
cx       write (ikey,1100) mmgeom%atmid(jj:kk)
       write (ikey,1100) alist(jj:kk)
       jj = jj + ii
      enddo
      write (ikey,*)
c  group background charges
      jj = cssnatm + 1
      do while(jj .le. ssnatm)
         kk = min(jj + ii - 1, ssnatm)
cx         write (ikey,1200) mmgeom%atmid(jj:kk)
         write (ikey,1200) alist(jj:kk)
         jj = jj + ii
      enddo
      write (ikey,*)
c
c  select groups
c
      write (ikey,1300) 1, 1, v11
      write (ikey,1300) 1, 2, v12
      write (ikey,1300) 2, 2, v22

1000  format(/2x,'chargeterm only ',
     *       /2x,'chg-12-scale 1.0 ',
     *       /2x,'chg-13-scale 1.0 ',
     *       /2x,'chg-14-scale 1.0 ',
     *       /2x,'chg-15-scale 1.0 ',/)
1100  format('group 1 ',2(i4,2x))
1200  format('group 2 ',2(i4,2x))
1300  format('group-select ',i2,2x,i2,2x,f16.8)
1400  format('charge  ',i4,2x,f16.8)
      return
      end
c**************************************************************

c*****************************************************************************
c  PROGESTHK
c*****************************************************************************
c
      subroutine progesthk1(prognme, esmeth, esbas, qmcharge,qmmult,
     *            ssgeom, optl, ener, ierr, qmmminp, ssmmgeom)
c
c     Subroutine progesthk
c
c     Created:  Feb. 22, 2004 by Hai Lin
c     Revised:  Mar. 28, 2006 by Yan Zhang
c
c     Purpose:  Get a single point energy for the small system in the 
c        presence of background charges with the
c        specified electronic structure package.
c
c     Description:  Calls a function to write a g03 SPE input file,
c         runs this calculation on the system, and then calls a fxn that
c         reads the energy from the formatted checkpoint file created by
c         g03.
c
c     Note:  Currently only g03 are enabled but it would be easily
c        expanded to other electronic structure programs.  new _inpe and
c        _oute subroutines would have to be written for each different
c        program though.
c
c     Called by:  optehk, qmmmehk
c     Calls:  g03inp, g03oute, mg03oute, system, orcainp, orcaoute
c
c     Variables defined here:
c
      use caltyp
      use files
      use input
      implicit none
c
c i/o variables
c
      character*10       prognme
      character*20       esmeth                      
      character*20       esbas                                                   
c
      character*80       optl(maxoptl)
      double precision   ener
      integer            ierr
      type (cartgeom) :: ssgeom
      type (mmvalen)  :: ssmmgeom   
      type (qmmminf)  :: qmmminp
c
c local variables
c
      character*20 ver_lab                                                 
      integer      qmcharge, qmmult
c
c  GAUSSIAN03
c
c  Add by Bo Wang 0103BW15
cx      if (prognme(:3).eq.'g03') then
cx         open (unit=g03inf, status='unknown', file=g03in, err=9993)
cx         call g03stinp(g03inf, esmeth, esbas, qmcharge, qmmult, ssgeom,  
cx     *      optl, tener, ver_lab, qmmminp, ssmmgeom)

c       if ((prognme(:3).eq.'g03') .or. (prognme(:3).eq.'g09')) then
       if ((prognme(:3).eq.'g03') .or. (prognme(:3).eq.'g09')           1020XW17
     * .or. (prognme(:3).eq.'g16')) then                                1020XW17
         open (unit=g03inf, status='unknown', file=g03in, err=9993)
         if (   qmmminp%ldampchg .ne. 0) then
           call g03stinp1(g03inf, esmeth, esbas, qmcharge, qmmult,
     *      ssgeom, optl, tener, ver_lab, qmmminp, ssmmgeom)
         else
           call g03stinp(g03inf,esmeth,esbas,qmcharge,qmmult,ssgeom,
     *      optl, tener, ver_lab, qmmminp, ssmmgeom)
         endif
c  End by Bo Wang 0103BW15

         close (g03inf, err=9993)
         call system('g03shuttle g03.inp g03.out')

         call g03oute(1,esmeth, ener)  
c
c     ORCA 3/28/2006 by Yan Zhang
c
      else if (prognme(:4).eq.'orca') then
         open (unit=orcainf, status='unknown', file=orcain, err=9994)
         call orcastinp(orcainf, esmeth, esbas, qmcharge,qmmult,ssgeom, 
     *      optl, tener, ver_lab, qmmminp, ssmmgeom)
         close (orcainf, err=9994)
         call system('orcashuttle orca.inp orca.out')
         call orcaoute(1,esmeth, ener)                                      

      endif




      return
9993  write (outfile,*) 'Error creating Gaussian03 SPE input file'
	stop 'progehk 3'


9994  write (outfile,*) 'Error creating Orca SPE input file'
      stop 'progehk 4'
c
c    end 3/28/2006 by Yan Zhang
c

      end
c
c
c*****************************************************************************
c  G03STINP
c*****************************************************************************
c
      subroutine g03stinp(iunit,esmeth,esbas,qmcharge, qmmult,
     *   ssgeom,optl,type,ver_lab, qmmminp, ssmmgeom)
c
c     Subroutine g03stinp
c     
c     Created:  Feb 22, 2004 by Hai Lin
c     Revised:  Aug. 4, 2006 by Yan Zhang 
c
c     Purpose:  Write a Gaussian03 input file for the calculation of 
c        the capped small system in presence of background point charges
c        Type indicates whether it is an energy, gradient, or
c        hessian calculation. 
c                                                                       
c
c     Called by:  progehk, mprogehk, progghk, proghhk, progohk
c     Calls:
c
      use files
      use caltyp                                                        
      use input
      implicit none
c
c i/o variables
c
      character*20 ver_lab
      character*20 esmeth
      character*20 esbas
c
      character*80 optl(maxoptl)   
      integer      iunit, type
      type (cartgeom) :: ssgeom   
      type (mmvalen)  :: ssmmgeom   
      type (qmmminf)  :: qmmminp
c
c local variables
c
      logical      linkzero
      integer      mfin, bfin, mend, bend, i, j, k
      
      integer      ii, jj, kk
      integer      lsymbol, ltypname
      integer      qmnatm,capnatm,cssnatm,lsnatm
      integer      ssnatm,bgchnatm,mmnatm
      
      character*20 oldbas	    
      character*12 tswit(4)

c
c added by Xin-Ping Wu, 07/25/2018
      integer  istrt
      integer  ierr
      integer  ixp
      character*80  basispath
c end by Xin-Ping Wu, 07/25/2018
c
c
c Revised by Hai Lin, 5/3/2009 
c   to add cartesian keyword for oniom gradient calculations
c
c      character*25 typekey1, typekey2, typekey3, typekey4
      character*40 typekey1, typekey2, typekey3, typekey4
c
c End by Hai Lin, 5/3/2009
c
      character*40 mmkey, connkey
      character*16 chkkey                                               1020XW17
      character*12 oniomkey, mmprmkey 
      character*4  keykey
      character*2  atmsymbol
      character*4  typname
      
      double precision  charge

      integer qmcharge, qmmult

      data tswit /'Energy', 'Gradient', 'Hessian', 'Optimization'/
      
      typekey1 = 'sp '
c
c Revised by Hai Lin, 5/3/2009
c   to add cartesian keyword for oniom gradient calculations
c
c     typekey2 = 'opt=(maxcyc=1,nomicro) '
      typekey2 = 'opt=(maxcyc=1,nomicro,cartesian) '
c
c End by Hai Lin, 5/3/2009
c
      typekey3 = 'freq '
      typekey4 = 'opt=(nomicro) '                 
      
      keykey   = '#p '
      oniomkey = 'oniom( '
      mmkey    = ':amber=softfirst)=(scalecharge=555) '
c      connkey  = 'fchk geom=connectivity nosym '                       1020XW17
      connkey  = 'geom=connectivity nosym '                             1020XW17
c
c added by Xin-Ping Wu, 10/20/2017
c
      chkkey   = '%chk=Test.chk'
c
c end by Xin-Ping Wu, 10/20/2017
c
c Revised by Hai Lin, 5/11/2009
c
c      mmprmkey = 'NonBon  0 0 '
      mmprmkey = 'NonBon  0 1 '
c
c End by Hai Lin, 5/11/2009
c

      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      ssnatm   = ssmmgeom%natoms
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + mmnatm
      bgchnatm = ssnatm - lsnatm
      
      mfin = 1
      bfin = 1
      mend = 0
      bend = 0	 

      write(outfile,999) tswit(type), esmeth, esbas
 999  format('Running Gaussian Job:',3x,a,3x,a10,3x,a10)

c
c  If the basis is pDZ+,g3large, mg3, or mg3s change the basis
c    specification to Gen so that the non-standard basis will be read in
c    from the foot of the input file. 
c
      if (esbas(:8).eq.'pdz+' .or. esbas(:3).eq.'mg3'
     *     .or. esbas(:7).eq.'g3large') then
         oldbas = esbas
         esbas = 'gen     '
      endif
c
c  Find ends of method and basis set specification
c
      call fspace(esmeth, mfin, mend)
      if (mend.eq.1) then
         mfin = len(esmeth)
      else
         mfin = mfin-1
      endif

      call fspace(esbas, bfin, bend)
      if (bend.eq.1) then
         bfin = len(esbas)
      else
         bfin = bfin-1
      endif
c
c     Memory and processors (specified in the user inputed options)      
c     linkzero is a logical (boolean) variable to determine if the current
c     line being read is a link0 command or a keyword option
c
      linkzero = .false.
      i = 1
      do 10 j=1,10
      if (optl(i)(j:j) .eq. '%' ) then
      linkzero = .true.
      endif
10    CONTINUE

      do while (linkzero .and. i.le.maxoptl)
         write(iunit,*) optl(i)
         i = i+1
         linkzero = .false.
          do 12 j=1,10
            if (optl(i)(j:j) .eq. '%' ) then
               linkzero = .true.
            endif
12    CONTINUE
      end do

c
c added by Xin-Ping Wu, 10/20/2017
c to generate the gaussian checkpoint file
c using the fchk keyword in G16 cannot generate a formatted
c checkpoint file including energy, gradient, and hessian
c 
      write(iunit,*) chkkey
c
c end by Xin-Ping Wu, 10/20/2017
c
c  Oniom keywords
c
      if( type .eq. tener ) then
         write(iunit,2000) keykey, typekey1(:3), oniomkey(:7), 
     *      esmeth(:mfin), '/', esbas(:bfin), mmkey(:36), connkey(:29)
      elseif( type .eq. tgrad ) then
c
c Revised by Hai Lin, 5/3/2009
c   to add cartesian keyword for oniom gradient calculations
c
c         write(iunit,2000) keykey, typekey2(:23), oniomkey(:7),
         write(iunit,2000) keykey, typekey2(:33), oniomkey(:7),
     *      esmeth(:mfin), '/', esbas(:bfin), mmkey(:36), connkey(:29)
c
c End by Hai Lin, 5/3/2009 
c
      elseif( type .eq. thess ) then
         write(iunit,2000) keykey, typekey3(:5), oniomkey(:7), 
     *      esmeth(:mfin), '/', esbas(:bfin), mmkey(:36), connkey(:29)
      elseif( type .eq. topt ) then
         write(iunit,2000) keykey, typekey4(:14), oniomkey(:7), 
     *      esmeth(:mfin), '/', esbas(:bfin), mmkey(:36), connkey(:29)
      endif
2000  format(a4,a,a,a,a1,a,a,/,'# ',a)
c
c  User inputed keywords
c
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
c
         do j=1, 79
            if (optl(i)(j:j) .eq. '!') then
               k = 1
            endif
         enddo
         if (k .eq. 0) then
            write(iunit,3000) '#', optl(i)
         endif
         i = i+1
      end do
c
3000  format (a1,a79)      
c
c  Title
c
      if (type .eq. tener) then
         write(iunit,3021) 
      elseif (type .eq. tgrad) then
         write(iunit,3022)
      elseif (type .eq. thess) then
         write(iunit,3023)
      elseif (type .eq. topt) then
         write(iunit,3024)
      endif

3021  format (/'Single Point Energy Calculation'/)
3022  format (/'Gradient Calculation'/)
3023  format (/'Hessian Calculations'/)
3024  format (/'External Optimization'/)
c
c  Charge and multiplicity
c
      write(iunit,3050) qmcharge, qmmult
3050  format (1x,i3,1x,i3)      
c
c  QM: atomic symbols and coordinates
c      atom type = 'CT' defined parameter

      do i=1, qmnatm
         lsymbol = 0
         atmsymbol = ssgeom%atm(i)	    	 
         do j=1, 2
            if (atmsymbol(j:j) .ne. ' ') then 
               lsymbol = lsymbol + 1
            endif 
         enddo

         ltypname = 2
         typname  = qmtypname        
c         ltypname = 0      
c         typname = ssmmgeom%atmtypname(i)
c         do j=1, 4
c            if (typname(j:j) .ne. ' ') then 
c               ltypname = ltypname + 1
c            endif
c         enddo
         
         write(iunit,4000)
     *      atmsymbol(:lsymbol), typname(:ltypname), 
     *      ssmmgeom%atmcharge(i),  ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)

      enddo     
4000  format (a,'-',a,'-',d10.4,t20,'0 ',3(d16.9,3x),'H')
c
c  Cap: atomic symbols and coordinates
c  Add by Bo Wang 0103BW15
cx       take H as symbol and HC as atomtype, as defined parameters
c        take the symbol as defined in the input and HC as atomtype
      do i = qmnatm + 1, cssnatm
         atmsymbol = ssgeom%atm(i)  !0103BW08 get the atomic symbol for the link atom, not just 'H'
cx         atmsymbol = capname
cx         lsymbol   = 1
         lsymbol   = 0
         do j=1, 2
            if (atmsymbol(j:j) .ne. ' ') then
               lsymbol = lsymbol + 1
            endif
         enddo
c End by Bo Wang
         typname   = captypname
         ltypname  = 2
         write(iunit,4000)
     *      atmsymbol(:lsymbol), typname(:ltypname),
     *      ssmmgeom%atmcharge(i),  ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)
      enddo
c
c  MM and M1/auxiliary charges: atomic symbols and coordinates
c     take He as symbol and HC as atomtype, as defined parameters     
c
      do i=cssnatm+1,ssnatm
         atmsymbol = chargename
         lsymbol   = 2
         typname   = chargetypname
         ltypname  = 2
         write(iunit,4100)
     *      atmsymbol(:lsymbol), typname(:ltypname),
     *      ssmmgeom%atmcharge(i),  ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)           	 	 
      enddo
      
      write(iunit,*)
4100  format (a,'-',a,'-',d10.4,t20,'0 ',3(d16.9,3x),'L')
c
c  Connectivity: only those for cap atoms are provided
c
c
c Revised by Hai Lin, 5/11/2009
C No longer need a bond between 1st and 2nd atoms, since NoBond is set to 0 1
c
c  revised by Hai Lin, 5/3/2009
c  add a bond between the 1st and 2nd QM atoms
c
c      do i=1, qmnatm
c         write (iunit,4500) i
c      enddo	 

c      write (iunit,4500) 1, 2, 1.0
c      do i=2, qmnatm
      do i=1, qmnatm
         write (iunit,4500) i
      enddo
c
c  end by Hai Lin, 5/3/2009
c End by Hai Lin, 5/11/2009 
c
      do i=qmnatm+1, cssnatm
         jj = ssmmgeom%atmvalenlist(i,1)
         do j=1, qmnatm
            if (jj .eq. qmmminp%qmatm(j)) then
               write (iunit,4500) i, j, 1.0 
            endif
         enddo
      enddo	     	        
      
      do i=cssnatm+1, ssnatm
         write (iunit,4500) i
      enddo	 
      
      write(iunit,*)          
4500  format (2x,i4,2x,i4,2x,f4.2)
c
c write additional options for molecule specification                   
c
      i = 1
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
         jj = 1
c
         do j=1, 79
            if (optl(i)(j:j+1) .eq. '! ') then
               k = 1
               jj = j + 1
            endif
         enddo
         if (k .eq. 1) then
c
            write(iunit,5000) optl(i)(jj:80)
         endif
         i = i + 1
      enddo
c      write(iunit,*)
cx 5000  format (a79)    !1014BW09
5000  format (a)         !1014BW09

c
c  Include basis set file at end of the input file if needed
c
      if (esbas(:3).eq.'gen') then
         if (oldbas(:8).eq.'pdz+') then
            write (iunit, 1051) '@', pDZsloc
         elseif (oldbas(:4).eq.'mg3s') then
            write (iunit, 1051) '@', mg3sloc
         elseif (oldbas(:3).eq.'mg3') then
            write (iunit, 1051) '@', mg3loc
         elseif (oldbas(:7).eq.'g3large') then
            write (iunit, 1051) '@', g3Lloc
         endif
      endif
1051  format (2a,/)
c
c  Change the basis set name back if they've been changed
c
      if (esbas(:3).eq.'gen') then
         esbas = oldbas
      endif
c
c  NonBon key
c
      write(iunit,*) mmprmkey

c
c    Write additional options for molecule specification behind "nonbon"                   
c    The keyword is labeled by "!2"
c       8/4/2006 by Yan Zhang
c
      write(iunit,*)
      i = 1
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
         jj = 1
c
         do j=1, 79
            if (optl(i)(j:j+1) .eq. '!2') then
               k = 1
               jj = j + 2
            endif
         enddo
         if (k .eq. 1) then
c
c added by Xin-Ping Wu, July 25, 2018
c            write(iunit,5000) optl(i)(jj:80)
            if (optl(i)(jj:jj) .eq. '@') then
               call scase (81-jj,optl(i)(jj:80),optl(i)(jj:80))
               write(iunit,5000) optl(i)(jj:80)
            else
               write(iunit,5000) optl(i)(jj:80)
            endif
c end by Xin-Ping Wu, July 25, 2018
c
         endif
         i = i + 1
      enddo
c
c     end 8/4/2006 by Yan Zhang
c  Close the file
c
      write(iunit,*)
      write(iunit,*)      
      close (iunit, err=9999)
      
      return

9999  write (outfile,*) 'Error creating Gaussian input file'
      stop 'g03inp'

1001  format (/)
1010  format (2a)
1011  format (4a,3(3x,a))
1012  format (4a,2(3x,a))

      end
c
c
c*****************************************************************************

c*****************************************************************************
c  G03STINP1
c*****************************************************************************
c
      subroutine g03stinp1(iunit,esmeth,esbas,qmcharge, qmmult,  !0209BW10 new subroutine
     *   ssgeom,optl,type,ver_lab, qmmminp, ssmmgeom)
c
c     Subroutine g03stinp1
c
c     Created:  Jan 28,2010 by Bo Wang
c               from g03stinp
c
c     Purpose:  Write a Gaussian03 input file for the calculation of
c        the capped small system in presence of background screened charges
c        or the smeared charges. The screened charges and smeared charges
c        are represented by the pseudopotentials.
c        Type indicates whether it is an energy, gradient, or
c        hessian calculation.
c
c     Called by:  progehk, mprogehk, progghk, proghhk, progohk
c     Calls:
c
      use files
      use caltyp
      use input
      use atminfo
      use damping
      implicit none
c
c i/o variables
c
      character*20 ver_lab
      character*20 esmeth
      character*20 esbas
c
      character*80 optl(maxoptl)
      integer      iunit, type
      type (cartgeom) :: ssgeom
      type (mmvalen)  :: ssmmgeom
      type (qmmminf)  :: qmmminp
c
c local variables
c
      logical      linkzero
      integer      mfin, bfin, mend, bend, i, j, k

      integer      ii, jj, kk
      integer      lsymbol, ltypname
      integer      qmnatm,capnatm,cssnatm,lsnatm
      integer      ssnatm,bgchnatm,mmnatm

      character*20 oldbas
      character*12 tswit(4)
c
c
c Revised by Hai Lin, 5/3/2009
c   to add cartesian keyword for oniom gradient calculations
c
c      character*25 typekey1, typekey2, typekey3, typekey4
      character*40 typekey1, typekey2, typekey3, typekey4
c
c End by Hai Lin, 5/3/2009
c
      character*40 mmkey, connkey
      character*16 chkkey                                               1020XW17
      character*12 oniomkey, mmprmkey
      character*4  keykey
      character*2  atmsymbol
      character*4  typname
      double precision  charge

      integer qmcharge, qmmult
      integer index,nterm,npower(50)
c  number of terms in PP and the power of r
      double precision zeta0,dampzeta,dampval,dampchg
c the zeta value and number of valence electrons
      double precision lambda, a0
c width of smeared charge and the 1 bohr radius
      double precision stocoeff(1:15,1:15),stoexp(1:15,1:15)
c convert factor to change STO to GTO, maximum 15 GTO
      integer ngto1,ngto2
c  number of GTO used to expand STO for real atoms and screened charges
      integer dampatm
c an integer to decide if the screening scheme is used for the atom, need to change the number of MM atoms
      double precision acharge,rho
      double precision rcwidth
c  the width of STO for redistributed charges
      double precision exponent(50),coeff(50)
c exponent and coeff for each ECP term, now the maximum number of terms is 18
      parameter (a0=0.5291772)

c factors to convert STO into 3 GTO
      data (stocoeff(3,index),index=1,3)/
     &  0.107150,0.343808,0.355483 /
      data (stoexp(3,index),index=1,3)/
     &  0.109818,0.405771,2.22766 /
c factors to convert STO into 6 GTO
      data (stocoeff(6,index),index=1,6)/
     &  0.021221,0.131906,0.238573,0.241818,0.184113,0.121984 /
      data (stoexp(6,index),index=1,6)/
     &  0.065110,0.158088,0.407099,1.185060,4.235920,23.103000 /
c factors to convert STO into 9 GTO
      data (stocoeff(9,index),index=1,9)/
     &  0.047713,0.074983,0.111897,0.154809,0.191480,0.195519,
     &  0.141046,0.053483,0.005656 /
      data (stoexp(9,index),index=1,9)/
     & 153.776713,28.206753,7.899558,2.721478,1.067073,0.457904,
     &  0.209886,0.100646,0.048724  /
c factors to convert STO into 12 GTO
      data (stocoeff(12,index),index=1,12)/
     &  0.020772,0.032834,0.049904,0.072474,0.100549,
     &  0.131966,0.159730,0.169464,0.143624,0.082581,
     &  0.024025,0.001897  /
      data (stoexp(12,index),index=1,12)/
     & 813.469054,149.220649,41.795981,14.402813,5.650086,
     & 2.427297,1.116125,0.541102,0.273581,0.142917,
     & 0.076222,0.040309  /
c factors to convert STO into 15 GTO
      data (stocoeff(15,index),index=1,15)/
     &  0.008428,0.013337,0.020344,0.029831,0.042355,0.058519,0.078820,
     &  0.103134,0.129253,0.150272,0.152837,0.123095,0.066428,0.017917,
     &  0.001299/
      data (stoexp(15,index),index=1,15)/
     &  4943.800944,906.886649,254.017045,87.530509,34.327684,
     &  14.730750,6.751351,3.248735,1.620678,0.831025,0.435907,0.233351,
     &  0.127172,0.070064,0.038035/

c Other parameters
      rcwidth=qmmminp%lambda/a0    !0410BW10 set the width of redistributed charges, change to atomic unit

      data tswit /'Energy', 'Gradient', 'Hessian', 'Optimization'/
      typekey1 = 'sp '
c
c Revised by Hai Lin, 5/3//2009
c   to add cartesian keyword for oniom gradient calculations
c
c     typekey2 = 'opt=(maxcyc=1,nomicro) '
      typekey2 = 'opt=(maxcyc=1,nomicro,cartesian) '
c      typekey2 = 'force '
c
c End by Hai Lin, 5/3/2009
c 
      typekey3 = 'freq '
      typekey4 = 'opt=(nomicro) '

      keykey   = '#p '
      mmkey    = '  pseudo=read    '
c      connkey  = 'fchk nosym '                                         1020XW17
      connkey  = 'nosym '                                               1020XW17
c
c added by Xin-Ping Wu, 10/20/2017
c
      chkkey  = '%chk=Test.chk'
c
c end by Xin-Ping Wu, 10/20/2017
c

      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      ssnatm   = ssmmgeom%natoms
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + mmnatm
      bgchnatm = ssnatm - lsnatm
      ngto1    = qmmminp%ngto1
      ngto2    = qmmminp%ngto2

      mfin = 1
      bfin = 1
      mend = 0
      bend = 0

      write(outfile,999) tswit(type), esmeth, esbas
 999  format('Running Gaussian Job:',3x,a,3x,a10,3x,a10)

c
c  If the basis is pDZ+,g3large, mg3, or mg3s change the basis
c    specification to Gen so that the non-standard basis will be read in
c    from the foot of the input file.
c 
      if (esbas(:8).eq.'pdz+' .or. esbas(:3).eq.'mg3'
     *     .or. esbas(:7).eq.'g3large') then
         oldbas = esbas
         esbas = 'gen     '
      endif
      if (esbas(:3) .eq. 'gen') then
         oldbas='gen'
      endif
c     
c  Find ends of method and basis set specification
c
      call fspace(esmeth, mfin, mend)
      if (mend.eq.1) then
         mfin = len(esmeth)
      else
         mfin = mfin-1
      endif

      call fspace(esbas, bfin, bend)
      if (bend.eq.1) then
         bfin = len(esbas)
      else
         bfin = bfin-1
      endif
c
c     Memory and processors (specified in the user inputed options)
c     linkzero is a logical (boolean) variable to determine if the current
c     line being read is a link0 command or a keyword option
      linkzero = .false.
      i = 1
      do 10 j=1,10
      if (optl(i)(j:j) .eq. '%' ) then
      linkzero = .true.
      endif
10    CONTINUE

      do while (linkzero .and. i.le.maxoptl)
         write(iunit,*) optl(i)
         i = i+1
         linkzero = .false.
          do 12 j=1,10
            if (optl(i)(j:j) .eq. '%' ) then
               linkzero = .true.
            endif
12    CONTINUE
      end do
c
c added by Xin-Ping Wu, 10/20/2017
c 
      write(iunit,*) chkkey
c
c end by Xin-Ping Wu, 10/20/2017
c
c
c  charge keywords
c
      if( type .eq. tener ) then
         write(iunit,2000) keykey, typekey1(:3),
     *      esmeth(:mfin), '/', esbas(:bfin), mmkey(:36),connkey(:29)
      elseif( type .eq. tgrad ) then
c
c Revised by Hai Lin, 5/3/2009
c   to add cartesian keyword for oniom gradient calculations
c
c         write(iunit,2000) keykey, typekey2(:23),
        write(iunit,2000) keykey, typekey2(:33),
c         write(iunit,2000) keykey, typekey2(:6),
     *      esmeth(:mfin), '/', esbas(:bfin), mmkey(:36), connkey(:29) 
c
c End by Hai Lin, 5/3/2009
c
      elseif( type .eq. thess ) then
         write(iunit,2000) keykey, typekey3(:5),
     *      esmeth(:mfin), '/', esbas(:bfin), mmkey(:36), connkey(:29)
      elseif( type .eq. topt ) then
         write(iunit,2000) keykey, typekey4(:14),
     *      esmeth(:mfin), '/', esbas(:bfin), mmkey(:36), connkey(:29)
      endif
2000  format(a4,a,a,a1,a,a,/,'# ',a)
c
c  User inputed keywords
c
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
c
         do j=1, 79
            if (optl(i)(j:j) .eq. '!') then
               k = 1
            endif
         enddo
         if (k .eq. 0) then
            write(iunit,3000) '#', optl(i)
         endif
         i = i+1
      end do
c
3000  format (a1,a79)
c
c  Title
c
      if (type .eq. tener) then
         write(iunit,3021)
      elseif (type .eq. tgrad) then
         write(iunit,3022)
      elseif (type .eq. thess) then
         write(iunit,3023)
      elseif (type .eq. topt) then
         write(iunit,3024) 
      endif

3021  format (/'Single Point Energy Calculation'/)
3022  format (/'Gradient Calculation'/)
3023  format (/'Hessian Calculations'/)
3024  format (/'External Optimization'/)
c
c
c  Charge and multiplicity
c
      write(iunit,3050) qmcharge, qmmult
3050  format (1x,i3,1x,i3)
c
c  QM and cap atoms: atomic symbols and coordinates
c
      do i=1,cssnatm
         lsymbol = 0
         atmsymbol = ssgeom%atm(i)
         do j=1, 2
            if (atmsymbol(j:j) .ne. ' ') then
               lsymbol = lsymbol + 1
            endif
         enddo 
         write(iunit,4000) 
     *      atmsymbol(:lsymbol),ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)
      enddo
4000  format (a,3x,3(d16.9,3x))
c   

c
c  MM and M1/auxiliary charges: atomic symbols and coordinates
c     take He as symbol
c
      do i=cssnatm+1,ssnatm
         atmsymbol = chargename
         lsymbol   = 2
         write(iunit,4000) 
     *      atmsymbol(:lsymbol),ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)
      enddo

      write(iunit,*)

c
c write additional options for molecule specification
c
      i = 1
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
         jj = 1
c
         do j=1, 79
           if (optl(i)(j:j+1) .eq. '! ') then
             k = 1
             jj = j + 1
           endif
         enddo
         if (k .eq. 1) then
c
            write(iunit,5000) optl(i)(jj:80)
         endif
         i = i + 1
      enddo
c      write(iunit,*)
cx5000  format (a79)
5000  format (a)
c
c  Include basis set file at end of the input file if needed
c
      if (esbas(:3).eq.'gen') then
         if (oldbas(:8).eq.'pdz+') then
            write (iunit, 1051) '@', pDZsloc
         elseif (oldbas(:4).eq.'mg3s') then
            write (iunit, 1051) '@', mg3sloc
         elseif (oldbas(:3).eq.'mg3') then
            write (iunit, 1051) '@', mg3loc
         elseif (oldbas(:7).eq.'g3large') then
            WRITE (iunit, 1051) '@', g3Lloc
         endif
      endif
1051  format (2a,/)
c
c  Change the basis set name back if they've been changed
c
      if (esbas(:3).eq.'gen') then
         esbas = oldbas
      endif
c
c    Write additional options for molecule specification
c    The keyword is labeled by "!2"
c       8/4/2006 by Yan Zhang
c
      i = 1
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
         jj = 1
c
         do j=1, 79
            if (optl(i)(j:j+1) .eq. '!2') then
               k = 1
               jj = j + 2
            endif
         enddo
         if (k .eq. 1) then
c
c added by Xin-Ping Wu, July 25, 2018
c            write(iunit,5000) optl(i)(jj:80)
            if (optl(i)(jj:jj) .eq. '@') then
               call scase (81-jj,optl(i)(jj:80),optl(i)(jj:80))
               write(iunit,5000) optl(i)(jj:80)
            else
               write(iunit,5000) optl(i)(jj:80)
            endif
c end by Xin-Ping Wu, July 25, 2018
c
         endif
         i = i + 1
      enddo

c      write(iunit,*)  

c  Write ECP  for MM and redistributed charges
c  For MM charges, use He ECP to represent the potential
c  the potential is depended on the MM charge and MM atomic number
c  For the redistributed charges, use He ECP to represent the potential
c  the potential is only related to the redistributed charge.
c

      do i=cssnatm+1,ssnatm
         atmsymbol = ssgeom%atm(i)
c dummy atom, smear the charge
         if (atmsymbol .eq. dummyname) then !0130BW10  dummy atom, only smear the charge (such as the redistributed charges)
           acharge=ssmmgeom%atmcharge(i)
           if (qmmminp%ldampchg .eq. 1 
     &             .or. qmmminp%ldampchg .eq. 3) then 
             nterm=2*ngto2                                            
             do k=1,nterm
               if (k .le. ngto2) then
                 exponent(k)=stoexp(ngto2,k)
                 coeff(k)=stocoeff(ngto2,k)
                 npower(k)=1 
               elseif (k.le. 2*ngto2) then
                 exponent(k)=stoexp(ngto2,k-ngto2)
                 coeff(k)=1.0/rcwidth*stocoeff(ngto2,k-ngto2) !0206BW10 rcwidth is the width of the redistributed charges in atomic unit.
                 npower(k)=2
               endif
               exponent(k) = exponent(k) * (2.0/rcwidth)**2
               coeff(k) = coeff(k) * acharge
             enddo
           else    !0813BW10 no damping for the dummy atom
             nterm=0
           endif
         else
c charges on atoms, use point charges or screened charges
           k  = ssmmgeom%atmid(i)
           dampatm=0
           do ii=1,qmmminp%dampnatm  !0131BW10 find out if the MM charge should be screened
             jj = qmmminp%dampatm(ii)
             if (k .eq. jj) then
              dampatm=1
              dampzeta=qmmminp%dampzeta(ii)  !0223BW10  transfer the zeta value for the screening MM atoms
              dampval=qmmminp%dampval(ii)    !0408BW10  transfer the number of valence electrons for the screening MM atoms
              dampchg=qmmminp%dampchg(ii)    !0415BW10  transfer the extra charge for the screening MM atoms
             endif
           enddo
           if (dampatm .eq. 0) then          !0131BW10 MM charge on atom i is not screened
              nterm=0
           else
             do j=1,36                       !0204BW10 find out the atomic number of the MM atom
               if (asymbu(j) .eq. atmsymbol) then
                 if (dampval .ne. -1.0) then
                   if (dampchg .ne. 999.0) then                     
                     acharge=-dampval+dampchg
                   else
                     acharge=-dampval+ssmmgeom%atmcharge(i)
                   endif
                 else
                   if (j .eq. 1) then                         !0616BW14 H atom
                     acharge=-scnval(j)+ssmmgeom%atmcharge(i)
                   else
                     acharge=-scnval(j)
                   endif
                 endif
                 if (dampzeta .ne. -1.0) then
                   zeta0=dampzeta
                 else
                   zeta0=scnzeta(j)
                 endif
                 if (j .le. 2) then                           !0204BW10 H,He
                   nterm=2*ngto1
                   do k=1,nterm
                     if (k .le. ngto1) then
                       exponent(k)=stoexp(ngto1,k)
                       coeff(k)=stocoeff(ngto1,k)
                       npower(k)=1
                     elseif (k.le. 2*ngto1) then
                       exponent(k)=stoexp(ngto1,k-ngto1)
                       coeff(k)=zeta0*stocoeff(ngto1,k-ngto1)
                       npower(k)=2
                     endif
                     exponent(k) = exponent(k) * (2.0*zeta0)**2
                     coeff(k) = coeff(k) * acharge
                   enddo
                 elseif (j .le. 10) then                       !0204BW10 Li~Ne
                   nterm=4*ngto1
                   do k=1,nterm
                     if (k .le. ngto1) then
                       exponent(k)=stoexp(ngto1,k)
                       coeff(k)=stocoeff(ngto1,k)
                       npower(k)=1
                     elseif (k.le. 2*ngto1) then
                       exponent(k)=stoexp(ngto1,k-ngto1)
                       coeff(k)=3.0/2.0*zeta0*stocoeff(ngto1,k-ngto1)
                       npower(k)=2
                     elseif (k .le. 3*ngto1) then
                       exponent(k)=stoexp(ngto1,k-2*ngto1)
                       coeff(k)=zeta0**2*stocoeff(ngto1,k-2*ngto1)
                       npower(k)=3
                     elseif (k .le. 4*ngto1) then
                       exponent(k)=stoexp(ngto1,k-3*ngto1)
                       coeff(k)=1.0/3.0*zeta0**3
     &                                   *stocoeff(ngto1,k-3*ngto1)
                       npower(k)=4
                     endif
                     exponent(k) = exponent(k) * (2.0*zeta0)**2
                     coeff(k) = coeff(k) * acharge
                   enddo
                 elseif (j .le. 20) then                      !0204BW10 Na~Ar
                   nterm=6*ngto1
                   do k=1,nterm
                     if (k .le. ngto1) then
                       exponent(k)=stoexp(ngto1,k)
                       coeff(k)=stocoeff(ngto1,k)
                       npower(k)=1
                     elseif (k.le. 2*ngto1) then
                       exponent(k)=stoexp(ngto1,k-ngto1)
                       coeff(k)=5.0/3.0*zeta0*stocoeff(ngto1,k-ngto1)
                       npower(k)=2
                     elseif (k .le. 3*ngto1) then
                       exponent(k)=stoexp(ngto1,k-2*ngto1)
                       coeff(k)=4.0/3.0*zeta0**2
     &                                     *stocoeff(ngto1,k-2*ngto1)
                       npower(k)=3
                     elseif (k .le. 4*ngto1) then
                       exponent(k)=stoexp(ngto1,k-3*ngto1)
                       coeff(k)=2.0/3.0*zeta0**3
     &                                    *stocoeff(ngto1,k-3*ngto1)
                       npower(k)=4
                     elseif (k .le. 5*ngto1) then
                       exponent(k)=stoexp(ngto1,k-4*ngto1)
                       coeff(k)=2.0/9.0*zeta0**4
     &                                    *stocoeff(ngto1,k-4*ngto1)
                       npower(k)=5
                     elseif (k .le. 6*ngto1) then
                       exponent(k)=stoexp(ngto1,k-5*ngto1)
                       coeff(k)=2.0/45.0*zeta0**5
     &                                   *stocoeff(ngto1,k-5*ngto1)
                       npower(k)=6
                     endif
                     exponent(k) = exponent(k) * (2.0*zeta0)**2
                     coeff(k) = coeff(k) * acharge
                   enddo
                 elseif (j .le. 36) then                           !0615BW10 K~Kr
                   nterm=8*ngto1
                   do k=1,nterm
                     if (k .le. ngto1) then
                       exponent(k)=stoexp(ngto1,k)
                       coeff(k)=stocoeff(ngto1,k)
                       npower(k)=1
                     elseif (k.le. 2*ngto1) then
                       exponent(k)=stoexp(ngto1,k-ngto1)
                       coeff(k)=7.0/4.0*zeta0*stocoeff(ngto1,k-ngto1)
                       npower(k)=2
                     elseif (k .le. 3*ngto1) then
                       exponent(k)=stoexp(ngto1,k-2*ngto1)
                       coeff(k)=3.0/2.0*zeta0**2
     &                                     *stocoeff(ngto1,k-2*ngto1)
                       npower(k)=3
                     elseif (k .le. 4*ngto1) then
                       exponent(k)=stoexp(ngto1,k-3*ngto1)
                       coeff(k)=5.0/6.0*zeta0**3
     &                                    *stocoeff(ngto1,k-3*ngto1)
                       npower(k)=4  
                     elseif (k .le. 5*ngto1) then
                       exponent(k)=stoexp(ngto1,k-4*ngto1)
                       coeff(k)=1.0/3.0*zeta0**4
     &                                    *stocoeff(ngto1,k-4*ngto1)
                       npower(k)=5
                     elseif (k .le. 6*ngto1) then
                       exponent(k)=stoexp(ngto1,k-5*ngto1)
                       coeff(k)=1.0/10.0*zeta0**5
     &                                    *stocoeff(ngto1,k-5*ngto1)
                       npower(k)=6
                     elseif (k .le. 7*ngto1) then
                       exponent(k)=stoexp(ngto1,k-6*ngto1)
                       coeff(k)=1.0/45.0*zeta0**6
     &                                    *stocoeff(ngto1,k-6*ngto1)
                       npower(k)=7
                     elseif (k .le. 8*ngto1) then
                       exponent(k)=stoexp(ngto1,k-7*ngto1)
                       coeff(k)=1.0/315.0*zeta0**7
     &                                   *stocoeff(ngto1,k-7*ngto1)
                       npower(k)=8
                     endif
                     exponent(k) = exponent(k) * (2.0*zeta0)**2
                     coeff(k) = coeff(k) * acharge
                   enddo
                 endif    
                 exit
               endif
             enddo
           endif
         endif

c
c write the ECP                                                       
c
         write(iunit,4100) i
         write(iunit,4200) 
         write(iunit,4300)
         write(iunit,4400) nterm+1
         write(iunit,4500) -ssmmgeom%atmcharge(i)
         do j=1, nterm
            write(iunit,4600) npower(j), exponent(j),coeff(j)
         enddo
         write(iunit,4700)                                              
      enddo

4100  format (1x,i3,1x,'0')
4200  format (1x,'ECP 1 2')                                              
4300  format (1x,'ALL')
4400  format (1x, i3) 
4500  format (4x,'1 0.0',3x,d14.6)                                      
4600  format  (1x,i3,1x,2(d16.6,3x))                                    
4700  format (1x, 'S-part', /, 1x, '1', /, 4x, '1 1.0 0.0')             

c     end 8/4/2006 by Yan Zhang
c  Close the file
      write(iunit,*)
      write(iunit,*)
      close (iunit, err=9999)

      return

9999  write (outfile,*) 'Error creating Gaussian input file'
      stop 'g03inp'

1001  format (/)
1010  format (2a)
1011  format (4a,3(3x,a))
1012  format (4a,2(3x,a))
      end
c
c
c*****************************************************************************

