c
c
c******************************************************************************
c  FBBPC
c******************************************************************************
c
      subroutine fbbpc(geom, mmgeom, ssgeom, ssmmgeom, qmmminp, 
     *                qmprognme, px) 

c     Subroutine fbbpc
c
c     Created:  Aug. 30, 2007 by Yan Zhang
c     Revised:  Apr. 15, 2009 by Yan Zhang
c                              
c               
c     Purpose: This subroutine calculates the added state weighting in the two states
c              flexible-boundary treatment base on the QEQRG, QEQBT and EEM method. 
c               
c
c     Called by: qmmmehk  
c     Calls: chkinp, difimu, gqtot, fbgacoor, dvarzero, polgchargpol, updatcharg,
c            fbpot, fbgpot, dqcon, calpx, fbgqtot, convg, muvspx 
c  
      use caltyp
      use input  
      use files  
      use energy 
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, ssmmgeom 
      type (qmmminf)  :: qmmminp

      character*10       qmprognme
      double precision   px
c
c local variables
c      
      integer  ii, j, ssnatm, icy
      integer  ng, idg, idfbg, it, ifconvg
      integer  maxstep, nfbga                
      integer  qmis, qmas
      character*2  fbatm(maxganum) 
      character*9  ccmeth
      double precision  muss, temprt, temprt1, qtot, qtot1
      double precision  maxdq, rmsdq, maxgdq, rmsgdq
      double precision  fbqtot, dqtot, tfq, fbqtot0, fbqtot1
      double precision  xg(maxganum), yg(maxganum), zg(maxganum)
      double precision  pot(maxatm), gpot(maxganum),qn(maxganum)
      double precision  qt(maxatm), de, corrmu, qcps               

      integer debug
c
c
      debug = 0
c
c  Check input variables in the flexible-boundary treatment. 
c
      if (qmmminp%embed .eq. 'mechanical') goto 9000     
      open(unit=chargfile,file=chargoutf,status='unknown')
      call chkinp(qmmminp)
c
c
      write(chargfile, 100)
      write(chargfile, 110)

      write(outfile, 100)
      write(outfile, 110)

100   format(80('-'))
110   format('Flexible-boundary calculation of two states')
c
c  Prepare parameters.
c
      ccmeth  = qmmminp%fbccmeth
      maxstep = qmmminp%maxfbcycl
      ng      = qmmminp%fbgnum
      idfbg   = qmmminp%fbgid
      ssnatm  = ssmmgeom%natoms
      qmis    = qmmminp%qmcharge
      qmas    = qmmminp%asqmcharge
      temprt  = qmmminp%temprt
c
c
      px      = 0.d0
      it      = 1
      ifconvg = 0
      tfq     = 0.d0
      qtot    = 0.d0
      qtot1   = 0.d0
      dqtot   = 0.d0
      muss    = 0.d0
      temprt1 = 0.d0
c
c  Correction of the chemical potential between QM and QEQ
c
      call difimu(qmprognme, geom, ssgeom, mmgeom, ssmmgeom,
     *           qmmminp, ccmeth, de, corrmu)

      write(chargfile,145) corrmu

145   format(/,"chemical potential correction(a.u.):",3x,f12.4)
c
c calculate the electronic temperature parameter T
c added by Yan Zhang, Apr 14, 2009
c
      if(temprt.lt.0.d0) then
          call fbtempcalc(geom, ssgeom, mmgeom, ssmmgeom,
     *           qmmminp, ccmeth, temprt1)
          temprt = temprt1
          qmmminp%temprt  = temprt
      endif

      write(chargfile,149) temprt

149   format("electronic temperature parameter T:",3x,f12.4)
150   format(60('-'))
c
c end  Apr 14, 2009
c
c  Calculate the initial FB group total charge
c
      write(chargfile,220)
      call gqtot(ssmmgeom%atmcharge, qmmminp, idfbg, fbqtot0)
      fbqtot1 = fbqtot0

      write(chargfile,101) fbqtot0
101   format('Group total charge: ',2x,f9.6,/)      
220   format(/,'calculate the initial FB group total charge')
c
c  Iterative calculations of the background point charge in
c  flexible-boundary and polarized groups base on the QEQRG, 
c  QEQBT, and EEM.
c
      qt(:ssnatm) = ssmmgeom%atmcharge(:ssnatm)
c
c  Debug. As weighting of added state increasing, output results.
c
      if(debug.eq.1) then
         call muvspx(geom, mmgeom, ssgeom, ssmmgeom, qmmminp,
     *                qmprognme, de, corrmu, fbqtot0)
      endif 
c
c Iteration               
c
      do while ((it.le. maxstep).and.(ifconvg.eq.0))

         write(chargfile,150)
         write(chargfile, 120) it      
         write(chargfile, 130)qmmminp%maxfbdq, qmmminp%rmsfbdq,
     *                   qmmminp%maxfbdqtot      

120   format('Flexible-boundary interative:',i4)  
130   format('Charge converged criterion:',/,'maxdq=',f9.7,4x,
     *       'rmsdq=', f9.7,4x,'Transfer charge=',f9.7)
c
c  Calculate FB group charge in the gas phase 
c
         write(chargfile,230)
         nfbga = qmmminp%fbgan(idfbg)
         call fbgacoor(geom, qmmminp, fbatm, idfbg, xg, yg, zg)
         call dvarzero(maxganum, gpot)
         write(chargfile,*)
         call polgchargcal(geom, ssgeom, mmgeom, ssmmgeom, qmmminp,
     *      ccmeth, idfbg, nfbga, fbatm, xg, yg, zg, gpot, fbqtot1, qn)
         call updatcharg(ssmmgeom%atmcharge, qmmminp, idfbg, qn)
         call gqtot(ssmmgeom%atmcharge, qmmminp, idfbg, qtot)
         write(chargfile,101)qtot

230   format(/'calculate the FB group charge without external field.')
c
c  Loop of iterative polarized charge calculations at the given x  
c  Electric potential on the atom of SS subsystem.
c
         write(chargfile,190)
         icy = 1
1000     write(chargfile,191)
         write(chargfile,189) icy
         call fbpot(qmprognme, geom, ssgeom, mmgeom, ssmmgeom,
     *           qmmminp, px, pot)

189   format('polarized cycle:',3x,i4)
190   format('Group charge SCF calculation with polarization of CPS')
191   format(20('-'))
c
c  Calculate charge of each group 
c         
         maxdq   = 0.d0
         rmsdq   = 0.d0
         do idg=1, ng
            nfbga = qmmminp%fbgan(idg)
c
c  Assign the coordinates of all atoms in a group.
c
            call fbgacoor(geom, qmmminp, fbatm, idg, xg, yg, zg)
            call gqtot(ssmmgeom%atmcharge, qmmminp, idg, qtot)
            call fbgpot(qmmminp, idg, pot, gpot)
            call polgchargcal(geom, ssgeom, mmgeom, ssmmgeom, qmmminp,
     *           ccmeth, idg, nfbga, fbatm, xg, yg, zg, gpot, qtot, qn)
            call dqcon(ssmmgeom, qmmminp, idg, qn, maxgdq, rmsgdq)
            call updatcharg(qt, qmmminp, idg, qn)
            call gqtot(qt, qmmminp, idg, qtot)
            if (idg.eq.idfbg) muss = qn(nfbga+1)
            if (idg.eq.idfbg) fbqtot = qtot
            if(maxgdq.gt.maxdq)  maxdq = maxgdq
            if(rmsgdq.gt.rmsdq)  rmsdq = rmsgdq
         enddo 
         ssmmgeom%atmcharge(:ssnatm) = qt(:ssnatm)
         write(chargfile,170)maxgdq, rmsgdq
         icy = icy+1
         if(icy.gt.maxstep) goto 9910
         if(maxdq.gt.qmmminp%maxfbdq.or.rmsdq.gt.qmmminp%rmsfbdq)
     *              goto 1000 
         write(chargfile,191)

170   format('GROUP:MAXDQ',3x,f9.6,6x,
     *       'GROUP:RMSDQ',3x,f9.6/) 
c
c Calculate next step px.
c
         call calpx(qmmminp, de, muss, corrmu, temprt, px)
c
c Calculate next step FB group total charge.
c 
         call fbgqtot(qmmminp,fbqtot0, px, fbqtot1)
         dqtot = fbqtot1 - fbqtot 
         tfq=tfq+dqtot
         write(chargfile,200)abs(dqtot),tfq

200   format('Transfer charge: ',f12.9,2x,
     *      'Total transfer charge: ',f12.9/)
c
c  Determine the convergence.
c
         call convg(qmmminp, maxdq, rmsdq, dqtot, ifconvg)

         it = it + 1
      end do

201   format("maxdq=",f9.6,3x,"rmsdq=",f9.6)
202   format("x=",f9.6,3x,"transfer dq=",f9.6)
c
c  print out if convergence
c
      if(ifconvg.eq.1) then
         qcps = (1.d0-px)*qmis + px*qmas
         write(chargfile,150)
         write(chargfile,300)
         write(chargfile,310) qmis, qmas
         write(chargfile,319)temprt
         write(chargfile,320) qcps
         write(chargfile,321) px
         write(chargfile,100)

         write(outfile,300)
         write(outfile,310) qmis, qmas
         write(outfile,319)temprt
         write(outfile,320) qcps
         write(outfile,321) px
         write(outfile,100)
       end if
300   format(/,'Flexible-boundary calculation convergence',/)
310   format('initial state charge: ',i2,2x, 
     *       'additional state charge: ',i2,2x)
319   format('Temperature:',f12.3)
320   format('final CPS charge: ',f9.6,2x)
321   format('final added state weighting px:',f12.9/)
c
c  for debug
c  Final charge calculation. 
c  Calculate FB group charge in the gas phase 
c
      if(debug.eq.1) then 
         write(chargfile,100)
         write(chargfile,100)
         write(chargfile,140) 
         write(chargfile,230)
         nfbga = qmmminp%fbgan(idfbg)
         call fbgacoor(geom, qmmminp, fbatm, idfbg, xg, yg, zg)
         call dvarzero(maxganum, gpot)
         write(chargfile,*)
         call polgchargcal(geom, ssgeom, mmgeom, ssmmgeom, qmmminp,
     *      ccmeth, idfbg, nfbga, fbatm, xg, yg, zg, gpot, fbqtot1, qn)
         call updatcharg(ssmmgeom%atmcharge, qmmminp, idfbg, qn)
         call gqtot(ssmmgeom%atmcharge, qmmminp, idfbg, qtot)
         write(chargfile,150)
c
c  Loop of iterative polarized charge calculations at the given x
c  Electric potential on the atom of SS subsystem.
c
         write(chargfile,190)
         icy = 0
2000     call fbpot(qmprognme, geom, ssgeom, mmgeom, ssmmgeom,
     *           qmmminp, px, pot)
c
c  Calculate charge of each group
c
         maxdq   = 0.d0
         rmsdq   = 0.d0
         write(chargfile,190)
         do idg=1, ng
            nfbga = qmmminp%fbgan(idg)
            call fbgacoor(geom, qmmminp, fbatm, idg, xg, yg, zg)
            call gqtot(ssmmgeom%atmcharge, qmmminp, idg, qtot)
            call fbgpot(qmmminp, idg, pot, gpot)
            call polgchargcal(geom, ssgeom, mmgeom, ssmmgeom, qmmminp,
     *           ccmeth, idg, nfbga, fbatm, xg, yg, zg, gpot, qtot, qn)
            call dqcon(ssmmgeom, qmmminp, idg, qn, maxgdq, rmsgdq)
            call updatcharg(qt, qmmminp, idg, qn)
            call gqtot(qt, qmmminp, idg, qtot)
            if (idg.eq.idfbg) muss = qn(nfbga+1)
            if (idg.eq.idfbg) fbqtot = qtot
            if(maxgdq.gt.maxdq)  maxdq = maxgdq
            if(rmsgdq.gt.rmsdq)  rmsdq = rmsgdq
         enddo
         ssmmgeom%atmcharge(:ssnatm) = qt(:ssnatm)
         write(chargfile,170)icy+1,maxgdq, rmsgdq
         write(chargfile,150)
         icy = icy + 1
         if(icy.gt.maxstep) goto 9920
         if(maxdq.gt.qmmminp%maxfbdq.or.rmsdq.gt.qmmminp%rmsfbdq)
     *              goto 2000
c
c
         qcps = (1.d0-px)*qmis + px*qmas 
         if(ifconvg.eq.1) then
            write(chargfile,150)
            write(chargfile,210)temprt, qmas, px, qcps
            write(chargfile,100)
         else
            write(chargfile,150)
            write(chargfile,270)temprt
         end if

         stop
      endif

140   format('Final charge calculation.'/)

210   format('Temperature: ',f12.2,2x,'W(C ',i2,'):',1x, 
     *       f12.6,2x,'q(PS): ',f12.6)
270   format('Temperature: ',f12.2,2x,'Not convergence')

      return
9000  write(outfile,*)'Error: Felxible-boundary treatment do not',
     *      ' support the mechanical embedding.'
      stop 'fbbpc 1'
9910  write(outfile,*)'Error: Polarized calculation not convergence'
      stop 'fbbpc 2'
9920  write(outfile,*)'Error: Polarized calculation not convergence'
      stop 'fbbpc 3'

      end      
c
c
c*****************************************************************************
c  CHKINP
c*****************************************************************************
c
      subroutine chkinp(qmmminp)
c
c     Subroutine chkinp
c     
c     Created:  Aug 28, 2007 by Yan Zhang
c
c     Purpose:  Check the input variables for the flexible-boundary calculation. 
c                              
c 
c     Called by: fbbpc 
c     Calls:	  
c
      use files
      use caltyp                                                        
      use input
      implicit none
c
c  i/o variables
c
      type (qmmminf)  :: qmmminp      
c
c  local variables
c
      integer :: i, j, k, ii, jj, kk, id
      integer :: nfbg, ifbp, nfbga, idfbga
c
c
      nfbg = qmmminp%fbgnum
c
c  Check the group number for the non-polarized flexible-boundary treatment.
c
      if((ifbp.eq.0).and.(nfbg.gt.1)) goto 1000
c
c  Check atom ID in the groups.
c
      do i=1,	nfbg
         nfbga = qmmminp%fbgan(i)
         do j=1, nfbga
            idfbga = qmmminp%fbgaid(i,j)
c
c  Flexible boundary groups.
c
            k=0
            do ii=1,nfbg
               kk = qmmminp%fbgan(ii)
               do jj=1, kk
                  id = qmmminp%fbgaid(ii,jj)
                  if(idfbga. eq. id) k=k+1
               end do
            end do
            if(k.ne.1) goto 2000
c
c  QM
c
            do ii=1, qmmminp%qmnatm
               jj = qmmminp%qmatm(ii)
               if(idfbga.eq.jj) goto 3000
            end do
c
c  M1
c
            do ii=1, qmmminp%mm1stlay%natm1stlay
               jj = qmmminp%mm1stlay%atm1stlay(ii)
               if(idfbga.eq.jj) goto 4000
            end do
c
c  MM
c
            k=0
            do ii=1, qmmminp%mmnatm
               jj = qmmminp%mmatm(ii)
               if(idfbga.eq.jj) k=k+1
            end do
            if(k.ne.1) goto 5000

         end do
      end do

      return
1000  write(outfile,*)'Error: One group in the non-polarized',
     *                 ' flexbile-boundary treatment.'
      stop 'chkinp 1'

2000	write(outfile,*)'Error: Reduplicate atom in the groups for the',
     *                 ' flexible-boundary treatment.'
      stop 'chkinp 2'

3000  write(outfile,*)'Error: unresonable atom in the groups for the',
     *                 ' flexible-boundary treatment.'
      stop 'chkinp 3'

4000  write(outfile,*)'Error: unresonable atom in the groups for the',
     *                 ' flexible-boundary treatment.'
      stop 'chkinp 4'

5000	write(outfile,*)'Error: Atom in the groups for the',
     *                 ' flexible-boundary treatment.'
      stop 'chkinp 5'
      end
c
c
c******************************************************************************
c  FBPOT
c******************************************************************************
c
      subroutine fbpot(qmprognme, geom, ssgeom, mmgeom, ssmmgeom,
     *           qmmminp, px, pot)
c
c     Subroutine fbpot
c
c     Created:  Aug. 24, 2007 by Yan Zhang
c     Revised:  
c                              
C               
c     Purpose: This subroutine calculates the electric poteintial of CPS on the 
c              SS subsystem at the given added state weighting px. The electric 
c              poteintial is the mixture of reduced and oxidized states.
c              pot = (1-px)*pot(reduced) + px*pot(oxidized).
c
c     Called by: fbbpc, muvspx 
c     Calls: progcpe  
c  
      use caltyp
      use input  
      use files  
      use energy 
      implicit none
c
c i/o variables
c
      character*10       qmprognme
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, ssmmgeom
      type (qmmminf)  :: qmmminp
      double precision   px, pot(maxatm)
c
c local variables
c      
      integer      i, id, ierr
      integer      qmnatm, capnatm, cssnatm, lsnatm
      integer      ssnatm, mmnatm
      integer      qmr, qmo
      double precision   energx, energxa, kb, ev
      double precision   pot1(maxatm), pot2(maxatm)

      integer debug
c
      debug = 0
c
c Constant. kb in eV/K.
c
      kb=8.617343D-5
      ev=27.21066
c
c
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      ssnatm   = ssmmgeom%natoms
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + mmnatm
      qmr      = qmmminp%qmcharge
      qmo      = qmmminp%asqmcharge
c
c QM calculation of the initial state on the capped primary system. 
c
      if(debug.eq.1) then
          if(qmr.lt.qmo)  write(chargfile, 100) 
          if(qmr.gt.qmo)  write(chargfile, 300)
      endif
      call progcpe(qmprognme, qmmminp%qmmeth, qmmminp%qmbas, 
     *            qmr, qmmminp%qmmult,
     *            qmmminp%qmoptl, geom, mmgeom,
     *            ssgeom, ssmmgeom,qmmminp, pot1,energx)
c
c QM calculation of the oxidized state on the capped primary system. 
c      
      if(debug.eq.1) then
          if(qmr.lt.qmo)  write(chargfile, 300)    
          if(qmr.gt.qmo)  write(chargfile, 100)
      endif

      call progcpe(qmprognme, qmmminp%qmmeth, qmmminp%qmbas, 
     *            qmo, qmmminp%asqmmult,
     *            qmmminp%qmoptl, geom, mmgeom,
     *            ssgeom, ssmmgeom,qmmminp, pot2,energxa)
c
c  Electric potential
c
      if(debug.eq.1) write(chargfile,700)
      do i = cssnatm+1,lsnatm
         id = ssmmgeom%atmid(i)           
         pot(id) = pot1(id)*(1-px) + px*pot2(id)
         if(debug.eq.1)
     *        write(chargfile,600)ssgeom%atm(i),id,qmr,pot1(id), 
     *                          qmo, pot2(id), pot(id)
      end do

c
100   format(/'QM calculation of the reduced state',
     *  ' on the capped primary system with background charges:')
300   format(/'QM calculation of the oxidized state',
     *  ' on the capped primary system with background charges:')                  
500   format('Electric potential on SS atoms produced by CPS:')
600   format('Atom: ',a2,1x,'ID:',1x, i5,1x,
     *       'C',i2,' state: ',f9.6,1x,'C',i2,' state: ',f9.6,
     *       1x,'AEP: ',f9.6)
700   format(/'Electric potential(a.u.)',
     *       ' on SS atoms produced by CPS')         

      return
      end

c
c******************************************************************************
c  FBGACOOR
c******************************************************************************
c
      subroutine fbgacoor(geom, qmmminp, fbatm, fbgid, x, y, z)
c
c     Subroutine fbgacoor
c
c     Created:  Aug 27, 2007 by Yan Zhang
c     Revised:  
c     Purpose:  To assign appropriate geometries to the group atoms 
c               in SS atoms. Output atomic coordinates is in atomic unit. 
c
c
c     Called by: fbbpc, muvspx  
c     Calls:
c
      use input
      use atminfo
      use files
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom
      type (qmmminf)  :: qmmminp

      double precision :: x(maxatm), y(maxatm), z(maxatm)
      character*2 fbatm(maxatm)
c
c local variables
c      
      integer  i, k, fbgid, fbgan
      double precision   angtoau

      integer  debug

      debug = 0
c     
c  unit conversion from bohr to angstrom.
c
      angtoau = 0.529177249d0
c
c  prepare parameters.
c  
      fbgan   = qmmminp%fbgan(fbgid)
c
c  Assign the flexbile-boundary SS atom coordinates. 
c
      if(debug.eq.1) write(chargfile,*)
      do i=1,fbgan
         k = qmmminp%fbgaid(fbgid,i)
         fbatm(i)= geom%atm(k)
         x(i) = geom%angcoord(3*k-2)
         y(i) = geom%angcoord(3*k-1)
         z(i) = geom%angcoord(3*k)
         if(debug.eq.1) 
     *   write(chargfile,100)fbatm(i),k,x(i),y(i),z(i)
      enddo    
c
c  unit converse
c
      do i=1,fbgan
        x(i)=x(i)/angtoau
        y(i)=y(i)/angtoau
        z(i)=z(i)/angtoau
      end do

100   format('Atom: ',a2,3x,'ID: ',i4,3x,'XYZ(ang.): ',
     *      3(f12.6,2x))   
c
      return     
      end
c
c******************************************************************************
c  GQTOT
c******************************************************************************
c
      subroutine gqtot(qt, qmmminp, idg, qtot)
c
c     Subroutine gqtot
c
c     Created:  Aug 27, 2007 by Yan Zhang
c     Revised:  
c     Purpose:  Calculate the total charge of group ID idg 
c
c     Called by: fbbpc, muvspx  
c     Calls:
c
      use input
      use atminfo
      use files
      implicit none
c
c i/o variables
c
      type (qmmminf)  :: qmmminp

      integer  idg
      double precision :: qt(maxatm), qtot
c
c local variables
c      
      integer  i, j, k, kk, fbgan
      integer  capnatm, qmnatm, mmnatm, cssnatm
      double precision  charg 

      integer debug

      debug = 0
c
c  prepare parameters.
c  
      fbgan   = qmmminp%fbgan(idg)
      capnatm = qmmminp%capatm%capnatm
      qmnatm  = qmmminp%qmnatm
      mmnatm  = qmmminp%mmnatm
      cssnatm = capnatm + qmnatm 
c
c  Calculate the total charges in the group idg. 
c
      qtot=0.d0
      do i=1, mmnatm
           k     = qmmminp%mmatm(i)
           charg = qt(i+cssnatm)

           do j=1,fbgan
             kk = qmmminp%fbgaid(idg,j)
             if(k.eq.kk) qtot = qtot + charg 
           enddo
      enddo    
      if(debug.eq.1) write(chargfile,100) qtot
100   format(/'Group total charge: ',2x,f9.6,/)
      return     
      end
c
c
c*****************************************************************************
c  FPCHARG
c*****************************************************************************
c
      subroutine fpcharg(ssmmgeom, qmmminp, fbgid, nfpc, idfpc)
c
c     Subroutine fpcharg
c     
c     Created:  Aug 27, 2007 by Yan Zhang
c
c     Purpose:  Assign the fixed background partial point charge.
c               
c 
c     Called by: polgchargcal 
c     Calls:	  
c
      use input
      use files
      use caltyp                                                        
      use input
      implicit none
c
c i/o variables
c
      type (mmvalen)  :: ssmmgeom
      type (qmmminf)  :: qmmminp
      
      integer  fbgid
c
c local variables
c      
      integer  ngatm, nfpc
      integer  ind, mmnatm, ssnatm, lsnatm, cssnatm
      integer  qmnatm, capnatm
      integer  idfpc(1)
      integer  i, j, k, kk
c
c
      capnatm  = qmmminp%capatm%capnatm
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      cssnatm  = capnatm + qmnatm 
      ssnatm   = ssmmgeom%natoms
      lsnatm   = cssnatm + mmnatm
c
c  Assign the fixed SS atom. 
c  MM atoms. 
c
      nfpc=0
      do i=1,mmnatm
         k=qmmminp%mmatm(i)

         ind=0
         ngatm = qmmminp%fbgan(fbgid)
         do j=1,ngatm
            kk = qmmminp%fbgaid(fbgid,j)
            if(k.eq.kk) ind=1
         end do

         if(ind.eq.0) then
            nfpc = nfpc + 1
            idfpc(nfpc) = k
         end if
      end do
c
c  Redistributed charges.
c
      do i=lsnatm+1,ssnatm
         k    = ssmmgeom%atmid(i)
         nfpc = nfpc + 1
         idfpc(nfpc) = k
      end do

      return
      end

c
c******************************************************************************
c  FBGPOT
c******************************************************************************
c
      subroutine fbgpot(qmmminp, fbgid, pot, gpot)
c
c     Subroutine fbgpot
c
c     Created:  Aug 29, 2007 by Yan Zhang
c     Revised:  
c     Purpose:  To assign appropriate electric potential to the group atoms 
c               in SS subsystem.
c
c
c     Called by: fbbpc, muvspx  
c     Calls:
c
      use input
      use atminfo
      use files
      implicit none
c
c i/o variables
c
      type (qmmminf)  :: qmmminp

      integer :: fbgid
      double precision :: gpot(maxganum), pot(maxatm)
c
c local variables
c      
      integer  i, j, k, kk, ifnd
      integer  capnatm, qmnatm, mmnatm,  cssnatm
      integer  nfbga

      integer  debug

      debug = 0
c
c  Prepare the parameters.
c
      capnatm = qmmminp%capatm%capnatm
      qmnatm  = qmmminp%qmnatm
      mmnatm  = qmmminp%mmnatm
      cssnatm = capnatm + qmnatm 
c
c  prepare parameters.
c  
      nfbga   = qmmminp%fbgan(fbgid)
c
c  Assign the appropriate electric potential to the group. 
c
      do i=1,nfbga
         k = qmmminp%fbgaid(fbgid,i)
         gpot(i) = pot(k)
         if(debug.eq.1)write(chargfile,100)k,pot(k)
      enddo    
      if(debug.eq.1)write(chargfile,*)
100   format('Atom ID: ',i4,2x,'Electronic potential by CPS: ',
     *       2x,f12.9)    
c
      return     
1000  write(outfile,*) 'Error: Can not find ecectric potetial on atom'
      stop 'fbgpot 1'
      end
c
c
c*****************************************************************************
c  POLGCHARGCAL
c*****************************************************************************
c
      subroutine polgchargcal(geom, ssgeom, mmgeom, ssmmgeom, qmmminp, 
     *           ccmeth, idg, nfbga, fbatm, x, y, z, 
     *                       gpot, qtot, qn)
c
c     Subroutine polgchargcal
c     
c     Created:  Aug 29, 2007 by Yan Zhang
c
c     Purpose:  Calculate the group charges with the CPS polarization by the QEQRG,
c               QEQBT and EEM method                
c               
c 
c     Called by: fbbpc, muvspx  
c     Calls: fpcharg, atmparam, fbccp, maxtrixj12, slinequ  
c
      use input
      use files
      use caltyp                                                        
      use input
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, ssmmgeom
      type (qmmminf)  :: qmmminp
      
      integer           idg, nfbga    
      character*2       fbatm(maxganum)
      character*9       ccmeth
      double precision  qtot
      double precision  x(maxganum), y(maxganum), z(maxganum)
      double precision  gpot(maxganum), qn(maxganum)
c
c local variables
c      
      integer  i, j, nfpc 
      integer  idfpc(maxatm)
      double precision  fpcpot, x0, jaa0
      double precision  xn0(maxganum), j12(nfbga+1,nfbga+1)
      double precision  jx(nfbga,nfbga)
c
c  Assign the fixed background point charges.
c
      call fpcharg(ssmmgeom, qmmminp, idg, nfpc, idfpc)
c
c  Calculate the maxtrix of right side for the linear equtaion of 
c  the charge calculation.
c
      do i=1,nfbga
         call atmparam(qmmminp, ccmeth, fbatm(i), x0, jaa0)
         xn0(i) = -x0
c
c  Fixed charg chemical potential
c
         call fpccp(geom, mmgeom, ssgeom, ssmmgeom, qmmminp, ccmeth,
     $          fbatm(i), x(i), y(i), z(i), nfpc, idfpc, fpcpot)
         xn0(i) = xn0(i) - fpcpot
c
c  Electric potential by QM subsystem.
c
         xn0(i) = xn0(i) - gpot(i)
      end do

      xn0(nfbga+1) = qtot
c
c  Determine the J matrix.
c
      call matrixj12(qmmminp, nfbga, fbatm, x, y, z, ccmeth, jx)

      do i=1,nfbga
         do j=1,nfbga 
           j12(i,j)=jx(i,j)
         end do
      end do
 
      do i=1,nfbga
         j12(i,nfbga+1) = -1.d0
         j12(nfbga+1,i) =  1.d0
      end do
      j12(nfbga+1,nfbga+1) = 0.d0
      
c
c  Sovle the linear equation for the charge calculation.
c
      call slinequ(nfbga+1, j12, xn0, qn)


      write(chargfile,100)
      do i=1, nfbga
         write(chargfile,200)fbatm(i),qmmminp%fbgaid(idg, i), qn(i)
      end do

100   format(/'QEQ or EEM charge: ')
200   format('Atom: ',a2,2x,'ID:',2x,i5,2x,'Charge: ',f12.6)
      return
      end

c
c***********************************************************************
c     ATMPARAM
c***********************************************************************
C
      subroutine atmparam(qmmminp, ccmeth, atmnam, x0, jaa0)

c     subroutine atmparam
c
c     Created:  Aug.  27, 2007 by Yan Zhang
c          
c     Revised:
c
c     Purpose: Determine the atomic parameters x0 and JAA0 in the charge calculation
c              by the QEQRG, QEQBT and EEM method. The parameters read from input would
c              replace the defaut parameter.
C      
c
c     Called by: polchargcal, cpsmu, cpsmu1 
c     Calls:	scase, dgesv
c
      use input
      use files
      use ehard
      use atminfo
      implicit none
c
c  i/o variable
c
      type (qmmminf)  :: qmmminp
      
      character*9 ccmeth
      character*2 atmnam
      double precision :: x0, jaa0
c
c  local variable
c
      integer  i, j, ifind1, ifind2, nparam

      integer        ierr
      integer 	   nparalw, ifindatm
      integer        lparalw(40)
c
      character*80   string, paralw(40)    

      character*2 lcan, asymp(103)
      double precision :: px0(103), pjaa0(103), cfloat
      double precision :: ev
c
c
      ev=27.21066
c
c  Read the x0 and JAA0 from the defaut values   
c
      x0     = 0.d0
      jaa0   = 0.d0
      nparam = 0
      ifind1 = 0
      ifind2 = 0  

      if(ccmeth.eq.'qeqrg') then
c
c  Parameters in J.Phys.Chem.1991,95,3358-3363
c
        call scase(2,atmnam,lcan)
        ifind1=0
        do j=1,16
          if(lcan.eq.qeqag(j)) ifind1=j
        end do
        x0   = qeqeg(ifind1)/ev
        jaa0 = qeqhg(ifind1)/ev
c
c  Parameters in J. comput. chem. 1995 17 87-108 
c
      else if(ccmeth.eq.'qeqbt') then
        call scase(2,atmnam,lcan)
        ifind1=0
        do j=1,4
          if(lcan.eq.qeqat(j)) ifind1=j
        end do
        x0   = qeqet(ifind1)/ev
        jaa0 = qeqht(ifind1)/ev
c
c  Parameters in J. Phys. Chem. A 108 2004 10359-10366
c
      else if(ccmeth.eq.'eem') then
        call scase(2,atmnam,lcan)
        ifind1=0
        do j=1,4
          if(lcan.eq.eemam(j)) ifind1=j
        end do
        x0   = eemem(ifind1)/ev
        jaa0 = 2.d0*eemhm(ifind1)/ev
      end if
c
c  Parameters read from input file 
c
      i=1
      string = qmmminp%sscp(i)
      do while (string.ne.'   ' .and. i.le.103)

c
        call wordstr(string,80,paralw,nparalw,lparalw,ierr)
        if(nparalw.ne.3)  goto 9000

        asymp(i)=paralw(1)(1:2)          
        call scase(2,asymp(i),asymp(i))
        ifindatm=0
        do j=1,103
           if(asymp(i).eq.asymb(j)) ifindatm=1
        end do
        if(ifindatm.eq.0) goto 9000

        px0(i)   = cfloat(paralw(2))
        pjaa0(i) = cfloat(paralw(3))

        i = i + 1
        string = qmmminp%sscp(i)
      enddo

      if(i.gt.1) nparam=i-1
c
c  Determine x0 and JAA0 from input parameter.
c
      if(nparam.gt.0) then
        call scase(2,atmnam,lcan)
        ifind2=0
        do j=1,nparam
          if(lcan.eq.asymp(j)) ifind2=j
        end do
        x0   = px0(ifind2)/ev
        jaa0 = pjaa0(ifind2)/ev
      endif

      if((ifind1.eq.0).and.(ifind2.eq.0)) goto 9100

c      write(chargfile,*)'nparam:',nparam
c      write(chargfile,*)'param:',qeqeg(ifind1),qeqhg(ifind1)       
c      write(chargfile,*)'param:',x0,jaa0,ifind1,ifind2

      return 

9000  write(outfile,*)'Error reading the atomic parameters for',
     $                ' the charge calculation.'
      write(outfile,*)string
      stop

9100  write(outfile,*)'Error: can not find the atomic parameters',
     $                ' for charge calculation.', atmnam
      stop 
      end
c
c*****************************************************************************
c  FPCCP
c*****************************************************************************
c
      subroutine fpccp(geom, mmgeom, ssgeom, ssmmgeom, qmmminp, ccmeth,
     $                 atmnam, x, y, z, nfpc, idfpc, fpcpot)
c
c     Subroutine fpccp
c     
c     Created: Aug. 27, 2007 by Yan Zhang
c
c     Purpose: Calculate the chemical potential produced by the fixed 
c              background charges on a atom.
c        
c 
c     Called by: polgchargcal 
c     Calls:	  
c
      use input
      use files
      use caltyp                                                        
      use atminfo,  only : asymb
      use input
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, ssmmgeom
      type (qmmminf)  :: qmmminp

      integer :: nfpc, idfpc(1)
      character*2  atmnam
      character*9  ccmeth
      double precision   :: x, y, z, fpcpot
c
c local variables
c      
      integer  i, k
      integer  capnatm, qmnatm, mmnatm
      integer  ssnatm, cssnatm, lsnatm      
      character*2 fatm
      double precision  xf, yf, zf, chargf
      double precision  r12, j12, x10, x20, jaa10, jaa20
      double precision   angtoau, eV

      integer debug

      debug = 0
c
c  Unit conversion.
c
      angtoau = 0.529177249d0
      ev=27.21066d0
c
c
      capnatm  = qmmminp%capatm%capnatm
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      cssnatm  = capnatm + qmnatm 
      ssnatm   = ssmmgeom%natoms
      lsnatm   = cssnatm + mmnatm
c
c  Determine the chemical potential by the fixed background point charges.
c
      fpcpot=0.d0
      do i=1,nfpc
         k  = idfpc(i)
c
c  MM fixed charges.
c
         if(k.le.lsnatm) then
            fatm   = geom%atm(k)
            xf     = geom%angcoord(3*k-2)/angtoau
            yf     = geom%angcoord(3*k-1)/angtoau
            zf     = geom%angcoord(3*k)/angtoau
            chargf = mmgeom%atmcharge(k)
c           if(debug.eq.1) write(chargfile,100)k, chargf                
c
c  Redistributed charges.
c
         else
            fatm   = 'H'
            xf     = ssgeom%angcoord(3*k-2)/angtoau
            yf     = ssgeom%angcoord(3*k-1)/angtoau
            zf     = ssgeom%angcoord(3*k)/angtoau
            chargf = ssmmgeom%atmcharge(k)
c           if(debug.eq.1) write(chargfile,300) k, chargf      
         end if
c
c  Calculate the chemical potential.
c  QEQRG
c    
         if(ccmeth.eq.'qeqrg') then
            r12    = (x-xf)**2 + (y-yf)**2 + (z-zf)**2
            j12    = 14.4d0/(2.d0*dsqrt(r12)*angtoau*ev)
            fpcpot = fpcpot + chargf*j12
c
c  QEQBT
c
         else if(ccmeth.eq.'qeqbt') then
            call atmparam(qmmminp, ccmeth, atmnam, x10, jaa10)
            call atmparam(qmmminp, ccmeth, fatm, x20, jaa20)
            r12    = (x-xf)**2 + (y-yf)**2 + (z-zf)**2
            j12    = (1.d0/(2*jaa10) + 1.d0/(2*jaa20))**2
            fpcpot = fpcpot + chargf/dsqrt(r12 + j12) 
c
c  EEM
c
         else if(ccmeth.eq.'eem') then
            r12    = (x-xf)**2 + (y-yf)**2 + (z-zf)**2
            fpcpot = fpcpot + chargf/dsqrt(r12)
         end if

      end do

c      write(outfile,200) atmnam, x,y,z,fpcpot
      if(debug.eq.1)write(chargfile,200)atmnam,fpcpot

100   format('MM atom of fixed charge:',i4,2x,'Charge(a.u.):',
     *         f16.10)
200   format('Atom:',1x,a2,2x,
     $       'Chemical potential (a.u.) due to fixed charges:',
     $       2x,f16.10)
300   format('Fixed redistributed charge:',i4,2x,'Charge(a.u.):',
     *         f16.10)

      return
      end
c
c
c*****************************************************************************
c  MATRIXJ12
c*****************************************************************************
c
      subroutine matrixj12(qmmminp, fbgan, fbatm, x, y, z, ccmeth, j12)
c
c     Subroutine matrixj12
c     
c     Created:  Aug 28, 2007 by Yan Zhang
c
c     Purpose:  Determine the matrix J for the charge calculations by the 
c               QEQRG, QEQBT, and EEM method.
c               
c 
c     Called by: polgchargcal, cpsmu, cpsmu1 
c     Calls:	  
c
      use input
      use files
      use caltyp                                                        
      use input
      implicit none
c
c i/o variables
c
      type (qmmminf)  :: qmmminp
      
      integer  fbgan
      character*2 fbatm(fbgan)
      character*9 ccmeth
      double precision  x(maxganum), y(maxganum), z(maxganum)
      double precision  j12(fbgan,fbgan)
c
c local variables
c      
      integer  i, j
      double precision  r12, x10, jaa10, x20, jaa20

      double precision  angtoau, ev
c
c
      angtoau = 0.529177249d0
      ev=27.21066d0
c
c  Matrix J
c
      do i=1,fbgan
c
c  QEQRG
c    
         if(ccmeth.eq.'qeqrg') then
            call atmparam(qmmminp, ccmeth, fbatm(i), x10, jaa10)
            do j=i+1, fbgan
               r12      = (x(i)-x(j))**2 + (y(i)-y(j))**2 + 
     $                    (z(i)-z(j))**2
               j12(i,j) = 14.4d0/(2.d0*dsqrt(r12)*angtoau*ev)
               j12(j,i) = j12(i,j)
c               write(chargfile,*)i,j,x10, jaa10, j12(i,j)
            end do
            j12(i,i) = jaa10
c		  
c  QEQBT
c
         else if(ccmeth.eq.'qeqbt') then
            call atmparam(qmmminp, ccmeth, fbatm(i), x10, jaa10)
            do j= i+1, fbgan
               call atmparam(qmmminp, ccmeth, fbatm(j), x20, jaa20)
               r12      = (x(i)-x(j))**2 + (y(i)-y(j))**2 + 
     $                    (z(i)-z(j))**2
               j12(i,j) = (1.d0/(2*jaa10) + 1.d0/(2*jaa20))**2
               j12(i,j) = 1.d0/dsqrt(r12 + j12(i,j))
               j12(j,i) = j12(i,j)
            end do
            j12(i,i) = jaa10 
c
c  EEM
c
         else if(ccmeth.eq.'eem') then
            call atmparam(qmmminp, ccmeth, fbatm(i), x10, jaa10)
            do j=i+1,fbgan
               r12 = (x(i)-x(j))**2 + (y(i)-y(j))**2 + (z(i)-z(j))**2
               j12(i,j) = 1.d0/dsqrt(r12)
               j12(j,i) = j12(i,j)
            end do
            j12(i,i) = 2.d0*jaa10
         end if

      end do

      return
      end
c
c*****************************************************************************
c  PROGCPE
c*****************************************************************************
c
      subroutine progcpe(prognme,  qmmeth, qmbas, qmcharge, qmmult,
     $            qmoptl, geom, mmgeom,
     *            ssgeom, ssmmgeom,qmmminp, pot,ener)
c
c     Subroutine progcp
c
c     Created:  Aug. 28, 2007 by Yan Zhang
c     Revised:  
c
c     Purpose:  Calculate the CPS energy, ESP charges and electric potential 
c               of the background point charge in the presence of background charges.
c
c
c     Called by: fbpot 
c     Calls:  g03bgpot, g03oute, g03outep, g03outesp, orcastpotinp,
c             orcaoute, orcaoutpot
c
c     Variables defined here:
c
      use caltyp
      use files
      use input
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, ssmmgeom   
      type (qmmminf)  :: qmmminp

      character*10       prognme
      character*20       qmmeth                      
      character*20       qmbas                                                   
c
      character*80       qmoptl(maxoptl)
      integer            qmcharge, qmmult
      double precision   pot(maxatm), ener
c
c local variables
c
      
      integer qmnatm, capnatm, cssnatm
      integer mmnatm, i, j, lsnatm
      character*20 ver_lab, esmeth                                                 
     
      integer debug

      debug = 0
c
c
c
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      cssnatm  = qmnatm + capnatm
      lsnatm   = qmnatm + capnatm + mmnatm  
c
c  GAUSSIAN03
c
      if (prognme(:3).eq.'g03') then
         open (unit=g03inf, status='unknown', file=g03in, err=9993)
         call g03bgpotinp(g03inf,qmmeth,qmbas,qmcharge, qmmult,
     *         ssgeom,qmoptl,1,ver_lab, qmmminp, ssmmgeom)
         close (g03inf, err=9993)
         call system('g03shuttle g03.inp g03.out')
         call g03oute(1,esmeth, ener)                                      
         call g03outep(geom, mmgeom, ssgeom,ssmmgeom, qmmminp, pot)
         write(chargfile,100)ener
         call g03outesp(cssnatm)
c
c  ORCA
c
      else if(prognme(:4).eq.'orca') then
         open (unit=orcainf, status='unknown', file=orcain, err=9994)
         call orcastpotinp(orcainf, qmmeth, qmbas, 
     *      qmcharge, qmmult, ssgeom, 
     *      qmoptl, 1, ver_lab, qmmminp, ssmmgeom)
         close (orcainf, err=9994)
         call system('orcapotshuttle orca.inp orca.out')
         call orcaoute(1,esmeth, ener)                                      
         call orcaoutpot(ssmmgeom, qmmminp, pot)	 
         write(chargfile,100)ener
c
c
      else
         write(outfile,*)' Only g03 and ORCA are available in progespp',
     $                    ' for the polarized calculation'
         stop 'in progespp'
      endif

100   format(/,'CPS QM Energy(a.u.) with background charge: ',f16.10,/)
      return
9993  write (outfile,*) 'Error creating Gaussian03 SPE',
     *                 ' input file of the polarized calculation'
      stop 'progcpe 1'

9994  write (outfile,*) 'Error creating Orca SPE', 
     *                  ' input file of the polarized calculation'
      stop 'progcpe 2'

      end
c
c
c*****************************************************************************
c  SLINEQU
c*****************************************************************************
c
      subroutine slinequ(n, c, d, q)
c
c     Subroutine slinequ
c     
c     Created:  Aug 29, 2007 by Yan Zhang
c
c     Purpose:  Solve the equation CQ=D to calculate the charges.               
c               
c 
c     Called by: polgchargcal, cpsmu, cpsmu1 
c     Calls: dgesv  
c
      implicit none
c
c i/o variables
c
      integer :: n
      double precision :: c(n,n), d(n), q(n)
c
c local variables
c      
      integer :: i, info, ipiv(n)
c
c  Solve equation.
c
      call dgesv(n, 1, c, n, ipiv, d, n, info )
      
      do i=1,n
         q(i)=d(i)
      end do

      return
      end
c
c******************************************************************************
c  CONVERG
c******************************************************************************
c
      subroutine dqcon(ssmmgeom, qmmminp, idg, qn, maxdq, rmsdq)

c
c     Subroutine converg
c
c     Created:  Aug. 30, 2007 by Yan Zhang
c     Revised:  
c
c
c     Purpose:  This subroutine calculates the max dq on an atom and RMS dq
c               in a group.
c              
c
c     Called by: fbbpc  
c     Calls:
c
      use input
      use atminfo
      use files, only : outfile, chargfile
      implicit none
c
c i/o variables
c
      type (mmvalen)  :: ssmmgeom
      type (qmmminf)  :: qmmminp

      integer idg
      double precision    maxdq,rmsdq
      double precision    qn(maxganum)
c
c local variables
c      
      integer  capnatm, qmnatm, mmnatm
      integer  ssnatm, cssnatm, lsnatm      
      integer  i, k, ii, jj, kk
      integer  ngan
      
      double precision  dq 
c
c  prepare atom numbers
c  
      ngan     = qmmminp%fbgan(idg)
      capnatm  = qmmminp%capatm%capnatm
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      cssnatm  = capnatm + qmnatm 
      ssnatm   = ssmmgeom%natoms
      lsnatm   = cssnatm + mmnatm
c
c  maxdq and rmsdq
c
      maxdq=0.d0
      rmsdq=0.d0
      
      do i=1, ngan
         k = qmmminp%fbgaid(idg,i)
         do ii=1,mmnatm
            jj = ii + cssnatm
            kk = qmmminp%mmatm(ii)
            if(k.eq.kk) then
               dq=dabs(ssmmgeom%atmcharge(jj)-qn(i))
               if(dq.gt.maxdq) maxdq=dq
               rmsdq=dq*dq+rmsdq
            end if
         end do
      end do

      rmsdq=dsqrt(rmsdq/ngan)

      return     
      end
c
c******************************************************************************
c  UPDATCHARG
c******************************************************************************
c
      subroutine updatcharg(qt, qmmminp, idg, qn)
c
c     Subroutine updatcharg
c
c     Created:  Aug. 30, 2007 by Yan Zhang
c     Revised:  
c
c     Purpose:  This subroutine updates the charges of variable qt.
c               Only update one group charges.
c              
c
c     Called by: fbbpc  
c     Calls:
c
      use input
      use atminfo
      use files
      implicit none
c
c i/o variables
c
      type (qmmminf)  :: qmmminp

      integer idg
      double precision    qt(maxatm), qn(maxganum)
c
c local variables
c      
      integer  capnatm, qmnatm, mmnatm
      integer  cssnatm, lsnatm      
      integer  i, k, ii, jj, kk, ngan
c
c  prepare atom numbers
c  
      ngan     = qmmminp%fbgan(idg)
      capnatm  = qmmminp%capatm%capnatm
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      cssnatm  = capnatm + qmnatm 
      lsnatm   = cssnatm + mmnatm
c
c  maxdq and rmsdq
c
      
      do i=1, ngan
           k = qmmminp%fbgaid(idg,i)
		  	       	       
           do ii=1,mmnatm
             jj = ii + cssnatm
             kk = qmmminp%mmatm(ii)
             if(k.eq.kk) qt(jj) = qn(i)
           end do
      end do
      return     
      end
c
c******************************************************************************
c  CONVG
c******************************************************************************
c
      subroutine convg(qmmminp, maxdq, rmsdq, dqtot, ifconvg)
c
c     Subroutine convg 
c
c     Created:  Aug. 30, 2007 by Yan Zhang
c     Revised:  
c
c     Purpose:  Determine the convergence of flexible-boundary calculation.
c
c     Called by: fbbpc  
c     Calls:
c
      use input
      use atminfo
      use files
      implicit none
c
c i/o variables
c
      type (qmmminf)  :: qmmminp
     
      integer  ifconvg
      double precision :: maxdq, rmsdq, dqtot
c
c local variables
c      
      double precision :: fbmaxdq, fbrmsdq, fbmaxdqtot
      character*3 mdq, rdq, mdqt
c
c  prepare threshold parameters for the convergence.
c  
      fbmaxdq    = qmmminp%maxfbdq
      fbrmsdq    = qmmminp%rmsfbdq
      fbmaxdqtot = qmmminp%maxfbdqtot
c
c
      ifconvg = 0
      mdq  =  'NO'
      rdq  =  'NO'
      mdqt =  'NO'
      if(maxdq.lt.fbmaxdq) mdq = 'YES'
      if(rmsdq.lt.fbrmsdq) rdq = 'YES'
      if(abs(dqtot).lt.fbmaxdqtot) mdqt = 'YES'
c
c  Print out in ml.charg file.         
c
      write(chargfile,80)
      write(chargfile,90)
      write(chargfile,100)
      write(chargfile,200)maxdq, fbmaxdq, mdq
      write(chargfile,300)rmsdq, fbrmsdq, rdq
      write(chargfile,400)abs(dqtot), fbmaxdqtot, mdqt
      write(chargfile,80)

80    format(60('-'))
90    format(22x,'Charge convergence')
100   format('Item',20x,'Value',10x,'Threshold',6x,'Converged')
200   format('Max dq',14x,f10.7,8x,f10.7,8x,a3)
300   format('RMS dq',14x,f10.7,8x,f10.7,8x,a3)
400   format('Transfer charge',5x,f10.7,8x,f10.7,8x,a3)
c
c  Determine the convergence.
c
      if(mdq.eq.'YES'.and.(rdq.eq.'YES').and.(mdqt.eq.'YES')) 
     *   ifconvg = 1

      return     
      end
c
c******************************************************************************
c  FBGQTOT
c******************************************************************************
c
      subroutine fbgqtot(qmmminp,qtot0, px, qtot1)
c
c     Subroutine fbgqtot
c
c     Created:  Aug 27, 2007 by Yan Zhang
c     Revised:  
c     Purpose: To calculate the next step FB group total charge.   
c
c     Called by: fbbpc  
c     Calls:
c
      use input
      use atminfo
      use files
      implicit none
c
c i/o variables
c
      type (qmmminf)  :: qmmminp

      double precision :: qtot0, qtot1, px
c
c local variables
c      
      double precision  qtot
      double precision  qm0, qm1, qmc, qmc0 
c
c  prepare parameters.
c  
      qm0  = qmmminp%qmcharge
      qm1  = qmmminp%asqmcharge
c
c
      qmc  = (1.d0-px)*qm0 + px*qm1
      qtot = qtot0 + qm0      
      qtot1= qtot - qmc
      write(chargfile,100) qmc, qtot1               
100   format('next step CPS total charge: ',f12.9,2x,
     *      'next step FB group total charge: ',2x,f9.6,/)
      return     
      end
c
c*****************************************************************************
c  CALPX 
c*****************************************************************************
c
      subroutine calpx(qmmminp, de, muss, corrmu, temprt, px)
c
c     Subroutine calpx 
c
c     Created: Sep. 19, 2007 by Yan Zhang
c
c     Purpose: Calculate the added state weighting. 
c
c
c     Called by: fbbpc                         
c     Calls:                                         
c
      use input
      use files
      use caltyp
      implicit none
c                        
c i/o variables
c
      type (qmmminf)  :: qmmminp
      double precision  de,  muss, px, temprt, corrmu                                    
c
c local variables
c
      double precision  dq, qmcr, qmco, temp
      double precision  kb, ev, dqc             
c
c Constant. kb in eV/K.
c
      kb=8.617343D-5
      ev=27.21066
c                   
c                                              
      qmcr = qmmminp%qmcharge
      qmco = qmmminp%asqmcharge
      dqc  = qmcr - qmco
c
c     
      temp = -de/abs(dqc)
      temp = -muss - temp + corrmu
      px   = temp*ev/(kb*temprt)              
      px   = exp(px)
      px   = 1.d0/(1.d0 + px)            

      if(qmcr.gt.qmco) px =1.d0 - px 
      write(chargfile, 100) px
100   format(/'next step added state weighting px: ',f12.9/)

      return 
      end
c
c
c******************************************************************************
c DIFIMU 
c******************************************************************************
c
      subroutine difimu(qmprognme, geom, ssgeom, mmgeom, ssmmgeom,
     *           qmmminp, ccmeth, ione, corrmu)
c
c     Subroutine difimu 
c
c     Created:  Sep. 20, 2007 by Yan Zhang
c     Revised:  
C               
c     Purpose: This subroutine calculates the chemical potential correction between 
c              QM and QEQ (EEM) calculation.  
c
c     Called by: fbbpc 
c     Calls: cpsgasi, cpsmu 
c  
      use caltyp
      use input  
      use files  
      use energy 
      implicit none
c
c i/o variables
c
      character*10       qmprognme
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, ssmmgeom
      type (qmmminf)  :: qmmminp
      integer            idfbg
      character*9        ccmeth
      double precision   corrmu
c
c local variables
c
      integer            debug
      integer            qmr, qmo
      integer            qmnatm, capnatm, cssnatm, lsnatm
      double precision   energx, energxa, kb, ev
      double precision   ione, qmmu, muss         
c
      debug = 0
c
c Constant. kb in eV/K.
c
      kb=8.617343D-5
      ev=27.21066
c
      qmr      = qmmminp%qmcharge
      qmo      = qmmminp%asqmcharge
      qmnatm   = qmmminp%qmnatm
      capnatm  = qmmminp%capatm%capnatm
      cssnatm  = qmnatm + capnatm
c
c Calculate the gas ionization potential of CPS
c
      if(debug.eq.1) write(chargfile, 100) 
      call cpsgasi(qmprognme, ssgeom, qmmminp, ione)
c
c Calculate the QEQ(EEM) chemical potential of CPS
c      
      if(debug.eq.1) write(chargfile, 200) 
      call cpsmu(geom, mmgeom, ssgeom, ssmmgeom, qmmminp,
     *                ccmeth, cssnatm, muss)
c
c Calculate the corrected value of the chemical potential. 
c
      qmmu = -ione/abs(qmr-qmo)
      corrmu = qmmu + muss    
      if(debug.eq.1) write(chargfile,300) -corrmu 
c
100   format(/'Calculation of the ionization potential', 
     *       ' (electronic affinity) of gas CPS:')
200   format(/'Calculation of gas CPS chemical potential ',
     *       ' by the QEQ (EEM) method: ')
300   format('Chemical potential correction: ',f12.8/)

      return
      end
c
c
c******************************************************************************
c FBTEMPCALC
c******************************************************************************
c
      subroutine fbtempcalc(geom, ssgeom, mmgeom, ssmmgeom,
     *           qmmminp, ccmeth, temprt)
c
c     Subroutine difimu 
c
c     Created:  Apr. 14, 2009 by Yan Zhang
c     Revised:
C
c     Purpose: This subroutine calculates the electronic temperature parameter T
c              The idea is that the slope (first-order derivative) of the gas
c              CPS mu(e-) (eq. 11 J. Chem. Theory Comput. 4, 2008, 414-425)
c              is equal to that obtained by QEQ. 
c
c     Called by: fbbpc
c     Calls: cpsgasi, cpsmu
c
      use caltyp
      use input  
      use files  
      use energy 
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, ssmmgeom
      type (qmmminf)  :: qmmminp
      character*9        ccmeth
      double precision   temprt
c
c local variables
c      
      integer            qmr, qmo
      integer            qmnatm, capnatm, cssnatm, lsnatm
      double precision   dqic, kmuss
      double precision   muss0, muss1, cpsq0, cpsq1

      double precision   kb, ev

      integer debug
c
      debug = 0
c
c Constant. kb in eV/K.
c
      kb=8.617343D-5
      ev=27.21066d0
c
      qmr      = qmmminp%qmcharge
      qmo      = qmmminp%asqmcharge
      qmnatm   = qmmminp%qmnatm
      capnatm  = qmmminp%capatm%capnatm
      cssnatm  = qmnatm + capnatm
c
c the slope (first-order derivative) of the gas
c CPS mu(e-) (J. Chem. Theory Comput. 4, 2008, 414-425)
c k(x)=dmu/dx=kb*T*1/(x**2-x).  (mu in eq 11) 
c k(x)=dmu/dx=kb*T*1/(x-x**2).  (mu in eq 12)
c
c Calculate the slope of CPS by EQE
c 
      cpsq0 = qmr*1.d0 + qmo*0.d0
      call cpsmu1(geom, mmgeom, ssgeom, ssmmgeom, qmmminp,
     *                ccmeth, cssnatm, cpsq0, muss0)

      cpsq1 = qmr*0.d0 + qmo*1.d0
      call cpsmu1(geom, mmgeom, ssgeom, ssmmgeom, qmmminp,
     *                ccmeth, cssnatm, cpsq1, muss1)

      kmuss   = (muss0 - muss1)/dble(cpsq1 - cpsq0)
      if(debug.eq.1) write(chargfile,100)cpsq0,muss0
      if(debug.eq.1) write(chargfile,100)cpsq1,muss1
100   format('CPS Charge:',f9.5,3x,'QEQ Mu:',f12.6)
c
c calculate the electronic temperature parameter T based on
c dx/dq = 1/(qmo-qmr)
c dmu/dq=-4kb*T/(qmo-qmr)=((-muss1)-(-muss0))/(cpsq1-cpsq0) (qmo>qmr) 
c dmu/dq=4kb*T/(qmo-qmr)=((-muss1)-(-muss0))/(cpsq1-cpsq0) (qmo<qmr) 
c as x=0.5
c
      temprt = 0.d0
      dqic = dble(qmo-qmr)
      if(qmo.gt.qmr) then
         temprt = - kmuss*dqic*ev/kb/4.d0
      else if(qmo.lt.qmr) then
         temprt = kmuss*dqic*ev/kb/4.d0
      endif

      
      if(debug.eq.1) write(chargfile,200) kmuss, ev/kb/4.d0
200   format('QEQ Mu slope:',f12.6,2x,'1/(4kb) (a.u.):',f12.6)       
c

      return
      end
c
c
c******************************************************************************
c CPSGASI 
c******************************************************************************
c
      subroutine cpsgasi(qmprognme, ssgeom, qmmminp, ione)
c
c     Subroutine cpsgasi 
c
c     Created:  Sep. 20, 2007 by Yan Zhang
c     Revised:
c
C
c     Purpose: This subroutine calculates the ionization potential (electronic 
c              affinity) of gas CPS. 
c
c
c     Called by: difimu
c     Calls: progehk
c
      use caltyp
      use input
      use files
      use energy
      implicit none
c
c i/o variables
c
      character*10       qmprognme
      type (cartgeom) :: ssgeom
      type (qmmminf)  :: qmmminp
      double precision   ione
c
c local variables
c
      integer      ierr
      integer      qmnatm, capnatm, cssnatm, lsnatm
      integer      ssnatm, mmnatm, natm
      integer      qmr, qmo
      double precision   energx, energxa 

      integer      debug
c
      debug = 0
c

      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + mmnatm
      qmr      = qmmminp%qmcharge
      qmo      = qmmminp%asqmcharge
      natm     = ssgeom%natoms
      ssgeom%natoms = qmnatm + capnatm
c
c QM calculation of the initial state on the gas capped primary system.
c
      if(debug.eq.1) write(chargfile, 100) qmmminp%qmcharge 
      call progehk(qmprognme, qmmminp%qmmeth, qmmminp%qmbas,
     *           qmmminp%qmcharge, qmmminp%qmmult,	    
     *           ssgeom, qmmminp%qmoptl, energx, ierr)
      if(debug.eq.1) write(chargfile,200)energx
c
c QM calculation of the added state on the gas capped primary system.
c
      if(debug.eq.1) write(chargfile, 300) qmmminp%asqmcharge
      call progehk(qmprognme, qmmminp%qmmeth, qmmminp%qmbas,
     *           qmmminp%asqmcharge, qmmminp%asqmmult,
     *           ssgeom, qmmminp%qmoptl, energxa, ierr)
      if(debug.eq.1) write(chargfile,200)energxa
c
c  Calculate the chemical potential(a.u.) on the background point charge.
c
      if(qmo.gt.qmr) then
        ione = energxa - energx
      elseif(qmo.lt.qmr) then
        ione = energx - energxa
      else
        write(outfile,*)'Error: reduced and oxidized charge'
        stop 'in CPSGASI of flexbound.F'
      endif
      if(debug.eq.1) write(chargfile,500) ione
c
      ssgeom%natoms = natm
c
100   format(/,'QM calculation of origianl state',
     *          ' (charge=',i3,'):')   
200   format('Gas QM energy (a.u.) : ',f16.10)
300   format(/'QM calculation of additianal state',
     *          ' (charge=',i3,'):')

500   format(/'Ionization potential (Electronic affinity): ',
     *       f16.10/)
      return
      return
      end
c
c
c******************************************************************************
c CPSMU 
c******************************************************************************
c
      subroutine cpsmu(geom, mmgeom, ssgeom, ssmmgeom, qmmminp, 
     *                ccmeth, cssnatm, muss) 
c     Subroutine cpsmu 
c
c     Created:  Sep. 20, 2007 by Yan Zhang
c     Revised:  
c                              
c     Purpose: This subroutine calculates the chemical potential of gas CPS 
c              by QEQ(EEM) method. 
c               
c     Called by: difimu 
c     Calls: cpscoor, atmparam, maxtrixj12, slinequ 
c  
      use caltyp
      use input  
      use files  
      use energy 
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, ssmmgeom 
      type (qmmminf)  :: qmmminp
      character*9        ccmeth
      integer            cssnatm
      double precision   muss
c
c local variables
c      
      integer  i, j, k, qmic, qmac 
      double precision  xn0(maxganum), j12(cssnatm+1,cssnatm+1)
      double precision  jx(cssnatm,cssnatm), qn(maxganum)
      character*2  fbatm(maxganum) 
      double precision  x(maxganum), y(maxganum), z(maxganum)
      double precision  x0, jaa0  

      integer  debug
c
      debug = 0
c
c
      qmic  = qmmminp%qmcharge
      qmac  = qmmminp%asqmcharge
c
c Assign the coordinates of the CPS
c
      call cpscoor(geom, ssgeom, qmmminp, fbatm, x, y, z)
c
c  Calculate the maxtrix of right side for the linear equtaion of
c  the charge calculation.
c
      do i=1, cssnatm 
         call atmparam(qmmminp, ccmeth, fbatm(i), x0, jaa0)
         xn0(i) = -x0
      end do
      xn0(cssnatm + 1) = qmic*0.5d0 + qmac*0.5d0 
c
c  Determine the J matrix.
c
      call matrixj12(qmmminp, cssnatm, fbatm, x, y, z, ccmeth, jx)
      do i=1,cssnatm
         do j=1,cssnatm
           j12(i,j)=jx(i,j)
         end do
      end do
      do i=1,cssnatm
         j12(i,cssnatm+1) = -1.d0
         j12(cssnatm+1,i) =  1.d0
      end do
      j12(cssnatm+1,cssnatm+1) = 0.d0
c
c  Sovle the linear equation for the charge calculation.
c
      call slinequ(cssnatm+1, j12, xn0, qn)

      muss =  qn(cssnatm + 1)

      if(debug.eq.1) write(chargfile,*)
      do i=1, cssnatm 
         if(debug.eq.1) write(chargfile,200)fbatm(i), qn(i)
      end do
      if(debug.eq.1) write(chargfile,100) -muss
c
100   format(/'QEQ(EEM) CPS chemical potential: ',f12.8/)
200   format('Atom: ',a2,2x,'Charge: ',f12.6)
      return
      end      
c
c******************************************************************************
c  CPSCOOR
c******************************************************************************
c
      subroutine cpscoor(geom, ssgeom, qmmminp, fbatm, x, 
     *                       y, z)
c
c     Subroutine cpscoor
c
c     Created:  Sep. 14, 2007 by Yan Zhang
c     Revised:
c     Purpose:  To assign appropriate geometries to CPS atoms. 
c
c
c     Called by: cpsmu, cpsmu1
c     Calls:
c
      use input
      use atminfo
      use files
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (qmmminf)  :: qmmminp
      integer          :: nfbcps
      double precision :: x(maxatm), y(maxatm), z(maxatm)
      character*2 fbatm(maxatm)
c
c local variables
c
      integer  i
      integer  qmnatm, capnatm, cssnatm
      double precision   angtoau

      integer  debug
c
      debug = 0
c
c  unit conversion from bohr to angstrom.
c
      angtoau = 0.529177249d0
c
c  prepare parameters.
c
      qmnatm   = qmmminp%qmnatm
      capnatm  = qmmminp%capatm%capnatm
      cssnatm  = qmnatm + capnatm
c
c  Assign the CPS atom coordinates.
c
      if(debug.eq.1) write(chargfile,200)cssnatm
      do i = 1, cssnatm
         x(i) = ssgeom%aucoord(3*i-2)
         y(i) = ssgeom%aucoord(3*i-1) 
         z(i) = ssgeom%aucoord(3*i)
         fbatm(i) = ssgeom%atm(i)
         if(debug.eq.1) write(chargfile,100)fbatm(i),i,x(i)*angtoau,
     *                    y(i)*angtoau,z(i)*angtoau
      end do
100   format('Atom: ',a2,3x,'ID: ',i4,3x,'XYZ(ang.): ',
     *      3(f12.6,2x))
200   format(/'Total CPS atom numbers:',2x,i5)
c
      return
      end
c
c
c******************************************************************************
c  DVARZERO 
c******************************************************************************
c
      subroutine dvarzero(n, varb)

c     Subroutine dvarzero 
c
c     Created:  Sep. 26, 2007 by Yan Zhang
c     Revised:
c
c     Purpose: Set one dimension double precision variable to be zero. 
c
c
c     Called by: fbbpc, muvspx
c     Calls:
c
      implicit none
c
c i/o variables
c
      integer            n
      double precision   varb(n)
c
c local variables
c
      integer            i       
c
c
      do i=1,n
         varb(i) = 0.d0
      end do
      return
      end 
c
c
c******************************************************************************
c muvspx 
c******************************************************************************
c
      subroutine muvspx(geom, mmgeom, ssgeom, ssmmgeom, qmmminp, 
     *                qmprognme, de, corrmu, fbqtot0) 
c     Subroutine muvspx 
c
c     Created:  Oct. 11, 2007 by Yan Zhang
c     Revised:  
c                              
c               
c     Purpose: This subroutine calculate chemical potential as a function of added state
c              weighting or CPS charge. 
c               
c
c     Called by: fbbpc 
c     Calls: cpsmu1, gqtot, fbgacoor, dvarzero, polgchargpol, updatcharg,
c            fbpot, fbgpot, dqcon   
c  
      use caltyp
      use input  
      use files  
      use energy 
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, ssmmgeom 
      type (qmmminf)  :: qmmminp
      character*10       qmprognme
      double precision   de, corrmu, fbqtot0 
c
c local variables
c      
      integer  ii, j, ssnatm, qmis, qmas
      integer  ng, idg, idfbg, nfbga, it, maxstep 
      integer  qmnatm, capnatm, cssnatm
      character*2  fbatm(maxganum) 
      character*9  ccmeth
      double precision  muss, temprt, qtot, px         
      double precision  maxdq, rmsdq, maxgdq, rmsgdq 
      double precision  xg(maxganum), yg(maxganum), zg(maxganum)
      double precision  pot(maxatm), gpot(maxganum),qn(maxganum)
      double precision  qt(maxatm), qx(maxatm)               
      double precision  de1, kb, ev, mug, mue, qcps, mucps

      integer debug
c
      debug = 0
c
c
      kb=8.617343D-5
      ev=27.21066

      write(chargfile,100)
      write(chargfile,100)
      write(chargfile,110)
c
c  Prepare parameters.
c
      ccmeth  = qmmminp%fbccmeth
      temprt  = qmmminp%temprt
      maxstep = qmmminp%maxfbcycl
      ng      = qmmminp%fbgnum
      idfbg   = qmmminp%fbgid
      ssnatm  = ssmmgeom%natoms
      qmis    = qmmminp%qmcharge
      qmas    = qmmminp%asqmcharge
      qmnatm  = qmmminp%qmnatm
      capnatm = qmmminp%capatm%capnatm
      cssnatm = qmnatm + capnatm
c
c
      qx(:ssnatm) = ssmmgeom%atmcharge(:ssnatm)
      qt(:ssnatm) = ssmmgeom%atmcharge(:ssnatm)
c
c Iteration as px changed.               
c
      do j=1,49
        px = j*0.02d0 
        if(qmis.gt.qmas) then
          write(chargfile,130) px
        else
          write(chargfile,120) px
        endif
c
c At the given px, calculate the gas mu(e) without correction. 
c          
        if(qmas.gt.qmis) then 
         mug = -de + kb*temprt*log((1.d0-px)/px)/ev 
        else
         mug = -de + kb*temprt*log(px/(1.d0-px))/ev
        endif
  
        qtot = fbqtot0 + qmis - (1.d0 - px)*
     *         qmis - px*qmas
        qcps = (1.d0 - px)*qmis + px*qmas
        write(chargfile,140)
        call cpsmu1(geom, mmgeom, ssgeom, ssmmgeom, qmmminp,
     *                ccmeth, cssnatm, qcps, mucps)
c
c  Calculate FB group charge 
c
         write(chargfile,150)
         nfbga = qmmminp%fbgan(idfbg)
         call fbgacoor(geom, qmmminp, fbatm, idfbg, xg, yg, zg)
         call dvarzero(maxganum, gpot)
         write(chargfile,*)
         call polgchargcal(geom, ssgeom, mmgeom, ssmmgeom, qmmminp,
     *      ccmeth, idfbg, nfbga, fbatm, xg, yg, zg, gpot, qtot, qn)
         call updatcharg(ssmmgeom%atmcharge, qmmminp, idfbg, qn)
         call gqtot(ssmmgeom%atmcharge, qmmminp, idfbg, qtot)
         write(chargfile,160)qcps, qmis, 1-px, qmas, px
         write(chargfile,170) qcps, mug, -mucps, -qn(nfbga+1)   
c
c  Loop of iterative polarized charge calculations at the given x  
c  Electric potential on the atom of SS subsystem.
c
         write(chargfile,180)
         it = 0
1000     call fbpot(qmprognme, geom, ssgeom, mmgeom, ssmmgeom,
     *           qmmminp, px, pot)
c
c  Calculate charge of each group by QEQ(EEM) method 
c         
         maxdq   = 0.d0
         rmsdq   = 0.d0
         write(chargfile,230)
         do idg=1, ng
            nfbga = qmmminp%fbgan(idg)
            call fbgacoor(geom, qmmminp, fbatm, idg, xg, yg, zg)
            call gqtot(ssmmgeom%atmcharge, qmmminp, idg, qtot)
            call fbgpot(qmmminp, idg, pot, gpot)
            call polgchargcal(geom, ssgeom, mmgeom, ssmmgeom, qmmminp, 
     *           ccmeth, idg, nfbga, fbatm, xg, yg, zg, gpot, qtot, qn)
            call dqcon(ssmmgeom, qmmminp, idg, qn, maxgdq, rmsgdq)
            call updatcharg(qt, qmmminp, idg, qn)
            call gqtot(qt, qmmminp, idg, qtot)
            if (idg.eq.idfbg) muss = qn(nfbga+1)                        
            if (idg.eq.idfbg) write(chargfile,270) -qn(nfbga + 1)
            if(maxgdq.gt.maxdq)  maxdq = maxgdq
            if(rmsgdq.gt.rmsdq)  rmsdq = rmsgdq
         enddo    
         ssmmgeom%atmcharge(:ssnatm) = qt(:ssnatm)
         write(chargfile,300)maxgdq, rmsgdq
         it = it+1
         if(it.ge. maxstep) goto 2000
         write(chargfile,210)
         if(maxdq.gt.qmmminp%maxfbdq.or.rmsdq.gt.qmmminp%rmsfbdq)
     *              goto 1000 
2000     continue  
         if(it.ge. maxstep) then
            write(chargfile,220)temprt, qcps
            write(chargfile,210)
            write(chargfile,210)
         else
            write(chargfile,200) temprt, qcps, mug-corrmu, -muss
            write(chargfile,210)
            write(chargfile,210)
         end if
      end do

      ssmmgeom%atmcharge(:ssnatm) =  qx(:ssnatm)

      write(chargfile,100)
      write(chargfile,100)

100   format(80('-'))
110   format('CPS charge scan calculation.')
120   format(/'Oxidized state weighting: ',f12.9)
130   format(/'Reduced state weighting: ',f12.9)
140   format(/'Gas CPS QEQ(EEM) calculation:'/)
150   format('calculate the gas FB group charge.')
160   format(/'Summary1: Q(CPS)=',f9.5,2x,'W(C',i2,
     *      ')=',f9.5,2x,'W(C',i2,')=',f9.5)
170   format('Summary2: Q(CPS)=',f9.5,2x,'nocorr Mu(e)=',
     *       f12.8,2x,'Gas Mu(CPS;QEQ)=',f9.5,2x,'Gas Mu(SS;QEQ)=',
     *       f12.8/)
180   format(/'SS atom charge calculation of the polarization.'/)
190   format('FB group chemical potential: ',f12.6/) 
200   format(/'Summary3: T=',f12.4,2x,'Q(CPS)=',f9.5,      
     *       2x,'corr Mu(e)=',f12.8,2x,'pol Mu(SS;QEQ)=',f12.8/)
210   format(60('-')) 
220   format(/'Summary3: T=',f12.4,2x,'Q(CPS)=',f9.5,
     *       2x,'Pol SS charge calculation not convergence'/)
230   format(/'Polarized charge calculation')
270   format('FB group chemical potential: ',f12.6/)
300   format('GROUP:MAXDQ',3x,f9.6,6x,'GROUP:RMSDQ',3x,f9.6/)  

      return
9000  write(outfile,*)'Error: Felxible-boundary treatment do not',
     *      ' support the mechanical embedding.'
      stop 'fbbpc 1'
      end      
c
c
c******************************************************************************
c CPSMU1
c******************************************************************************
c
      subroutine cpsmu1(geom, mmgeom, ssgeom, ssmmgeom, qmmminp,
     *                ccmeth, cssnatm, qcps, muss)
c     Subroutine cpsmu
c
c     Created:  Sep. 20, 2007 by Yan Zhang
c     Revised:
c
c
c     Purpose: This subroutine calculates the chemical potential of gas CPS
c              by QEQ(EEM) method.
c
c     Called by: muvcpx
c     Calls: cpscoor, atmparam, maxtrixj12, slinequ
c 
      use caltyp
      use input
      use files
      use energy
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom, ssgeom
      type (mmvalen)  :: mmgeom, ssmmgeom
      type (qmmminf)  :: qmmminp
      character*9        ccmeth
      integer            cssnatm
      double precision   muss, qcps
c
c local variables
c
      integer  i, j ,k
      double precision  xn0(maxganum), j12(cssnatm+1,cssnatm+1)
      double precision  jx(cssnatm,cssnatm), qn(maxganum)
      character*2  fbatm(maxganum)
      double precision  x(maxganum), y(maxganum), z(maxganum)
      double precision  x0, jaa0

      integer debug
c
      debug = 0
c
c Assign the coordinates of the CPS
c
      call cpscoor(geom, ssgeom, qmmminp, fbatm, x, y, z)
c
c  Calculate the maxtrix of right side for the linear equtaion of
c  the charge calculation.
c
      do i=1, cssnatm
         call atmparam(qmmminp, ccmeth, fbatm(i), x0, jaa0)
         xn0(i) = -x0
      end do
      xn0(cssnatm + 1) = qcps 
c
c  Determine the J matrix.
c
      call matrixj12(qmmminp, cssnatm, fbatm, x, y, z, ccmeth, jx)
      do i=1,cssnatm
         do j=1,cssnatm
           j12(i,j)=jx(i,j)
         end do
      end do
      do i=1,cssnatm
         j12(i,cssnatm+1) = -1.d0
         j12(cssnatm+1,i) =  1.d0
      end do
      j12(cssnatm+1,cssnatm+1) = 0.d0
c
c  Sovle the linear equation for the charge calculation.
c
      call slinequ(cssnatm+1, j12, xn0, qn)
      muss =  qn(cssnatm + 1)

      if(debug.eq.1) then 
          do i=1, cssnatm
             write(chargfile,200)fbatm(i), qn(i)
          end do
          write(chargfile,100)muss
      endif
c
100   format(/'QEQ(EEM) CPS chemical potential: ',f12.8/)
200   format('Atom: ',a2,2x,'Charge: ',f12.6)
      return
      end
c
c
c*****************************************************************************
c  G03BGPOTINP
c*****************************************************************************
c
      subroutine g03bgpotinp(iunit,esmeth,esbas,qmcharge, qmmult,
     *   ssgeom,optl,type,ver_lab, qmmminp, ssmmgeom)
c
c     Subroutine g03bgpotinp
c     
c     Created:  Oct.19, 2007 by Yan Zhang
c     Revised:  
c
c     Purpose:  Write a Gaussian03 input file for the ESP charge and electric 
c               potential in presence of background point charges.
c                                                                       
c 
c     Called by: progcpe  
c     Calls:
c
      use files
      use caltyp                                                        
      use input
      implicit none
c
c i/o variables
c 
      character*20 ver_lab
      character*20 esmeth
      character*20 esbas
c
      character*80 optl(maxoptl)   
      integer      iunit, type
      type (cartgeom) :: ssgeom   
      type (mmvalen)  :: ssmmgeom   
      type (qmmminf)  :: qmmminp
c
c local variables
c
      logical      linkzero
      integer      mfin, bfin, mend, bend, i, j, k
      
      integer      ii, jj, kk
      integer      qmnatm,capnatm,cssnatm,lsnatm
      integer      ssnatm,bgchnatm,mmnatm,lsymbol
      integer      qmcharge, qmmult
      
      character*2  atmsymbol
      character*4  typname
      
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      ssnatm   = ssmmgeom%natoms
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + mmnatm

      mfin = 1
      bfin = 1
      mend = 0
      bend = 0	 
c
c  Find ends of method and basis set specification
c
      call fspace(esmeth, mfin, mend)
      if (mend.eq.1) then
         mfin = len(esmeth)
      else
         mfin = mfin-1
      endif
      call fspace(esbas, bfin, bend)
      if (bend.eq.1) then
         bfin = len(esbas)
      else
         bfin = bfin-1
      endif
c
c     Memory and processors (specified in the user inputed options)      
c     linkzero is a logical (boolean) variable to determine if the current
c     line being read is a link0 command or a keyword option
c
      linkzero = .false.
      i = 1                                                             
      do 10 j=1,10                                                      
      if (optl(i)(j:j) .eq. '%' ) then                                  
      linkzero = .true.                                                 
      endif                                                             
10    CONTINUE                                                          
      do while (linkzero .and. i.le.maxoptl)             
         write(iunit,*) optl(i)                                         
         i = i+1                                                        
         linkzero = .false.                                             
          do 12 j=1,10                                                  
            if (optl(i)(j:j) .eq. '%' ) then                            
               linkzero = .true.                                        
            endif                                                       
12    CONTINUE                                                          
      end do                                                            
c
c  Method and basis 
c
      write(iunit,2000) esmeth(:mfin), '/', esbas(:bfin)  
2000  format('# ',a,a1,a)
c
c  User inputed keywords
c
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
c
         do j=1, 79
            if (optl(i)(j:j) .eq. '!') then
               k = 1
            endif
         enddo
         if (k .eq. 0) then
            write(iunit,3000) '#', optl(i)
         endif
         i = i+1
      end do
c
3000  format (a1,a79)      
     
c
c   ESP charge, charge and potential
c
      write(iunit,3001) '#', 'charge'
      write(iunit,3002) '#','prop=(read,potential)'
3001  format (a1,5x,a6,2x,'fchk')      
3002  format (a1,5x,a25)      
c
c  Title
c
      write(iunit,3021) 
3021  format (/'Electric Potential Calculation'/)
c
c  Charge and multiplicity
c
      write(iunit,3050) qmcharge, qmmult
3050  format (1x,i3,1x,i3)      
c
c  QM: atomic symbols and coordinates
c
      do i=1, qmnatm
         lsymbol = 0
         atmsymbol = ssgeom%atm(i)	    	 
         do j=1, 2
            if (atmsymbol(j:j) .ne. ' ') then 
               lsymbol = lsymbol + 1
            endif 
      enddo
         
         write(iunit,4000)
     *      atmsymbol(:lsymbol), ssgeom%angcoord(3*i-2), 
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)
      enddo     
4000  format (a,3x,3(d16.9,3x))
c
c  Cap: atomic symbols and coordinates
c       take H as symbol and HC as atomtype, as defined parameters
c
      do i = qmnatm + 1, cssnatm
         atmsymbol = capname
         write(iunit,4000)
     *      atmsymbol(:lsymbol), ssgeom%angcoord(3*i-2), 
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)				 
      enddo
c
c Options before 'charge' 
c
      i = 1
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
         jj = 1
c
         do j=1, 79
            if (optl(i)(j:j+6) .eq. '!charge') then
               k = 1
               jj = j + 7
            endif
         enddo
         if (k .eq. 1) then
c
            write(iunit,5000) optl(i)(jj:80)
         endif
         i = i + 1
      enddo
c
5000  format (a79)      
c
c  MM and M1/auxiliary charges: atomic symbols and coordinates
c     take He as symbol and HC as atomtype, as defined parameters     
c
      write(iunit,*)
      do i=cssnatm+1,ssnatm
         write(iunit,4100)
     *      ssgeom%angcoord(3*i-2),ssgeom%angcoord(3*i-1),
     *      ssgeom%angcoord(3*i),ssmmgeom%atmcharge(i)           	 	 
      enddo
      
4100  format (3x,3(d16.9,3x),2x,d10.4)
c
c write additional options for molecule specification
c before 'prop'
c
      i = 1
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
         jj = 1
c
         do j=1, 79
            if (optl(i)(j:j+4) .eq. '!prop') then
               k = 1
               jj = j + 5
            endif
         enddo
         if (k .eq. 1) then
c
            write(iunit,5000) optl(i)(jj:80)
         endif
         i = i + 1
      enddo
c
c  MM and M1/auxiliary charges: electric potential 
c
      write(iunit,*)
      do i=cssnatm+1,ssnatm
         write(iunit,4200)
     *      ssgeom%angcoord(3*i-2),ssgeom%angcoord(3*i-1),
     *      ssgeom%angcoord(3*i)
      enddo

      write(iunit,*)
4200  format (3x,3(d16.9,3x))
c
c  Close the file
c
      write(iunit,*)
      write(iunit,*)      
      close (iunit, err=9999)
      
      return
9999  write (outfile,*) 'Error creating Gaussian input file'
      stop 'g03bgpotinp'
      end
c
c
c*****************************************************************************
c  G03OUTEP
c*****************************************************************************
c
      subroutine g03outep(geom, mmgeom, ssgeom,ssmmgeom, qmmminp, pot)
c
c     Subroutine g03outep
c     
c     Created:  Oct.19, 2007 by Yan Zhang
c     Revised:  
c
c     Purpose:  Read the background charge potential in the g03.out file.
c                                                                       
c 
c     Called by: progcpe  
c     Calls: case
c
      use files
      use caltyp                                                        
      use input
      implicit none
c
c i/o variables
c 
      type (cartgeom) :: geom, ssgeom   
      type (mmvalen)  :: mmgeom,ssmmgeom   
      type (qmmminf)  :: qmmminp
      
      double precision   pot(maxatm)
c
c local variables
c
      integer      i, j, k, ii, jj, kk
      integer      qmnatm,capnatm,cssnatm,lsnatm
      integer      ssnatm,bgchnatm,mmnatm
      integer      m1st, m2nd, m3rd
      double precision ptmp
 
      character*4  ctmp
c
      character*80 string1, string, case
c
c
      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      ssnatm   = ssmmgeom%natoms
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + mmnatm
      bgchnatm = ssnatm - lsnatm
      m1st = qmmminp%mm1stlay%natm1stlay
      m2nd = qmmminp%mm2ndlay%natm2ndlay     
      m3rd = qmmminp%mm3rdlay%natm3rdlay
      string1 = '   '
c
c    Open the g03.out
c
      open(unit=100,status='unknown', file='g03.out', err=9900)            
c
c    Read QM and cap atom
c
      string1 = '      '
c
      read( 100, '(a80)', end=9910 ) string                               
      string = case(string)
      do while( (string1(16:23).ne.'electric').or.
     &             (string(16:24).ne.'potential'))
         string1=string
c
         read( 100, '(a80)', end=9910 ) string                            
         string = case(string)
      end do
c
      read( 100, '(a80)', end=9910 ) string
      do i=1,cssnatm
         read(100, *, end=9910) j, ctmp, ptmp
      end do
c
c    Read background charge electric potential
c
      do i=cssnatm+1,lsnatm
         k=ssmmgeom%atmid(i)
         read(100,*,end=9910)ii,pot(k)
      enddo
      close(100)
      return
9900  write( outfile,*) 'Error opening Gaussian output file in g03outep'
      stop 'g03outep 1'
9910  write( outfile,*) 'Error reading Gaussian output file in g03outep'
      stop 'g03outep 2'
      end
c
c*******************************************************************
c*******************************************************************
c      




