c************************************************************************
c     Subroutine freqcal 
c************************************************************************
c 
      subroutine freqcal(geom)
c  
c     Created:  Feb. 21, 2005   Hai Lin
c               based on Yan Zhao's multilevel version
c     Purpose:  Normal mode analysis is performed in this subroutine and 
c     frequencies and normal mode coordinate are calculated from the hessian
c      matrix.
c 
c     Called by:  mlhhook
c     Calls: 
c
      use optimize, only : hess
      use input, only : general, cartgeom
      use files, only : outfile, infile
c
      type (cartgeom) :: geom
      integer  i, j, k, n3tm, ij, m, ii, ll, mm, lstr, lend
      integer  kstr,kend
      double precision temp
      double precision, dimension(:,:), allocatable :: hessms, u, uu
      double precision, dimension(:), allocatable :: hessc, massvec,nn
      double precision, dimension(:), allocatable :: eigval, uc, freq 
      double precision, dimension(:,:), allocatable :: hessmsa, ua
      double precision, dimension(:), allocatable :: eigvala, uca
      double precision, dimension(:), allocatable :: hessca,freqa
      
      character*1, dimension(:,:), allocatable :: xyz
      character*1, dimension(:),allocatable :: ima, imaa
      integer, dimension(:), allocatable :: ind
c
      n3tm = 3*geom%natoms
c     
c    
      allocate (hessms(n3tm,n3tm),massvec(n3tm),hessc(n3tm*n3tm))
      allocate  (xyz(3,n3tm),ima(n3tm),ind(n3tm),uu(n3tm,n3tm),nn(n3tm))
      allocate (u(n3tm,n3tm),uc(n3tm*n3tm),eigval(n3tm),freq(n3tm))
      allocate (hessmsa(n3tm,n3tm), hessca(n3tm*n3tm),freqa(n3tm))
      allocate (eigvala(n3tm), ua(n3tm,n3tm))
      allocate (uca(n3tm*n3tm), imaa(n3tm))
c
c     get the mass vector 
c
      do i=1, n3tm
         massvec(i)=geom%atmmss(((i-1)/3)+1)
      enddo
c     
c     calculate the mass scaled hessian  
c
      do i=1, n3tm
        do j=1, i
          hessms(i,j)=hess(i,j)/(sqrt(massvec(i)*massvec(j)))
          hessms(j,i)=hessms(i,j)
          hessmsa(i,j)=hessms(i,j)
          hessmsa(j,i)=hessms(j,i)
        enddo
      enddo
      call prjfc(hessms,geom%aucoord,n3tm,geom%atmmss) 
c
c      diagonalize  projected and unprojected hessian 
c 
      ij = 0    
      do i = 1, n3tm
        do j = 1, i
          ij = ij+1
          hessc(ij) = hessms(i,j)  
          hessca(ij) = hessmsa(i,j)
        enddo
      enddo
      call rsp(hessc,n3tm,n3tm,eigval,uc) 
      call rsp(hessca, n3tm, n3tm, eigvala, uca)
      ij = 0
      do i = 1, n3tm
        do j = 1, n3tm
          ij = ij+1
          u(j,i) = uc(ij)
          ua(j,i)= uca(ij)
        enddo
       enddo
c
c    calculate the frequencies from the projected Hessian
c
      do i=1, n3tm 
       if (eigval(i) .ge. 0.0D0) then
        ima(i) = " "
        freq(i) = sqrt( eigval(i)*26422760.0)
       else
        ima(i) = "i"
        freq(i) = sqrt(-1*eigval(i)*26422760.0)
       endif
      enddo
c
c    calculate the frequencies from the unprojected Hessisan
c
      do i=1, n3tm
       if (eigvala(i) .ge. 0.0D0) then
        imaa(i) = " "
        freqa(i) = sqrt( eigvala(i)*26422760.0)
       else
        imaa(i) = "i"
        freqa(i) = sqrt(-1*eigvala(i)*26422760.0)
       endif
      enddo

c
c      calculate the normal mode coordinates in cartesian coordinates
c
      do i = 1, n3tm
        temp = 0
       do j = 1, n3tm
        if (freq(i) .le. 5.0) then 
          u(j,i)= ua(j,i)
        endif
        uu(j,i) = u(j,i)/sqrt(massvec(j))
        temp = temp + uu(j,i)* uu(j,i)
       enddo
        nn(i) = sqrt(temp) 
      enddo        
      do i = 1, n3tm
       do j = 1, n3tm
        uu(j,i) = uu(j,i)/nn(i)
       enddo
      enddo
c
c      write out the frequencies and normal mode
c
      do i=1, n3tm
       ind(i) = i
       xyz(1,i) = 'X'
       xyz(2,i) = 'Y'
       xyz(3,i) = 'Z'
      enddo
      write(outfile,3300)
      do m = n3tm, 1, -2
       lstr = m
       lend = m-1
       if (lend .le. 1) lend = 1 
       write(outfile,4000) (n3tm+1-ind(ii), ii=lstr,lend,-1)
       write(outfile,4100) (freq(ii),ima(ii),ii=lstr,lend,-1)
       write(outfile,4200) ((xyz(mm,ll),mm=1,3),ll=lstr,lend,-1)
       na = 1
       do i = 1, n3tm, 3
         kstr = i
         kend = i+2
         write(outfile,4300) na,((u(k,l), k=kstr,kend),l=lstr,lend,-1) 
         na = na + 1
       enddo
      enddo
      write(outfile,3400)
      do m = n3tm, 1, -2
        lstr = m
        lend = m-1
        if (lend .le. 1) lend = 1
        write(outfile,4000) (n3tm+1-ind(ii), ii=lstr,lend,-1)
        write(outfile,4100) (freq(ii),ima(ii),ii=lstr,lend,-1)
        write(outfile,4200) ((xyz(mm,ll),mm=1,3),ll=lstr,lend,-1)
        na = 1
        do i = 1, n3tm, 3
         kstr = i
         kend = i+2
         write(outfile,4300) na,((uu(k,l), k=kstr,kend),l=lstr,lend,-1)
         na = na + 1
        enddo
       enddo

c      write(outfile,3500)
c      do m = n3tm, 1, -2
c        lstr = m
c        lend = m-1
c        if (lend .le. 1) lend = 1
c        write(outfile,4000) (n3tm+1-ind(ii), ii=lstr,lend,-1)
c        write(outfile,4100) (freqa(ii),imaa(ii),ii=lstr,lend,-1)
c        write(outfile,4200) ((xyz(mm,ll),mm=1,3),ll=lstr,lend,-1)
c        na = 1
c        do i = 1, n3tm, 3
c         kstr = i
c         ckend = i+2
c         write(outfile,4300) na,((ua(k,l), k=kstr,kend),l=lstr,lend,-1)
c         na = na + 1
c        enddo
c       enddo
     
 3300 FORMAT(/1X,'Frequencies and normalized eigenvector components ',
     *       /1X,'in mass-scaled space-fixed cartesian coordinates')
 3400 FORMAT(//1X,'Frequencies and normalized eigenvector components ',
     *       /1X,'in cartesian coordinates')
 3500 FORMAT(//1X,'Frequencies and normalized eigenvector components',
     *  /1X,'in mass-scaled space-fixed cartesian coordinates(unprj)')

 4000  FORMAT(/,1X,4HMode,16X,I3,27X,I3)
 4100  FORMAT(1X,'omega (cm**-1)',2X,F9.2,1X,A1,(19X,F9.2,1X,A1))
 4200  FORMAT(/,1X,4HAtom,2(3(8X,A1),3X))
 4300  FORMAT(1X,I3,5X,2(3(F7.3,2X),3X))
 2001  Format(t20,d15.8,t40,d15.8,t60,d15.8) 
       return
      end

c*******************************************************************
c      subroutine prjfc(f,xparam,nvar,atmass) 
c*******************************************************************
c
      subroutine prjfc(f,xparam,nvar,atmass) 
c                                                                       
c  calculates projected force constant matrix (f).                      
c  this routine came originally from polyrate. it is used by permission
c  of d. truhlar. the current version is lifted from gamess and 
c  adapted by f.jensen, odense, dk             
c  if we are at a stationary point (stpt=.t.), i.e. gnorm .lt. 10,      
c  then the rotational and translational modes are projected out        
c  and their frequencies become identical zero. if not at a stationary  
c  point then the mass-weighted gradient is also projected out and      
c  the corresponding frequency become zero.                             
c ************************************************                      
c   x : mass-weighted coordinate                                        
c   dx: normalized mass-weighted gradient vector                        
c   f : mass-weighted force constant matrix                             
c   rm: inversion of square root of mass                                
c   p, cof: buffer                                                      
c                                                                       
      use files, only : outfile, infile
      
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION X(nvar),RM(nvar),F(nvar,nvar)   
      DIMENSION P(nvar,nvar),COF(nvar,nvar), atmass(nvar/3) 
      DIMENSION TENS(3,3,3),ROT(3,3),SCR(3,3),ISCR(6),CMASS(3)  
      dimension coord(3,nvar/3),dx(nvar),xparam(nvar)
      PARAMETER (ZERO=0.0d+00, ONE=1.0d+00, EPS=1.0d-14,
     *           CUT5=1.0d-05, CUT8=1.0d-08)                            
      integer l, ij, i, j, ia, ib, ic, ja, jb, jc, ip, jp
C                                                                       
C TOTALLY ASYMMETRIC CARTESIAN TENSOR.                                  
      DATA TENS/ 0.0d+00,  0.0d+00,  0.0d+00,                           
     X           0.0d+00,  0.0d+00, -1.0d+00,                           
     X           0.0d+00,  1.0d+00,  0.0d+00,                           
     Y           0.0d+00,  0.0d+00,  1.0d+00,                           
     Y           0.0d+00,  0.0d+00,  0.0d+00,                           
     Y          -1.0d+00,  0.0d+00,  0.0d+00,                           
     Z           0.0d+00, -1.0d+00,  0.0d+00,                           
     Z           1.0d+00,  0.0d+00,  0.0d+00,                           
     Z           0.0d+00,  0.0d+00,  0.0d+00  /                         
C                                                                       
      natm=nvar/3                                                        
      nc1=nvar
      ij=1
      do 2 i=1,natm                                                    
         coord(1,i)=xparam(ij)
         coord(2,i)=xparam(ij+1)
         coord(3,i)=xparam(ij+2)
         ij=ij+3
2     continue                                                          
C     CALCULATE 1/SQRT(MASS)                                            
      L=0                                                               
      DO 3  I=1, natm 
         TMP=ONE/SQRT(ATMASS(I))                                        
         DO 3 J=1,3                                                     
            L=L+1                                                       
           RM(L)=TMP
3     continue

C     PREPARE GRADIENT                                                  
         DO 4 I=1,nvar                                                   
4        DX(I)=ZERO                                                     
C     FIND CMS AND CALCULATED MASS WEIGHTED COORDINATES                 
      totm=zero                                                         
      cmass(1)=zero                                                     
      cmass(2)=zero                                                     
      cmass(3)=zero                                                     
      DO 6 I=1,NATM                                                     
         TOTM=TOTM+ATMASS(I)                                            
         DO 6 J=1,3                                                     
            CMASS(J)=CMASS(J)+ATMASS(I)*COORD(J,I)                      
6     CONTINUE                                                          
      DO 7 J=1,3                                                        
7     CMASS(J)=CMASS(J)/TOTM       
      L=0                                                               
      DO 8 I=1,NATM                                                     
         DO 8 J=1,3                                                     
         TMP=SQRT(ATMASS(I))                                            
             L=L+1                                                      
             X(L)=TMP*(COORD(J,I)-CMASS(J))
8     CONTINUE 
C
C 2. COMPUTE INERTIA TENSOR.          
C                                  
      DO 10 I=1,3                                                       
       DO 10 J=1,3                                                      
   10   ROT(I,J)=ZERO                                                   
      DO 20 I=1,NATM                                                    
       L=3*(I-1)+1                                                      
       ROT(1,1)=ROT(1,1)+X(L+1)**2+X(L+2)**2    
       ROT(1,2)=ROT(1,2)-X(L)*X(L+1)       
       ROT(1,3)=ROT(1,3)-X(L)*X(L+2)  
       ROT(2,2)=ROT(2,2)+X(L)**2+X(L+2)**2  
       ROT(2,3)=ROT(2,3)-X(L+1)*X(L+2)      
   20  ROT(3,3)=ROT(3,3)+X(L)**2+X(L+1)**2    
      ROT(2,1)=ROT(1,2)       
      ROT(3,1)=ROT(1,3)        
      ROT(3,2)=ROT(2,3)   
C                                                                       
CHECK THE INERTIA TENSOR.                                               
      CHK=ROT(1,1)*ROT(2,2)*ROT(3,3)                                    
      IF(ABS(CHK).GT.CUT8) GO TO 21                                     


      IF(ABS(ROT(1,1)).GT.CUT8) GO TO 11                                
C X=0                                                                   
      IF(ABS(ROT(2,2)).GT.CUT8) GO TO 12                                
C X,Y=0                                                                 
      IF(ABS(ROT(3,3)).GT.CUT8) GO TO 13                                
      WRITE(6,14) ROT(1,1),ROT(2,2),ROT(3,3)                            
   14 FORMAT(1X,'EVERY DIAGONAL ELEMENTS ARE ZERO ?',3F20.10)           
      RETURN                                                            
C                                                                       
C* 1. X,Y=0 BUT Z.NE.0                                                  
   13 ROT(3,3)=ONE/ROT(3,3)                                             
      GO TO 22                                                          
C Y.NE.0                                                                
   12 IF(ABS(ROT(3,3)).GT.CUT8) GO TO 15                                

C* 2. X,Z=0 BUT Y.NE.0                                                  
      ROT(2,2)=ONE/ROT(2,2)                                             
      GO TO 22                                                          
C X.NE.0                                                                
   11 IF(ABS(ROT(2,2)).GT.CUT8) GO TO 16                                
      IF(ABS(ROT(3,3)).GT.CUT8) GO TO 17                                

C* 3. Y,Z=0 BUT X.NE.0                                                  
      ROT(1,1)=ONE/ROT(1,1)                                             
      GO TO 22                                                          

C* 4. X,Y.NE.0 BUT Z=0                                                  
   16 DET=ROT(1,1)*ROT(2,2)-ROT(1,2)*ROT(2,1)                           
      TRP=ROT(1,1)                                                      
      ROT(1,1)=ROT(2,2)/DET                                             
      ROT(2,2)=TRP/DET                                                  
      ROT(1,2)=-ROT(1,2)/DET                                            
      ROT(2,1)=-ROT(2,1)/DET                                            
      GO TO 22                                                          

C* 5. X,Z.NE.0 BUT Y=0                                                  
   17 DET=ROT(1,1)*ROT(3,3)-ROT(1,3)*ROT(3,1)                           
      TRP=ROT(1,1)                                                      
      ROT(1,1)=ROT(3,3)/DET                                             
      ROT(3,3)=TRP/DET                                                  
      ROT(1,3)=-ROT(1,3)/DET                                            
      ROT(3,1)=-ROT(3,1)/DET                                            
      GO TO 22                                                          

C* 6. Y,Z.NE.0 BUT X=0                                                  
   15 DET=ROT(3,3)*ROT(2,2)-ROT(3,2)*ROT(2,3)                           
      TRP=ROT(3,3)                                                      
      ROT(3,3)=ROT(2,2)/DET                                             
      ROT(2,2)=TRP/DET                                                  
      ROT(3,2)=-ROT(3,2)/DET                                            
      ROT(2,3)=-ROT(2,3)/DET                                            
      GO TO 22                                                          
   21 CONTINUE                                                          

C                                                                       
C 4. COMPUTE INVERSION MATRIX OF ROT.                                   
C
      INFO=0                                                            
      CALL DGEFA(ROT,3,3,ISCR,INFO)                                     
      IF(INFO.NE.0) STOP                                                
      DET=ZERO                                                          
      CALL DGEDI(ROT,3,3,ISCR,DET,SCR,1)                                
C                                                                       
   22 CONTINUE                                                          
C                                                                       
C 5. TOTAL MASS ---> TOTM.                                              
C                                                                       
C 6. COMPUTE P MATRIX                                                   
C    ----------------                                                   
      DO 100 IP=1,NATM                                                  
       INDX=3*(IP-1)                                                    
       DO 100 JP=1,IP                                                   
        JNDX=3*(JP-1)                                                   
        DO 70 IC=1,3                                                    
         JEND=3                                                         
         IF(JP.EQ.IP) JEND=IC                                           
         DO 70 JC=1,JEND                                                
          SUM=ZERO                                                      
          DO 50 IA=1,3                                                  
           DO 50 IB=1,3                                                 
            IF(TENS(IA,IB,IC).EQ.0) GO TO 50                            
            DO 30 JA=1,3                                                
             DO 30 JB=1,3                                               
              IF(TENS(JA,JB,JC).EQ.0) GO TO 30                          
              SUM=SUM+TENS(IA,IB,IC)*TENS(JA,JB,JC)*ROT(IA,JA)*         
     &                X(INDX+IB)*X(JNDX+JB)                             
   30         CONTINUE                                                  
   50       CONTINUE                                                    
          II=INDX+IC                                                    
          JJ=JNDX+JC                                                    
          P(II,JJ)=SUM+DX(II)*DX(JJ)                                    
          IF(IC.EQ.JC) P(II,JJ)=P(II,JJ)+ONE/(RM(II)*RM(JJ)*TOTM)       
   70     CONTINUE                                                      
  100   CONTINUE 
C                                                                       
C 7. COMPUTE DELTA(I,J)-P(I,J)                                          
      DO 110 I=1,NC1                                                    
       DO 110 J=1,I                                                     
        P(I,J)=-P(I,J)                                                  
        IF(I.EQ.J) P(I,J) = ONE +P(I,J)                                 
  110   CONTINUE                                                        
C                                                                       
C 8. NEGLECT SMALLER VALUES THAN 10**-8.                
C                
      DO 120 I=1,NC1                                                    
       DO 120 J=1,I                                                     
        IF(ABS(P(I,J)).LT.CUT8) P(I,J)=ZERO                             
        P(J,I)=P(I,J)                                                   
  120   CONTINUE              
                                          
C                                                                       
C 10. POST AND PREMULTIPLY F BY P.                                      
C     USE COF FOR SCRATCH.  
      DO 150 I=1,NC1                                                    
       DO 150 J=1,NC1                                                   
        SUM=ZERO                                                        
        DO 140 K=1,NC1                                                  
  140    SUM=SUM+F(I,K)*P(K,J)                                          
  150   COF(I,J)=SUM
C                                                                       
C 11. COMPUTE P*F*P.
C                                                    
      DO 200 I=1,NC1                                                    
       DO 200 J=1,NC1                                                   
        SUM=ZERO                                                        
        DO 190 K=1,NC1                                                  
  190    SUM=SUM+P(I,K)*COF(K,J)                                        
  200   F(I,J)=SUM                                                      
2000  Format(t20,d15.8,t40,d15.8,t60,d15.8)
 
      RETURN                                                            
      END            

c******************************************************************
c      subroutine dgefa(a,lda,n,ipvt,info)                               
c*****************************************************************
c
      subroutine dgefa(a,lda,n,ipvt,info)                               
      implicit double precision(a-h,o-z)
      dimension a(lda,*),ipvt(*)                                        
c                                                                       
c     dgefa factors a double precision matrix by gaussian elimination.  
c                                                                       
c     dgefa is usually called by dgeco, but it can be called            
c     directly with a saving in time if  rcond  is not needed.          
c     (time for dgeco) = (1 + 9/n)*(time for dgefa) .                   
c                                                                       
c     on entry                                                          
c                                                                       
c        a       double precision(lda, n)                               
c                the matrix to be factored.                             
c                                                                       
c        lda     integer                                                
c                the leading dimension of the array  a .                
c                                                                       
c        n       integer                                                
c                the order of the matrix  a .                           
c                                                                       
c     on return                                                         
c                                                                       
c        a       an upper triangular matrix and the multipliers         
c                which were used to obtain it.                          
c                the factorization can be written  a = l*u  where       
c                l  is a product of permutation and unit lower          
c                triangular matrices and  u  is upper triangular.       
c                                                                       
c        ipvt    integer(n)                                             
c                an integer vector of pivot indices.                    
c                                                                       
c        info    integer                                                
c                = 0  normal value.                                     
c                = k  if  u(k,k) .eq. 0.0 .  this is not an error       
c                     condition for this subroutine, but it does        
c                     indicate that dgesl or dgedi will divide by zero  
c                     if called.  use  rcond  in dgeco for a reliable   
c                     indication of singularity.                        
c                                                                       
c     linpack. this version dated 08/14/78 .                            
c     cleve moler, university of new mexico, argonne national lab.      
c                                                                       
c     subroutines and functions                                         
c                                                                       
c     blas daxpy,dscal,idamax                                           
c                                                                       
c     gaussian elimination with partial pivoting                        
c                                                                       
      INFO = 0                                                          
      NM1 = N - 1                                                       
      IF (NM1 .LT. 1) GO TO 70                                          
      DO 60 K = 1, NM1                                                  
         KP1 = K + 1                                                    
C                                                                       
C        FIND L = PIVOT INDEX                                           
C                                                                       
         L = IDAMAX(N-K+1,A(K,K),1) + K - 1                             
         IPVT(K) = L                                                    
C                                                                       
C        ZERO PIVOT IMPLIES THIS COLUMN ALREADY TRIANGULARIZED          
C                                                                       
         IF (A(L,K) .EQ. 0.0D+00) GO TO 40                              
C                                                                       
C           INTERCHANGE IF NECESSARY                                    
C                                                                       
            IF (L .EQ. K) GO TO 10                                      
               T = A(L,K)                                               
               A(L,K) = A(K,K)                                          
               A(K,K) = T                                               
   10       CONTINUE                                                    
C                                                                       
C           COMPUTE MULTIPLIERS                                         
C                                                                       
            T = -1.0D+00/A(K,K)                                         
            CALL DSCAL(N-K,T,A(K+1,K),1)                                
C                                                                       
C           ROW ELIMINATION WITH COLUMN INDEXING                        
C                                                                       
            DO 30 J = KP1, N                                            
               T = A(L,J)                                               
               IF (L .EQ. K) GO TO 20                                   
                  A(L,J) = A(K,J)                                       
                  A(K,J) = T                                            
   20          CONTINUE                                                 
               CALL DAXPY(N-K,T,A(K+1,K),1,A(K+1,J),1)                  
   30       CONTINUE                                                    
         GO TO 50                                                       
   40    CONTINUE                                                       
            INFO = K                                                    
   50    CONTINUE                                                       
   60 CONTINUE                                                          
   70 CONTINUE                                                          
      IPVT(N) = N                                                       
      IF (A(N,N) .EQ. 0.0D+00) INFO = N                                 
      RETURN                                                            
      END           

c***************************************************************
c      subroutine dgedi(a,lda,n,ipvt,det,work,job)               
c***************************************************************
c      
      subroutine dgedi(a,lda,n,ipvt,det,work,job)                       
      implicit double precision(a-h,o-z)
      dimension a(lda,*),det(2),work(*),ipvt(*)                         
c                                                                       
c     dgedi computes the determinant and inverse of a matrix            
c     using the factors computed by dgeco or dgefa.                     
c                                                                       
c     on entry                                                          
c                                                                       
c        a       double precision(lda, n)                               
c                the output from dgeco or dgefa.                        
c                                                                       
c        lda     integer                                                
c                the leading dimension of the array  a .                
c                                                                       
c        n       integer                                                
c                the order of the matrix  a .                           
c                                                                       
c        ipvt    integer(n)                                             
c                the pivot vector from dgeco or dgefa.                  
c                                                                       
c        work    double precision(n)                                    
c                work vector.  contents destroyed.                      
c                                                                       
c        job     integer                                                
c                = 11   both determinant and inverse.                   
c                = 01   inverse only.                                   
c                = 10   determinant only.                               
c                                                                       
c     on return                                                         
c                                                                       
c        a       inverse of original matrix if requested.               
c                otherwise unchanged.                                   
c                                                                       
c        det     double precision(2)                                    
c                determinant of original matrix if requested.           
c                otherwise not referenced.                              
c                determinant = det(1) * 10.0**det(2)                    
c                with  1.0 .le. abs(det(1)) .lt. 10.0                   
c                or  det(1) .eq. 0.0 .                                  
c                                                                       
c     error condition                                                   
c                                                                       
c        a division by zero will occur if the input factor contains     
c        a zero on the diagonal and the inverse is requested.           
c        it will not occur if the subroutines are called correctly      
c        and if dgeco has set rcond .gt. 0.0 or dgefa has set           
c        info .eq. 0 .                                                  
c                                                                       
c     linpack. this version dated 08/14/78 .                            
c     cleve moler, university of new mexico, argonne national lab.      
c                                                                       
c     subroutines and functions                                         
c                                                                       
c     blas daxpy,dscal,dswap                                            
c     fortran abs,mod                                                   
c                                                                       
c     compute determinant                                               
c                                                                       
      IF (JOB/10 .EQ. 0) GO TO 70                                       
         DET(1) = 1.0D+00                                               
         DET(2) = 0.0D+00                                               
         TEN = 10.0D+00                                                 
         DO 50 I = 1, N                                                 
            IF (IPVT(I) .NE. I) DET(1) = -DET(1)                        
            DET(1) = A(I,I)*DET(1)                                      
C        ...EXIT                                                        
            IF (DET(1) .EQ. 0.0D+00) GO TO 60                           
   10       IF (ABS(DET(1)) .GE. 1.0D+00) GO TO 20                      
               DET(1) = TEN*DET(1)                                      
               DET(2) = DET(2) - 1.0D+00                                
            GO TO 10                                                    
   20       CONTINUE                                                    
   30       IF (ABS(DET(1)) .LT. TEN) GO TO 40                          
               DET(1) = DET(1)/TEN                                      
               DET(2) = DET(2) + 1.0D+00                                
            GO TO 30                                                    
   40       CONTINUE                                                    
   50    CONTINUE                                                       
   60    CONTINUE                                                       
   70 CONTINUE                                                          
C                                                                       
C     COMPUTE INVERSE(U)                                                
C                                                                       
      IF (MOD(JOB,10) .EQ. 0) GO TO 150                                 
         DO 100 K = 1, N                                                
            A(K,K) = 1.0D+00/A(K,K)                                     
            T = -A(K,K)                                                 
            CALL DSCAL(K-1,T,A(1,K),1)                                  
            KP1 = K + 1                                                 
            IF (N .LT. KP1) GO TO 90                                    
            DO 80 J = KP1, N                                            
               T = A(K,J)                                               
               A(K,J) = 0.0D+00                                         
               CALL DAXPY(K,T,A(1,K),1,A(1,J),1)                        
   80       CONTINUE                                                    
   90       CONTINUE                                                    
  100    CONTINUE                                                       
C                                                                       
C        FORM INVERSE(U)*INVERSE(L)                                     
C                                                                       
         NM1 = N - 1                                                    
         IF (NM1 .LT. 1) GO TO 140                                      
         DO 130 KB = 1, NM1                                             
            K = N - KB                                                  
            KP1 = K + 1                                                 
            DO 110 I = KP1, N                                           
               WORK(I) = A(I,K)                                         
               A(I,K) = 0.0D+00                                         
  110       CONTINUE                                                    
            DO 120 J = KP1, N                                           
               T = WORK(J)                                              
               CALL DAXPY(N,T,A(1,J),1,A(1,K),1)                        
  120       CONTINUE                                                    
            L = IPVT(K)                                                 
            IF (L .NE. K) CALL DSWAP(N,A(1,K),1,A(1,L),1)               
  130    CONTINUE                                                       
  140    CONTINUE                                                       
  150 CONTINUE                                                          
      RETURN                                                            
      END            

c****************************************************************
c      subroutine daxpy(n,da,dx,incx,dy,incy)
c****************************************************************
c
      subroutine daxpy(n,da,dx,incx,dy,incy)
c
c     constant times a vector plus a vector.
c     uses unrolled loops for increments equal to one.
c     jack dongarra, linpack, 3/11/78.
c
      double precision dx(1),dy(1),da
      integer i,incx,incy,ix,iy,m,mp1,n
c
      if(n.le.0)return
      if (da .eq. 0.0d0) return
      if(incx.eq.1.and.incy.eq.1)go to 20
c
c        code for unequal increments or equal increments
c          not equal to 1
c
      ix = 1
      iy = 1
      if(incx.lt.0)ix = (-n+1)*incx + 1
      if(incy.lt.0)iy = (-n+1)*incy + 1
      do 10 i = 1,n
        dy(iy) = dy(iy) + da*dx(ix)
        ix = ix + incx
        iy = iy + incy
   10 continue
      return
c
c        code for both increments equal to 1
c
c
c        clean-up loop
c
   20 m = mod(n,4)
      if( m .eq. 0 ) go to 40
      do 30 i = 1,m
        dy(i) = dy(i) + da*dx(i)
   30 continue
      if( n .lt. 4 ) return
   40 mp1 = m + 1
      do 50 i = mp1,n,4
        dy(i) = dy(i) + da*dx(i)
        dy(i + 1) = dy(i + 1) + da*dx(i + 1)
        dy(i + 2) = dy(i + 2) + da*dx(i + 2)
        dy(i + 3) = dy(i + 3) + da*dx(i + 3)
   50 continue
      return
      end
       
