c
c*****************************************************************************
c  gmsINP
c*****************************************************************************
c
      subroutine gmsinp(iunit,esmeth,esbas,qmcharge, qmmult,
     *    geom,optl,type,ver_lab)
c
c     Subroutine gmsinp
c     
c     Created:  May 17, 2006 by Yan Zhang
c
c     Purpose:  Write an Gamess input file for the calculation
c        specified.  Type indicates whether it is an energy, gradient, or
c        hessian calculation. 
c
c 
c     Called by: progehk, progghk, proghhk, progohk
c     Calls:	    gmsmeth, scase
c
      use files
      use caltyp                                                        
      use atminfo
      use input, only : cartgeom, maxoptl
      implicit none
c
c i/o variables
c 
      character*20 ver_lab
      character*20 esmeth
      character*20 esbas
      
      type (cartgeom) :: geom
c
      character*80       optl(maxoptl)
      integer            iunit
c                        
c local variables	          
c
      integer         i, j, k,type
      character*12   tswit(4)
c
      character*80   blockopt
      character*2    atmname,lett2
      integer        ifind,iasym,charge

      integer qmcharge, qmmult

      data tswit /'Energy','Gradient','Hessian','Optimization'/
c

      write(outfile,999) tswit(type), esmeth, esbas
 999  format('Running Gamess Job:',3x,a,3x,a10,3x,a10)
c
c  print out method
c
       	call gmsmeth(iunit,esmeth)
c
c   write the coord and unit
c
       	write(iunit,*)'$contrl coord=unique units=angs $end'

c
c print out calculation type
c
      if (type .eq. tener) then
        write(iunit,*)'$contrl  runtyp=energy $end  ' 
      elseif (type .eq. tgrad) then
        write(iunit,*)'$contrl  runtyp=gradient $end  ' 
      elseif (type .eq. thess) then
        write(iunit,*)'$contrl  runtyp=hessian $end  ' 
      elseif (type .eq. topt) then
        write(iunit,*)'$contrl  runtyp=optimize $end  ' 
      endif

c
c  print out basis
c
      esbas='readoptions'
      write(outfile,998)
998   format('The basis of GAMESS is input in the options keyword')	 
c
c  print out input options. The basis are input in this keyword. 
c    

      i = 1
      do while (i.le.maxoptl)

        if(optl(i).ne.'   ') then
            blockopt=optl(i)
        
            j=1
c
            do while(blockopt(j:j).eq.' '.and.j.le.80)
                j=j+1
            enddo

            if(blockopt(j:j).eq.'!') then   
                k=j+1
c
                do while(blockopt(k:k).eq.' '.and.j.le.80)
                    k=k+1
                end do
                if(blockopt(k:k).eq.'$') then
c
                    write(iunit,*)blockopt(k:80)
                else
c
                    write(iunit,1060)blockopt(j+1:80)
                end if
            endif

        endif

        i = i + 1
      enddo
c
c  Print out charge and multiplicity
c
      write(iunit,*)'$contrl icharg=',qmcharge,
     &	' mult=',qmmult,' $end'
c
c  Print out atomic symbols and coordinates
c
      write(iunit,'(a6)')' $data'
      if (type .eq. tener) then
         write(iunit,1021) 
      elseif (type .eq. tgrad) then
         write(iunit,1022)
      elseif (type .eq. thess) then
         write(iunit,1023)
      elseif (type .eq. topt) then
         write(iunit,1024)
      endif

c
c    print out the $data group
c
      write(iunit,'(a3)')' c1'	 
      do 20 i=1, geom%natoms
c
c  find the atomic charge
c		  
         lett2=geom%atm(i)
         call scase(2,lett2,atmname) 
         ifind = 0
         iasym = 1
         charge= 0
         do while(ifind.eq.0.and.iasym.le.103)
           if (atmname.eq.asymb(iasym)) then
              charge=iasym
              ifind = 1  
           endif
           iasym = iasym + 1  
         end do
c
c   print out atomic symbols,charge and coordinates
c
      
         write(iunit,1041) geom%atm(i),charge,
     &       geom%angcoord(3*i-2),
     *      geom%angcoord(3*i-1), geom%angcoord(3*i)
  20  continue
      write(iunit,'(a5)')' $end'

      return

9999  write (outfile,*) 'Error creating Gamess input file'
      stop 'gmsinp'

1021  format ('# Single Point Energy Calculation')
1022  format ('# Gradient Calculation')
1023  format ('# Hessian Calculations')
1024  format ('# External Optimization')
1031  format (a5,1x,i3,1x,i3)
1041  format (a2,5x,i4,3x,3(f16.9,3x))
1051  format (2a,/)
c
1060  format (a79)

      end
c
c*****************************************************************************
c  gmsMETH
c*****************************************************************************
c
      subroutine gmsmeth(iunit,esmeth)
c
c     Subroutine gmmethd
c     
c    Created:  May 18, 2006 by Yan Zhang
c
c     Purpose:  Write an Gamess input method for the calculation
c               specified.  
c 
c     Called by:  gmsinp
c     Calls:	    lenword, scase
c
      use files
      implicit none
c
      character*20 esmeth
      character*20 method
      integer      ifst, ilst,ierr
      integer      iunit

c
c     convert the upper case to low. find the location of the 
C     first and last letter. 
c
      call  scase(20,esmeth,method)

      call lenword(20,method,ifst,ilst,ierr)

c
c write HF method into the input files of program Gamess
c
      if((method.eq.'rhf').or.(method.eq.'uhf').
     &    or.(method.eq.'rohf').or.(method.eq.'gvb').
     &	or.(method.eq.'mcscf')) then 
       write(iunit,*)'$contrl scftyp=',method(ifst:ilst),' $end'
c
c write MP2 method into the input files of program Gamess 
c
      else if(method.eq.'mp2') then
       write(iunit,*)'$contrl mplevl=2 $end'
c
c write CI computation into the input files of program Gamess 
c
      else if((method.eq.'cis').or.(method.eq.'aldet').
     &    or.(method.eq.'ormas').or.(method.eq.'fsoci').
     &	or.(method.eq.'genci').or.(method.eq.'guga')) then
       write(iunit,*)'$contrl cityp=',method(ifst:ilst),' $end'
c
c write a coupled-cluster computation into the input files of program Gamess 
c
      else if((method.eq.'lccd').or.(method.eq.'ccd').
     &    or.(method.eq.'ccsd').or.(method.eq.'ccsd(t)').
     &    or.(method.eq.'cr-cc').or.(method.eq.'r-cc').
     &	  or.(method.eq.'cr-ccl').or.(method.eq.'ccsd(tq)').
     &    or.(method.eq.'eom-ccsd').or.(method.eq.'cr-cc(q)').
     &    or.(method.eq.'cr-eom')) then
       write(iunit,*)'$contrl scftyp=rhf cctyp=',
     &    method(ifst:ilst),' $end'
c
c write relativistic transformations into the input files of program Gamess
c
c	else if((method.eq.'dk').or.(method.eq.'resc').
c     &    or.(method.eq.'nesc')) then
c       write(iunit,*)' $contrl relwfn=',method(ifst:ilst),' $end'
c
c
c write DFT into the input files of program Gamess
c
      else if((method.eq.'slater').or.(method.eq.'becke').
     &    or.(method.eq.'gill').or.(method.eq.'pbe').
     &	  or.(method.eq.'vwn').or.(method.eq.'lyp'). 
     &    or.(method.eq.'op').or.(method.eq.'svwn').
     &	  or.(method.eq.'slyp').or.(method.eq.'sop'). 
     &    or.(method.eq.'glyp').or.(method.eq.'gvwn').
     &	  or.(method.eq.'gop').or.(method.eq.'pbevwn'). 
     &    or.(method.eq.'pbelyp').or.(method.eq.'pbeop').
     &	  or.(method.eq.'bhhlyp').or.(method.eq.'b3lyp'). 
c
     &	  or.(method.eq.'bvwn').or.(method.eq.'blyp'). 
     &    or.(method.eq.'bop').or.(method.eq.'xalpha').
     &	  or.(method.eq.'depristo').or.(method.eq.'cama'). 
     &    or.(method.eq.'half').or.(method.eq.'pwloc').
     &	  or.(method.eq.'bpwloc').or.(method.eq.'gamb'). 
     &	  or.(method.eq.'xvwn').or.(method.eq.'xpwloc')) then
       write(iunit,*)'$dft dfttyp=',method(ifst:ilst),' $end'
c
c In the semi-impirical calculation, the method is input 
C in the group $BASIS.
c
      else if((method.eq.'pm3').or.(method.eq.'am1').
     &       or.(method.eq.'mndo')) then
       write(iunit,*)'$basis gbasis=',method(ifst:ilst),' $end'
c
c
      else
      write(outfile,*)'Error: the QM method',
     &   ' of the calculation with Gamess.'
      stop 'gmsmeth 1'
      end if
       
      return
      end

c
c
c*****************************************************************************
c  gmsOUTE
c*****************************************************************************
c
      subroutine gmsoute (ppnum,esmeth, ener)
c
c     Subroutine gmsoute
c     
c     Created:  May 19, 2006	by Yan Zhang
c     Revised:
c     Purpose:  Get a single energy out of the GAMESS out file. 
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c
c     Called by:  progehk, progghk, proghhk
c     Calls:	wordstr, case
c
c     Variables defined here:
c
      use files
      use numfmt
      implicit none

      character*20      esmeth                                                    
      double precision  ener
      integer           ppnum,outenum                                           

c
      character*80 string, case
      character*30 etxt

      integer      nword,lword(40),ierr
c
      character*80 word(40)
      integer      iln,ical,isuc,i,j
c
c  Open the formatted checkpoint file.
c

      open(unit=gmsoutf,status='unknown', file=gmsout, err=9900)            
      iln=0
      j=0
      etxt=' '

      do while(iln.lt.1)
c
        read(gmsoutf, '(a80)', end=8000 ) string                            
        string=case(string)
c
c   Estimat wether or not it is SCF, DFT or semiempirical calculation. 
C   If it is SCF, DFT or semiempirical calculation and is successful,
C   read the single-point energy from the gms.out file.  
c
        ical=0
        isuc=0

c
        call wordstr(string,80,word,nword,lword,ierr)

        if(((word(2).eq.'scf').and.(word(3).eq.'calculation')).or.
     &   ((word(1).eq.'mcscf').and.(word(2).eq.'calculation'))) then
         ical=1
         etxt=' '
        end if

        do while((ical.eq.1).and.(isuc.eq.0))
c
          read(gmsoutf,'(a80)',end=9910) string
          string=case(string)
c
          call wordstr(string,80,word,nword,lword,ierr)
          if(((word(1).eq.'density').and.(word(2).eq.'converged')).or.
     &      ((word(1).eq.'lagranging').and.(word(2).eq.'converged')))
     &        isuc=1
        end do


        do while((ical.eq.1).and.(isuc.eq.1))
c
          read(gmsoutf,'(a80)')string
          string=case(string)
c
          call wordstr(string,80,word,nword,lword,ierr)
          if(word(1).eq.'final') then
          etxt=word(5)(1:lword(5)+1)
          ical=0
          end if
        end do

c
c   Estimat wether or not it is MP2 calculation. If it is MP2 calculation
C   and is successful,read the single-point energy from the gms.out file.  
c
        ical=0
        isuc=0
        if((word(1).eq.'rhf-mp2').or.(word(1).eq.'uhf-mp2').or.
     &      (word(5).eq.'mp2').or.(word(1).eq.'(mr-mp2)')) then
          ical=1
          etxt=' '
        end if

        do while((ical.eq.1).and.(isuc.eq.0))
c
          read(gmsoutf,'(a80)',end=9910) string
          string=case(string)
c
          call wordstr(string,80,word,nword,lword,ierr)
          if((word(1)(1:6).eq.'e(mp2)').or.
     &       (word(4)(1:6).eq.'e(mp2)')) isuc=1
        end do
        if((isuc.eq.1).and.(word(1)(1:6).eq.'e(mp2)')) then
          etxt=string(27:42)
        else if((isuc.eq.1).and.(word(4)(1:6).eq.'e(mp2)')) then
          etxt=string(57:73)
        end if
c
c   Estimat wether or not it is CC(coupled-cluster) calculation. 
C   If it is CC calculation and is successful,read the single-point 
C   energy from the gms.out file.  
c
        ical=0
        isuc=0
        if(string(6:32).eq.'coupled cluster calculation') then
          ical=1
          etxt=' '
        end if

        do while((ical.eq.1).and.(isuc.eq.0))
c
          read(gmsoutf,'(a80)',end=9910) string
          string=case(string)
c
          call wordstr(string,80,word,nword,lword,ierr)
          if((word(1).eq.'coupled-cluster').and.
     &       (word(2).eq.'energy')) isuc=1
        end do
        if((isuc.eq.1)) then
c
          do i=1,80
             if(string(i:i).eq.'=') then
              j=i+1
             end if
          enddo 
          etxt=string(j:j+25)
        end if

c
c   Estimat wether or not it is CI calculation. 
C   If it is CI calculation and is successful,read the single-point 
C   energy from the gms.out file.  
c
        ical=0
        isuc=0
      if(string(14:51).eq.'atomic orbital basis ci-singles energy')then
          ical=1
          etxt=' '
        end if

        do while((ical.eq.1).and.(isuc.eq.0))
c
          read(gmsoutf,'(a80)',end=9910) string
          string=case(string)
c
          call wordstr(string,80,word,nword,lword,ierr)
          if((word(1).eq.'excited').and.(word(2).eq.'state').and.
     &    (word(4).eq.'energy=')) isuc=1
        end do

        if((isuc.eq.1)) then
          etxt=word(5)(1:lword(5)+1)
        end if

c
c
      end do
8000	iln=1

        if(etxt.eq.' ')goto 9910

      open(50,file='temp1')
      write(50,*)etxt
      close(50)
      open(60,file='temp1')
      read(60,*)ener
      close(60)

c	ener=cfltfmt(etxt,1)

      close( unit=gmsoutf )

      return

9900  write( outfile,*) 'Error opening Gamess out file'
      stop 'gmsoute 1'
9910  write( outfile,*) 'Error reading Gamess out file'
      stop 'gmsoute 2'

      end
c
c*****************************************************************************
c  gmsOUTG
c*****************************************************************************
c  
      subroutine gmsoutg (ppnum, esmeth, grad, natoms,ener)
c											 
c     Subroutine gmsoutg
c
c     Created:  May 21, 2006	 by Yan Zhang
c     Revised:
c     Purpose:  Get the gradient out of the Gamess out file.  
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c     Called by:  progghk, proghhk
c     Calls: wordstr, case
c
c     Variables defined here: 
c     
      use files
      use numfmt
      implicit none
      
      character*20  esmeth
      integer       natoms, ppnum                   

      double precision  grad(3*natoms),ener,char
c
      character*80      string, case
      integer           nword,lword(40),ierr
c
      character*80      word(40)
      integer           i,jj
      character*30      etxt
      character*2       atm
c                                                                     
c  Open the formatted gms.dat file.
c

      open(unit=gmsdatf,status='unknown',file=gmsdat,err=9900)

c
      read( gmsdatf, '(a80)', end=9910 ) string
      string = case(string)

      do while( string(2:6).ne.'$grad') 
c
         read( gmsdatf, '(a80)', end=9910 ) string
         string = case(string)
      enddo
c
c  Get single-point energy and gradient in the $grad group  
c  
c
      read( gmsdatf, '(a80)', end=9910 ) string
c
      call wordstr(string,80,word,nword,lword,ierr)
      etxt=word(2)(1:lword(2)+1)
      open(50,file='temp1')
      write(50,*)etxt
      close(50)
      open(60,file='temp1')
      read(60,*)ener
      close(60)

      do i=1,natoms
        jj=3*(i-1)
        read(gmsdatf,*)atm,char,grad(jj+1),grad(jj+2),
     &     grad(jj+3)
      end do

c
      read(gmsdatf,'(a80)')string
      string=case(string)
      if(string(2:5).ne.'$end') goto 9910  

      close( unit=gmsdatf )
      return

9900  write( outfile,*) 'Error opening Gamess gradient file'
      stop 'gmsoutg 1'
9910  write( outfile,*) 'Error reading Gamess gradient file'
      stop 'gmsoutg 2'

      end

c
c
c*****************************************************************************
c  gmsOUTH
c*****************************************************************************
c  
      subroutine gmsouth (esmeth, hess, natoms)
c
c     Subroutine gmsouth
c
c     Created:  May 21, 2006	by Yan Zhang
c     Revised:
c     Purpose:  Get the hessian out of the Orca hessian file.
c
c     Called by:  proghhk
c     Calls:		case
c
c     Variables defined here: 
c     
      use files
      use numfmt
      implicit none
      
      character*20  esmeth
      integer ctr1, ctr2,  natoms

      double precision  hess(3*natoms,3*natoms)

      integer n, j, m,i, k
c
      character*80 string, case
      integer l
c                                                                     
c  Open the formatted gms.dat file.
c
      open( unit=gmsdatf, status='unknown', file=gmsdat, err=9900)

c
      read( gmsdatf, '(a80)', end=9910 ) string
      string = case(string)

      do while( string(2:6).ne.'$hess' )
c
         read( gmsdatf, '(a80)', end=9910 ) string
         string = case(string)
      enddo

c
c  Get hessian in the $hess group  
c
      read( gmsdatf,*) string

      k=int((3*natoms)/5)
      l=3*natoms
      j=mod(l,5)

      do i=1,3*natoms

        if(k.gt.0) then
          do l=1,k
            m=(l-1)*5
            read(gmsdatf,8000)ctr1,ctr2,hess(i,m+1),hess(i,m+2),
     &           hess(i,m+3),hess(i,m+4),hess(i,m+5)
          end do
        end if
        if(j.gt.0) then
            read(gmsdatf,8010)ctr1,ctr2,(hess(i,k*5+n),n=1,j)
        end if

        if(ctr1.ne.i) goto 9910
      end do

c
      read(gmsdatf,'(a80)')string
      string=case(string)
      if(string(2:5).ne.'$end') goto 9910  

      close( unit=gmsdatf )

8000  format(i2,i3,5e15.8)
8010  format(i2,i3,5e15.8)
      return

9900  write( outfile,*) 'Error opening Gamess hessian file gms.dat'
      stop 'gmsouth 1'
9910  write( outfile,*) 'Error reading Gamess hessian file gms.dat'
      stop 'gmsouth 2'

      end

c
c******************************************************************************
c  gmsOUTO
c******************************************************************************
c
      subroutine gmsouto (geom, ener,grad,natoms,ierr)
c
c     Subroutine gmsouto
c
c     Created:  May 21, 2006	by Yan Zhang
c
c     Purpose:  Get optimized geometry out of the Gamess output file.   
c
c
c     Called by:  progohk
c     Calls:	wordstr, case
c
      use files
      use numfmt
      use input, only : cartgeom

      implicit none
c
c i/o variables
c
      type (cartgeom)  :: geom
      integer             ierr,opt            
      character*20        etxt
c
c local variables
c
      character*80 string, case
      character*3 name
      integer natoms,i,jj

c
      character*80 word(40)
      integer      lword(40),nword

      double precision, parameter :: angtoau = 0.529177249d0
      double precision  charge,ener,grad(3*natoms)
      character*2 atm
c
c  Check that the optimization is complete.
c
      opt  = 0
      ierr=0

      open(unit=extoptdatf, status='unknown',file=extoptdat, err=9920 )

c
      read( extoptdatf,'(a80)', end=9930 ) string
      string = case(string)
c
      call wordstr(string,80,word,nword,lword,ierr)

      do while( (word(2) .ne. 'results').and.
     &        (word(4) .ne. 'successful').and.
     &         (word(6) .ne. 'geometry').and.
     $          (word(7) .ne. 'search'))
c
         read( extoptdatf, '(a80)', end=9930 ) string	 
         string = case(string)
c
         call wordstr(string,80,word,nword,lword,ierr)
      end do
c     
c  get the optimized geometry in the file extopt.dat.
c
      do i=1,4
c
        read(extoptdatf,'(a80)') string
      end do

      do i=1,natoms
        read(extoptdatf,*) atm,charge,geom%angcoord(3*i-2),
     &           geom%angcoord(3*i-1),geom%angcoord(3*i)
      end do 

      geom%aucoord = geom%angcoord/angtoau
c
c  Get single-point energy and gradient in the $grad group  
c    energy
c

c
      read( extoptdatf, '(a80)', end=9920 ) string
      string = case(string)

      do while( string(2:6).ne.'$grad') 
c
         read( extoptdatf, '(a80)', end=9920 ) string
         string = case(string)
      enddo

c
      read( extoptdatf, '(a80)', end=9920 ) string
c
      call wordstr(string,80,word,nword,lword,ierr)
      etxt=word(2)(1:lword(2)+1)
      open(50,file='temp1')
      write(50,*)etxt
      close(50)
      open(60,file='temp1')
      read(60,*)ener
      close(60)
c
c   gradient
c
      do i=1,natoms
        jj=3*(i-1)
        read(extoptdatf,*)atm,charge,grad(jj+1),grad(jj+2),
     &     grad(jj+3)
      end do

c
      read(extoptdatf,'(a80)')string
      string=case(string)
      if(string(2:5).ne.'$end') goto 9930  

      close( unit=extoptdatf )

      return

9920  write( outfile,*) 'Error opening Gamess optimized output 
     $                    file-extopt.dat.'
      stop 'gmsouto 1'
9930  ierr=1
      return

      end

c
c****************************************************************************
c  scas
c****************************************************************************
c
      subroutine scase(n,string,string1)
c
c     subroutine scase
c
c     Created:  May 18, 2006 by Yan Zhang
c          
c     Revised:
c
c     Description:  this function converts a string (lenth=n) to lower case.
c
c     Called by: gmsmeth,gmsinp
c     Calls: lcasel
c
c     Variables defined here: 
c
c
      implicit none
c
      integer  n,i
      character(len=n) string,string1
      character*1 lcasel,lett
c
      do i=1,n
      lett=lcasel(string(i:i))
      string1(i:i)=lett
      end do
c
c
      return
      end
c
c


c
c****************************************************************************
c  lcasl
c****************************************************************************
c
      function lcasel(lett)
c
c     Function lcasl
c
c     Created:  May 9, 2006 by Yan Zhang
c          
c     Revised:
c
c     Description:  this function converts a upper case letter to lower case.
c
c     Called by:
c     Calls:
c
c     Variables defined here: 
c
c
      implicit none
c
      integer  itry
      character*1 lett, lcasel
c
        itry = ichar (lett)
        if (lett .ge. 'A' .and. lett .le. 'Z') then
          itry = itry + 32
          lett = char (itry)
        endif
c
      lcasel=lett
c
      return
      end
c
c

c
c
c*****************************************************************************
c LENWORD
c*******************************************************************

      subroutine lenword(lenth,string,ifst,ilst,ierr)

c
c     subroutine lenword
c
c     Created: May 18, 2006 by Yan Zhang 
c
c     This subroutine finds the location of the first and last letter 
C     for the fisrt word in a  string. 
c     
c    	call by: gmsmeth 
c
c	call:
c

      implicit none
c
      integer ifst,ilst,ierr,lenth
      character(len=lenth)  string
      integer  ip      
c
      ierr=0
      ifst=0
      ilst=0
      ip=0
c
c     find the first nonblank character in a string
c
20    ip = ip + 1
      if (string(ip:ip) .eq. ' ' .and. ip .lt. lenth) go to 20           


C
C     Find the last character in the first word
C
      if(ip.lt.lenth) then
        ifst = ip 

30      ip = ip + 1
        if (string(ip:ip) .ne. ' ' .and. ip .lt. lenth) goto 30 
      else
        ierr=1
      end if

      ilst=ip

      return
c
      end

