c**********************************************************************
c  gau_ext_opt
c**********************************************************************
c
      subroutine gau_ext_opt(optinp,geom,mmgeom,ierr)
c
c     Subroutine gau_ext_opt
c
c     Created : Aug 21, 2006 by Yan Zhang  
c     Revised : Oct 31, 2007 by Yan Zhang              
c
c     Purpose : If ipartcal=0, call subroutine gau_ext_opt1(OPT all atoms).
c               If ipartcal=1, call subroutine gau_ext_opt2(OPT partial atoms).
c
c*     Called by:  mlohook
c*     Calls:  gau_ext_opt1, gau_ext_opt2 
c*
      use files
      use input, only : mlopt, mlinp,mmvalen,cartgeom
      use optimize, only : ener, grad, hess
      implicit  none

c
c i/o variables
c
      integer            ierr
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (mlopt)    :: optinp                  
c
c local variables
c      
      integer            i, j, k, n3tm, iopt
      character*20       basis, method,v3s  
c
c    
      iopt=optinp%part%ipartcal

      if(iopt.eq.0) then
       call gau_ext_opt1(optinp,geom,ierr)
   
      elseif(iopt.eq.1) then      
       call gau_ext_opt2(optinp,geom,mmgeom,ierr)

      end if 
      end     
c
c**********************************************************************
c  gau_ext_opt2
c**********************************************************************
c
      subroutine gau_ext_opt2(optinp, geom, mmgeom, ierr)
c
c     Subroutine gau_ext_opt2
c
c     Created:  Aug. 21, 2006 by Yan Zhang
c     Revised:  Oct. 31, 2007 by Yan Zhang
c
c     Purpose:  To perform  partial atomic optimization using Gaussian's optimizer 
c     
c     Called by: gau_ext_opt  
c     Calls: partafatm, g03partinp, g03partouto  
c
      use files
      use input, only : mlopt, mlinp,mmvalen,cartgeom
      use optimize, only : ener
      implicit  none

c
c i/o variables
c
      integer            ierr
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom                                         
      type (mlopt)    :: optinp                  
c
c local variables
c      
      integer            i, j, k, n3tm
      character*20       basis, method,v3s  
c
c
      n3tm = 3*mlinp%geom%natoms
      basis = ' '     
      method = 'external' 
      v3s = 'v3s'
c         
c                  
      call partaftm(optinp%part, geom, mmgeom) 

      open (unit=extoinf, status='unknown', file=extin, err=9991)
      call g03partinp(extoinf, optinp%part, geom, optinp%optl)
      call system('g03shuttle extopt.inp extopt.out')
      call g03partouto(optinp, geom, ener, ierr) 
      if (ierr.eq.1) go to 7000 

      mlinp%geom = geom
c
c  Display the results of the optimization
c
      write(outfile,1100)
      write(outfile,1250)  ener
      write(outfile,1400)
      write(outfile,1450)

      do 20 i=1, geom%natoms
         write(outfile,1500) geom%atm(i), geom%aucoord(3*i-2),
     *       geom%aucoord(3*i-1), geom%aucoord(3*i)
  20  continue
      write(outfile,1550)
      do 40 i=1, geom%natoms
         write(outfile,1500) geom%atm(i), geom%angcoord(3*i-2),
     *       geom%angcoord(3*i-1), geom%angcoord(3*i)
  40  continue

1100  format(/2(/80('-')),//2x,'External Optimization with Gaussian',
     *         a,//80('-')/)
1250  format(/2x,'Optimized Energy:',t30,f19.12)
1400  format(/2x,'Optimized Geometry:')
1450  format(//5x,'Atomic Coordinates (in a.u.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)
1500  format(9x,a2,t20,d15.8,t40,d15.8,t60,d15.8)
1550  format(//5x,'Atomic Coordinates (in ang.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)  
      return

9991  write (outfile,*) 'Error creating Gaussian98 Opt input file'
      stop 'gau_ext_opt2 1'
7000  write (outfile,'(/a)') 'ERROR: External Optimization failed.'
      stop 'gau_ext_opt2 2'
      end     
c
c*****************************************************************************
c  g03partINP
c*****************************************************************************
c
      subroutine g03partinp(iunit, part, geom, optl)
c
c     Subroutine g03partinp
c     
c     Created:  Aug 21, 2006 by Yan Zhang
c
c     Purpose:  Write a Gaussian03 partial atomic input file.
c              If the options line begins with '!' then to be written after
c              the molecule specification.
c 
c     Called by: gau_ext_opt2
c     Calls:                   
c
      use files
      use caltyp                                                        
      use input, only : cartgeom, maxoptl, mlopt, partial
      implicit none
c
c i/o variables
c 
      character*20 ver_lab
      
      type (cartgeom) :: geom
      type (partial)  :: part
c
      character*80       optl(maxoptl)
      integer            iunit
c                        
c local variables	          
c
      character*20   esmeth, esbas                                        
      integer        mfin, bfin, mend, bend, i, j, type
      character*12   tswit(4)
      logical        linkzero
           
      integer        k, jj, pidatm    
      integer        partcharge, partmult

c
      mfin = 1
      bfin = 1
      mend = 0
      bend = 0

      esmeth     = 'external '
      esbas      = ' '
      partcharge = part%pcharge
      partmult   = part%pmult

c     memory and processors are now specified in the options      
c     linkzero is a logical (boolean) variable to determine if the current
c     line being read is a link0 command or a keyword option

      linkzero = .false.
      i = 1                                                             
      do 10 j=1,10                                                      
      if (optl(i)(j:j) .eq. '%' ) then                                  
      linkzero = .true.                                                 
      endif                                                             
10    CONTINUE                                                          

      do while (linkzero .and. i.le.maxoptl)                            
         write(iunit,*) optl(i)                                         
         i = i+1                                                        
         linkzero = .false.                                             
          do 12 j=1,10                                                  
            if (optl(i)(j:j) .eq. '%' ) then                            
               linkzero = .true.                                        
            endif                                                       
12    CONTINUE                                                          
      end do                                                            
c
c added by Xin-Ping Wu, 10/20/2017
c
         write(iunit,*) '%chk=Test.chk'
c
c end by Xin-Ping Wu, 10/20/2017
c
         write(iunit,1011) '#','external','/',esbas(:bfin),
c     *           'FChk=All', 'NoSymm'
     *           'NoSymm'                                               1020XW17
c         write(iunit,1071) 
1071  format('#',2x,'OPT=(newton)') 

c
c skip the line if begin with '!'                                       
c
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
c
         do j=1, 79
            if (optl(i)(j:j) .eq. '!') then 
                k = 1
            endif
      enddo 
      if (k .eq. 0) then    
            write(iunit,1016) '#', optl(i)
      endif    
         i = i+1
      end do
c
c
         write(iunit,1021) 
c
c  Print out charge, multiplicity, and atomic symbols and coordinates
c
      write(iunit,1031) partcharge, partmult
c
c  Moving atom            
c
      do i=1,part%partatmnum
         pidatm = part%partatmid(i)

         write(iunit,1041) geom%atm(pidatm), geom%angcoord(3*pidatm-2),
     *      geom%angcoord(3*pidatm-1), geom%angcoord(3*pidatm)
      end do
c
c  layer 1 atom           
c
      do i=1,part%extlayer1atmnum
        pidatm = part%layer1atmid(i)
        write(iunit,1041) geom%atm(pidatm), geom%angcoord(3*pidatm-2),
     *      geom%angcoord(3*pidatm-1), geom%angcoord(3*pidatm)
      end do
c
c  layer 2 atom           
c
      do i=1,part%extlayer2atmnum
        pidatm = part%layer2atmid(i)
        write(iunit,1041) geom%atm(pidatm), geom%angcoord(3*pidatm-2),
     *      geom%angcoord(3*pidatm-1), geom%angcoord(3*pidatm)
      end do
c
c  Frozen layer 1 atom
c
      pidatm = 0
      if(part%extlayernum.gt.0) then
        write(iunit,*)
        do i=1,part%extlayer1atmnum
          pidatm = part%partatmnum + i  
         write(iunit,1061) pidatm 
        end do
      endif
c
c  Frozen layer 2 atom
c
      pidatm = 0
      do i=1,part%extlayer2atmnum
        pidatm = part%partatmnum + part%extlayer1atmnum + i  
        write(iunit,1061) pidatm 
      end do

c
c write additional options for molecule specification                   
c
      i = 1
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
         jj = 1
c
         do j=1, 79
           if (optl(i)(j:j) .eq. '!') then
             k = 1
             jj = j + 1
          endif   
         enddo    
         if (k .eq. 1) then
c
            write(iunit,1060) optl(i)(jj:80)
         endif
         i = i + 1
      enddo

      write(iunit,*)
      write(iunit,*)
c
c  Close the file
c
      close (iunit, err=9999)
      
1001  format (/)
1010  format (2a)
c1011  format (4a,3(3x,a))
1011  format (4a,2(3x,a))                                               1020XW17
1012  format (4a,2(3x,a))
1016  format (a1,a75)
1021  format (/'External Partial Optimization'/)
1031  format (1x,i3,1x,i3)
1041  format (a2,5x,3(d16.9,3x))
1051  format (2a,/)
c
1060  format (a79)

c
c modified by Xin-Ping Wu, Sept. 21, 2017
c corrected for the right syntax in g09
c
1061  format (8x,'X',1x,i6,4x,'f')
c1061  format (10x,i6,4x,'f')
c
c end by Xin-Ping Wu, Sept. 21, 2017
c

      return

9999  write (outfile,*) 'Error creating Gaussian input file'
      stop 'g03inp'

      end
c
c******************************************************************************
c  g03PARTOUTO
c******************************************************************************
c
      subroutine g03partouto (optinp, geom, ener, ierr)
c
c     Subroutine g03partouto
c
c     Created:  Aug. 21, 2006  by Yan Zhang
c     Modified: Oct. 2, 2017  by Xin-Ping Wu. The subroutine is reorganized
c                to be compatible with G03, G09, and G16.
c
c     Purpose:  Get the partial atomic energy and optimized geometry out of
c               the Gaussian formatted checkpoint file.   
c
c     Called by:  gau_ext_opt2                   
c     Calls:
c
      use files
      use numfmt
      use input, only : cartgeom, mlopt
      implicit none
c
c i/o variables
c
      type (cartgeom)  :: geom
      type (mlopt)     :: optinp
      double precision    ener
      integer             ierr            
c
c local variables
c
      double precision   cfltfmt

      integer      i, ctr, j, gstrt, gend, opt, pidatm
      integer   layer1atmnum, layer2atmnum, extatmnum

c
      character*80 string, case
      character*30 label
      character*22 etxt    
      character*15 gtxt(3*geom%natoms), otxt(3*geom%natoms)
      double precision patmc(3*geom%natoms) 

      double precision, parameter :: angtoau = 0.529177249d0
      integer   npartatm, nmovfro               
      logical finde, findc                                              1002XW17
c
c
      npartatm     = optinp%part%partatmnum  
      layer1atmnum = optinp%part%extlayer1atmnum
      layer2atmnum = optinp%part%extlayer2atmnum  
      nmovfro      = npartatm + layer1atmnum + layer2atmnum
c
c  Check that the optimization is complete.
c
      opt  = 0

      open( unit=extoout, status='unknown', file=extout, err=9920 )

c
      read( extoout, '(a80)', end=9930 ) string
      string = case(string)
c
c change the search string to include
c   "Optimization completed on the basis of negligible forces"
c
      do while( string(:23) .ne. ' optimization completed')
c
         read( extoout, '(a80)', end=9930 ) string	 
         string = case(string)
      end do
           
      close ( unit=extoout )
c
      write(outfile,*) string(1:80)

      finde = .false.                                                   1002XW17
      findc = .false.                                                   1002XW17

c     
c  Open the formatted checkpoint file.
c
      open( unit=fchk, status='unknown', file=testfchk, err=9900)
c
c modified by Xin-Ping Wu, Oct. 2, 2017
c
c  Find energy
c
100   read( fchk, '(a80)', end=9910 ) string                            
      string = case(string)
      label = string(:llen(efmt))
c      etxt = string(nstrt(efmt):nstrt(efmt)+nlen(efmt)-1)              
      
      if ( label(:12).ne.'total energy' ) then
          goto 110
      else
        etxt = string(nstrt(efmt):nstrt(efmt)+nlen(efmt)-1)
        ener = cfltfmt(etxt,efmt)
        finde = .true.
        goto 140
      endif
110   continue

c      do while( label(:12).ne.'total energy' )
c
c         read( fchk, '(a80)', end=9910 ) string
c         string = case(string)
c         label = string(:llen(efmt))
c         etxt = string(nstrt(efmt):nstrt(efmt)+nlen(efmt)-1)
c      end do  
c 
c      ener = cfltfmt(etxt,efmt)
            
c
c  Find geometry
c
      if ( string(:29) .ne. 'current cartesian coordinates') then
         goto 120
      else
        ctr = 1
        do while( ctr .le. 3*nmovfro )
c
           read( fchk, '(a80)', end=9910 ) string
           string = case(string)
           i = 0
           do while( ctr.le.3*nmovfro .and. i.lt.5 )
              gstrt = i*(nlen(gfmt)+1)+2
              gend  = (i+1)*(nlen(gfmt)+2)
              otxt(ctr) = string(gstrt:gend)
              ctr = ctr+1
              i = i+1
           end do
        end do

        do 10 j=1, 3*nmovfro
           patmc(j) = cfltfmt(otxt(j),gfmt)
  10    continue

        do i=1, npartatm         
           pidatm = optinp%part%partatmid(i)
           geom%aucoord(3*pidatm-2) = patmc(3*i-2)
           geom%aucoord(3*pidatm-1) = patmc(3*i-1)
           geom%aucoord(3*pidatm  ) = patmc(3*i)
        end do

        geom%angcoord = geom%aucoord*angtoau
        findc = .true.
        goto 140
      endif
120   continue

c      do while( string(:29) .ne. 'current cartesian coordinates')
c
c         read( fchk, '(a80)', end=9910 ) string
c         string = case(string)
c      enddo
c
c      ctr = 1
c      do while( ctr .le. 3*nmovfro )
c
c         read( fchk, '(a80)', end=9910 ) string
c         string = case(string)
c         i = 0
c         do while( ctr.le.3*nmovfro .and. i.lt.5 )
c            gstrt = i*(nlen(gfmt)+1)+2  
c            gend  = (i+1)*(nlen(gfmt)+2)  
c            otxt(ctr) = string(gstrt:gend)
c            ctr = ctr+1
c            i = i+1
c         end do
c      end do
c      
c      do 10 j=1, 3*nmovfro 
c         patmc(j) = cfltfmt(otxt(j),gfmt)
c  10  continue
c
c      do i=1, npartatm         
c         pidatm = optinp%part%partatmid(i)
c         geom%aucoord(3*pidatm-2) = patmc(3*i-2)
c         geom%aucoord(3*pidatm-1) = patmc(3*i-1)
c         geom%aucoord(3*pidatm  ) = patmc(3*i)
c      end do
c
c      geom%angcoord = geom%aucoord*angtoau
c
140   if (finde .and. findc ) then
        goto 150
      else
        goto 100
      endif

150   close( unit=fchk )

c      close( unit=fchk )
c
c end by Xin-Ping Wu, Oct. 2, 2017
c
 
      return
      
c9900  write( outfile,*) 'Error opening Gaussian03 checkpoint file.' 
9900  write( outfile,*) 'Error opening Gaussian checkpoint file.'       1002XW17
      stop 'g03outo 1'
c9910  write( outfile,*) 'Error reading Gaussian03 checkpoint file.'
9910  write( outfile,*) 'Error reading Gaussian checkpoint file.'       1002XW17
      stop 'g03outo 2'
c9920  write( outfile,*) 'Error opening Gaussian03 output file.'
9920  write( outfile,*) 'Error opening Gaussian output file.'           1002XW17
      stop 'g03outo 3'

9930  ierr=1

      end
c
c
c*******************************************************************************
c  PARTAFATM
c*******************************************************************************
c
      subroutine partaftm(part, geom, mmgeom) 
c
c     Subroutine partafatm
c
c     Created:  Oct 31, 2007 by Yan Zhang
c
c     Purpose:  This subroutine determines partial atoms including active atoms and 
c               frozen atoms.  
c
c     Called by: gau_ext_opt2                
c     Calls: partoptatm, partextlayer, partatm, dispartoptinf                          
c
      use input
      use files
      implicit none
c
c  i/o variables
c
      type (partial)  :: part
      type (mmvalen)  :: mmgeom
      type (cartgeom) :: geom
c
c  local variables
c
      integer   npartatm, npartlayer
      integer   i, j, k, ii, jj, kk, id1, id2, id3, id4
      integer   layer1atmnum, layer2atmnum, extatmnum
      integer   atmid(maxatm), nmovfro
      integer   init, ipt
c
c
      ipt   = part%ipartyp              
      init  = part%initgpart
c
c  determine the active atoms in the optimization if ipt=2.          
c  if ipt=1, the active atoms were read from input file.
c
      if(ipt.eq.2) then
        call partoptatm(part, geom, mmgeom)
      end if
c
c  determine the layer atoms.               
c
      call partextlayer(part, mmgeom)
c
c output partial atoms ID including active atoms and layer atoms 
c in the partatmid file.    
c
      call partatm(part)
c
c print out information of partial optimization            
c
      if(init.eq.1) then
        call dispartoptinf(part, geom, mmgeom)
      endif

      return 
      end 
c
c
c*******************************************************************************
c  PARTEXTLAYER 
c*******************************************************************************
c
      subroutine partextlayer(part, mmgeom)
c
c     Subroutine partextlayer                      
c
c     Created:  Oct 26, 2007 by Yan Zhang 
c
c     Purpose:  This subroutine determine the extented layer atom in the partial
c               optimization. The max value of exteted layer is 2.  
c
c
c     Called by: partaftm 
c     Calls: 
c
      use input                               
      use files                         
      implicit none
c
c  i/o variables
c
      type (mmvalen)  :: mmgeom
      type (partial)  :: part                  
c
c  local variables
c                  
      integer   npartatm, npartlayer                                     
      integer   i, j, k, ii, jj, kk, id1, id2, id3, id4        
      integer   layer1atmnum, layer2atmnum, extatmnum    
      integer   nvalen
c      
c  Set the moving atom number. 
c
      npartatm    =  part%partatmnum
      npartlayer  =  part%extlayernum                       
c
c  set the first extented layer around moving atoms.                  
c
      layer1atmnum=  0
      if(npartlayer.gt.0) then
        do i=1, npartatm                                                
          id1    = part%partatmid(i)   
          nvalen = mmgeom%atmvalen(id1)

          do j=1, nvalen
            id2 = mmgeom%atmvalenlist(id1,j)  

            ii=0
            do k=1,npartatm
              id3 = part%partatmid(k)   
              if(id3.eq.id2) ii=1
            enddo

            do k=1, layer1atmnum
              id3 = part%layer1atmid(k)   
              if(id3.eq.id2) ii=1
            enddo

            if(ii.eq.0) then
              layer1atmnum = layer1atmnum + 1
              part%layer1atmid(layer1atmnum) = id2
c          write(outfile,*)'layer1',layer1atmnum,id2
            end if

          enddo

        end do
      endif                    
      part%extlayer1atmnum = layer1atmnum    
c      write(outfile,*)'layer1atmnum',part%extlayer1atmnum
c
c set the second extent layer atoms in the partial optimization. 
c   
      layer2atmnum=  0
      if(npartlayer.gt.1) then
        do i=1, layer1atmnum 
          id1    = part%layer1atmid(i)
          nvalen = mmgeom%atmvalen(id1)

          do j=1, nvalen
            id2 = mmgeom%atmvalenlist(id1,j)
              
            ii=0
            do k=1,npartatm
              id3 = part%partatmid(k)
              if(id3.eq.id2) ii=1
            enddo

            do k=1, layer1atmnum
              id3 = part%layer1atmid(k)
              if(id3.eq.id2) ii=1
            enddo

            do k=1, layer2atmnum
              id3 = part%layer2atmid(k)
              if(id3.eq.id2) ii=1
            enddo

            if(ii.eq.0) then
              layer2atmnum = layer2atmnum + 1
              part%layer2atmid(layer2atmnum) = id2
c          write(outfile,*)'layer2',layer2atmnum,id2
            end if

          enddo

        end do
      endif
      part%extlayer2atmnum = layer2atmnum
c      write(outfile,*)'layer2atmnum',part%extlayer2atmnum
      return
      end
c
c*******************************************************************************
c PARTOPTATM 
c*******************************************************************************
c
      subroutine partoptatm(part, geom, mmgeom)
c
c     subroutine partoptatm
c
c     Created:  Oct. 31, 2007 by Yan Zhang
c     Revised:
c     Purpose:  determine the partial optimized active atoms base on the 
c               selected center. The center XYZ coordinates are given by 
c               two ways. One is that user input the center atom ID, the other is 
c               that user input the center XYZ coordinates in angstrom.                              
c                 
c
c
c     Called by: partaftm 
c     Calls:  
c
      use input
      use files
      implicit none
c
c  i/o variables
c
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (partial)  :: part 
      type (general)  :: mlgeninp
c
c  local variables
c                             
      integer i, iatm,icutoff,natm
      integer idcent,icent,iunit,lhess
      integer id, npartatm
      double precision r0, rx 
      double precision x0, y0, z0, x, y, z
      double precision angtoau
c
c  parameter
c
      angtoau = 0.529177249d0
c
c  prepare variables
c
      natm    = geom%natoms
      idcent  = part%partcentid
      r0      = part%partrad            
      icent   = part%ipartcent               
c
c   Determind the center for the partial optimization.
c

      if(icent.eq.1) then 
         x0=geom%angcoord(3*idcent-2)
         y0=geom%angcoord(3*idcent-1)
         z0=geom%angcoord(3*idcent)
      elseif(icent.eq.2) then
         x0=part%partcentx                           
         y0=part%partcenty                
         z0=part%partcentz                                  
      end if
c
c   Determind the partial optimized atoms.                
c
      npartatm = 0
      do i=1,natm
        iatm=mmgeom%atmid(i)
        x=geom%angcoord(3*iatm-2)
        y=geom%angcoord(3*iatm-1)
        z=geom%angcoord(3*iatm)
        rx=(x-x0)**2+(y-y0)**2+(z-z0)**2
        rx=dsqrt(rx)
        if(rx.lt.r0) then
          npartatm = npartatm + 1
          id = mmgeom%atmid(i)
          part%partatmid(npartatm) = id              
        endif 
      end do

      part%partatmnum = npartatm

        return
        end
c
c*******************************************************************************
c  PARTATM 
c*******************************************************************************
c
      subroutine partatm(part)
c
c     Subroutine partatm
c
c     Created:  Oct 29, 2007 by Yan Zhang
c
c     Purpose:  This subroutine writes a file of partial atomic ID
c               including moving atoms and external layers atoms.     
c
c     Called by: partaftm           
c     Calls:
c
      use input
      use files
      implicit none
c
c  i/o variables
c
      type (partial)  :: part
c
c  local variables
c
      integer   npartatm, npartlayer
      integer   i, j, k, ii, jj, kk, id1, id2, id3, id4
      integer   layer1atmnum, layer2atmnum, extatmnum
      integer   atmid(maxatm), nmovfro               
c
c
      npartatm     = part%partatmnum  
      layer1atmnum = part%extlayer1atmnum
      layer2atmnum = part%extlayer2atmnum  
      nmovfro      = npartatm + layer1atmnum + layer2atmnum
c
c Set partial atoms ID     
c
      do i=1,part%partatmnum
         atmid(i) = part%partatmid(i)
      end do
      do i=1, part%extlayer1atmnum
        j = part%partatmnum + i
        atmid(j) =  part%layer1atmid(i)
      end do
      do i=1, part%extlayer2atmnum
        j = part%partatmnum + part%extlayer1atmnum + i
        atmid(j) =  part%layer2atmid(i)
      end do
c
c  Write partial atomic ID file                  
c
      open(unit=paidf,status='unknown',file=paid, err=9920 )
      write(paidf,101)atmid(:nmovfro)
      close(paidf)
101   format(5(i6,3x))
      return
9920  write(outfile,*)'Error: open PARTATMID file.'   
      stop 'partatm 1'
      end
c
c*******************************************************************************
c DISPARTOPTINF 
c*******************************************************************************
c
      subroutine dispartoptinf(part, geom, mmgeom)
c
c     Subroutine dispartoptinf 
c
c     Created:  Oct. 31, 2007 by Yan Zhang
c     Revised:
c
c     Purpose:  display the information in the partial optimization 
c
c     Called by: partaftm                     
c     Calls:
c
      use input
      use files
      implicit none
c
c  i/o variables
c
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (partial)  :: part
c
c  local variables             
c
      integer   i, j, k, ii, jj, kk
      integer   npartatm, npartlayer
      integer   layer1atmnum, layer2atmnum, extatmnum
      integer   init, nmovfro
      integer   pidatm
c
c
      npartatm     = part%partatmnum  
      layer1atmnum = part%extlayer1atmnum
      layer2atmnum = part%extlayer2atmnum  
      npartlayer   = part%extlayernum                       
      init         = part%initgpart
      nmovfro      = npartatm + layer1atmnum + layer2atmnum
c
c
      write(outfile,100)
      write(outfile,200)  npartatm
      write(outfile,300)
      write(outfile,400) part%partatmid(:npartatm)
      write(outfile,500) layer1atmnum
      write(outfile,550) 
      write(outfile,400) part%layer1atmid(:layer1atmnum)                      
      write(outfile,600) layer2atmnum
      write(outfile,650)           
      write(outfile,400) part%layer2atmid(:layer2atmnum)
      write(outfile,*)

      write(outfile,700) nmovfro
      write(outfile,800) part%pcharge,part%pmult
      write(outfile,900)
c
c  Moving atom            
c
      do i=1,npartatm         
        pidatm = part%partatmid(i)
        j = i
      write(outfile,1000)j, pidatm, geom%atm(pidatm),                  
     * geom%angcoord(3*pidatm-2), geom%angcoord(3*pidatm-1),
     * geom%angcoord(3*pidatm)
        end do
c
c  layer 1 atom           
c
      do i=1,part%extlayer1atmnum
        pidatm = part%layer1atmid(i)
        j = npartatm + i
      write(outfile,1000)j, pidatm, geom%atm(pidatm),               
     *  geom%angcoord(3*pidatm-2), geom%angcoord(3*pidatm-1),
     *  geom%angcoord(3*pidatm)
      end do
c
c  layer 2 atom           
c
      do i=1,part%extlayer2atmnum
        pidatm = part%layer2atmid(i)
        j = npartatm + part%extlayer1atmnum + i
       write(outfile,1000)j, pidatm, geom%atm(pidatm),            
     *  geom%angcoord(3*pidatm-2), geom%angcoord(3*pidatm-1),
     *  geom%angcoord(3*pidatm)
      end do

      stop  

100   format('Partial Gauext optimization')
200   format('Active atomic numbers:',2x,i7)
300   format('Active atomic numbers ID:')
400   format(5(i6,3x))
500   format('Extend forzen layer 1 numbers:',2x,i7)
550   format('Extend forzen layer 1 ID:')             
600   format('Extend forzen layer 2 numbers:',2x,i7)
650   format('Extend forzen layer 2 ID:')
700   format('Total partial atomic numbers:',2x,i7)
800   format('Charge:',2x,i3,2x,'Multiplicity:',2x,i3)                 
900   format(/'Atomic Coordinates (in ang.):',//'Part ID',
     * t10,'Orig ID',t20,'Atom', t35, 'X',
     *         t52, 'Y', t69, 'Z'/)
1000  format(i6,t10,i6,t21,a2,t27,f15.8,t44,f15.8,t61,f15.8)

      return
      end

c
c********************************************************************************
c********************************************************************************
c
