c
c******************************************************************************
c  MLGHOOK
c******************************************************************************
c
      subroutine mlghook(progrms, geom, mcoptl,
     *   ltest, ltestmm, mmgeom, 
     *   lqmmm, qmmminp,
     *   tmeth, tbas, toptl, ierr)
c  
c     Subroutine mlghook
c
c     Created:  Feb 22, 2004 by Hai Lin
c               Jun. 28, 2010 by Soroosh Pezeshki
c
c     Purpose:  To call the appropriate gradient subhook
c
c     Called by:  main
c     Calls:  imohcghk, mg3ghk, progmmghkmd, qmmmghkmd
c             mqcghk, mcomp2ghk, mccoghk, progghk, progmmghk   
c          
c
      use input,    only : cartgeom, proglist, mmvalen,        
     *                     qmmminf, maxoptl                    
      use files,    only : outfile
      use optimize, only : ener, grad
c      use caltyp,   only : tgrad
      use caltyp,   only : tgrad,tdynamics                              0628SP10
      use gradient, only : qmmmgrad,totalgrad                           1010SP10
      use energy,   only : qmmmener

      implicit none
c      
c  i/o variables
c also add mmgeom definition below                                      
c
      type (proglist) :: progrms
      integer            ierr, limohc, ltest, ltestmm, lqmmm            
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      character*20    :: tmeth, tbas
c
      character*80    :: mcoptl(maxoptl), toptl(maxoptl)
      type (qmmminf)  :: qmmminp                                        
c
c  local variables
c
      integer n3tm, i            
      
      n3tm = 3*geom%natoms

c
c  Make calls to subhooks depending on activated switches
c
c  TEST
c
      if (ltest.eq.1)  then
         call progghk(1, progrms%testprog,tmeth,tbas, 
     *         geom%charge, geom%mult, geom , toptl, 
     *                ener, totalgrad(:n3tm), ierr, 'null')             1010SP10
c     *                ener, grad(:n3tm), ierr, 'null')
c            call testdisp( geom%natoms, tgrad )
         grad(:n3tm)=totalgrad(:n3tm)                                   0520SP10
         if (tdynamics.ne.1) call testdisp( geom%natoms, tgrad )        0628SP10
      endif
c
c  TESTMM                                                               
c
      if (ltestmm .eq. 1)  then                                         
        if (tdynamics.ne.1) then                                        0628SP10
            call progmmghk(1, progrms%testmmprog, tmeth, tbas, geom,    1010SP10
     *      mmgeom, toptl, ener, totalgrad(:n3tm), ierr, 'null')               
c     *      mmgeom, toptl, ener, grad(:n3tm), ierr, 'null')               
            grad(:n3tm)=totalgrad(:n3tm)                                0520SP10
            call testdisp( geom%natoms, tgrad )
c
c  add June 28, 2010 by Soroosh Pezeshki
c
        else
            call progmmghkmd(1, progrms%testmmprog, tmeth, tbas, geom,
     *      mmgeom, toptl, ener, totalgrad(:n3tm), ierr, 'null')               
        endif
c
c   end add June 28, 2010 by Soroosh Pezeshki
c
      endif            
c
c  QMMM                                                                 
c
      if (lqmmm .eq. 1)  then
        if (tdynamics.ne.1) then                                        0304SP11
            call qmmmghk(progrms%qmprog, progrms%mmprog, geom, mmgeom,
     *            qmmminp, ierr)
            ener = qmmmener
c
c  add March 04, 2011 by Soroosh Pezeshki
c
         else
            call qmmmghkmd(progrms%qmprog, progrms%mmprog, geom,
     *             mmgeom,qmmminp, ierr, qmmmgrad,qmmmener)

            totalgrad(:n3tm) = qmmmgrad(:n3tm)
            ener=qmmmener
        endif
c
c   end add March 04, 2011 by Soroosh Pezeshki
c
      endif                                                         

      return
      end
c
c     
c*****************************************************************************
c  PROGGHK
c*****************************************************************************
c
      subroutine progghk(ppnum, prognme, esmeth, esbas, qmcharge, 
     *                    qmmult, geom, optl, 
     *                    ener, grad, ierr, ver_lab)
c
c     Subroutine progghk
c
c     Created:  Feb. 21, 2005 by Hai Lin
C     Revised:  Apr. 14, 2006 by Yan Zhang
c               May  22, 2006 by Yan Zhang
c               Aug. 15, 2010 by Soroosh Pezeshki
c
c     Purpose:  Get a single point energy and gradient for the given geom
c        with the specified electronic structure package.
c
c     Description:  Calls a function to write a G03 FORCE input file,
c         runs this calculation on the system, and then calls a fxn that
c         reads the energy and gradient from the formatted checkpoint file
c         created by G03.
c
c     Note:  Currently only G03 are enabled but it would be easily
c        expanded to other electronic structure programs.  new _inpe and
c        _oute subroutines would have to be written for each different
c        program though.
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c     Called by:  optghk, qmmmghk
c     Calls:  g03inp, g03oute, g03outg, system, orcaoute, orcaoutg, CPU_TIME
c
c     Variables defined here:
c
      use caltyp
      use files
      use input, only : cartgeom, maxoptl
      implicit none

      character*10 prognme
      character*20 esmeth, esbas, ver_lab                                                
      type (cartgeom) :: geom
c
      character*80 optl(maxoptl)
      double precision ener, grad(3*geom%natoms)
      integer ierr, ppnum
      
      integer qmcharge, qmmult
c      
c    by Soroosh Pezeshki
c  timing variables
c
      real timing1, timing2, deltatiming
      integer timingw1, timingw2, deltatimingw
      call CPU_TIME(timing1)
      call SYSTEM_CLOCK(timingw1)
c
c end timinig by Soroosh Pezeshki
c
c
c  GAUSSIAN03
c
c      if ((prognme(:3).eq.'g03') .or. (prognme(:3).eq.'g09')) then      !1222BW10 add g09
      if ((prognme(:3).eq.'g03') .or. (prognme(:3).eq.'g09')            1020XW17
     * .or. (prognme(:3).eq.'g16')) then                                1020XW17
         open (unit=g03inf, status='unknown', file=g03in, err=9993)
         call g03inp(g03inf,esmeth,esbas,qmcharge,qmmult,
     *               geom,optl,tgrad,ver_lab)
         close (g03inf, err=9993)
c
c    by Soroosh Pezeshki
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  Gaussian input timing ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif
c
c end timinig by Soroosh Pezeshki
c

         call system('g03shuttle g03.inp g03.out')

c
c    by Soroosh Pezeshki
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  Gaussian run timing ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif
c
c end timinig by Soroosh Pezeshki
c

         call g03oute(1,esmeth, ener)
         call g03outg(1,esmeth, grad, geom%natoms)                         
c
c    by Soroosh Pezeshki
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  Gaussian output timing ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif
c
c end timinig by Soroosh Pezeshki
c


c  
c  ORCA 4/10/2006 by Yan Zhang
c
         else if(prognme(:4).eq.'orca') then
         open (unit=orcainf, status='unknown', file=orcain, err=9994)
         call orcainp(orcainf,esmeth,esbas,qmcharge,qmmult,
     *                geom,optl,tgrad,ver_lab)
         close (orcainf, err=9994)
c
c    by Soroosh Pezeshki
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  Orca input timing ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif
c
c end timinig by Soroosh Pezeshki
c

         call system('orcashuttle orca.inp orca.out')
c
c    by Soroosh Pezeshki
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  Orca run timing ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif
c
c end timinig by Soroosh Pezeshki
c

         call orcaoute(1,esmeth, ener)
         call orcaoutg(1,esmeth, grad, geom%natoms,geom%natoms) 
c
c   end  4/10/2006 by Yan Zhang
c
c
c    by Soroosh Pezeshki
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  Orca output ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif
c
c end timinig by Soroosh Pezeshki
c

c   Gamess   5/22/2006 by Yan Zhang
c
      else if (prognme(:6).eq.'gamess') then
         open (unit=gmsinf, status='unknown', file=gmsin, err=9995)
         call gmsinp(gmsinf,esmeth,esbas,qmcharge,qmmult,
     *              geom,optl,tgrad,ver_lab)
         close(gmsinf,err=9995)
         call system('gmsshuttle gms.inp gms.out')
         call gmsoutg (1, esmeth, grad, geom%natoms,ener)
c
c   end 5/22/2006  by Yan Zhang
c

      endif


      return

c9993  write (outfile,*) 'Error creating Gaussian03 Forces input file'
9993  write (outfile,*) 'Error creating Gaussian Forces input file'     1020XW17
      flush (outfile)                                                   0713SP11
      stop 'progghk 3'

9994  write (outfile,*) 'Error creating Orca Forces input file'
      flush (outfile)                                                   0713SP11
      stop 'progghk 4'

9995  write (outfile,*) 'Error creating Gamess Forces input file'
      flush (outfile)                                                   0713SP11
      stop 'progghk 5'

      end
c
c
c*****************************************************************************
c  g03OUTG
c*****************************************************************************
c  
      subroutine g03outg (ppnum, esmeth, grad, natoms)
c
c     Subroutine g03outg
c
c     Created:  Feb. 22, 2005 by Hai Lin
c     Revised:
c     Purpose:  Get the gradient out of the Gaussian03 formatted
c        checkpoint file.  
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c     Called by:  progghk, proghhk
c     Calls:
c
c     Variables defined here: 
c     
      use files
      use numfmt
      implicit none
      
      character*20  esmeth
      integer ctr, i, natoms, j, gstrt, gend, ppnum,fchknum                    
      double precision  grad(3*natoms), cfltfmt
c
      character*80 string, case
      character*15 gtxt(3*natoms)
      character*14 fchkname                                                    

c                                                                     
c  Open the formatted checkpoint file.
c
      fchknum = fchk                                                          
      fchkname = testfchk                                                     

      open( unit=fchknum, status='unknown', file=fchkname, err=9900)

c
      read( fchknum, '(a80)', end=9910 ) string
      string = case(string)

      do while( string(:18).ne.'cartesian gradient' .and. 
     *              string(:16).ne.'cartesian forces' )
c
         read( fchknum, '(a80)', end=9910 ) string
         string = case(string)
      enddo
c
c  Get each element of gradient as a piece of text and put into an array
c    of strings.
c
      ctr = 1
      do while( ctr .le. 3*natoms )
c
         read( fchknum, '(a80)', end=9910 ) string
         string = case(string)
         i = 0
         do while( ctr.le.3*natoms .and. i.lt.5 )
            gstrt = i*(nlen(gfmt)+1)+2
            gend  = (i+1)*(nlen(gfmt)+2)
            gtxt(ctr) = string(gstrt:gend)
            ctr = ctr+1
            i = i+1
         end do
      end do
c
c  Convert strings to floating points
c
      do 20 j=1, 3*natoms
         grad(j) = cfltfmt(gtxt(j),gfmt)
  20  continue

      close( unit=fchknum )

      return

c9900  write( outfile,*) 'Error opening Gaussian03 checkpoint file'
9900  write( outfile,*) 'Error opening Gaussian checkpoint file'        1020XW17
      stop 'g03outg 1'
c9910  write( outfile,*) 'Error reading Gaussian03 checkpoint file'
9910  write( outfile,*) 'Error reading Gaussian checkpoint file'        1020XW17
      stop 'g03outg 2'

      end
c
c
c*****************************************************************************
c  PROGMMGHK
c*****************************************************************************
c
      subroutine progmmghk(ppnum, prognme, esmeth, esbas, geom, 
     *                   mmgeom, optl, ener, grad, ierr, ver_lab)
c
c     Subroutine progmmghk
c
c     Created:  Feb. 22, 2004 by Hai Lin 
c
c     Purpose:  Get a single point energy and gradient for the given geom
c        with the specified force field package.
c
c     Description:  Calls a subroutine to write an MM input file,
c         runs two (energy and gradient) calculations on the system, 
c         and then calls a subroutine 
c         that reads the energy and gradient from the MM output file
c         created by the force field package.
c
c     Note:  Currently only TINKER is enabled but it would be easily
c        expanded to other force field programs.  new _inpe and
c        _oute subroutines would have to be written for each different
c        program though.
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c     Called by:  optghk, qmmmghk
c
c     Calls:  t41inp, t41outg, system
c
c     Variables defined here:
c
      use caltyp
      use files
      use input, only : cartgeom, mmvalen, maxoptl
      implicit none
c
c  i/o variables
c
      character*10 prognme
      character*20 esmeth, esbas, ver_lab                                                
      type (cartgeom) :: geom
      type (mmvalen) :: mmgeom      
c
      character*80 optl(maxoptl)
      double precision ener, grad(3*geom%natoms)
      integer ierr, ppnum
c
c  local variables
c      
c
c  TINKER 
c
      if (prognme(:6).eq.'tinker') then
         open (unit=t41inf, status='unknown', file=t41in, err=9994)
         open (unit=t41keyf, status='unknown', file=t41key, err=9995)
         call t41inp(t41inf, t41keyf,
     *    esmeth, esbas, geom, mmgeom,optl,tgrad,ver_lab)
         close (t41inf, err=9994)
         close (t41keyf, err=9995)
c
c read the atomic MM charge
c  also print MM parameters if required
c
         if ((tprm .eq. 1) .or. (mmgeom%lprnatmprm .eq. 1)) then
            call system('t41shuttle t41 p')
            call t41outp(mmgeom)
        endif    
c
c calculate energy and gradient
c	 
         call system('t41shuttle t41 e')
         call t41oute(1,esmeth,ener)
         call system('t41shuttle t41 g')
         call t41outg(1,esmeth,ener, grad, geom%natoms)
      endif
      return

9994  write (outfile,*) 'Error creating TINKER .xyz file'
      stop 'progghk 4'
9995  write (outfile,*) 'Error creating TINKER .key file'
      stop 'progghk 5'

      end
c
c
c*****************************************************************************
c  PROGMM0GHK
c*****************************************************************************
c
      subroutine progmm0ghk(ppnum, prognme, esmeth, esbas, geom, 
     *                   mmgeom, optl, ener, grad, ierr, ver_lab,
     *                   qmmminp)
c
c     Subroutine progmm0ghk
c
c     Created:  Dec. 03, 2004 by Hai Lin 
c
c     Purpose:  Get a single point energy and gradient for the given geom
c        with the specified force field package, whith the MM charges on 
c        QM atoms set to zero.
c
c     Description:  Calls a subroutine to write an MM input file,
c         runs two (energy and gradient) calculations on the system, 
c         and then calls a subroutine 
c         that reads the energy and gradient from the MM output file
c         created by the force field package.
c
c     Note:  Currently only TINKER is enabled but it would be easily
c        expanded to other force field programs.  new _inpe and
c        _oute subroutines would have to be written for each different
c        program though.
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c     Called by:  optghk, qmmmghk
c
c     Calls:  t41inp, t41outg, system
c
c     Variables defined here:
c
      use caltyp
      use files
      use input, only : cartgeom, mmvalen, maxoptl, qmmminf
      implicit none
c
c  i/o variables
c
      character*10  prognme
      character*20  esmeth, esbas, ver_lab                                                
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom      
c
      character*80  optl(maxoptl)      
      double precision   ener, grad(3*geom%natoms)
      integer            ierr, ppnum
      type (qmmminf)  :: qmmminp       
c
c  local variables
c      
      integer      i,ii   
      integer      j, mmatm, mmnum             !0817BW10
      double precision mmcharge,mm1charge      !0126BW13 for balanced M1 charge in E(SS,Coulomb) calculation

c
c  TINKER 
c
      if (prognme(:6).eq.'tinker' ) then
         open (unit=t41inf, status='unknown', file=t41in, err=9994)
         open (unit=t41keyf, status='unknown', file=t41key, err=9995)
         call t41inp(t41inf, t41keyf,
     *    esmeth, esbas, geom, mmgeom,optl,tgrad,ver_lab)
         close (t41inf, err=9994)
c
c set MM charges on QM atoms to zero
c  for the entire system, the index read in for the QM atoms will be used
c  for the capped system, the index will be from 1 to N, where N is the 
c      summed number for the QM and HL atoms.
c  the tcssmmkey in the common block caltyp specify the system, i.e.
c      whether the entire or the capped system
         
        if (tcssmmkey .eq. 1) then
           ii = qmmminp%qmnatm
           do i=1, ii
              write (t41keyf, *) 'charge ', -qmmminp%qmatm(i), ' 0.0'
           enddo		  
        elseif (tcssmmkey .eq. 2) then
           ii = qmmminp%qmnatm + qmmminp%capatm%capnatm 
           do i=1, ii
              write (t41keyf, *) 'charge ', -i, ' 0.0'
           enddo
        else
           write (outfile,*) 'Error! Please specify CPS or ES.'
           stop 'progmm0ghk 6'
        endif
         
        close (t41keyf, err=9995)
c
c read the atomic MM charge
c  also print MM parameters if required
c
         if ((tprm .eq. 1) .or. (mmgeom%lprnatmprm .eq. 1)) then
            call system('t41shuttle t41 p')
            call t41outp(mmgeom)
         endif    

c   start by Bo Wang 0203BW15
c   Do MM charge correction for M1 atom
         if (tcssmmkey .eq. 1 .and. qmmminp%lchgcorr .ne. 0) then
             open (unit=t41keyf, status='old',position='append',
     &                     file=t41key, err=9995)
             do i=1, qmmminp%mm1stlay%natm1stlay
               ii = qmmminp%mm1stlay%atm1stlay(i)
               mmcharge=0
               mmnum = qmmminp%balgroupn(i)
               do j=2,mmnum
                 mmatm=qmmminp%balgroup(i,j)
                 mmcharge = mmcharge + mmgeom%atmcharge(mmatm)
               enddo
               mm1charge = qmmminp%groupchg(i) - mmcharge
               write (t41keyf, 100) -ii,mm1charge
100   format (1x, 'charge',3x,i5,3x,d12.6)
             enddo
             close (t41keyf, err=9995)
         endif
c   end by Bo Wang 0203BW15 

c
c calculate energy and gradient
c	 
         call system('t41shuttle t41 e')
         call t41oute(1,esmeth,ener)
         call system('t41shuttle t41 g')
         call t41outg(1,esmeth,ener, grad, geom%natoms)
      endif
      return

9994  write (outfile,*) 'Error creating TINKER .xyz file'
      stop 'progmm0ghk 4'
9995  write (outfile,*) 'Error creating TINKER .key file'
      stop 'progmm0ghk 5'

      end
c
c*****************************************************************************
c  T41OUTG
c*****************************************************************************
c  
      subroutine t41outg (ppnum, esmeth, ener, grad, natoms)
 
c
c     Subroutine t41outg
c
c     Created:  Feb. 22, 2004 by Hai Lin
c     Revised:  Jun. 28, 2010 by Soroosh Pezeshki
c     Purpose:  Get the gradient out of the TINKER output file
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c     Called by:  progmmghk, progmmhhk
c     Calls:
c     
      use files
      use numfmt
      use caltyp
      use input, only : skipgoutlines                                   0422SP11
      implicit none
c 
c  i/o variables
c      
      character*20      esmeth
      integer           natoms,ppnum
      double precision  ener
      double precision  grad(3*natoms)
c
c  local variables
c 
      integer          i,nwords,isec,ieof,iscend
      integer          iefind, igfind
      double precision kcal2au, au2kcal, ekcal, eau, eread
      double precision kcalpang2au, au2kcalpang
      double precision au2ang, ang2au
      double precision greadkcal(3)
      double precision cfloat
      
c
      character*80     words(40), case

c
      character*80  string
      integer gradbegin


c
c  Define unit converting factor
c
      au2kcal = 627.5095d0
      kcal2au = 1.0d0 / au2kcal
      au2ang = 0.529177249d0
      ang2au = 1.0d0/au2ang
      kcalpang2au = kcal2au/ang2au
      au2kcalpang = 1.0d0/kcalpang2au
c
      open(unit=t41outf,status='unknown', file=t41out, err=9900)
      ieof = 0
      nwords = 6
      iefind = 0
      igfind = 0

c
c Search for 'Total Potential Energy :'
c      read the energy and convert it into au.
c
      do while (ieof .ne. 1)
         call readln(t41outf,words,nwords,isec,ieof,iscend)

         if(words(1) .eq. 'TOTAL'     .and. 
     *      words(2) .eq. 'POTENTIAL' .and.
     *      words(3) .eq. 'ENERGY'    .and. 
     *      words(4) .eq. ':')         then
               
        eread = cfloat(words(5))
            iefind = 1
            if(words(6) .eq. 'KCAL/MOLE') then
                ekcal = eread
                eau = ekcal * kcal2au
                goto 100
            elseif (words(6) .eq. 'AU') then
                eau = eread
                ekcal = eau *au2kcal
                goto 100
            else
                write(outfile,*) 'Error: No energy unit given'
                flush (outfile)                                         0713SP11
                stop 't41oute 3'
            endif            
         endif
      enddo

100   if(iefind .eq. 0) then
         write(outfile, *) 
     *    'Error: No energy given in TINKER gradient output.'
         flush (outfile)                                                0713SP11
         stop 't41outg 4'
      endif
c
c Debug:
c
      if (tdebug .eq. 1) then
         write( outfile,1000) ekcal,eau,ener
      endif  
1000  format(/1x,'Total energy in gradient calculation',(3f20.8))
c
c Check if the gradient and energy calculations yield the same energy   
c      
      if (tdynamics.ne.1)then                                           0628SP10
        if(ener .ne. eau) then
          write(outfile, *)
     *    'Error: Different energies given in TINKER',
     *    'energy and gradient outputs:', ener, eau
          flush (outfile)                                               0713SP11
          stop 't41outg 4'
        endif
c
c add by Soroosh Pezeshki July 7, 2010
c
c
c  skip the unimportant output
c
      else
        gradbegin=natoms*skipgoutlines
        do i=1,gradbegin
c
            read(t41outf,'(a80)',end=500) string
        enddo

c
c end add  by Soroosh Pezeshki July 7, 2010
c
      endif                                                             0628SP10

      ener=eau

c
c  Read and write the gradient components
c
c  tinker 4.1
c      search for 'Cartesian Gradient Breakdown over Individual Atoms :'
c
c  tinker3.5
c      search for 'Energy Derivative Breakdown over Individual Atoms :'
c

      do while (ieof .ne. 1) 
         call readln(t41outf,words,nwords,isec,ieof,iscend)  
             
         if(((words(1) .eq. 'CARTESIAN' .and. 
     *        words(2) .eq. 'GRADIENT') .or.
     *       (words(1) .eq. 'ENERGY' .and.
     *        words(2) .eq. 'DERIVATIVE')) .and.
     *       words(3) .eq. 'BREAKDOWN'  .and. 
     *       words(4) .eq. 'OVER'       .and.
     *       words(5) .eq. 'INDIVIDUAL' .and. 
     *       words(6) .eq. 'ATOMS') then
            igfind = 1
            goto 200
         endif
      enddo
c hailin01172005end

200   if(igfind .eq. 0) then
          write(outfile, *) 
     *    'Error: No gradient given in TINKER output.'
         flush (outfile)                                                0713SP11
         stop 't41outg 5'
      endif

c
c      search 'Anlyt'
c      read the analytic gradient and convert it into au.
c     
      i = 0
      do while ((ieof .ne. 1) .and. (i .lt. natoms))
         call readln(t41outf,words,nwords,isec,ieof,iscend)

         if(words(1) .eq. 'ANLYT') then
            i = i + 1
            greadkcal(1) = cfloat(case(words(3)))
            greadkcal(2) = cfloat(case(words(4)))
            greadkcal(3) = cfloat(case(words(5)))
            grad(i*3-2) = greadkcal(1) * kcalpang2au
            grad(i*3-1) = greadkcal(2) * kcalpang2au
            grad(i*3)   = greadkcal(3) * kcalpang2au
         endif
      enddo
c
c     check if all atoms are calculated
c
      if(i .ne. natoms) then
         write(outfile, *) 
     *    'Error: Not all atom gradients given in TINKER output.'
         flush (outfile)                                                0713SP11
         stop 't41outg 6'
      endif

      close( unit=t41outf )
      


      return

9900  write( outfile,*) 'Error opening TINKER output file'
      flush (outfile)                                                   0713SP11
      stop 't41outg 1'
500   write( outfile,*) 'Error No gradient given in TINKER output.'
      flush (outfile)                                                   0713SP11
      stop 't41outg 7'
      end
c
c
c******************************************************************************
c  QMMMGHK
c******************************************************************************
c
      subroutine qmmmghk(qmprognme, mmprognme, geom, 
     * mmgeom, qmmminp, ierr)
c
c     Subroutine qmmmehk
c
c     Created:  Feb 22, 2004 by Hai Lin
c     Revised:  May 22. 2006 by Yan Zhang
c               May 25, 2006 by Yan Zhang
c               Aug. 8, 2006 by Yan Zhang
c
c     Purpose:  To determine the QM/MM energy and gradient
c   
c      Please see QMMMEHK for notes.   
c
c     Called by:  mlghook, optghk
c     Calls:  progmmghk, progqmghk, qmmmdisp
c  
      use caltyp
      use input,  only   : cartgeom, mmvalen, qmmminf, maxatm
      use files,  only   : outfile
      use energy, only   : qmmmener, qmes, mmes
      use gradient, only : qmmmgrad, qmmmqmgr, qmmmmmgr
      implicit none
c
c i/o variables
c
      character*10       qmprognme, mmprognme
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (qmmminf)  :: qmmminp
      integer            ierr
c
c local variables
c      
      type (cartgeom) :: ssgeom, bgchgeom
      type (mmvalen)  :: ssmmgeom
      integer            lsnatm, ssnatm, bgchnum
      integer            sumtyp
c
c for flexible boundary treatment
c add by Yan Zhang Apr. 17, 2009
c 
      double precision   px, qme1, qme2
      double precision   polmmg1(maxatm*3)
      double precision   polmmg2(maxatm*3)
c end
c      double precision   mmst
c
c
c      tdebug=1

c     
c     The program can not calculate the QM/MM single-point gradient
C     using Gamess with electric embedding
c	revised 5/22/2006 by Yan Zhang
c
        if((qmprognme(:6).eq.'gamess') 
     $      .and.(qmmminp%embed .ne. 'mechanical')) then
        write(outfile,*) 'The program can not calculate',
     $  ' the QM/MM single-point gradient using Gamess with electric '
     $          ,'embedding.'
         stop 'qmmmghk 1'
        end if
c
c    end 5/22/2006 by Yan Zhang
C
c
c  Save the prnprm switch value
c
      ssmmgeom%lprnatmprm = mmgeom%lprnatmprm
      sumtyp = 0
cx      call qmmmgsum(qmmminp, mmgeom, geom%natoms, 
cx     *   qmmmgrad(:geom%natoms*3), sumtyp, ierr)
      call qmmmgsum(qmmminp, mmgeom, geom, geom%natoms,     !0106BW10 add geom
     *   qmmmgrad(:geom%natoms*3), sumtyp, ierr)

c
c  MM calculation on the entire system
c      
      write(outfile,100)
      write(outfile,110)
      tcssmmkey = 1
      if (qmmminp%embed .eq. 'mechanical') then            
         call progmmghk(1,mmprognme, qmmminp%mmmeth, qmmminp%mmbas, 
     *      geom, mmgeom, qmmminp%mmoptl, mmes(2), 
     *      qmmmmmgr(2,:geom%natoms*3), 
     *      ierr, 'null')
      else
         call progmm0ghk(1,mmprognme, qmmminp%mmmeth, qmmminp%mmbas, 
     *      geom, mmgeom, qmmminp%mmoptl, mmes(2), 
     *      qmmmmmgr(2,:geom%natoms*3),
     *      ierr, 'null', qmmminp)
      endif                  
      write(outfile,115) mmes(2)

100   format(80('-'))
110   format(/2x,'MM calculation on the entire system:',/)
115   format('MM energy (a.u.) for the entire system:',f20.10)      
116   format(/,10('Debug'))
117   format(2x,'MM gradient on entire system:')
118   format(4(d20.10))
119   format(10('Debug'),/)

      if (tdebug .eq. 1) then
          write (outfile,116)
          write (outfile,117)
          write (outfile,118) qmmmmmgr(2,:geom%natoms*3)
          write (outfile,119)
      endif
      sumtyp = 1 
cx      call qmmmgsum (qmmminp, mmgeom, geom%natoms,
cx     *   qmmmmmgr(2,:geom%natoms*3), sumtyp, ierr)
      call qmmmgsum (qmmminp, mmgeom, geom, geom%natoms,   !0106BW10 add geom
     *   qmmmmmgr(2,:geom%natoms*3), sumtyp, ierr)

c
c  Assign the appropriate geometries to ssgeom
c      
      write(outfile,100)
      write(outfile,120)
      call qmmmssgeom(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
120   format(' Construct the capped primary system:')
      
c
c  MM calculation on the primary system, no point charges import
c
      write(outfile,100)
      write(outfile,130)
      tprm = 0
      tcssmmkey = 2
      if (qmmminp%embed .eq. 'mechanical') then            
         call progmmghk(1,mmprognme, qmmminp%mmmeth, qmmminp%mmbas, 
     *      ssgeom, ssmmgeom, qmmminp%mmoptl, mmes(1), 
     *      qmmmmmgr(1,:ssgeom%natoms*3), ierr, 'null')
      else
         call progmm0ghk(1,mmprognme, qmmminp%mmmeth, qmmminp%mmbas,
     *      ssgeom, ssmmgeom, qmmminp%mmoptl, mmes(1),
     *      qmmmmmgr(1,:ssgeom%natoms*3), ierr, 'null', qmmminp)
      endif     	        
      write(outfile,135) mmes(1)
130   format(//2x,'MM calculation on the capped primary system',
     *   ' without background charges:',/)
135   format(/2x,'MM energy (a.u.) for capped primary system:',
     *   f20.10)
137   format(2x,'MM gradient on primary system:')
      
      if (tdebug .eq. 1) then
        write (outfile,116)
        write (outfile,137)
        write (outfile,118) -qmmmmmgr(1,:ssgeom%natoms*3)
        write (outfile,119)
      endif
      sumtyp = 2
cx      call qmmmgsum (qmmminp, mmgeom, ssgeom%natoms,
cx     *   -qmmmmmgr(1,:ssgeom%natoms*3), sumtyp, ierr)
      call qmmmgsum (qmmminp, mmgeom, geom, ssgeom%natoms,   !0106BW10 add geom
     *   -qmmmmmgr(1,:ssgeom%natoms*3), sumtyp, ierr)
            
c
c  Work on background charges
c
      if (qmmminp%embed .eq. 'mechanical') goto 2000
      
      write(outfile,100)
      write(outfile,1000)
1000  format(//2x,'Treatment for the background point charges:')
c
c  Scale M1, M2 charges
c
      call bgchscale(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
      	   
      if ((qmmminp%bordercharge .eq. 'scale')) goto 3000

c		  
c     
c  Shift/redistribute M1 charges
c
      call bgchshift(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)   
c
c  Add charges to the primary system
c
3000  call qmmmbgchgeom(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
c
c  EE polarized charge calculation with the QEQ or EEM method
c  8/8/2006 by Yan Zhang                                                           
c
      call polareech(qmprognme,mmprognme,geom, mmgeom, 
     &                     ssgeom, ssmmgeom, qmmminp)

c
c  end 8/8/2006 by Yan Zhang1
c
c  Flexible-boundary charge calculation
c
      if(qmmminp%iflexbound .eq. 1) then
         call fbbpc(geom, mmgeom, ssgeom, ssmmgeom, qmmminp, 
     *    qmprognme, px) 
      endif
c
c  End by Yan Zhang Apr. 17, 2009
c
c  Electricstatic interactions within the background point charges
c
c
c    add 5/25/2006 by Yan Zhang
c  The single-point energy with background point charge calculated by ORCA
C  does not include the electricstatic interactions between the background 
C  point charge. It is not to subtract the electricstatic interactions from
C  the total energy.
c
      if(qmprognme.eq.'orca') goto 2000
c
c     end 5/25/2006 by Yan Zhang
c

      write(outfile,100)
      write(outfile,400)
400   format(/2x,'Electricstatic interactions within ',
     *  'the background point charges:',/)

      tcssmmkey = 2
      call progmmstghk(1,mmprognme, qmmminp%mmmeth, qmmminp%mmbas,
     *  ssgeom, ssmmgeom, qmmminp%mmoptl, mmes(3), 
     *  qmmmmmgr(3,:ssgeom%natoms*3), ierr, 'null', qmmminp)
      write(outfile,435) mmes(3)
        
      mmes(1) = mmes(1) + mmes(3)
435   format(/2x,'Additional MM energy (a.u.):',t50,f20.10,/2x,
     *  '(due to electricstatic interactions within ', 
     *  'background point charges)')			   
     
437   format(2x,'Additonal MM gradient')
     
      if (tdebug .eq. 1) then
         write (outfile,116)
         write (outfile,437)
         write (outfile,118) qmmmmmgr(3,:ssgeom%natoms*3)
         write (outfile,119)
      endif

      if (qmmminp%bordercharge .eq. 'scale')       then
         sumtyp = 4
      elseif (qmmminp%bordercharge .eq. 'shift')   then
         sumtyp = 5
      elseif (qmmminp%bordercharge .eq. 'redist1') then
         sumtyp = 6
      elseif (qmmminp%bordercharge .eq. 'rc')      then
         sumtyp = 6
      elseif (qmmminp%bordercharge .eq. 'redist2') then
         sumtyp = 7										 
      elseif (qmmminp%bordercharge .eq. 'rcd2')    then
         sumtyp = 7										 
      elseif (qmmminp%bordercharge .eq. 'redist3') then
         sumtyp = 6      
      elseif (qmmminp%bordercharge .eq. 'rcd')     then
         sumtyp = 6      
c add by Bo Wang 0203BW15 
      elseif (qmmminp%bordercharge .eq. 'rcbal')   then
         sumtyp = 6
      elseif (qmmminp%bordercharge .eq. 'rcbal2')  then
         sumtyp = 3
      elseif (qmmminp%bordercharge .eq. 'rcbal3')  then
         sumtyp = 3
      elseif (qmmminp%bordercharge .eq. 'rcdbal')  then
         sumtyp = 6
      elseif (qmmminp%bordercharge .eq. 'amber2') then
         sumtyp = 3
      elseif (qmmminp%bordercharge .eq. 'seebal')  then
         sumtyp = 4
      elseif (qmmminp%bordercharge .eq. 'amber1') then
         sumtyp = 3
      elseif (qmmminp%bordercharge .eq. 'shiftbal') then
         sumtyp = 5
      elseif (qmmminp%bordercharge .eq. 'shiftbal2') then
         sumtyp = 5
c end by Bo Wang 0203BW15 
      else
         sumtyp = -1
      endif
cx      call qmmmgsum (qmmminp, mmgeom, ssgeom%natoms,
cx     *   -qmmmmmgr(3,:ssgeom%natoms*3), sumtyp, ierr)	     					         
      call qmmmgsum (qmmminp, mmgeom, geom, ssgeom%natoms,  !0106BW10 add geom
     *   -qmmmmmgr(3,:ssgeom%natoms*3), sumtyp, ierr)

c
2000  continue

c
c  QM or QM** calculation on the primary system
c
      write(outfile,100)
      write(outfile,140)
c
c for flexible boundary treatment added by Yan Zhang, Apr. 15, 2009
c modified by Yan Zhang, May 30, 2009  
c      if(qmmminp%iflexbound .eq. 1) goto 5000
      if(qmmminp%iflexbound .eq. 1) goto 4900
c
c end Apr. 17, 2009
c
      if (qmmminp%embed .eq. 'mechanical') then      
         call progghk(1, qmprognme, qmmminp%qmmeth, qmmminp%qmbas, 
     *      qmmminp%qmcharge, qmmminp%qmmult,
     *      ssgeom, qmmminp%qmoptl, qmes(1), 
     *      qmmmqmgr(:ssgeom%natoms*3), ierr, 'null')
      else
         call proggsthk(1, qmprognme, qmmminp%qmmeth, qmmminp%qmbas,
     *      qmmminp%qmcharge, qmmminp%qmmult,
     *      ssgeom, qmmminp%qmoptl, qmes(1), 
     *      qmmmqmgr(:ssgeom%natoms*3), ierr, 'null',
     *      qmmminp, ssmmgeom)	 
      endif
      
4900  continue
c
c for flexible boundary treatment added by Yan Zhang, Apr. 15, 2009
c
      if(qmmminp%iflexbound .eq. 1) then
         call proggsthk(1, qmprognme, qmmminp%qmmeth, qmmminp%qmbas,
     *      qmmminp%qmcharge, qmmminp%qmmult,
     *      ssgeom, qmmminp%qmoptl, qme1, 
     *      polmmg1(:ssgeom%natoms*3), ierr, 'null',
     *      qmmminp, ssmmgeom)	 
      
         call proggsthk(1, qmprognme, qmmminp%qmmeth, qmmminp%qmbas,
     *      qmmminp%asqmcharge, qmmminp%asqmmult,
     *      ssgeom, qmmminp%qmoptl, qme2, 
     *      polmmg2(:ssgeom%natoms*3), ierr, 'null',
     *      qmmminp, ssmmgeom)	 
         qmes(1) = qme1*(1.d0-px) + qme2*px
         qmmmqmgr(:ssgeom%natoms*3) = 
     *      polmmg1(:ssgeom%natoms*3)*(1.d0-px)+
     *                                polmmg2(:ssgeom%natoms*3)*px
         write(outfile,9001) qme1
         write(outfile,9002) qme2
         write(outfile,9003) qmes(1)
      endif
9001  format('CPS* energy of initial state:',f16.10)
9002  format('CPS* energy of added state:',f16.10)
9003  format('final CPS* energy:',f16.10)
c
c end Apr. 15, 2009
c
      write(outfile,145) qmes(1)
140   format(/2x,'QM calculation on the capped primary system:',/)
145   format('QM energy (a.u.) for capped primar system:',f20.10)      
147   format(2x,'QM (QM**) gradient on primary (primary + ',
     *   'background-charge) system:')

      if (tdebug .eq. 1) then
          write (outfile,116)
          write (outfile,147)
          write (outfile,118) qmmmqmgr(:ssgeom%natoms*3)
          write (outfile,119)
      endif
      if (qmmminp%embed .eq. 'mechanical')            then
         sumtyp = 2
      elseif (qmmminp%embed .eq. 'electric')          then
         if (qmmminp%bordercharge .eq. 'scale')       then
            sumtyp = 4
         elseif (qmmminp%bordercharge .eq. 'shift')   then
            sumtyp = 5
         elseif (qmmminp%bordercharge .eq. 'redist1') then
            sumtyp = 6
         elseif (qmmminp%bordercharge .eq. 'rc')      then
            sumtyp = 6
         elseif (qmmminp%bordercharge .eq. 'redist2') then
            sumtyp = 7
         elseif (qmmminp%bordercharge .eq. 'rcd2')    then
            sumtyp = 7
         elseif (qmmminp%bordercharge .eq. 'redist3') then
            sumtyp = 6	 
         elseif (qmmminp%bordercharge .eq. 'rcd')     then
            sumtyp = 6	 
c add by Bo Wang 0203BW15
         elseif (qmmminp%bordercharge .eq. 'rcbal')   then
            sumtyp = 6
         elseif (qmmminp%bordercharge .eq. 'rcbal2')  then
            sumtyp = 3
         elseif (qmmminp%bordercharge .eq. 'rcbal3')  then
            sumtyp = 3
         elseif (qmmminp%bordercharge .eq. 'rcdbal')  then
            sumtyp = 6
         elseif (qmmminp%bordercharge .eq. 'amber2') then
            sumtyp = 3
         elseif (qmmminp%bordercharge .eq. 'seebal')  then
            sumtyp = 4
         elseif (qmmminp%bordercharge .eq. 'amber1') then
            sumtyp = 3
         elseif (qmmminp%bordercharge .eq. 'shiftbal') then
            sumtyp = 5
         elseif (qmmminp%bordercharge .eq. 'shiftbal2') then
            sumtyp = 5
c end by Bo Wang 0203BW15
         else
            sumtyp = -1    	    	    
         endif    
      else
            sumtyp = -1	        
      endif
cx      call qmmmgsum (qmmminp, mmgeom, ssgeom%natoms,
cx     *   qmmmqmgr(:ssgeom%natoms*3), sumtyp, ierr)	   
      call qmmmgsum (qmmminp, mmgeom, geom, ssgeom%natoms,    !0106BW10 add geom
     *   qmmmqmgr(:ssgeom%natoms*3), sumtyp, ierr)
	         	 
c
5000  continue

c
c  Calculate the QMMM energy
c
      write(outfile,100)
      write(outfile,9000)            
      qmmmener = qmes(1) - mmes(1) + mmes(2)
      write(outfile,9100) qmmmener, qmes(1), mmes(1), mmes(2)
      
9000  format(/,'Display of QMMM energy (a.u.):',/2x,'QMMM      ',
     *   10x,'QM;CPS**',10x,'MM;CPS*',10x,'MM;ES') 
9100  format(4(f16.10,2x))      
c
c  Display gradient results
c
      write(outfile,100)
      write(outfile,9200)
      write(outfile,9210) qmmmgrad(:geom%natoms*3)        
9200  format(/,'Display of QMMM gradient (a.u.):')
9210  format(3(e20.12))			    				   		      		       
      return 
      end

c******************************************************************************
c  QMMMGSUM
c******************************************************************************
c
cx      subroutine qmmmgsum(qmmminp, mmgeom, dim, grad, sumtyp, ierr)
      subroutine qmmmgsum(qmmminp,mmgeom,geom,dim,grad,sumtyp,ierr)
c  0106BW10 add geom(coordinates for Q1 and M1 atoms) for fixed bond method

c
c     Subroutine qmmmgsum
c
c     Created:  Feb. 22, 2004 by Hai Lin
c     Revised:  
c
c     Purpose:  To add the qm or mm gradient (input as grad) into the 
c               qmmmgrad gradient
c
c     Called by:  qmmmghk
c     Calls:  
c
      use caltyp
      use input, only    : cartgeom, mmvalen, qmmminf
      use files, only    : outfile
      use gradient, only : qmmmgrad, qmmmqmgr, qmmmmmgr
      implicit none
c
c  i/o variables
c  
      type (cartgeom) :: geom, lsgeom, ssgeom   !0106BW10 uncomment it
      type (mmvalen)  :: mmgeom
      type (qmmminf)  :: qmmminp
      integer            ierr
      integer            dim
      integer            sumtyp
      double precision   grad(dim*3)
c
c  local variables
c
      integer  capnatm, qmnatm, mmnatm, m1chnatm
      integer  ssnatm, cssnatm, lsnatm
      integer  i,j,k, ii,jj,kk
      integer  m1st, m2nd
      
      double precision  bs1, bs2
      double precision  qs1, qs2, mqs1, mqs2, pqs1, pqs2
      double precision dx,dy,dz,dr,dist                 !0106BW10
c
c  prepare atom numbers
c
      capnatm = qmmminp%capatm%capnatm
      qmnatm  = qmmminp%qmnatm
      mmnatm  = qmmminp%mmnatm
      ssnatm  = mmgeom%natoms
      cssnatm = capnatm + qmnatm
      lsnatm  = capnatm + qmnatm + mmnatm
      m1chnatm = ssnatm - lsnatm
      m1st = qmmminp%mm1stlay%natm1stlay
      m2nd = qmmminp%mm2ndlay%natm2ndlay
      
c
c  prepare the auxiliary charge bondlength scale factor
c
      qs1  = qmmminp%chredist%chposit
      qs2  = 1.0d0 - qs1
      mqs1 = qmmminp%chredist%dipposit - 
     *          qmmminp%chredist%dipdist * 0.5d0
      mqs2 = 1.0d0 - mqs1
      pqs1 = qmmminp%chredist%dipposit + 
     *          qmmminp%chredist%dipdist * 0.5d0
      pqs2 = 1.0d0 - pqs1	   

c
c  switch sumtyp
c
c -1) Special: do nothing
c
      if (sumtyp .eq. -1) then
         return
      endif
c      	 
c  0) Special: Initialization grad
c
      if (sumtyp .eq. 0) then
         grad(:dim*3) = 0.0d0
      endif	 
c
c  1) Large (real) system
c         
      if (sumtyp .eq. 1) then
         qmmmgrad(:lsnatm*3) = qmmmgrad(:lsnatm*3) + grad(:dim*3)
      endif	 
c
c  >2) CSS system: QM + Cap
c         
      if (sumtyp .ge. 2) then
         do i=1, qmnatm
            j = qmmminp%qmatm(i)
            qmmmgrad(j*3-2:j*3) = qmmmgrad(j*3-2:j*3) 
     *         + grad(i*3-2:i*3)
         enddo   
         do i=1, capnatm
c start by Bo Wang 0203BW15
cx            bs1 = qmmminp%capatm%capbondscale(i)
cx            bs2 = 1.0d0 - bs1
cx            ii = i + qmnatm
cx            j = qmmminp%capatm%capatmqm(i)
cx            k = qmmminp%capatm%capatmm1(i)
cx            qmmmgrad(j*3-2:j*3) = qmmmgrad(j*3-2:j*3)
cx     *         + grad(ii*3-2:ii*3) * bs2
cx            qmmmgrad(k*3-2:k*3) = qmmmgrad(k*3-2:k*3)
cx     *         + grad(ii*3-2:ii*3) * bs1
           ii = i + qmnatm
           j = qmmminp%capatm%capatmqm(i)     !0106BW10  Q1 atom
           k = qmmminp%capatm%capatmm1(i)     !0106BW10  M1 atom
           if (qmmminp%capatm%capconstrain .eq. 'scale') then
             bs1 = qmmminp%capatm%capbondscale(i)
             bs2 = 1.0d0 - bs1
             qmmmgrad(j*3-2:j*3) = qmmmgrad(j*3-2:j*3)
     *         + grad(ii*3-2:ii*3) * bs2
             qmmmgrad(k*3-2:k*3) = qmmmgrad(k*3-2:k*3)
     *         + grad(ii*3-2:ii*3) * bs1
           elseif (qmmminp%capatm%capconstrain .eq. 'fix') then !0106BW10 the gradients when the bond length of Q1-Link is fixed
             dx = geom%angcoord(3*k-2) - geom%angcoord(3*j-2)
             dy = geom%angcoord(3*k-1) - geom%angcoord(3*j-1)
             dz = geom%angcoord(3*k)   - geom%angcoord(3*j)
             dr = sqrt (dx**2 + dy**2 + dz**2)
             dist = qmmminp%capatm%capbondfix(i)
             qmmmgrad(j*3-2) = qmmmgrad(j*3-2)+grad(ii*3-2)*(1-dist/dr)
     *                          +grad(ii*3-2)*(dist*dx*dx/dr**3)
     *                          +grad(ii*3-1)*(dist*dx*dy/dr**3)
     *                          +grad(ii*3)*(dist*dx*dz/dr**3)
             qmmmgrad(j*3-1) = qmmmgrad(j*3-1)+grad(ii*3-1)*(1-dist/dr)
     *                          +grad(ii*3-2)*(dist*dy*dx/dr**3)
     *                          +grad(ii*3-1)*(dist*dy*dy/dr**3)
     *                          +grad(ii*3)*(dist*dy*dz/dr**3)
             qmmmgrad(j*3)   = qmmmgrad(j*3)  +grad(ii*3)  *(1-dist/dr)
     *                          +grad(ii*3-2)*(dist*dz*dx/dr**3)
     *                          +grad(ii*3-1)*(dist*dz*dy/dr**3)
     *                          +grad(ii*3)*(dist*dz*dz/dr**3)
             qmmmgrad(k*3-2) = qmmmgrad(k*3-2)+grad(ii*3-2)*(dist/dr)
     *                          -grad(ii*3-2)*(dist*dx*dx/dr**3)
     *                          -grad(ii*3-1)*(dist*dx*dy/dr**3)
     *                          -grad(ii*3)*(dist*dx*dz/dr**3)
             qmmmgrad(k*3-1) = qmmmgrad(k*3-1)+grad(ii*3-1)*(dist/dr)
     *                          -grad(ii*3-2)*(dist*dy*dx/dr**3)
     *                          -grad(ii*3-1)*(dist*dy*dy/dr**3)
     *                          -grad(ii*3)*(dist*dy*dz/dr**3)
             qmmmgrad(k*3)   = qmmmgrad(k*3)  +grad(ii*3)  *(dist/dr)
     *                          -grad(ii*3-2)*(dist*dz*dx/dr**3)
     *                          -grad(ii*3-1)*(dist*dz*dy/dr**3)
     *                          -grad(ii*3)*(dist*dz*dz/dr**3)
           endif
c end by Bo Wang 0203BW15
         enddo   
      endif
c
c  >3) MM atoms after QM + Cap
c      
      if (sumtyp .ge. 3) then
        do i=1, mmnatm
            ii = i + cssnatm
            j = qmmminp%mmatm(i)
            qmmmgrad(j*3-2:j*3) = qmmmgrad(j*3-2:j*3)
     *         + grad(ii*3-2:ii*3)		  	    
        enddo
      endif
c
c  4) M1 atom after QM + Cap + MM
c      
      if (sumtyp .eq. 4) then
         do i=1, m1st
            ii = i + lsnatm
            j = qmmminp%mm1stlay%atm1stlay(i)
            qmmmgrad(j*3-2:j*3) = qmmmgrad(j*3-2:j*3)
     *         + grad(ii*3-2:ii*3)	    	    
         enddo
      endif	 
c
c  5) bordercharge = shift: dipole mimic charges
c   also for shiftbal/shiftbal2                    !0520BW09
      if (sumtyp .eq. 5) then
         do i=1, m2nd
            j = qmmminp%mm2ndlay%atm2ndlaym1(i)
            k = qmmminp%mm2ndlay%atm2ndlay(i)	    		 	    
            ii = i*2 - 1 + lsnatm
            qmmmgrad(j*3-2:j*3) = qmmmgrad(j*3-2:j*3)
     *         + grad(ii*3-2:ii*3) * mqs2
            qmmmgrad(k*3-2:k*3) = qmmmgrad(k*3-2:k*3)
     *         + grad(ii*3-2:ii*3) * mqs1
            ii = ii + 1
            qmmmgrad(j*3-2:j*3) = qmmmgrad(j*3-2:j*3)
     *         + grad(ii*3-2:ii*3) * pqs2
            qmmmgrad(k*3-2:k*3) = qmmmgrad(k*3-2:k*3)
     *         + grad(ii*3-2:ii*3) * pqs1
         enddo		  		  	    
      endif
c
c  6) bordercharge = redist1/rc/redist3/rcd: redistribute charges
c      also for rcbal/rcdbal                !0520BW09
      if (sumtyp .eq. 6) then
         do i=1, m2nd
            ii = i + lsnatm
            j = qmmminp%mm2ndlay%atm2ndlaym1(i)
            k = qmmminp%mm2ndlay%atm2ndlay(i)
            qmmmgrad(j*3-2:j*3) = qmmmgrad(j*3-2:j*3)
     *         + grad(ii*3-2:ii*3) * qs2
            qmmmgrad(k*3-2:k*3) = qmmmgrad(k*3-2:k*3)
     *         + grad(ii*3-2:ii*3) * qs1				   	    
        enddo
      endif
c
c  7) bordercharge = redist2/rcd2: redistribute and dipole mimic charges
c
      if (sumtyp .eq. 7) then
         do i=1, m2nd
            j = qmmminp%mm2ndlay%atm2ndlaym1(i)
            k = qmmminp%mm2ndlay%atm2ndlay(i)
            ii = i*3 - 2 + lsnatm
            qmmmgrad(j*3-2:j*3) = qmmmgrad(j*3-2:j*3)
     *         + grad(ii*3-2:ii*3) * qs2
            qmmmgrad(k*3-2:k*3) = qmmmgrad(k*3-2:k*3)
     *         + grad(ii*3-2:ii*3) * qs1
            ii = ii + 1
            qmmmgrad(j*3-2:j*3) = qmmmgrad(j*3-2:j*3)
     *         + grad(ii*3-2:ii*3) * mqs2
            qmmmgrad(k*3-2:k*3) = qmmmgrad(k*3-2:k*3)
     *         + grad(ii*3-2:ii*3) * mqs1
            ii = ii + 1
            qmmmgrad(j*3-2:j*3) = qmmmgrad(j*3-2:j*3)
     *         + grad(ii*3-2:ii*3) * pqs2
            qmmmgrad(k*3-2:k*3) = qmmmgrad(k*3-2:k*3)
     *         + grad(ii*3-2:ii*3) * pqs1	           	         	    	         	         	    	    	    	    
        enddo
      endif
c
c  if debug, show the uptodate gradient
c      
      if (tdebug .eq. 1) then
         write (outfile,1000)
         write (outfile,1010) sumtyp
         write (outfile,1020) qmmmgrad(:lsnatm*3)
         write (outfile,1005)
      endif
1000  format(/,10('Debug'))
1005  format(10('Debug'),/)
1010  format('Sumtyp = ',i2,8x,'Uptodate gradient for real system:')
1020  format(5(e16.8))
        
      return
      
      end      

c*****************************************************************************
c  PROGMMSTGHK
c*****************************************************************************
c
      subroutine progmmstghk1(ppnum, prognme, esmeth, esbas, geom, 
     *                   mmgeom, optl, ener, grad, ierr, ver_lab,
     *                   qmmminp)
c
c     Subroutine progmmstghk
c
c     Created:  Feb. 22, 2004 by Hai Lin 
c     Revised:  Dec. 03, 2004 by Hai Lin
c                Change v12 = 1.0 to v12 = 0.0
c
c     Purpose:  Get a single point interaction gradient between the given 
c               geom and the background point charge with the 
c               specified force field package.
c
c
c     Description:  Calls a subroutine to write an MM input file,
c         runs two (energy and gradient) calculations on the system, 
c         and then calls a subroutine 
c         that reads the energy and gradient from the MM output file
c         created by the force field package.
c
c     Note:  Currently only TINKER is enabled but it would be easily
c        expanded to other force field programs.  new _inpe and
c        _oute subroutines would have to be written for each different
c        program though.
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c     Called by:  optghk, qmmmghk
c     Calls:  t41inp, t41outg, system
c
      use caltyp
      use files
      use input, only : cartgeom, mmvalen, qmmminf, maxoptl
      implicit none
c
c  i/o variables
c
      character*10       prognme
      character*20       esmeth, esbas, ver_lab                                                
c
      character*80       optl(maxoptl)
      integer            ierr, ppnum

      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom	          
      type (qmmminf)  :: qmmminp
      double precision   ener, grad(geom%natoms*3)             
c
c  local variables
c      
      double precision   v11, v12, v22
c
c  TINKER 
c
      if (prognme(:6).eq.'tinker') then
         open (unit=t41inf, status='unknown', file=t41in, err=9994)
         open (unit=t41keyf, status='unknown', file=t41key, err=9995)
c
c
         call t41inp(t41inf, t41keyf,
     *      esmeth, esbas, geom, mmgeom,optl,tgrad,ver_lab)
      
c
c  additional key words
c add by Bo Wang 0203BW15
cx         v11 = 0.0d0
c  change v12 = 1.0 to 0.0 	 
cx         v12 = 0.0d0
cx         v22 = 1.0d0
cx         call t41keyadd(t41keyf, geom, mmgeom, qmmminp, 
cx     *      v11, v12, v22, ierr)
         if (qmmminp%ldampchg .eq. 0) then
         v11 = 0.0d0
         v12 = 0.0d0
         v22 = 1.0d0
         call t41keyadd(t41keyf, geom, mmgeom, qmmminp,
     *      v11, v12, v22, ierr)
         else                    !screening scheme
            v11 = 0.0d0
            v12 = 1.0d0
            v22 = 0.0d0
            call t41keyadd1(t41keyf, geom, mmgeom, qmmminp,
     *      v11, v12, v22, ierr)
         endif
c end by Bo Wang 0203BW15
     
         close (t41inf, err=9994)
         close (t41keyf, err=9995)        	    
         close (t41inf, err=9994)
         close (t41keyf, err=9995)
c
c read the atomic MM charge
c  also print MM parameters if required
c
         if ((tprm .eq. 1) .or. (mmgeom%lprnatmprm .eq. 1)) then
            call system('t41shuttle t41 p')
            call t41outp(mmgeom)
         endif    
c
c calculate energy and gradient
c	 
         call system('t41shuttle t41 e')
         call t41oute(1,esmeth,ener)
         call system('t41shuttle t41 g')
         call t41outg(1,esmeth,ener, grad, geom%natoms)
      endif
      return

9994  write (outfile,*) 'Error creating TINKER .xyz file'
      stop 'progghk 4'
9995  write (outfile,*) 'Error creating TINKER .key file'
      stop 'progghk 5'

      end
c

c*****************************************************************************
c  PROGGSTHK
c*****************************************************************************
c
      subroutine proggsthk1(ppnum, prognme, esmeth, esbas, 
     *   qmcharge, qmmult, ssgeom, 
     *   optl, ener, grad, ierr, ver_lab, qmmminp, ssmmgeom)
c
c     Subroutine proggsthk
c
c     Created:  Feb. 22, 2004 by Hai Lin
c     Revised:  Apr. 10, 2006 by Yan Zhang
c               July 22, 2011 by Soroosh Pezeshki

c     Purpose:  Get a single point energy and gradient for the small 
c        system in the presence of background charges with the
c        specified electronic structure package.
c
c     Description:  Calls a function to write a G03 input file,
c         runs this calculation on the system, and then calls a fxn that
c         reads the energy and gradient from the formatted checkpoint file
c         created by G03.
c
c     Note:  Currently only G03 is enabled but it would be easily
c        expanded to other electronic structure programs.  new _inpe and
c        _oute subroutines would have to be written for each different
c        program though.
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c     Called by:  qmmmghk
c
c     Calls:  g03inp, g03oute, g03outg, system, orcainp, orcaoute, orcaoutg
c
      use caltyp
      use files
      use input, only : cartgeom, mmvalen, qmmminf, maxoptl
      implicit none
c
c  i/o variables
c
      character*10 prognme
      character*20 esmeth, esbas, ver_lab                                                
      
      type (cartgeom) :: ssgeom
      type (mmvalen)  :: ssmmgeom   
      type (qmmminf)  :: qmmminp
      
c
      character*80 optl(maxoptl)
      double precision ener, grad(3*ssgeom%natoms)
      integer ierr, ppnum
      integer capnatm,qmnatm,cssnatm
      integer qmcharge, qmmult

      capnatm = qmmminp%capatm%capnatm
      qmnatm  = qmmminp%qmnatm
      cssnatm = capnatm+qmnatm
c
c  GAUSSIAN03
c
c      if ((prognme(:3).eq.'g03') .or. (prognme(:3).eq.'g09')) then !1222BW10 add g09
      if ((prognme(:3).eq.'g03') .or. (prognme(:3).eq.'g09')            1020XW17
     * .or. (prognme(:3).eq.'g16')) then                                1020XW17
         open (unit=g03inf, status='unknown', file=g03in, err=9993)
      if (tesp .eq. 0) then                                             0722SP11
        if (qmmminp%ldampchg .ne. 0) then            !0815BW10  screening schemes
         call g03stinp1(g03inf, esmeth, esbas,       !0204BW10
     *      qmcharge, qmmult, ssgeom,                !0204BW10
     *      optl, tgrad, ver_lab, qmmminp, ssmmgeom) !0204BW10
        else
         call g03stinp(g03inf, esmeth, esbas,
     *      qmcharge, qmmult, ssgeom,
     *      optl, tgrad, ver_lab, qmmminp, ssmmgeom)
        endif
c
c Add by Soroosh Pezeshki July 22, 2011
c
      else
         call g03stinpesp(g03inf, esmeth, esbas, 
     *      qmcharge, qmmult, ssgeom, 
     *      optl, ver_lab, qmmminp, ssmmgeom)
      endif
c
c end add by Soroosh Pezeshki July 22, 2011
c
         close (g03inf, err=9993)
         call system('g03shuttle g03.inp g03.out')

         call g03oute(1,esmeth, ener)
         call g03outg(1,esmeth, grad, ssgeom%natoms)

c   
c  ORCA added 4/10/2006 by Yan Zhang
c                            
       else if (prognme(:4).eq.'orca') then
         open (unit=orcainf, status='unknown', file=orcain, err=9994)
         call orcastinp(orcainf, esmeth, esbas, 
     *      qmcharge, qmmult, ssgeom, 
     *      optl, tgrad, ver_lab, qmmminp, ssmmgeom)
         close (orcainf, err=9994)
         call system('orcashuttle orca.inp orca.out')
         call orcaoute(1,esmeth, ener)                                      
         call orcaoutg(1,esmeth, grad, cssnatm, ssgeom%natoms)
      endif


      return

c9993  write (outfile,*) 'Error creating Gaussian03 Forces input file'
9993  write (outfile,*) 'Error creating Gaussian Forces input file'     1020XW17
      stop 'progghk 3'

9994  write (outfile,*) 'Error creating Orca Forces input file'
      stop 'progghk 4'
c
c       end 4/10/2006 by Yan Zhang
c
      end
c
c

