c
c******************************************************************************
c   MAIN
c******************************************************************************
c
      program main
c
c     QMMM is a computer program developed on the basis of Multilevel 3.1.
c       for performing QM/MM calculations. 
c
c     Current version: 1.0
c
c     Code Reference: QMMM
c                     Hai Lin, and Donald G. Truhlar 
c                     University of Minnesota, Minneapolis, 2004
c
c     Functions: single-point energies, gradients, or Hessians, 
c                or geometry optimizations.
c
c
c     Program main.F
c
c     Created:  Feb. 22, 2005 by Hai Lin
c     Revised:  May 25, 2010 by Soroosh Pezeshki
c
c     Purpose:  Main program driver for the QMMM code
c
c     Description: main.f opens and closes all the necessary QMMM input 
c                  and output files.  it also initiates all calls to 
c                  the subroutines.  
c
c     Calls: mlhedr, read5, mlehook, mlghook, mlhhook, mlohook, progohk
c            sumdisp 
c            mldhook                                                    0525SP10
c
c     
      use input
      use files
      use caltyp                                                        
c
      implicit none
c
      logical  lexit
      integer  ierr

      type (general)   :: mlgeninp
      integer          :: lqmmm, ltest, ltestmm
      integer          :: prtype                                        
c
c   add 2/10/2007 by Christoph Riplinger
c   hn3tm: nr of hessian atoms, 
c
      integer          :: hn3tm, lhessType
c
c   end 2/10/2007 by Christoph 
c
c
c   add 24/05/2010 by Soroosh Pezeshki
c
      integer :: ldynamic
      type (dynamicsinf) :: dyninp
c
c  end add 5/24/2010 Soroosh Pezeshki
c

c
      character*80     :: mcoptl(maxoptl), toptl(maxoptl)
      character*20     :: tmeth, tbas
      type (qmmminf)   :: qmmminp                                       
      type (mlopt)     :: mloptinp
      type (extopt)    :: extoptinp
      type (proglist)  :: progrms
c
c     open the input and output files (ml.inp and ml.out).  any additional
c        files will be opened and closed as needed.
c
      ierr = 0
c
      open (unit=outfile, file=mlout, status='unknown',
     *      form='formatted', err=6000)
      open (unit=infile, file=mlin, status='old', 
     *      form='formatted', err=5000)
c
c     write program header to outfile
c
      call mlhedr
c
c     read the input from infile (ml.inp) and write a summary to outfile
c
      call read5( mlgeninp, progrms, mloptinp, extoptinp, mcoptl,       
     *      lqmmm, qmmminp, ltest, ltestmm, 
     *      ldynamic, dyninp,                                           0525SP10
     *      tmeth, tbas, toptl )                                        

c
c Single-level external pre-optimization calls
c
      if ((mlgeninp%lextopt .eq. 1) 
     *  .and. (mlgeninp%calmodel .eq. 'mm')) then
         call progmmohk(mlgeninp%geom, mlgeninp%mmgeom, 
     *              extoptinp, progrms%extoptprog, ierr)               
      elseif (mlgeninp%lextopt .eq. 1) then
         call progohk(mlgeninp%geom, extoptinp, progrms%extoptprog, 
     *              ierr)
      endif
      if (ierr.eq.1)  goto 7000

c
c Multi-level optimization calls
c

      if (mlgeninp%lopt .eq. 1) then
         call mlohook(progrms, mlgeninp%geom,
     *      mloptinp, 
     *      qmmminp, mlgeninp%mmgeom,                         
     *      tmeth, tbas, toptl, ierr)                         
         mlgeninp%lener = 1
      endif
      if (ierr.eq.1)  goto 8000
c
c  Call either the hessian, gradient or energy hooks - but only call one,
c    the most expensive chosen - it will produce all the rest of the
c    information.
c
      if (mlgeninp%lhess .eq. 1) then
c
c   modified 2/10/2007 by Christoph Riplinger
c
c
c         call mlhhook(progrms, mlgeninp%geom,
c     *       mcoptl, ltest, ltestmm, mlgeninp%mmgeom,          
c     *       lqmmm, qmmminp,
c     *       tmeth, tbas, toptl, ierr)
         lhessType = mlgeninp%lhessType
         call mlhhook(progrms, mlgeninp%geom,
     *       mcoptl, ltest, ltestmm, mlgeninp%mmgeom,
     *       mlgeninp, mloptinp, lhessType,
     *       lqmmm, qmmminp,
     *       tmeth, tbas, toptl, ierr)
c
c   end 2/10/2007 by Christoph 
c
      elseif (mlgeninp%lgrad .eq. 1) then
         call mlghook(progrms, mlgeninp%geom,
     *       mcoptl, ltest, ltestmm, mlgeninp%mmgeom,          
     *       lqmmm, qmmminp,
     *       tmeth, tbas, toptl, ierr)                         
      elseif (mlgeninp%lener .eq. 1) then
         call mlehook(progrms, mlgeninp%geom, 
     *       mcoptl, ltest, ltestmm, mlgeninp%mmgeom,          
     *       lqmmm, qmmminp,
     *       tmeth, tbas, toptl, ierr)                         
      endif
c
c  add by Soroosh 05/25/2010
c  performe dynamics (MD)
c
      if (ldynamic .eq. 1) then
        call mldhook(progrms, mlgeninp%geom, 
     *       mcoptl, ltest, ltestmm, mlgeninp%mmgeom,          
     *       ldynamic, dyninp, lqmmm, qmmminp, mlgeninp,
     *       tmeth, tbas, toptl, ierr)                         
      endif
c
c  end add  by Soroosh 05/25/2010
c


      write(outfile,1070)
1070  format(//,80('-'),/80('-')/)
      write(*,*)

c
c     write summary file for interface program MultilevelRate  
c
      if (mlgeninp%lprsum .eq. 1) then                                  
          open (unit=sumfile, file=mlsum, status='unknown',             
     *          form='formatted', err=4000)                             

          if ( mlgeninp%lhess .eq. 1) then                              
              prtype = thess                                            
          else if (mlgeninp%lgrad .eq. 1) then                          
              prtype = tgrad                                            
          else if (mlgeninp%lener .eq. 1) then                          
              prtype = tener                                            
          end if                                                        
c
c   modified 2/10/2007 by Christoph Riplinger
c   if we calculate the partial hessian, there are only hessian values available
c   for the partial atoms
c
c          call sumdisp(mlgeninp%geom, 3*mlgeninp%geom%natoms, prtype,   
c     *                 ltest, ltestmm, 
c     *                 lqmmm, qmmminp)                          
          if (mlgeninp%part%ipartcal .eq. 1) then
             hn3tm = 3*mlgeninp%part%partatmnum
          else
             hn3tm = 3*mlgeninp%geom%natoms
          endif
          call sumdisp(mlgeninp%geom, 3*mlgeninp%geom%natoms, hn3tm,
     *                 prtype, ltest, ltestmm,
     *                 lqmmm, qmmminp)
c
c   end 2/10/2007 by Christoph 
c
      end if


      goto 9999


4000  write(*,'(/a)') 'ERROR: Cannot open the summary file.'            
      close(sumfile)                                                    
      close(outfile)                                                    
      close(infile)                                                     
      stop 'main 3'                                                     


5000  write (*,'(/a)') 'ERROR: Cannot open the input file.'
      close (infile)
      close (outfile)
      stop 'main 2'

6000  write (*,'(/a)') 'ERROR: Cannot open the output file.'
      close (outfile)
      stop 'main 1'

7000  write (outfile,'(/a)') 'ERROR: External Optimization failed.'
      goto 9999

8000  write (outfile,'(/a)') 'ERROR: Multilevel Optimization failed.'
      goto 9999

9999  close (infile)
      close (outfile)

      if (mlgeninp%lprsum .eq. 1) then
           close (sumfile)                                              
      end if

      end

