c******************************************************************************
c     MODULE:PROGDEF
c******************************************************************************
c
      module progdef
c
c     Created:  Feb. 21, 2005 by Hai Lin
c     Revised:
c     Purpose:  To define the default electronic structure program called
c        for each multilevel method.
c
c     Called by:  defprog
c
      implicit none
c
c  External Geometry Optimization
c  Default for *EXTOPT-PROGRAM
c
      character*10, parameter :: defextoptp = 'g03 '
c
c  Test Calculations
c  Default for *TEST-PROGRAM
c
      character*10, parameter :: deftestp = 'g03 '
c
c  Low level Hessian
c  Default for *MULTIOPT-HPROG
c
      character*10, parameter :: defhessp = 'g03 '
c  Test MM Calculations
c  Default for *TESTMM-PROGRAM
c
      character*10, parameter :: deftestmmp = 'tinker '
c
c  QM calculation of IMOMM
c  Default for IMOMM-LLKEY-PROGRAM
	character*10, parameter :: defqmp = 'g03 '
c  MM calculation of IMOMM
c  Default for IMOMM-LLKEY-PROGRAM
	character*10, parameter :: defmmp = 'tinker '
c
      end module progdef
c
c
c        
c******************************************************************************
c     MODULE:NUMFMT
c******************************************************************************
c
      module numfmt
c
c     Created:  Feb. 21, 2005 by Hai Lin
c     Revised:
c     Purpose:  To define the types of formatted numbers found in G03 and
c        G94 formatted checkpoint files.
c
c     Called by:  cfltfmt, g03oute, mg03oute, g03outg, g03outh, g03outo
c
c
      implicit none
c
c  Sets parameters for the different  type of number formats to be read.
c
      integer, parameter :: efmt = 1
      integer, parameter :: gfmt = 2
      integer, parameter :: hfmt = 3
c
c  Gives locations of signs, decimal points, and numbers in the formatted
c    checkpoint file when the number is an energy, a gradient, or a 
c    hessian.
c
      integer :: isign(3), inum(3), idec(3), decbeg(3), decfin(3)
      integer :: iexp(3), iesign(3), expbeg(3), expfin(3)
      data isign  / 1, 1, 1/
      data inum   / 2, 2, 2/
      data idec   / 3, 3, 3/
      data decbeg / 4, 4, 4/
      data decfin /18,11,11/
      data iexp   /19,12,12/
      data iesign /20,13,13/
      data expbeg /21,14,14/
      data expfin /22,15,15/
c
c  Gives length of the initial label, the starting point of the energy,
c    and the length of the floating point numbers in Test.FChk.
c
      integer :: llen(3), nstrt(3), nlen(3)
      data llen   /30, 0, 0/
      data nstrt  /50, 0, 0/
      data nlen   /22,15,15/
      end module numfmt
c 
c
c******************************************************************************
c     MODULE:INPUT
c******************************************************************************
c
      module input
c     
c     Created:  Feb. 21, 2005 by Hai Lin
c     Revised:  Jul. 21, 2006 by Yan Zhang
c               Aug.  7, 2006 by Yan Zhang
c               Aug. 23, 2006 by Yan Zhang
c               Dec. 11, 2006 by Yan Zhang
c               May   7, 2009 by Hai Lin
c               May  24, 2010 by Soroosh Pezeshki
c               Sept. 21, 2017 by Xin-Ping Wu
c  
c     Purpose:  To define the structures that the input information will
c        be placed in.  Also saves the internel optimization input info
c        from access to access.
c     
      implicit none

c  maxoptl gives the max input lines for QM or MM options

      integer, parameter :: maxoptl = 999                               
c
c
c  Note by Hai Lin, 5/5/2009:
c
c  The maxatm is set to 9900, because tinker by default handles 9999 atoms.
c  This can be changed to a bigger number if the number in tinker changed.
c
c  When change this number, please also change the maxatm number in 
c  modules param and gradient.
c
c  The maximum numbers in Hessian calculations and in geometry optimization 
c  using internal optimizer is different from the maxatm set here; they are 
c  set in modules hessian and optimization, respectiviely.
c
c  end Note by Hai Lin, 5/5/2009
c    
      integer, parameter :: maxatm = 9900                               
c
c changed by Xin-Ping Wu, Sept. 21, 2017
c
      integer, parameter :: maxvalen = 8
c      integer, parameter :: maxvalen = 6                                
c
c end by Xin-Ping Wu, Sept. 21, 2017
c
c      integer, parameter :: maxcapatm = 30
      integer, parameter :: maxcapatm = 100                             0712SP11
      integer, parameter :: maxchargelay = 3
      integer, parameter :: max1stlay =  30
      integer, parameter :: max2ndlay =  90
      integer, parameter :: maxredist =  90
      integer, parameter :: max3rdlay = 270
c
c Soroosh Pezeshki 07/19/2010
c
c maximum number of groups in Hot spot, ONIOM-XS, sorted AP and DAS
      integer, parameter :: maxbuffergroupps=100
c maximum repeats in recursive subroutines
      integer, parameter :: maxrecursive=10000
c maximum restraints atoms or groups
      integer, parameter :: maxrestraints=1000
c maximum number of high level calculation per step in permuted AP
      integer, parameter :: maxpcalc=10000
c maximum number of buffer groups in permuted AP (DON'T SET >64)
      integer, parameter :: maxpbgroups=64
c maximum order in permuted AP (2^maxporder MUST BE <maxpcalc)
      integer, parameter :: maxporder=13
c
c end Sorosh Pezeshki 07/19/2010
c


c
c Hai Lin, 5/5/2009
c
c   This is to set the maximum atoms in a group that is involved 
c   in any group-based treatment, such as the in a polarizable group 
c   in polarizable-boundary or flexible-boundary calculations.
c
      integer, parameter :: maxidnum     = 9999                         0725XW18
c
c end by Hai Lin, 5/5/2009
c
c
c  max in polar 8/7/2006 by Yan Zhang
c
      integer, parameter :: maxpolgnum   = 100
c
c changed by Hai Lin, 5/5/2009
c
c      integer, parameter :: maxpolgidnum = 300 
      integer, parameter :: maxpolgidnum = maxidnum
c
c end by Hai Lin, 5/5/2009
c
      integer, parameter :: maxpolcycl   = 1000
c
c    end 8/7/2006 by Yan Zhang
c
c  Add by Yan Zhang 9/04/2007
c  Max values for flexible-boundary treatment.
c
      integer, parameter :: maxgnum  = 100
c
c changed by Hai Lin, 5/5/2009
c
c      integer, parameter :: maxganum = 300 
      integer, parameter :: maxganum = maxidnum
c
c end by Hai Lin, 5/5/2009
c
      integer, parameter :: maxfbcycl= 1000
c
c  End by Yan Zhang 9/04/2007
c
c  max in partial optimizatiom added by Yan Zhang 10/29/2007
c
      integer, parameter :: maxpartatm = 1000 
      integer, parameter :: maxpart1stlay = 2000 
      integer, parameter :: maxpart2ndlay = 3000
c
c  end by Yan Zhang 10/29/2007
c  max in number of groups in charge balancing 0126BW14
      integer, parameter :: maxbalgroup  = 100
c
c  Cap atom name in qm and mm (amber96) calculations                    
c      
      integer, parameter     :: capams        = 1
      character*2, parameter :: capname       = 'H '
      character*2, parameter :: captypname    = 'HC'
      character*2, parameter :: qmtypname     = 'CT'
      integer, parameter     :: chargeams     = 2
      character*2, parameter :: chargename    = 'He'
      character*2, parameter :: chargetypname = 'HC'
c add by Bo Wang 0205BW15
c  Dummy atom used for redistributed charge
      character*2, parameter :: dummyname    = 'DU'
      character*2, parameter :: dummytypname = 'DU'
      integer,parameter      :: dummytyp     = 400
      integer,parameter      :: dummyams     = 1
c  H atom for M2 atom in the case of balanced RC 2
      character*2, parameter :: hname    = 'H'
      character*2, parameter :: htypname = 'HC'
      integer,parameter      :: htyp     = 23
c end by Bo Wang 0203BW15

c            
c
c      Number for cap atoms is the same as number for atoms
c      in the 1st MM layer. The number for atoms in the 2nd
c      MM layer is at most 3 times as that in the 1st layer.
c      (M1 bounded to less than 3 M2 atoms.)
c      The number for redistributed
c      charges are the same as the atoms in the 2nd layer.
c      The number of 3rd layer atoms is at most 3 times as 
c      that in the 2nd layer.
c      (M2 bounded to less than 3 M3 atoms.)
c
c  Input for all the possible different programs to be called.
c
      type proglist
         character*10 :: extoptprog, testprog, hessprog,
     *     testmmprog, qmprog, mmprog                                    
      end type proglist
c
c   add 2/10/2007 by Christoph Riplinger
c   we need the partial information in general and opt input
c
      type partial
         integer  pcharge, pmult
         integer  ipartcal
         integer  partatmnum
         integer  partatmid(maxpartatm)
c
c  Partial extend layer varialbe for g03
c  add by Yan Zhang 10/29/2007
c
         integer  extlayernum
         integer  layer1atmid(maxpart1stlay)
         integer  layer2atmid(maxpart2ndlay)
         integer  extlayer1atmnum, extlayer2atmnum
         integer  partcentid, ipartcent, ipartyp, initgpart
         double precision  partrad, partcentx, partcenty, partcentz                           
c
c  end by Yan Zhang 10/29/2007
c
      end type partial
c
c   end 2/10/2007 by Christoph
c
c
c  The geometry of the molecule (and all other information needed to write
c    the geom into an input file).
c
      type cartgeom
         integer             natoms
         integer             charge
         integer             mult
         character*2         atm(maxatm)
         double precision    atmmss(maxatm)
         double precision    angcoord(3*maxatm)
         double precision    aucoord(3*maxatm)
      end type cartgeom
c
c The MM atomic index, types, typenames, connectivities (valence)       
c
      type mmvalen                                                      
         integer             natoms    
         integer             lprnatmprm
         double precision    atmcharge(maxatm)
         integer             atmid(maxatm) 
         character*4         atmtypname(maxatm)
         integer             atmtyp(maxatm)    
         integer             atmvalen(maxatm)  
         integer             atmvalenlist(maxatm,maxvalen)  
         integer             maxmmopt                                   0510SP11
      end type mmvalen                                                  
c                                 
c The QM/MM data
c                                                                       
c
c The total number, MM host index, atom type, constraint, and bondlenght
c    scale factor for cap atoms
c
      type capatminf
         integer          capnatm
         integer          capatm(maxcapatm)
         integer          capatmtyp(maxcapatm)
         integer          capatmqm(maxcapatm)
         integer          capatmm1(maxcapatm)
         character*20     capatmtypname(maxcapatm)
         character*20     capconstrain                           
         double precision capbondscale(maxcapatm)         
         double precision capbondfix(maxcapatm)    !0106BW10 Q1-link fixed distance
      end type capatminf
c      
c The total number of layers and scale factors for charge scale scheme
c      
      type chargescaleinf
         integer          chargelay
         double precision chargescale(maxchargelay)
      end type chargescaleinf
c
c The total number, index, and point charge of atoms in 1st MM layer
c            
      type m1stlayerinf
         integer          natm1stlay
         integer          atm1stlay(max1stlay)
         integer          atm1stlayqm(max1stlay)
         integer          atm1stlaym2num(max1stlay)
         integer          atm1stlaym2(max1stlay,maxvalen)
         double precision atm1stlaycharge(max1stlay)          
      end type m1stlayerinf 
c
c The total number, index, and point charge of atoms in 2nd MM laye
c
      type m2ndlayerinf
         integer          natm2ndlay
         integer          atm2ndlay(max2ndlay)
         integer          atm2ndlaym1(max2ndlay)
         double precision atm2ndlaycharge(max2ndlay)          
      end type m2ndlayerinf 
c 
c The total number, index, and point charge of atoms in 3rd MM laye
c
      type m3rdlayerinf
         integer          natm3rdlay
         integer          atm3rdlay(max3rdlay)  
         integer          atm3rdlaym2(max3rdlay)   
         double precision atm3rdlaycharge(max3rdlay)
      end type m3rdlayerinf
c
c The total number, values, and target locations for charges at M1 atoms 
c   to be redistibuted.
c Also the total number, values, and target locations for charges used 
c   to preserve M1-M2 dipole, depended on the distane ratio (r/R),
c   where r is the distance between -Q and +Q while R is the distance 
c   between M1-M2.
c M3 atoms only for scale, not involved in redist or shift
c
      type chargeredistinf
         integer          nredist
         double precision chposit
         double precision dipposit
         double precision dipdist
         integer          m1(maxredist)
         integer          m2(maxredist)  
         double precision  q(maxredist)
         double precision qd(maxredist) 
c  qb,qc :  redistributed charges to keep MM region neutral !1123BW08
c  qd2   :  dipole for balance shift2 method  !1123BW09
         double precision qb(maxredist)       !1123BW08
         double precision qc(maxredist)       !1123BW08
         double precision qd2(maxredist)      !0220BW09
         double precision  x(maxredist),  y(maxredist),  z(maxredist)
         double precision mx(maxredist), my(maxredist), mz(maxredist)
         double precision px(maxredist), py(maxredist), pz(maxredist)
      end type chargeredistinf
c
c Total number and index for QM and MM atoms, QM and MM methods,
c   switches for border treatment.
c Note:
c   mmmethod = tinker version
c   mmbas    = force field
c   these two are read in for display, and are not checked for correctness
c
      type qmmminf
         integer          qmnatm, mmnatm
         integer          qmatm(maxatm) 
         integer          mmatm(maxatm)
         integer          mlist(maxatm)
         integer          m3list(maxatm)
         character*20     qmmeth, qmbas
         character*20     mmmeth, mmbas
c
c    add qmmmcutoff variable 8/23/2006 by Yan Zhang
c
         integer          lcutoff
         integer          ccutoffid
         integer          selcent
         integer          cutoffatm(maxatm)
         double precision rcutoff
         double precision cutoffcoor(3)
c
c     end 8/23/2006
c
c     add POLAR variable 8/7/2006 by yan Zhang
c
         character*20     polmeth
         character*20     polmpot
c
         character*80     polparal(103)
         integer          polgid(maxpolgnum, maxpolgidnum)
         integer          polgnum 
         integer          polgidnum(maxpolgnum)
         integer          polcycl
         double precision   polmaxdq
         double precision   polrmsdq
         
c
c     end 8/7/2006 by Yan Zhang
c   QM charge and multplicity add 7/21/2006 by Yan Zhang
c
         integer          qmcharge
         integer          qmmult
c
c     end 7/21/2006 by Yan Zhang
c
         character*80     qmoptl(maxoptl), mmoptl(maxoptl)
         character*20     border
         character*20     embed
         character*20     bordercharge
         type (capatminf)       :: capatm 
         type (chargescaleinf)  :: chscale
         type (m1stlayerinf)    :: mm1stlay
         type (m2ndlayerinf)    :: mm2ndlay
         type (m3rdlayerinf)    :: mm3rdlay
         type (chargeredistinf) :: chredist
c
c  Add by Yan Zhang 9/03/2007
c
c  Variables for flexible-boundary treatment
c
         integer     iflexbound, asqmcharge, asqmmult, maxfbcycl
         integer     fbgid, fbgan(maxgnum), fbgnum
         integer     fbgaid(maxgnum, maxganum) 
c
         character*80   sscp(103) 
         character*20   fbccmeth
         double precision    maxfbdq, rmsfbdq, maxfbdqtot, temprt
c
c  Chang charges in the MM calculation of background charges interaction.
c
         integer    modmmcharg
c
c  End by Yan Zhang 9/03/2007
c
c  add by Bo Wang 0126BW13
c  Damping charge scheme
          integer  ldampchg
          double precision lambda
          integer  dampnatm                  ! number of atoms that needs to be screened
          integer  dampatm(maxatm)           ! the atom id of the screened charge atoms
          double precision dampzeta(maxatm)  ! the zeta value for the screened charge atoms
          double precision dampval(maxatm)   ! the number of valence electrons to be considered in STO orbital
          double precision dampchg(maxatm)   ! the extra charge for the screened charge atoms
          integer ngto1                      ! number of GTO used to fit STO for screened charged 
          integer ngto2                      ! number of GTO used to fit STO for smeared redistributed charges
c  Whether to modify the M1 charge for SS system calculations
          integer lchgcorr
c  Whether to use pseudo F as the link atom
          integer lppf
c  Whether the mm charges are read in the .dat input (only for mmff)
          integer lrdmmchg
c groups for charge balancing
          integer lchgbal                          !whether use balanced schemes
          double precision groupchg(maxbalgroup)   !total charge of group i
          integer balgroup(maxbalgroup,maxatm)     !the atom numbers in group i
          integer balgroupn(maxbalgroup)           !number of atoms in group i
          integer nbalgroup                        !total number of groups
c end by Bo Wang 0126BW13

      end type qmmminf
c                                                                       
c
c
c  The Multilevel optimization options.                                 
c
      type mlopt
         integer  algrthm, iop, htyp, reorient
         integer  llinmn                                                
         character*20  mlmeth, hmeth, hbas
         double precision  dlx2, gcomp, scale, stptol, hscale
         double precision  rmin, rmax, omin, ddmax, ddmaxts                
         integer  hrec, niter, retry, sdstrt, ihunit,iupd
         integer constant(3*maxatm)
c
         character*80  optl(maxoptl)
c
c   Partial atom add by Yan Zhang 8/21/2006
c
c         integer  pcharge, pmult
c         integer  ipartcal
c         integer  partatmnum
c         integer  partatmid(maxidnum)
c
c   modified 2/10/2007 by Christoph Riplinger
c
         type (partial)       :: part 
c
c   end 2/10/2007 by Christoph 
c
c   end by Yan Zhang
c   LBFGS optimization added by Yan Zhang 12/11/2006
c
         double precision rmsgrad, maxstep, minstep, cappa  
         double precision maxslope, maxang, minenr, maxdx, rmsdx
         integer intpolat 
c
c   end by Yan Zhang 12/11/2006
c
c   added by Yan Zhang 02/14/2008
c
         integer frozenatm, frozeatmid(maxatm)
c
c   end by Yan Zhang 02/14/2008
c
      end type mlopt
c
c  External optimization options.
c
      type extopt
         double precision  mmconverg                                    
         character*20  smmconverg    
         character*20  meth, bas
c
         character*80  optl(maxoptl)
      end type extopt
c
c  General section input information.
c
      type general
c
         character*80  mtitle(5)
         character*20  calmodel
         integer  geomunit, geomtype             
         type (cartgeom) :: geom                 
      type (mmvalen) :: mmgeom                                       
         integer  lener, lgrad, lhess, lopt, lextopt
         integer lprsum                                                 
         integer lmmvalen                                               
c
c   add 2/10/2007 by Christoph Riplinger
c
         integer lhessType
         type (partial)       :: part
c
c   end 2/10/2007 by Christoph 
c
      end type general

c
c  add 5/24/2010 by Soroosh
c

c
c dynamic cap atoms
c
      type dynamiccap
         integer          list(maxcapatm), nlist
         integer          atmtyp(maxcapatm), defaulttyp
         character*20     typname(maxcapatm), defaulttypname
         double precision bondscale(maxcapatm), defaultbondscale
      end type dynamiccap
c
c  Parameters for the dynamic simulations
c

      type dynamicsinf
        integer*8 seed
        integer steps, ensemble, nbfreq, trjdumps
        double precision  timestepl, pressure
        double precision  tempcoupling, temperature
        integer firsttimestep, outputfreq, lmmvalen 
        integer restart, fixcom, readpath
        double precision    velocity(3*maxatm), force(3*maxatm)
        double precision  average(4), sumwall, eps
        double precision  vdwcutoff, chargecutoff
        double precision  vdwtaper, chargetaper
c
c Nose Hoover thermostat
c
        double precision  v1, v2, q1, q2, x1, x2
c g1, g2, 
        integer thermostat

c
c annelaing
c
        double precision  initaltemp, finaltemp, deltatemp
        integer       equistep, coolstep, lanneal, periodic
c
c  periodic box, center of mass
c
        double precision comsm(maxatm),com(3*maxatm)
        double precision comvelocity(3), commass, tempnow
        double precision boxsize(3), boxmin(3), boxmax(3), buffersize
        integer molecule(maxatm), maxmolecule
c
c rattle
c
        integer rattlewater(maxatm/3,3), rattle, maxrattle
        integer  rattlemaxiteriation
        double precision rattleangle, rattledistance, rattleeps
        integer rattlehyd(maxatm/2,2), rattleh, maxrattleh, rattleskip
        double precision rattlehdist(maxatm/2), coordold(3*maxatm)
c
c restraints
c
        integer smdatom(maxrestraints,2), restraints
        integer natmcvsmd, ngrpcvsmd, instantmove
        integer ngrpcfsmd, natmcfsmd, pmfatom, pmfgroup
        integer natmrestraints, ngrprestraints
        integer firstatmcv, firstatmcf, firstgrpcf, firstgrpcv
        double precision smd(maxrestraints,4), work, pmfi(3)
        double precision gradforce(3), smoothforce(3)

c
c Partitioning
c
        integer apmethode, activecenter, numberbuffergroupps, selcent
        integer buffergroupps(maxbuffergroupps), ap_atom_zone(maxatm)
        integer group(maxatm), maxgroups
        integer groupatom(maxatm), groupcom, ap_group_zone(maxatm)
        integer groupcharge(maxatm)
c        integer groupsize(maxatm)
        double precision activecentcoord(3), zeroenergy, hzeroenergy
        double precision activeradius, partbuffer
        double precision groupmass(maxatm), sum1
        integer averageactive, averagebuffer, avghl
        integer minbuffer, maxbuffer, maxactive, minactive
        integer userporder, userpcalc, reduceorder
        integer argontest, litest, forceOutAtom
        double precision argonswitch, argoncutoff, switchdist
        double precision shiftFLJ, shiftFM, shiftPLJ, shiftPM
         integer nagroups
c
c zero energy
c
        double precision ape0_1_qm(maxatm), ape0_1_mm(maxatm)
        double precision ape0_2_qm(maxatm), ape0_2_mm(maxatm)
        integer iape0_2(maxatm,2), iape0_3(maxatm,3)
        double precision ape0_3_qm(maxatm), ape0_3_mm(maxatm)
        double precision ape0_4_qm(maxatm/4), ape0_4_mm(maxatm/4)
        integer iape0_4(maxatm/4,4), iape0_5(maxatm/5,5)
        double precision ape0_5_qm(maxatm/5), ape0_5_mm(maxatm/5)
        double precision ape0_6_qm(maxatm/6), ape0_6_mm(maxatm/6)
        integer iape0_6(maxatm/6,6), iape0_7(maxatm/7,7)
        double precision ape0_7_qm(maxatm/7), ape0_7_mm(maxatm/7)
        double precision ape0_8_qm(maxatm/8), ape0_8_mm(maxatm/8)
        integer iape0_8(maxatm/8,8), iape0_9(maxatm/9,9)
        double precision ape0_9_qm(maxatm/9), ape0_9_mm(maxatm/9)
        double precision ape0_10_qm(maxatm/10), ape0_10_mm(maxatm/10)
        integer iape0_10(maxatm/10,10), iape0_11(maxatm/11,11)
        double precision ape0_11_qm(maxatm/11), ape0_11_mm(maxatm/11)
        double precision ape0_12_qm(maxatm/12), ape0_12_mm(maxatm/12)
        integer iape0_12(maxatm/12,12), iape0_13(maxatm/13,13)
        double precision ape0_13_qm(maxatm/13), ape0_13_mm(maxatm/13)
        double precision ape0_14_qm(maxatm/14), ape0_14_mm(maxatm/14)
        integer iape0_14(maxatm/14,14), iape0_15(maxatm/15,15)
        double precision ape0_15_qm(maxatm/15), ape0_15_mm(maxatm/15)



        type (dynamiccap)  :: capatm 
      end type dynamicsinf


c skip lines in the tinker gradient files to speed up 
        integer, parameter :: skipgoutlines=21

c conversion factors to a.u. (m_e=1, e=1, hbar=1, k_B=1)
c Conversion factor for temperature from E_h/k_B to K
        double precision, parameter :: tempconvert=3.1577464d5
c Conversion factor for mass 
c from atomic unit (electron mass) to atomic mass unit u
        double precision, parameter :: massconvert=5.4857990943d-4
c conversion factor for velocity from a_0*E_h/hbar to Ang/fs
        double precision, parameter :: velocityconvert=21.876912633d0
c  conversion factor for force from E_h/a_0 to N
c        double precision, parameter :: forceconvert=8.2387225d-8
c conversion factor for force from E_h/a_0 to uAng/fs^2
        double precision, parameter :: forceconvert=0.496147526d0
c conversion factor for force from nN to uAng/fs^2
        double precision, parameter :: forceconvert2=6.02214149d-3
c conversion factor for force from E_h/a_0 to kcal/(mol Ang)
        double precision, parameter :: forceconvert3=1185.8209516d0
c conversion factor for length from Bohr radius a_0 to Angstrom
        double precision, parameter :: lengthconvert= 0.529177249d0
c  conversion factor for time from hbar/E_h to fs
        double precision, parameter :: timeconvert=2.418884326505d-2
c conversion factor for energy from hartree energy to kcal/mol
        double precision, parameter :: energyconvert= 627.509469d0
c conversion factor for pressure from atm to E_h/a_0^3
        double precision, parameter :: pressureconvert= 290370734.d0
c Boltzmann constant k_B in SI Joule/Kelvin
c        double precision, parameter :: kB=1.3806488d-23
c Boltzmann constant k_B in amu*Angstrom^2/fs^2 /Kelvin
        double precision, parameter :: k_B=8.31446215d-7

c
c   end 5/24/2010 by Soroosh 
c


c
c
c  The information structure utilized by optehk, optghk, and opthhk to
c    make their call to the appropriate Multilevel method.
c
      type mloptinf
         type (proglist)  :: progrms
         type (cartgeom)  :: geom
         type (mmvalen)   :: mmgeom                                     0222HL04
         character*20     :: mlmeth
c
         character*80     :: mcoptl(maxoptl), toptl(maxoptl)
         type (qmmminf)   :: qmmminp                                    0222HL04     
         integer          :: htyp
         character*20     :: hbas, hmeth, tbas, tmeth
         double precision :: hscale
      end type mloptinf
      type (mloptinf) :: mlinp
      save  mlinp
      end module input



c
c
c******************************************************************************
c     MODULE:FILES
c******************************************************************************
c
      module files
c
c     Created:  Feb. 21, 2005 by Hai Lin
c     Revised:  Apr.  3, 2006 by Yan Zhang
c               May  22, 2006 by Yan Zhang
c               Aug.  3, 2006 by Yan Zhang
c               June 14, 2010 by Soroosh Pezeshki
C
c     Purpose:  To define all the files and filehandles to be used
c       throughout QMMM. 
c
      implicit none 
      integer, parameter :: infile = 5, outfile = 6
      integer, parameter :: g03inf = 17, g03outf = 18                    
      integer, parameter :: t41inf = 71, t41outf = 72                    
      integer, parameter :: t41keyf = 73, t41crdf = 74                   
      integer, parameter :: extoinf = 7, extoout = 8
      integer, parameter :: fchk = 19, fchk2 = 20                        
      integer, parameter :: sumfile = 25                                 
      integer, parameter :: moldenfile = 26                              
      
c
c    orca files 4/3/2006 by Yan Zhang
c
      integer, parameter :: orcainf=75, orcaoutf=76
      character*8,  parameter :: orcain='orca.inp'
      character*8,  parameter :: orcaout='orca.out'
      integer, parameter :: ogradn=77
      character*11,  parameter :: ogradf='orca.engrad'
      integer, parameter :: ohessn=78
      character*11,  parameter :: ohessf='orca.hess'
      integer, parameter :: ogradpcn=79
      character*11,  parameter :: ogradpcf='orca.pcgrad'
      integer, parameter :: outoon=80
      character*11,  parameter :: outoof='extopt.xyz'

c
c   end   4/3/2006  by Yan Zhang
c
c   add 2/10/2007 by Christoph Riplinger
c   the numerical hessian is not symmetric, so we need to store the full hessian
c   to transfer it to orca, we'll do this in the orca hess-file format (.hess)
c
        integer, parameter :: orcawhessn=86
        character*18,  parameter :: orcawhessf='hessForExtopt.hess'
c
c   end 2/10/2007 by Christoph 
c
c
c     Gamess files 5/22/2006 by Yan Zhang
c
      integer, parameter :: gmsinf=81, gmsoutf=82
      character*8,  parameter :: gmsin='gms.inp'
      character*8,  parameter :: gmsout='gms.out'
      integer, parameter :: gmsdatf=83
      character*8,  parameter :: gmsdat='gms.dat'
      integer, parameter :: extoptdatf=84
      character*11,  parameter :: extoptdat='extopt.dat'
c
c   end   5/22/2006  by Yan Zhang
c
c    charge file 8/3/2006   by Yan Zhang
c
      integer, parameter :: chargfile=85
      character*8,  parameter :: chargoutf='ml.charg'
c
c     end 8/3/2006  by Yan Zhang
c
c  Partial atoms ID. add by Yan Zhang 10/29/2007
c
      integer, parameter :: paidf = 86              
      character*9, parameter :: paid = 'partatmid'
c
c  end by Yan Zhang 10/29/2007                   
c 

c
c  dynamics trajectory file 06/14/2010 by Soroosh Pezeshki
c
      integer, parameter :: trajfile =9786                               
      character*14, parameter :: trajfilename ='trajectory.arc'
      integer, parameter :: restartfile =9787                               
      character*16, parameter :: restartfilename ='dynamics.restart'
      integer, parameter :: groupfile =9788                               
      character*16, parameter :: groupfilename ='groups.log'
      integer, parameter :: restraintsfile =9789                               
c      character*16, parameter :: restraintsfilename ='restraints.log'
      integer, parameter :: pathfile =9790                               
      integer, parameter :: restartin =9791                               
      integer, parameter :: smdf =9792
      integer, parameter :: smddummy =9793
      character*16, parameter :: smdfilename ='smd.log'
      character*16, parameter :: smddummyname ='smdDummy.arc'
c
c end dynamics trajectory file 06/14/2010 by Soroosh Pezeshki
c

      character*6, parameter :: mlin = 'ml.inp'
      character*6, parameter :: mlout = 'ml.out'
      character*7, parameter :: g03in = 'g03.inp'                        
      character*7, parameter :: g03out = 'g03.out'                       
      character*7, parameter :: t41in = 't41.xyz'                        
      character*7, parameter :: t41key = 't41.key'                       
      character*7, parameter :: t41out = 't41.out'                       
      character*7, parameter :: t41crd = 't41.crd'                       
      character*10, parameter :: extin = 'extopt.inp'
      character*10, parameter :: extout = 'extopt.out'
      character*6, parameter :: mlsum = 'ml.sum'                         
      character*10, parameter :: molden5 = 'molden.xyz'                   
      
      character*9, parameter :: testfchk = 'Test.FChk'
      character*30, parameter :: pDZsloc = './pdz+.gbs'
      character*30, parameter :: g3Lloc = './g3large.gbs'
      character*30, parameter :: mg3loc = './mg3.gbs'
      character*30, parameter :: mg3sloc = './mg3s.gbs'                  
      end module files
c
c
c******************************************************************************
c     MODULE:PARAM 
c******************************************************************************
c
      module param
c 
c     Created:  Feb. 21, 2005 by Hai Lin
c     Revised:  Apr. 5,  2006 by Yan Zhang
c               May  7,  2009 by Hai Lin
c 
c     Purpose:  To define parameters for the methods and bases used. 
c
      implicit none
c
c  5 chosen to represent saddle point to follow Polyrate standard
c
      integer, parameter :: linear=1, nonlin=2, lints=5, nonlints=6
c
c     4/5/2006 by Yan Zhang
c     5/3/2009 by Hai Lin
c 
c
c   modified 2/10/2007 by Christoph Riplinger
c
c      integer, parameter :: uniths = 0, qmmmhs=1, mmhs=2, lowqmhs=3
      integer, parameter :: uniths = 0, qmmmhs=1, mmhs=2, lowqmhs=3,
     *     hybridhs=4, numhs=5, esths=6
c
c   end 2/10/2007 by Christoph 
c
c    end 4/5/2006 by Yan Zhang 
c
c Note by Hai Lin, 5/5/2009:
c
c   Please set the maxatm here to the same as in the module input and gradient
c
c end note by Hai Lin, 5/5/2009
c
      integer, parameter :: maxatm = 9900

      integer, parameter :: nodef = 0
      integer, parameter :: def = 1
      integer, parameter :: na   =  0
      integer, parameter :: nstd = -1
      integer, parameter :: ptz  = 1
      integer, parameter :: pdz  = 2
      integer, parameter :: pdzs = 3
      integer, parameter :: dz   = 4
      integer, parameter :: numes = 6
      integer, parameter :: numbas = 3
      integer, parameter :: nummc = 6
 
      integer, parameter :: numcalc = 44
c  f___ indicates a formatted list for printing output
      character*20 esmlist(-1:6), fesmlist(-1:6)
      character*20 baslist(-1:4), fbaslist(-1:4)
      data esmlist /'Non-Std. ','n/a ','ccsd(t) ','mp4 ','ccsd ',
     *                 'mp4sdq ','mp2 ','hf '/
      data fesmlist /'Non-Standard','N/A','CCSD(T)','MP4','CCSD',
     *                 'MP4SDQ','MP2','HF'/
      data baslist /'Non-Std. ','n/a ','cc-pvtz ','cc-pvdz ',
     *                 'pdz+ ','6-31+g(d,2p) '/
      data fbaslist /'Non-Standard','N/A','cc-pVTZ','cc-pVDZ',
     *                 'pDZ+','6-31+g(d,2p)'/
      end module param
c
c
c******************************************************************************
c     MODULE:ENERGY
c******************************************************************************
c
      module energy
c 
c     Created:  Feb. 21, by Hai Lin 
c     Revised:  
c     Purpose:  To create common block for all the energies calculated by
c        Multilevel and gathered from the electronic structure program.
c
      implicit none
      integer, parameter, private :: numes    = 6
      integer, parameter, private :: numbas   = 3
      integer, parameter, private :: numcalc  = 44
      double precision esener(numes, numbas)
      double precision qmmmener, qmes(1), mmes(3)                       
      COMMON /ENER/ esener, qmmmener, qmes, mmes                                             0222HL04
      end module energy
c
c
c******************************************************************************
c     MODULE:ATMINFO
c******************************************************************************
c
      module atminfo
c 
c     Created:  Feb. 22, 2005 by Hai Lin
C
c     Purpose:  To define all the atomic masses and names to be used in
c        reading the geometry in from the infile and interpreting it.
c
c     Called by:  rgeom, rmmgeom
c
      implicit none
      integer, private :: a
      double precision ams(103)
      character*2 asymb(103)
      character*2 asymbu(103)
c
c array of atomic masses
c
      data (ams(a),a=1,54)  /
     *   1.007825d+00,4.0026d+00,7.01600d+00,9.01218d+00,11.00931d+00,
     *   12.0d+00,14.00307d+00,15.99491d+00,18.99840d+00,19.99244d+00,
     *   22.9898d+00,23.98504d+00,26.98153d+00,27.97693d+00,
     *   30.97376d+00,31.97207d+00,34.96885d+00,39.948d+00, 
     *   38.96371d+00,39.96259d+00,44.95592d+00,47.90d+00,50.9440d+00,
     *   51.9405d+00,54.9381d+00,55.9349d+00,58.9332d+00,57.9353d+00, 
     *   62.9298d+00,63.9291d+00,68.9257d+00,73.9219d+00,74.9216d+00, 
     *   79.9165d+00,78.9183d+00,83.9115d+00,
     *   84.9117d+00,87.9056d+00,89.9054d+00,89.9043d+00,92.9060d+00,
     *   97.9055d+00,97.0d+00,101.9037d+00,102.9048d+00,105.9032d+00,
     *   106.9041d+00,113.9036d+00,114.9041d+00,119.9022d+00,
     *   120.9038d+00,129.9067d+00,126.9044d+00,131.9042d+00/
      data (ams(a),a=55,103)  /
     *   132.9054d+00,137.9052d+00,138.9063d+00,139.9054d+00,
     *   140.9076d+00,141.9077d+00,144.9127d+00,151.9197d+00,
     *   152.9212d+00,157.9241d+00,158.9253d+00,163.9292d+00,
     *   164.9303d+00,165.9303d+00,168.9342d+00,173.9389d+00,
     *   174.9408d+00,179.9465d+00,180.9480d+00,183.9509d+00,
     *   186.9557d+00,191.9615d+00,192.9629d+00,194.9648d+00,
     *   196.9665d+00,201.9706d+00,
     *   204.9744d+00,207.9766d+00,208.9804d+00,208.9824d+00,
     *   209.9871d+00,222.0176d+00,
     *   223.0197d+00,226.0254d+00,
     *   227.0278d+00,232.0381d+00,231.0359d+00,238.0508d+00,
     *   237.0482d+00,244.0642d+00,243.0614d+00,247.0703d+00,
     *   247.0703d+00,251.0796d+00,252.0829d+00,257.0751d+00,
     *   258.0986d+00,259.1009d+00,260.1053d+00/
c
c array of atomic symbols
c
      data (asymb(a),a=1,103)  /
     *   'h ','he','li','be','b ','c ','n ','o ','f ','ne',
     *   'na','mg','al','si','p ','s ','cl','ar',
     *   'k ','ca','sc','ti','v ','cr','mn','fe','co','ni','cu',
     *        'zn','ga','ge','as','se','br','kr',
     *   'rb','sr','y ','zr','nb','mo','tc','ru','rh','pd','ag',
     *        'cd','in','sn','sb','te','i ','xe',
     *   'cs','ba','la','ce','pr','nd','pm','sm','eu','gd','tb',
     *        'dy','ho','er','tm','yb','lu','hf','ta','w ','re',
     *        'os','ir','pt','au','hg','tl','pb','bi','po','at','rn', 
     *   'fr','ra','ac','th','pa','u','np','pu','am','cm','bk',
     *        'cf','es','fm','md','no','lr'/
c
c array of atomic symbols in upper case
c
      data (asymbu(a),a=1,103)  /
     *   'H ','HE','LI','BE','B ','C ','N ','O ','F ','NE',
     *   'NA','MG','AL','SI','P ','S ','CL','AR',
     *   'K ','CA','SC','TI','V ','CR','MN','FE','CO','NI','CU',
     *        'ZN','GA','GE','AS','SE','BR','KR',
     *   'RB','SR','Y ','ZR','NB','MO','TC','RU','RH','PD','AG',
     *        'CD','IN','SN','SB','TE','I ','XE',
     *   'CS','BA','LA','CE','PR','ND','PM','SM','EU','GD','TB',
     *        'DY','HO','ER','TM','YB','LU','HF','TA','W ','RE',
     *        'OS','IR','PT','AU','HG','TL','PB','BI','PO','AT','RN',
     *   'FR','RA','AC','TH','PA','U','NP','PU','AM','CM','BK',
     *        'CF','ES','FM','MD','NO','LR'/
      end module atminfo
c
c
c******************************************************************************
c     MODULE:GRADIENT
c******************************************************************************
c  
      module gradient
c
c     Created:  Feb. 21, 2005 by Hai Lin
c     Revised:  May   7, 2009 by Hai Lin
c
c     Purpose:  To create common block for all the gradients calculated by
c        Multilevel and gathered from the electronic structure program.
c
c Note by Hai Lin, 5/5/2009:
c  
c         1. Set the maxatm to the same as that in the module input.
c         2. Set maxgrad by adding 6*maxcapatom for additional point 
c            charges. As maxcapatm = 30, we add 180.
c 
c end note by Hai Lin, 5/5/2009
c 
      implicit none
      integer, parameter, private :: numes    = 6
      integer, parameter, private :: numbas   = 3
      integer, parameter, private :: numcalc  = 44
      integer, parameter, private :: maxatm   = 9900                      
      integer, parameter, private :: maxgrad  = 3*maxatm+540                
      double precision esgrad(numes, numbas,maxgrad)
      double precision qmmmgrad(maxgrad), qmmmqmgr(maxgrad),
     *                 qmmmmmgr(3,maxgrad), totalgrad(3*maxatm)
c                                                                       
c Note: qmmmqmgr(:)   is for QM (CSS) or QM* (all)
c       qmmmmmgr(1,:) is for MM (all)
c       qmmmmmgr(2,:) is for MM (CSS)
c       qmmmmmgr(3,:) is for MM*-MM (all)
c
      COMMON /GRAD/ esgrad, 
     * qmmmgrad, qmmmqmgr, qmmmmmgr, totalgrad                        
     
      end module gradient
c     
c
c******************************************************************************
c     MODULE:HESSIAN
c******************************************************************************
c
      module hessian
c
c     Created:  Feb. 21, 2005 by Hai Lin
c     Revised:  May   7, 2009 by Hai Lin
c
c     Purpose:  To create common block for all the hessians calculated by
c        Multilevel and gathered from the electronic structure program.
c
      implicit none
 
      integer, parameter, private :: numes    = 6
      integer, parameter, private :: numbas   = 3
      integer, parameter, private :: numcalc  = 44
c
c Note by Hai Lin, 5/4/2009
c
c      1. The maximum number for Hessian calculation differs from
c         the maxatm defined in general for energy and gradient
c      2. The maximum number for Hessian calculation should be the 
c         same for module optimization.
c 
       integer, parameter :: hessmaxatm = 800                           0725XW18
       integer, parameter, private :: maxhess  = 3*hessmaxatm
c
c end by Hai Lin, 5/4/2009
c
      double precision eshess(numes, numbas,maxhess, maxhess)
      double precision qmmmhess(maxhess,maxhess), 
     *   qmmmqmhs(maxhess,maxhess), qmmmmmhs(3,maxhess,maxhess)
c                                                                       
c Note: qmmmqmhs(:,:)   is for QM (CSS) or QM* (all) 
c       qmmmmmhs(1,:,:) is for MM (all)
c       qmmmmmhs(2,:,:) is for MM (CSS)
c       qmmmmmhs(3,:,:) is for MM*-MM (all)
c
	   
      COMMON /HESS/ eshess, 
     * qmmmhess, qmmmqmhs, qmmmmmhs                                     
     
      end module hessian
c
c
c******************************************************************************
c     MODULE:CALTYP
c******************************************************************************
c
      module caltyp
c
c     Created:  Feb 22, 2005 by Hai Lin
c     Revised:  Jun. 28, 2010 by Soroosh Pezeshki
c
c     Purpose:  To indicate the type of calculation being called in a
c        program hook or the type of calculation being displayed by a disp
c        subroutine 
c
      implicit none
      integer, parameter :: tener = 1
      integer, parameter :: tgrad = 2
      integer, parameter :: thess = 3
      integer, parameter :: topt  = 4
c
c   add 2/10/2007 by Christoph Riplinger
c   extopt: external optimization, testhess: estimated hessian
      integer, parameter :: textopt  = 5
      integer, parameter :: testhess  = 6
c
c   end 2/10/2007 by Christoph 
c
c     To read MM parameter (point charge)                               
c
      integer  tprm 
      COMMON /readprm/ tprm
c
c     To write special MM keywords from MM options started with '!'     
c
      integer  tcssmmkey
      COMMON /cssmmkey/ tcssmmkey
c
c     To debug                                                          
c
      integer  tdebug
      COMMON /dodebug/ tdebug
      
c
c  add by Soroosh Pezeshki June 28, 2010
c  tdynamics: run dynamics, don't repeat output every step
c  time the single steps
c  ttiming: meassure calculation time
c  tap: do adaptive partitioing
c
      integer  tdynamics, ttiming, tap, tesp
      COMMON /dodynamics/ tdynamics
      COMMON /doap/ tap
      COMMON /dotiming/ ttiming
      COMMON /doesp/ tesp
c
c semi empirical calculation in orca can't be run with basis set in the input file
c
      integer  tsemiempirical
      COMMON  tsemiempirical

c
c end add by Soroosh Pezeshki
c

      end module caltyp
c
c
c******************************************************************************
c     MODULE:OPTIMIZE
c******************************************************************************
c
      module optimize
c
c     Created:  Feb. 21, 2005 by Hai Lin
c     Revised:  May   5, 2009 by Hai Lin
c
c     Purpose:  To create a common block for the information necessary in
c        the optimization routines (which it is too complicated to pass
c        around as an argument 
c
c Note by Hai Lin, 5/5/2009:
c
c     1. The maxatm here should be the same as in module hessian.
c     2. The maxatm here is not necessaily the same as in modules 
c        input, gradient, and param
c
c end note by Hai Lin, 5/5/2009
c  
      implicit none
      integer, parameter, private :: maxatm = 500
      integer, parameter, private :: n3tm = 3*maxatm
 
      double precision :: ener, grad(n3tm), hess(n3tm,n3tm)
      double precision :: xcom(n3tm), xicom(n3tm), lutiny, cutoff
      COMMON /OPT/  ener, grad, hess, xcom, xicom, lutiny, cutoff
      end module optimize
c
c******************************************************************************
c     MODULE:EHARD
c******************************************************************************
c
      module ehard
c 
c     Created:  Jun. 19, 2006 by Yan Zhang
c     Revised:  May   7, 2009 by Hai Lin
c
c     Purpose:  To define the QEQ and EEM atomic parameters to be used in
c              the calculations of the polarized background charge.
c 
c     Note:
c           Set the maxeeatm to be the same as maxatm in module input
c
c
c     Called by:  rgeom, rmmgeom
c
      implicit none
c
c Hai Lin, 5/4/2009
c
c     Please set this number to the same as the maximum number of atoms defined
c     in the module input
c
      integer, parameter, private :: maxeeatm  = 9900                
c
c  by Hai Lin, 5/4/2009
c

      integer, private :: a

      character*2 qeqag(16)
      double precision qeqeg(16), qeqhg(16) 

      character*2 qeqat(4)
      double precision qeqet(4), qeqht(4) 

      character*2 eemab(5)
      double precision eemeb(5), eemhb(5) 

      character*2 eemam(4)
      double precision eemem(4), eemhm(4) 
c
c array of QEQ atomic parameters(J.Phys.Chem.1991,95,3358-3363)
c
      data (qeqag(a),a=1,16)  /
     *   'li','c ','n ','o ','f ','na','si','p ','s ','cl',
     *   'k ','br','rb','i ','cs','h '/
      data (qeqeg(a),a=1,16)  /
     *   3.006d+00,5.343d+00,6.899d+00,8.741d+00,10.874d+00,
     *   2.843d+00,4.168d+00,5.463d+00,6.928d+00,8.564d+00,
     *   2.421d+00,7.790d+00,2.331d+00,6.822d+00,2.183d+00,
     *   4.528d+00/ 
      data (qeqhg(a),a=1,16)  /
     *   4.772d+00,10.126d+00,11.760d+00,13.364d+00,14.948d+00,
     *   4.592d+00,6.974d+00,8.000d+00,8.972d+00,9.892d+00,
     *   3.840d+00,8.850d+00,3.692d+00,7.524d+00,3.422d+00,
     *   13.8904d+00/
c
c array of QEQ atomic parameters(J. comput. chem. 1995 17 87-108)
c
      data (qeqat(a),a=1,4)  /
     *   'c ','n ','o ','h '/
      data (qeqet(a),a=1,4)  /
     *   5.07305d+00,7.73699d+00,8.27885d+00,4.42211d+00/ 
      data (qeqht(a),a=1,4)  /
     *   10.06444d+00,12.96908d+00,14.93241d+00,13.84036d+00/
c
c array of EEM atomic parameters(J.Phys.Chem.A 2004,108,10359-10366)
c
      data (eemab(a),a=1,5)  /
     *   'h ','c ','n ','o ','f '/
      data (eemeb(a),a=1,5)  /
     *   1.00d+00,1.77d+00,14.80d+00,29.46d+00,62.84d+00/
      data (eemhb(a),a=1,5)  /
     *   20.57d+00,9.24d+00,10.74d+00,15.85d+00,46.05d+00/
c
c array of EEM atomic parameters(J. Phys. Chem. A 108 2004 10359-10366)
c
      data (eemam(a),a=1,4)  /
     *   'h ','c ','n ','o '/
      data (eemem(a),a=1,4)  /
     *   1.00d+00,2.30d+00,7.20d+00,5.1d+00/
      data (eemhm(a),a=1,4)  /
     *   13.8d+00,9.1d+00,13.2d+00,11.1d+00/

      end module ehard

c******************************************************************************
c     MODULE:DAMPING
c******************************************************************************
c
      module damping
c
c     Created:  June 16, 2014 by Bo Wang
c
c     Purpose:  To define the default parameters used in ODS screening
c
c     Called by:
c
      implicit none
c

      integer, private :: index
      double precision :: scnzeta(36), scnval(36)

      data (scnzeta(index), index=1,36) /
     &     1.32,-1.0,-1.0,-1.0,0.72,0.92,0.92,1.20,1.16,-1.0,
     &     -1.0,-1.0,-1.0,0.73,0.68,0.90,0.98,-1.0,-1.0,-1.0,
     &     -1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,-1.0,
     &     -1.0,0.83,0.88,0.95,1.01,-1.0/

      data (scnval(index),index=1,36) /
     &     1.0,0.0,0.0,0.0,1.0,1.0,1.0,1.0,1.0,0.0,
     &     0.0,0.0,0.0,1.0,1.0,1.0,1.0,0.0,0.0,0.0,
     &     0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,
     &     0.0,1.0,1.0,1.0,1.0,0.0/

      end module damping

