c
c******************************************************************************
c  QMMMNUMHHK
c******************************************************************************
c
      subroutine qmmmnumhhk(qmprognme, mmprognme, geom,mmgeom,
     *       qmmminp, optinp, part, ipartcal, ierr)
c
c     The subroutine calculates QMMM energy and gradient and the 
c     numerical QMMM hessian. So far it is only available for a 
c     single point calculation and optimization with external optimizer
c
c     Subroutine qmmmnumhhk
c
c     Created: Oct 02, 2007 by Christoph Riplinger
c     Revised: May 07, 2009 by Hai Lin
c
c     Purpose: To determine the QM/MM energy, gradient and numerical hessian with all
c              available QM packages.
c
c     Called by: mlhhook
c     Calls:  qmmmghk, orcawritehess, mtoltm0
c
      use caltyp
c
c Hai Lin, 5/4/2009
c
c      use input,  only   : cartgeom, mmvalen, qmmminf, mlopt, mlinp,
c     *                     partial
      use input,  only   : cartgeom, mmvalen, qmmminf, mlopt, mlinp,
     *                     partial, maxatm
c
c end by Hai Lin, 5/4/2009
c
      use files   
      use energy, only   : qmmmener, qmes, mmes
      use gradient, only : qmmmgrad, qmmmqmgr, qmmmmmgr
c
c modified by Yan zhang 11/2/2007
c      use hessian, only  : qmmmhess
c
      use hessian, only  : qmmmhess, hessmaxatm                  
c
c end 11/1/2007 by Yan Zhang
      implicit none
c
c i/o variables
c
      character*10       qmprognme, mmprognme
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (qmmminf)  :: qmmminp
      type (mlopt)    :: optinp
      type (partial)  :: part
      integer            ierr, ipartcal
c
c local variables
c      
      type (cartgeom) :: ssgeom, bgchgeom
      type (mmvalen)  :: ssmmgeom
      integer            lsnatm, ssnatm, bgchnum
      integer            sumtyp
      integer            i, j, k, l, hessnatm 
      integer            n3tesnatm, actcoordid
      integer            esnatm
      integer            actatomid
      integer            acthesscoord,secatomid,sechesscoord,seccoordid
c
c   DX is the displacement for the numerical hessian calculation
c
      double precision, parameter  :: DX = 0.005
      double precision, parameter  :: angtoau = 0.529177249d0
c        please check module.F:hessian:maxatm value
c
c modified by Yan zhang 11/1/2007
c      integer, parameter :: hessmax = 100*3
c      integer, parameter :: hessmaxnatm = 100
      integer, parameter :: hessmax = hessmaxatm*3          
      integer, parameter :: hessmaxnatm = hessmaxatm
c
c end by Yan zhang 11/1/2007   
c
c        please check module.F:input:maxatm value
c
c Hai Lin, 5/4/2009
c
c      integer, parameter :: inputmax = 9900*3
c      integer, parameter :: inputnatm = 9900
      integer, parameter :: inputmax = maxatm*3 
      integer, parameter :: inputnatm = maxatm 
c
c
c end by Hai Lin, 5/4/2009
c
      integer               hessatmid(hessmaxnatm)
      double precision      savegeomaucoord(inputmax)
      double precision      savegeomangcoord(inputmax)
      double precision      initialgrad(inputmax)
      double precision      displgrad(inputmax)
      double precision      hs(hessmax*(hessmax+1)/2)
      double precision, dimension (:,:), allocatable :: tmpHess
c
c  Save the prnprm switch value
c
      ssmmgeom%lprnatmprm = mmgeom%lprnatmprm
c
c  number of atoms in the system
c
      esnatm = mmgeom%natoms
c
c   save the coordinates of the input geometry
c
      n3tesnatm = 3*esnatm
      savegeomaucoord(:n3tesnatm) = geom%aucoord(:n3tesnatm)
      savegeomangcoord(:n3tesnatm) = geom%angcoord(:n3tesnatm)
c
c   determine the size of the hessian
c
      hessnatm = mmgeom%natoms
      if(ipartcal .eq. 1) then
          hessnatm = part%partatmnum
      endif
c
c   ml.f should prevent that this happens, but nevertheless...
c
      if(hessnatm .gt. hessmaxnatm) then
       write(outfile,*) 'The program can not calculate',
     $  ' the QM/MM hessian with more than ', hessmaxnatm, ' atoms.'
       stop 'qmmmnumhhk 1'
      endif
c
c   determine which atoms have to be displaced
c
      if(ipartcal .eq. 0) then
          do i=1, hessnatm
              hessatmid(i) = mmgeom%atmid(i)
          end do
      else if(ipartcal .eq. 1) then
          do i=1, hessnatm
              hessatmid(i) = part%partatmid(i)
          end do
      endif
c
c   calculate the gradient at input geometry
c
      call qmmmghk(qmprognme, mmprognme, geom, mmgeom,
     *            qmmminp, ierr)
      initialgrad(:n3tesnatm) = qmmmgrad(:n3tesnatm)
c
c  Calculate the QMMM energy for the input geometry
c
      qmmmener = qmes(1) - mmes(1) + mmes(2)
      if (tdebug .eq. 1) then
          write (outfile,116)
          write (outfile,117)
          write (outfile,118) initialgrad(:n3tesnatm)
          write (outfile,116)
      endif
c
c   loop over all atoms for which we want to have the hessian
c
      do i = 1, hessnatm
        actatomid = hessatmid(i)
c
c   loop over coordinates
c
        do j = -2, 0
c
c   which coordinate to displace
c   acthesscoord: only the subset of the hessian atoms
c   actcoordid: corresponding ID of acthesscoord among all atoms
c
          acthesscoord = 3*i + j
          actcoordid = 3*actatomid + j
          write (outfile,119) actatomid, j+3
c
c   take the input geometry and displace the respective coordinate
c
c   first in the coordinate vector in au
          geom%aucoord(:n3tesnatm) = savegeomaucoord(:n3tesnatm)
          geom%aucoord(actcoordid) = geom%aucoord(actcoordid) + DX
c   then in the coordinate vector in Angstrom
          geom%angcoord(:n3tesnatm) = savegeomangcoord(:n3tesnatm)
          geom%angcoord(actcoordid) = 
     *        geom%angcoord(actcoordid) + (DX*angtoau)
c
c    calculate the gradient for this geometry and save it
c    in the displaced gradient vector
c
          tprm=1
          call qmmmghk(qmprognme, mmprognme, geom, mmgeom,
     *            qmmminp, ierr)
          displgrad(:n3tesnatm) = qmmmgrad(:n3tesnatm)
            if (tdebug .eq. 1) then
                write (outfile,116)
                write (outfile,120)
                write (outfile,118) displgrad(:n3tesnatm)
                write (outfile,116)
            endif
c
c    now calculate the hessian entry for this coordinate
c    (the respective column of the hessian)
c    for this purpose we only need the entries of the
c    gradient defined in hessatmid
c
          do k = 1, hessnatm
            secatomid = hessatmid(k)
            do l = -2, 0
              sechesscoord = 3*k + l
              seccoordid = 3*secatomid + l
c
c    the numerical hessian is defined as 
c    H(i,j) = [grad(i)_input(j+DX) - grad(i)_input]/DX
c
              qmmmhess(sechesscoord,acthesscoord) = 
     *             (displgrad(seccoordid)-initialgrad(seccoordid))/DX
            end do
          end do
        end do
      end do
c
c    store the qmmmhess in a matrix tmpHess with dimension 3*hessnatm x 3*hessnatm
c
      allocate (tmpHess(3*hessnatm, 3*hessnatm))
      tmpHess(:3*hessnatm,:3*hessnatm)=qmmmhess(:3*hessnatm,:3*hessnatm)
c
c    store the hessian in the orca-hess file-format. We must do this, because
c    the obtained hessian is not symmetrical, so the lower triangular hessian is 
c    eventually not sufficient for an optimization
c
      open (unit=orcawhessn,status='unknown',file=orcawhessf,err=9994)
      call orcawritehess(orcawhessn, 3*hessnatm, tmpHess, geom, part,
     *                    ipartcal)
      close (orcawhessn, err=9994)
c
c    set the coordinates back to the input geometry
c
      geom%aucoord(:n3tesnatm) = savegeomaucoord(:n3tesnatm)
      geom%angcoord(:n3tesnatm) = savegeomangcoord(:n3tesnatm)
c
c    debug-output
c
      if (tdebug .eq. 1) then
          write (outfile,116)
          write (outfile,121)
          call mtoltm0 (i, qmmmhess(:i,:i), hs(:i*(i+1)/2))
          write (outfile,122) hs(:i*(i+1)/2)
          write (outfile,116)
      endif
c
c    set the gradient back to the gradient at input geometry
c
      qmmmgrad(:n3tesnatm)=initialgrad(:n3tesnatm)
c
c    We still have to display our results
c
c    Energy first
c
      write(outfile,9000)
      write(outfile,9100) qmmmener, qmes(1), mmes(1), mmes(2)
c
c    Display gradient and hessian results
c
      write(outfile,100)
      write(outfile,9200)
      write(outfile,9210) qmmmgrad(:n3tesnatm)
      write(outfile,9220)
      i = hessnatm*3
      call mtoltm0 (i, qmmmhess(:i,:i), hs(:i*(i+1)/2))
      write(outfile,9210) hs(:i*(i+1)/2)
      return
100   format(80('-'))
116   format(/,10('Debug'))
117   format(2x,'Gradient at input geometry:')
118   format(3(d20.10))
119   format(/,'Displacing atom',/2x,i4,/2x,'coordinate',/2x,i4)
120   format(2x,'Gradient at displaced geometry:')
121   format(2x,'Numerical Hessian:')
122   format(4(d20.10))
9000  format(/,'Display of QMMM energy (a.u.):',/2x,'QMMM      ',
     *   10x,'QM;CPS**',10x,'MM;CPS*',10x,'MM;ES') 
9100  format(4(f16.10,2x))      
9200  format(/,'Display of QMMM gradient (a.u.):')
9210  format(3(e20.12))
9220  format(/,'Display of QMMM hessian (a.u.):')
9994  write (outfile,*) 'Error creating orca hess-file'
      stop 'qmmmnumhhk 1'
      end
c
c******************************************************************************
c  QMMMHHK
c******************************************************************************
c
      subroutine qmmmhybridhhk(qmprognme, mmprognme, geom,
     *   mmgeom, qmmminp, optinp, part, ipartcal, ierr)
c
c     The subroutine calculates the QM hessian 
c     and an Almloef-hessian with orca and merges both
c
c
c     Subroutine orcaqmmmapprhhk
c
c     Created: Oct 02, 2007 by Christoph Riplinger
c
c     Purpose: To determine the hybrid hessian with orca
c
c     Called by: mlhhook
c     Calls:  qmmmssgeom, bgchscale, bgchshift, qmmmbgchgeom, polareech, orcastinp,
c             orcaouth, mtoltm0, orcapartinp, orcainp, orcawritehess ,qmmmghk
c
      use caltyp
      use input,  only   : cartgeom, mmvalen, qmmminf, mlopt, 
     *                     partial,maxoptl
      use files
      use energy, only   : qmmmener, qmes, mmes
      use gradient, only : qmmmgrad, qmmmqmgr, qmmmmmgr
c
c modified by Yan zhang 11/1/2007
c      use hessian, only  : qmmmqmhs, qmmmhess
c 
      use hessian, only  : qmmmqmhs, qmmmhess, hessmaxatm                
c
c end by Yan zhang 11/1/2007
c
      use caltyp,  only  : testhess
      implicit none
c
c i/o variables
c
      character*20       qmprognme, mmprognme, ver_lab
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (qmmminf)  :: qmmminp
      type (mlopt)    :: optinp
      type (partial)  :: part
      integer            ierr, sizemergedhess, partext
      integer            partextatmnum, ipartcal
c
      character*80       optl(maxoptl)
c
c local variables
c      
      character*20       esmeth, esbas
      type (cartgeom) :: ssgeom, bgchgeom
      type (mmvalen)  :: ssmmgeom
      integer            lsnatm, ssnatm, bgchnum, fullhessnatm
      integer            i, j, k, l, m, n, o, p
      integer            partnatm, qmnatm, capnatm, cssnatm
      integer            qmcharge, qmmult
c
c        please check module.F:hessian:maxatm value
c  modified by Yan zhang 11/1/2007
c      integer, parameter :: hessmax = 100*3
c      integer, parameter :: hessmaxnatm = 100
c
      integer, parameter :: hessmax = hessmaxatm*3
c
c end by Yan zhang 11/1/2007
c            
      double precision  cpshess(hessmax,hessmax)
c
c   fullhess might be bigger than qmmmhess for the partial hessian, since it includes 
c   the atoms that are bonded to the partial atoms
c
      double precision  fullhess(2*hessmax,2*hessmax)
      double precision  hs(hessmax*(hessmax+1)/2)
      double precision, dimension (:,:), allocatable :: tmpHess
c
c   Save the prnprm switch value
c
      ssmmgeom%lprnatmprm = mmgeom%lprnatmprm
c
c   We take the method and basis from the QM input for calculating the CPS hessian
c
      esmeth = qmmminp%qmmeth
      esbas = qmmminp%qmbas
c
c   Determine the size of the different systems (CPS, QM, Partial)
c
      capnatm = qmmminp%capatm%capnatm
      qmnatm  = qmmminp%qmnatm
      cssnatm = capnatm+qmnatm
      partnatm = part%partatmnum
c
c   Size of the Hessian (partial or entire system)
c
      if (ipartcal.eq.0) then
          sizemergedhess = 3*geom%natoms
      else if(ipartcal.eq.1) then
          sizemergedhess = 3*partnatm
      endif
c
c  Assign the appropriate geometries to ssgeom
c
      write(outfile,100)
      write(outfile,120)
      call qmmmssgeom(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
120   format(' Construct the capped primary system:')
c
c  Work on background charges
c
      if (qmmminp%embed .eq. 'mechanical') goto 2000
      write(outfile,100)
      write(outfile,1000)
1000  format(//2x,'Treatment for the background point charges:')
c
c  Scale M1, M2 charges
c
      call bgchscale(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
      if ((qmmminp%bordercharge .eq. 'scale')) goto 3000
c		  
c     
c  Shift/redistribute M1 charges
c
      call bgchshift(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)   
c
c  Add charges to the primary system
c
3000  call qmmmbgchgeom(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
c
c  EE polarized charge calculation with the QEQ or EEM method
c  8/8/2006 by Yan Zhang                                                           
c
      call polareech(qmprognme,mmprognme,geom, mmgeom, 
     &                     ssgeom, ssmmgeom, qmmminp)
2000  continue
c  The single-point energy with background point charge calculated by ORCA
C  does not include the electrocstatic interactions between the background 
C  point charge. It is not to subtract the electricstatic interactions from
C  the total energy.
c
c  QM** hessian calculation on the capped primary system
c
      open (unit=orcainf, status='unknown', file=orcain, err=9994)
      call orcastinp(orcainf, esmeth,esbas,qmmminp%qmcharge,
     *  qmmminp%qmmult,
     *   ssgeom,qmmminp%qmoptl,thess,ver_lab,qmmminp,ssmmgeom)
      close (orcainf, err=9994)
      call system('orcashuttle orca.inp orca.out')
c
c   we store the pure CPS hessian and output
c
      call system('cp orca.hess ../orca.qm.hess')
      call system('cp orca.out ../orca.qm.out')
c
c   we store the CPS-Hessian in cpshess
c
      call orcaouth(esmeth, cpshess, cssnatm)
147   format(2x,'QM (QM**) Hessian on primary (primary + ',
     *  'background-charge) system:')
148   format(2x,'Estimated hessian on partial system')
149   format(2x,'Estimated hessian on entire system')
      if (tdebug .eq. 1) then
          write (outfile,116)
          write (outfile,147)
          call mtoltm0 (3*cssnatm,
     *       cpshess(:3*cssnatm,:3*cssnatm),
     *       hs(:3*cssnatm*(3*cssnatm+1)/2))
c
c    The numerical hessian is not symmetrical, but for simplicity we
c    display just the lower triangular matrix
c
          write (outfile,118) hs(:3*cssnatm*(3*cssnatm+1)/2)
          write (outfile,119)
      endif
c
c  Estimated hessian calculation on the entire system (or partial system, if given)
c
      open (unit=orcainf, status='unknown', file=orcain, err=9994)
      if (ipartcal.eq.1) then
          partext = 1
          partextatmnum = 0
          call orcapartinp(orcainf, part, geom,
     *            optinp%optl, testhess, mmgeom, partext, partextatmnum)
      else
          call orcainp(orcainf, esmeth,esbas,qmmminp%qmcharge,
     *                 qmmminp%qmmult,geom,optinp%optl,testhess,ver_lab)
      endif
      close (orcainf, err=9994)
      call system('orcashuttle orca.inp orca.out')
c
c   we store the estimated hessian and the output
c
      call system('cp orca.hess ../orca.appr.hess')
      call system('cp orca.out ../orca.appr.out')
c
c   we store the estimated hessian in fullhess
c
      if (ipartcal.eq.1) then
          fullhessnatm = partextatmnum
      else
          fullhessnatm = geom%natoms
      endif
      call orcaouth(esmeth, fullhess, fullhessnatm)
      if (tdebug .eq. 1) then
          write (outfile,116)
          if (ipartcal.eq.1) then
          write (outfile,148)
          else
          write (outfile,149)
          endif
          call mtoltm0 (3*fullhessnatm,
     *       fullhess(:3*fullhessnatm,:3*fullhessnatm),
     *       hs(:3*fullhessnatm*(3*fullhessnatm+1)/2))
          write (outfile,118) hs(:3*fullhessnatm*(3*fullhessnatm+1)/2)
          write (outfile,119)
      endif
c
c    we still have to merge both hessians
c
      if (ipartcal.eq.1) then
c
c   we store the partial Hessian in the atom order given in the input
c
        do i=1,partnatm
            do j=1,partnatm
c
c   coordinate order x(1), y(2), z(3)
c
              do o=1,3
                do p=1,3
                  qmmmhess(3*(i-1)+o,3*(j-1)+p)
     *               = fullhess(3*(i-1)+o,3*(j-1)+p)
                end do
              end do
            end do
        end do
c
c   this time we need the ID of the partial atoms, since QM and partial list are usually different
c
        do i=1,partnatm
          k = part%partatmid(i)
          do j=1,partnatm
            l = part%partatmid(j)
            do m=1,qmnatm
              if (k .eq. qmmminp%qmatm(m)) then
                do n=1,qmnatm
                  if (l .eq. qmmminp%qmatm(n)) then
c
c   if the IDs are the same, we can replace the entries in the Hessian
c
                    do o=1,3
                      do p=1,3
                        qmmmhess(3*(i-1)+o,3*(j-1)+p)
     *                     = cpshess(3*(m-1)+o,3*(n-1)+p)
                      end do
                    end do
                  endif
                end do
              endif
            end do
          end do
        end do
c
c   we store the entire Hessian in the normal atom order
c
      else
        do i=1,geom%natoms
           m = i
           do j=1,geom%natoms
              n = j
c
c   coordinate order x(1), y(2), z(3)
c
              do o=1,3
                do p=1,3
                  qmmmhess(3*(i-1)+o,3*(j-1)+p)
     *               = fullhess(3*(m-1)+o,3*(n-1)+p)
                 end do
               end do
            end do
         end do
c
c   and for the QM atoms we replace the Hessian entries
c
        do m=1,qmnatm
           k = qmmminp%qmatm(m)
           do n=1,qmnatm
              l = qmmminp%qmatm(n)
              do o=1,3
                 do p=1,3
                    qmmmhess(3*(k-1)+o,3*(l-1)+p)
     *                = cpshess(3*(m-1)+o,3*(n-1)+p)
                  end do
               end do
            end do
         end do
      endif
c
c    store the qmmmhess in a matrix tmpHess with dimension 3*hessnatm x 3*hessnatm
c
      allocate (tmpHess(sizemergedhess, sizemergedhess))
      tmpHess(:sizemergedhess,:sizemergedhess)
     *       =qmmmhess(:sizemergedhess,:sizemergedhess)
c
c    store the hessian in the orca-hess file-format. We must do this, because
c    the obtained hessian is not symmetrical, so the lower triangular hessian is 
c    eventually not sufficient for an optimization
c
      open (unit=orcawhessn,status='unknown',file=orcawhessf,err=9995)
      call orcawritehess(orcawhessn, sizemergedhess, tmpHess,
     *                    geom, part, ipartcal)
      close (orcawhessn, err=9995)
c
c    we still need the energy and the gradient
c
      call qmmmghk(qmprognme, mmprognme, geom, mmgeom,
     *            qmmminp, ierr)
c
c    Display the energy
c
      write(outfile,100)
      write(outfile,9000)
      qmmmener = qmes(1) - mmes(1) + mmes(2)
      write(outfile,9100) qmmmener, qmes(1), mmes(1), mmes(2)
9000  format(/,'Display of QMMM energy (a.u.):',/2x,'QMMM      ',
     *   10x,'QM;CPS**',10x,'MM;CPS*',10x,'MM;ES') 
9100  format(4(f16.10,2x))      
c
c    Display gradient and hessian results
c
      i = sizemergedhess
      write(outfile,100)
      write(outfile,9200)
      write(outfile,9210) qmmmgrad(:geom%natoms*3)   
      write(outfile,9220)
      call mtoltm0 (i, qmmmhess(:i,:i),
     *   hs(:i*(i+1)/2))
      write (outfile,118) hs(:i*(i+1)/2)
      return 
100   format(80('-'))
116   format(/,10('Debug'))
117   format(2x,'MM Hessian on entire system:')
118   format(4(d20.10))
119   format(10('Debug'),/)
9200  format(/,'Display of QMMM gradient (a.u.):')
9210  format(3(e20.12))
9220  format(/,'Display of QM** hessian (a.u.):')
9994  write (outfile,*) 'Error creating Orca input file'
      stop 'proghhk 4'
9995  write (outfile,*) 'Error creating orca hess-file'
      stop 'qmmmhybridhhk 2'
      end
c
c******************************************************************************
c  QMMMHHK
c******************************************************************************
c
      subroutine orcaesthhk(qmprognme, mmprognme, geom,
     *   mmgeom, qmmminp, optinp, part, ipartcal,
     *   ierr)
c
c     The subroutine calculates the estimated hessian.
c     This is the Almloef-hessian from orca
c
c
c     Subroutine orcaesthhk
c
c     Created: Oct. 02, 2007 by Christoph Riplinger
c
c     Purpose: To determine the initial estimated hessian with orca
c
c     Called by: orca_ext_opt2
c     Calls:  orcapartinp, orcainp, orcaouth, orcawritehess, mtoltm0 , qmmmghk
c
      use caltyp
      use input,  only   : cartgeom, mmvalen, qmmminf, mlopt, partial
      use files
      use energy, only   : qmmmener, qmes, mmes
      use gradient, only : qmmmgrad, qmmmqmgr, qmmmmmgr
c
c Hai Lin, 5/4/2009
c
c      use hessian, only  : qmmmqmhs, qmmmhess
      use hessian, only  : qmmmqmhs, qmmmhess, hessmaxatm
c
c end by Hai Lin, 5/4/2009
c
      use caltyp,  only  : testhess
      implicit none
c
c i/o variables
c
      character*20       qmprognme, mmprognme, ver_lab
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (qmmminf)  :: qmmminp
      type (mlopt)    :: optinp
      type (partial)  :: part
      integer            ierr, sizeEstHess, partext
      integer            partextatmnum, ipartcal
c
c local variables
c
      character*20       esmeth, esbas
      type (cartgeom) :: ssgeom, bgchgeom
      type (mmvalen)  :: ssmmgeom
      integer            lsnatm, ssnatm, bgchnum
      integer            fullhessnatm
      integer            i, j, k, l, m, n, o, p
      integer            partnatm, qmnatm, capnatm, cssnatm
      integer            qmcharge, qmmult
c
c Hai Lin, 5/4/2009
c
c        please check module.F:hessian:maxatm value
c      integer, parameter :: hessmax = 100*3
c      integer, parameter :: hessmaxnatm = 100
      integer, parameter :: hessmax = hessmaxatm
      integer, parameter :: hessmaxnatm = hessmaxatm * 3
c
c end by Hai Lin, 5/4/2009 
c

      double precision, dimension (:,:), allocatable :: fullhess
      double precision, dimension (:,:), allocatable :: estHess
      double precision, dimension (:), allocatable :: hs
c
c  Save the prnprm switch value
c
      ssmmgeom%lprnatmprm = mmgeom%lprnatmprm
c
c   We store the method and basis from the QM input
c   but actually orca won't need any method or basis
c
      esmeth = qmmminp%qmmeth
      esbas = qmmminp%qmbas
      partnatm = part%partatmnum
      if (ipartcal.eq.0) then
          sizeEstHess = 3*geom%natoms
      else if(ipartcal.eq.1) then
          sizeEstHess = 3*partnatm
      endif
c
c   We use dynamic allocation, since the hessian might get very big
c
      allocate (estHess(sizeEstHess, sizeEstHess))
      allocate (hs(sizeEstHess*(sizeEstHess+1)/2))
100   format(80('-'))
      write(outfile,100)
c
c  Estimated hessian calculation on partial or entire system.
c  For partial Hessian:
c     The hessian should be approximated better if we also include the atoms that are bonded
c     to the border atoms of the partial system. We'll call these atoms P1 (Partial1, first
c     layer around the partial system)
c
      open (unit=orcainf, status='unknown', file=orcain, err=9994)
      if (ipartcal.eq.1) then
          partext = 1
          partextatmnum = 0
          call orcapartinp(orcainf, part, geom,
     *            optinp%optl, testHess, mmgeom, partext, partextatmnum)
          allocate (fullhess(3*partextatmnum, 3*partextatmnum))
      else
          call orcainp(orcainf, esmeth,esbas,qmmminp%qmcharge,
     *                 qmmminp%qmmult,geom,optinp%optl,testHess,ver_lab)
          allocate (fullhess(3*geom%natoms, 3*geom%natoms))
      endif
      close (orcainf, err=9994)
      call system('orcashuttle orca.inp orca.out')
c
c   we store the estimated hessian and the output
c
      call system('cp orca.hess ../orca.appr.hess')
      call system('cp orca.out ../orca.appr.out')
c
c   we store the estimated hessian in fullhess
c
      if (ipartcal.eq.1) then
          fullhessnatm = partextatmnum
      else
          fullhessnatm = geom%natoms
      endif
      call orcaouth(esmeth, fullhess, fullhessnatm)
      if (tdebug .eq. 1) then
          write (outfile,116)
          if (ipartcal.eq.1) then
              write (outfile,120)
          else
              write (outfile,117)
          endif
          call mtoltm0 (i, fullhess(:i,:i),
     *       hs(:i*(i+1)/2))
          write (outfile,118) hs(:i*(i+1)/2)
          write (outfile,119)
      endif
c
c    we still have to merge both hessians
c
      if (ipartcal.eq.1) then
c
c   we store the partial Hessian in the atom order given in the input
c   The additional atoms (P1) are listed after the partial atoms, so 
c   if we just go from 1 up to partnatm, we are fine
c
        do i=1,partnatm
           do j=1,partnatm
c
c   coordinate order x(1), y(2), z(3)
c
              do o=1,3
                 do p=1,3
                    estHess(3*(i-1)+o,3*(j-1)+p)
     *                 = fullhess(3*(i-1)+o,3*(j-1)+p)
                  end do
               end do
            end do
         end do
c
c   we store the entire Hessian in the normal atom order
c
      else
         do i=1,geom%natoms
            do j=1,geom%natoms
               do o=1,3
                  do p=1,3
       write(outfile,*) i, j, o, p, fullhess(3*(i-1)+o,3*(j-1)+p)
                     estHess(3*(i-1)+o,3*(j-1)+p)
     *                 = fullhess(3*(i-1)+o,3*(j-1)+p)
                   end do
                end do
             end do
          end do
       endif
c    store the hessian in the orca-hess file-format. We must do this, because
c    the obtained hessian is not symmetrical, so the lower triangular hessian is 
c    eventually not sufficient for an optimization
c
      open (unit=orcawhessn,status='unknown',file=orcawhessf,err=9995)
      call orcawritehess(orcawhessn, sizeEstHess, estHess,
     *                    geom, part, ipartcal)
      close (orcawhessn, err=9995)
c
c   we have to copy the qm hessian to the total hessian, so that we can find sth. in the sum-file
c   if the estimated hessian is too big, we just store the first (:hessmaxnatm,:hessmaxnatm) part
c   the hessian that will be used by orca will be stored in xxx.hess anyway
c
      if(sizeEstHess .ge. hessmaxnatm) then
          qmmmhess(:hessmaxnatm,:hessmaxnatm)
     *        = estHess(:hessmaxnatm,:hessmaxnatm)
      else
          qmmmhess(:sizeEstHess,:sizeEstHess)
     *        = estHess(:sizeEstHess,:sizeEstHess)
      endif
c
c    we still need the energy and the gradient
c
      call qmmmghk(qmprognme, mmprognme, geom, mmgeom,
     *            qmmminp, ierr)
      if (tdebug .eq. 1) then
          write (outfile,116)
          write (outfile,121)
          write (outfile,118) qmmmmmgr(2,:geom%natoms*3)
          write (outfile,119)
      endif
c
c    Display the energy
c
      write(outfile,100)
      write(outfile,9000)
      qmmmener = qmes(1) - mmes(1) + mmes(2)
      write(outfile,9100) qmmmener, qmes(1), mmes(1), mmes(2)
c
c  Display gradient and Hessian results
c
      i = sizeEstHess
      write(outfile,100)
      write(outfile,9200)
      write(outfile,9210) qmmmgrad(:geom%natoms*3)   
      write(outfile,9220)
      call mtoltm0 (i, qmmmhess(:i,:i),
     *   hs(:i*(i+1)/2))
      write (outfile,118) hs(:i*(i+1)/2)
      return 
116   format(/,10('Debug'))
117   format(2x,'MM Hessian on entire system:')
118   format(4(d20.10))
119   format(10('Debug'),/)
120   format(2x,'MM Hessian on partial system (+P1):')
121   format(2x,'QMMM Gradient on entire system:')
9000  format(/,'Display of QMMM energy (a.u.):',/2x,'QMMM      ',
     *   10x,'QM;CPS**',10x,'MM;CPS*',10x,'MM;ES') 
9100  format(4(f16.10,2x))      
9200  format(/,'Display of QMMM gradient (a.u.):')
9210  format(3(e20.12))
9220  format(/,'Display of QM** hessian (a.u.):')
9994  write (outfile,*) 'Error creating Orca input file'
      stop 'proghhk 4'
9995  write (outfile,*) 'Error creating orca hess-file'
      stop 'qmmmesthhk 2'
      end

