c
c******************************************************************************
c  MLOHOOK
c******************************************************************************
c
      subroutine mlohook(progrms, geom, optinp, 
     *   qmmminp, mmgeom,                                     
     *   tmeth, tbas, toptl, ierr)                                      
c  
c     Subroutine mlohook
c
c     Created:  Feb. 22, 2005 by Hai Lin
c     Revised:  Dec. 12, 2006 by Yan Zhang
c
c     Purpose:  To call the appropriate appropriate optimization
c        subroutine.  All information to get the requested energies,
c        gradients and hessians is stored in mlinp
c
c     Currently only the NEWT optimizer is available for optimization.
c
c     Called by:  main
c     Calls:  newt, newt2, rotmol, ef, Gau_ext_opt, minlbfgs
c
c     Variables defined here:
c
      use files, only : outfile
      use input, only : cartgeom, mlopt, mlinp, proglist,
     *                   maxoptl, qmmminf, mmvalen
      implicit none
c
c i/o variables
c
      integer            ierr
      character*20       mlmeth, tmeth, tbas
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom                                         
      type (mlopt)    :: optinp
c
      character*80    :: mcoptl(maxoptl), toptl(maxoptl)
      type (qmmminf)  :: qmmminp                                        
      type (proglist) :: progrms
c
c   modified 2/10/2007 by Christoph Riplinger
c
c      character*6     :: aswit(0:5)
      character*7     :: aswit(0:6)
c
c   end 2/10/2007 by Christoph Riplinger
c
c local variables
c
      integer            i, n
      integer, dimension(:), allocatable :: ind
	      
c      double precision, parameter :: angtoau = 0.52918d0
      double precision, parameter :: angtoau = 0.529177249d0
c
c   modified 2/10/2007 by Christoph Riplinger
c
c      data aswit /'NR','BFGS','DFP','EF','GAUEXT','LBFGS'/
      data aswit /'NR','BFGS','DFP','EF','GAUEXT','LBFGS','ORCAEXT'/
c
c   end 2/10/2007 by Christoph Riplinger
c
      allocate( ind(3*geom%natoms-5) )

c
c  Initialize the optimization portion of the output file
c
      write(outfile,1100) optinp%mlmeth, aswit(optinp%algrthm)
1100  format(/2(/80('-')),//2x,'Multilevel Optimization',/5x,
     *  'Method:',t20,a,/5x,'Algorithm:',t20,a,//80('-')/)
c
c  Set up indices to be optimized
c
      n = 0
      do 21 i=1, 3*geom%natoms
         if (optinp%constant(i).eq.0) then
            n = n+1
            ind(n) = i
         endif
  21  continue

c
c  Reorient the molecule along the appropriate axes, unless using Gauext
c
      mlinp%geom   = geom
      mlinp%mmgeom = mmgeom                                             
            
c
c   modified 2/10/2007 by Christoph Riplinger
c
c      if (optinp%algrthm.ne.4) then
      if ((optinp%algrthm.ne.4) .and. (optinp%algrthm.ne.6)) then
c
c   end 2/10/2007 by Christoph Riplinger
c
         call rotmol(geom%aucoord, mlinp%geom%aucoord, geom%natoms, 
     *    optinp%constant, optinp%iop)
         mlinp%geom%angcoord = angtoau*mlinp%geom%aucoord
       endif	 
c
c  Determine type of optimization occuring and set appropriate slot in 
c    the structure mlinp.
c
      mlmeth(:) = optinp%mlmeth(:)

      mlinp%progrms = progrms
      mlinp%mlmeth(:) = optinp%mlmeth(:)
      do 5 i=1, maxoptl - 1
         mlinp%mcoptl(i) = mcoptl(i)                                    
   5  continue
      mlinp%htyp = optinp%htyp
      mlinp%hbas = optinp%hbas
      mlinp%hmeth = optinp%hmeth
      mlinp%tbas = tbas
      mlinp%tmeth = tmeth
      mlinp%toptl = toptl
      mlinp%hscale = optinp%hscale

      if (mlmeth(:4) .eq. 'qmmm')   then                                
         mlinp%qmmminp = qmmminp	       	 	       
      endif
c
c  Call the appropriate optimization algorithm based on the user input.
c  Currently only the NR, DFP, and BFGS algorithms are supported (in
c  newt and newt2).
c
      if (optinp%algrthm.eq.0) then
         call newt(optinp, ind, n, ierr)
      elseif (optinp%algrthm.eq.1 .or. optinp%algrthm.eq.2) then
         call newt2(optinp, ind, n, ierr)
      elseif (optinp%algrthm.eq.3) then
         call ef(optinp, ind, n, ierr)	 
      elseif (optinp%algrthm.eq.4) then
c
c  Call gaussian external optimizer                                     
c      
         call gau_ext_opt(optinp,geom,mmgeom,ierr)
         return	
c
c  Call minlbfgs added by Yan Zhang 12/12/2006
c
      elseif (optinp%algrthm.eq.5) then
         call minlbfgs(optinp, ind, n, ierr)
c
c   add 2/10/2007 by Christoph Riplinger
c   Call orca external optimizer 
c
      elseif (optinp%algrthm.eq.6) then
         call orca_ext_opt(optinp,geom,mmgeom,ierr,qmmminp,progrms)
         return
c
c   end 2/10/2007 by Christoph Riplinger
c
c  end by Yan Zhang 12/212/2006
c	 
      endif
c
c  Set geom to new geometry (in initial orientation) if optimization
c       successful.  And print out the reoriented geometry.
c
c  Hai Lin: reorient the geom for restart calculations even if ierr = 1
c
c      if (ierr.eq.0 .and. optinp%reorient.eq.1) then
      if (optinp%reorient.eq.1) then
         if (ierr .eq. 1) then
            write(outfile,1800)
         endif
         call rotmol(mlinp%geom%aucoord, geom%aucoord, geom%natoms, 
     *       optinp%constant, -optinp%iop)

         geom%angcoord = angtoau*geom%aucoord
         write(outfile,1400)
         write(outfile,1450)
         do 20 i=1, geom%natoms
            write(outfile,1500)  geom%atm(i), geom%aucoord(3*i-2),
     *          geom%aucoord(3*i-1), geom%aucoord(3*i)
  20     continue
         write(outfile,1550)
         do 40 i=1, geom%natoms
            write(outfile,1500)  geom%atm(i), geom%angcoord(3*i-2),
     *         geom%angcoord(3*i-1), geom%angcoord(3*i)
  40     continue
         write(outfile,*)
c      elseif (ierr.eq.0) then
      else
         if (ierr.eq.1) then
            write(outfile,1800)
         endif
         geom%aucoord = mlinp%geom%aucoord
         geom%angcoord = mlinp%geom%angcoord	 	       	     
      endif

1400  format(/2(/80('-')),//2x,'Geometry after Reorientation:',
     *     //80('-')/)
1450  format(//2x,'Atomic Coordinates (in a.u.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)
1500  format(9x,a2,t20,d15.8,t40,d15.8,t60,d15.8)
1550  format(//2x,'Atomic Coordinates (in ang.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)
1800  format(//2x,'Geometry optimization failed.',//2x,
     *   'The geometry at the last step is given below:',/)
      return
      end
c
c******************************************************************************
c  NEWT
c******************************************************************************
c
      subroutine newt(optinp, ind, n, ierr)
c
c     Subroutine newt
c
c     Created:  July 14, 1999 by J. M. Rodgers
c               Adapted from the NEWT routines in Ben's IMOHC code
c                 and in POLYRATE.
c     Revised:  Feb. 22, 2004  by Hai Lin
c               May   4, 2006  by Hai Lin 
c
c     Purpose:  To perform NR optimization with Brent line minimization.
c        For each iteration the gradient is recalculated and the hessian
c        may or may not be depending on HREC as specified in the input
c     
c     Called by:  mlohook
c     Calls:  optghk, opthhk, optdisp, ludcmp, lubksb, linmn
c
c     Variables defined here:  none.
c
      use files, only : outfile, moldenfile, molden5                    
      use caltyp, only : tgrad
      use input, only : mlopt, mlinp
      use optimize, only : ener, grad, hess
      use param,    only : lowqmhs, uniths,qmmmhs,mmhs
      
      implicit none
c
c i/o variables
c
      integer            ierr
      type (mlopt)    :: optinp
      integer            n, ind(n)
c
c local variables
c                  
      logical         :: check

      integer, dimension(:), allocatable :: work, indx
      integer  idorec, nrecal, n3tm, jrank
      integer  its, mxind, fail, linmin, i, j, status, k
      double precision, dimension(:), allocatable :: xxi, x0, dx0,
     *    prod, tempsol
      double precision, dimension(:,:), allocatable :: luhess
      double precision  gtol, h2, gcomp, abxxi, fret, abgc,
     *    stepc, stepmx, d, sum, detf, chksum

      double precision, parameter :: eps = 1.0d-14
c      double precision, parameter :: angtoau = 0.52918d0
      double precision, parameter :: angtoau = 0.529177249d0
      double precision, parameter :: maxstep = 1.0d0

      n3tm = 3*mlinp%geom%natoms

      allocate (work(n3tm), xxi(n3tm), x0(n3tm), dx0(n3tm), 
     *    prod(n3tm), indx(n3tm), luhess(n,n), tempsol(n) )

      fail = 0
      idorec = optinp%hrec
      nrecal = 0
c
c  Open molden file                                                     
c
      open (unit=moldenfile, status='unknown', file=molden5, err=9993)      	       
c      
c
c  Turn off the Brent line minimization if required                     
c
      if (optinp%llinmn .eq. 1) then
         linmin = 1
      else	 
         linmin = 0
      endif	 
c
      status = 0

      if (optinp%iop .gt. 4) then
         linmin = 0
      endif
c
c  Set up the default convergent values for the energy and derivative
c
      if (optinp%gcomp .le. eps) then
         gtol = 1.0d-7
      else
         gtol = optinp%gcomp
      endif
      if (optinp%niter .lt. 0) goto 300
      h2 = 2.0d0*optinp%dlx2
c
c  Get initial gradient
c
      call optghk

      gcomp = abs( grad(1) )
      do 5 i=2, n3tm
         abgc = abs( grad(i) )
         if (gcomp.lt.abgc)  gcomp = abgc
   5  continue
c
c  Make initial display of info
c
         do 10 i=1, n3tm
            xxi(i) = 0.0000d0
  10     continue
         call optdisp(0, mlinp%geom, ener, grad(:n3tm), xxi(:n3tm),
     *           gcomp, gtol)
c
c  Check if the geometry is already optimized.
c
      if (gcomp.le.gtol) then
         status = 3
         goto 300
      endif
c
c  Get initial hessian (by hhook if INITHESS is on;  I otherwise)
c
      if (optinp%ihunit.eq.1) then
         call opthhk
      elseif (optinp%ihunit.eq.0) then
         do 21 i=1, n3tm
            do 22 j=1, n3tm
               hess(i,j) = 0.0d0
  22        continue
            hess(i,i) = optinp%hscale
  21     continue
      endif
c
c  MAIN LOOP to perform the geometry optimization
c
      do 200 its=1, optinp%niter
c
c  initialize dx0 and x0
c
         do 40 i=1, n3tm
            dx0(i) = grad(i)
            x0(i) = mlinp%geom%aucoord(i)
  40     continue
c         write(outfile,'(/,a,i3)') 'Initial geometry at step ', its
c         write(outfile,1018) x0
1018     format(3(1x,f10.6))
c         write(outfile,'(/,a,i3)') 'Initial gradient at step ', its
c         write(outfile,1023) dx0
1023     format(3(1x,d15.8))
c
c  Write out the Hessian.
c
         write(outfile,'(/a)') 'Hessian:'
         write(outfile,'(6(1x,d12.5))') hess(:n3tm,:n3tm)
c
c  Calculate Newton-Raphson step - different from way done in Polyrate
c
         do 29 i=1, n
            tempsol(i) = -dx0(ind(i))
            do 30 j=1, n
               luhess(i,j) = hess(ind(i),ind(j))
  30        continue
  29     continue
         call ludcmp(luhess, n, n, indx, d)
c
c  Write out the LU decomposed Hessian
c
c         write(outfile,'(/a)') 'LU Decomposed Hessian:'
c         write(outfile,'(6(1x,d12.5))') luhess(:n,:n)
         call lubksb(luhess, n, n, indx, tempsol)
c
c  Check solution to H*XXI = -Grad
c
         do 41 i=1, n
            prod(i) = 0.0d0
            do 42 j=1, n
               prod(i) = prod(i) + hess(ind(i),ind(j))*tempsol(j)
  42        continue
  41     continue

         chksum = 0.0d0
         do 59 i=1, n
            chksum = chksum + prod(i) + grad(ind(i))
  59     continue

         write(outfile,'(/a,3x,d12.5)') 'Sum that should be ~0:',
     *           chksum

         do 43 i=1, n3tm
            xxi(i) = 0.0000d0
  43     continue
         do 44 i=1, n
            xxi(ind(i)) = tempsol(i)
  44     continue

         write(outfile,'(/a,i3)') 'Calculated XXI at step ', its
         write(outfile,1023) xxi(:n3tm)
c
c  Scale calculated step so that rms value not greater than scale
c
         sum = 0.0d0
         do 7 i=1, n3tm
            sum = sum + xxi(i)*xxi(i)
   7     continue
         sum = sqrt(sum)
         if (sum.gt.optinp%scale)  xxi = xxi*optinp%scale/sum
	 
         stepc = abs(xxi(1))
         mxind = 1
         do 105 i=2, n3tm
            abxxi = abs(xxi(i))
            if (stepc.lt.abxxi) then
                stepc = abxxi
                mxind = i
            endif
 105     continue
         write(outfile,'(/,a,i3)') 'XXI at step ', its
         write(outfile,1018) xxi	 
c
c  Add to make sure that step not zero before send into linmn.
c
         if (stepc .eq. 0.0d0) then
            status = 4
            goto 300
         endif
c
c  ... then scale the Newton-Raphson step (yet not bigger than scale)
c
         stepmx = optinp%scale/stepc
         if (linmin.eq.1) then
	 
            call linmn(x0, xxi, n3tm, fret, stepmx)	    
            sum = 0.0d0
            do 8 i=1, n3tm
               sum = sum + xxi(i)*xxi(i)
   8        continue
            sum = sqrt(sum)
            if (sum.gt.optinp%scale)  xxi = xxi*optinp%scale/sum
            stepc = abs(xxi(mxind))
         endif

c
c  Move to a new geometry and calculate new energy and gradient
c  Be careful with units - Gaussian is called with angcoord, yet the
c    optimizer operates on aucoord
c
         do 120 i=1, n3tm
            mlinp%geom%aucoord(i) = x0(i) + xxi(i)
 120     continue
         mlinp%geom%angcoord = mlinp%geom%aucoord*angtoau

c         write(outfile,'(/,a)') 'Ang. Geom after step'
c         write(outfile,1018) mlinp%geom%angcoord(:n3tm)         
c         write(outfile,'(/,a)') 'A.U. Geom after step'
c         write(outfile,1018) mlinp%geom%aucoord(:n3tm)	 
c
c  Get gradient and energy
c
         call optghk

         gcomp = abs(grad(1))
         do 130 i=2, n3tm
            abgc = abs(grad(i))
            if (gcomp.lt.abgc)  gcomp=abgc
 130     continue
c
c  Test for convergence
c
         if (gcomp.le.gtol) then
            status = 1
            goto 300
         endif
c
c  Test for magnitude of stepsize
c
         if (stepc.lt.optinp%stptol) then
            fail = 1
            write(outfile, 1350) stepc
1350        format(/,'Maximum displacement component change smaller ',
     *               'than STPTOL:',3x,d12.5)
            if (optinp%retry .eq. 0)  then
               status = 2
               goto 300
            endif
         endif

         if (fail .eq. 1) then
            write(outfile, 1375)
1375        format(/,'Switching to full Newton optimization')
            idorec = 1
            nrecal = idorec
            linmin = 0
            fail = 0
         else
            nrecal = nrecal+1
         endif
c
c  Display current step in optimization
c
         call optdisp(its, mlinp%geom, ener, grad(:n3tm), xxi(:n3tm), 
     *           gcomp, gtol)
c
c  Write the geometry to molden file                                    
c
         write (moldenfile,*) mlinp%geom%natoms
         write (moldenfile,9000) its, ener
         do i=1, mlinp%geom%natoms
            write (moldenfile,9010) mlinp%geom%atm(i), 
     *         mlinp%geom%angcoord(3*i-2), mlinp%geom%angcoord(3*i-1),
     *         mlinp%geom%angcoord(3*i)
         enddo
9000     format ('Iteration: ',i5,2x,'Energy (a.u.): ',f16.8)	 
9010     format (2x,a2,3(3x,f16.8))
c
c  Get new hessian if necessary (but not if on last step of optimization.
c
c  If the Hessian type is unit matrix, not recalculation is done.
c  Added by Hai Lin, 5/5/2006 
c
         if (mlinp%htyp.ne.uniths .and. 
     *       nrecal.eq.idorec .and. its.ne.optinp%niter) then
c  
c  End. By Hai Lin 5/5/2006
c
            nrecal = 0
            write (outfile, 1400)
1400        format(/,'Hessian recalculated.')
            call opthhk 
         else
            write (outfile, 1450)
1450        format(/,'Hessian kept frozen.')
         endif
	
 200  continue
c
c  If at this point the iterations have maxed out - print message and 
c    exit routine.
c
      write (outfile, 2200)
2200  format (/5x,'OPTIMIZATION FAILURE: Max iterations reached.',/)
      ierr = 1

c
c  Close molden file                                                    
c 
      close (moldenfile,err=9993)
      return

 300  continue
c
c  Display final step in optimization and print appropriate exit message.
c

      if (status .eq. 1) then
         call optdisp(its, mlinp%geom, ener, grad(:n3tm), xxi(:n3tm),
     *           gcomp, gtol)
         write (outfile, 2000)
      elseif (status .eq. 2) then
         call optdisp(its, mlinp%geom, ener, grad(:n3tm), xxi(:n3tm),
     *           gcomp, gtol)
         write (outfile, 2100)
         ierr = 1
      elseif (status .eq. 3) then
         write (outfile, 2300)
      elseif (status .eq. 4) then
         write (outfile, 2400)
         ierr = 1
      endif

c
c  Close molden file                                                    
c
      close (moldenfile,err=9993)
	    
      return

2000  format (/80('*'),/28x,'OPTIMIZATION SUCCESSFUL.',
     *          /80('*'),/)
2100  format (/5x,'OPTIMIZATION FAILURE: Geometry not changing.',/)
2300  format (/80('*'),/26x,'GEOMETRY ALREADY OPTIMIZED.',
     *          /80('*'),/)
2400  format (/5x,'OPTIMIZATION FAILURE: Step Size is 0.',/)

9993  write (outfile,*) 'Error creating Molden input file'
      stop 'newt 1'      
      
      end
c
c******************************************************************************
c  NEWT2
c******************************************************************************
c
      subroutine newt2(optinp, ind, n, ierr)
c
c     Subroutine newt
c
c     Created:  July 30, 1999 by J. M. Rodgers
c               Adapted from the NEWT routines in Ben's IMOHC code
c                 and in POLYRATE with the update formulas taken from
c                 Numerical Recipes (instead of polyrate).
c     Revised:  Feb 22, 2004 by Hai Lin
c               May  5, 2006 by Hai Lin
c
c     Purpose:  To perform NR optimization with BFGS or DFP hessian
c        updates by calling the appropriate ghook based on the user input.
c        Currently only valid for use with the scaled unit hessian.
c 
c     NOTE: testing on methanol has shown updates on the unit matrix to be
c        an inefficient idea.
c     
c     Called by:  mlohook
c     Calls:  optghk, opthhk, optdisp, linmn, mxlneq
c
      use files, only : outfile, moldenfile, molden5
      use input, only : mlopt, mlinp
      use optimize, only : ener, grad, hess
      use param,    only : lowqmhs, uniths,qmmmhs,mmhs
      
      implicit none
c
c i/o variables
c
      integer            ierr
      type (mlopt)    :: optinp
      integer            n, ind(n)
c
c local variables
c            
      logical         :: check

      integer, dimension(:), allocatable :: work
      integer  idorec, nrecal, n3tm
      integer  jrank, its, mxind, fail, linmin, i, j, status, k
      double precision, dimension(:), allocatable :: xxi, x0, dx0,
     *    deltag, prod, hdg
      double precision, dimension(:,:), allocatable :: hessin
      double precision  gtol, h2, gcomp, abxxi, fret, abgc,
     *    detf, stepc, stepmx, d, fac, fae, sumdg, sumxi, fad, sum

      double precision, parameter :: eps = 1.0d-14
c      double precision, parameter :: angtoau = 0.52918d0
      double precision, parameter :: angtoau = 0.529177249d0
      double precision, parameter :: maxstep = 1.0d0

      n3tm = 3*mlinp%geom%natoms

      allocate (work(n3tm), xxi(n3tm), x0(n3tm), dx0(n3tm), 
     *    deltag(n), hessin(n,n), prod(n3tm), hdg(n) )

      fail = 0
      idorec = optinp%hrec
      nrecal = 0
c
c  Open molden file                                                     
c
      open (unit=moldenfile, status='unknown', file=molden5, err=9993)            
c
c  Turn off Brent line minimization if required
c
      if (optinp%llinmn .eq. 1) then         
         linmin = 1      
      else	 
         linmin = 0
      endif
      	 
      status = 0

      if (optinp%iop .gt. 4) then
         linmin = 0
      endif
c
c  Set up the default convergent values for the energy and derivative
c
      if (optinp%gcomp .le. eps) then
         gtol = 1.0d-7
      else
         gtol = optinp%gcomp
      endif
      if (optinp%niter .lt. 0) goto 300
      h2 = 2.0d0*optinp%dlx2
c
c  Get initial gradient
c
      call optghk

      gcomp = abs( grad(1) )
      do 5 i=2, n3tm
         abgc = abs( grad(i) )
         if (gcomp.lt.abgc)  gcomp = abgc
   5  continue
c
c  Make initial display of info
c
         do 10 i=1, n3tm
            xxi(i) = 0.0000d0
  10     continue
         call optdisp(0, mlinp%geom, ener, grad(:n3tm), xxi(:n3tm),
     *           gcomp, gtol)
c
c  Check if the geometry is already optimized.
c
      if (gcomp.le.gtol) then
         status = 3
         goto 300
      endif
c
c  Set initial hessian (by hhook if INITHESS is on; I otherwise)
c
      if (optinp%ihunit.eq.1) then
         call opthhk
      elseif (optinp%ihunit.eq.0) then
         do 21 i=1, n3tm
            do 22 j=1, n3tm
               hess(i,j) = 0.0d0
  22        continue
            hess(i,i) = optinp%hscale
  21     continue
      endif
c
c  Get inverse of the small hessian
c
      do 23 i=1, n
         do 24 j=1, n
            hessin(i,j) = hess(ind(i),ind(j))
  24     continue
  23  continue
      call mxlneq( hessin(:n,:n),n,n,detf,jrank,eps,work,0,n )

      sum = 0.0
      do 25 i=1, n
         do 26 j=1, n
            sum = sum + hessin(i,j)
  26     continue
  25  continue
      sum = sum - dble(n)
      write(outfile,'(/a,3x,d12.5)') 'Sum that should be ~0:', sum
c
c  MAIN LOOP to perform the geometry optimization
c
      do 200 its=1, optinp%niter
c
c  initialize dx0 and x0
c
         do 40 i=1, n3tm
            dx0(i) = grad(i)
            x0(i) = mlinp%geom%aucoord(i)
  40     continue
c         write(outfile,'(/,a,i3)') 'Initial geometry at step ', its
c         write(outfile,1018) x0
1018     format(3(1x,f10.6))
c         write(outfile,'(/,a,i3)') 'Initial gradient at step ', its
c         write(outfile,1023) dx0
1023     format(3(1x,d15.8))
c
c  Calculate Newton-Raphson step
c
         do 29 i=1, n3tm
            xxi(i) = 0.0d0
  29     continue
         do 30 i=1, n
            do 31 j=1, n
               xxi(ind(i)) = xxi(ind(i)) - hessin(i,j)*dx0(ind(j))
  31        continue
  30     continue
c
c  Scale calculated step so that rms value not greater than scale
c
         sum = 0.0d0
         do 7 i=1, n3tm
            sum = sum + xxi(i)*xxi(i)
   7     continue
         sum = sqrt(sum)
         if (sum.gt.optinp%scale)  xxi = xxi*optinp%scale/sum
 
         stepc = abs(xxi(1))
         mxind = 1  
         do 105 i=2, n3tm
            abxxi = abs(xxi(i))
            if (stepc.lt.abxxi) then
                stepc = abxxi
                mxind = i
            endif
 105     continue
c         write(outfile,'(/,a,i3)') 'XXI at step ', its
c         write(outfile,1018) xxi
c
c  Add to make sure that step not zero before send into linmn.
c
         if (stepc .eq. 0.0d0) then
            status = 4
            goto 300  
         endif
c
c  ... then scale the Newton-Raphson step
c
         stepmx = optinp%scale/stepc
         if (linmin.eq.1) then
            call linmn(x0, xxi, n3tm, fret, stepmx)
            sum = 0.0d0
            do 8 i=1, n3tm
               sum = sum + xxi(i)*xxi(i)
   8        continue
            sum = sqrt(sum)
            if (sum.gt.optinp%scale)  xxi = xxi*optinp%scale/sum
            stepc = abs(xxi(mxind))
         endif
c
c  Move to a new geometry and calculate new energy and gradient
c  Be careful with units - Gaussian is called with angcoord, yet the
c    optimizer operates on aucoord
c
         do 120 i=1, n3tm
            mlinp%geom%aucoord(i) = x0(i) + xxi(i)
 120     continue
         mlinp%geom%angcoord = mlinp%geom%aucoord*angtoau

c         write(outfile,'(/,a)') 'Ang. Geom after scaling'
c         write(outfile,1018) mlinp%geom%angcoord(:n3tm)         
c         write(outfile,'(/,a)') 'A.U. Geom after scaling'
c         write(outfile,1018) mlinp%geom%aucoord(:n3tm)
c
c  Get gradient and energy
c
         call optghk

         gcomp = abs(grad(1))
         do 130 i=2, n3tm
            abgc = abs(grad(i))
            if (gcomp.lt.abgc)  gcomp=abgc
 130     continue
c
c  Test for convergence
c
         if (gcomp.le.gtol) then
            status = 1
            goto 300
         endif
c
c  Test for magnitude of stepsize
c
         if (stepc.lt.optinp%stptol) then
            fail = 1
            write(outfile, 1350) stepc
1350        format(/,'Maximum displacement component change smaller ',
     *               'than STPTOL:',3x,d12.5)
            if (optinp%retry .eq. 0)  then
               status = 2
               goto 300
            endif
         endif

         if (fail .eq. 1) then
            write(outfile, 1375)
1375        format(/,'Switching to full Newton optimization')
            idorec = 1
            nrecal = idorec
            linmin = 0
            fail = 0
         else
            nrecal = nrecal+1
         endif
c
c  Display current step in optimization
c
         call optdisp(its, mlinp%geom, ener, grad(:n3tm), xxi(:n3tm), 
     *           gcomp, gtol)
c
c  Write the geometry to molden file                                    
c
         write (moldenfile,*) mlinp%geom%natoms
         write (moldenfile,9000) its, ener
         do i=1, mlinp%geom%natoms
            write (moldenfile,9010) mlinp%geom%atm(i),
     *         mlinp%geom%angcoord(3*i-2), mlinp%geom%angcoord(3*i-1),
     *         mlinp%geom%angcoord(3*i)		     
         enddo
9000     format ('Iteration: ',i5,2x,'Energy (a.u.): ',f16.8)
9010     format (2x,a2,3(3x,f16.8))
c
c  Get new hessian if necessary (but not if on last step of optimization.
c
c  If Hessian type is unit matrix, no recalculation is done.
c  Added by Hai Lin, 5/5/2006 
c
         if (mlinp%htyp.ne.uniths .and. 
     *       (nrecal.eq.idorec .and. its.ne.optinp%niter)) then
c
c  End. Hai Lin 5/5/2006
c
            nrecal = 0
            write (outfile, 1400)
1400        format(/,'Hessian recalculated.')
            call opthhk
c
c  Get inverse of the small hessian
c
            do 101 i=1, n   
               do 102 j=1, n
                  hessin(i,j) = hess(ind(i),ind(j))
 102           continue
 101        continue
            call mxlneq( hessin(:n,:n),n,n,detf,jrank,eps,work,0,n )
 
            sum = 0.0
            do 103 i=1, n
               do 104 j=1, n
                  sum = sum + hessin(i,j)
 104           continue
 103        continue   
            sum = sum - dble(n)
            write(outfile,'(/a,3x,d12.5)') 'Sum that should be ~0:', sum
c
c  Otherwise, do BFGS or DFP update
c
         else
c
c  ... Compute differences of gradients and difference times hessin
c
            do 152 i=1, n
               deltag(i) = grad(ind(i)) - dx0(ind(i))
 152        continue
            do 153 i=1, n
               hdg(i) = 0.0d0
               do 154 j=1, n
                  hdg(i) = hdg(i)+hessin(i,j)*deltag(j)
 154           continue
 153        continue
c
c  ... Calculate dot products for the denominators
c
            fac = 0.0d0
            fae = 0.0d0
            sumdg = 0.0d0
            sumxi = 0.0d0
            do 155 i=1, n
               fac = fac + deltag(i)*xxi(ind(i))
               fae = fae + deltag(i)*hdg(i)
               sumdg = sumdg + deltag(i)*deltag(i)
               sumxi = sumxi + xxi(ind(i))*xxi(ind(i))
 155        continue
c
c  ... Skip update if fac not sufficiently positive
c
            if (fac*fac.gt.eps*sumdg*sumxi) then
               fac = 1.0d0/fac
               fad = 1.0d0/fae
c
c  ... Compute DFP update
c
               do 156 i=1, n
                  do 157 j=1, n
                     hessin(i,j) = hessin(i,j) 
     *                  + fac*xxi(ind(i))*xxi(ind(j))
     *                  - fad*hdg(i)*hdg(j)
 157              continue
 156           continue
c
c  ... Add on special BFGS element if BFGS has been chosen.
c
               if (optinp%algrthm.eq.1) then
                  do 160 i=1, n
                     deltag(i) = fac*xxi(ind(i)) - fad*hdg(i)
 160              continue
                  do 158 i=1, n
                     do 159 j=1, n
                        hessin(i,j) = hessin(i,j) 
     *                     + fae*deltag(i)*deltag(j)
 159                 continue
 158              continue
                 write (outfile, 1410)
1410             format(/,'Hessian updated using BFGS formula')
               elseif (optinp%algrthm.eq.2) then
                  write (outfile,1420)
1420              format(/,'Hessian updated using DFP formula')
               endif
            else
               write (outfile,1092)
1092           format(/,'Update skipped because factor too small.')
            endif
         endif

c         write(outfile,1027) hessin(:n,:n)
1027     format(6(1x,d12.5))

 200  continue
c
c  If at this point the iterations have maxed out - print message and 
c    exit routine.
c
      write (outfile, 2200)
2200  format (/5x,'OPTIMIZATION FAILURE: Max iterations reached.',/)
      ierr = 1
c
c  Close molden file                                                    
c
      close (moldenfile,err=9993) 
      return

 300  continue
c
c  Display final step in optimization and print appropriate exit message.
c

      if (status .eq. 1) then
         call optdisp(its, mlinp%geom, ener, grad(:n3tm), xxi(:n3tm),
     *           gcomp, gtol)
         write (outfile, 2000)
      elseif (status .eq. 2) then
         call optdisp(its, mlinp%geom, ener, grad(:n3tm), xxi(:n3tm),
     *           gcomp, gtol)
         write (outfile, 2100)
         ierr = 1
      elseif (status .eq. 3) then
         write (outfile, 2300)
      elseif (status .eq. 4) then
         write (outfile, 2400)
         ierr = 1
      endif

2000  format (/80('*'),/28x,'OPTIMIZATION SUCCESSFUL.',
     *          /80('*'),/)
2100  format (/5x,'OPTIMIZATION FAILURE: Geometry not changing.',/)
2300  format (/80('*'),/26x,'GEOMETRY ALREADY OPTIMIZED.',
     *          /80('*'),/)
2400  format (/5x,'OPTIMIZATION FAILURE: Step Size is 0.',/)
c
c  Close molden file                                                    
c
      close (moldenfile,err=9993)
      return

9993  write (outfile,*) 'Error creating Molden input file'
      stop 'newt 1'
            
      end
c
c***********************************************************************
c  LINMN
c***********************************************************************
c
      subroutine linmn (xl, xil, n, fret, stpmx)
c
c     Subroutine linmn
c
c     Created:  July 15, 1999 by J. M. Rodgers
c               Adapted from the LINMN routines in Ben's IMOHC code
c                 and in POLYRATE.
c     Revised:
c     Purpose:  A line search routine.
c
c     Called by:  newt, newt2
c     Calls:  mnbrak, brent
c
c     Variables defined here:  none.
c
      use files, only : outfile
      use optimize, only : xcom, xicom

      implicit none

      external f1dim

      integer n, ncom, i
      double precision  fret, stpmx, tol, ax, xx, bx, sum
      double precision  fa, fx, fb, brent, xmin, xl(n), xil(n)

      data tol /1.0000d-4/

c         write(outfile,'(/,a)') 'XL at start of LINMN:'
c         write(outfile,1018) xl   
1018     format(3(5x,d12.5))

c         write(outfile,'(/,a)') 'XIL at start of LINMN:'
c         write(outfile,1018) xil   

c      write(outfile,'(/,a)') 'Step initially:'
c      write(outfile,1018) xil

      ncom = n
      do 10 i=1, n
         xcom(i) = xl(i)
         xicom(i) = xil(i)
  10  continue

      ax = 0.0000d0
      if (stpmx .gt. 2.0d0) then
         xx = 1.0d0
         bx = 2.0d0
      else
         xx = stpmx*0.5d0
         bx = stpmx
      endif

      call mnbrak(ax,xx,bx,fa,fx,fb,f1dim)
      fret = brent(ax,xx,bx,f1dim,tol,xmin)

c
c  abs(XMIN) only allowed to be greater than 0.0001
c
      xmin = sign(max(abs(xmin),1.0d-4),xmin)


      do 20 i=1, n
         xil(i) = xil(i)*xmin
  20  continue


      return
      end
c
c***********************************************************************
c  MNBRAK
c***********************************************************************
c
      subroutine mnbrak (ax, bx, cx, fa, fb, fc, func)
c  
c     Subroutine mnbrak
c
c     Created:  July 15, 1999 by J. M. Rodgers
c               Adapted from the MNBRAK routines in Ben's IMOHC code
c                 and in POLYRATE.
c     Revised:
c     Purpose:  Given a function func and distinct initial points ax
c       and bx, this routine searches in the downhill direction and
c       returns new points ax, bx, and cx which bracket a minimum of the
c       function.
c        
c     Called by:  linmn
c     Calls:  func
c
c     Variables defined here:
c     
      use files, only : outfile

      implicit none

      external func

      double precision  ax, bx, cx, fa, fb, fc, fu, r, q, u, a
      double precision  dum, ulim, func, gold, glimit, tiny

      data gold /1.618034d0/, glimit /100.0d0/, tiny /1.0d-20/

1001  format(/5x,'ax = ',d15.8,t40,'fa = ',d22.15,
     *       /5x,'bx = ',d15.8,t40,'fb = ',d22.15,
     *       /5x,'cx = ',d15.8,t40,'fc = ',d22.15,
     *       /5x,'u  = ',d15.8,t40,'fu = ',d22.15 )

      fa = func(ax)
c      write(outfile,1001) ax,fa,bx,fb,cx,fc,u,fu
      fb = func(bx)
c      write(outfile,1001) ax,fa,bx,fb,cx,fc,u,fu

      if (fb .gt. fa) then
         dum = ax
         ax = bx
         bx = dum
         dum = fb
         fb = fa
         fa = dum
      endif

      cx = bx + gold*(bx-ax)
      fc = func(cx)
c      write(outfile,1001) ax,fa,bx,fb,cx,fc,u,fu

  10  if (fb .ge. fc) then
		
         r = (bx-ax)*(fb-fc)
         q = (bx-cx)*(fb-fa)
         u = bx-((bx-cx)*q-(bx-ax)*r)/
     *        (2.0d0*sign(max(abs(q-r),tiny),q-r))
         ulim = bx + glimit*(cx-bx)
	 
         if ((bx-u)*(u-cx) .gt. 0.0d0) then
            fu = func(u)
c            write(outfile,1001) ax,fa,bx,fb,cx,fc,u,fu
            if (fu .lt. fc) then
               ax = bx
               fa = fb
               bx = u
               fb = fu
               goto 10
            elseif (fu .gt. fb) then
               cx = u
               fc = fu
               goto 10
            endif
            u = cx + gold*(cx-bx)
            fu = func(u)
c            write(outfile,1001) ax,fa,bx,fb,cx,fc,u,fu
         elseif ((cx-u)*(u-ulim) .gt. 0.0d0) then
            fu = func(u)
            if (fu .lt. fc) then
               bx = cx
               cx = u
               u = cx + gold*(cx-bx)
               fb = fc
               fc = fu
               fu = func(u)
c               write(outfile,1001) ax,fa,bx,fb,cx,fc,u,fu
            endif
         elseif ((u-ulim)*(ulim-cx) .ge. 0.0d0) then
            u = ulim
            fu = func(u)
c            write(outfile,1001) ax,fa,bx,fb,cx,fc,u,fu
         else
            u = cx + gold*(cx-bx)
            fu = func(u)
c            write(outfile,1001) ax,fa,bx,fb,cx,fc,u,fu          
         endif

         ax = bx
         bx = cx
         cx = u
         fa = fb
         fb = fc
         fc = fu
         goto 10

      endif

      return
      end
c
c***********************************************************************
c  BRENT
c***********************************************************************
c
      function brent (ax, bx, cx, f, tol, xmin)
c
c     function brent
c
c     Created:  July 15, 1999 by J. M. Rodgers
c               Adapted from the BRENT routines in Ben's IMOHC code
c                 and in POLYRATE.
c     Revised:   
c     Purpose:  Given a function func and a bracketing triplet of
c       abscissas ax, bx, and cx (such that bx is between ax and cx and
c       f(bx) is less than both f(ax) and f(cx), this routine isolates the
c       minimum to a fractional precision of about tol using Brent's
c       method.
c
c     Called by:  linmn
c     Calls:  f
c        
c     Variables defined here:
c
      use files, only : outfile
         
      implicit none
         
      external f

      double precision  ax, bx, cx, f, tol, xmin, brent
      double precision  z, b, v, w, x, e, a
      double precision  fx, fv, fw, xm, tol1, tol2, r, q, p
      double precision  etemp, d, u, fu

      integer iter
  
      integer, parameter :: itmax = 100
      double precision, parameter :: cgold = 0.381966d0
      double precision, parameter :: zeps = 1.0d-10

      a = min(ax,cx)
      b = max(ax,cx)
      v = bx
      w = v
      x = v
      e = 0.0d0
      fx = f(x)
      fv = fx
      fw = fx
      do 50 iter = 1, itmax
         xm = 0.5d0*(a+b)
         tol1 = tol*abs(x) + zeps
         tol2 = 2.0d0*tol1

         if (abs(x-xm) .le. (tol2-0.5d0*(b-a)) )  goto 100

         if (abs(e) .gt. tol1) then
            r = (x-w)*(fx-fv)
            q = (x-v)*(fx-fw)
            p = (x-v)*q - (x-w)*r
            q = 2.0d0*(q-r)
            if (q .gt. 0.0d0)  p = -p
            q = abs(q)
            etemp = e
            e = d
            if ( abs(p).ge.abs(0.5d0*q*etemp) .or. p.le.q*(a-x) .or.
     *           p.ge.q*(b-x) )  goto 10
            d = p/q
            u = x + d
            if ( (u-a).lt.tol2 .or. (b-u).lt.tol2)  d = sign(tol1,xm-x)
            goto 20
         endif

  10     if (x.ge.xm) then
            e = a-x
         else
            e = b-x
         endif

         d = cgold*e

  20     if (abs(d) .ge. tol1) then
            u = x+d
         else
            u = x+sign(tol1,d)
         endif

         fu = f(u)

         if (fu .le. fx) then
            if (u .ge. x) then
               a = x
            else
               b = x
            endif
            v = w
            fv = fw
            w = x
            fw = fx
            x = u
            fx = fu
         else
            if (u .lt. x) then
               a = u
            else
               b = u
            endif
            if (fu.le.fw .or. w.eq.x) then
               v = w
               fv = fw
               w = u
               fw = fu
            elseif (fu.le.fv .or. v.eq.x .or. v.eq.w) then
               v = u
               fv = fu
            endif
         endif

  50  continue

      write(outfile, 1001)
1001  format('ERROR:  BRENT exceeded the maximum of 100 iterations.')

 100  xmin = x
      brent = fx

      return
      end
c
c***********************************************************************
c  F1DIM
c***********************************************************************
c
      function f1dim (rfac)
c
c     function f1dim
c
c     Created:  July 15, 1999 by J. M. Rodgers
c               Adapted from the F1DIM routines in Ben's IMOHC code
c                 and in POLYRATE.
c     Revised:
c     Purpose:  Returns the energy specified within mlinp based on a
c        geometry altered according to xcom, xicom, and rfac.
c
c     Called by:  mnbrak, brent, linmn
c     Calls:  
c
c     Variables defined here:
c
      use files, only : outfile
      use input, only : mlinp
      use optimize, only : xcom, xicom, ener

      implicit  none

      double precision  f1dim, rfac
      integer i, n3tm, n

c      double precision, parameter :: angtoau = 0.52918d0
      double precision, parameter :: angtoau = 0.529177249d0
      
      n = mlinp%geom%natoms
      n3tm = 3*n

      mlinp%geom%aucoord(:n3tm) = xcom(:n3tm) + rfac*xicom(:n3tm)
      mlinp%geom%angcoord(:n3tm) = mlinp%geom%aucoord(:n3tm)*angtoau

      call optehk

c         write(outfile,'(/5x,a,d12.5,a)') 'Step Scaled by ',rfac,' to:'
c         write(outfile,1018) rfac*xicom(:n3tm)
1018     format(3(5x,d12.5))

      f1dim = ener

      return
      end
c     
c***********************************************************************
c  MXLNEQ
c***********************************************************************
c
      subroutine mxlneq (a, nn, ida, dett, jrank, eps, in, mm, nabsm)
c  
c     subroutine mxlneq
c  
c     Created:  July 15, 1999 by J. M. Rodgers
c               Adapted from the MXLNEQ routines in Ben's IMOHC code
c                 and in POLYRATE.
c     Revised:
c     Purpose:  Solves for the matrix X in C*X = B.  C must be a square
c        matrix.  A(NN,NN+abs(MM)) contains matrices C and B, A(i,j) =
c        C(i,j), A(i,j+NN) = B(i,j)
c
c       IDA = first dimension of A
c       NN = order of matrix C (i.e. # of eqns)
c       IN(NN) = work array
c       MM = 0 is used to calculate the inverse of C
c
c       Notice that NABSM = NN + abs(MM)
c
c PROGRAMMED BY R. HOTCHKISS, U. COMP. CTR., U. OF MINN., REVISED OCT.73
c      modified for the VAX by Bruce Garrett, Nov. 1980
c
c    Dimensions changed by Tom Joseph Jan. 1988.
c
c     Called by:  newt2
c     Calls:
c
c     Variables defined here:
c     
      use files, only : outfile

      implicit none

      integer  nn, ida, jrank, mm, nabsm, intmx, id, in(ida)
      integer  nm, nmida, k1, nflag, mod, k, l, i
      integer  j, i1, m, n
      double precision  dett, eps, machep, ops, det, a(ida,nabsm)
      double precision  zero, one, pivot, piv, z, p

      data  zero, one  / 0.0d0, 1.0d0 / , machep /1.0d+38/
      data  intmx  /400000/
c
c ?
c
      save
c
c  Initiate some local and output variables
c
      jrank = nn
      n = nn
      id = ida
      ops = eps
c
c  Error checking
c
      if (n.gt.0 .and. n.le.id)  goto 10
      write (outfile,1700)
      write (outfile,1000) n, id
      stop 'mxlneq 1'
  10  if (id.ge.n .and. id.lt.intmx)  goto 20
      write (outfile,1700)
      write (outfile,1100) id, n, intmx
      stop 'mxlneq 2'
  20  if ((one+ops).ne.one .and. ops.ge.zero)  goto 30
      write (outfile,1700)
      write (outfile,1200)  ops
      stop 'mxlneq 3'
  30  mod = mm
      nm = abs(mm)
      if (nm.lt.intmx)  goto 40
      write (outfile,1700) 
      write (outfile,1300)  mm, intmx
      stop 'mxlneq 4'
  40  nm = nn + nm
      nmida = nm*id*2
      if (nmida .lt. intmx)  goto 50
      write (outfile, 1700)
      write (outfile, 1400)  nmida, intmx
  50  continue
c
c  Continue with initialization
c
      k1 = 1
      nflag = 0
      det = one
c
c  Main Gauss-Jordan loop begins
c
      do 150 k=1, n
c
c  Search for largest pivot candidate in remaining lower right square
c    matrix.
c
         piv = zero
         l = k
         do 60 i=k, n
            p = abs(a(i,k))
            if (piv.ge.p)  goto 60
            piv = p
            l = i
  60     continue
c
c  Pivot with abs value piv and subscripts l and k has been found.
c	
         pivot = a(l,k)
c
c  Continue if piv larger than user eps
c
         if (piv.gt.ops)  goto 80
         if (eps.eq.ops)  jrank = k-1
c
c  EPS test failed, check for zero pivot
c
         if (piv.gt.zero)  goto 70
c
c  Pivot is zero, terminate program
c
         write (outfile, 1700)
         write (outfile, 1500) k
         stop 'mxlneq 6'
c
c  issue non-fatal message, piv .le. eqs
c
  70     write (outfile, 1700)
         write (outfile, 1600) k, eps
c
c  set ops to 0 so solution may continue after error message
c
         ops = zero
         piv = pivot
c
c  calculate determinant and check for overflow
c
  80     det = pivot*det
         if (abs(det).lt.machep)  goto 90
         write (outfile,1700)
         write (outfile,1800) k, det
         stop 'mxlneq 7'
c
c  Reset leading row do index for det only and lin eqn only case
c
  90     if (mod.lt.0) k1 = k
c
c  Save pivot index
c
         in(k) = l
c
c  Check for row interchange
c
         if (l.eq.k)  goto 110
         det = -det
c
c  Interchange row containing pivot and current row.
c  Only partial rows need be exchanged for det only or lin eqn only
c    since lower left partially formed triangle is not needed.
c
         do 100 j=k1, nm
            z = a(l,j)
            a(l,j) = a(k,j)
            a(k,j) = z
 100     continue
c
c  Pivot element is now on diagonal.  save division time by using
c    reciprocal of pivot.
c
 110     pivot = one/pivot
c
c  Pre-divide necessary portion of pivot row
c
         do 120 j=k1, nm
            a(k,j) = a(k,j)*pivot
 120     continue
c
c  Set pivot element to 0 so main reduction step doesn't operate on pivot
c    row
c
         a(k,k) = zero
c
c  Sweep through all or part of matrix using kth row, pivot row, to reduce
c    the matrix.
c
         do 140 i=k1, n
            z = a(i,k)
            if (z.eq.zero) goto 140
c
c  This check not only prevents operating on pivot row, but catches other
c    zeroes in pivot columns.  Thes other zeroes would leave jth row
c    unchanged in following loop so considerable time may be saved by
c    skipping operation.
c
            do 130 j=k1, nm
               a(i,j) = a(i,j) - z*a(k,j)
 130        continue
c
c  The inverse is created in place by substituting an identity matrix col
c    by col, since we are substituting the pivot row from off diagonal 0
c    elements at this point, we now, place -A(i,k)/A(k,k) at this point
c    in the pivot col. 
c
            a(i,k) = -z*pivot
 140     continue
c
c  Similarly dividing pivot row by the pivot is equivalent to placing
c    1/a(k,k) at the pivot point for the inverse.
c
         a(k,k) = pivot
 150  continue
      if (n.eq.1)  goto 210
      if (mod.ge.0)  goto 180
c
c  Back substitution for lin eqn only case
c
      k1 = k1+1
      do 170 k=k1, nm
         i = n
         do 170 l=2, n
            i1 = i
            i = i-1
            z = zero
            do 160 j=i1, n
               z = z+a(i,j)*a(j,k)
 160        continue
            a(i,k) = a(i,k) - z
 170  continue

      goto 210
c
c  Final reordering of the matrix
c
 180  k = n
c
c  Skip last step since no interchange could occur there
c
      do 200 j=2, n
c
c  Perform interchanges in exact reverse order of previous execution
c
         k = k-1
c
c  Row interchange during inversion implies col interchange here
c
         m = in(k)
         if (m.eq.k)  goto 200
c
c  Col interchange
c
         do 190 i=1, n
            z = a(i,k)
            a(i,k) = a(i,m)
            a(i,m) = z
 190     continue
 200  continue

 210  dett = det
      return

1000  format(11H arg 2, n = ,I10/7X,4H id=,I10/
     *       29H n must be .ge. 1 and .le. id)
1100  format(12H arg 3, id = ,I10 /9X,3H n=,I10/
     *       27H id must be .ge. N and .le.,I10)
1200  format(13H arg 6, eps = ,1PE13.5/
     *       33H eps must be .ge. zero and finite)
1300  format(11H arg 8, m = ,I10/
     *       20H abs(m) must be .le. ,I10 )
1400  format(7H size = ,I10/
     *       36H size = id*(n+abs(m))*2 must be .le. ,I10)
1500  format(4H k =,I10/
     *       46H at step k a gauss-jordan pivot value was zero)
1600  format(4H k = ,I10/6H eps = ,1PE13.5/
     *       50H at step k a gauss-jordan pivot value was .le. eps )
1700  format(15H *** mxlneq *** )
1800  format(4H k = ,I10/6H det = ,1PE13.5/
     *        27H at step k det is too large )

      end
c
c******************************************************************************
c  PROGOHK
c******************************************************************************
c
      subroutine progohk( geom, optinp, prognme, ierr )
c
c     Subroutine progohk
c
c     Created:  Feb. 21, 2005 by Hai Lin
c     Revised:  Apr. 13, 2006 by Yan Zhang
c               May  22, 2006 by Yan Zhang
c
c     Purpose:  To call the specified external program to optimize the
c       geometry at the specified theory and basis level.
c     
c     Note:  Currently only G03 is enabled but should be easily expandable
c
c     Called by:  main
c     Calls:  g03inp, g03outo, system, orcainp, orcaoute, orcaoutg, orcaouto
c
c     Variables defined here:  none.
c
      use input, only : cartgeom, extopt
      use caltyp, only : topt,tgrad
      use files

      implicit none

      character*10     :: prognme
      type (cartgeom)  :: geom, oldgeom
      type (extopt)    :: optinp
      integer          :: ierr, i
      double precision :: ener, grad(3*geom%natoms)
      character*20 ver_lab 

      oldgeom = geom
      ierr=0
c 
c  GAUSSIAN03  
c
c      if (prognme(:3).eq.'g03') then
      if ((prognme(:3).eq.'g03') .or. (prognme(:3).eq.'g09')            1020XW17
     * .or. (prognme(:3).eq.'g16')) then                                1020XW17
         open (unit=extoinf, status='unknown', file=extin, err=9991)
         call g03inp(extoinf, optinp%meth, optinp%bas, 
     &            geom%charge, geom%mult, geom,
     *            optinp%optl, topt)
         close (extoinf, err=9991)
         call system('g03shuttle extopt.inp extopt.out')
         call g03outo(geom, ener, grad, ierr)                              
         if (ierr.eq.1)  return

c 
c   ORCA  4/13/2006 by Yan Zhang
c  
      else if(prognme(:4).eq.'orca') then
         open (unit=extoinf, status='unknown', file=extin, err=9992)
         call orcainp(extoinf, optinp%meth, optinp%bas, 
     *            geom%charge, geom%mult, geom,
     *            optinp%optl, topt,ver_lab)
         close (extoinf, err=9992)
         call system('orcashuttle extopt.inp extopt.out')
         call orcaouto(geom,ierr)
c	                                 
c	calculate  energy with optimized coordinate
c
      if (ierr.eq.1)  return

         open (unit=orcainf, status='unknown', file=orcain, err=9993)
         call orcainp(orcainf, optinp%meth, optinp%bas, 
     *       geom%charge, geom%mult,
     $	   geom, optinp%optl,tgrad,ver_lab)
         close (orcainf, err=9993)
         call system('orcashuttle orca.inp orca.out')
         call orcaoute(1,optinp%meth, ener)                                      
         call orcaoutg(1,optinp%meth, grad, geom%natoms,geom%natoms)                         
c
c  end  4/13/2006 by Yan Zhang
c
c
c   Gamess   5/22/2006 by Yan Zhang
c
      else if (prognme(:6).eq.'gamess') then
         open (unit=extoinf, status='unknown', file=extin, err=9994)
         call gmsinp(extoinf,optinp%meth,optinp%bas,
     *      geom%charge, geom%mult,
     &      geom,optinp%optl,topt,ver_lab)
         close(extoinf,err=9994)
         call system('gmsshuttle extopt.inp extopt.out')
         call gmsouto (geom, ener,grad,geom%natoms,ierr)
         if (ierr.eq.1)  return
c
c   end 5/22/2006  by Yan Zhang
C

      endif
c
c  Display the results of the optimization
c
      write(outfile,1100)  prognme
      write(outfile,1150)  optinp%meth
      write(outfile,1200)  optinp%bas
      write(outfile,1250)  ener
      write(outfile,1300)
      write(outfile,1350)  grad
      write(outfile,1400)
      write(outfile,1450)
      do 20 i=1, geom%natoms
         write(outfile,1500) geom%atm(i), geom%aucoord(3*i-2),
     *       geom%aucoord(3*i-1), geom%aucoord(3*i)
  20  continue
      write(outfile,1550)
      do 40 i=1, geom%natoms
         write(outfile,1500) geom%atm(i), geom%angcoord(3*i-2),
     *       geom%angcoord(3*i-1), geom%angcoord(3*i)
  40  continue

1100  format(/2(/80('-')),//2x,'External Optimization with ',
     *         a,//80('-')/)
1150  format(/2x,'Theory:',t30,a)
1200  format(2x,'Basis Set:',t30,a)
1250  format(/2x,'Optimized Energy:',t30,f19.12)
1300  format(/2x,'Optimized Cartesian Gradient:')
1350  format(t20,d15.8,t40,d15.8,t60,d15.8)
1400  format(/2x,'Optimized Geometry:')
1450  format(//5x,'Atomic Coordinates (in a.u.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)
1500  format(9x,a2,t20,d15.8,t40,d15.8,t60,d15.8)
1550  format(//5x,'Atomic Coordinates (in ang.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)  

      return

c9991  write (outfile,*) 'Error creating Gaussian03 Opt input file'
9991  write (outfile,*) 'Error creating Gaussian Opt input file'        1020XW17
      stop 'progohk 1'

c 
c   ORCA  4/13/2006 by Yan Zhang
c  
9992  write (outfile,*) 'Error creating orca opt input file'
      stop 'progohk 2'

9993  write (outfile,*) 'Error creating orca gradient input file'
      stop 'progohk 3'

9994  write (outfile,*) 'Error creating Gamess input file'
      stop 'progohk 4'

c
c  end  4/13/2006 by Yan Zhang
c

      end
c
c******************************************************************************
c  g03OUTO
c******************************************************************************
c
      subroutine g03outo (geom, ener, grad, ierr)
c
c     Subroutine g03outo
c
c     Created:  Feb. 22, 2005  by Hai Lin
c     Modified: Dec. 21, 2010  by Bo Wang. The subroutine is reorganized
c                to be compatible with both G03 and G09. G03 and G09
c                Test.fchk files have energy, geometry,
c                gradient, and hessian appeared in different order
c                0203BW15
c     Purpose:  Get the energy, gradient, and optimized geometry out of
c        the Gaussian formatted checkpoint file.
c
c        Optimization can be considered to be successful sometimes for
c        large molecules based on solely on the Force
c        (without Displacement).
c
c     Called by:  progohk, gau_ext_opt
c     Calls:
c
      use files
      use numfmt
      use input, only : cartgeom

      implicit none
c
c i/o variables
c
      type (cartgeom)  :: geom
      double precision    ener, grad(3*geom%natoms)
      integer             ierr
c local variables
c
      double precision   cfltfmt

      integer      i, ctr, j, gstrt, gend, opt
c
      character*80 string, case
      character*30 label
      character*22 etxt
      character*15 gtxt(3*geom%natoms), otxt(3*geom%natoms)

c      double precision, parameter :: angtoau = 0.52918d0
      double precision, parameter :: angtoau = 0.529177249d0
      logical finde, findc, findg
c
c  Check that the optimization is complete.
c
      opt  = 0

      open( unit=extoout, status='unknown', file=extout, err=9920 )

c
      read( extoout, '(a80)', end=9930 ) string
      string = case(string)
c
c change the search string to include
c   "Optimization completed on the basis of negligible forces"
c
      do while( string(:23) .ne. ' optimization completed')
c
         read( extoout, '(a80)', end=9930 ) string
         string = case(string)
      end do

      close ( unit=extoout )
c
      write(outfile,*) string(1:80)

      finde = .false.
      findc = .false.
      findg = .false.
c
c  Open the formatted checkpoint file.
c
      open( unit=fchk, status='unknown', file=testfchk, err=9900)
c
c  Find energy
c
100   read( fchk, '(a80)', end=140 ) string
      string = case(string)
      label = string(:llen(efmt))

      if ( label(:12).ne.'total energy' ) then
          goto 110
      else
        etxt = string(nstrt(efmt):nstrt(efmt)+nlen(efmt)-1)
        ener = cfltfmt(etxt,efmt)
        finde = .true.
        goto 140
      endif
110   continue

c
c  Find geometry
c
      if ( string(:29) .ne. 'current cartesian coordinates') then
         goto 120
      else
        ctr = 1
        do while( ctr .le. 3*geom%natoms )
c
           read( fchk, '(a80)', end=9910 ) string
           string = case(string)
           i = 0
           do while( ctr.le.3*geom%natoms .and. i.lt.5 )
              gstrt = i*(nlen(gfmt)+1)+2
              gend  = (i+1)*(nlen(gfmt)+2)
              otxt(ctr) = string(gstrt:gend)
              ctr = ctr+1
              i = i+1
           end do
        end do

        do 10 j=1, 3*geom%natoms
           geom%aucoord(j) = cfltfmt(otxt(j),gfmt)
  10    continue
        geom%angcoord = geom%aucoord*angtoau
        findc = .true.
        goto 140
      endif
120   continue

c
c  Find gradient
c
      if ( string(:18).ne.'cartesian gradient' .and.
     *              string(:16).ne.'cartesian forces' ) then
         goto 130
      else
        ctr = 1
        do while( ctr .le. 3*geom%natoms )
c
           read( fchk, '(a80)', end=9910 ) string
           string = case(string)
           i = 0
           do while( ctr.le.3*geom%natoms .and. i.lt.5 )
              gstrt = i*(nlen(gfmt)+1)+2
              gend  = (i+1)*(nlen(gfmt)+2)
              gtxt(ctr) = string(gstrt:gend)
              ctr = ctr+1
              i = i+1
           end do
        end do

        do 20 j=1, 3*geom%natoms
         grad(j) = cfltfmt(gtxt(j),gfmt)
  20    continue
        findg = .true.
        goto 140
      endif
130   continue
140   if (finde .and. findc .and. findg ) then
        goto 150
      else
        goto 100
      endif

150   close( unit=fchk )

      return

c9900  write( outfile,*) 'Error opening Gaussian03 checkpoint file.'
9900  write( outfile,*) 'Error opening Gaussian checkpoint file.'       1002XW17
      stop 'g03outo 1'
c9910  write( outfile,*) 'Error reading Gaussian03 checkpoint file.'
9910  write( outfile,*) 'Error reading Gaussian checkpoint file.'       1002XW17
      stop 'g03outo 2'
c9920  write( outfile,*) 'Error opening Gaussian03 output file.'
9920  write( outfile,*) 'Error opening Gaussian output file.'           1002XW17
      stop 'g03outo 3'

9930  ierr=1
      return

      end
c
c******************************************************************************
c  PROGMMOHK
c******************************************************************************
c
      subroutine progmmohk( geom, mmgeom, optinp, prognme, ierr )
c
c     Subroutine progmmohk
c
c     Created:  Feb 22, 2004 by Hai Lin
c
c     Purpose:  To call the specified external program to optimize the
c       geometry at the specified theory and basis level.
c     
c     Note:  Currently only Tinker is enabled but should be easily expandable
c
c     Called by:  main
c     Calls:  t41inp, t41outo, system
c
      use input, only : cartgeom, mmvalen, extopt
      use caltyp, only : topt
      use files
c
      implicit none
c
c  i/o variables
c
      character*10     :: prognme
      type (cartgeom)  :: geom
      type (mmvalen)   :: mmgeom
      type (extopt)    :: optinp
      integer          :: ierr
c
c  local variables
c
      type (cartgeom)  :: oldgeom
      type (mmvalen)   :: oldmmgeom
      integer          :: i, j, k, lenbase, lenoptcommd
      double precision :: ener, grad(3*geom%natoms)
      character*20        ver_lab, smmconverg      
      character*40        optcommd      
c
      oldgeom   = geom
      oldmmgeom = mmgeom
      smmconverg = optinp%smmconverg
c
c Write the mm convergence threshold
c
      write(outfile,*) 'Convergence threshold for MM optimization: ', 
     * smmconverg
c
c  TINKER 
c
      if (prognme(:6).eq.'tinker') then
c
c Build the command line for calling Tinker
c
         optcommd  = 't41shuttle t41 o '
         lenbase = 17
         k = lenbase + 1
         lenoptcommd = len(optcommd)
         do j = k, lenoptcommd
            optcommd(j:j) = ' ' 
         enddo
	     
         i = len(smmconverg)
         do j = 1, i
            optcommd(j+k:j+k) = smmconverg(j:j)
         enddo		        
c
c write .xyz and .key files    
c      
         open (unit=t41inf, status='unknown', file=t41in, err=9994)
         open (unit=t41keyf, status='unknown', file=t41key, err=9995)
         call t41inp(t41inf, t41keyf,
     *    optinp%meth, optinp%bas, geom, mmgeom, optinp%optl,
     *    topt, ver_lab)
         close (t41inf, err=9994)
         close (t41keyf, err=9995)        	    
c
c read the atomic MM charge 
c  also print MM parameters if required
c 	 
         call system('t41shuttle t41 p')
         call t41outp(mmgeom)
c
c calculate the energy and gradient using the input geometry
c
         write(outfile,*) ' ' 	
         write(outfile,*) 
     *    '----- Energy and gradient using input geometry -----'

         call system('t41shuttle t41 e')
         call t41oute(1, optinp%meth, ener)
         call system('t41shuttle t41 g')
         call t41outg(1, optinp%meth, ener, grad, geom%natoms)
         write(outfile,*) 'Gradients (in au) at input geometry:'
         write(outfile,1350)  grad
      	 
c
c Now do the optimization
c	 	 
         write(outfile,*) ' '
         write(outfile,*) 
     *    '--------- Optimization performing with TINKER --------'
         write(outfile,*) ' '

         call system(optcommd)
         call t41outo(ener, geom, ierr)

         write(outfile,*) ' '	 
         if(ierr .ne. 1) then
            write(outfile,*) 
     *    '------- Optimization with TINKER is successful ------'
         else
            write(outfile,*) 
     *    '--------- Optimization with TINKER fail !! --------'
            goto 1000
        endif     	    	 	 
        write(outfile,*) ' '
c
c re-compute and output energy and gradient using optimized geometryc		  	 	 
c
c re-write .xyz and .key files    
c      
         open (unit=t41inf, status='unknown', file=t41in, err=9994)
         open (unit=t41keyf, status='unknown', file=t41key, err=9995)
         call t41inp(t41inf, t41keyf,
     *    optinp%meth, optinp%bas, geom, mmgeom, optinp%optl,
     *    topt, ver_lab)
         close (t41inf, err=9994)
         close (t41keyf, err=9995)        	    

         write(outfile,*) ' '
         write(outfile,*) 
     *    '----- Energy and gradient using optimized geometry -----'
	      	      						       
         call system('t41shuttle t41 e')
         call t41oute(1, optinp%meth, ener)
         call system('t41shuttle t41 g')
         call t41outg(1, optinp%meth, ener, grad, geom%natoms)
         write(outfile,*) 'Gradients (in au) at optimized geometry:'
         write(outfile,1350)  grad

         write(outfile,*) ' '
         write(outfile,*)
     *    '----- exit the TINKER program -----'
     	 	 	 
      endif
            
1000  if (ierr.eq.1)  return

c
c  Display the results of the optimization
c
      write(outfile,1100)  prognme
      write(outfile,1150)  optinp%meth
      write(outfile,1200)  optinp%bas
      write(outfile,1250)  ener
      write(outfile,1300)
      write(outfile,1350)  grad
      write(outfile,1400)
      write(outfile,1450)
      do 20 i=1, geom%natoms
         write(outfile,1500) geom%atm(i), geom%aucoord(3*i-2),
     *       geom%aucoord(3*i-1), geom%aucoord(3*i)
  20  continue
      write(outfile,1550)
      do 40 i=1, geom%natoms
         write(outfile,1500) geom%atm(i), geom%angcoord(3*i-2),
     *       geom%angcoord(3*i-1), geom%angcoord(3*i)
  40  continue

1100  format(/2(/80('-')),//2x,'External Optimization with ',
     *         a,//80('-')/)
1150  format(/2x,'Theory:',t30,a)
1200  format(2x,'Force field:',t30,a)
1250  format(/2x,'Optimized Energy:',t30,f19.12)
1300  format(/2x,'Optimized Cartesian Gradient:')
1350  format(t20,d15.8,t40,d15.8,t60,d15.8)
1400  format(/2x,'Optimized Geometry:')
1450  format(//5x,'Atomic Coordinates (in a.u.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)
1500  format(9x,a2,t20,d15.8,t40,d15.8,t60,d15.8)
1550  format(//5x,'Atomic Coordinates (in ang.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)  
	    
      return
c
9994  write (outfile,*) 'Error creating TINKER .xyz file'
      stop 'progehk 4'
9995  write (outfile,*) 'Error creating TINKER .key file'
      stop 'progehk 5'
      end
c
c
c*****************************************************************************
c  T41OUTO
c*****************************************************************************
c  
      subroutine t41outo (ener, geom, ierr)
c
c     Subroutine t41outh
c
c     Created:  Feb. 22, 2004 by Hai Lin
c     Revised:
c     Purpose:  Get the optimized geometry out of the TINKER output file.
c
c     Called by:  progmmohk
c     Calls:
c     
      use files
      use numfmt
      use input, only : cartgeom
      implicit none
c
c  i/o variables
c      
      type (cartgeom)  :: geom
      double precision    ener
      integer             ierr
c
c  local variables
c
      integer          i, j 
c
      character*80     string
      integer          natoms, natom3 
      integer          nwords, isec, ieof, iscend
      integer          ioptfind
      integer          icint
      double precision au2ang, ang2au
      double precision xyzread
      double precision cfloat
c
      character*80     words(40)
c
      au2ang = 0.529177249d0
      ang2au = 1.0d0 / au2ang
c
      open(unit=t41outf,status='unknown', file=t41out, err=9900)
      ieof = 0
      nwords = 6
      ioptfind = 0
c
c Search for 'TNCG  --  Normal Termination due to SmallGrad'
c
        
      do while (ieof .ne. 1)
         call readln(t41outf,words,nwords,isec,ieof,iscend)

         if(words(1) .eq. 'TNCG'          .and. 
     *      words(2) .eq. '--'            .and.
     *      words(3) .eq. 'NORMAL'        .and.
     *      words(4) .eq. 'TERMINATION'   .and.
     *      words(5) .eq. 'DUE'           .and.
     *      words(6) .eq. 'TO'            .and.
     *      words(7) .eq. 'SMALLGRAD')     then
            ioptfind = 1
            goto 100
         endif
      enddo

100   if(ioptfind .eq. 0) then
         write(outfile, *) 
     *      'Error: TINKER optimization failed.'
         ierr = 1
         return
c         stop 't41outo 4'
      endif
c
c  Read three lines for statistical info.
c
      do i = 1, 3
         call readln(t41outf,words,nwords,isec,ieof,iscend)
         write(outfile,'(4(a10),a40)') 
     *      words(1), words(2), words(3), words(5)
      enddo
c
c  Read the number of atoms
c
      call readln(t41outf,words,nwords,isec,ieof,iscend)
      natoms = icint(words(1))
      natom3 = natoms * 3
      if(natoms .ne. geom%natoms) then 
         write(outfile, *)
     *      'Error: the number of atoms does not match ...'
         stop 't41outo 5'
      endif
c
c  Read the optimized geometry (fields 3 - 5)
c         
      i = 0
      do while (ieof .ne. 1)
         if(i .ge. natom3) goto 200
         call readln(t41outf,words,nwords,isec,ieof,iscend)
         do j = 3, 5 
            i = i + 1
            xyzread = cfloat(words(j))
            geom%angcoord(i) = xyzread 
            geom%aucoord(i) = xyzread * ang2au
         enddo   
      enddo

200   if(i .gt. natom3) then
         write(outfile, *)
     *      'Error: Too many TINKER cartesian coordinates ...'
         stop 't41outh 6'
      endif
      	      	   
      close( unit=t41outf )
      
      return

9900  write( outfile,*) 'Error opening TINKER output file'
      stop 't41outo 1'

      end
c
c
c******************************************************************************
c  OPTEHK
c******************************************************************************
c
      subroutine optehk
c
c     Subroutine optehk
c
c     Created:  Feb 22, 2005  by Hai Lin
c
c     Purpose:  Based on the specifed ML optimization method, call the
c       appropriate sub ehook and set ener to the appropriate value.
c
c     Called by:  f1dim
c     Calls:  qmmmehk, progehk
c     
      use energy
      use input,    only : mlinp
      use optimize, only : ener
      use caltyp,   only : tener

      implicit none
c
c  i/o variables   none
c  local variables 
c
      integer  ierr

c TEST
      if (mlinp%mlmeth(:4).eq.'test') then
         call progehk(mlinp%progrms%testprog,
     *      mlinp%tmeth,mlinp%tbas,
     *      mlinp%geom%charge,mlinp%geom%mult,
     *      mlinp%geom , mlinp%toptl, ener, ierr)
         call testdisp ( mlinp%geom%natoms, tener )
c QMMM                                                                  
      elseif (mlinp%mlmeth(:4).eq.'qmmm') then
         call qmmmehk(mlinp%progrms%qmprog, mlinp%progrms%mmprog,
     *      mlinp%geom, mlinp%mmgeom, mlinp%qmmminp, ierr)
      ener = qmmmener
		           
      endif

      return
      end
c
c******************************************************************************
c  OPTGHK
c******************************************************************************
c
      subroutine optghk
c
c     Subroutine optghk
c
c     Created:  Feb 22, 2005  by Hai Lin
c
c     Purpose:  Based on the specifed ML optimization method, call the
c       appropriate sub ghookk and set ener and grad to the appropriate
c       values.
c
c     Called by:  newt, newt2
c     Calls:  progghk, qmmmghk
c     
c     Variables defined here:  none.
c    
      use optimize, only : ener, grad
      use input, only : mlinp
      use energy
      use gradient
      use caltyp, only : tgrad
 
      implicit none

      integer  n3tm, natoms, ierr

      natoms = mlinp%geom%natoms
      n3tm = 3*natoms

c TEST
      if (mlinp%mlmeth(:4).eq.'test') then
c      if (mlinp%mlmeth(:2).eq.'qm') then
         call progghk(mlinp%progrms%testprog,mlinp%tmeth,
     *      mlinp%tbas,mlinp%geom%charge,mlinp%geom%mult,
     *      mlinp%geom , mlinp%toptl, ener, grad(:n3tm), ierr)
         call testdisp ( mlinp%geom%natoms, tgrad )
c QMMM                                                                  
      elseif (mlinp%mlmeth(:4).eq.'qmmm') then
         call qmmmghk(mlinp%progrms%qmprog, mlinp%progrms%mmprog,
     *	    mlinp%geom, mlinp%mmgeom, mlinp%qmmminp, ierr) 
         ener = qmmmener	 
         grad(:n3tm) = qmmmgrad(:n3tm)
	 	         
      endif

      return
      end
c
c******************************************************************************
c  OPTHHK
c******************************************************************************
c
      subroutine opthhk
c
c     Subroutine opthhk
c
c     Created:  Feb 22, 2005  by Hai Lin 
c     Revised:  Apr 21, 2006  by Yan Zhang
c
c     Purpose:  Based on the specifed ML optimization method, call the
c       appropriate sub hhk and set ener, grad, and hess to the
c       appropriate values, if a high level hessian has been requested.
c       If a low level hessian or a scaled unit matrix has been requested
c       then only hess is updated 
c
c     Called by:  newt
c     Calls:  proghhk, qmmmhhk
c     
c     Variables defined here:  none.
c     
      use optimize, only : ener, grad, hess
      use input,    only : mlinp
      use energy
      use gradient
      use hessian
      use param,    only : lowqmhs, uniths,qmmmhs,mmhs
      use caltyp,   only : thess
 
      implicit none

      double precision dum
      double precision, dimension(:), allocatable :: dum2
      integer  n3tm, natoms, ierr, i, j
      character*20 ver_lab

      natoms = mlinp%geom%natoms
      n3tm = 3*natoms

      allocate(dum2(n3tm))

c
c Unit Hessian scaled by hscale
c
      if (mlinp%htyp.eq.uniths) then
         do 21 i=1, n3tm
            do 22 j=1, n3tm
               hess(i,j) = 0.0d0
  22        continue
            hess(i,i) = mlinp%hscale
  21     continue
c
c Low level Hessian - make sure that not set energy and gradient to low
c   level values as well.
c
c   Use of a low QM level specified by the user to obtain a Hessian
c
c	 4/21/2006 revised by Yan Zhang

c      elseif (mlinp%htyp.eq.lowhs) then 
c
      elseif (mlinp%htyp.eq.lowqmhs) then 
         call proghhk(mlinp%progrms%hessprog,mlinp%hmeth,mlinp%hbas,
     *      mlinp%geom%charge,mlinp%geom%mult,
     *      mlinp%geom, mlinp%toptl, dum, dum2,
     *      hess(:n3tm,:n3tm), ierr)
         call lowdisp( mlinp%geom%natoms )
c
c Otherwise will be doing a high level Hessian from one of these
c  methods.
c
c TEST
c
c   Now this keyword is not supported
c 
c      elseif (mlinp%mlmeth(:4).eq.'test') then
c         call proghhk(mlinp%progrms%testprog,
c     *      mlinp%tmeth,mlinp%tbas,
c     *      mlinp%geom , mlinp%toptl, ener, grad(:n3tm),
c     *      hess(:n3tm,:n3tm), ierr)
c         call testdisp( mlinp%geom%natoms, thess )

c QMMM                    	                                        
c   Calculated the Hessian at the QM/MM level
c
c      elseif (mlinp%mlmeth(:4).eq.'qmmm') then
c
      elseif (mlinp%htyp.eq.qmmmhs) then
         call qmmmhhk(mlinp%progrms%qmprog, mlinp%progrms%mmprog,
     *     mlinp%geom, mlinp%mmgeom, mlinp%qmmminp, ierr)
         ener = qmmmener
         grad(:n3tm) = qmmmgrad(:n3tm)
         hess(:n3tm,:n3tm) = qmmmhess(:n3tm,:n3tm)

c MM
c   Calculated the Hessian at the MM level
c

      elseif (mlinp%htyp.eq.mmhs) then
         call progmmhhk(mlinp%progrms%mmprog, mlinp%qmmminp%mmmeth, 
     *          mlinp%qmmminp%mmbas,mlinp%geom, 
     *	     mlinp%mmgeom, mlinp%qmmminp%mmoptl, 
     *         ener, 
     *      grad(:n3tm), hess(:n3tm,:n3tm), 
     *      ierr, ver_lab)
c
c     end 4/21/2006 by Yan Zhang
c
      endif

      return
      end
c
c******************************************************************************
c  OPTDISP
c******************************************************************************
c
      subroutine optdisp( iter, geom, ener, grad, xxi, gcomp, gtol )
c
c     Subroutine optdisp
c
c     Created:  July 16, 1999 by J. M. Rodgers
c     Revised:
c     Purpose:  Display all information for the current optimization step.
c
c     Called by:  newt, newt2
c     Calls:  
c
c     Variables defined here:  none.
c
      use files, only : outfile
      use input, only : cartgeom

      implicit none

      type (cartgeom) :: geom

      integer iter, i
      double precision  ener, grad(3*geom%natoms), xxi(3*geom%natoms)
      double precision  gcomp, gtol

      write(outfile,1000)  iter
      write(outfile,1150)  ener
      write(outfile,1200)
      write(outfile,1250)  grad
      write(outfile,1300)  gcomp, gtol
      write(outfile,1350)
      write(outfile,1250)  xxi
      write(outfile,1450)
      do 20 i=1, geom%natoms
         write(outfile,1500)  geom%atm(i), geom%aucoord(3*i-2),
     *       geom%aucoord(3*i-1), geom%aucoord(3*i)
  20  continue
      write(outfile,1550)
      do 40 i=1, geom%natoms
         write(outfile,1500)  geom%atm(i), geom%angcoord(3*i-2),
     *      geom%angcoord(3*i-1), geom%angcoord(3*i)
  40  continue
      write(outfile,*)

1000  format(/2x,14('-'),/2x,'Iteration: ',i3,/2x,14('-')/)
1150  format(/2x,'Energy:',t30,f19.12)
1200  format(/2x,'Cartesian Gradient:')
1250  format(t20,d15.8,t40,d15.8,t60,d15.8)
1300  format(/2x,'Maximum Force = ',d10.3,t40,
     *          'Convergence Requirement = ',d10.3)
1350  format(/2x,'Scaled Step:')
1450  format(//2x,'Atomic Coordinates (in a.u.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)
1500  format(9x,a2,t20,d15.8,t40,d15.8,t60,d15.8)
1550  format(//2x,'Atomic Coordinates (in ang.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)  

      return
      end
c
c***********************************************************************
c  LUDCMP
c***********************************************************************
c
      SUBROUTINE ludcmp(a,n,np,indx,d)
c           
c     subroutine ludcmp
c
c     Created:  July 22, 1999 by J. M. Rodgers
c               From ludcmp in Numerical Recipes
c     Revised:
c     Purpose:  Carry out an LU decomposition of the matrix a
c
c       a    - on entry the matrix a, will be destroyed on exit
c              on exit the LU decomposition of the matrix
c       n    - order of the matrix a
c       np   - the leading dimension of the array a >= max(1,n)
c       indx - a work  array containing the row
c       d    - on exit +/- 1 depending on whether odd/even # of
c                 interchanges
c
c     Called by:  newt
c
      use files, only : outfile

      implicit none

      integer  n, np, indx(np), i, imax, j, k
      double precision  d, a(np,np), aamax, dum, sum
      double precision, dimension(:), allocatable :: vv

      double precision, parameter :: one = 1.0d0, zero = 0.0d0,
     *  tiny = 1.0d-20

      allocate( vv(np) )
c
c  No row interchanges yet.
c
      d=one
c
c  Loop over the rows to get the implicit scaling information and save it
c    in vv.
c
      do 12 i=1, n
         aamax = zero
         do 11 j=1, n
            if (abs(a(i,j)) .gt. aamax)  aamax = abs(a(i,j))
  11     continue
         if (aamax.eq.zero)  then
            write (outfile,*) 'ERROR:  hessian is a singular matrix.'
            stop 'ludcmp 1'
         endif
         vv(i) = one/aamax
  12  continue
c
c  Loop over columns of Crout's method.
c
      do 19 j=1, n
         do 14 i=1, j-1
            sum=a(i,j)
            do 13 k=1, i-1
               sum = sum-a(i,k)*a(k,j)
  13        continue
            a(i,j) = sum
  14     continue
c
c  Search for the largest pivot element.
c
         aamax = zero
         do 16 i=j, n
            sum = a(i,j)
            do 15 k=1, j-1
               sum = sum-a(i,k)*a(k,j)
  15        continue
            a(i,j) = sum
            dum = vv(i)*abs(sum)
            if (dum.ge.aamax) then
               imax = i
               aamax = dum
            endif
  16     continue
c
c  Interchange rows, change the parity of d, and interchange the scale
c    factor if necessary.
c
         if (j.ne.imax) then
            do 17 k=1, n
               dum = a(imax,k)
               a(imax,k) = a(j,k)
               a(j,k) = dum
  17        continue
            d = -d
            vv(imax) = vv(j)
         endif
         indx(j) = imax
c
c  If the pivot element is zero then the matrix is singular.
c
         if (abs(a(j,j)).lt.tiny) then
            write (outfile,*) 'ERROR:  Singular Matrix.'
            write (outfile,*) '  Substituting in a TINY value.'
            a(j,j) = sign(tiny,a(j,j))
c            stop 'ludcmp 2'
         endif

         if (j.ne.n) then
            dum = one/a(j,j)
            do 18 i=j+1, n
               a(i,j) = a(i,j)*dum
  18        continue
         endif
  19  continue

      return
      end
c
c***********************************************************************
c     LUBKSB
c***********************************************************************
c
      SUBROUTINE lubksb(a,n,np,indx,b)
c
c     subroutine lubksb
c
c     Created:  July 22, 1999 by J. M. Rodgers
c               From lubksb in Numerical Recipes
c     Revised:
c     Purpose:  Solves a*x = b.  Second step in finding an inverse
c
c       a    - the LU decomposition of the matrix
c       n    - order of the matrix a
c       np   - the leading dimension of the array a >= max(1,n)
c       indx - a work  array from ludcmp
c       b    - on entry, the vector b in the equation a*x = b
c              on exit, the vector x in the same equation
c
c     Called by:  newt
c
      implicit none

      integer  n, np, indx(np), i, ii, j, ll
      double precision  a(np,np), b(np), sum

      ii = 0
c 
c  Do the forward substition
c  ii will be set to the index of the first nonvanishing element of b.
c  Once ii is nonzero the sum below will actually be calculated.
c
      do 12 i=1, n
         ll = indx(i)
         sum = b(ll)
         b(ll) = b(i)
         if (ii.ne.0) then
            do 11 j=ii, i-1
               sum = sum-a(i,j)*b(j)
  11        continue
         elseif (sum.ne.0) then
            ii = i
         endif
         b(i) = sum
  12  continue
c
c  Now do the back substition.
c
      do 14 i=n, 1, -1
         sum = b(i)
         do 13 j=i+1, n
            sum = sum-a(i,j)*b(j)
  13     continue
c
c  Store component of solution in b
c
         b(i) = sum/a(i,i)
  14  continue

      return
      end
c
c******************************************************************************
c  ROTMOL
c******************************************************************************
c
      subroutine rotmol(oldcoord, newcoord, n, constant, iop)
c  
c     Subroutine rotmol
c
c     Created:  August 11, 1999 by J. M. Rodgers
c     Revised:
c     Purpose:  
c        IF IOP>0   To rotate the molecule to new coordinates so that the
c            appropriate atoms are at the origin, on the specified axis,
c            and on the specifed plane.
c        IF IOP<0   To reverse the rotation so that the 3 molecules which
c            were supposed to be on the origin/axis/plane are returned to
c            the plane they were originally on.  Rotation angles, axes,
c            atoms, and translation amounts are saved from the first run
c            with IOP>0 so that this may be possible 
c
c     Called by:  mlohooks
c     Calls:  setrot, rotcol
c
c     Variables defined here:
c
      use param, only : linear, nonlin, lints, nonlints
      use files, only : outfile
      implicit none

      double precision, parameter :: rtod = 180.0d0/3.14159d0
      integer n
      double precision  oldcoord(3*n), newcoord(3*n), xyz(3,n)
      double precision  savept(3), angle(3)
      integer  constant(3*n), atm(3), axis(3,2), i, iop
c
c  Save the rotation angles for when come into routine to undo rotation
c
      save

      write(outfile,1119)
1119  format(/2(/80('-')),/,'Reorienting Geometry:',/80('-'))
      write(outfile,'(/a)') 'Initial Coordinates:'
      do 20 i=1, n
         xyz(1,i) = oldcoord(3*i-2)
         xyz(2,i) = oldcoord(3*i-1)
         xyz(3,i) = oldcoord(3*i)
         write(outfile,1018) xyz(:,i)
  20  continue

      if (iop .gt. 0) then

         call setrot( constant, n, atm, axis )
         savept(:) = xyz(:,atm(1))
c
c  Move atm(1) to the origin
c
         do 25 i=1, n
            xyz(:,i) = xyz(:,i) - savept(:)
  25     continue
c
c  Rotate geometry so that the value of atm(2)'s coordinate on axis(1,2)
c     is zero.
c
         angle(1) = datan2( xyz(axis(1,2),atm(2)),
     *         xyz(axis(1,1),atm(2)) )
         call rotcol( xyz(axis(1,1),:), xyz(axis(1,2),:), n, atm(1),
     *                  0, -angle(1) )
c
c  Rotate the geometry so that the value of atm(2)'s coordinate on
c     axis(2,2) is zero.
c
         angle(2) = datan2( xyz(axis(2,2),atm(2)), 
     *         xyz(axis(2,1),atm(2)) )     
         call rotcol( xyz(axis(2,1),:), xyz(axis(2,2),:), n, atm(1),
     *                  0, -angle(2) )

c
c  For non-linear molecules, rotate the geometry so that the value of
c    atm(3)'s coordinate on axis(3,2) is zero.
c
         if (iop.eq.nonlin .or. iop.eq.nonlints) then
            angle(3) = datan2( xyz(axis(3,2),atm(3)), 
     *            xyz(axis(3,1),atm(3)) )     
            call rotcol( xyz(axis(3,1),:), xyz(axis(3,2),:), n, atm(1),
     *                  atm(2), -angle(3) )
         endif

         write(outfile,'(/a)') 'Rotation Angles:'
         write(outfile,1019) -angle(:)*rtod
c
c  Undo Rotation if IOP is less than zero. - Just do the above in the
c    exact reverse order with the angles and the translation negated.
c
      elseif (iop .lt.0) then
         if (iop.eq.-nonlin .or. iop.eq.-nonlints) then
            call rotcol( xyz(axis(3,1),:), xyz(axis(3,2),:), n, atm(1),
     *                  atm(2), angle(3) )
         endif
         call rotcol( xyz(axis(2,1),:), xyz(axis(2,2),:), n, atm(1),
     *                  0, angle(2) )
         call rotcol( xyz(axis(1,1),:), xyz(axis(1,2),:), n, atm(1),
     *                  0, angle(1) )
         do 40 i=1, n
            xyz(:,i) = xyz(:,i) + savept(:)
  40     continue

         write(outfile,'(/a)') 'Rotation Angles:' 
         write(outfile,1019) angle(3)*rtod,angle(2)*rtod,angle(1)*rtod
      endif

      write(outfile,'(/a)') 'Reoriented Coordinates:'
      do 60 i=1, n
         write(outfile,1018) xyz(:,i)
         newcoord(3*i-2) = xyz(1,i)
         newcoord(3*i-1) = xyz(2,i)
         newcoord(3*i) = xyz(3,i)
  60  continue
      write(outfile,1025)
1025  format(/,80('-'),//)

1018  format(3(3x,d16.9))
1019  format(3(3x,f10.4))

      return
      end
c
c******************************************************************************
c  SETROT
c******************************************************************************
c
      subroutine setrot( c, n, atm, axis )
c
c     Subroutine setrot
c
c     Created:  August 11, 1999 by J. M. Rodgers
c     Revised:
c     Purpose:  To set the atoms and axes important for the rotation of
c        the molecule based on the information provided in constant in
c        mlopt.
c
c     Note:  axis(3,2) has the two columns in the geometry which may
c       change in the ith rotation in axis(i,1) and axis(i,2)
c
c     Called by:  rotmol
c     
c     Variables defined here:
c     
      use files, only : outfile
      implicit none

      integer n, c(3*n), atm(3), axis(3,2), i

      do 10 i=1, 3
         atm(i) = 0
         axis(i,1) = 0
         axis(i,2) = 0
  10  continue

      do 20 i=1, n
         if (c(3*i-2).eq.1 .and. c(3*i-1).eq.1 .and. 
     *            c(3*i).eq.1)  then
            atm(1) = i
         elseif (c(3*i-2).eq.1 .and. c(3*i-1).eq.1)  then
            atm(2) = i
            axis(1,2) = 1
            axis(2,2) = 2
            axis(1,1) = 3
            axis(2,1) = 3
         elseif (c(3*i-2).eq.1 .and. c(3*i).eq.1)  then
            atm(2) = i
            axis(1,2) = 1
            axis(2,2) = 3
            axis(1,1) = 2
            axis(2,1) = 2
         elseif (c(3*i-1).eq.1 .and. c(3*i).eq.1)  then
            atm(2) = i
            axis(1,2) = 2
            axis(2,2) = 3
            axis(1,1) = 1
            axis(2,1) = 1
         elseif (c(3*i-2).eq.1)  then
            atm(3) = i
            axis(3,2) = 1
         elseif (c(3*i-1).eq.1)  then
            atm(3) = i
            axis(3,2) = 2
         elseif (c(3*i).eq.1)  then
            atm(3) = i
            axis(3,2) = 3
         endif
  20  continue

      if (axis(1,1) .eq. axis(3,2)) then
         write (outfile,*) 'ERROR in creating rotation axes:'
         write (outfile,1115)
         stop 'setrot 1'
      endif

1115  format(2x,'The frozen coordinate for the third atom must be ',
     *    /5x,'the same as one of the frozen coordinates for ',
     *    /5x,'the second atom.')

      axis(3,1) = 6

      if (axis(1,1).eq.1 .or. axis(3,2).eq.1)  axis(3,1) = axis(3,1)-1
      if (axis(1,1).eq.2 .or. axis(3,2).eq.2)  axis(3,1) = axis(3,1)-2
      if (axis(1,1).eq.3 .or. axis(3,2).eq.3)  axis(3,1) = axis(3,1)-3

      return
      end
c
c******************************************************************************
c  ROTCOL
c******************************************************************************
c
      subroutine rotcol( col1, col2, n, atm1, atm2, a0 )
c
c     Subroutine rotcol
c
c     Created:  August 11, 1999 by J. M. Rodgers
c     Revised:
c     Purpose:  To rotate col1 and col2 about the remaining 3rd axis by an
c        angle a0.  atm1 and atm2 specify atoms which should remain
c        constant.
c
c     Called by:  rotmol
c
c     Variables defined here:
c
      implicit none

      integer n, atm1, atm2, i
      double precision  col1(n), col2(n), a0, a, r

      do 20 i=1, n
         if (i.eq.atm1 .or. i.eq.atm2)   goto 20
         r = sqrt(col1(i)**2 + col2(i)**2)
         a = datan2( col2(i), col1(i) )

         col1(i) = r*dcos(a+a0)
         col2(i) = r*dsin(a+a0)
  20  continue

      return
      end

