c
c
c*****************************************************************************
c  OrcaSTINP
c*****************************************************************************
c
      subroutine orcastinp(iunit,esmeth,esbas,
     *   qmcharge, qmmult, ssgeom,optl,type,ver_lab,
     *   qmmminp, ssmmgeom)
c
c     Subroutine orcastinp
c     
c     Created:  Apr.  4, 2006 by Yan Zhang
c     Revised:  March 11, 2011 by Soroosh Pezeshki
c
c     Purpose:  Write a ORCA input file for the calculation of 
c        the capped small system in presence of background point charges
c        Type indicates whether it is an energy, gradient, or
c        hessian calculation. 
c                                                                       
c 
c     Called by:  progehk, mprogehk, progghk, proghhk, progohk
c     Calls:
c
      use files
      use caltyp                                                        
      use input
c
c   add 2/10/2007 by Christoph Riplinger
c
      use param, only : numhs, hybridhs, esths
c
c   end 2/10/2007 by Christoph Riplinger
c
      implicit none
c
c i/o variables
c 
      character*20 ver_lab
      character*20 esmeth
      character*20 esbas
c
      character*80 optl(maxoptl)   
      integer      iunit, type
      type (cartgeom) :: ssgeom   
      type (mmvalen)  :: ssmmgeom   
      type (qmmminf)  :: qmmminp
c
c local variables
c
      logical      linkzero
      integer      mfin, bfin, mend, bend, i, j, k
      
      integer      ii, jj, kk
      integer      lsymbol, ltypname
      integer      qmnatm,capnatm,cssnatm,lsnatm
      integer      ssnatm,bgchnatm,mmnatm
      
      character*20 oldbas       
      character*12 tswit(4)
      
      character*40 mmkey, connkey
      character*12 oniomkey, mmprmkey 
      character*4  keykey
      character*2  atmsymbol
      character*4  typname
      
      double precision  charge

c
      character*80   wmeth(40),wbas(40),blockopt
      integer  nwmeth,nwbas,ierr
      integer  lwmeth(40),lwbas(40)

      integer  qmcharge, qmmult

      data tswit /'Energy', 'Gradient', 'Hessian', 'Optimization'/
      

      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      ssnatm   = ssmmgeom%natoms
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + mmnatm
      bgchnatm = ssnatm - lsnatm
      
      mfin = 1
      bfin = 1
      mend = 0
      bend = 0   

      if (tdynamics.ne.1) then                                          0312SP11
      write(outfile,999) tswit(type), esmeth, esbas
 999  format('Running Orca Job:',3x,a,3x,a10,3x,a10)
      endif                                                             0312SP11
c
c  Find ends of method and basis set specification
c
      call wordstr(esmeth,20,wmeth,nwmeth,lwmeth,ierr)

      call wordstr(esbas,20,wbas,nwbas,lwbas,ierr)

c
c  Title
c
      if (type .eq. tener) then
         write(iunit,3021) 
      elseif (type .eq. tgrad) then
         write(iunit,3022)
      elseif (type .eq. thess) then
         write(iunit,3023)
      elseif (type .eq. topt) then
         write(iunit,3024)
      endif
c
c  
c  print out method
c
      write(iunit,1013)'%method  method',wmeth(1)(:lwmeth(1))
      write(iunit,'(10x,a6)')'end'
c
c  print out basis
c

c
c add March 11, 2011 by Soroosh Pezeshki
c

c      write(iunit,1013)'%basis  basis',wbas(1)(:lwbas(1))
c      write(iunit,'(10x,a6)')'end'

c
c only write basis if not semi empirical
c
      if (tsemiempirical.eq.0) then
            write(iunit,1013)'%basis  basis',wbas(1)(:lwbas(1))
            write(iunit,'(10x,a6)')'end'
      endif

c
c end add  March 11, 2011 by Soroosh Pezeshki
c

c
c print out calculation type
c
      if (type .eq. tener) then
         write(iunit,'(a8)')'! energy' 
      elseif (type .eq. tgrad) then
         write(iunit,'(a8)')'! engrad'
      elseif (type .eq. thess) then
         write(iunit,'(a9)')  '! NumFreq'
      elseif (type .eq. topt) then
         write(iunit,'(a5)') '! opt'
      endif
c
c  print out input block
c    ORCA use END as keyword in several inputs.
c    In order to distinguish the END keywords (i.e., QRCA END keyword and 
c     the QMMM END keyword), we require that the ORCA option keywords 
c      start with an indicator >.  
c
      i = 1
      do while (i.le.maxoptl)

       if(optl(i).ne.'   ') then
        blockopt=optl(i)
        j=1
c
        do while(blockopt(j:j).eq.' '.and.j.le.80)
         j=j+1
        enddo
        if(blockopt(j:j).eq.'>')    
c
     $        write(iunit,'(a79)') blockopt(j+1:80)
        endif
        i = i + 1
      enddo
c
c  print out background point charge file 
c    The background charges are listed in a separated file 
c    called orca.pc.
c
c   modified 2/10/2007 by Christoph Riplinger
c   if orca.pc is empty, orca aborts
c
c         write(iunit,'(a24)') '%pointcharges "orca.pc" '
      if(ssnatm .gt. cssnatm) then
         write(iunit,'(a24)') '%pointcharges "orca.pc" ' 
      endif
c
c   end 2/10/2007 by Christoph Riplinger
c

3021  format ('#Single Point Energy Calculation'/)
3022  format ('#Gradient Calculation'/)
3023  format ('#Hessian Calculations'/)
3024  format ('#External Optimization'/)
c
c  Charge and multiplicity
c
      write(iunit,3050) '* xyz',qmcharge, qmmult
3050  format (a5,1x,i3,1x,i3)      
c
c  QM: atomic symbols and coordinates
c      

      do i=1, qmnatm
         lsymbol = 0
         atmsymbol = ssgeom%atm(i)           
         do j=1, 2
            if (atmsymbol(j:j) .ne. ' ') then 
               lsymbol = lsymbol + 1
            endif 
         enddo

         write(iunit,4000)
     *      atmsymbol(:lsymbol), 
     *      ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)

      enddo     
4000  format (2x,a3,2x,3(f16.9,3x))

c
c  Cap: atomic symbols and coordinates
c    
c
      do i = qmnatm + 1, cssnatm
         atmsymbol = capname
         lsymbol   = 1
         typname   = captypname
         ltypname  = 2
         write(iunit,4000)
     *      atmsymbol(:lsymbol),
     *      ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)
      enddo
      write(iunit,*)'*'

c
c  MM and M1/auxiliary charges: atomic symbols and coordinates
c          
c
      open(unit=200,file='orca.pc',err=9999)
      write(200,'(i5)')ssnatm-cssnatm

      do i=cssnatm+1,ssnatm
         atmsymbol = chargename
         lsymbol   = 2
         typname   = chargetypname
         ltypname  = 2
         write(200,4100)
     *      ssmmgeom%atmcharge(i),  ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)                 
      enddo
      
      write(200,*)
4100  format (f7.4,2x,3(f16.9,3x))
c
c  Close the file
c
      close(200,err=9999)
      
      return

9999  write (outfile,*) 'Error creating Orca input file'
      stop 'orcastinp'

1001  format (/)
1010  format (2a)
1011  format (4a,3(3x,a))
1012  format (4a,2(3x,a))
1013    format (a,2x,a)
      end

c
c*****************************************************************************
c  orcaINP
c*****************************************************************************
c
      subroutine orcainp(iunit,esmeth,esbas,
     *    qmcharge, qmmult, geom,optl,type,ver_lab)
c
c     Subroutine orcainp
c     
c     Created:  Mar. 30, 2006 by Yan Zhang
c     Revised:  March 11, 2011 by Soroosh Pezeshki
c
c     Purpose:  Write an Orca input file for the calculation
c        specified.  Type indicates whether it is an energy, gradient, or
c        hessian calculation. 
c
c 
c     Called by:  progehk, mprogehk, progghk, proghhk, progohk
c     Calls:        wordstr
c
      use files
      use caltyp                                                        
      use input, only : cartgeom, maxoptl
c
c   add 2/10/2007 by Christoph Riplinger
c
      use input, only : mlinp
      use param, only : numhs, hybridhs, esths
c
c   end 2/10/2007 by Christoph Riplinger
c
      implicit none
c
c i/o variables
c 
      character*20 ver_lab
      character*20 esmeth
      character*20 esbas
      
      type (cartgeom) :: geom
c
      character*80       optl(maxoptl)
      integer            iunit
c                        
c local variables             
c
      character*20   oldbas                                        
c
c   modified 2/10/2007 by Christoph Riplinger
c
c      integer         i, j, type
c      character*12   tswit(4)
      integer         i, j, k, type
      character*22   tswit(6)
c
c   end 2/10/2007 by Christoph Riplinger
c
      character*80   wmeth(40),wbas(40),blockopt
      integer  nwmeth,nwbas,ierr
      integer  lwmeth(40),lwbas(40)

      integer qmcharge, qmmult

      if (tdynamics.ne.1) then                                          0312SP11
c
c   modified 2/10/2007 by Christoph Riplinger
c
c      data tswit /'Energy','Gradient','Hessian','Optimization'/
      data tswit /'Energy','Gradient','Hessian',
     *    'Optimization (Full QM)','External Optimization',
     *    'Estimating the Hessian'/
c
c   end 2/10/2007 by Christoph Riplinger
c
      write(outfile,999) tswit(type), esmeth, esbas
 999  format('Running Orca Job:',3x,a,3x,a10,3x,a10)
      endif                                                             0312SP11
c
c  Find  method and basis set specification
c
      call wordstr(esmeth,20,wmeth,nwmeth,lwmeth,ierr)

      call wordstr(esbas,20,wbas,nwbas,lwbas,ierr)

c
c  Print out Route specifications - different for each possible type
c    specified.
c
      if (type .eq. tener) then
         write(iunit,1021) 
      elseif (type .eq. tgrad) then
         write(iunit,1022)
      elseif (type .eq. thess) then
         write(iunit,1023)
      elseif (type .eq. topt) then
         write(iunit,1024)
c
c   add 2/10/2007 by Christoph Riplinger
c
c
      elseif (type .eq. textopt) then
         write(iunit,1025)
      elseif (type .eq. testhess) then
         write(iunit,1026)
c
c   end 2/10/2007 by Christoph Riplinger
c
      endif
c
c   add 2/10/2007 by Christoph Riplinger
c   we don't need/have that information for external opt. and etimated hessian
c
      if((type .ne. textopt) .and. (type .ne. testhess)) then
c
c  print out method
c
        write(iunit,1011)'%method  method',wmeth(1)(:lwmeth(1))
        write(iunit,'(10x,a6)')'end'
c
c  print out basis
c
c
c add March 11, 2011 by Soroosh Pezeshki
c

c      write(iunit,1011)'%basis  basis',wbas(1)(:lwbas(1))
c      write(iunit,'(10x,a6)')'end'

c
c only write basis if not semi empirical
c
        if (tsemiempirical.eq.0) then
            write(iunit,1011)'%basis  basis',wbas(1)(:lwbas(1))
            write(iunit,'(10x,a6)')'end'
        endif

c
c end add  March 11, 2011 by Soroosh Pezeshki
c


      endif
c
c   end 2/10/2007 by Christoph Riplinger
c
c
c  @Christoph should the following 4 commands not be commented out?     0311SP11
c
c
c  print out method
c
c      write(iunit,1011)'%method  method',wmeth(1)(:lwmeth(1))
c      write(iunit,'(10x,a6)')'end'
c
c  print out basis
c
c      write(iunit,1011)'%basis  basis',wbas(1)(:lwbas(1))
c      write(iunit,'(10x,a6)')'end'
c
c print out calculation type
c
      if (type .eq. tener) then
         write(iunit,'(a8)')'! energy' 
      elseif (type .eq. tgrad) then
         write(iunit,'(a8)')'! engrad'
      elseif (type .eq. thess) then
         write(iunit,'(a9)')  '! NumFreq'
      elseif (type .eq. topt) then
         write(iunit,'(a5)') '! opt'
c
c   add 2/10/2007 by Christoph Riplinger
c
      elseif (type .eq. textopt) then
         write(iunit,'(a15)') '! HF SVP extopt'
      elseif (type .eq. testhess) then
         write(iunit,'(a20)') '! HF SVP calcesthess'
c
c   end 2/10/2007 by Christoph Riplinger
c
      endif
c
c   add 2/10/2007 by Christoph Riplinger
c   we tell orca to calculate the initial hessian
c
      if(type .eq. textopt) then
        if((mlinp%htyp.eq.numhs).or.
     *     (mlinp%htyp.eq.hybridhs).or.
     *     (mlinp%htyp.eq.esths)) then
         write(iunit,*) '%geom'
         write(iunit,*) '     Calc_Hess true'
         write(iunit,*) '   end'
        endif
      endif
c
c   end 2/10/2007 by Christoph Riplinger
c
c  print out input block
c    ORCA use END as keyword in several inputs.
c    In order to distinguish the END keywords (i.e., QRCA END keyword and 
c     the QMMM END keyword), we require that the ORCA option keywords 
c      start with an indicator >.  
c
      i = 1
      do while (i.le.maxoptl)

       if(optl(i).ne.'   ') then
        blockopt=optl(i)
        j=1
c
        do while(blockopt(j:j).eq.' '.and.j.le.80)
         j=j+1
        enddo
        if(blockopt(j:j).eq.'>')    
c
     $        write(iunit,1060) blockopt(j+1:80)
       endif
       i = i + 1
      enddo


c
c  Print out charge, multiplicity, and atomic symbols and coordinates
c
      write(iunit,1031)'* xyz', qmcharge, qmmult
      do 20 i=1, geom%natoms
         write(iunit,1041) geom%atm(i), geom%angcoord(3*i-2),
     *      geom%angcoord(3*i-1), geom%angcoord(3*i)
  20  continue
      write(iunit,*)'*'
      return

9999  write (outfile,*) 'Error creating Orca input file'
      stop 'orcainp'

1001  format (/)
1010  format (2a)
1011  format (a,2x,a)
1012  format (4a,2(3x,a))
1016  format (a1,a75)
1021  format ('#Single Point Energy Calculation'/)
1022  format ('#Gradient Calculation'/)
1023  format ('#Hessian Calculations'/)
c
c   modified 2/10/2007 by Christoph Riplinger
c
c1024  format ('#External Optimization'/)
1024  format ('#External Optimization (Full QM)'/)
1025  format ('#External Optimization'/)
1026  format ('#Estimated Hessian Calculation'/)
c
c   end 2/10/2007 by Christoph Riplinger
c
1031  format (a5,1x,i3,1x,i3)
1041  format (a2,5x,3(f16.9,3x))
1051  format (2a,/)
c
1060  format (a79)

      end

c
c
c*****************************************************************************
c  orcaOUTE
c*****************************************************************************
c
      subroutine orcaoute (ppnum,esmeth, ener)
c
c     Subroutine orcaoute
c     
c     Created:  Mar. 30, 2006	by Yan Zhang
c     Revised:
c     Purpose:  Get a single energy out of the Orca out file. 
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c
c     Called by:  progehk, progghk, proghhk
c     Calls:
c
c     Variables defined here:
c
      use files
      use numfmt
      implicit none
 
      character*20  esmeth                                                 
      double precision  ener, cfltfmt
      integer enerfnd,ppnum,outenum                                         

c
      character*80 string, case
      character*30 label1,label2
      character*22 etxt
      character*14 outename                                                

      integer nword,lword(40),ierr,i,ical
c
      character*80 word(40)
c
c  Open the formatted checkpoint file.
c  outoenum=out-orca-energy-number
c  outoename=out-orca-energy-file
c

      outenum = orcaoutf                                                   
      outename=orcaout                                                      

      open(unit=outenum,status='unknown', file=outename, err=9900)           

c
      read( outenum, '(a80)', end=9910 ) string                              
      string=case(string)

      do while( string(:16).ne.'total scf energy' )
c
        read( outenum, '(a80)', end=9910 ) string                          
        string=case(string)
      end do

      do i=1,3
c
        read( outenum, '(a80)', end=9910 ) string                            
        string=case(string)
      end do

      if(string(1:12).eq.'total energy') then
        etxt=string(25:43)
      else
        goto 9910
      end if

c
      read( outenum, '(a80)', end=9910 ) string                            
      string=case(string)

c
c   the keyword to search the energy of the MP2 calculations
c      is 'mp2 total energy'.
c
      ical=0
      do while(string(32:51).ne.'orca mp2 calculation')
c
        read( outenum, '(a80)', end=9000 ) string                            
        string=case(string)
      end do
      if(string.ne.'orca mp2 calculation') then
        ical=1
         etxt=' '
      end if

      if(ical.eq.1) then 
        do while(string(:16).ne.'mp2 total energy')
c
          read( outenum, '(a80)', end=9910 ) string                         
          string=case(string)
        end do
        if(string(:16).eq.'mp2 total energy')
     $    etxt=string(25:38)
      end if

9000	continue

      open(50,file='temp1')
      write(50,*)etxt
      close(50)
      open(60,file='temp1')
      read(60,*)ener
      close(60)

c	ener=cfltfmt(etxt,1)

      close( unit=outenum )

      return

9900  write( outfile,*) 'Error opening Orca out file'
      stop 'orca 1'
9910  write( outfile,*) 'Error reading Orca out file'
      stop 'orca 2'

      end
c
c*****************************************************************************
c  orcaOUTG
c*****************************************************************************
c  
      subroutine orcaoutg (ppnum, esmeth, grad, cssnatm, natoms)
c
c     Subroutine orcaoutg
c
c     Created:  Mar. 31, 2006	 by Yan Zhang
c     Revised:
c     Purpose:  Get the gradient out of the Orca out file.  
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c     Called by:  progghk, proghhk
c     Calls:
c
c     Variables defined here: 
c     
      use files
      use numfmt
      implicit none
      
      character*20  esmeth
      integer ctr, natoms, ppnum,ogradnum                    

      double precision  grad(3*natoms)
c
      character*80 string, case
      character*14 ogradname                                                
      integer cssnatm,pcnum,totalnum,i
c                                                                     
c  Open the formatted orca.engrad file.
c  ograd=orca-energygradient
c
      ogradnum = ogradn                                                     
      ogradname = ogradf                                                     

      open(unit=ogradnum,status='unknown',file=ogradname,err=9900)

c
      read( ogradnum, '(a80)', end=9910 ) string
      string = case(string)
 
      do while( string(7:23).ne.'current gradient') 
c
         read( ogradnum, '(a80)', end=9910 ) string
         string = case(string)
      enddo
c
c  Get each element of gradient 
c  
c
      read( ogradnum, '(a80)', end=9910 ) string
      ctr = 1
      do while( ctr .le. 3*cssnatm )
         read( ogradnum,*) grad(ctr)
         ctr=ctr+1
      end do

      close( unit=ogradnum )
c                    
c  If there are background charges, which is indicated by the difference in 
c    the numbers of total atoms and the number of QM atoms,  
c  then open the formatted orca.pcgrad file and read the gradient of 
c    background charge.
c  ogradpcn=orca-charge-gradient-number
c
      if((natoms-cssnatm).gt.0) then
        open(unit=ogradpcn,status='unknown',file=ogradpcf,err=9920)

        read(ogradpcn,*)pcnum
        totalnum=pcnum+cssnatm

        if(totalnum.ne.natoms) then 
          write(outfile,*)'Error reading the orca gradient'
          stop 'error orcatoug'
        end if
	
        do i=1,pcnum
          ctr=(i+cssnatm)*3
          read(ogradpcn,*)grad(ctr-2),grad(ctr-1),grad(ctr)
        end do
        close(ogradpcn)
      end if

      return

9900  write( outfile,*) 'Error opening Orca gradient file'
      stop 'orcaoutg 1'
9910  write( outfile,*) 'Error reading Orca gradient file'
      stop 'orcaoutg 2'
9920  write( outfile,*) 'Error opening Orca background charge',
     $                   ' gradient file'
      stop 'orcaoutg 3'

      end

c
c
c*****************************************************************************
c  orcaOUTH
c*****************************************************************************
c  
      subroutine orcaouth (esmeth, hess, natoms)
c
c     Subroutine orcaouth
c
c     Created:  Mar. 31, 2006	by Yan Zhang
c     Revised:
c     Purpose:  Get the hessian out of the Orca hessian file.
c
c     Called by:  proghhk
c     Calls:		wordstr
c
c     Variables defined here: 
c     
      use files
      use numfmt
      implicit none
      
      character*20  esmeth
      integer ctr, i, k, natoms, j

      double precision  hess(3*natoms,3*natoms)

      integer nhess
c
      character*80 string, case,word(40)
      integer lnum,lword(40),ierr
c     
c  Open the Orca hessian output file.
c  ohessf=orca-hessian
c
c  ORCA outputs the Hessian matrix as exemplified as below by 
c    a three-atom system. 
c    The 9*9 output hessian of triatomic ORCA calculation 
C       in orca.hess file as
c
c                  0          1          2          3          4          5    
c      0      -0.002908  -0.006057   0.005645   0.021282  -0.003012  -0.023875
c      1      -0.009176  -0.054807   0.066835   0.021424  -0.003259  -0.015974
c      2       0.006336   0.062166  -0.034199  -0.015140  -0.000100   0.011024
c      3       0.021743   0.018023  -0.014995  -0.001608   0.001528   0.007532
c      4      -0.004409  -0.005219  -0.001155   0.002098   0.021460  -0.015552
c      5      -0.027449  -0.014232   0.021444   0.010071  -0.010442  -0.110228
c      6      -0.024451  -0.016135   0.016253   0.005877  -0.000533  -0.005356
c      7       0.010004   0.057753  -0.064311  -0.021893  -0.000910   0.017525
c      8      -0.007695  -0.061807   0.035845   0.016671  -0.000101  -0.012818
c                  6          7          8          
c      0      -0.023078   0.010391  -0.007096   
c      1      -0.017418   0.065894  -0.069718  
c      2       0.014922  -0.070236   0.037811  
c      3       0.007643  -0.021255   0.018330  
c      4      -0.000736  -0.003210  -0.000811  
c      5      -0.007889   0.022499  -0.016654  
c      6       0.014236   0.018653  -0.015250  
c      7       0.017353  -0.061009   0.067777  
c      8      -0.015273   0.068113  -0.035346  
c
c

      open( unit=ohessn, status='unknown', file=ohessf, err=9900)

c
      read( ohessn, '(a80)', end=9910 ) string
      string = case(string)

      do while( string.ne.'$hessian' )
c
         read( ohessn, '(a80)', end=9910 ) string
         string = case(string)
      enddo
      
      read( ohessn,*) nhess
      k=0

c
      read( ohessn, '(a80)', end=9910 ) string
      string=case(string)

      do while(string.ne.' ')

c
        call wordstr(string,80,word,lnum,lword,ierr)

        do j=1,nhess
            read(ohessn,*)ctr,(hess(j,k+i),i=1,lnum)
        end do
        k=k+lnum

c
        read( ohessn, '(a80)', end=9910 ) string
        string=case(string)
      end do

      if(k.ne.nhess.or.nhess.ne.3*natoms) stop 'Error number of hessian'

      close( unit=ohessn )

      return

9900  write( outfile,*) 'Error opening Orca hessian file'
      stop 'orcaouth 1'
9910  write( outfile,*) 'Error reading Orca hessian file'
      stop 'orcaouth 2'

      end

c
c******************************************************************************
c  orcaOUTO
c******************************************************************************
c
      subroutine orcaouto (geom, ierr)
c
c     Subroutine orcaouto
c
c     Created:  Mar. 31, 2006	by Yan Zhang
c
c     Purpose:  Get the energy, gradient, and optimized geometry out of
c        the Orca output file.   
c
c
c     Called by:  progohk
c     Calls:
c
      use files
      use numfmt
      use input, only : cartgeom

      implicit none
c
c i/o variables
c
      type (cartgeom)  :: geom
      integer             ierr,opt            
c
c local variables
c

c
      character*80 string, case
      character*3 name
      integer natoms,i

      double precision, parameter :: angtoau = 0.529177249d0
c
c  Check that the optimization is complete.
c  ooptf=orca-opt-file
c
      opt  = 0
      ierr=0

      open( unit=extoout, status='unknown', file=extout, err=9920 )

c
      read( extoout,'(a80)', end=9930 ) string
      string = case(string)
c
c the search string is
c   "the optimization has converged"
c
      do while( string(32:61) .ne. 'the optimization has converged')
c
         read( extoout, '(a80)', end=9930 ) string	 
         string = case(string)
      end do
           
      close ( unit=extoout )
c
      write(outfile,*) string(1:80)

c     
c  Open the orca optimized output coordinate file.
c
      open( unit=outoon, status='unknown', file=outoof, err=9920 )

      read(outoon,*) natoms

c
      read(outoon,'(a80)') string
      if(natoms.ne.geom%natoms) 
     $	stop  'Error reading the number of atoms in orca.xyz'
     
       do i=1,natoms
      read(outoon,*) name,geom%angcoord(3*i-2),geom%angcoord(3*i-1),
     $               geom%angcoord(3*i)
       end do 

      geom%aucoord = geom%angcoord/angtoau

      close( unit=outoon )
 
      return
      
9910  write( outfile,*) 'Error reading Orca optimized output file.'
      stop 'orcao 1'
9920  write( outfile,*) 'Error opening orca optimized output 
     $                    file-extopt.xyz.'
      stop 'orcao 2'
9940  write( outfile,*) 'Error reading orca optimized output 
     $                    file-extopt.xyz.'
      stop 'orcao 3'


9930  ierr=1
      return

      end


c
c
c*****************************************************************************
c WORDSTR
c*******************************************************************

      subroutine wordstr(string,longstr,word,nword,lword,ierr)

c
c	 subroutine wordstr
c
c     Created: Mar. 30, 2006 by Yan Zhang 
c
c     This subroutine finds the number of the non-comment words in a  
c     line. 
c     
c    	call by: orcastinp, orcainp, orcaouth, gmsoute, gmsoutg, gmsouto
c
c	call:
c

      implicit none
c
      integer longstr,ierr,lword(40)
      character (len=longstr)  string,line
      integer  nword
c
      character*80  word(40),case
      integer  ip, icom, ibeg, ii      
c
      ierr=0
      nword= 0
      ip = 0
c
c  
      do ii=1,40
         word(ii)='  '
      end do
c
c     find the first nonblank character in a line
c
20    ip = ip + 1
      if (string(ip:ip) .eq. ' ' .and. ip .lt. longstr) go to 20           

c
c     convert to lowercase
c

      line=case(string)
C
C     Find the position of a comment character, if any,
C     in a nonblank noncomment line
C

      icom = ip 

      do while(line(icom:icom).ne. '#' .and. icom .lt. longstr)
        icom=icom+1
      end do



C
55    if (ip .lt. icom) then
         ibeg = ip
60       ip = ip + 1
         if (line(ip:ip) .ne. ' ' .and. ip .lt. icom) goto 60 
         nword = nword + 1
         lword(nword)=ip-ibeg
         word(nword)(1:lword(nword)) = line(ibeg:ip-1)
70       ip = ip + 1
         if (line(ip:ip) .eq. ' ' .and. ip .lt. icom) goto 70
         goto 55
      endif

80      return
c
      end


