c**********************************************************************
c  orca_ext_opt
c**********************************************************************
c
      subroutine orca_ext_opt(optinp,geom,mmgeom,ierr,qmmminp,progrms)
c
c     Subroutine orca_ext_opt
c
c	Created : Oct 02, 2007 by Christoph Riplinger 
c       (template: gau_part_opt.F and gau_ext_opt.F)
c
c
c     Purpose : If ipartcal=0, call subroutine orca_ext_opt1(OPT all atoms).
c               If ipartcal=1, call subroutine orca_ext_opt2(OPT partial atoms).
c
c*     Called by:  mlohook
c*     Calls:  optghk, opthhk 
c*
      use files
      use input, only : mlopt, mlinp,cartgeom, proglist,
     *                  mmvalen, qmmminf
      use optimize, only : ener, grad, hess
      implicit  none
c
c i/o variables
c
      type (proglist) :: progrms
      integer            ierr
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (mlopt)    :: optinp
      type (qmmminf)  :: qmmminp
c
c local variables
c
      integer            ipartcal
c
c
      mlinp%progrms = progrms
      ipartcal=optinp%part%ipartcal
c
c
      if(ipartcal.eq.0) then
         call orca_ext_opt1(optinp,geom,ierr)
      elseif(ipartcal.eq.1) then
         call orca_ext_opt2(optinp,geom,mmgeom,ierr)
      end if
      return
      end

c**********************************************************************
c  orca_ext_opt
c**********************************************************************
c
      subroutine orca_ext_opt1(optinp,geom,ierr)
c
c     Subroutine orca_ext_opt
c
c*    orca_ext_opt is a subroutine which uses orca's external keyword (extopt)
c*    and orca's optmizer
c*
c*      MULTILEVEL  <-->  orca <--> orca_external <--> MULTILEVEL
c*
c*     Created:  Oct. 02 2007 by Christoph Riplinger 
c*     (template: gau_ext_opt1)
c
c*     Purpose:  To perform  optimization using orca's optimizer 
c*     
c*     Called by:  orca_ext_opt
c*     Calls:  orcainp, orcaouto
c*
      use files
      use input, only : mlopt, mlinp,cartgeom
      use optimize, only : ener, grad, hess
      implicit  none
 
c
c i/o variables
c
      integer            ierr
      type (cartgeom) :: geom
      type (mlopt)    :: optinp                  
c
c local variables
c
      integer            i, j, k, n3tm
      character*20       basis, method,v3s  
c
      n3tm = 3*mlinp%geom%natoms
c
c  For the external optimizer we don't need any basis or method
c
      basis = ''
      method = ''
      v3s = 'v3s'
c
      if(method(:3).eq.'g03')	stop 'Error the method for ext-opt'
c
      open (unit=extoinf, status='unknown', file=extin, err=9991)
      call orcainp(extoinf, method, basis, 
     *            geom%charge, geom%mult, geom,
     *            optinp%optl, 5, v3s)
      close (extoinf, err=9999)
      call system('orcashuttle extopt.inp extopt.out')
      call orcaouto(geom, ierr) 
            if (ierr.eq.1) go to 7000
c
c  Update the geometry
c
      mlinp%geom = geom
c
c  Display the results of the optimization
c
      write(outfile,1100)
      write(outfile,1250)  ener
      write(outfile,1300)
      write(outfile,1350)  grad(:n3tm)
      write(outfile,1400)
      write(outfile,1450)
      do 20 i=1, geom%natoms
         write(outfile,1500) geom%atm(i), geom%aucoord(3*i-2),
     *       geom%aucoord(3*i-1), geom%aucoord(3*i)
  20  continue
      write(outfile,1550)
      do 40 i=1, geom%natoms
         write(outfile,1500) geom%atm(i), geom%angcoord(3*i-2),
     *       geom%angcoord(3*i-1), geom%angcoord(3*i)
  40  continue

1100  format(/2(/80('-')),//2x,'External Optimization with orca',
     *         a,//80('-')/)
1250  format(/2x,'Optimized Energy:',t30,f19.12)
1300  format(/2x,'Optimized Cartesian Gradient:')
1350  format(t20,d15.8,t40,d15.8,t60,d15.8)
1400  format(/2x,'Optimized Geometry:')
1450  format(//5x,'Atomic Coordinates (in a.u.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)
1500  format(9x,a2,t20,d15.8,t40,d15.8,t60,d15.8)
1550  format(//5x,'Atomic Coordinates (in ang.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)  
      return
9991  write (outfile,*) 'Error creating orca extopt input file'
      stop 'orca_ext_opt1 1'
9999  write (outfile,*) 'Error creating orca extopt input file'
      stop 'orca_ext_opt1 2'
7000  write (outfile,'(/a)') 'ERROR: External Optimization failed.'
      stop 'orca_ext_opt1 3'
      end

c
c**********************************************************************
c  orca_ext_opt2
c**********************************************************************
c
      subroutine orca_ext_opt2(optinp,geom,mmgeom,ierr)
c
c     Subroutine orca_ext_opt2
c
c     Created:  Oct. 02, 2007 by Christoph Riplinger
c
c     Purpose:  To perform  partial atomic optimization using orca's optimizer 
c
c     Called by: orca_ext_opt
c     Calls:     orcapartinp, orcapartouto
c
      use files
      use caltyp, only   : textopt
      use input, only    : mlopt, mlinp, cartgeom, mmvalen
      use optimize, only : ener, grad, hess
      implicit  none

c
c i/o variables
c
      integer            ierr, partextatmnum, partext
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (mlopt)    :: optinp                  
c
c local variables
c
      integer            i, j, k, n3tm
      character*20       basis, method,v3s  
c
      n3tm = 3*mlinp%geom%natoms
      basis = ''
      method = '' 
      v3s = 'v3s'
      if(method(:3).eq.'g03') stop 'Error the method for ext-opt'
c
c   partext (partial extended) is only used for the calculation of the estimated hessian
c   here -> partext = 0
c
      partext = 0
      partextatmnum = 0
      open (unit=extoinf, status='unknown', file=extin, err=9991)
      call orcapartinp(extoinf, optinp%part, geom, 
     *            optinp%optl, textopt, mmgeom, partext, partextatmnum)
      close (extoinf, err=9999)
      call system('orcashuttle extopt.inp extopt.out')
      call orcapartouto(optinp%part, geom, ierr) 
           if (ierr.eq.1) go to 7000 
c
c  Update the geometry
c
      mlinp%geom = geom
c
c  Display the results of the optimization
c
      write(outfile,1100)
      write(outfile,1250)  ener
      write(outfile,1300)
      write(outfile,1350)  grad(:n3tm)
      write(outfile,1400)
      write(outfile,1450)
      do 20 i=1, geom%natoms
         write(outfile,1500) geom%atm(i), geom%aucoord(3*i-2),
     *       geom%aucoord(3*i-1), geom%aucoord(3*i)
  20  continue
      write(outfile,1550)
      do 40 i=1, geom%natoms
         write(outfile,1500) geom%atm(i), geom%angcoord(3*i-2),
     *       geom%angcoord(3*i-1), geom%angcoord(3*i)
  40  continue

1100  format(/2(/80('-')),//2x,'External Optimization with orca',
     *         a,//80('-')/)
1250  format(/2x,'Optimized Energy:',t30,f19.12)
1300  format(/2x,'Optimized Cartesian Gradient:')
1350  format(t20,d15.8,t40,d15.8,t60,d15.8)
1400  format(/2x,'Optimized Geometry:')
1450  format(//5x,'Atomic Coordinates (in a.u.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)
1500  format(9x,a2,t20,d15.8,t40,d15.8,t60,d15.8)
1550  format(//5x,'Atomic Coordinates (in ang.):',//8x,'Atom', t28, 'X',
     *         t48, 'Y', t68, 'Z'/)  
      return

9991  write (outfile,*) 'Error creating orca extopt input file'
      stop 'orca_ext_opt2 1'
9999  write (outfile,*) 'Error creating orca input file'
      stop 'orca_ext_opt2 2'
7000  write (outfile,'(/a)') 'ERROR: External Optimization failed.'
      stop 'orca_ext_opt2 3'
      end     

c
c*****************************************************************************
c  orcapartINP
c*****************************************************************************
c
      subroutine orcapartinp(iunit,part, geom, optl, type, 
     *                         mmgeom, partext, partextatmnum)
c
c     Subroutine orcapartinp
c
c     Created:  Oct. 02, 2007 by Christoph Riplinger
c               (template: orcainp)
c
c     Purpose:  Write an Orca partial atomic input file.
c               This routine is for external optimization 
c               and for the calculation of a partial Hessian
c 
c     Called by:  orca_ext_opt2, qmmmnumhhk, qmmmhybridhhk, orcaesthhk
c     Calls:	    wordstr
c
      use files
      use caltyp
      use input, only : cartgeom, maxoptl, mlinp,
     *                   mmvalen, maxidnum, partial
      use param, only : numhs, hybridhs, esths
      implicit none
c
c i/o variables
c
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (partial)  :: part
c
      character*80       optl(maxoptl)
      character*22       tswit(6)
      integer            iunit, partext, partextatmnum
c 
c local variables
c
      integer        mfin, bfin, mend, bend, i, j, type
c
      character*80   blockopt
      logical        linkzero
      integer        partextatmid(maxidnum)
      integer        k, jj, pidatm, iivalen, actvalenid, actpartextatmid
      integer        putitin
      integer        partcharge, partmult
      data tswit /'Energy','Gradient','Hessian',
     *    'Optimization (Full QM)','External Optimization',
     *    'Estimating the Hessian'/ 
c
      mfin = 1
      bfin = 1
      mend = 0
      bend = 0
c
      partcharge = part%pcharge
      partmult = part%pmult
      write(*,999) tswit(type)
 999  format('Running Orca Job:',3x,a)
c
c  To improve the estimated Hessian for a partial system we include
c  the first layer of bonded atoms to the partial atoms (called P1)
c
      if(partext .eq. 1) then
          partextatmnum = 0
c
c  initialize the list for the extended partial system
c
          do i=1,maxidnum
             partextatmid(i) = 0
          end do
c
c  store the IDs of the partial atoms in partextatmid (= partial extended atom ID)
c
          do i=1, part%partatmnum
               partextatmnum = partextatmnum + 1
               pidatm = part%partatmid(i)
               partextatmid(partextatmnum) = pidatm
            end do
c
c  go through the list of all partial atoms
c
          do i=1, part%partatmnum
               pidatm = part%partatmid(i)
               iivalen = mmgeom%atmvalen(pidatm)
c
c  look at the atoms (actvalenid) connected to the actual partial atom (pidatm)
c
               do j=1, iivalen
                   actvalenid = mmgeom%atmvalenlist(pidatm,j)
                   putitin = 1
                   do k=1, partextatmnum
                       actpartextatmid = partextatmid(k)
c
c  is actvalenid already in the list of the extended partial system?
c
                       if (actpartextatmid .eq. actvalenid) then
                           putitin = 0
                       endif
                    end do
c
c  if no, store the P1 atom in the list
c
                   if (putitin .eq. 1) then
                       partextatmnum = partextatmnum + 1
                       partextatmid(partextatmnum) = actvalenid
                    endif
                end do
           end do
       endif
c
c  Print out Route specifications
c  Only input files for external optimization or hessian calculation are written by this subroutine
c

      if (type .eq. textopt) then
         write(iunit,1025)
      elseif (type .eq. testhess) then
         write(iunit,1026)
      endif
c
c  print out calculation type
c
      if (type .eq. textopt) then
         write(iunit,'(a15)') '! HF SVP extopt'
c
c  if the user calls qmmm to calculate an initial qmmm-hessian
c  in addition the user can tell orca to recalculate the qmmm-hessian after a specific number
c  of optimization cycles
c
         if((mlinp%htyp.eq.numhs).or.
     *      (mlinp%htyp.eq.hybridhs).or.
     *      (mlinp%htyp.eq.esths)) then
            write(iunit,*) '%geom'
            write(iunit,*) '     Calc_Hess true'
            write(iunit,*) '   end'
         endif
      elseif (type .eq. testhess) then
         write(iunit,'(a20)') '! HF SVP calcemphess'
      endif
c
c  print out input block
c    ORCA use END as keyword in several inputs.
c    In order to distinguish the END keywords (i.e., ORCA END keyword and 
c     the QMMM END keyword), we require that the ORCA option keywords 
c      start with an indicator >.  
c
      i = 1
      do while (i.le.maxoptl)

       if(optl(i).ne.'   ') then
        blockopt=optl(i)
        j=1
c
        do while(blockopt(j:j).eq.' '.and.j.le.80)
         j=j+1
         enddo
        if(blockopt(j:j).eq.'>')    
c
     $     write(iunit,1060) blockopt(j+1:80)
         endif
        i = i + 1
       enddo
c
c  Print out charge, multiplicity, and atomic symbols and coordinates
c
      write(iunit,1031)'* xyz', partcharge, partmult
      if(partext .ne. 1) then
        do i=1, part%partatmnum
          pidatm = part%partatmid(i)
          write(iunit,1041) geom%atm(pidatm), geom%angcoord(3*pidatm-2),
     *          geom%angcoord(3*pidatm-1), geom%angcoord(3*pidatm)
         end do
      else 
        do i=1, partextatmnum
          pidatm = partextatmid(i)
          write(iunit,1041) geom%atm(pidatm), geom%angcoord(3*pidatm-2),
     *          geom%angcoord(3*pidatm-1), geom%angcoord(3*pidatm)
         end do
      endif
      write(iunit,*)'*'
      return

1001  format (/)
1010  format (2a)
1011  format (a,2x,a)
1012  format (4a,2(3x,a))
1016  format (a1,a75)
1025  format ('#External Optimization'/)
1026  format ('#Empirical Hessian Calculations'/)
1031  format (a5,1x,i3,1x,i3)
1041  format (a2,5x,3(f16.9,3x))
1051  format (2a,/)
c
1060  format (a79)

      end

c
c******************************************************************************
c  orcaPARTOUTO
c******************************************************************************
C
      subroutine orcapartouto (part, geom, ierr)
c
c     Subroutine orcapartouto
c
c     Created:  Oct. 02, 2007  by Christoph Riplinger 
c               (template: g03partouto)
c
c     Purpose:  Get the partial atomic energy, gradient, and optimized geometry out of
c        the orca output file.
c
c     Called by:  orca_ext_opt2
c     Calls:
c
      use files
      use numfmt
      use input, only : cartgeom, mlopt, partial

      implicit none
C
c i/o variables
c
      type (cartgeom)  :: geom
      type (partial)   :: part
      integer             ierr,opt
c
c local variables
c

      character*80 string, case
      character*3 name
      integer natoms,i,pidatm
      character*15 gtxt(3*geom%natoms), otxt(3*geom%natoms)
      double precision patmc(3*part%partatmnum) 

      double precision, parameter :: angtoau = 0.529177249d0
c
c  Check that the optimization is complete.
c  ooptf=orca-opt-file
c
      opt  = 0
      ierr=0
      open( unit=extoout, status='unknown', file=extout, err=9920 )

c
      read( extoout,'(a80)', end=9930 ) string
      string = case(string)
c
c the search string is
c   "the optimization has converged"
c
      do while( string(32:61) .ne. 'the optimization has converged')
c
         read( extoout, '(a80)', end=9930 ) string	 
         string = case(string)
      end do
      close ( unit=extoout )
c
      write(outfile,*) string(1:80) 
c
c  Open the orca optimized output coordinate file.
c
      open( unit=outoon, status='unknown', file=outoof, err=9920 )
c
      read(outoon,*) natoms
c
      read(outoon,'(a80)') string
      if(natoms.ne.part%partatmnum) 
     $	stop  'Error reading the number of atoms in orca.xyz'
c
      do i=1,natoms
        read(outoon,*) name,patmc(3*i-2),patmc(3*i-1),
     $               patmc(3*i)
       end do 
      do i=1,part%partatmnum
         pidatm = part%partatmid(i)
         geom%angcoord(3*pidatm-2) = patmc(3*i-2)
         geom%angcoord(3*pidatm-1) = patmc(3*i-1)
         geom%angcoord(3*pidatm  ) = patmc(3*i)
        end do

      geom%aucoord = geom%angcoord/angtoau

      close( unit=outoon )
c
      return
c
9910  write( outfile,*) 'Error reading Orca optimized output file.'
      stop 'orcao 1'
9920  write( outfile,*) 'Error opening orca optimized output 
     $                    file-extopt.xyz.'
      stop 'orcao 2'
9940  write( outfile,*) 'Error reading orca optimized output 
     $                    file-extopt.xyz.'
      stop 'orcao 3'
c
9930  ierr=1
      return

      end

c
c*****************************************************************************
c  orcawritehess
c*****************************************************************************
c
      subroutine orcawritehess(iunit, sizehess, hessMat, geom,
     *                           part, ipartcal)
c
c     Subroutine orcawritehess
c     
c     Created:  Oct. 02, 2007 by Christoph Riplinger
c
c     Purpose:  write an Orca hess file.
c               This routine stores the full qmmm hessian or the partial qmmm hessian. 
c              Important for the numerical hessian, since it is not symmetric.
c
c     Called by:  orca_ext_opt
c     Calls:        wordstr
c
      use files
      use input,  only   : cartgeom, partial
      implicit none
c
c i/o variables
c 

      integer            iunit
      integer            sizehess
      integer            ipartcal
      type (cartgeom) :: geom
      type (partial)  :: part
      double precision   hessMat(sizehess,sizehess)

c
c local variables
c
      integer        natoms, i, j, actcoord, pidatm
      integer        entriesInLastBlock, nrOfBlocks
c
      character*80   blockopt
      double precision fakePos

      natoms = sizehess/3
      i=MOD(sizehess,3)
      if(i.ne.0) then
         write(outfile,*) 'The number of columns in the hessian is not',
     $  ' a multiple of 3:', sizehess
         stop 'orcawritehess 1'
      endif
c
c  Print out the information that the orca-hessian-file needs
c
      write(iunit,2000)'$orca_hessian_file'
c
c  the number of the atom whose hessian values were calculated last,
c  we calculated all hessian values, thus this is the last atom
c
      write(iunit,2001)'$act_atom',natoms-1
c
c  the number of the coordinate (x,y,z) whose hessian values were calculated last,
c  we calculated all hessian values, thus this is the last coordinate (z=2)
c
      actcoord=2
      write(iunit,2001)'$act_coord',actcoord
      write(iunit,2001)'$hessian',sizehess
c
c  format of the orca-hessian-file (for a 3-atom-system):
c
c                  0          1          2          3          4          5    
c      0      -0.002908  -0.006057   0.005645   0.021282  -0.003012  -0.023875
c      1      -0.009176  -0.054807   0.066835   0.021424  -0.003259  -0.015974
c      2       0.006336   0.062166  -0.034199  -0.015140  -0.000100   0.011024
c      3       0.021743   0.018023  -0.014995  -0.001608   0.001528   0.007532
c      4      -0.004409  -0.005219  -0.001155   0.002098   0.021460  -0.015552
c      5      -0.027449  -0.014232   0.021444   0.010071  -0.010442  -0.110228
c      6      -0.024451  -0.016135   0.016253   0.005877  -0.000533  -0.005356
c      7       0.010004   0.057753  -0.064311  -0.021893  -0.000910   0.017525
c      8      -0.007695  -0.061807   0.035845   0.016671  -0.000101  -0.012818
c                  6          7          8
c      0      -0.023078   0.010391  -0.007096
c      1      -0.017418   0.065894  -0.069718
c      2       0.014922  -0.070236   0.037811
c      3       0.007643  -0.021255   0.018330
c      4      -0.000736  -0.003210  -0.000811
c      5      -0.007889   0.022499  -0.016654
c      6       0.014236   0.018653  -0.015250
c      7       0.017353  -0.061009   0.067777
c      8      -0.015273   0.068113  -0.035346
c
c
c  determine the number of full hessian blocks
c
      nrOfBlocks=INT(sizehess/6)
c
c  determine the number of half hessian blocks
c
      entriesInLastBlock=MOD(sizehess,6)
c
c  first write the full blocks
c
      do i=1, nrOfBlocks
         write(iunit,2002) (i-1)*6+0, (i-1)*6+1, (i-1)*6+2,
     *                     (i-1)*6+3, (i-1)*6+4, (i-1)*6+5
         write(outfile,2002) (i-1)*6+0, (i-1)*6+1, (i-1)*6+2,
     *                     (i-1)*6+3, (i-1)*6+4, (i-1)*6+5
         do j=1, sizehess
            write(iunit,2003)j-1,
     *            hessMat(j,(i-1)*6+1),hessMat(j,(i-1)*6+2),
     *            hessMat(j,(i-1)*6+3),hessMat(j,(i-1)*6+4),
     *            hessMat(j,(i-1)*6+5),hessMat(j,(i-1)*6+6)
            write(outfile,2003)j-1,
     *            hessMat(j,(i-1)*6+1),hessMat(j,(i-1)*6+2),
     *            hessMat(j,(i-1)*6+3),hessMat(j,(i-1)*6+4),
     *            hessMat(j,(i-1)*6+5),hessMat(j,(i-1)*6+6)
         end do
      end do
c
c  then write the half block, if necessary
c
      if(entriesInLastBlock.eq.3) then
         write(iunit,2004) nrOfBlocks*6+0,nrOfBlocks*6+1,nrOfBlocks*6+2
         do j=1, sizehess
            write(iunit,2005)j-1,hessMat(j,nrOfBlocks*6+1),
     *           hessMat(j,nrOfBlocks*6+2),hessMat(j,nrOfBlocks*6+3)
         end do
      else if(entriesInLastBlock.eq.0) then
         continue
      else
         write(outfile,*) 'The number of columns in the hessian is not',
     $  ' a multiple of 3'
         stop 'orcawritehess 2'
      endif
c
c  orca still needs the coordinates of the atoms, 
c
      write(iunit,2001)'$atoms',natoms
c
c  we don't need the fifth column
c
      fakePos=0.0
      if(ipartcal .eq. 1) then
        do i=1, part%partatmnum
          pidatm = part%partatmid(i)
          write(iunit,2006) geom%atm(pidatm), geom%angcoord(3*pidatm-2),
     *          geom%angcoord(3*pidatm-1), geom%angcoord(3*pidatm),
     *          fakePos
         end do
      else 
        do i=1, geom%natoms
          write(iunit,2006) geom%atm(i), geom%angcoord(3*i-2),
     *          geom%angcoord(3*i-1), geom%angcoord(3*i), fakePos
         end do
      endif
2000  format(/,A)
2001  format(/,A,/,I4)
2002  format(6x,6(I12))
2003  format(I7,5x,6(f10.6,1x))
2004  format(6x,3(I12))
2005  format(I7,5x,3(f10.6,1x))
2006  format (a2,4x,4(f10.6,2x))
      end

