c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine kcharge  --  assign partial charge parameters  ##
c     ##                                                            ##
c     ################################################################
c
c
c     "kcharge" assigns partial charges to the atoms within
c     the structure and processes any new or changed values
c
c
      subroutine kcharge
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'charge.i'
      include 'chgpot.i'
      include 'couple.i'
      include 'inform.i'
      include 'iounit.i'
      include 'kchrge.i'
      include 'keys.i'
      include 'potent.i'
      include 'dampchg.i'         !0108BW10
      integer i,j,k,m
      integer ia,next
      integer nc12(maxatm)
      real*8 cg
      logical header
      character*20 keyword
      character*120 record
      character*120 string
      integer ndamped         ! number of atoms are screened 0108BW10
      integer dampat          !0108BW10
      real*8 zetaat,nvalat,nchgat !0108BW10


c    by Bo Wang 14
c    initialize the parameters for charge screening (damping)
      do i = 1, n
         use_dampatm(i) = .false.
         zeta(i) = -1.0
         nval(i)=-1.0
         nchg(i)=999.0
      end do
      use_dampchg = 0
      ndamped = 0
      use_damprc  = 0
      slambda = 2.0
c     end by Bo Wang 14

c
c
c     process keywords containing partial charge parameters
c
      header = .true.
      do i = 1, nkey
         next = 1
         record = keyline(i)
         call gettext (record,keyword,next)
         call upcase (keyword)
         if (keyword(1:7) .eq. 'CHARGE ') then
            ia = 0
            cg = 0.0d0
            string = record(next:120)
            read (string,*,err=40,end=40)  ia,cg
            if (ia .gt. 0) then
               if (header) then
                  header = .false.
                  write (iout,10)
   10             format (/,' Additional Atomic Partial Charge',
     &                       ' Parameters :',
     &                    //,5x,'Atom Type',10x,'Charge',/)
               end if
               if (ia .le. maxtyp) then
                  chg(ia) = cg
                  write (iout,20)  ia,cg
   20             format (4x,i6,8x,f12.4)
               else
                  write (iout,30)
   30             format (/,' KCHARGE  --  Too many Partial Charge',
     &                       ' Parameters')
                  abort = .true.
               end if
            end if
   40       continue
         end if
      end do
c
c     find and store all the atomic partial charges
c
      do i = 1, n
         pchg(i) = chg(type(i))
      end do
c
c     process keywords containing atom specific partial charges
c
      header = .true.
      do i = 1, nkey
         next = 1
         record = keyline(i)
         call gettext (record,keyword,next)
         call upcase (keyword)
         if (keyword(1:7) .eq. 'CHARGE ') then
            ia = 0
            cg = 0.0d0
            string = record(next:120)
            read (string,*,err=70,end=70)  ia,cg
            if (ia.lt.0 .and. ia.ge.-n) then
               ia = -ia
               if (header) then
                  header = .false.
                  write (iout,50)
   50             format (/,' Additional Partial Charges for',
     &                       ' Specific Atoms :',
     &                    //,6x,'Atom',14x,'Charge',/)
               end if
               write (iout,60)  ia,cg
   60          format (4x,i6,8x,f12.4)
               pchg(ia) = cg
            end if
   70       continue
         end if
      end do
c
c     remove zero partial charges from the list of charges
c
      nion = 0
      do i = 1, n
         chglist(i) = 0
c BW14   do not remove 0 charges since they can still be screened
cx          if (pchg(i) .ne. 0.0d0) then
            nion = nion + 1
            iion(nion) = i
            jion(nion) = i
            kion(nion) = i
            pchg(nion) = pchg(i)
            chglist(i) = nion
cx         end if
      end do

c
c     optionally use neutral groups for neighbors and cutoffs
c
      if (neutnbr .or. neutcut) then
         do i = 1, n
            nc12(i) = 0
            do j = 1, n12(i)
               k = chglist(i12(j,i))
               if (k .ne. 0)  nc12(i) = nc12(i) + 1
            end do
         end do
         do i = 1, nion
            k = iion(i)
            if (n12(k) .eq. 1) then
               do j = 1, n12(k)
                  m = i12(j,k)
                  if (nc12(m) .gt. 1) then
                     if (neutnbr)  jion(i) = m
                     if (neutcut)  kion(i) = m
                  end if
               end do
            end if
         end do
      end if

c by Bo Wang 14
c  process keywords containing screened charge atoms
c
      do i = 1, nkey
         next = 1
         record = keyline(i)
         call gettext (record,keyword,next)
         call upcase (keyword)

c  do screening for all atoms suggested in the MM paper
c   MM calculations using MMODS parameters
         if (keyword(1:5) .eq. 'MMODS') then
           use_dampchg = 1
           do k=1, n
            if (scnval(atomic(k)) .ne. 0.0 ) then
              ndamped = ndamped + 1
              use_dampatm(k)= .true.
              zeta(k)=scnzeta(atomic(k))
              nval(k)=scnval(atomic(k))
              if (atomic (k) .eq. 1) then
                nchg(k)=pchg(k)
              else
                nchg(k)=0.0
              endif
            endif
           enddo
c  QDS and DAMPING are only used for correction in screened QM/MM
c  DAMPING3 and DAMPING4 can be used for both screened QM/MM
c  and screened MM
c  DAMPING suggest the screened atoms for QMMM correction
c  and the default values are the ones suggested in the manual
         elseif (keyword(1:3) .eq. 'ODS') then
           use_dampchg = 2
         elseif (keyword(1:8) .eq. 'DAMPING ') then
           if (use_dampchg == 0) then
             use_dampchg = 2
           endif
           string = record(next:120)
           read (string,*,err=80,end=80) dampat
   80      continue
           if (.not. use_dampatm(dampat)) then
             ndamped = ndamped + 1
             use_dampatm(dampat) = .true.
           endif
           zeta(dampat)=scnzeta0(atomic(dampat))
           nval(dampat)=scnval0(atomic(dampat))
           if ( atomic(dampat) .eq. 1) then
             nchg(dampat)=pchg(dampat)
           else
             nchg(dampat)=0.0
           endif
cx           write (*,*) dampat, use_dampatm(dampat), zeta(dampat),
cx     &                          nval(dampat), nchg(dampat) !debug BW

         elseif (keyword(1:8) .eq. 'DAMPING4') then
           if (use_dampchg == 0) then
             use_dampchg = 1
           endif
           string = record(next:120)
           read (string,*,err=90,end=90) dampat,zetaat,nvalat,nchgat
   90      continue
           if (zetaat .ne. -1.0) then
             if (.not. use_dampatm(dampat)) then
               ndamped = ndamped + 1
               use_dampatm(dampat) = .true.
             endif
             zeta(dampat)=zetaat
             nval(dampat)=nvalat
             nchg(dampat)=nchgat
           elseif (zetaat .eq. -1.0 .and.
     &            use_dampatm(dampat)) then
             use_dampatm(dampat) = .false.
             ndamped = ndamped - 1
           endif
cx           write (*,*) dampat, use_dampatm(dampat), zeta(dampat),
cx     &                          nval(dampat), nchg(dampat) !debug BW
         elseif (keyword(1:8) .eq. 'DAMPING3') then
           if (use_dampchg == 0) then
             use_dampchg = 1
           endif
           string = record(next:120)
           read (string,*,err=100,end=100) dampat, zetaat,nvalat
  100      continue
           if (zetaat .ne. -1.0) then
             if (.not. use_dampatm(dampat)) then
               ndamped = ndamped + 1
               use_dampatm(dampat) = .true.
             endif
             zeta(dampat)=zetaat
             nval(dampat)=nvalat
             nchg(dampat)=pchg(dampat)
           elseif (zetaat .eq. -1.0 .and.
     &         use_dampatm(dampat)) then
             use_dampatm(dampat) = .false.
             ndamped = ndamped - 1
           endif
cx           write (*,*) dampat, use_dampatm(dampat), zeta(dampat),
cx     &                          nval(dampat), nchg(dampat) !debug BW
         elseif (keyword(1:5) .eq. 'NGTO1') then
           string = record(next:120)
           read (string,*,err=110,end=110)  ngto1
  110      continue
         elseif (keyword(1:5) .eq. 'NGTO2')  then
           string = record(next:120)
           read (string,*,err=120,end=120)  ngto2
  120      continue
         elseif (keyword(1:6) .eq. 'DAMPRC') then
           string = record(next:120)
           read (string,*,err=130,end=130)  use_damprc
           use_dampchg = 2
           do k=1, n
            if (atomic(k) .eq. 0 ) then
              ndamped = ndamped + 1
              use_dampatm(k)= .true.
            endif
           enddo
  130      continue
         elseif (keyword(1:6) .eq. 'LAMBDA') then
           string = record(next:120)
           read (string,*,err=140,end=140) slambda
  140      continue
         end if
      end do

c  end by Bo Wang
c
c     turn off charge-charge and charge-dipole terms if not used
c
      if (nion .eq. 0) then
         use_charge = .false.
         use_chgdpl = .false.
      end if
      return
      end


