c
c
c*******************************************************************************
c  RDYNAMICS
c*******************************************************************************
c
      subroutine rdynamics(dyninp, 
     *     string, iend, istrt, geom)
c
c     Created:  May 24, 2010 by Soroosh Pezeshki
c     Revised:
c     based on rtestmm
c
c     Purpose:  read the DYNAMICS section
c
c     Called by:  read5
c     Calls:  rline, rlist, rvar, readrestraintfile, readrestraint, 
c               readinstantmove
c
c     Note: Basis is the force field.
c           Method is mm.
c
      use files, only : outfile, infile, pathfile, restartin
      use input, only : dynamicsinf, maxporder, maxpcalc, maxcapatm,
     *                  cartgeom
      use caltyp, only : ttiming, tdebug, tesp
      implicit none
c
c  i/o variables
c      
      integer       istrt, iend, icint, natoms
      type (dynamicsinf) :: dyninp
      type (cartgeom) :: geom
c
      character*80  string, filename
      double precision cfloat

c
c  local variables
c
      integer       isect, iscend
      character*20  rvar, string2
c
c  set default values
c
      natoms = geom%natoms 
      dyninp%steps = 100
      dyninp%ensemble = 1
      dyninp%nbfreq = 2
      dyninp%timestepl = 1.0
      dyninp%trjdumps = 1000
      dyninp%temperature = 298.0
      dyninp%pressure = 1.0
      dyninp%firsttimestep = 0
      dyninp%periodic = 1
      dyninp%fixcom = 0 
      dyninp%tempcoupling = 400.d0
      dyninp%buffersize = 2.d0
      dyninp%boxsize(:) = 0.d0
c      dyninp%seed = time()  !0330BW15 may not work with certain versions of compiler 
      dyninp%seed = 2005     !0331BW15
      dyninp%restraints = 0
      dyninp%instantmove = 0
      dyninp%readpath = 0
      dyninp%vdwcutoff = 14.d0
      dyninp%chargecutoff = 14.d0
      dyninp%vdwtaper = 13.d0
      dyninp%chargetaper = 13.d0
      dyninp%eps = 1.d-5
      tesp = 0
      dyninp%pmfgroup = 0
      dyninp%pmfatom = 0
      dyninp%thermostat = 1

c
c Nose-Hover thermostat defaults
c
      dyninp%q1 = 0.d0
      dyninp%q2 = 0.d0
      dyninp%v1 = 1.d0
      dyninp%v2 = 0.d0
      dyninp%x1 = 0.d0
      dyninp%x2 = 0.d0

c anneal default values
c
      dyninp%finaltemp = 1.0
      dyninp%equistep = 0
      dyninp%coolstep = 2000
      dyninp%lanneal = 0
      dyninp%outputfreq = 1000
c
c rattle
c
      dyninp%rattle = 0
      dyninp%rattleh = 0
      dyninp%rattleeps = 1.d-5
      dyninp%rattlemaxiteriation = 10000
c
c Partitioning
c
        dyninp%apmethode = 0
        dyninp%activecenter = 0
        dyninp%activeradius = 5.0
        dyninp%partbuffer = 0.5
        dyninp%userporder = 4
        dyninp%userpcalc = 1000
        dyninp%reduceorder = 1
        dyninp%argontest = 0
        dyninp%litest = 0
        dyninp%groupcom = 1
        dyninp%forceOutAtom = 0
c
c dynamic cap atoms
c
        dyninp%capatm%nlist = 0
        dyninp%capatm%defaulttypname = 'HLL'
        dyninp%capatm%defaulttyp = 18
        dyninp%capatm%defaultbondscale = 0.713d0
        dyninp%capatm%list(:) = 0
        dyninp%capatm%typname(:) = '   '
        dyninp%capatm%atmtyp(:) = 0
        dyninp%capatm%bondscale(:) = 0.713d0


c                  
      call rline(infile,string,istrt,isect,iscend,iend)
c
c loop through the keywords of this section until the end of the
c section is reached.
c
      do while (isect.eq.0 .and. iend.eq.0 .and. iscend.eq.0)
         if (string(istrt:istrt+4).eq.'steps') then
            dyninp%steps = 
     *          icint(rvar(string,istrt,'DYNAMICS:STEPS'))
            if (dyninp%steps .le. 0) then               
               write (outfile,502) 'STEPS'
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+12) .eq. 'firsttimestep') then
            dyninp%firsttimestep = 
     *          icint(rvar(string,istrt,'DYNAMICS:FIRSTTIMESTEP'))
         elseif (string(istrt:istrt+7) .eq. 'timestep') then
            dyninp%timestepl = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:TIMESTEP'))
            if (dyninp%timestepl .le. 0.d0) then               
               write (outfile,501) 'TIMESTEP'
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+7) .eq. 'trjdumps') then
            dyninp%trjdumps = 
     *          icint(rvar(string,istrt,'DYNAMICS:TRJDUMPS'))
           if (dyninp%trjdumps .le. 0) then               
               write (outfile,502) 'trjdumps'
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+3) .eq. 'seed') then
            dyninp%seed = 
     *          icint(rvar(string,istrt,'DYNAMICS:SEED'))
         elseif (string(istrt:istrt+10) .eq. 'outputdumps') then
            dyninp%outputfreq = 
     *          icint(rvar(string,istrt,'DYNAMICS:OUTPUTDUMPS'))
         elseif (string(istrt:istrt+7) .eq. 'nofixcom') then
            dyninp%fixcom = 0 
         elseif (string(istrt:istrt+5) .eq. 'fixcom') then
            dyninp%fixcom = 1
         elseif (string(istrt:istrt+4) .eq. 'noesp') then
            tesp = 0 
         elseif (string(istrt:istrt+2) .eq. 'esp') then
            tesp = 1
         elseif (string(istrt:istrt+6) .eq. 'restart') then
            dyninp%restart = 1 
            filename=rvar(string,istrt,'DYNAMICS:RESTART')
            open (unit=restartin, file=filename,
     *       status='old', form='formatted', err=5001)
         elseif (string(istrt:istrt+3) .eq. 'path') then
            dyninp%readpath = 1
            filename=rvar(string,istrt,'DYNAMICS:PATH')
            open (unit=pathfile, file=filename,
     *       status='old', form='formatted', err=5000)
         elseif (string(istrt:istrt+8) .eq. 'vdwcutoff') then
            dyninp%vdwcutoff = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:VDWCUTOFF'))
            if (dyninp%vdwcutoff .le. 0.d0) then               
               write (outfile,501) 'VDWCUTOFF'
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+8) .eq. 'chgcutoff') then
            dyninp%chargecutoff = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:CHGCUTOFF'))
            if (dyninp%chargecutoff .le. 0.d0) then               
               write (outfile,501) 'CHGCUTOFF'
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+7) .eq. 'vdwtaper') then
            dyninp%vdwtaper = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:VDWTAPER'))
            if(dyninp%vdwtaper.le.0.d0) then               
               write (outfile,501) 'VDWTAPER'
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+7) .eq. 'chgtaper') then
            dyninp%chargetaper = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:CHGTAPER'))
            if (dyninp%chargetaper .le. 0.d0) then               
               write (outfile,501) 'CHGTAPER'
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+6) .eq. 'epsioln') then
            dyninp%eps = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:EPSIOLN'))
            if (dyninp%eps .le. 0.d0) then               
               write (outfile,501) 'EPSIOLN'
               stop 'rdynamics 2' 
            endif

c
c restraints
c
         elseif (string(istrt:istrt+9) .eq. 'restraints') then
            dyninp%restraints = 1
            call readrestraints(dyninp, infile, geom)
         elseif (string(istrt:istrt+12) .eq. 'restraintfile') then
            dyninp%restraints = 1
            filename=rvar(string,istrt,'DYNAMICS:RESTRAINTFILE')
            call readrestraintfile(dyninp, filename, geom)
         elseif (string(istrt:istrt+13) .eq. 'constraintscan') then
            dyninp%instantmove = 1
            call readinstantmove(dyninp, infile)
         elseif (string(istrt:istrt+6) .eq. 'pmfatom') then
            dyninp%pmfatom = 
     *          icint(rvar(string,istrt,'DYNAMICS:PMFATOM'))
           if ((dyninp%pmfatom .le. 0) 
     *           .or.  (dyninp%pmfatom .gt. natoms)) then
              write(outfile,*) 'Error: Invalid pmf atom:'
     *                 ,dyninp%activecenter
              stop 'rdynamics 7'
            endif
            dyninp%pmfgroup = 0
         elseif (string(istrt:istrt+7) .eq. 'pmfgroup') then
            dyninp%pmfgroup = 
     *          icint(rvar(string,istrt,'DYNAMICS:PMFGROUP'))
            dyninp%pmfatom = 0
c
c Ensemble
c
         elseif (string(istrt:istrt+7) .eq. 'ensemble') then
            string2 =rvar(string,istrt,'DYNAMICS:ENSEMBLE')
            if (string2 .eq. 'nvt') then
                dyninp%ensemble = 2
            elseif (string2 .eq. 'nve') then
                dyninp%ensemble = 1
            else
               write (outfile,503)
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+10) .eq. 'temperature') then
            dyninp%temperature = 
     *         cfloat(rvar(string,istrt,'DYNAMICS:TEMPERATURE'))
           if (dyninp%temperature .le. 0.d0) then               
               write (outfile,501) 'TEMPERATURE'
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+9) .eq. 'thermostat') then
            string2 = rvar(string,istrt,'DYNAMICS:THERMOSTAT')
            if(string2.eq.'berendsen') then
                dyninp%thermostat = 1
            elseif (string2.eq.'nose-hoover') then 
                dyninp%thermostat = 2
            else
               write (outfile,*) 'THERMOSTAT must be BERENDSEN or ',
     *          'NOSE-HOOVER'
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+8) .eq. 'tcoupling') then
            dyninp%tempcoupling = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:TCOUPLING'))
            if (dyninp%tempcoupling .le. 0.d0) then 
               write (outfile,501) 'TCOUPLING'
               stop 'rdynamics 2' 
            endif
c
c Anneal arguments
c
         elseif (string(istrt:istrt+8) .eq. 'finaltemp') then
            dyninp%finaltemp = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:FINALTEMP'))
            if (dyninp%finaltemp .lt. 0.d0) then               
               write (outfile,501) 'FINALTEMP'
               stop 'rdynamics 2' 
            endif
         elseif  (string(istrt:istrt+12).eq.'equilibration') then
            dyninp%equistep = 
     *          icint(rvar(string,istrt,'DYNAMICS:EQUILIBRATION'))
         elseif (string(istrt:istrt+11) .eq. 'coolingsteps') then
            dyninp%coolstep = 
     *          icint(rvar(string,istrt,'DYNAMICS:COOLINGSTEPS'))
           if (dyninp%coolstep .le. 0) then               
               write (outfile,502) 'COOLINGSTEPS'
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+5) .eq. 'anneal') then
            dyninp%lanneal = 1
         elseif (string(istrt:istrt+7) .eq. 'noanneal') then
            dyninp%lanneal = 0
c
c periodic box
c
         elseif (string(istrt:istrt+7) .eq. 'periodic') then
                dyninp%periodic = 1
         elseif (string(istrt:istrt+9) .eq. 'noperiodic') then
                dyninp%periodic = 0
         elseif (string(istrt:istrt+9) .eq. 'buffersize') then
            dyninp%buffersize = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:BUFFERSIZE'))
         elseif (string(istrt:istrt+4) .eq. 'xsize') then
            dyninp%boxsize(1) = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:XSIZE'))
         elseif  (string(istrt:istrt+4) .eq. 'ysize') then
            dyninp%boxsize(2) = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:YSIZE'))
         elseif (string(istrt:istrt+4) .eq. 'zsize') then
            dyninp%boxsize(3) = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:ZSIZE'))
c
c Rattle
c
        elseif (string(istrt:istrt+11) .eq. 'rattlewater') then
            string2 =rvar(string,istrt,'DYNAMICS:RATTLEWATER')
            if (string2 .eq. 'tip') then
                dyninp%rattle = 1
            elseif (string2 .eq. 'spc') then
                dyninp%rattle = 2
            elseif (string2.eq.'no'.or.string2.eq.'off') then
                dyninp%rattle = 0
            else
               write (outfile,504)
               stop 'rdynamics 2' 
            endif
        elseif  (string(istrt:istrt+13).eq.'rattlehydrogen') then
            string2 =rvar(string,istrt,'DYNAMICS:RATTLEHYDROGEN')
            if (string2 .eq. 'qmmm') then
                dyninp%rattleh = 1
            elseif (string2 .eq. 'mm') then
                dyninp%rattleh = 2
            elseif (string2.eq.'no'.or.string2.eq.'off') then
                dyninp%rattleh = 0
            else
               write (outfile,504)
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+12) .eq. 'rattleepsioln') then
            dyninp%rattleeps = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:RATTLEEPS'))
            if (dyninp%rattleeps .le. 0.d0) then               
               write (outfile,501) 'RATTLEEPSIOLN'
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+14) .eq. 'rattleiteration') then
            dyninp%rattlemaxiteriation = 
     *          icint(rvar(string,istrt,'DYNAMICS:RATTLEIT'))
            if (dyninp%rattlemaxiteriation .le. 0) then               
               write (outfile,502) 'RATTLEITERATION'
               stop 'rdynamics 2' 
            endif
c
c Adaptive partitioning
c
         elseif (string(istrt:istrt+11) .eq. 'partitioning') then
            string2 =rvar(string,istrt,'DYNAMICS:PARTITIONING')
            if (string2 .eq. 'sortedap') then
                dyninp%apmethode = 1
            elseif (string2 .eq. 'permutedap') then
                dyninp%apmethode = 2
            elseif (string2 .eq. 'hotspot') then
                dyninp%apmethode = 3
            elseif (string2 .eq. 'oniomxs') then
                dyninp%apmethode = 4
            elseif (string2 .eq. 'das') then
                dyninp%apmethode = 5
            elseif (string2 .eq. 'nobuffer') then
                dyninp%apmethode = 6
            elseif (string2.eq.'no'.or.string2.eq.'off') then
                dyninp%apmethode = 0
            else
               write (outfile,505) string2
               stop 'rdynamics 2' 
            endif
         elseif (string(istrt:istrt+8) .eq. 'groupatom') then
            dyninp%groupcom = 0
         elseif (string(istrt:istrt+7) .eq. 'groupcom') then
            dyninp%groupcom = 1
         elseif (string(istrt:istrt+11) .eq. 'activecenter') then
            dyninp%activecenter = 
     *          icint(rvar(string,istrt,'DYNAMICS:ACENTER'))
           if ((dyninp%activecenter .le. 0) 
     *           .or.  (dyninp%activecenter .gt. natoms)) then
              write(outfile,*) 'Error: Invalid active center:'
     *                 ,dyninp%activecenter
              stop 'rdynamics 7'
           endif
         elseif (string(istrt:istrt+12) .eq. 'activecentxyz') then
           dyninp%activecentcoord(1) = 
     &      cfloat(rvar(string,istrt,'DYNAMICS:ACENTER'))
           dyninp%activecentcoord(2) = 
     &      cfloat(rvar(string,istrt,'DYNAMICS:ACENTER'))
           dyninp%activecentcoord(3) = 
     &      cfloat(rvar(string,istrt,'DYNAMICS:ACENTER'))
           dyninp%selcent=2 
         elseif (string(istrt:istrt+11) .eq. 'activeradius') then
            dyninp%activeradius = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:ACTIVERADIUS'))
         elseif (string(istrt:istrt+14) .eq. 'partitionbuffer') then
            dyninp%partbuffer = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:PBUFFER'))
         elseif (string(istrt:istrt+12) .eq. 'permutedorder') then
            dyninp%userporder = 
     *          icint(rvar(string,istrt,'DYNAMICS:PERMUTEDORDER'))
            if (dyninp%userporder .le. 0) then               
               write (outfile,502) 'PERMUTEDORDER'
               stop 'rdynamics 2' 
            endif
            if (dyninp%userporder .gt. maxporder) then
                write (outfile,*) '   ERROR: PERMUTEDORDER must be '
     *                 , 'less than ', maxporder+1
                stop 'rdynamics 2'
            endif
         elseif (string(istrt:istrt+11) .eq. 'pcalculation') then
            dyninp%userpcalc = 
     *          icint(rvar(string,istrt,'DYNAMICS:PCALCULATION'))
            if (dyninp%userpcalc .le. 0) then               
               write (outfile,502) 'PCALCULATION'
               stop 'rdynamics 2' 
            endif
            if (dyninp%userpcalc .gt. maxpcalc) then
                write (outfile,*) '   ERROR: PCALCULATION must be less'
     *                 , ' than ', maxpcalc+1
                stop 'rdynamics 2'
            endif
         elseif  (string(istrt:istrt+10) .eq. 'reduceorder') then
            dyninp%reduceorder = 1
         elseif  (string(istrt:istrt+12) .eq. 'noreduceorder') then
            dyninp%reduceorder = 0
c
c Ar and Li test
c
         elseif (string(istrt:istrt+8) .eq. 'argontest') then
            dyninp%argontest = 
     *          icint(rvar(string,istrt,'DYNAMICS:ARGONTEST'))
            dyninp%argoncutoff = 
     *          cfloat(rvar(string,istrt,'DYNAMICS:ARGONCUTOFF'))
            if (dyninp%argoncutoff .lt. 0.d0) then               
               write (outfile,501) 'ARGONCUTOFF'
               stop 'rdynamics 2' 
            endif
            if (dyninp%argontest .eq. 2) then
                dyninp%argonswitch = 
     *              cfloat(rvar(string,istrt,'DYNAMICS:ARGONSWITCH'))
                if (dyninp%argonswitch .gt. dyninp%argoncutoff) then
                    write (outfile,*) '   ERROR: ARGONSWITCH must be',
     *                  ' less than ARGONCUTOFF'
                    stop 'rdynamics 2' 
                endif
                if (dyninp%argonswitch .lt. 0.d0) then
                    write (outfile,501) 'ARGONSWITCH'
                    stop 'rdynamics 2' 
                endif
            endif
         elseif (string(istrt:istrt+5) .eq. 'litest') then
                dyninp%litest = 1
         elseif (string(istrt:istrt+7) .eq. 'nolitest') then
                dyninp%litest = 0
         elseif (string(istrt:istrt+11) .eq. 'forceoutatom') then
                dyninp%forceOutAtom = 3 *
     *              icint(rvar(string,istrt,'DYNAMICS:FORCEOUTATOM'))
c
c dynamic cap atom
c
         elseif (string(istrt:istrt+10) .eq. 'dynamiclink') then
                call readdyncap(dyninp)
c
c TIMING                                                                 
c
         elseif (string(istrt:istrt+5) .eq. 'timing') then
            ttiming = 1
         elseif (string(istrt:istrt+7) .eq. 'notiming') then
            ttiming = 0
c
c comments
c
         elseif (string(istrt:istrt) .eq. ' ') then
            string=string
         else
c
c unkown keyword
c
c
            write (outfile,1000) string(istrt:80)
            stop 'rdynamics 1'
 1000      format(3x,'ERROR: the following is not a valid keyword in ',
     *               'the DYNAMICS section: ',a)
         endif

222      call rline(infile,string,istrt,isect,iscend,iend)

      enddo
c
c NVE and NpE not possible with annealing
c
      if (dyninp%lanneal .gt. 0)then
         if (mod(dyninp%ensemble,2) .eq. 1)  then
            dyninp%ensemble=dyninp%ensemble+1
            write (outfile,515)
  515       format(3x, 'WARNING: NVE ensemble does not work with'
     *           ' anneal. Changing ensemble to NVT' )
         endif
      endif

c
c set nobuffer as Hotspot with infinitesimal buffersize
c
      if (dyninp%apmethode .eq. 6) then
        dyninp%partbuffer = 1.d-9
      endif

      return
  501 format(3x, 'ERROR: ',a,' must be a positive real number')
  502 format(3x, 'ERROR: ',a,' must be a positive integer')
  503 format(3x, 'ERROR: ENSEMBLE must be NVT or NVE')
  504 format(3x, 'ERROR: RATTLEWATER must be OFF, TIP or SPC')
  505 format(3x, 'ERROR: Partitioning type: ',a, ' not supported')
  506 format(3x, 'ERROR: RATTLEHYDROGEN must be OFF, MM or QMMM')

5000  write (*,'(/a)') 'ERROR: Cannot open the path file.'
6000  stop 'rdynamics 3'
5001  write (*,'(/a)') 'ERROR: Cannot open the restart file.'
6001  stop 'rdynamics 4'

      end

c                     
c******************************************************************************
c  READDYNCAP
c******************************************************************************
c
      subroutine readdyncap(dyninp)                                      
c  
c     Subroutine readdyncap
c
c     Created:  March 25, 2011 by Soroosh Pezeshki
c
c     Purpose:  Read the dynamic cap atom section
c
c     Called by:  rdynamics
c     Calls:   rline
c             
c
c
      use input, only : dynamicsinf, maxcapatm
      use files,  only : outfile, infile
      implicit none
c
c  i/o variables
c also add mmgeom definition below                                      
c
      integer j ,jj, nwords,isec,ieof,iscend2, icint
      double precision  cfloat
      type (dynamicsinf) :: dyninp
c
      character*80 words(40), string

      call readln(infile,words,nwords,isec,ieof,iscend2) 
      j = 0
      do while (iscend2.eq.0.and.isec.eq.0.and.ieof.eq.0)
        if (nwords .lt. 3) then
            write(outfile,250) j+1 
250         format(/3x,'ERROR: More info needed for the '
     *                 ,i4,'-th Cap atom.')
            stop 'rqmmm 1'
        endif
        string = words(1)
        if (string(1:7) .eq. 'DEFAULT') then
            dyninp%capatm%defaulttypname=words(2)
            dyninp%capatm%defaulttyp=icint(words(3))
            if (nwords .gt. 3) dyninp%capatm%defaultbondscale
     *          = cfloat(words(4))
            goto 300
        endif
        jj = icint(string)
        j = j + 1
c        if ((jj .lt. 1) .or. (jj .gt. natoms)) then
        if (jj .lt. 1) then
            write(outfile,*) 'Error: Invalid Cap atom:',jj
            stop 'rqmmm 8'
        endif
c
c copy values for cap atom change on the fly
c
        dyninp%capatm%list(j) = jj
        dyninp%capatm%typname(j) = words(2)
        dyninp%capatm%atmtyp(j) = icint(words(3))
        if (nwords .gt. 3) then
            dyninp%capatm%bondscale(j) = cfloat(words(4))
        endif
300     continue
        call readln(infile,words,nwords,isec,ieof,iscend2) 
      enddo

      dyninp%capatm%nlist = j
      end

c                     
c******************************************************************************
c  MLDHOOK
c******************************************************************************
c
      subroutine mldhook(progrms, geom, mcoptl,
     *   ltest, ltestmm, mmgeom,                      
     *   ldynamic, dyninp, lqmmm, qmmminp, mlgeninp,
     *   tmeth, tbas, toptl, ierr)                                      
c  
c     Subroutine mldhook
c
c     Created:  May 25, 2010 by Soroosh Pezeshki
c
c     Purpose:  To call the appropriate subhook and repeat
c
c     Called by:  main
c     Calls:   initmd, runmd
c             
c
c
      use input,    only : cartgeom, proglist, dynamicsinf, 
     *          mmvalen, qmmminf, maxoptl, general

      implicit none
c
c  i/o variables
c also add mmgeom definition below                                      
c
      integer            ierr, limohc, ltest, ltestmm, lqmmm, ldynamic  
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom     
      type (dynamicsinf) :: dyninp
      type (proglist) :: progrms
      type (general) :: mlgeninp
c
      character*80    :: mcoptl(maxoptl), toptl(maxoptl)
      character*20    :: tmeth, tbas
      type (qmmminf)  :: qmmminp                                        

c
c  local variables
c
           mlgeninp%lgrad = 1
c initialize the MD calculation
c
      call initMD(dyninp, geom, mmgeom, toptl,mlgeninp, 
     *                 qmmminp, ltestmm, tmeth, tbas,progrms, lqmmm)

c
c  calculate the dynamic
c
      call runmd(progrms, geom, mmgeom, dyninp,  mlgeninp,
     &          qmmminp, lqmmm, ltest, ltestmm,
     &         tmeth, tbas, toptl,ierr)

      return
      end

c
c
c
c*****************************************************************************
c  INITMD
c*****************************************************************************
c
      subroutine initmd(dyninp, geom, mmgeom, toptl, mlgeninp,
     *                qmmminp, ltestmm, esmeth, esbas, progrms, lqmmm)
c
c     Created:  May 25, 2010 Soroosh Pezeshki
c
c     Purpose:  
c
c     Description:  Initialize the MD simulations
c                   Distribiute velocity randomly based on temperature
c
c
c     Called by:  mldhook 
c     Calls:  readrestart, findmolecules, gauss, boxsize, dout0,t41inp,
c               calculateArgonShift, moloutbox, rattleinit, readgroups
c
      use input
      use files
      use caltyp, only: tdebug, tdynamics, tgrad, tap, ttiming
      use gradient, only : qmmmmmgr
      use energy, only   : mmes
      implicit none
      type (dynamicsinf) :: dyninp
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (general)  :: mlgeninp
      type (qmmminf)  :: qmmminp                                        
      type (proglist) :: progrms
c
      character*80    :: toptl(maxoptl), keystring
      character*20 esmeth, esbas


        double precision amas, sd
        integer natoms, ltestmm, lqmmm, digits8, maxmmopt, n3tm
        integer i, j, a, i3, cycels, lmmvalen, ierr
        double precision box(9), speed, temperature, cooltime
        double precision gaussrandom, msystem, p(3)
        double precision degfreedom, degperma, tempconst, ekin
        character axis


c
c  timing variables
c
      real timing1, timing2
      integer timings1, timings2
      call CPU_TIME(timing1)


        natoms = geom%natoms
        n3tm = 3 * geom%natoms
        dyninp%lmmvalen = mlgeninp%lmmvalen
        box(:) = 0.d0

        dyninp%sum1 = 0.d0
        dyninp%sumwall = 0.d0
        digits8 = 0
        dyninp%work = 0.d0

c
c set inital values for AP statistics
c
        dyninp%minbuffer = maxbuffergroupps
        dyninp%minactive = geom%natoms
        dyninp%maxbuffer = 0
        dyninp%maxactive = 0
        dyninp%averageactive = 0
        dyninp%averagebuffer = 0
        dyninp%average(:) = 0.d0
        dyninp%avghl = 0
        dyninp%groupatom(:) = 0
c
c For permuted AP:
c check if PCALCULATION and PERMUTEDORDER have reasonable values
c
      if(dyninp%apmethode.eq.2) then
        if (2**dyninp%userporder.gt.dyninp%userpcalc) then
            dyninp%userpcalc = 2**dyninp%userporder + 1
            if (dyninp%userpcalc.gt.maxpcalc) then 
                write(*,*) "ERROR: PCALCULATION has a pointless value"
                write(*,*) "   Please reduce PERMUTEDORDER"
                stop 'initmd 8'
            endif
            write(*,*) "Warning: PCALCULATION has a pointless value"
            write(*,*) "   PCALCULATION changed to", dyninp%userpcalc
        endif
      endif

c
c set nobuffer as Hotspot with infinitesimal buffersize
c
      if (dyninp%apmethode.eq.6) then
        dyninp%apmethode = 3
      endif



c set the dynamics flag to reduce the output if not the debug flag is set.
c
        tdynamics = 1
        tap = ltestmm
c        if (tdebug.eq.0) tdynamics=1

c calculate delta temperature in annealing simulations
c
        if (dyninp%lanneal.eq.1) then
            dyninp%initaltemp = dyninp%temperature
            cycels = dyninp%steps / (dyninp%coolstep+dyninp%equistep)
            cooltime = 1.d0 * dyninp%coolstep * cycels
            dyninp%deltatemp = (dyninp%finaltemp - dyninp%initaltemp)
     *                 / cooltime
        endif

c
c  Distribiute inital velocities as GAUSSIAN DISTRIBIUTION
c        boltz=1.0
c
        temperature = dyninp%temperature / tempconvert
        dyninp%comvelocity(:) = 0.d0
c restart a simultion if restart is set
c
      if (dyninp%restart.eq.1) then
        call readrestart(geom,dyninp)
      else
        do i = 1, natoms
            i3 = i * 3
            amas = geom%atmmss(i) / massconvert
            sd = dsqrt(temperature / amas)
            speed = gaussrandom(dyninp%seed) * sd
            dyninp%velocity(i3-2) = speed * velocityconvert
            speed = gaussrandom(dyninp%seed) * sd
            dyninp%velocity(i3-1) = speed * velocityconvert
            speed = gaussrandom(dyninp%seed) * sd
            dyninp%velocity(i3) = speed * velocityconvert
            dyninp%comvelocity(1:3) = dyninp%comvelocity(:) +
     *                    geom%atmmss(i) * dyninp%velocity(i3-2:i3)
        enddo
      endif


c
c if partitioning is used check if active center is defined
c
      if(dyninp%apmethode.gt.0) then
            i = dyninp%activecenter + qmmminp%selcent + dyninp%selcent
            if (i.le.0) then
                write (outfile,*) "ERROR: No active center;",
     *              "please set either QMMMCUTOFF or ACTIVECENTER"
                STOP "InitMD 1"
            endif
c
c if active center is defined in dynamics section as atom index
c
            if (dyninp%activecenter.gt.0) then
                dyninp%selcent = 1
c
c if active center is defined in dynamics section as coordinate
c
            elseif (dyninp%selcent.eq.2) then
                a = natoms + 1
                i = a * 3
                dyninp%activecenter = a
                if (dyninp%restart.eq.0) then
                    geom%angcoord(i-2:i) = dyninp%activecentcoord(1:3)
                endif
                geom%atmmss(a) = 1.d6
c
c else if use the qmmm cutoff atom or coordinate
c
            elseif (qmmminp%selcent.eq.1) then
                dyninp%activecenter = qmmminp%ccutoffid
                dyninp%selcent = 1
c
c define dummy atom if xyz is set
c
            elseif (qmmminp%selcent.eq.2) then
                dyninp%selcent = 2
                a = natoms + 1
                i = a * 3
                dyninp%activecenter = a
                if (dyninp%restart.eq.0) then
                    geom%angcoord(i-2:i) = qmmminp%cutoffcoor(1:3)
                endif
                geom%atmmss(a) = 1.d6
            endif

c
c if argon test calculate the square of distances
c if argon test with shift calculate shift
c
        if (dyninp%argontest.gt.0) then
            dyninp%shiftFM = 0.d0
            dyninp%shiftPM = 0.d0
            dyninp%shiftPLJ = 0.d0
            dyninp%shiftFLJ = 0.d0
            dyninp%switchdist = dyninp%argoncutoff-dyninp%argonswitch
            dyninp%argoncutoff= dyninp%argoncutoff*dyninp%argoncutoff
        endif
        if (dyninp%argontest.eq.3) then
            call calculateArgonShift(dyninp)
        endif

      endif

c
c  mark each covalently bond molecule with a molecule id
c
      call findmolecules(natoms,dyninp,mmgeom,geom, qmmminp)

c
c  extract the com velocity from the velocity
c
      if (dyninp%restart.eq.0) then
        ekin = 0.d0
        p(:) = 0.d0
        msystem = dyninp%commass
        if (dyninp%selcent.eq.2) msystem = msystem - geom%atmmss(a)
        dyninp%comvelocity(1:3) = dyninp%comvelocity(1:3) / msystem
        do i = 1, natoms
            i3 = i * 3
            dyninp%velocity(i3-2:i3) = dyninp%velocity(i3-2:i3)
     *              - dyninp%comvelocity(:)
            ekin = ekin + geom%atmmss(i) * (dyninp%velocity(i3)**2 +
     *            dyninp%velocity(i3-1)**2 +dyninp%velocity(i3-2)**2)
c            p(1:3)=p(1:3)+dyninp%velocity(i3-2:i3)*geom%atmmss(i)
            p(1) = p(1) + dyninp%velocity(i3-2) * geom%atmmss(i)
            p(2) = p(2) + dyninp%velocity(i3-1) * geom%atmmss(i)
            p(3) = p(3) + dyninp%velocity(i3) * geom%atmmss(i)
        enddo
        degfreedom = n3tm - 3
        tempconst = 2.d0 / (1.d0 * degfreedom) * tempconvert
        ekin = ekin*0.5d0/velocityconvert/velocityconvert/massconvert
        dyninp%tempnow = ekin * tempconst
        write(outfile,2111) ekin
        write(outfile,2112) dyninp%tempnow
        write(outfile,2113) p(1:3)
2111    format(5x,'Initial kinetic energy [Hartree]=', d20.5)
2112    format(5x,'Initial temperature [K]=', f8.3)
2113    format(5x,'Initial Momentum [uAng./fs]=',d13.5,d13.5,d13.5)
      endif
      if ((tdebug.eq.1).or.(tdebug.eq.2)) then
           write(outfile,*) 
     *      "DebugDebugDebugDebugDebugDebugDebugDebugDebugDebug"
           write(outfile,*) "Velocities [Angstrom/fs]"
           write(outfile,2055) dyninp%velocity(:n3tm)
           write(outfile,*) 
     *      "DebugDebugDebugDebugDebugDebugDebugDebugDebugDebug"
2055  format(t20,f15.8,t40,f15.8,t60,f15.8)
      endif

c
c set 'digits 8' if not set by user
c
      j = 1
      if (ltestmm.eq.1) then
          do while (toptl(j).ne.'   ' .and. j.le.maxoptl)
              if (toptl(j).eq.'digits 8') digits8 = 1
              j = j + 1
          enddo
      else
          do while (qmmminp%mmoptl(j).ne.'   ' .and. j.le.maxoptl)
              if (qmmminp%mmoptl(j).eq.'digits 8') digits8 = 1
              j = j + 1
          enddo
      endif
      if (digits8.eq.0) then 
        if (ltestmm.eq.1) then
            toptl(j) = 'digits 8'
        else
            qmmminp%mmoptl(j) = 'digits 8'
        endif
        j = j + 1
      endif
      maxmmopt = j
      mmgeom%maxmmopt = j - 1
c
c calculate the box size and set cutoff parameters
c
      write(outfile,'(//)')
      call boxsize(geom, box)
1112    format('vdw-cutoff ',f7.2)
1113    format('chg-cutoff ',f7.2)
1114    format('vdw-taper ',f7.2)
1115    format('chg-taper ',f7.2)
      if (ltestmm.eq.1) then
        write(keystring,1112) dyninp%vdwcutoff
        toptl(maxmmopt) = keystring
        maxmmopt = maxmmopt + 1
        write(keystring,1114) dyninp%vdwtaper
        toptl(maxmmopt) = keystring
        maxmmopt = maxmmopt + 1
        write(keystring,1113) dyninp%chargecutoff
        toptl(maxmmopt) = keystring
        maxmmopt = maxmmopt + 1
        write(keystring,1115) dyninp%chargetaper
        toptl(maxmmopt) = keystring
        maxmmopt = maxmmopt + 1
      else
        write(keystring,1112) dyninp%vdwcutoff
        qmmminp%mmoptl(maxmmopt) = keystring
        maxmmopt = maxmmopt + 1
        write(keystring,1114) dyninp%vdwtaper
        qmmminp%mmoptl(maxmmopt) = keystring
        maxmmopt = maxmmopt + 1
        write(keystring,1113) dyninp%chargecutoff
        qmmminp%mmoptl(maxmmopt) = keystring
        maxmmopt = maxmmopt + 1
        write(keystring,1115) dyninp%chargetaper
        qmmminp%mmoptl(maxmmopt) = keystring
        maxmmopt = maxmmopt + 1
      endif
      
      if(dyninp%periodic.eq.1) then
        do i = 1, 3
            axis = char(119 + i)
            dyninp%boxmin(i) = box(i) - dyninp%buffersize
c        write (outfile,5001) 'Min.', axis, dyninp%boxmin(i)
            dyninp%boxmax(i) = box(i + 3) + dyninp%buffersize
c        write (outfile,5001) 'Max.', axis, dyninp%boxmax(i)
            if(dyninp%boxsize(i).le.0.d0) then
                dyninp%boxsize(i) = dabs(dyninp%boxmax(i) -
     *                dyninp%boxmin(i))
            endif
            write (outfile,5002) axis, dyninp%boxsize(i)
            axis = char(96 + i)
            write (keystring,1111) axis, dyninp%boxsize(i)
1111    format(a1,'-axis ',f9.4)
            if (ltestmm.eq.1) then
                toptl(maxmmopt) = keystring
            else
                qmmminp%mmoptl(maxmmopt) = keystring
            endif
            maxmmopt = maxmmopt + 1
        enddo
      endif


c
c if QM atoms defined set first QM atom as center of box
c if AP is used set the active atom as center of box
c
      dyninp%ap_atom_zone(:) = 0
      if(ltestmm .ne. 1) then
          a = 3 * qmmminp%qmatm(1)
          if (dyninp%selcent.gt.0) a = 3 * dyninp%activecenter
          if(dyninp%periodic.eq.1) then
            dyninp%boxmin(1:3) = geom%angcoord(a-2:a) -
     *              dyninp%boxsize(1:3) / 2.d0
            dyninp%boxmax(1:3) = geom%angcoord(a-2:a) +
     *              dyninp%boxsize(1:3) / 2.d0
            call moloutbox(geom, dyninp)
          endif
c
c set partitionzone=1 for qm atoms
c
          do i= 1, qmmminp%qmnatm
              j = qmmminp%qmatm(i)
              dyninp%ap_atom_zone(j) = 1
          enddo
      endif
c      if (ltestmm.ne.1) toptl(:) = qmmminp%mmoptl(:)

c write keywords for tinker
c
        open (unit=t41inf, status='unknown', file=t41in, err=9994)
        open (unit=t41keyf, status='unknown', file=t41key, err=9995)
        if (ltestmm.eq.1) then
            call t41inp(t41inf, t41keyf,
     *          esmeth, esbas, geom, mmgeom,toptl,tgrad,'null')
        elseif (lqmmm .eq. 1)  then
            call t41inp(t41inf, t41keyf,esmeth, esbas,
     *          geom, mmgeom,qmmminp%mmoptl,tgrad,'null')
        endif
        close (t41inf, err=9994)
        close (t41keyf, err=9995)

c
c constraint scan (time step length=0)
c
      if(dyninp%instantmove.gt.0) then
c        dyninp%timestepl = 0.d0
        dyninp%rattle = 0
        dyninp%rattleh = 0
        if (dyninp%restraints.gt.0) then
             write (*,'(/a)') 'ERROR: Either CONSTRAINTSCAN or ',
     *          'RESTRAINT can be set but not both together'
                stop 'initMD 2'
        endif
      endif
c
c debug output restraints
c
      if ((dyninp%restraints.gt.0) .and. (tdebug .gt. 0)) then
        write(*,*) "     --------------------------------------------"
        write(*,*) "     Restraint parameter"
        write(outfile,*) dyninp%natmcfsmd, " cf on atom"
        do a = 0, dyninp%natmcfsmd - 1
            i = a + dyninp%firstatmcf
            write(outfile,21) dyninp%smdatom(i,1), dyninp%smd(i,1:3)
        enddo
        write(outfile,*) dyninp%ngrpcfsmd, " cf on group"
        do a = 0, dyninp%ngrpcfsmd - 1
            i = a + dyninp%firstgrpcf
            write(outfile,21) dyninp%smdatom(i,1), dyninp%smd(i,1:3)
        enddo
        write(outfile,*) dyninp%natmcvsmd, " cv on atom"
        do a = 0, dyninp%natmcvsmd - 1
            i = a + dyninp%firstatmcv
            write(outfile,22) dyninp%smdatom(i,1:2), dyninp%smd(i,1:3)
        enddo
        write(outfile,*) dyninp%ngrpcvsmd, " cv on group"
        do a = 0, dyninp%ngrpcvsmd - 1
            i= a + dyninp%firstgrpcv
            write(outfile,22) dyninp%smdatom(i,1:2), dyninp%smd(i,1:3)
        enddo
        write(*,*) "     --------------------------------------------"
      endif
21    format(5x,i6,3d15.8)
22    format(5x,i6,i2,3d15.8)

c
c initialize rattle
c
      dyninp%maxrattle = 0
      if(dyninp%rattle + dyninp%rattleh.gt.0) then
        call rattleinit(geom,mmgeom,dyninp)
      endif

c
c if partitioning is set or if group restraints is set, 
c  read groups and set the first partition
c
      if (dyninp%apmethode + dyninp%ngrprestraints + 
     *      dyninp%ngrpcvsmd + dyninp%ngrpcfsmd .gt. 0) then
c
c calculate the charges if electrostaic embeding
c
        if ((dyninp%apmethode .gt. 0))  then
            call progmmghk(1,'tinker',qmmminp%mmmeth,qmmminp%mmbas,
     *          geom, mmgeom, qmmminp%mmoptl, mmes(2), 
     *          qmmmmmgr(2,:geom%natoms*3), ierr, 'null')
        endif
c
c read groups
c
        call readgroups(dyninp, geom, mmgeom)
      endif

c
c If partitioning with Orca don't use AUTOSTART
c Prevents error in the orca calculation
c
      if ((dyninp%apmethode.gt.0) .and.
     *      (progrms%qmprog(:4).eq.'orca')) then
        j = 1
        do while (qmmminp%qmoptl(j).ne.'   ' .and. j.le.maxoptl)
            j = j + 1
        enddo
        if (j.gt.maxoptl) then
            write (*,'(/a)') 'ERROR: Too many QMKEY keywords'
            stop 'initMD 3'
        endif
        qmmminp%qmoptl(j) = '        > ! NOAUTOSTART'
      endif

c      
c open the trajectory file
c
      open (unit = trajfile, file = trajfilename, status = 'unknown',
     *      form = 'formatted', err = 5000)

c write out the inital positions
c
      call dout0(geom, mmgeom, natoms, dyninp%lmmvalen,dyninp)


! c
! c create sub directory if parallel
! c
!       call create_thread_dir(dyninp%nthreads)
! 

c     The program can not calculate the QM/MM single-point gradient
C     using Gamess with electric embedding
c
      if (ltestmm.ne.1) then
        if((progrms%qmprog(:6) .eq. 'gamess') .and.
     $        (qmmminp%embed .ne. 'mechanical')) then
            write(outfile,*) 'The program can not calculate the ',
     $           'QM/MM single-point gradient using Gamess with ',
     $           'electric embedding.'
            stop 'initmd 9'
        end if
      endif


c
c    CPU time after initalisation of MD
c
      if (ttiming.eq.1) then
        call CPU_TIME(timing2)
        write (outfile,*) '     initMD timing=', timing2 - timing1
      endif


5001  format(5x, a3,' ', a1, ' box size set to ', f10.2)
5002  format(5x, a1, ' box size set to ', f10.2)
      return
9994  write (outfile,*) 'Error creating TINKER .xyz file'
      stop 'initMD 4'
9995  write (outfile,*) 'Error creating TINKER .key file'
      stop 'initMD 5'
5000  write (outfile,'(/a)') 'ERROR: Cannot open the trajectory file.'
      stop 'initMD 6'
      end

c
c
c
c*****************************************************************************
c  RUNMD
c*****************************************************************************
c
      subroutine runmd(progrms, geom, mmgeom,dyninp, mlgeninp,
     *                 qmmminp, lqmmm, ltest, ltestmm,
     *                tmeth, tbas, optl, ierr)
c
c     Created:  May 25, 2010 Soroosh Pezeshki
c
c     Purpose:  
c
c     Description:  Performe MD simulations
c
c
c     Called by:  mldhook 
c     Calls:  mlghook, calccom, moloutbox, tempannealing, tempcontrol,
c               dout, doutt, partition, rattleposition,
c               restraints, instantmove, rattlevelocity, tempcontrolnh
c
      use input 
      use gradient, only : totalgrad
      use optimize, only : ener
      use files
      use caltyp, only: tdebug, tdynamics, ttiming                      
      implicit none


c
c  i/o variables
c
      integer          :: lqmmm, ltest, ltestmm            
c
      character*80     :: mcoptl(maxoptl)
      type (cartgeom) :: geom
      type (mmvalen) ::  mmgeom
      type (general) ::  mlgeninp
      type (dynamicsinf) :: dyninp
c
      character*80       optl(maxoptl)
      integer            ierr, stepnow, deltastep
      type (proglist)  :: progrms
      type (qmmminf)  :: qmmminp  
      character*20    :: tmeth, tbas
      double precision box(9)
      integer coolstep, equistep
c
c  local variables
c
      integer i,j,i3,a, pbox, degfreedom, degperma
      integer natoms, n3tm, natomsd
      double precision sumforce(3), sumabsforce
      double precision p(3), ekin, ekincons, com, dt, v, dt2, dth
      double precision tempconst, tempnow, temperature
      double precision epotold, integralgradient
c      double precision paverage(3), eaverage, paverage2(3), eaverage2, enerold
c
c Timing variables
c
      double precision timingtime, wdeltatiming, deltatimingw
      real timestart, timeend
      real timing2, timing1, deltatiming
      integer timing_rate, timing_max, wtimestart, wtimeend
      integer timingw1, timingw2

      call CPU_TIME(timing1)
      natoms = geom%natoms
c
c Add dummy atom to natomsd
c
      natomsd = natoms
      if (dyninp%selcent.eq.2) natomsd = natomsd + 1
      n3tm = 3 * natomsd
      dt = dyninp%timestepl
      dt2 = 0.5d0 * dt * dt
      dth = 0.5d0 * dt
      coolstep = 0
      equistep = 0
      stepnow = 0
      box(:) = 0.d0
c
c constant for calculation of kinetic energy in atmoic units
c
      ekincons = 0.5d0 /velocityconvert /velocityconvert /massconvert
c
c  Flag for periodic boundary useage
c
      pbox = dyninp%periodic
c
c tempconst=2/(k_B*(3*natoms-N_C))
c Temperature=tempconst*kinetic energy
c Boltmann constant k_B=1  (using a.u.)          !1.380 6504(24)E−23 J/K
c Boltmann constant k_B=0.00198721 kcal/mol/K
c degree of freedom with fix center of mass: 3*natoms-3
c     -23*number of constraint water molecules 
c
      degperma = n3tm - 3*dyninp%fixcom - 3*dyninp%maxrattle - 3*pbox
      degfreedom = degperma - dyninp%maxrattleh + dyninp%rattleskip
c      degfreedom=3*natoms-3*dyninp%fixcom
c      tempconst=2.d0/(1.d0*degfreedom)*tempconvert
      tempconst= 2.d0 / (0.00198721d0 * degfreedom) * energyconvert
      write(outfile,"(5x,a,I5)") "Number of degree of freedom: ",
     *         degfreedom
c
c Nose-Hover thermostat defaults
c
      dyninp%q1 = dble(degfreedom+1)*dyninp%temperature*k_B*
     *               dyninp%tempcoupling*dyninp%tempcoupling
      dyninp%q2 = dyninp%q1
      dyninp%v1 = dsqrt(dyninp%temperature*k_B/dyninp%q1)
      dyninp%tempcoupling = dyninp%timestepl / dyninp%tempcoupling

      flush (outfile)
c
c calculate the forces at t=0
c calculate the gradient if it is not done before
c
      if(dyninp%restart.eq.0) then
        if (dyninp%apmethode.gt.0) then
            call partition(progrms, geom, mcoptl,
     *          ltest, ltestmm, mmgeom, lqmmm, qmmminp,
     *          tmeth, tbas, optl, ierr, dyninp,stepnow)
        else
            call mlghook(progrms, geom,
     *       mcoptl, ltest, ltestmm, mmgeom,          
     *       lqmmm, qmmminp,
     *       tmeth, tbas, optl, ierr)                         
        endif
        dyninp%force(1:n3tm) = -1.d0 * totalgrad(1:n3tm) * forceconvert
      endif

c
c    CPU time before loop
c
      if (ttiming.eq.1) then
        call CPU_TIME(timing2)
        deltatiming=timing2-timing1
        timing1=timing2
        write (outfile,*) '     Iteration start timing: ', deltatiming
      endif
c
c    CPU time steps
c
      call CPU_TIME(timestart)
      call SYSTEM_CLOCK(timingw1, timing_rate, timing_max)
      wtimestart=timingw1


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c Loop over all dynamic steps
c
      do stepnow = 1, dyninp%steps
c
c reset and store variables
c
!         enerold=ener+ekin
        ekin = 0.d0
        p(:) = 0.d0
        dyninp%comvelocity(:) = 0.d0
        epotold = ener
        dyninp%coordold(1:n3tm) = geom%angcoord(1:n3tm)

        if (tdebug .gt. 0) then
            write(*,*) "Timestep: ", stepnow
        endif

cccccccccccccccccccccccccccc
c integrate the POSITION
c

c
c ATTENTION: aucoord are not updated!
c
        do i = 1, natomsd
            do j = -2,0
                i3 = i * 3 + j
                geom%angcoord(i3) = geom%angcoord(i3) +
     *              dt * dyninp%velocity(i3) +
     *              dt2 * dyninp%force(i3) / geom%atmmss(i)

            enddo
        enddo
c
c in case of peiodic boundary keep "molecules in the box"
c
        if (pbox.ne.0) then
c
c If it is pure MM calculation keep the size of the box
c
            if(ltestmm .ne. 1) then
c
c set the first QM atom or in AP the center of active zone as the center of box
c
                a = 3 * qmmminp%qmatm(1)
                if(dyninp%selcent.ge.1) a = 3 * dyninp%activecenter
                dyninp%boxmin(1:3) = geom%angcoord(a-2:a) -
     *              dyninp%boxsize(1:3) / 2.d0
                dyninp%boxmax(1:3) = geom%angcoord(a-2:a) +
     *              dyninp%boxsize(1:3) / 2.d0
            endif
            call calccom(natoms, dyninp, geom)
            call moloutbox(geom, dyninp)
        endif

c
c rattle algorithm to constraint positions
c
      if(dyninp%rattle + dyninp%rattleh .gt. 0) then
        call rattleposition(geom, mmgeom, dyninp)
            degfreedom = degperma-dyninp%maxrattleh+dyninp%rattleskip
cc            tempconst=2.d0/(1.d0*degfreedom)*tempconvert
            tempconst = 2.d0/(0.00198721d0*degfreedom)*energyconvert
      endif

c
c If preset path read coordinates
c
        if(dyninp%readpath .gt. 0) then
            call readpath(geom)
        endif



c
c    CPU time Coord. calculation
c
        if (ttiming .eq. 1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,701) 'calc. coordinates',
     *                  stepnow, deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=(1.d0*(timingw2-timingw1))/(1.d0*timing_rate)
            timingw1=timingw2
            write (outfile,*) '   Wall time=',deltatimingw
        endif


ccccccccccccccccccccccccccccc
c  integrate the VELOCITY part 1
c
        do i = 1, natomsd
            i3 = i * 3
            dyninp%velocity(i3-2:i3) = dyninp%velocity(i3-2:i3) +
     *                dth*dyninp%force(i3-2:i3) / geom%atmmss(i)
        enddo
! c
! c Nose Hoover temperature control in NVT at half time step
! c
!         if (mod(dyninp%ensemble,2).eq.0) then
!             if (dyninp%thermostat .eq. 2) then
!                 call tempcontrolnh(dyninp, ekin, p, dt, degfreedom,
!      *              ekincons)
!             endif
!         endif

cccccccccccccccccccccccccccc
c calculate the FORCES
c

c
c check if the integral of the gradient over the position change is equal 
c the energy change PART I
c
        if (tdebug .gt. 0) then
            integralgradient = 0.d0
            do i = 1, n3tm
                integralgradient = integralgradient + totalgrad(i)
     *              * (geom%angcoord(i) - dyninp%coordold(i))
            enddo
        endif


        if (dyninp%apmethode .gt. 0) then
            call partition(progrms, geom, mcoptl,
     *          ltest, ltestmm, mmgeom, lqmmm, qmmminp,
     *          tmeth, tbas, optl, ierr, dyninp,stepnow)
        else
            call mlghook(progrms, geom, mcoptl, ltest,
     *          ltestmm, mmgeom, lqmmm, qmmminp,
     *          tmeth, tbas, optl, ierr)
        endif
        dyninp%force(1:n3tm) = -1.d0 * totalgrad(1:n3tm) * forceconvert
c
c Check if the sum of the forces is 0
c
        sumforce(:) = 0.d0
        sumabsforce = 0.d0
        do i = 3, n3tm, 3
            sumforce(1:3) = sumforce(1:3) + dyninp%force(i-2:i)
            sumabsforce = sumabsforce + dabs(dyninp%force(i-2)) +
     *              dabs(dyninp%force(i-1)) + dabs(dyninp%force(i))
        enddo
        if ((dabs(sumforce(1))+dabs(sumforce(2))+dabs(sumforce(3)))
     *          .gt. 1.d-9*sumabsforce) then
c            write(outfile,1010) sumabsforce
            write(outfile,1012)
            write(outfile,1011) sumforce(1:3)
1010        format(5x, "Note: sum of the forces is not 0",
     *              " (Abs. sum ", D14.7,")")
1012        format(5x, "Note: sum of the forces is not 0")
1011       format(5x,d19.8 , d19.8,d19.8)
        endif
        if ((tdebug.eq.1) .or. (tdebug.eq.2)) then
            write(outfile,1013) 
            write(outfile,1011) sumforce(:)/natoms
            write(*,*) "     Force magnitude per atom: ", 
     *          dsqrt(sumforce(1) * sumforce(1) + 
     *                sumforce(2) * sumforce(2) +
     *                sumforce(3) * sumforce(3)) / natoms
1013        format(5x, "Sum of forces per atom [amu Angstrom/fs^2]:")
        endif
c
c check if the integral of the gradient over the position change is equal 
c the energy change
c
        if (tdebug.gt.0) then
            do i = 1, n3tm
                integralgradient = integralgradient + totalgrad(i)
     *              * (geom%angcoord(i) - dyninp%coordold(i))
            enddo
            integralgradient = integralgradient * lengthconvert /2.d0
            if(integralgradient + epotold - ener .gt.
     *              dyninp%eps * integralgradient) then
                write(outfile,1014)
c                write(*,1015) integralgradient + epotold, ener, 
c     *              integralgradient + epotold - ener
                write(outfile,1016) integralgradient + epotold - ener
1014  format(5x,"Note: Integral gradient is not eq. potential energy")
1015  format(9x, d15.5,t30, d15.5, t50, d15.5)
1016  format(9x, 'Difference [hartree]= ', d15.5)
            endif
        endif
c
c if restraints is set apply
c
        if (dyninp%restraints .gt. 0) then
            call restraints (dyninp, geom, stepnow)
        endif

c
c Output DAS energy for last time step
c
!         if (dyninp%apmethode.eq.5) then
!             write(*,*) "DAS energy ",(deltastep-1)*dt, 
!      *          enerold+dyninp%wbk/2.d0
!         endif
c
c    CPU time force calculation
c
        if (ttiming .eq. 1) then
            call CPU_TIME(timing2)
            deltatiming = timing2 - timing1
            timing1 = timing2
            write (outfile,701) 'calc. forces', stepnow, deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw = 1.d0*(timingw2-timingw1)/(1.d0*timing_rate)
            timingw1 = timingw2
            write (outfile,*) '   Wall time=',deltatimingw
        endif


c
c If constraint scan shift and skip the velocity part
c
        if(dyninp%instantmove .gt. 0) then
            call instantmove(dyninp, geom)
            goto 300
        endif


ccccccccccccccccccccccccccccc
c  integrate the VELOCITY
c
        do i = 1, natomsd
c            do j=-2,0
c                i3=i*3+j
                i3 = i * 3
                dyninp%velocity(i3-2:i3) = dyninp%velocity(i3-2:i3) +
     *            dth * (dyninp%force(i3-2:i3)) / geom%atmmss(i)
c                dyninp%velocity(i3)=dyninp%velocity(i3)+dth*
c     *            (dyninp%force(i3))/geom%atmmss(i)
c            enddo
        enddo

c
c rattle algorithm to constraint velocity
c
        if(dyninp%rattle+dyninp%rattleh .gt. 0) then
            call rattlevelocity(geom, mmgeom, dyninp)
        endif

c
c calculate the velocity of com
c
        do i = 1, natomsd
                i3 = i * 3
                dyninp%comvelocity(1:3) = dyninp%comvelocity(1:3) +
     *                       dyninp%velocity(i3-2:i3) * geom%atmmss(i)
        enddo
        dyninp%comvelocity(:3) = dyninp%comvelocity(:3)/dyninp%commass

c
c  extract the com velocity from the velocity if fixcom
c and calculate the kinetic energy and the momentum
c
        if (dyninp%fixcom .eq. 1) then
            do i = 1, natomsd
                do j = -2, 0
                    i3 = i * 3 + j
                    v = dyninp%velocity(i3) - dyninp%comvelocity(3+j)
                    dyninp%velocity(i3) = v
                    p(3+j) = p(3+j) + v * geom%atmmss(i)
                    ekin = ekin + geom%atmmss(i) * v * v
                enddo
            enddo
            ekin = ekin * ekincons
            dyninp%tempnow = ekin * tempconst
c
c  extract the com velocity from the temperature if nofixcom
c and calculate the kinetic energy and the momentum
c
        else
!             tempnow=0.d0
            do i = 1, natomsd
                do j = -2, 0
                    i3 = 3 + j
                    v = dyninp%velocity(i * 3 + j)
!                      tempnow=tempnow+geom%atmmss(i)*
!      *                         (v-dyninp%comvelocity(i3))**2
                    p(i3) = p(i3) + v * geom%atmmss(i)
                    ekin = ekin + geom%atmmss(i) * v * v
                enddo
            enddo
            ekin = ekin * ekincons
            dyninp%tempnow = ekin * tempconst
!             dyninp%tempnow=tempnow*ekincons*tempconst
        endif

c
c temperature control in NVT and NpT and annealing
c
        if (mod(dyninp%ensemble,2).eq.0) then
            if (dyninp%lanneal.eq.1) then
                call tempannealing(dyninp, coolstep, equistep)
            endif
            if (dyninp%thermostat .eq. 1) then
                call tempcontrol(dyninp, ekin, n3tm, p, ekincons)
            else
                call tempcontrolnh(dyninp, ekin, p, dt, degfreedom, 
     *              ekincons)
            endif
        endif

c
c    CPU time velocity calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming = timing2 - timing1
            timing1 = timing2
            write (outfile,701) 'calc. velocities',stepnow,deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=1.d0*(timingw2-timingw1)/(1.d0*timing_rate)
            timingw1 = timingw2
            write (outfile,*) '   wall time=',deltatimingw
        endif
300     continue

cccccccccccccccccccccccccc
c output results and trajectory
c

        dyninp%average(1) = dyninp%average(1) + ener + ekin
        dyninp%average(2) = dyninp%average(2) + ekin
        dyninp%average(3) = dyninp%average(3) + ener
        dyninp%average(4) = dyninp%average(4) + dyninp%tempnow

c
c   WRITE results to file
c
        if (modulo(stepnow, dyninp%outputfreq).eq.0) then
           deltastep = stepnow + dyninp%firsttimestep
c
c    CPU time step calculation
c
           call CPU_TIME(timeend)
           timingtime = 1.d0 * (timeend - timestart)
           timingtime = timingtime / (1.d0 * dyninp%outputfreq)
           timestart = timeend
           call SYSTEM_CLOCK(wtimeend)
           wdeltatiming = (1.d0 * (wtimeend - wtimestart))/
     *                    (1.d0 * timing_rate)
           dyninp%sumwall = dyninp%sumwall + wdeltatiming
           wdeltatiming = wdeltatiming / (1.d0 * dyninp%outputfreq)
           wtimestart = wtimeend

           call dout(box, deltastep, dyninp, ekin, p, geom,
     *          timingtime, wdeltatiming)
c
c    CPU time output calculation
c
            if (ttiming.eq.1) then
                call CPU_TIME(timing2)
                deltatiming = timing2 - timing1
                timing1 = timing2
                write (outfile,701) 'write output and restart',
     *                  stepnow, deltatiming
                call SYSTEM_CLOCK(timingw2)
                deltatimingw = 1.d0 * (timingw2 - timingw1) /
     *                    (1.d0 * timing_rate)
                timingw1 = timingw2
                write (outfile,*) '   Wall time=', deltatimingw
            endif
        endif

c write the trajectory
c
        if (modulo(stepnow, dyninp%trjdumps).eq.0) then
            deltastep = stepnow + dyninp%firsttimestep
            call doutt(deltastep,geom,mmgeom, dyninp%lmmvalen,dyninp)
c
c    CPU time trajectory output
c
            if (ttiming.eq.1) then
                call CPU_TIME(timing2)
                deltatiming = timing2 - timing1
                timing1 = timing2
                write (outfile,701) 'write trajectory', 
     *              stepnow, deltatiming
                call SYSTEM_CLOCK(timingw2)
                deltatimingw = 1.d0 * (timingw2 - timingw1) /
     *                    (1.d0 * timing_rate)
                timingw1 = timingw2
                write (outfile,*) '   Wall time=', deltatimingw
            endif
        endif

      enddo
c
c dynamic step loop end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      write (outfile,501) 'Average wall time per time step [s] =  '
     *               , dyninp%sumwall / (1.d0 * dyninp%steps)
      dyninp%steps = dyninp%steps + 1
      if (dyninp%apmethode.gt.0) then
        write (outfile,501) 'Average number of groups in active zone '
     *               , (1.d0*dyninp%averageactive)/(1.d0*dyninp%steps)
        write (outfile,501) 'Average number of groups in buffer zone '
     *               , (1.d0*dyninp%averagebuffer)/(1.d0*dyninp%steps)
        write (outfile,502) 'Minimum number of groups in active zone '
     *                   , dyninp%minactive
        write (outfile,502) 'Maximum number of groups in active zone '
     *                   , dyninp%maxactive
        write (outfile,502) 'Minimum number of groups in buffer zone '
     *                   , dyninp%minbuffer
        write (outfile,502) 'Maximum number of groups in buffer zone '
     *                   , dyninp%maxbuffer
        if (dyninp%apmethode.eq.2) then
            write (outfile,501) 
     *           'Average number of QM calculation in a time step: ',
     *            (1.d0 * dyninp%avghl) / (1.d0 * dyninp%steps)
        endif
501   format(5x,a,f8.1)
502   format(5x,a,I6)
701   format(5x,'Time to ',a,' in step ',I7,' [s]: 'f8.5)
!702   format(5x,'Run: ',f8.5, ' User: ',f8.5, ' System:'f8.5)
      endif

      close(trajfile)
      if (dyninp%pmfatom+dyninp%pmfgroup.gt.0) then
        close (smdf) 
        close (smddummy) 
      endif

      return
      end


c
c
c
c*****************************************************************************
c  RANDOM
c*****************************************************************************
c
      function random(seed)
c
c     Created:  May 25 2010 Soroosh Pezeshki
c
c     Purpose:  
c
c     Description:  LCG - generate random numbers
c
c
c     Called by:  gauss
c     Calls:  
c
      implicit none

       integer*8  seed, m, c, a
       double precision random
c
c generates random numbers; independent of the system

       c   =    12345
       a   =    1103515245
       m   =    2147483648

       seed   = mod((a * seed + c), m)
       random = 1.d0 * seed / (1.d0 * m)

        return
      end


c*****************************************************************************
c  GAUSSRANDOM
c*****************************************************************************
c
      function gaussrandom(seed)
c
c     Created:  May 25 2010 Soroosh Pezeshki
c
c     Purpose:  
c
c     Description:  generate random NORMAL DISTRIBIUTION
c       see A.R. Leach: Molecular Modelling
c
c     Called by:  initmd
c     Calls:  random
c
      implicit none

      double precision a,  gaussrandom, random
      integer*8 i, seed

      a = 0.d0

      do i = 1, 12
        a = a + random(seed)
      enddo

      gaussrandom = a - 6.d0
      return
      end


c
c
c
c*****************************************************************************
c  BOXSIZE
c*****************************************************************************
c
      subroutine boxsize(geom, boxs)
c
c     Created:  June 02 2010 Soroosh Pezeshki
c
c     Purpose:  
c
c     Description:  Set the minimum and maximum of the box.
c
c
c     Called by:  mdrun, initmd
c     Calls:  
c
      use input, only : cartgeom
      implicit none

      type (cartgeom) :: geom
      double precision xi, yi, zi
      double precision bxmin, bxmax, bymin, bymax, bzmin, bzmax
      double precision boxs(9)
      integer i
c
c set the values equal the coordinates of atom 1
c
      bxmin = geom%angcoord(1)
      bxmax = geom%angcoord(1)
      bymin = geom%angcoord(2)
      bymax = geom%angcoord(2)
      bzmin = geom%angcoord(3)
      bzmax = geom%angcoord(3)
c
c search for bigger values
c
      do i = 2, geom%natoms
        xi = geom%angcoord(i*3-2)
        yi = geom%angcoord(i*3-1)
        zi = geom%angcoord(i*3)
        if (xi .lt. bxmin) bxmin = xi
        if (xi .gt. bxmax) bxmax = xi
        if (yi .lt. bymin) bymin = yi
        if (yi .gt. bymax) bymax = yi
        if (zi .lt. bzmin) bzmin = zi
        if (zi .gt. bzmax) bzmax = zi
      enddo
      boxs(1) = bxmin
      boxs(4) = bxmax
      boxs(2) = bymin
      boxs(5) = bymax
      boxs(3) = bzmin
      boxs(6) = bzmax
c
c calculate the length
c
      boxs(7) = dabs(bxmax - bxmin)
      boxs(8) = dabs(bymax - bymin)
      boxs(9) = dabs(bzmax - bzmin)

      return
      end



c
c
c*****************************************************************************
c  T41INPXYZ
c*****************************************************************************
c
      subroutine t41inpxyz(ixyz,geom,mmgeom)
c
c     Subroutine t41inpxyz
c     
c     Created:  Jun. 11, 2010 by Soroosh Pezeshki
c     Based on  t41inp
c
c     Purpose:  Write a TINKER xyz file for the calculation
c                                                                       
c 
c     Called by:  progmmeghkmd, progmme0ghkmd
c     Calls:
c
c     Variables defined here:
c
      use files
      use input, only : cartgeom, mmvalen
      implicit none
c      
c i/o variables 
c

      integer ixyz
      
      type (cartgeom) :: geom   
      type (mmvalen)  :: mmgeom   
      
c
c local variables
c
       
      integer          iiatom, iivalen, iid, iityp, i, j
      double precision mmx, mmy, mmz

c
c  Write the .xyz file
c
      iiatom = geom%natoms
      write(ixyz,*) iiatom
c      
      do i = 1, iiatom
         iid = mmgeom%atmid(i)
         iityp = mmgeom%atmtyp(i)         
         iivalen = mmgeom%atmvalen(i)
         mmx = geom%angcoord(3*i-2)
         mmy = geom%angcoord(3*i-1)
         mmz = geom%angcoord(3*i)

c
c write the atom valence
c
c modified by Xin-Ping Wu, Sept. 21, 2017
c
         if(iivalen .le. 8) then
c         if(iivalen .le. 6) then
            write(ixyz,2000) iid,mmgeom%atmtypname(i), mmx,mmy,mmz,
     *        iityp, (mmgeom%atmvalenlist(i,j),j=1,iivalen)     
         endif
      enddo            
c
c end by Xin-Ping Wu, Sept. 21, 2017
c
                    
      return

c
c modified by Xin-Ping Wu, Sept. 21, 2017
c
2000  format(i4,1x,a2,1x,3(f10.5,1x),i4,1x,8(i4,1x))                    
c 2000  format(i6,2x,a4,3(f11.6,1x),i5,1x,6(i5,1x))
c
c end by Xin-Ping Wu, Sept. 21, 2017
c

      end


c
c
c*****************************************************************************
c  READRESTART
c*****************************************************************************
c
      subroutine readrestart(geom,dyninp)
c
c     Subroutine readrestart
c     
c     Created:  Jun. 14, 2010 by Soroosh Pezeshki
c
c     Purpose:  read the atomic coordinates, velocities 
c               and forces from restart file for restarting simulations.
c                                                                       
c 
c     Called by:  initmd
c     Calls: readln,
c
c     Variables defined here:
c
      use input, only:  cartgeom, dynamicsinf
      use files, only:  outfile, restartin
      implicit none


c
c  i/o variables
c
      type (dynamicsinf) :: dyninp
      type (cartgeom) :: geom

c
c  local variables
c
      double precision cfloat
      integer natms, i,j, nrestart, icint
c
      character*80  words(40)      
      integer ieof, isec, nwords, iscend

      natms=geom%natoms
      if (dyninp%selcent.eq.2) natms=natms+1


c        open (unit=restartfile, file=restartfilename, 
c     *      status='old', form='formatted', err=5000)
c
c     read the first section of the input file
c
        call readln(restartin,words,nwords,isec,ieof,iscend) 
c
c     check that the input file was not empty
c
        if (ieof .eq. 1) then
            write (outfile,1000)
            stop 'initmd 2'
        endif
c
c read the firsttimestep
c
        dyninp%firsttimestep = icint(words(1))
        write (outfile,1001) dyninp%firsttimestep

c check the number of atoms
c
        nrestart = icint(words(2))
        if (nrestart.ne.natms) then
            write (outfile,1002)
            stop 'initmd 3'
        endif

c read the positions
c
        do i = 1, natms
            call readln(restartin,words,nwords,isec,ieof,iscend)
            if (ieof .eq. 1) then
                write (outfile,1003)
                stop 'initmd 4'
            endif
            do j = -2, 0
                geom%angcoord(3*i+j) = cfloat(words(3+j))
            enddo
        enddo

c read the velocities
c
        do i = 1, natms
            call readln(restartin,words,nwords,isec,ieof,iscend) 
            if (ieof .eq. 1) then
                write (outfile,1003)
                stop 'initmd 5'
            endif
            do j = -2, 0
                dyninp%velocity(3*i+j) = cfloat(words(3+j))
            enddo
        enddo

c read the forces
c
        do i = 1, natms
            call readln(restartin,words,nwords,isec,ieof,iscend) 
            if (ieof .eq. 1) then
                write (outfile,1003)
                stop 'initmd 6'
            endif
            do j = -2, 0
                dyninp%force(3*i+j) = cfloat(words(3+j))
            enddo
        enddo

        call readln(restartin,words,nwords,isec,ieof,iscend) 
        if(ieof.eq.1) goto 2000
c
c read the average values
c
        call readln(restartin,words,nwords,isec,ieof,iscend) 
        if(ieof.eq.1) goto 2000
        dyninp%average(1) = cfloat(words(1))
        dyninp%average(2) = cfloat(words(2))
        call readln(restartin,words,nwords,isec,ieof,iscend) 
        dyninp%average(3) = cfloat(words(1))
        dyninp%average(4) = cfloat(words(2))


2000    close (restartin)


      return
5000  write (*,'(/a)') 'ERROR: Cannot open the restart file.'
      stop 'initmd 1'
1000   format (3x, 'Error:  restart file was empty')
1001   format (/,5x, 'First time step set to:',t50, I8)
1002   format ('ERROR: Number of atoms in restart file is not correct')
1003   format ('Error: insufficient data in restart file')

      end
c
c
c*****************************************************************************
c  WRITERESTART
c*****************************************************************************
c
      subroutine writerestart(geom,dyninp,stepnow)
c
c     Subroutine writerestart
c     
c     Created:  Jun. 14, 2010 by Soroosh Pezeshki
c
c     Purpose:  write the dynamics results to restartfile
c                                                                       
c 
c     Called by:  dout
c     Calls: 
c
c     Variables defined here:
c
      use input, only:  cartgeom, dynamicsinf
      use files, only: restartfile, restartfilename
      implicit none


c
c  i/o variables
c
      type (dynamicsinf) :: dyninp
      type (cartgeom) :: geom
      integer            stepnow
c
c  local variables
c
      integer n3tm


      open (unit=restartfile, file=restartfilename, 
     *      status='unknown', form='formatted', err=5000)


c
c write the step number and the number of atoms (+dummy atom)
c
      n3tm = 3 * geom%natoms
      if (dyninp%selcent.eq.2) n3tm = n3tm + 3
      write(restartfile, 1000) stepnow, geom%natoms
c
c write the coordinate, velocity and force of each atom
c
      write(restartfile,2055) geom%angcoord(:n3tm)
      write(restartfile,2056) dyninp%velocity(:n3tm)
      write(restartfile,2057) dyninp%force(:n3tm)
c
c write the book keeping energy of the DAS algorithm and close file
c
!       write(restartfile,*) dyninp%wbk
      write(restartfile,*) 0.0
      write(restartfile,2058) dyninp%average(1:2)
      write(restartfile,2058) dyninp%average(3:4)
c
c write position smd dummy
c
      if (dyninp%pmfgroup+dyninp%pmfatom.gt.0) then
        write(restartfile,2055) dyninp%pmfi(1:3)
      endif
      close (restartfile)



1000  format(4x, I8, t20, I8)
2055  format(f9.4, t11, f9.4, t21, f9.4)
2056  format(d19.9, t25, d19.9, t50, d19.9)
2057  format(d19.9, t21, d19.9, t42, d19.9)
2058  format(d19.9, t21, d19.9)
      return
5000  write (*,'(/a)') 'ERROR: Cannot open the restart file.'
      stop 'writerestart 1'
      end

c
c
c*****************************************************************************
c  DOUT
c*****************************************************************************
c
      subroutine dout(box, stepnow, dyninp, ekin,p, geom, 
     *         timingtime, wdeltatiming)
c
c     Subroutine dout
c     
c     Created:  Jun. 14, 2010 by Soroosh Pezeshki
c
c     Purpose:  write the dynamics results to outfile and to trajfile
c                                                                       
c 
c     Called by:  runmd
c     Calls: boxsize, writerestart
c
c     Variables defined here:
c
      use input, only : dynamicsinf, cartgeom, energyconvert,
     *          pressureconvert, lengthconvert, tempconvert, 
     *          forceconvert2, forceconvert
      use optimize, only : ener
      use gradient, only : totalgrad
      use files, only: outfile, smdf
      use caltyp, only: tdebug
      implicit none


c
c  i/o variables
c
      type (dynamicsinf) :: dyninp
      type (cartgeom) :: geom
      integer            ierr, stepnow, i,j, vstat(100)
      double precision p(3), ekin, v, timingtime, wdeltatiming
      double precision pressure, vmax, ppN
c      double precision paverage(3), eaverage, paverage2(3), eaverage2
c      double precision energyfluct, pfluctuate(3)

c
c  local variables
c
      integer n3tm, deltasteps, deltas
      double precision box(9), volume, time, rwall, efactor, peratom

       deltas = stepnow - dyninp%firsttimestep
c       deltas=stepnow
       n3tm = 3 * geom%natoms
       peratom = (1.d0 * geom%natoms)
       efactor = energyconvert / peratom

       call boxsize(geom, box)
c!        eaverage=eaverage/dyninp%outputfreq
c!        paverage(:3)=paverage(:3)/dyninp%outputfreq
c!        eaverage2=eaverage2/dyninp%outputfreq
c!        paverage2(:3)=paverage2(:3)/dyninp%outputfreq
c!        energyfluct=eaverage2-eaverage**2
c!        pfluctuate(:3)=paverage2(:3)-paverage(:3)**2
c!        if (energyfluct.gt.0.d0) energyfluct=sqrt(energyfluct)
c!        if (pfluctuate(1).gt.0.d0) pfluctuate(1)=sqrt(pfluctuate(1))
c!        if (pfluctuate(2).gt.0.d0) pfluctuate(2)=sqrt(pfluctuate(2))
c!        if (pfluctuate(3).gt.0.d0) pfluctuate(3)=sqrt(pfluctuate(3))

c write to output file
c
       write(outfile,2005) 
       write(outfile,*) "   Summary for "

       volume = box(7) * box(8) * box(9)
       time = stepnow * dyninp%timestepl
       write(outfile,2001) "Timestep=", stepnow, "Time=" , 
     *            time, " fs"
       write (outfile,2003) 'CPU time per step [s]=', timingtime 
       write (outfile,2003) 'Wall time per step [s]=', wdeltatiming
       write (outfile,*) '     CPU time does not incl. ',
     *          ' external programs'
c
c rest wall
c
       deltasteps = dyninp%steps - deltas
       rwall = (dyninp%sumwall * deltasteps) / (1.d0*deltas)
       if (rwall .gt. 100000.d0) then
          rwall = rwall / 86400.d0
          write(outfile,2007) deltasteps, '[d]=', rwall
       elseif (rwall .gt. 4000.d0) then
          rwall = rwall / 3600.d0
          write(outfile,2007) deltasteps, '[h]=', rwall
       elseif (rwall .gt. 100.d0) then
          rwall = rwall / 60.d0
          write(outfile,2007) deltasteps, '[m]=', rwall
       else
          write(outfile,2007) deltasteps, '[s]=', rwall
       endif

       write(outfile,*) "  -Results for This Time Step-"
       write(outfile,2003) "Temperature [K]=", dyninp%tempnow
       write(outfile,2002) "Volume [Angstrom^3]=", volume

       write(outfile,2004) "Momentum [uAng./fs]=", p(:3)
       write(outfile,2002) "Kinetic Energy [hartree]=", ekin
       write(outfile,2002) "Potential Energy [hartree]=", ener
       write(outfile,2002) "Total Energy [hartree]=", ener + ekin

       write(outfile,2002) "Kinetic Energy [kcal/mol]=",
     *          ekin * energyconvert
       write(outfile,2002) "Potential Energy [kcal/mol]=", 
     *          ener * energyconvert
       write(outfile,2002) "Total Energy [kcal/mol]=",
     *          (ener + ekin) * energyconvert

       write(outfile,*) "  -Average Over Number of Atoms-"
       write(outfile,2002) "Total Energy [hartree]=",
     *          (ener + ekin) / peratom
       write(outfile,2002) "Total Energy [kcal/mol]=",
     *          (ener + ekin) * efactor
       ppN = dsqrt(p(1) * p(1) + p(2) * p(2) + p(3) * p(3)) / peratom
       write(outfile,2002) "Momentum [uAng./fs]=", ppN


       write(outfile,*) "  -Time Average Energy Per Atom-"
       write(outfile,2002) "Kinetic Energy [kcal/mol]=",
c     *      dyninp%average(2)/deltas*efactor
     *      dyninp%average(2) / stepnow * efactor
       write(outfile,2002) "Potential Energy [kcal/mol]=", 
c     *      dyninp%average(3)/deltas*efactor
     *      dyninp%average(3) / stepnow * efactor
       write(outfile,2002) "Total Energy [kcal/mol]=", 
c     *      dyninp%average(1)/deltas*efactor
     *      dyninp%average(1) / stepnow * efactor

       write(outfile,2002) "Average Temperature [K]=", 
     *      dyninp%average(4) / deltas
       if (dyninp%apmethode .gt. 0) then
          write(outfile,*) "  -Infinit seperated Energies-"
          write(outfile,2002) "Energy [hartree]=", 
     *          dyninp%zeroenergy
          write(outfile,2002) "Energy + E_pot [hartree]=", 
     *          dyninp%zeroenergy + ener
          write(outfile,2002) "Energy [kcal/mol]=", 
     *          dyninp%zeroenergy * energyconvert
          write(outfile,2002) "Energy + E_pot [kcal/mol]=",
     *          (dyninp%zeroenergy + ener) * energyconvert
       endif


c write additional information if debug is set
c
       if ((tdebug .eq. 1) .or. (tdebug .eq. 2)) then
           write(outfile,*) 
     *      "DebugDebugDebugDebugDebugDebugDebugDebugDebugDebug"
           pressure = geom%natoms*dyninp%tempnow/tempconvert/volume
           pressure = pressure * pressureconvert * lengthconvert**3
           write(outfile,2002) "ideal gas pressure [atm]=",pressure
           write(outfile,*) "Gradient [Hartree/Bohr radius]:"
           write(outfile,2055) totalgrad(:n3tm)
           write(outfile,*) "Force [amu Angstrom/fs^2]:"
           write(outfile,2055) dyninp%force(:n3tm)
           write(outfile,*) "Velocities [Angstrom/fs]"
           write(outfile,2055) dyninp%velocity(:n3tm)
           write(outfile,*) 
     *      "DebugDebugDebugDebugDebugDebugDebugDebugDebugDebug"
       endif

c
c debug: Velocity statistics
c
        if ((tdebug .eq. 4) .or. (tdebug .eq. 1)) then
            vmax = dabs(dyninp%velocity(1))
            vstat(:) = 0
            do i = 2, n3tm
                if (dabs(dyninp%velocity(i)) .gt. vmax) 
     *              vmax = dabs(dyninp%velocity(i))
            enddo
            do i = 1, n3tm
                j = ceiling(100.0*dabs(dyninp%velocity(i))/vmax)
                vstat(j) = vstat(j) + 1
            enddo
            write(outfile,*) 
     *      "DebugDebugDebugDebugDebugDebugDebugDebugDebugDebug"
            write(outfile,*) "Velocity statistics", vmax
            do i = 1, 100
                write(outfile,*) i * vmax / 100.d0, vstat(i)
            enddo
             write(outfile,*) 
     *      "DebugDebugDebugDebugDebugDebugDebugDebugDebugDebug"
!              v=dyninp%velocity(36)**2+dyninp%velocity(37)**2+
!      *              dyninp%velocity(38)*dyninp%velocity(38)
!              write(*,*) "v_12=", dsqrt(v)
!              v=dyninp%velocity(39)**2+dyninp%velocity(40)**2+
!      *              dyninp%velocity(41)*dyninp%velocity(41)
!              write(*,*) "v_13=", dsqrt(v)
!              v=dyninp%force(36)**2+dyninp%force(37)**2+
!      *              dyninp%force(38)*dyninp%force(38)
!              write(*,*) "f_12=", dsqrt(v)
!              v=dyninp%force(39)**2+dyninp%force(40)**2+
!      *              dyninp%force(41)*dyninp%force(41)
!              write(*,*) "f_13=", dsqrt(v)
       endif
       if (dyninp%forceOutAtom.gt.0) then
           write(outfile,203) "F [nN]: ", 
     *         dyninp%force(dyninp%forceOutAtom-2:dyninp%forceOutAtom)
     *           /forceconvert2
           write(outfile,203) "dES [nN]: ", 
     *         dyninp%gradforce(1:3)*forceconvert/forceconvert2
           write(outfile,203) "dSE [nN]: ", 
     *         dyninp%smoothforce(1:3)*forceconvert/forceconvert2
       endif

       write(outfile,2005) 
c write restart file
c
       call writerestart(geom,dyninp,stepnow)
       flush(outfile)
       if(dyninp%pmfatom+dyninp%pmfgroup.gt.0) then
            flush(smdf)
       endif

c!        eaverage=0.d0
c!        eaverage2=0.d0
c!        paverage(:)=0.d0
c!        paverage2(:)=0.d0

203   format(a, e12.4, t40, e12.4,t60, e12.4) 

2001  format(a10,I10,t40,a5,f15.2, a3)
2002  format(4x,a30,t50,e15.8)
2003  format(4x,a30,t50,f15.8)
2004  format(4x,a20,t30,d13.5,t45,d13.5,t60,d13.5)
2005  format(80('.'),/)
2007  format(4x,'Approx. time needed for ',i9,' steps ',a,t52,f6.1)
2055  format(t20,f15.8,t40,f15.8,t60,f15.8)


      end

c
c
c*****************************************************************************
c  DOUT0
c*****************************************************************************
c
      subroutine dout0 (geom, mmgeom, natoms, lmmvalen, dyninp)
c
c     Subroutine dout0
c     
c     Created:  Jun. 14, 2010 by Soroosh Pezeshki
c
c     Purpose:  write the dynamics results to outfile and to trajfile
c                                                                       
c 
c     Called by:  initmd
c     Calls:
c
c     Variables defined here:
c
      use input, only: cartgeom, mmvalen, dynamicsinf
      use files
      use caltyp, only: tdebug
      implicit none

c
c  i/o variables
c
      type (dynamicsinf) :: dyninp
      type (cartgeom) :: geom
      type (mmvalen) ::  mmgeom
      integer            ierr, stepnow, lmmvalen
c
c  local variables
c
      integer i, i3, j, natoms, frominput
      integer iid, iityp, iivalen
      double precision mmx, mmy, mmz, time

c   Write the initial coordinates to the trajectory file
c
      if (((tdebug.eq.3) .or. (tdebug.eq.1) .or. (tdebug.eq.5)) .and.
     *      dyninp%selcent.eq.2) then 
        write(trajfile,*) natoms + 1
      else
        write(trajfile,*) natoms
      endif
      do i = 1, natoms
         iityp = mmgeom%atmtyp(i) 
         iivalen = mmgeom%atmvalen(i)
         mmx = geom%angcoord(3*i-2)
         mmy = geom%angcoord(3*i-1)
         mmz = geom%angcoord(3*i)
         if (lmmvalen .eq. 1) then
c
c modified by Xin-Ping Wu, Sept. 21, 2017
c
            if(iivalen .le. 8) then
c            if(iivalen .le. 6) then
c end by Xin-Ping Wu, Sept. 21, 2017
c
                write(trajfile,1213) i,mmgeom%atmtypname(i), mmx,mmy,
     *              mmz,iityp, (mmgeom%atmvalenlist(i,j),j=1,iivalen)
            endif
         else
            write(trajfile,1214) i, geom%atm(i), mmx, mmy, mmz
         endif
      enddo
c
c debug: Dummy Atom output
c
       if ((tdebug.eq.3) .or. (tdebug.eq.1) .or. (tdebug.eq.5)) then
            if(dyninp%selcent .eq. 2) then
                i = natoms + 1
                i3 = i * 3
                write(trajfile,1214) i, 'Dumm',
     *              geom%angcoord(i3-2), geom%angcoord(i3-1), 
     *              geom%angcoord(i3)
            endif
        endif

c
c modified by Xin-Ping Wu, Sept. 21, 2017
c
c1213  format(i6,2x,a4,3(f11.6,1x),i5,1x,6(i5,1x))
c1214  format(i6,2x,a4,3(f11.6,1x))
1213  format(i4,1x,a2,1x,3(f10.5,1x),i4,1x,8(i4,1x))
1214  format(i4,1x,a2,1x,3(f10.5,1x))                                   
c
c end by Xin-Ping Wu, Sept. 21, 2017
c
      write(outfile,1030)
1030  format(//,80('-'),/80('-')//3x,
     *   'Dynamics Calculation Requested by User:',//80('-')/)

      end


c
c
c*****************************************************************************
c  DOUTT
c*****************************************************************************
c
      subroutine doutt (stepnow, geom, mmgeom, lmmvalen, dyninp)
c
c     Subroutine doutt
c     
c     Created:  Jun. 14, 2010 by Soroosh Pezeshki
c
c     Purpose:  write the trajectory to trajfile
c                                                                       
c 
c     Called by:  runmd
c     Calls:
c
c     Variables defined here:
c
      use input, only: cartgeom, mmvalen, dynamicsinf
      use files, only: trajfile, smddummy
      use caltyp, only: tdebug                                               
      implicit none


c
c  i/o variables
c
      type (cartgeom) :: geom
      type (mmvalen) ::  mmgeom
      type (dynamicsinf) :: dyninp 
      integer            ierr, stepnow, lmmvalen
c
c  local variables
c
      integer i, i3
      double  precision smdr(3)


c write trajectory
c
        write(trajfile,*) stepnow
      if (lmmvalen.eq.1) then
        do i = 1, geom%natoms
            i3 = i * 3
            write(trajfile,1212) mmgeom%atmid(i), mmgeom%atmtypname(i),
     *        geom%angcoord(i3-2), geom%angcoord(i3-1), 
     *          geom%angcoord(i3)
        enddo
      else
        do i = 1, geom%natoms
             i3 = i * 3
             write(trajfile,1212) i, geom%atm(i),
     *        geom%angcoord(i3-2), geom%angcoord(i3-1), 
     *          geom%angcoord(i3)
        enddo
      endif
c
c Dummy atom
c
      if ((tdebug.eq.3) .or. (tdebug.eq.1) .or. (tdebug.eq.5)) then
            if(dyninp%selcent .eq. 2) then
                i = geom%natoms + 1
                i3 = i * 3
                write(trajfile,1212) i, 'Dumm',
     *              geom%angcoord(i3-2), geom%angcoord(i3-1), 
     *              geom%angcoord(i3)
                write(737,1213) dyninp%velocity(i3-2), 
     *              dyninp%velocity(i3-1), dyninp%velocity(i3)
                write(747,1213) dyninp%force(i3-2), 
     *              dyninp%force(i3-1), dyninp%force(i3)
            endif
      endif
      flush(trajfile)

      if (dyninp%pmfgroup+dyninp%pmfatom.gt.0) then
        write(smddummy,*) stepnow
        write(smddummy,1212) 1, 'Du', dyninp%pmfi(1:3)
        flush(smddummy)
      endif
c
c debug: Velocity and gradient output
c
       if ((tdebug.eq.3) .or. (tdebug.eq.1)) then
            write(737,*) stepnow
            write(747,*) stepnow
            do i = 1, geom%natoms
                i3 = i * 3
                write(737,1213) dyninp%velocity(i3-2), 
     *              dyninp%velocity(i3-1), dyninp%velocity(i3)
                write(747,1213) dyninp%force(i3-2), 
     *              dyninp%force(i3-1), dyninp%force(i3)
            enddo
            if(dyninp%selcent .eq. 2) then
                i = geom%natoms + 1
                i3 = i * 3
                write(737,1213) dyninp%velocity(i3-2), 
     *              dyninp%velocity(i3-1), dyninp%velocity(i3)
                write(747,1213) dyninp%force(i3-2), 
     *              dyninp%force(i3-1), dyninp%force(i3)
            endif
            flush(737)
            flush(747)
       endif

c1212  format(i6,2x,a4,3(f11.6,1x))
1212  format(i4,1x,a2,1x,3(f10.5,1x))                                   0921XW17
1213  format(2x,3(D20.8,2x))

      end


c
c
c*****************************************************************************
c  FINDMOLECULES
c*****************************************************************************
c
      subroutine findmolecules (natoms, dyninp, mmgeom, geom, qmmminp)
c
c     Subroutine findmolecules
c     
c     Created:  Jul. 19, 2011 by Soroosh Pezeshki
c
c     Purpose:  finds all molecules and set the flags in molecule
c                                                                       
c 
c     Called by:  initmd
c     Calls:  calccom
c
c     Variables defined here:
c
      use input, only: cartgeom,dynamicsinf, mmvalen, qmmminf
      use caltyp, only: tdebug                                               
      use files, only : outfile
      implicit none
       type (dynamicsinf) :: dyninp
       type (mmvalen)  :: mmgeom
       type (cartgeom) :: geom
       type (qmmminf)  :: qmmminp
c
c  local variables
c
      integer i, j, k, m, natoms, molid, valence, segcomp, idnow

c set the molid of atom 1 and all direct bondatoms to 1
c
      molid = 2
      dyninp%molecule(:) = 0
      dyninp%molecule(1) = 1  
      do i = 1, mmgeom%atmvalen(1)
        valence = mmgeom%atmvalenlist(1,i)
        dyninp%molecule(valence) = 1
      enddo
c
c loop over all atoms beside atom 1
c
      do i = 2, natoms
        idnow = dyninp%molecule(i)
c
c if no molid is set set it to molid and than incr. molid
c
        if (idnow .eq. 0) then
            idnow = molid
        endif
c
c set all direct bonds with the same molid if the molid is bigger or 0
c else set the molid of atom i
c loop over all direct bonds
c
        do j = 1, mmgeom%atmvalen(i)
            valence = mmgeom%atmvalenlist(i,j)
            segcomp = dyninp%molecule(valence)
c
c if the molid of a direct bond is less than the molid of atom i
c set all atom with molid to segcomp
c
            if ((segcomp .ne. 0) .and. (segcomp .lt. idnow)) then
                do k = 2, i
                    if (dyninp%molecule(k) .eq. idnow) then
                        dyninp%molecule(k) = segcomp
                    endif
                enddo
                idnow = segcomp
c
c else if the molid of the direct bonded atom is greater set to molid
c
            elseif (segcomp .gt. idnow) then
                do k = 2, i
                    if (dyninp%molecule(k) .eq. segcomp) then
                        dyninp%molecule(k) = idnow
                    endif
                enddo
c
c if the direct bonded atom has the molid 0 set it to the same as of atom i
c
            elseif (segcomp .eq. 0) then
                dyninp%molecule(valence) = idnow
            endif
        enddo
c
c incr molid if used
c
        if (idnow .eq. molid) molid = molid + 1
        dyninp%molecule(i) = idnow
      enddo
      dyninp%maxmolecule = molid - 1


      if ((tdebug .eq. 1) .or. (tdebug .eq. 2)) 
     *      write(outfile,2002) 'Atom', 'Molecule'

c sum up the mass of the molecules and calculate the center of mass
c
      do i = 1, natoms
          molid = dyninp%molecule(i)
          dyninp%comsm(molid) = dyninp%comsm(molid) + geom%atmmss(i)
          dyninp%commass = dyninp%commass + geom%atmmss(i)
c
c write the molid of each atom to outfile if debug option is set
c
          if ((tdebug .eq. 1) .or. (tdebug .eq. 2)) then
            write(outfile,2001) i, molid
          endif
      enddo
      if (dyninp%selcent .eq. 2) then
          dyninp%commass = dyninp%commass + geom%atmmss(natoms+1)
      endif

      call calccom (natoms, dyninp, geom)
2001  format(I10,t20,I8)
2002  format(a4,t20,a8)

      end

c
c
c*****************************************************************************
c  CALCCOM
c*****************************************************************************
c
      subroutine calccom (natoms, dyninp, geom)
c
c     Subroutine calccom
c     
c     Created:  Jun. 14, 2010 by Soroosh Pezeshki
c
c     Purpose:  calculates the center of mass
c                                                                       
c 
c     Called by:  findmolecules, runmd
c     Calls:
c
c     Variables defined here:
c
      use input, only: cartgeom, dynamicsinf, maxatm 
      implicit none
       type (dynamicsinf) :: dyninp
       type (cartgeom) :: geom
      integer natoms

c
c  local variables
c
      integer i, j, m
      double precision comsrm(3 * maxatm)

c reset the variable
c
        comsrm(:) = 0.d0

c sum up the com for each coordinate
c
        do i = 1, natoms
            m = dyninp%molecule(i)
            comsrm (3 * m - 2) = comsrm(3 * m - 2) +
     *                    geom%atmmss(i) * geom%angcoord(3 * i - 2)
            comsrm(3 * m - 1) = comsrm(3 * m - 1) +
     *                    geom%atmmss(i) * geom%angcoord(3 * i - 1)
            comsrm(3 * m) = comsrm(3 * m) +
     *                    geom%atmmss(i) * geom%angcoord(3 * i)
        enddo

c divide the sum through the mass of the molecule
c
        do m = 1, dyninp%maxmolecule
            if (dyninp%comsm(m) .ne. 0.d0) then
                dyninp%com(3*m-2) = comsrm(3*m-2) / dyninp%comsm(m)
                dyninp%com(3*m-1) = comsrm(3*m-1) / dyninp%comsm(m)
                dyninp%com(3*m)   = comsrm(3*m)   / dyninp%comsm(m)
            endif
        enddo
      end


c
c
c*****************************************************************************
c  MOLOUTBOX
c*****************************************************************************
c
      subroutine moloutbox (geom, dyninp)
c
c     Subroutine moloutbox
c     
c     Created:  Jun. 14, 2010 by Soroosh Pezeshki
c
c     Purpose:  search for molecules (COM) that left the periodic box
c                                                                       
c 
c     Called by:  runmd
c     Calls: shiftmol
c
c     Variables defined here:
      use input, only: cartgeom, dynamicsinf
      use caltyp, only: tdebug                                               
      implicit none
       type (dynamicsinf) :: dyninp
       type (cartgeom) :: geom

c
c  local variables
c
c
      integer j, j3, m
      double precision  bsize, bmin, bmax, com

c
c Output for debugging
c
        if ((tdebug .eq. 1) .or. (tdebug .eq. 2)) then
            write(*,*) 'Moving molecules in periodic box'
            write(*,*) 'Axes, Molecule, box length, direction,' 
     *                    ,'coordinate, limit' 
        endif

        do m = 1, dyninp%maxmolecule
            do j = -2, 0
                j3 = 3 + j
                bsize = dyninp%boxsize(j3)
                bmin = dyninp%boxmin(j3)
                bmax = dyninp%boxmax(j3)
                j3 = m * 3 + j
                com = dyninp%com(j3)
                if ((com .lt. bmin) .and. (com .ne. 0.d0)) then
                    if ((tdebug.eq.1)  .or. (tdebug .eq. 2))
     *                  write(*,*) 3+j,m,bsize,1,com,bmin
                    call shiftmol (geom, dyninp, j, m, bsize, 1)
                elseif ((com .gt. bmax) .and. (com .ne. 0.d0)) then
                    if ((tdebug .eq. 1) .or. (tdebug .eq. 2))
     *                   write(*,*) 3 + j, m, bsize, -1,
     *                                            com, bmax
                    call shiftmol (geom, dyninp, j, m, bsize, -1)
                endif
            enddo
        enddo

      end

c
c
c*****************************************************************************
c  SHIFTMOL
c*****************************************************************************
c
      subroutine shiftmol (geom, dyninp, j, mol, bsize, direct)
c
c     Subroutine shiftmol
c     
c     Created:  Jun. 14, 2010 by Soroosh Pezeshki
c
c     Purpose:  moves a molecule that is out of the periodic box
c                                                                       
c 
c     Called by:  moloutbox
c     Calls:
c
c     Variables defined here:
c
      use input, only: cartgeom, dynamicsinf
      implicit none
       type (dynamicsinf) :: dyninp
       type (cartgeom) :: geom
      integer direct
      integer j, mol
      double precision  bsize

c
c  local variables
c
      integer i, i3

      do i = 1, geom%natoms
c
c move only atoms os the molecule
c
        if (dyninp%molecule(i) .eq. mol) then
            i3 = 3 * i + j
            geom%angcoord(i3) = geom%angcoord(i3) + direct * bsize
        endif
      enddo
      end


c
c
c*****************************************************************************
c  TEMPCONTROL
c*****************************************************************************
c
      subroutine tempcontrol (dyninp, ekin, n3tm, p)
c
c     Subroutine tempcontrol
c     
c     Created:  Jun. 16, 2010 by Soroosh Pezeshki
c
c     Purpose:  Berendsen temperature control
c                                                                       
c 
c     Called by:  runmd
c     Calls:
c
c     Variables defined here:
c
      use input, only: dynamicsinf
      use caltyp, only: tdebug 
      implicit none
       type (dynamicsinf) :: dyninp
      double precision  ekin, p(3)
c
c  local variables
c
      double precision  lambda
      integer n3tm
c
c avoid 0 division
c
      if (dyninp%tempnow .eq. 0.d0) dyninp%tempnow = 1.d0 

c calculate the scaling factor
c
      lambda = dsqrt(dabs(1.d0 + dyninp%tempcoupling *
     *        ((dyninp%temperature / dyninp%tempnow) - 1.d0)))

c apply scaling on velocity, momentum and kinetic energy 
c
      dyninp%velocity(:n3tm) = dyninp%velocity(:n3tm) * lambda
      p(:3) = p(:3) * lambda
      ekin = ekin * lambda * lambda
      dyninp%tempnow = dyninp%tempnow * lambda * lambda


c write out if debug
c
      if (tdebug .gt. 0) then
        write(*,*) 'Berendsen temperature control factor: ', lambda
      endif

      end


c
c
c*****************************************************************************
c  TEMPANNEALING
c*****************************************************************************
c
      subroutine tempannealing (dyninp, coolstep, equistep)
c     
c     Created:  Jun. 22, 2010 by Soroosh Pezeshki
c
c     Purpose:  set the temperature in annealing simulations
c                                                                       
c 
c     Called by:  runmd
c     Calls:
c
c     Variables defined here:
c
      use input, only: dynamicsinf
      implicit none
      type(dynamicsinf) :: dyninp
      integer coolstep, equistep
c
c  local variables
c

c if the the number of equilibration step is not yet fullfieled do noting
c
      if (equistep .lt. dyninp%equistep) then
        equistep = equistep + 1
      else
c
c during cooling change the bath temperature
c
        coolstep = coolstep + 1
        dyninp%temperature = dyninp%temperature + dyninp%deltatemp
        if (coolstep .eq. dyninp%coolstep) then
            coolstep = 0
            equistep = 0
        endif
      endif

      end


c
c
c*****************************************************************************
c  RATTLEVELOCITY
c*****************************************************************************
c
      subroutine rattlevelocity (geom, mmgeom, dyninp)
c
c     Subroutine rattlevelocity
c
c     Created:  Jul. 15, 2010 by Soroosh Pezeshki
c
c     Purpose:  Apply the rattle algorithm on velocities
c     Description:  
c
c
c     Called by:  runmd
c
c     Calls:  
c
c     Variables defined here:
c
      use caltyp, only : tdebug
      use files, only : outfile
      use input, only : cartgeom, mmvalen, dynamicsinf, maxatm
      implicit none
c
c  i/o variables
c

      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (dynamicsinf)  :: dyninp
c
c  local variables
c      
      integer i, ii, i3, l, j, j3, m, n, natoms, redo
      double precision eps, k, rij(3), vij(3), rv

c inital values
c
      natoms = geom%natoms
c      dt=dyninp%timestepl
      eps = dyninp%rattleeps
      redo = 1
      m = 0
      rij(:) = 0.d0
      vij(:) = 0.d0

c loop as long as atoms are moved but not more than maxiteriation
c
      do while (redo.eq.1 .and. m.lt.dyninp%rattlemaxiteriation)
        redo = 0
        m = m + 1
c
c loop over all constrainted water bonds
c
        do l = 1, dyninp%maxrattle
            i = dyninp%rattlewater(l,1)
            do n = 2, 3
                j = dyninp%rattlewater(l, n)
                rij(1) = geom%angcoord(i*3-2) - geom%angcoord(j*3-2)
                vij(1)=dyninp%velocity(i*3-2)-dyninp%velocity(j*3-2)
                rij(2) = geom%angcoord(i*3-1) - geom%angcoord(j*3-1)
                vij(2)=dyninp%velocity(i*3-1)-dyninp%velocity(j*3-1)
                rij(3) = geom%angcoord(i*3) - geom%angcoord(j*3)
                vij(3) = dyninp%velocity(i*3) -dyninp%velocity(j*3)

                rv = vij(1)*rij(1) + vij(2)*rij(2) + vij(3)*rij(3)
c
c if the distance is as wanted 
c
                if (dabs(rv) .gt. eps) then
                    redo = 1
                    k = rv / (dyninp%rattledistance *
     *                  (1.d0/geom%atmmss(i) + 1.d0/geom%atmmss(j)))
                    do ii = -2, 0
                        i3 = i * 3 + ii
                        j3 = j * 3 + ii
                        dyninp%velocity(i3) = dyninp%velocity(i3)
     *                        - k * rij(3+ii) / geom%atmmss(i)
                        dyninp%velocity(j3) = dyninp%velocity(j3)
     *                        + k * rij(3+ii) / geom%atmmss(j)
                    enddo
                endif
            enddo
c
c angle constraint
c
            j = dyninp%rattlewater(l,3)
            i = dyninp%rattlewater(l,2)
            rij(1) = geom%angcoord(i*3-2) - geom%angcoord(j*3-2)
            vij(1) = dyninp%velocity(i*3-2) - dyninp%velocity(j*3-2)
            rij(2) = geom%angcoord(i*3-1) - geom%angcoord(j*3-1)
            vij(2) = dyninp%velocity(i*3-1) - dyninp%velocity(j*3-1)
            rij(3) = geom%angcoord(i*3) - geom%angcoord(j*3)
            vij(3) = dyninp%velocity(i*3) - dyninp%velocity(j*3)

            rv = vij(1) * rij(1) + vij(2) * rij(2) + vij(3) * rij(3)
            if (dabs(rv) .gt. eps) then
                redo = 1
                k = rv / (dyninp%rattleangle *
     *              (1.d0 / geom%atmmss(i) + 1.d0 / geom%atmmss(j)))
                do ii = -2, 0
                    i3 = i * 3 + ii
                    j3 = j * 3 + ii
                    dyninp%velocity(i3) = dyninp%velocity(i3)
     *                        - k * rij(3+ii) / geom%atmmss(i)
                    dyninp%velocity(j3) = dyninp%velocity(j3)
     *                        + k * rij(3+ii) / geom%atmmss(j)
                enddo
            endif
        enddo

c
c loop over all constrainted non water bonds
c

        do l = 1, dyninp%maxrattleh
            i = dyninp%rattlehyd(l,1)
c
c skip if QM atom and rattle only in MM
c
            if ((dyninp%ap_atom_zone(i) .eq. 0) .or.
     *              (dyninp%rattleh .eq. 1)) then
                j = dyninp%rattlehyd(l,2)
                rij(1) = geom%angcoord(i*3-2) - geom%angcoord(j*3-2)
                vij(1) = dyninp%velocity(i*3-2)-dyninp%velocity(j*3-2)
                rij(2) = geom%angcoord(i*3-1) - geom%angcoord(j*3-1)
                vij(2)  =dyninp%velocity(i*3-1)-dyninp%velocity(j*3-1)
                rij(3) = geom%angcoord(i*3) - geom%angcoord(j*3)
                vij(3) = dyninp%velocity(i*3)-dyninp%velocity(j*3)

                rv = vij(1)*rij(1) + vij(2)*rij(2) + vij(3)*rij(3)
                if (dabs(rv) .gt. eps) then
                    redo = 1
                    k = rv / (dyninp%rattlehdist(l) *
     *                  (1.d0 / geom%atmmss(i) + 1.d0 / geom%atmmss(j)))
                    do ii = -2, 0
                        i3 = i * 3 + ii
                        j3 = j * 3 + ii
                        dyninp%velocity(i3) = dyninp%velocity(i3)
     *                        - k * rij(3+ii) / geom%atmmss(i)
                        dyninp%velocity(j3) = dyninp%velocity(j3)
     *                        + k * rij(3+ii) / geom%atmmss(j)
                    enddo
                endif
            endif
        enddo

      enddo


c
c If after maxiteriation no convergence is reached stop
c
      if (m.eq.dyninp%rattlemaxiteriation) then
            write(*,*) "ERROR: velocity rattle failed"
            STOP "Rattle"
      endif

      return
      end

c
c
c*****************************************************************************
c  RATTLEPOSITION
c*****************************************************************************
c
      subroutine rattleposition(geom,mmgeom,dyninp)
c
c     Subroutine rattleposition
c
c     Created:  Jul. 13, 2010 by Soroosh Pezeshki
c
c     Purpose:  Apply the rattle algorithm on position
c
c     Description:  
c
c
c     Called by:  runmd
c
c     Calls:  
c
c     Variables defined here:
c
      use caltyp, only : tdebug
      use files, only : outfile
      use input, only : cartgeom, mmvalen, dynamicsinf, maxatm
      implicit none
c
c  i/o variables
c

      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom      
      type (dynamicsinf)  :: dyninp       
c
c  local variables
c      
      integer i, ii, i3, l, j, j3, m, n, natoms, n3tm, redo
      double precision dt, diff, eps, g, rij(3), dotpr,  rijold(3)

c inital values
c
      natoms = geom%natoms
      n3tm = 3 * natoms
      dt = dyninp%timestepl
      eps = dyninp%rattleeps
      redo = 1
      m = 0
      rij(:) = 0.d0
      dyninp%rattleskip = 0

c
c loop as long as atoms are moved but not more than maxiteriation
c
      do while (redo.eq.1 .and. m.lt.dyninp%rattlemaxiteriation)
        redo = 0
        m = m + 1
c
c loop over all constrainted bonds (water)
c
        do l = 1, dyninp%maxrattle
            i = dyninp%rattlewater(l,1)
            do n = 2, 3
                j = dyninp%rattlewater(l,n)
                do ii = -2, 0
                    i3 = i * 3 + ii
                    j3 = j * 3 + ii
                    rij(3+ii) = geom%angcoord(i3) - geom%angcoord(j3)
                enddo
                diff = rij(1)*rij(1) + rij(2)*rij(2) + rij(3)*rij(3)
     *               - dyninp%rattledistance
c
c if the distance is as wanted 
c
                if (dabs(diff) .gt. eps) then
                    redo = 1
                    rijold(1) = dyninp%coordold(i*3-2) -
     *                      dyninp%coordold(j*3-2)
                    rijold(2) = dyninp%coordold(i*3-1) -
     *                      dyninp%coordold(j*3-1)
                    rijold(3) = dyninp%coordold(i*3) -
     *                      dyninp%coordold(j*3)
                    dotpr = rij(1)*rijold(1) + rij(2)*rijold(2)
     *                      + rij(3)*rijold(3)
                    g = diff / ((2.d0 * dt) * dotpr *
     *                  (1.d0/geom%atmmss(i) + 1.d0/geom%atmmss(j)))
                    do ii = -2, 0
                        i3 = i * 3 + ii
                        j3 = j * 3 + ii
                        geom%angcoord(i3) = geom%angcoord(i3) - dt *
     *                      g * rijold(3+ii) / geom%atmmss(i)
                        geom%angcoord(j3) = geom%angcoord(j3) + dt *
     *                      g * rijold(3+ii) / geom%atmmss(j)
c
c adjust the velocities
c
                        dyninp%velocity(i3) = dyninp%velocity(i3) -
     *                      g * rijold(3+ii) / geom%atmmss(i)
                        dyninp%velocity(j3) = dyninp%velocity(j3) +
     *                      g * rijold(3+ii) / geom%atmmss(j)
                    enddo
                endif
            enddo
c
c angle constraints (constraint H to H distance)  (water)
c
            i = dyninp%rattlewater(l,2)
            do ii = -2, 0
                i3 = i * 3 + ii
                j3 = j * 3 + ii
                rij(3+ii) = geom%angcoord(i3) - geom%angcoord(j3)
            enddo
            diff = rij(1)*rij(1) + rij(2)*rij(2) + rij(3)*rij(3)
     *               - dyninp%rattleangle
c
c if the distance is as wanted 
c
            if (dabs(diff) .gt. eps) then
                redo = 1
                rijold(1) = dyninp%coordold(i*3-2) -
     *                  dyninp%coordold(j*3-2)
                rijold(2) = dyninp%coordold(i*3-1) -
     *                  dyninp%coordold(j*3-1)
                rijold(3) = dyninp%coordold(i*3)-dyninp%coordold(j*3)
                dotpr = rij(1) * rijold(1) + rij(2) * rijold(2)
     *                      + rij(3) * rijold(3)
                g = diff / ((2.d0*dt) * dotpr * (2.d0/geom%atmmss(j)))
                do ii = -2, 0
                    i3 = i * 3 + ii
                    j3 = j * 3 + ii
                    geom%angcoord(i3) = geom%angcoord(i3) - dt *
     *                 g * rijold(3+ii) / geom%atmmss(i)
                    geom%angcoord(j3) = geom%angcoord(j3) + dt *
     *                 g * rijold(3+ii) / geom%atmmss(j)
c
c adjust the velocities
c
                    dyninp%velocity(i3) = dyninp%velocity(i3) -
     *                   g * rijold(3+ii) / geom%atmmss(i)
                    dyninp%velocity(j3) = dyninp%velocity(j3) +
     *                   g * rijold(3+ii) / geom%atmmss(j)
                enddo
            endif
        enddo


c
c loop over all constrainted bonds (non water)
c
        do l = 1, dyninp%maxrattleh
            i = dyninp%rattlehyd(l,1)
c
c skip if QM atom and rattle only in MM
c
            if ((dyninp%ap_atom_zone(i) .gt. 0) .and.
     *              (dyninp%rattleh .gt. 1)) then
                if(m.eq.1) dyninp%rattleskip = dyninp%rattleskip + 1
                cycle
            endif
            j = dyninp%rattlehyd(l,2)
            do ii = -2, 0
                i3 = i * 3 + ii
                j3 = j * 3 + ii
                rij(3+ii) = geom%angcoord(i3) - geom%angcoord(j3)
            enddo
            diff = rij(1)*rij(1) + rij(2)*rij(2) + rij(3)*rij(3)
     *               - dyninp%rattlehdist(l)
c
c if the distance is as wanted 
c
            if(dabs(diff) .gt. eps) then 
                redo = 1
                rijold(1) = dyninp%coordold(i*3-2) -
     *                  dyninp%coordold(j*3-2)
                rijold(2) = dyninp%coordold(i*3-1) -
     *                  dyninp%coordold(j*3-1)
                rijold(3) = dyninp%coordold(i*3)-dyninp%coordold(j*3)
                dotpr = rij(1)*rijold(1) + rij(2)*rijold(2)
     *                      + rij(3)*rijold(3)
                g = diff / ((2.d0 * dt) * dotpr *
     *              (1.d0 / geom%atmmss(i) + 1.d0 / geom%atmmss(j)))
                do ii = -2, 0
                    i3 = i * 3 + ii
                    j3 = j * 3 + ii
                    geom%angcoord(i3) = geom%angcoord(i3) - dt *
     *                 g * rijold(3+ii) / geom%atmmss(i)
                    geom%angcoord(j3) = geom%angcoord(j3) + dt *
     *                 g * rijold(3+ii) / geom%atmmss(j)
c
c adjust the velocities
c
                    dyninp%velocity(i3) = dyninp%velocity(i3) -
     *                   g * rijold(3+ii) / geom%atmmss(i)
                    dyninp%velocity(j3) = dyninp%velocity(j3) +
     *                   g * rijold(3+ii) / geom%atmmss(j)
                enddo
            endif
400     enddo

      enddo

c
c If after maxiteriation no convergence is reached stop
c
      if (m.eq.dyninp%rattlemaxiteriation) then
            write(*,*) "ERROR: position rattle failed"
            STOP "Rattle"
      endif

      return
      end


c
c
c*****************************************************************************
c  RATTLEINIT
c*****************************************************************************
c
      subroutine rattleinit (geom, mmgeom, dyninp)
c
c     Subroutine rattleinit
c
c     Created:  Jul. 14, 2010 by Soroosh Pezeshki
c
c     Purpose:  Initalize the rattle algorithm
c
c     Description:  
c
c
c     Called by:  initmd
c
c     Calls:  
c
c     Variables defined here:
c
      use caltyp, only : tdebug
      use files, only : outfile
      use atminfo
      use input, only : cartgeom, mmvalen, dynamicsinf
      implicit none
c
c  i/o variables
c

      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom      
      type (dynamicsinf)  :: dyninp       
c
c  local variables
c      
      integer l, m, n, n3, i, j, j3, natoms, typeH, typeO
      double precision distance, x, y, z
      double precision cfloat


      natoms = geom%natoms

c Rigid water
c
      if (dyninp%rattle .gt. 0) then
      l = 1
c
c set distance=r(OH)^2 for 1) TIP r(OH)=0.9572 2) SPC water r(OH)=1
c set angle=r(HH)^2 for 1) TIP r(HH)=1.5139 2) SPC water r(HH)=1.63298
c
        if (dyninp%rattle.eq.1) then
            dyninp%rattledistance = 0.91623184d0
            dyninp%rattleangle = 2.29189519d0
        elseif (dyninp%rattle .eq. 2) then
            dyninp%rattledistance = 1.d0
            dyninp%rattleangle = 2.66662d0
        endif
c
c mark all water oxygens and the bonded hydrogens
c
        do n = 1, natoms
            if ((mmgeom%atmtypname(n) .eq. "OT") .or.
     *               (mmgeom%atmtypname(n) .eq. "OW")) then
                dyninp%rattlewater(l,1) = n
                dyninp%rattlewater(l,2) = mmgeom%atmvalenlist(n,1)
                dyninp%rattlewater(l,3) = mmgeom%atmvalenlist(n,2)
                l = l + 1
            endif
        enddo
c
c set the maximal number of rigid water
c
        dyninp%maxrattle = l - 1
        write(outfile,111) l - 1
111     format(5x,"Number of constraint water molecules: ", t50, I8)

      endif


c
c rattle all other hydrogens to the inital distance to the bond
c
      if (dyninp%rattleh.gt.0) then
      l = 1
c
c mark all non water hydrogens
c
        do n = 1, natoms
            if (geom%atmmss(n) .eq. ams(1)) then
                j = mmgeom%atmvalenlist(n,1)
                if ((mmgeom%atmtypname(j) .ne. "OT") .and.
     *               (mmgeom%atmtypname(j) .ne. "OW")) then
                    n3 = n * 3
                    j3 = j * 3
                    dyninp%rattlehyd(l,1) = n
                    dyninp%rattlehyd(l,2) = j
                    x = geom%angcoord(n3-2) - geom%angcoord(j3-2)
                    y = geom%angcoord(n3-1) - geom%angcoord(j3-1)
                    z = geom%angcoord(n3) - geom%angcoord(j3)
                    dyninp%rattlehdist(l) = (x*x + y*y + z*z)
                    l = l + 1
                endif
            endif
103     enddo

        dyninp%maxrattleh = l - 1
        write(outfile,112) l - 1
112     format(5x,"Number of constraint non water bonds: ", t50, I8)

      endif


      return
      end

c
c
c*****************************************************************************
c  READRESTRAINTS
c*****************************************************************************
c
      subroutine readrestraints (dyninp, infile, geom)
c
c     Subroutine readrestraints
c     
c     Created:  Nov. 30, 2010 by Soroosh Pezeshki
c
c     Remarks: in cf dyninp%smd(i,1:3) = force _x, _y, and _z
c              in cv  dyninp%smd(i,1) = ref. position
c                     dyninp%smd(i,2) = force constnat
c                     dyninp%smd(i,3) = ref. velocity*dt
c
c     Purpose:  reads the restraints parameter from input file
c     Called by:    RDYNAMICS
c     Calls:        RESTRAINTSLINE, readln, rline
c
c     Variables defined here:
c
      use input, only : dynamicsinf, maxrestraints, forceconvert,
     *                  cartgeom
      use files, only : outfile, smdf, smdfilename, 
     *                   smddummy, smddummyname
      use caltyp, only : tdebug
      implicit none
      type (dynamicsinf) :: dyninp
      type (cartgeom) :: geom
c
      character*80  restraintsfilename, string
      integer isec, iscend,ieof, icint, infile
c
c  local variables
c                 
      integer a, i
      integer iiatom, nwords
      integer ifin, ibeg, ifind, iasym, chindex, istrt, isect
c
      character*80  words(40), axes
      double precision cfloat


      call rline(infile,string,istrt,isect,iscend,ieof)
c      
!       if (iscend .eq. 1 .or. ieof .eq. 1) then
!         write (*,'(/a)') 'ERROR: The restraints file is empty.'
!         stop 'readrestraints 2'
!       endif
      i = 1
      dyninp%natmcvsmd = 0
      dyninp%ngrpcvsmd = 0
      dyninp%ngrpcfsmd = 0
      dyninp%natmcfsmd = 0
! c 
! c number of restraint atoms and molecules
! c
!          if (nwords .lt. 2) then
!             write (*,'(/a)') 'ERROR: please specify the number of',
!      *          ' atoms and groups.'
!             stop 'readrestraints 6'
!         endif
! 
!         dyninp%natmrestraints = icint(words(3))
!         dyninp%ngrprestraints = icint(words(4))
!         dyninp%natmcfsmd = icint(words(1))
!         dyninp%ngrpcfsmd = icint(words(2))
!         fin2 = dyninp%natmrestraints + dyninp%ngrprestraints +
!      *      dyninp%natmcfsmd + dyninp%ngrpcfsmd
! 
!         if (fin2 .gt. maxrestraints) then
!             write (*,'(/a)') 'ERROR: # of restraints is too high.'
!             flush(outfile)
!             stop 'readrestraints 3'
!         endif
! c skip comment before atom restraints
!       call readln (restraintsfile, words, nwords, isec, ieof, iscend)
!       call readln (restraintsfile, words, nwords, isec, ieof, iscend)

      do while ((ieof .eq. 0) .and. (iscend .eq. 0))
c
c constant force restraints on atoms
c
        if (string(istrt:istrt+5) .eq. "cfatom") then
            dyninp%firstatmcf = i
            call readln (infile, words, nwords, isec,
     *            ieof, iscend)
            do while (iscend .eq. 0)
c read the index of the atom
                dyninp%smdatom(i,1) = icint(words(1))
c check if all parameter are given
                if (nwords .lt. 4) then
                    write (*,'(/a)') 'ERROR: Not enough cf restraint',
     *                  ' parameters for atom ', dyninp%smdatom(i,1)
                    stop 'readrestraints 7'
                endif
c read forces
                dyninp%smd(i,1) = -1.d0*cfloat(words(2)) * forceconvert
                dyninp%smd(i,2) = -1.d0*cfloat(words(3)) * forceconvert
                dyninp%smd(i,3) = -1.d0*cfloat(words(4)) * forceconvert
c read next line
                call readln (infile, words, nwords, isec, ieof, iscend)
                i = i + 1
            enddo
            dyninp%natmcfsmd = i - dyninp%firstatmcf
        endif

c
c constant force restraints on groups
c
        if (string(istrt:istrt+6) .eq. "cfgroup") then
            dyninp%firstgrpcf = i
            call readln (infile, words, nwords, isec,
     *            ieof, iscend)
            do while (iscend .eq. 0)
c read the index of the atom
                dyninp%smdatom(i,1) = icint(words(1))
c check if all parameter are given
                if (nwords .lt. 4) then
                    write (*,'(/a)') 'ERROR: Not enough cf restraint',
     *                  ' parameters for group ', dyninp%smdatom(i,1)
                    stop 'readrestraints 8'
                endif
c read forces
                dyninp%smd(i,1) = -1.d0*cfloat(words(2)) * forceconvert
                dyninp%smd(i,2) = -1.d0*cfloat(words(3)) * forceconvert
                dyninp%smd(i,3) = -1.d0*cfloat(words(4)) * forceconvert
c read next line
                call readln (infile, words, nwords, isec, ieof, iscend)
                i = i + 1
            enddo
            dyninp%ngrpcfsmd = i - dyninp%firstgrpcf
        endif

c
c constant velocity restraints on atoms
c
        if (string(istrt:istrt+5) .eq. "cvatom") then
            dyninp%firstatmcv = i
            call readln (infile, words, nwords, isec,
     *            ieof, iscend)
            do while (iscend .eq. 0)
c read the index of the atom
                dyninp%smdatom(i,1) = icint(words(1))
c check if all parameter are given
                if (nwords .lt. 2) then
                    write (*,'(/a)') 'ERROR: Not enough restraint',
     *                  ' parameters for atom ',dyninp%smdatom(i,1),
     *                  ' on line ', i+2
                    flush(outfile)
                    stop 'readrestraints 4'
                endif
                call restraintsline (dyninp, words, i, nwords, geom)
c read next line
                call readln (infile, words, nwords, isec, ieof, iscend)
                i = i + 1
            enddo
            dyninp%natmcvsmd = i - dyninp%firstatmcv
        endif

c
c constant velocity restraints on groups
c
        if (string(istrt:istrt+6) .eq. "cvgroup") then
            dyninp%firstgrpcv = i
            call readln (infile, words, nwords, isec,
     *            ieof, iscend)
            do while (iscend .eq. 0)
c read the index of the atom
                dyninp%smdatom(i,1) = icint(words(1))
c check if all parameter are given
                if (nwords .lt. 3) then
                    write (*,'(/a)') 'ERROR: Not enough restraint',
     *                  ' parameters for group ',dyninp%smdatom(i,1),
     *                  ' on line ', i+3
                    stop 'readrestraints 5'
                endif
                call restraintsline (dyninp, words, i, nwords, geom)
c read next line
                call readln (infile, words, nwords, isec, ieof, iscend)
                i = i + 1
            enddo
            dyninp%ngrpcvsmd = i - dyninp%firstgrpcv
        endif


        if (i .gt. maxrestraints) then
            write (*,'(/a)') 'ERROR: # of restraints is too high.'
            flush(outfile)
            stop 'readrestraints 3'
        endif
        call rline(infile,string,istrt,isect,iscend,ieof)
      enddo

c
c open smd file if cv-smd is used
c
      if (dyninp%pmfatom + dyninp%pmfgroup .gt. 0) then
        open (unit = smdf, file = smdfilename, status = 'unknown',
     *      form = 'formatted', err = 5001) 
        open (unit = smddummy, file = smddummyname,
     *       status = 'unknown', form = 'formatted', err = 5000) 
        write(smddummy,*) 1
        write(smddummy,*) "1 Du 0.0 0.0 0.0"
      endif

      return

5001  write (outfile,'(/a)') 'ERROR: Cannot open the smd file.'
      flush (outfile)
      stop 'READRESTRAINT 10'
5000  write (*,'(/a)') 'ERROR: Cannot open the smdDummy file.'
6000  stop 'readrestraint 11'


      end

c
c
c*****************************************************************************
c  READRESTRAINTFILE
c*****************************************************************************
c
      subroutine readrestraintfile (dyninp, restraintsfilename, geom)
c
c     Subroutine readrestraintsfile
c     
c     Created:  Nov. 30, 2010 by Soroosh Pezeshki
c
c     Purpose:  reads the restraints parameter from restaintfile
c     Called by:  RDYNAMICS
c     Calls:      RESTRAINTSLINE, readrestraints
c
c     Variables defined here:
c
      use input, only : dynamicsinf, cartgeom
      use files, only : restraintsfile, outfile
      implicit none
      type (dynamicsinf) :: dyninp
      type (cartgeom) :: geom
c
      character*80  restraintsfilename
c
c  local variables
c                 

c      
c open the restraint file
c
      open (unit=restraintsfile, file=restraintsfilename,
     *       status='old', form='formatted', err=5000)

        call readrestraints (dyninp, restraintsfile, geom)

      close (restraintsfile)
      return
5000  write (*,'(/a)') 'ERROR: Cannot open the restraints file.'
6000  stop 'readrestraintfile 1'

      end


c
c
c*****************************************************************************
c  RESTRAINTSLINE
c*****************************************************************************
c
      subroutine restraintsline (dyninp, words, i, nwords, geom)
c
c     Subroutine restraintsline
c     
c     Created:  Feb. 16, 2011 by Soroosh Pezeshki
c
c     Purpose:  read a line of the restraints parameters
c     Remarks: in cv  dyninp%smd(i,1) = ref. position
c                     dyninp%smd(i,2) = force constnat
c                     dyninp%smd(i,3) = ref. velocity*dt
c     Called by:  readrestraints, readrestraintsfile
c     Calls:
c
c     Variables defined here:
c
      use caltyp, only : tdebug
      use input, only : dynamicsinf, forceconvert2, cartgeom
      implicit none
      type (dynamicsinf) :: dyninp
      type (cartgeom) :: geom
      integer i, nwords, j
c
      character*80 words(40), axes
      double precision cfloat, dt

      dt = dyninp%timestepl

c read the axes on which constraint should be applied
      axes = words(2)
      if (axes(1:1).eq.'X') dyninp%smdatom(i,2) = 2
      if (axes(1:1).eq.'Y') dyninp%smdatom(i,2) = 1
      if (axes(1:1).eq.'Z') dyninp%smdatom(i,2) = 0

c read the inital coordinate for restraints
      if (nwords.gt.2) then
        dyninp%smd(i,1) = cfloat(words(3))
      else
        j = 3 * dyninp%smdatom(i,1) - dyninp%smdatom(i,2)
        dyninp%smd(i,1) = geom%angcoord(j)
      endif

c read the force constant a=on the atom if set; else default
      if (nwords .gt. 3) then
        dyninp%smd(i,2) = -1.d0 * cfloat(words(4)) * forceconvert2
      else
        dyninp%smd(i,2) = -1.d0 * forceconvert2
      endif

c read velocity for smd if set else 0
      if (nwords.gt.4) then
         dyninp%smd(i,3) = cfloat(words(5)) * dt
      else
         dyninp%smd(i,3) = 0.d0
      endif

      end

c
c
c*****************************************************************************
c  RESTRAINTS
c*****************************************************************************
c
      subroutine restraints (dyninp, geom, stepnow)
c
c     Subroutine restraints
c     
c     Created:  Nov. 30, 2010 by Soroosh Pezeshki
c
c     Purpose:  apply restraints to atom
c     Remarks: in cf dyninp%smd(i,1:3) = force _x, _y, and _z
c              in cv  dyninp%smd(i,1) = ref. position
c                     dyninp%smd(i,2) = force constnat
c                     dyninp%smd(i,3) = ref. velocity*dt
c     Called by:  runmd
c     Calls: shiftsmd
c
c     Variables defined here:
c
      use caltyp, only : tdebug
      use input, only : dynamicsinf, cartgeom, forceconvert2,
     *                  forceconvert
      use files, only : outfile, smdf
      implicit none
       type (dynamicsinf) :: dyninp
       type (cartgeom) :: geom
c
c  local variables
c                 
      integer i, j, j3, l, l3, i1, stepnow
      double precision tmp, com, massratio, work, wext
      double precision deltar, deltar2, ftmp, kcalconvert
      character axes

c convert amu*Angstrom^2/fs^2 to kcal/mol
      kcalconvert = 2390.05559d0
      
c
c save the coordinate of the dummy atom
c
      if (dyninp%pmfatom .gt. 0) then
        j = 3 * dyninp%pmfatom
        dyninp%pmfi(1:3) = geom%angcoord(j-2:j)
      endif
      if (dyninp%pmfgroup .gt. 0) then
        do l = 1, geom%natoms
            if (dyninp%group(l) .eq. dyninp%pmfgroup) then
                l3 = l * 3
                dyninp%pmfi(1:3) = geom%angcoord(l3-2:l3)
                exit
            endif
        enddo
      endif

c--------------------
c cf SMD Atom
c
      do i1 = 0, dyninp%natmcfsmd - 1
        i = i1 + dyninp%firstatmcf
        l = dyninp%smdatom(i,1)
        l3 = l * 3
        dyninp%force(l3-2) = dyninp%force(l3-2) + dyninp%smd(i,1)
        dyninp%force(l3-1) = dyninp%force(l3-1) + dyninp%smd(i,2)
        dyninp%force(l3) = dyninp%force(l3) + dyninp%smd(i,3)
      enddo

c--------------------
c cf SMD group
c
      do i1 = 0, dyninp%ngrpcfsmd - 1
        i = i1 + dyninp%firstgrpcf
        j = dyninp%smdatom(i,1)
        do l = 1, geom%natoms
            if (dyninp%group(l) .eq. j) then
                l3 = l * 3
                massratio = geom%atmmss(l) / dyninp%groupmass(j)
                dyninp%force(l3-2) = dyninp%force(l3-2)
     *              + dyninp%smd(i,1) * massratio
                dyninp%force(l3-1) = dyninp%force(l3-1)
     *              + dyninp%smd(i,2) * massratio
                dyninp%force(l3) = dyninp%force(l3)
     *              + dyninp%smd(i,3) * massratio
            endif
        enddo
      enddo


c------------------------------
c Restraints and cv SMD on Atom
c
      do i1 = 0, dyninp%natmcvsmd - 1
        i = i1 + dyninp%firstatmcv
        tmp = 0.d0
        deltar = 0.d0
        j = dyninp%smdatom(i,1)
        j3 = j * 3 - dyninp%smdatom(i,2)
c        ftmp = dyninp%force(j3)
        deltar = (geom%angcoord(j3) - dyninp%smd(i,1))
        if (dyninp%periodic .eq. 1) then
            call shiftsmd (dyninp,deltar,i,3-dyninp%smdatom(i,2),
     *                      geom%angcoord(j3))
        endif
        tmp = deltar * dyninp%smd(i,2)
        dyninp%force(j3) = dyninp%force(j3) + tmp

c debug output
        if ((tdebug .eq. 1) .or. (tdebug .eq. 2)) then
            if (dyninp%smdatom(i,2) .eq. 2) axes="x"
            if (dyninp%smdatom(i,2) .eq. 1) axes="y"
            if (dyninp%smdatom(i,2) .eq. 0) axes="z"
            write(*,*) 'Restraints on atom:', j, 
     *          'in the ', axes, ' direction'
            write(*,*) tmp, dyninp%smd(i,1), geom%angcoord(j3),
     *           geom%angcoord(j3) - dyninp%smd(i,1)
        endif
c
c output in SMD file
c
        if (j .eq. dyninp%pmfatom) then
            deltar2 = geom%angcoord(j3) -
     *           dyninp%coordold(j3)
            if (dyninp%smdatom(i,2) .eq. 2) axes="x"
            if (dyninp%smdatom(i,2) .eq. 1) axes="y"
            if (dyninp%smdatom(i,2) .eq. 0) axes="z"
            write(smdf,*) "Timestep: ", stepnow
            write(smdf,*) "Position SMD atom [Ang]", axes, 
     *                  geom%angcoord(j3)
            write(smdf,201) "Velocity SMD atom [Ang/fs]",
     *                  dyninp%velocity(j3)
            write(smdf,201) "change of position SMD atom [Ang]: ", 
     *                  deltar2
            write(smdf,201) "Reference position [Ang]: ", 
     *                  dyninp%smd(i,1)
            write(smdf,201) "Distance SMD atom to reference [Ang]: ", 
     *                  deltar
            write(smdf,201) "F_ext [nN]: ", tmp / forceconvert2
            wext = (tmp * deltar) / -2.d0 * kcalconvert
            write(smdf,*) "k*f*r/2 [kcal/mol]: ", wext
c dyninp%smd(i,3) = v*dt
            dyninp%work = dyninp%work + dyninp%smd(i,3)*tmp*kcalconvert
            write(smdf,*) "work_extern [kcal/mol]: ", dyninp%work
            write(smdf,*) "W [kcal/mol]: ", dyninp%work - wext
            dyninp%pmfi(3-dyninp%smdatom(i,2)) = dyninp%smd(i,1)
        endif
        dyninp%smd(i,1) = dyninp%smd(i,1) + dyninp%smd(i,3)
      enddo


c--------------------------------
c Restraints and cv SMD on Groups
c
      do i1 = 0,  dyninp%ngrpcvsmd- 1
        i = i1 + dyninp%firstgrpcv
        tmp = 0.d0
        com = 0.d0
        ftmp = 0.d0
        deltar = 0.d0
        deltar2 = 0.d0
        j = dyninp%smdatom(i,1)
c
c sum up the com for each coordinate
c
        do l = 1, geom%natoms
            l3 = l * 3 - dyninp%smdatom(i,2)
            if (dyninp%group(l) .eq. j) then
                com = com + geom%atmmss(l) * geom%angcoord(l3)
                deltar2 = deltar2 +  geom%atmmss(l) *
     *             (geom%angcoord(l3) - dyninp%coordold(l3))
            endif
        enddo
c
c divide the sum through the mass of the groups
c
        if (dyninp%groupmass(j) .ne. 0.d0) then
            com = com / dyninp%groupmass(j)
            deltar2 = deltar2 / dyninp%groupmass(j)
        endif

        deltar = (com - dyninp%smd(i,1))
        if (dyninp%periodic .eq. 1) then
            call shiftsmd (dyninp, deltar, i, 
     *                  3-dyninp%smdatom(i,2), com)
        endif
        tmp = deltar * dyninp%smd(i,2)
        dyninp%smd(i,1) = dyninp%smd(i,1) + dyninp%smd(i,3)
c debug output
        if ((tdebug .eq. 1) .or. (tdebug .eq. 2)) then
            if (dyninp%smdatom(i,2) .eq. 2) axes = "x"
            if (dyninp%smdatom(i,2) .eq. 1) axes = "y"
            if (dyninp%smdatom(i,2) .eq. 0) axes = "z"
            write(*,*) 'Restraints on group:', j, 
     *           'in the ', axes, ' direction'
            write(*,*) tmp, dyninp%smd(i,1), com,
     *           com - dyninp%smd(i,1)
        endif

        do l = 1, geom%natoms
            if (dyninp%group(l) .eq. j) then
                l3 = l * 3 - dyninp%smdatom(i,2)
                massratio = geom%atmmss(l) / dyninp%groupmass(j)
                ftmp = ftmp + dyninp%force(l3)
                dyninp%force(l3) = dyninp%force(l3) + tmp*massratio
            endif
        enddo
c
c output in SMD file
c
        if (j .eq. dyninp%pmfgroup) then
            if (dyninp%smdatom(i,2) .eq. 2) axes="x"
            if (dyninp%smdatom(i,2) .eq. 1) axes="y"
            if (dyninp%smdatom(i,2) .eq. 0) axes="z"
            write(smdf,*) "Timestep: ", stepnow
            write(smdf,*) "Position SMD group [Ang]:", axes, com
            write(smdf,201) "Change of position SMD group [Ang]:",
     *                  deltar2
            write(smdf,201) "Reference position [Ang]:",
     *                  dyninp%smd(i,1)
            write(smdf,201) "Distance SMD group to reference [Ang]:", 
     *                  deltar
            write(smdf,201) "F_ext [nN]: ", tmp / forceconvert2
            wext = (tmp * deltar) / -2.d0 * kcalconvert
            write(smdf,*) "kfz [kcal/mol]: ", wext
            dyninp%work = dyninp%work + dyninp%smd(i,3)*tmp*kcalconvert
            write(smdf,*) "work_extern [kcal/mol]: ", dyninp%work
            write(smdf,*) "W [kcal/mol]: ", dyninp%work - wext
            dyninp%pmfi(3-dyninp%smdatom(i,2)) = dyninp%smd(i,1)
        endif
      enddo
201   format(a, e12.4) 

      return
      end

c
c
c*****************************************************************************
c  INSTANTMOVE
c*****************************************************************************
c
      subroutine instantmove (dyninp, geom)
c
c     Subroutine instantmove
c     
c     Created:  Feb. 28, 2011 by Soroosh Pezeshki
c
c     Purpose:  Constraint scan method, move atoms and groups instantly
c     Called by:  runmd
c     Calls: 
c
c     Variables defined here:
c
      use caltyp, only : tdebug
      use input, only : dynamicsinf, cartgeom
      implicit none
       type (dynamicsinf) :: dyninp
       type (cartgeom) :: geom
c
c  local variables
c                 
      integer i, j, j3, l, l3, i1

      do i = 1, dyninp%natmrestraints
        j = dyninp%smdatom(i,1)
        j3 = j * 3
        geom%angcoord(j3-2) = geom%angcoord(j3-2) + dyninp%smd(i,1)
        geom%angcoord(j3-1) = geom%angcoord(j3-1) + dyninp%smd(i,2)
        geom%angcoord(j3) = geom%angcoord(j3) + dyninp%smd(i,3)
c debug output
            if ((tdebug .eq. 1) .or. (tdebug .eq. 2)) then
                write(*,*) 'Move atom:', j
                write(*,*) geom%angcoord(j3-2),
     *               geom%angcoord(j3-1), geom%angcoord(j3)
            endif
      enddo

      do i1 = 1, dyninp%ngrprestraints
        i = i1 + dyninp%natmrestraints
        j = dyninp%smdatom(i,1)
        do l = 1, geom%natoms
            if (dyninp%group(l) .eq. j) then
                l3 = l * 3
                geom%angcoord(l3-2) = geom%angcoord(l3-2) +
     *              dyninp%smd(i,1)
                geom%angcoord(l3-1) = geom%angcoord(l3-1) +
     *              dyninp%smd(i,2)
                geom%angcoord(l3) = geom%angcoord(l3)+dyninp%smd(i,3)
c debug output
                if ((tdebug .eq. 1) .or. (tdebug .eq. 2)) then
                    write(*,*) 'Move atom:', l
                    write(*,*) geom%angcoord(l3-2),
     *               geom%angcoord(l3-1), geom%angcoord(l3)
                endif
            endif
        enddo

      enddo

      return
      end


c
c
c*****************************************************************************
c  SHIFTSMD
c*****************************************************************************
c
      subroutine shiftsmd (dyninp, tmp, i, j, com)
c
c     Subroutine shiftsmd
c     
c     Created:  Feb. 21, 2011 by Soroosh Pezeshki
c
c     Purpose:  moves a restraint reference position that is out of the periodic box
c                                                                       
c 
c     Called by:  restraints
c     Calls:
c
c     Variables defined here:
c
      use input, only: dynamicsinf
      implicit none
      type (dynamicsinf) :: dyninp
      integer i, j
      double precision  tmp, com
c
c  local variables
c
      double precision  bsize

      bsize = dyninp%boxsize(j)
      if (dabs(tmp) .gt. bsize/2.d0) then
          if ((dyninp%smd(i,1) .lt. dyninp%boxmin(j))) then
              dyninp%smd(i,1) = dyninp%smd(i,1) + bsize
          elseif ((dyninp%smd(i,1) .gt. dyninp%boxmax(j))) then
              dyninp%smd(i,1) = dyninp%smd(i,1) - bsize
          endif
          tmp = (com - dyninp%smd(i,1))
      endif
      end

c
c
c*****************************************************************************
c  READINSTANTMOVE
c*****************************************************************************
c
      subroutine readinstantmove (dyninp, infile)
c
c     Subroutine readinstantmove
c     
c     Created:  March 01, 2011 by Soroosh Pezeshki
c
c     Purpose:  reads the constraint scan parameter from input file
c     Called by:  RDYNAMICS
c     Calls: readln
c
c     Variables defined here:
c
      use input, only : dynamicsinf, maxrestraints
      use files, only : outfile
      use caltyp, only : tdebug
      implicit none
       type (dynamicsinf) :: dyninp
c
c  local variables
c                 
      integer i, isec, iscend, ieof, icint, infile
      integer iiatom, nwords
      integer ifin, ibeg, ifind, iasym, chindex
c
      character*80  words(40)
      double precision cfloat
      integer rotateatom, rotategroup

      call readln (infile, words, nwords, isec, ieof, iscend) 
c 
c number of restraint atoms and molecules (linear and rotation)
c
        dyninp%natmrestraints = icint(words(1))
        dyninp%ngrprestraints = icint(words(2))

        if (icint(words(1))+icint(words(2)) .gt. maxrestraints) then
            write (outfile,'(/a)') 'ERROR: # of steering is too high.'
            stop 'readinstantmove 3'
        endif

        call readln (infile, words, nwords, isec, ieof, iscend)
c
c restraint atoms and groups
c
        do i = 1, dyninp%natmrestraints + dyninp%ngrprestraints
c read the index of the atom
            dyninp%smdatom(i,1) = icint(words(1))
            dyninp%smdatom(i,2) = 0
c check if all parameter are given
            if (nwords .lt. 4) then
                if (i .le. dyninp%natmrestraints) then
                    write (*,'(/a)') 'ERROR: Not enough steering',
     *                  ' parameters for atom ', dyninp%smdatom(i,1)
                stop 'readinstantmove 4'
                else
                    write (*,'(/a)') 'ERROR: Not enough steering',
     *                  ' parameters for group ', dyninp%smdatom(i,1)
                    stop 'readinstantmove 5'
                endif
            endif
c read replacement for constraint scan in x, y, and z
            dyninp%smd(i,1) = cfloat(words(2))
            dyninp%smd(i,2) = cfloat(words(3))
            dyninp%smd(i,3) = cfloat(words(4))
c debug output
            if ((tdebug .eq. 1) .or. (tdebug .eq. 2)) then
                if (i .le. dyninp%natmrestraints) then
                    write(*,100) dyninp%smdatom(i,1)
                else
                    write(*,103) dyninp%smdatom(i,1)
                endif
                write(*,102) dyninp%smd(i,1:3)
100   format(3x,'Instat move: atom ',i5)
102   format(3x,'Displacement: ',f8.4,f8.4,f8.4)
103   format(3x,'Instat move: group ',i5)
            endif

c read next line
            call readln (infile, words, nwords, isec, ieof, iscend)
            if (ieof .eq. 1) then
                write (*,'(/a)') 'ERROR: Not enough steering;',
     *       ' Please set ',dyninp%natmrestraints, ' atom steering',
     *          ' and ', dyninp%ngrprestraints, ' group steering'
                stop 'readinstantmove 6'
            endif
      enddo

      do while (iscend .eq. 0 .and. isec .eq. 0 .and. ieof .eq. 0)
            call readln (infile, words, nwords, isec, ieof, iscend)
            if (iscend .eq. 0) then
                write(*,*) '   Warning: skipping line ', words(1),
     *              ' ... To many lines in CONSTRAINTSCAN definition.'
            endif
      enddo

      return
      end

c
c
c*****************************************************************************
c  TEMPCONTROLNH
c*****************************************************************************
c
      subroutine tempcontrolnh (dyninp, ekin, p, dt, degfreedom, 
     *              ekincons)
c
c     Subroutine tempcontrolnh
c     
c     Created:  Oct. 25, 2011 by Soroosh Pezeshki
c
c     Purpose:  Nose Hoover temperature control
c                                                                       
c 
c     Called by:  runmd
c     Calls:
c
c     Variables defined here:
c
      use input, only: dynamicsinf, tempconvert, k_B
      use files, only: outfile
      use caltyp, only: tdebug 
      implicit none
       type (dynamicsinf) :: dyninp
      double precision  ekin, p(3), dt, ekincons
      integer degfreedom
c
c  local variables
c
      double precision dt2, dt4, dt8, s, g1, gm, v1, vm, q1, qm
      double precision fkT, kT, ek
c
      dt2 = dt*0.5d0
      dt4 = dt2*0.5d0
      dt8 = dt4*0.5d0
      q1 = dyninp%q1
      qm = dyninp%q2
      v1 = dyninp%v1
      vm = dyninp%v2

c
c ek, kT and fkT in amu Angstrom^2/fs^2
c
      kT = dyninp%temperature  * k_B
      fkT = dble(degfreedom)*kT
c      ek = 1.d0* ekin * 4.35974394d-18
      ek = ekin / ekincons
      gm = (q1*v1*v1 - kT) / qm
      vm = vm + dt4*gm
      v1 = v1 * dexp(-dt8*vm)
      g1 = (ek - fkT) / q1
      v1 = (v1 + dt4*g1) * dexp(-dt8*vm)
      s = dexp(-dt2*v1)
      dyninp%x1 = dyninp%x1 + dt2*v1
      dyninp%x2 = dyninp%x2 + dt2*vm
      ekin = ekin*s*s
      v1 = v1 * dexp(-dt8*vm)
      g1 = (ek*s*s-fkT) / q1
      v1 = (v1 + dt4*g1) * dexp(-dt8*vm)
      gm = (q1*v1*v1 - kT) / qm
      vm = vm + dt4*gm
      dyninp%velocity(:) = dyninp%velocity(:) * s
      p(:3) = p(:3) * s
      dyninp%tempnow = dyninp%tempnow * s * s

      dyninp%v1 = v1
      dyninp%v2 = vm


c write out if debug
c
      if (tdebug .gt. 0) then
        write(*,*) 'Nose Hoover temperature control factor: ', s
      endif

      end


c
c
c*****************************************************************************
c  PROGMMGHKMD
c*****************************************************************************
c
      subroutine progmmghkmd(ppnum, prognme, esmeth, esbas, geom, 
     *                   mmgeom, optl, ener, grad, ierr, ver_lab)
c
c     Subroutine progmmghkmd
c
c     Created:  Jun. 10, 2010 by Soroosh Pezeshki
c     Based on progmmghk
c
c     Purpose:  Get a single point energy and gradient for the given geom
c        with the specified force field package.
c
c     Description:  Calls a subroutine to write an MM input file,
c         runs two (energy and gradient) calculations on the system, 
c         and then calls a subroutine 
c         that reads the energy and gradient from the MM output file
c         created by the force field package.
c
c     Note:  Currently only TINKER is enabled but it would be easily
c        expanded to other force field programs.  new
c        _oute subroutines would have to be written for each different
c        program though.
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c     Called by:  mlghook, qmmmghkmd, initmd
c
c     Calls:  t41outg, system, CPU_TIME, t41inpxyz, t41inp
c
c     Variables defined here:
c
      use caltyp
      use files
      use input, only : cartgeom, mmvalen, maxoptl
      implicit none
c
c  i/o variables
c
      character*10 prognme
      character*20 esmeth, esbas, ver_lab                                                
      type (cartgeom) :: geom
      type (mmvalen) :: mmgeom      
c
      character*80 optl(maxoptl)
      double precision ener, grad(3*geom%natoms)
      integer ierr, ppnum
c
c  local variables
c      
c  timing variables
c
      real timing1, timing2, deltatiming
      integer timingw1, timingw2, deltatimingw
      call CPU_TIME(timing1)
      call SYSTEM_CLOCK(timingw1)


c
c  TINKER 
c   For speed reason this is comment out until another option to Tinker is there
c      if (prognme(:6).eq.'tinker') then
c

        if(tap.eq.1) then
          open (unit=t41inf, status='unknown', file=t41in, err=9994)
          call t41inpxyz(t41inf, geom, mmgeom)
          close (t41inf, err=9994)
        else
          open (unit=t41inf, status='unknown', file=t41in, err=9994)
          open (unit=t41keyf, status='unknown',file=t41key, err=9995)
          call t41inp(t41inf, t41keyf,
     *    esmeth, esbas, geom, mmgeom,optl,tgrad,ver_lab)
          close (t41inf, err=9994)
          close (t41keyf, err=9995)
        endif
c
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  Tinker input timing ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw

        endif

c read the atomic MM charge
c  also print MM parameters if required
c
c!       if ((tprm .eq. 1) .or. (mmgeom%lprnatmprm .eq. 1)) then
c!             call system('t41shuttle t41 p')
c!             call t41outp(mmgeom)
c!       endif    
c
c calculate energy and gradient
c    
c!       call system('t41shuttle t41 e')
c!       call t41oute(1,esmeth,ener)
c      call system('t41md '//tt)
       call system('t41shuttle t41 g')
c
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  Tinker run timing ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
       endif

      call t41outg(1,esmeth,ener, grad, geom%natoms)
c
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  Tinker output timing ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif

c      endif
      return

 9994  write (outfile,*) 'Error creating TINKER .xyz file'
       stop 'progmmghkmd 4'
9995  write (outfile,*) 'Error creating TINKER .key file'
      stop 'progghkmd 5'

      end


c
c
c*****************************************************************************
c  PROGMM0GHKMD
c*****************************************************************************
c
      subroutine progmm0ghkmd(ppnum, prognme, esmeth, esbas, geom, 
     *                   mmgeom, optl, ener, grad, ierr, ver_lab,
     *                   qmmminp)
c
c     Subroutine progmm0ghkmd
c
c     Created:  Jun. 10, 2010 by Soroosh Pezeshki
c     Based on  progmm0ghk
c
c     Purpose:  Get a single point energy and gradient for the given geom
c        with the specified force field package, whith the MM charges on 
c        QM atoms set to zero.
c
c     Description:  Calls a subroutine to write an MM input file,
c         runs two (energy and gradient) calculations on the system, 
c         and then calls a subroutine 
c         that reads the energy and gradient from the MM output file
c         created by the force field package.
c
c     Note:  Currently only TINKER is enabled but it would be easily
c        expanded to other force field programs.  
c        _oute subroutines would have to be written for each different
c        program though.
c
c  ppnum determines the name of the input and output files for parallel runs
c  reserved for future use
c
c     Called by:  qmmmghkmd
c
c     Calls:  t41outg, system, t41inp
c
c     Variables defined here:
c
      use caltyp
      use files
      use input, only : cartgeom, mmvalen, maxoptl, qmmminf
      implicit none
c
c  i/o variables
c
      character*10  prognme
      character*20  esmeth, esbas, ver_lab                                                
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom      
c
      character*80  optl(maxoptl)      
      double precision   ener, grad(3*geom%natoms)
      integer            ierr, ppnum
      type (qmmminf)  :: qmmminp       
c
c  local variables
c      
      integer      i,ii

c
c  TINKER 
c
c      if (prognme(:6).eq.'tinker' ) then
         open (unit=t41inf, status='unknown', file=t41in, err=9994)
         open (unit=t41keyf, status='unknown', file=t41key, err=9995)
         call t41inp(t41inf, t41keyf,
     *    esmeth, esbas, geom, mmgeom,optl,tgrad,ver_lab)
         close (t41inf, err=9994)

c
c set MM charges on QM atoms to zero
c  for the entire system, the index read in for the QM atoms will be used
c  for the capped system, the index will be from 1 to N, where N is the 
c      summed number for the QM and HL atoms.
c  the tcssmmkey in the common block caltyp specify the system, i.e.
c      whether the entire or the capped system
         
        if (tcssmmkey .eq. 1) then
           ii = qmmminp%qmnatm
           do i=1, ii
              write (t41keyf, *) 'charge ', -qmmminp%qmatm(i), ' 0.0'
           enddo          
        elseif (tcssmmkey .eq. 2) then
           ii = qmmminp%qmnatm + qmmminp%capatm%capnatm 
           do i=1, ii
              write (t41keyf, *) 'charge ', -i, ' 0.0'
           enddo
        else
           write (outfile,*) 'Error! Please specify CPS or ES.'
           stop 'progmm0ghk 6'
        endif
         
        close (t41keyf, err=9995)
c
c read the atomic MM charge
c  also print MM parameters if required
c
c!       if ((tprm .eq. 1) .or. (mmgeom%lprnatmprm .eq. 1)) then
c!             call system('t41shuttle t41 p')
c!             call t41outp(mmgeom)
c!       endif    
c
c calculate energy and gradient
c    
c!       call system('t41shuttle t41 e')
c!       call t41oute(1,esmeth,ener)

c      call system('t41md '//tt)
      call system('t41shuttle t41 g')

      call t41outg(1,esmeth,ener, grad, geom%natoms)

c      endif
      return
9994  write (outfile,*) 'Error creating TINKER .xyz file'
      stop 'progmm0ghkmd 4'
9995  write (outfile,*) 'Error creating TINKER .key file'
      stop 'progmm0ghkmd 5'
      end



c
c
c******************************************************************************
c  QMMMGHKMD
c******************************************************************************
c
      subroutine qmmmghkmd(qmprognme, mmprognme, geom, 
     * mmgeom, qmmminp, ierr,qmmmgrad,qmmmener)
c
c     Subroutine qmmmghkmd
c
c     Created:  March 04, 2011 by Soroosh Pezeshki
c     Based on qmmmghk by Hai Lin
c
c     Purpose:  To determine the QM/MM energy and gradient
c   
c      Please see QMMMEHK for notes.   
c
c     Called by:  mlghook, hotspot, oniomxs, sortedap, permutedgradient
c     Calls:  progmmghkmd, progqmghkmd, qmmmdisp, CPU_TIME, qmmmgsum
c             progmm0ghkmd, qmmmssgeom, bgchscale, bgchshift,
c             qmmmbgchgeom, polareech, fbbpc, progmmstghk, progghk, proggsthk
c  
      use caltyp
      use input,  only   : cartgeom, mmvalen, qmmminf, maxatm, maxoptl
      use files,  only   : outfile
      use energy, only   : qmes, mmes
      use gradient, only : qmmmqmgr, qmmmmmgr
      implicit none
c
c i/o variables
c
      character*10       qmprognme, mmprognme
      type (cartgeom) :: geom
      type (mmvalen)  :: mmgeom
      type (qmmminf)  :: qmmminp
      integer            ierr
      double precision qmmmgrad(3*maxatm+540),  qmmmener
c
      character*80 optl(maxoptl), keystring

c
c local variables
c      
      type (cartgeom) :: ssgeom, bgchgeom
      type (mmvalen)  :: ssmmgeom
      integer            lsnatm, ssnatm, bgchnum
      integer            sumtyp, i
c
c for flexible boundary treatment
c add by Yan Zhang Apr. 17, 2009
c 
      double precision   px, qme1, qme2
      double precision   polmmg1(maxatm*3)
      double precision   polmmg2(maxatm*3)
c end
c      double precision   mmst
c
c
c      tdebug=1
c   add by Soroosh Pezeshki July 07,2010   
c  timing variables
c
      real timing1, timing2, deltatiming
      integer timingw1, timingw2, deltatimingw
      call CPU_TIME(timing1)
      call SYSTEM_CLOCK(timingw1)
      
c
c end add by Soroosh Pezeshki July 07,2010
c    
c
c  Save the prnprm switch value
c
      ssmmgeom%lprnatmprm = mmgeom%lprnatmprm
      sumtyp = 0
cx      call qmmmgsum(qmmminp, mmgeom, geom%natoms, 
      call qmmmgsum(qmmminp, mmgeom, geom, geom%natoms,     !0208BW15 add geom
     *   qmmmgrad(:geom%natoms*3), sumtyp, ierr)

c
c  MM calculation on the entire system
c      
        tcssmmkey = 1
        if (qmmminp%embed .eq. 'mechanical') then            
            call progmmghkmd(1,mmprognme,qmmminp%mmmeth,qmmminp%mmbas,
     *          geom, mmgeom, qmmminp%mmoptl, mmes(2), 
     *          qmmmmmgr(2,:geom%natoms*3), 
     *          ierr, 'null')
        else
           call progmm0ghkmd(1,mmprognme,qmmminp%mmmeth,qmmminp%mmbas,
     *          geom, mmgeom, qmmminp%mmoptl, mmes(2), 
     *          qmmmmmgr(2,:geom%natoms*3),
     *          ierr, 'null', qmmminp)
        endif


100   format(80('-'))
110   format(/2x,'MM calculation on the entire system:',/)
115   format('MM energy (a.u.) for the entire system:',f20.10)      
116   format(/,10('Debug'))
117   format(2x,'MM gradient on entire system:')
118   format(4(d20.10))
119   format(10('Debug'),/)

      if (tdebug .eq. 1) then
          write (outfile,116)
          write (outfile,117)
          write (outfile,118) qmmmmmgr(2,:geom%natoms*3)
          write (outfile,119)
      endif
      sumtyp = 1 
cx      call qmmmgsum (qmmminp, mmgeom, geom%natoms,
      call qmmmgsum(qmmminp, mmgeom, geom, geom%natoms,     !0208BW15 add geom
     *   qmmmmmgr(2,:geom%natoms*3), sumtyp, ierr)
c
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  MM calculation on the entire system:'
     &      ,' timing ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif
c
c  Assign the appropriate geometries to ssgeom
c      
      call qmmmssgeom(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
120   format(' Construct the capped primary system:')
      
c
c  MM calculation on the primary system, no point charges import
c
        tprm = 0
        tcssmmkey = 2
        if (qmmminp%embed .eq. 'mechanical') then            
            call progmmghkmd(1,mmprognme, qmmminp%mmmeth,qmmminp%mmbas,
     *          ssgeom, ssmmgeom,qmmminp%mmoptl, mmes(1),
     *          qmmmmmgr(1,:ssgeom%natoms*3), ierr, 'null')
        else
           call progmm0ghkmd(1,mmprognme,qmmminp%mmmeth,qmmminp%mmbas,
     *           ssgeom, ssmmgeom,qmmminp%mmoptl, mmes(1),
     *          qmmmmmgr(1,:ssgeom%natoms*3), ierr, 'null', qmmminp)
        endif                 


130   format(//2x,'MM calculation on the capped primary system',
     *   ' without background charges:',/)
135   format(/2x,'MM energy (a.u.) for capped primary system:',
     *   f20.10)
137   format(2x,'MM gradient on primary system:')
      
      if (tdebug .eq. 1) then
        write (outfile,116)
        write (outfile,137)
        write (outfile,118) -qmmmmmgr(1,:ssgeom%natoms*3)
        write (outfile,119)
      endif
      sumtyp = 2
cx      call qmmmgsum (qmmminp, mmgeom, ssgeom%natoms,
      call qmmmgsum(qmmminp, mmgeom, geom, ssgeom%natoms,     !0208BW15 add geom
     *   -qmmmmmgr(1,:ssgeom%natoms*3), sumtyp, ierr)
c
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  MM calc. no point charges import:'
     &      ,' timing ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif
            
c
c  Work on background charges
c
      if (qmmminp%embed .eq. 'mechanical') goto 2000
c
c  Scale M1, M2 charges
c
      call bgchscale(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
           
      if ((qmmminp%bordercharge .eq. 'scale')) goto 3000
         
c         
c     
c  Shift/redistribute M1 charges
c
      call bgchshift(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)   
c
c  Add charges to the primary system
c
3000  call qmmmbgchgeom(geom,mmgeom,ssgeom,ssmmgeom,bgchgeom,
     *                qmmminp)
c
c  EE polarized charge calculation with the QEQ or EEM method
c  8/8/2006 by Yan Zhang                                                           
c
      call polareech(qmprognme,mmprognme,geom, mmgeom, 
     &                     ssgeom, ssmmgeom, qmmminp)

c
c  end 8/8/2006 by Yan Zhang1
c
c  Flexible-boundary charge calculation
c
      if(qmmminp%iflexbound .eq. 1) then
         call fbbpc(geom, mmgeom, ssgeom, ssmmgeom, qmmminp, 
     *    qmprognme, px) 
      endif
c
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*)'  Flexible-boundary charge calculation:'
     &      ,' timing ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif
c
c  End by Yan Zhang Apr. 17, 2009
c
c  Electricstatic interactions within the background point charges
c
c
c    add 5/25/2006 by Yan Zhang
c  The single-point energy with background point charge calculated by ORCA
C  does not include the electricstatic interactions between the background 
C  point charge. It is not to subtract the electricstatic interactions from
C  the total energy.
c
      if(qmprognme.eq.'orca') goto 2000
c
c     end 5/25/2006 by Yan Zhang
c

      optl(:)= '   '
      optl(:mmgeom%maxmmopt)=qmmminp%mmoptl(1:mmgeom%maxmmopt)
      write(keystring,"(a,f7.2)") "chg-cutoff ", 
     *                      2.d0*qmmminp%rcutoff+2.d0
      optl(mmgeom%maxmmopt+1)= keystring
      write(keystring,"(a,f7.2)") "chg-taper ", 
     *                      2.d0*qmmminp%rcutoff+1.d0
      optl(mmgeom%maxmmopt+2)= keystring
      tcssmmkey = 2
      call progmmstghk(1,mmprognme, qmmminp%mmmeth, qmmminp%mmbas,
     *  ssgeom, ssmmgeom, optl, mmes(3), 
     *  qmmmmmgr(3,:ssgeom%natoms*3), ierr, 'null', qmmminp)
        
       mmes(1) = mmes(1) + mmes(3)
435   format(/2x,'Additional MM energy (a.u.):',t50,f20.10,/2x,
     *  '(due to electricstatic interactions within ', 
     *  'background point charges)')               
     
437   format(2x,'Additonal MM gradient')
     
      if (tdebug .eq. 1) then
         write (outfile,116)
         write (outfile,437)
         write (outfile,118) qmmmmmgr(3,:ssgeom%natoms*3)
         write (outfile,119)
      endif

      if (qmmminp%bordercharge .eq. 'scale')       then
         sumtyp = 4
      elseif (qmmminp%bordercharge .eq. 'shift')   then
         sumtyp = 5
      elseif (qmmminp%bordercharge .eq. 'redist1') then
         sumtyp = 6
      elseif (qmmminp%bordercharge .eq. 'rc')      then
         sumtyp = 6
      elseif (qmmminp%bordercharge .eq. 'redist2') then
         sumtyp = 7                                      
      elseif (qmmminp%bordercharge .eq. 'rcd2')    then
         sumtyp = 7                                      
      elseif (qmmminp%bordercharge .eq. 'redist3') then
         sumtyp = 6      
      elseif (qmmminp%bordercharge .eq. 'rcd')     then
         sumtyp = 6      
      else
         sumtyp = -1
      endif
cx      call qmmmgsum (qmmminp, mmgeom, ssgeom%natoms,
      call qmmmgsum(qmmminp, mmgeom, geom, ssgeom%natoms,     !0208BW15  add geom
     *   -qmmmmmgr(3,:ssgeom%natoms*3), sumtyp, ierr)
c
c
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  Elect. int. within the background:'
     &      ,' time=', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif

2000  continue

c
c  QM or QM** calculation on the primary system
c
c
c for flexible boundary treatment added by Yan Zhang, Apr. 15, 2009
c modified by Yan Zhang, May 30, 2009  
c      if(qmmminp%iflexbound .eq. 1) goto 5000
      if(qmmminp%iflexbound .eq. 1) goto 4900
c
c end Apr. 17, 2009
c
      if (qmmminp%embed .eq. 'mechanical') then      
         call progghk(1, qmprognme, qmmminp%qmmeth, qmmminp%qmbas, 
     *      qmmminp%qmcharge, qmmminp%qmmult,
     *      ssgeom, qmmminp%qmoptl, qmes(1), 
     *      qmmmqmgr(:ssgeom%natoms*3), ierr, 'null')
      else
         call proggsthk(1, qmprognme, qmmminp%qmmeth, qmmminp%qmbas,
     *      qmmminp%qmcharge, qmmminp%qmmult,
     *      ssgeom, qmmminp%qmoptl, qmes(1), 
     *      qmmmqmgr(:ssgeom%natoms*3), ierr, 'null',
     *      qmmminp, ssmmgeom)   
      endif
      
c
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  QM or QM** calculation on the:'
     &      ,' time=', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif
4900  continue
c
c for flexible boundary treatment added by Yan Zhang, Apr. 15, 2009
c
      if(qmmminp%iflexbound .eq. 1) then
         call proggsthk(1, qmprognme, qmmminp%qmmeth, qmmminp%qmbas,
     *      qmmminp%qmcharge, qmmminp%qmmult,
     *      ssgeom, qmmminp%qmoptl, qme1, 
     *      polmmg1(:ssgeom%natoms*3), ierr, 'null',
     *      qmmminp, ssmmgeom)   
      
         call proggsthk(1, qmprognme, qmmminp%qmmeth, qmmminp%qmbas,
     *      qmmminp%asqmcharge, qmmminp%asqmmult,
     *      ssgeom, qmmminp%qmoptl, qme2, 
     *      polmmg2(:ssgeom%natoms*3), ierr, 'null',
     *      qmmminp, ssmmgeom)   
         qmes(1) = qme1*(1.d0-px) + qme2*px
         qmmmqmgr(:ssgeom%natoms*3) = 
     *      polmmg1(:ssgeom%natoms*3)*(1.d0-px)+
     *                                polmmg2(:ssgeom%natoms*3)*px
      endif
9001  format('CPS* energy of initial state:',f16.10)
9002  format('CPS* energy of added state:',f16.10)
9003  format('final CPS* energy:',f16.10)
c
c end Apr. 15, 2009
c
c      write(outfile,145) qmes(1)
140   format(/2x,'QM calculation on the capped primary system:',/)
145   format('QM energy (a.u.) for capped primar system:',f20.10)      
147   format(2x,'QM (QM**) gradient on primary (primary + ',
     *   'background-charge) system:')


      if (tdebug .eq. 1) then
          write (outfile,145) qmes(1)
          write (outfile,116)
          write (outfile,147)
          write (outfile,118) qmmmqmgr(:ssgeom%natoms*3)
          write (outfile,119)
      endif
      if (qmmminp%embed .eq. 'mechanical')            then
         sumtyp = 2
      elseif (qmmminp%embed .eq. 'electric')          then
        if (qmmminp%bordercharge .eq. 'scale')       then
            sumtyp = 4
        elseif (qmmminp%bordercharge .eq. 'shift')   then
            sumtyp = 5
        elseif (qmmminp%bordercharge .eq. 'redist1') then
            sumtyp = 6
        elseif (qmmminp%bordercharge .eq. 'rc')      then
            sumtyp = 6
        elseif (qmmminp%bordercharge .eq. 'redist2') then
            sumtyp = 7
        elseif (qmmminp%bordercharge .eq. 'rcd2')    then
            sumtyp = 7
        elseif (qmmminp%bordercharge .eq. 'redist3') then
            sumtyp = 6   
        elseif (qmmminp%bordercharge .eq. 'rcd')     then
            sumtyp = 6   
        else
            sumtyp = -1                 
        endif    
      else
        sumtyp = -1         
      endif
cx      call qmmmgsum (qmmminp, mmgeom, ssgeom%natoms,
      call qmmmgsum(qmmminp, mmgeom, geom, ssgeom%natoms,     !0208BW15 add geom
     *   qmmmqmgr(:ssgeom%natoms*3), sumtyp, ierr)     
                 
c
c
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  Flexible boundary treatment:'
     &      ,' timing ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif
5000  continue

c
c  Calculate the QMMM energy
c
      qmmmener = qmes(1) - mmes(1) + mmes(2)

c
c    CPU time force calculation
c
        if (ttiming.eq.1) then
            call CPU_TIME(timing2)
            deltatiming=timing2-timing1
            timing1=timing2
            write (outfile,*) '  QMMM energy timing: ', deltatiming
            call SYSTEM_CLOCK(timingw2)
            deltatimingw=timingw2-timingw1
            timingw1=timingw2
            write (outfile,*) '  System clock: ', deltatimingw
        endif

      return 
      end

c
c
c*****************************************************************************
c  READPATH
c*****************************************************************************
c
      subroutine readpath (geom)
c
c     Subroutine readpath
c
c     Created:  Apr. 05, 2011 by Soroosh Pezeshki
c
c     Purpose:  Read atom coordinates from path file
c
c     Description:  
c
c
c     Called by:  runmd
c
c     Calls:  readln, icint, cfloat
c
c     Variables defined here:
c
      use caltyp, only : tdebug
      use files, only : outfile, pathfile
      use input, only : cartgeom
      implicit none
c
c  i/o variables
c

      type (cartgeom) :: geom
c
c  local variables
c      
      integer i, i3, nwords,isec,ieof,iscend, icint
      double precision cfloat
c
      character*80  words(40)

      call readln (pathfile, words, nwords, isec, ieof, iscend)


      do while ((iscend .eq. 0) .and. (ieof .eq. 0))
        if (nwords .lt. 4) goto 100
        i = icint(words(1))
        if (i .gt. geom%natoms) goto 200
        i3 = i * 3
        geom%angcoord(i3-2) = cfloat(words(2))
        geom%angcoord(i3-1) = cfloat(words(3))
        geom%angcoord(i3) = cfloat(words(4))

        call readln (pathfile, words, nwords, isec, ieof, iscend)
      enddo

      return
100   write(outfile,*) 'ERROR: Not enough values in the path file.',
     *    ' Please specify the atom index and the x, y and z ',
     *      'coordinate in each line' 
      stop 'readpath 1'
200   write(outfile,*) 'ERROR: Atom index',i,' does not exist.'
      stop 'readpath 2'
      end

c
c
c*****************************************************************************
c  G03STINPESP
c*****************************************************************************
c
      subroutine g03stinpesp(iunit,esmeth,esbas,qmcharge, qmmult,
     *   ssgeom,optl,ver_lab, qmmminp, ssmmgeom)
c
c     Subroutine g03stinpesp
c     
c     Created:  Jul. 22, 2011 by Soroosh Pezeshki
c     Based on: g03stinp by Hai Lin
c
c     Purpose:  Write a Gaussian03 input file for the calculation of 
c        the capped small system in presence of background point charges
c        calculated with the ESP methode
c                                                                       
c
c     Called by:  progehk, mprogehk, progghk, proghhk, progohk
c     Calls:
c
      use files
      use caltyp                                                        
      use input
      implicit none
c
c i/o variables
c
      character*20 ver_lab
      character*20 esmeth
      character*20 esbas
c
      character*80 optl(maxoptl)   
      integer      iunit
      type (cartgeom) :: ssgeom   
      type (mmvalen)  :: ssmmgeom   
      type (qmmminf)  :: qmmminp
c
c local variables
c
      logical      linkzero
      integer      mfin, bfin, mend, bend, i, j, k
      
      integer      ii, jj, kk
      integer      lsymbol, ltypname
      integer      qmnatm,capnatm,cssnatm,lsnatm
      integer      ssnatm,bgchnatm,mmnatm

c
c
      
      character*20 oldbas       
      character*12 tswit(4)
      
      double precision  charge

      integer qmcharge, qmmult
      character*12 mmprmkey 
      character*2  atmsymbol
      character*4  typname
      

      qmnatm   = qmmminp%qmnatm
      mmnatm   = qmmminp%mmnatm
      capnatm  = qmmminp%capatm%capnatm
      ssnatm   = ssmmgeom%natoms
      cssnatm  = qmnatm + capnatm
      lsnatm   = cssnatm + mmnatm
      bgchnatm = ssnatm - lsnatm
      
      mfin = 1
      bfin = 1
      mend = 0
      bend = 0   

c
c  If the basis is pDZ+,g3large, mg3, or mg3s change the basis
c    specification to Gen so that the non-standard basis will be read in
c    from the foot of the input file. 
c
      if (esbas(:8).eq.'pdz+' .or. esbas(:3).eq.'mg3'
     *     .or. esbas(:7).eq.'g3large') then
         oldbas = esbas
         esbas = 'gen     '
      endif
c
c  Find ends of method and basis set specification
c
      call fspace(esmeth, mfin, mend)
      if (mend.eq.1) then
         mfin = len(esmeth)
      else
         mfin = mfin-1
      endif

      call fspace(esbas, bfin, bend)
      if (bend.eq.1) then
         bfin = len(esbas)
      else
         bfin = bfin-1
      endif
c
c     Memory and processors (specified in the user inputed options)      
c     linkzero is a logical (boolean) variable to determine if the current
c     line being read is a link0 command or a keyword option
c
      linkzero = .false.
      i = 1
      do 10 j=1,10
      if (optl(i)(j:j) .eq. '%' ) then
      linkzero = .true.
      endif
10    CONTINUE

      do while (linkzero .and. i.le.maxoptl)
         write(iunit,*) optl(i)
         i = i+1
         linkzero = .false.
          do 12 j=1,10
            if (optl(i)(j:j) .eq. '%' ) then
               linkzero = .true.
            endif
12    CONTINUE
      end do

c
c added by Xin-Ping Wu, 10/20/2017
c
            write(iunit,*) '%chk=Test.chk'
c
c end by Xin-Ping Wu, 10/20/2017
c

         if( esmeth(:4).eq.'mp4 ' .or.
     *        esmeth(:4).eq.'ccsd' .or. esmeth(:8).eq.'qcisd(t)' ) then
            if(esmeth(:12) .eq. 'qcisd(t)=e4t') then                    
               write(iunit,1011) '#','qcisd(t,e4t,maxcycle=100)',       
c     *         '/',esbas(:bfin),'Force=EnOnly','FChk=All',
     *         '/',esbas(:bfin),'Force=EnOnly',                         1020XW17
     *              'Charge NoSymm'      
            else                                                        
            write(iunit,1011) '#',esmeth(:mfin),'/',esbas(:bfin),
c     *              'Force=EnOnly','FChk=All','Charge NoSymm'
     *              'Force=EnOnly','Charge NoSymm'                      1020XW17
            endif                                                       
         else
            write(iunit,1011) '#',esmeth(:mfin),'/',esbas(:bfin),
c     *              'Force','FChk=All','Charge NoSymm'
     *              'Force','Charge NoSymm'                             1020XW17
         endif


c
c  User inputed keywords
c
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
c
         do j=1, 79
            if (optl(i)(j:j) .eq. '!') then
               k = 1
            endif
         enddo
         if (k .eq. 0) then
            write(iunit,3000) '#', optl(i)
         endif
         i = i+1
      end do
c
3000  format (a1,a79)      
c
c  Title
c
         write(iunit,3022)

3022  format (/'Gradient Calculation'/)

c
c  Charge and multiplicity
c
      write(iunit,3050) qmcharge, qmmult
3050  format (1x,i3,1x,i3)      
c
c  QM: atomic symbols and coordinates
c      atom type = 'CT' defined parameter

      do i=1, qmnatm
         lsymbol = 0
         atmsymbol = ssgeom%atm(i)           
         do j=1, 2
            if (atmsymbol(j:j) .ne. ' ') then 
               lsymbol = lsymbol + 1
            endif 
         enddo

         ltypname = 2
         typname  = qmtypname        
c         ltypname = 0      
c         typname = ssmmgeom%atmtypname(i)
c         do j=1, 4
c            if (typname(j:j) .ne. ' ') then 
c               ltypname = ltypname + 1
c            endif
c         enddo
         
         write(iunit,4000)
     *      atmsymbol(:lsymbol), typname(:ltypname), 
     *      ssmmgeom%atmcharge(i),  ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)

      enddo     
4000  format (a,'-',a,'-',d10.4,t20,'0 ',3(d16.9,3x),'H')
c
c  Cap: atomic symbols and coordinates
c       take H as symbol and HC as atomtype, as defined parameters
c
      do i = qmnatm + 1, cssnatm
         atmsymbol = capname
         lsymbol   = 1
         typname   = captypname
         ltypname  = 2
         write(iunit,4000)
     *      atmsymbol(:lsymbol), typname(:ltypname),
     *      ssmmgeom%atmcharge(i),  ssgeom%angcoord(3*i-2),
     *      ssgeom%angcoord(3*i-1), ssgeom%angcoord(3*i)
      enddo
c
c  MM and M1/auxiliary charges: atomic symbols and coordinates
c     take He as symbol and HC as atomtype, as defined parameters     
c
      write(iunit,*)

      do i=cssnatm+1,ssnatm
         write(iunit,4100)
     *      ssgeom%angcoord(3*i-2), ssgeom%angcoord(3*i-1),
     *      ssgeom%angcoord(3*i), ssmmgeom%atmcharge(i)
      enddo
      
      write(iunit,*)
c4100  format (3(f16.9,3x),f10.4, ' 0.0 1.0')
4100  format (3(f16.9,3x),f10.4)
c
c  Connectivity: only those for cap atoms are provided
c
c
c Revised by Hai Lin, 5/11/2009
C No longer need a bond between 1st and 2nd atoms, since NoBond is set to 0 1
c
c  revised by Hai Lin, 5/3/2009
c  add a bond between the 1st and 2nd QM atoms
c
c      do i=1, qmnatm
c         write (iunit,4500) i
c      enddo     

c      write (iunit,4500) 1, 2, 1.0
c      do i=2, qmnatm
      do i=1, qmnatm
         write (iunit,4500) i
      enddo
c
c  end by Hai Lin, 5/3/2009
c End by Hai Lin, 5/11/2009 
c
      do i=qmnatm+1, cssnatm
         jj = ssmmgeom%atmvalenlist(i,1)
         do j=1, qmnatm
            if (jj .eq. qmmminp%qmatm(j)) then
               write (iunit,4500) i, j, 1.0 
            endif
         enddo
      enddo                 
      
      do i=cssnatm+1, ssnatm
         write (iunit,4500) i
      enddo
      
      write(iunit,*)          
4500  format (2x,i4,2x,i4,2x,f4.2)
c
c write additional options for molecule specification                   
c
      i = 1
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
         jj = 1
c
         do j=1, 79
            if (optl(i)(j:j+1) .eq. '! ') then
               k = 1
               jj = j + 1
            endif
         enddo
         if (k .eq. 1) then
c
            write(iunit,5000) optl(i)(jj:80)
         endif
         i = i + 1
      enddo
c      write(iunit,*)
c
5000  format (a79)      
c
c  Include basis set file at end of the input file if needed
c
      if (esbas(:3).eq.'gen') then
         if (oldbas(:8).eq.'pdz+') then
            write (iunit, 1051) '@', pDZsloc
         elseif (oldbas(:4).eq.'mg3s') then
            write (iunit, 1051) '@', mg3sloc
         elseif (oldbas(:3).eq.'mg3') then
            write (iunit, 1051) '@', mg3loc
         elseif (oldbas(:7).eq.'g3large') then
            write (iunit, 1051) '@', g3Lloc
         endif
      endif
1051  format (2a,/)
c
c  Change the basis set name back if they've been changed
c
      if (esbas(:3).eq.'gen') then
         esbas = oldbas
      endif
c
c  NonBon key
c
!       write(iunit,*) mmprmkey

c
c    Write additional options for molecule specification behind "nonbon"
c    The keyword is labeled by "!2"
c       8/4/2006 by Yan Zhang
c
      write(iunit,*)
      i = 1
      do while (optl(i).ne.'   ' .and. i.le.maxoptl)
         k = 0
         jj = 1
c
         do j=1, 79
            if (optl(i)(j:j+1) .eq. '!2') then
               k = 1
               jj = j + 2
            endif
         enddo
         if (k .eq. 1) then
c
c added by Xin-Ping Wu, July 25, 2018
c            write(iunit,5000) optl(i)(jj:80)
            if (optl(i)(jj:jj) .eq. '@') then
               call scase (81-jj,optl(i)(jj:80),optl(i)(jj:80))
               write(iunit,5000) optl(i)(jj:80)
            else
               write(iunit,5000) optl(i)(jj:80)
            endif
c end by Xin-Ping Wu, July 25, 2018
c
         endif
         i = i + 1
      enddo
c
c     end 8/4/2006 by Yan Zhang
c  Close the file
c
      write(iunit,*)
      write(iunit,*)      
      close (iunit, err=9999)
      
      return

9999  write (outfile,*) 'Error creating Gaussian input file'
      stop 'g03inp'

1001  format (/)
1010  format (2a)
c1011  format (4a,3(3x,a))
1011  format (4a,2(3x,a))                                               1020XW17
1012  format (4a,2(3x,a))

      end
c
c
c*****************************************************************************

c
c
c*******************************************************************************
c  SETTSEMIEMPIRICAL
c*******************************************************************************
c
      subroutine settsemiempirical(tmeth)
c
c     Created:  March 11, 2011 by Soroosh Pezeshki
c     Revised:
c     Purpose:  checked if semi empirical calculations are selected
c                and set tsemiempirical
c
c     Called by:  RTEST, RQMMM
c
      use caltyp, only : tsemiempirical
      implicit none
c
      character*20  tmeth

      tsemiempirical=0

      if (tmeth(1:3).eq.'am1') tsemiempirical=1
      if (tmeth(1:3).eq.'pm3') tsemiempirical=1
      if (tmeth(1:4).eq.'mndo') tsemiempirical=1
      if (tmeth(1:6).eq.'nddo/2') tsemiempirical=1
      if (tmeth(1:6).eq.'nddo/1') tsemiempirical=1
      if (tmeth(1:7).eq.'zindo/2') tsemiempirical=1
      if (tmeth(1:7).eq.'zindo/1') tsemiempirical=1
      if (tmeth(1:7).eq.'zindo/s') tsemiempirical=1


      end

